
#ifndef CALCINFO_H
#define CALCINFO_H

#include <qobject.h>

#include <F100_Op.h>
#include <casio.h>
#include <APIFunction.h>

using namespace Casio;

/*!
	\class CCalcInfo CalcInfo.h
	\brief Informations sur une calculatrice Casio
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt
*/
class DLLEXPORT CCalcInfo : public QObject
{
	Q_OBJECT;
public:
	/** Constructeur
	 * @param[in] parent Parent
	 * @param[in] nCalcType Type de la calculatrice
	 */
	CCalcInfo(QObject* parent, CasioModel nCalcType = CM_INVALID);

	/** Destructeur */
	virtual ~CCalcInfo();

	/** @return Retourne le modle de la calculatrice */
	CasioModel     GetCalcModel() const;

	/** @return Retourne vrai si le modle de la calculatrice est de type Graph100(+) */
	bool           IsAFX2LIKE() const;

	/** @return Retourne le nom du modle de la calculatrice */
	QString        GetCalcName() const;

	/** Permet de dfinir le modle de la calculatrice
	 * @param[in] nCalcModel Modle de la calculatrice
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	bool           SetCalcModel(CasioModel nCalcModel);

	/** @return Retourne le rpertoire o sont stocks les fichiers de sauvegarde */
	const QString& GetCalcDir() const;

	/** Permet de dfinir le rpertoire des fichiers de sauvegarde  lire
	 * @param[in] sDir Chemin du rpertoire
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	bool           SetCalcDir(const QString& sDir);

	/** Permet de dfinir le nom du fichier  ouvrir
	 * @param[in] sCalc Nom du fichier
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	bool           SetCalcFile(const QString& sCalc);

	/** @return Retourne le nom du fichier de sauvegarde actuellement ouvert */
	const QString& GetCalcFile() const;

	/* Permet de connaitre le nombre d'upload d'un lecteur
	 * @param[in] nDrive Numro du lecteur de [0..6]
	 * @return -1 Erreur \n >= 0 Nombre d'upload sur le lecteur \a nDrive
	 * @remarks Cette mthode n'est valable que si le modle de calculatrice est une Graph100(+)
	 */
	int            GetCounter(int nDrive) const;

	/* Permet de dfinir le nombre d'upload d'un lecteur
	 * @param[in] nDrive Numro du lecteur de [0..6]
	 * @param[in] nValue Nombre d'upload
	 * @return Vrai pour le succs, sinon faux
	 * @remarks Cette mthode n'est valable que si le modle de calculatrice est une Graph100(+)
	 */
	bool           SetCounter(int nDrive, int nValue);

	/* Permet d'incrmenter le nombre d'upload d'un lecteur
	 * @param[in] nDrive Numro du lecteur de [0..6]
	 * @return Vrai pour le succs, sinon faux
	 * @remarks Cette mthode n'est valable que si le modle de calculatrice est une Graph100(+)
	 * @see SetCounter(int, int)
	 */
	bool           IncCounter(int nDrive);

public:
	/** Permet de convertir une modle de calculatrice en une chane de caractre
	 * @param[in] nCalcModel Modle de calculatrice
	 * @return Retourne le nom du modle suivant la langue slectionne (Qt Translation)
	 */
	static QString GetCalcName(CasioModel nCalcModel);

	/** Permet de convertir une type de calculatrice en une chane de caractre
	 * @param[in] nCalcType Type de calculatrice
	 * @return Retourne le nom du type suivant la langue slectionne (Qt Translation)
	 */
	static QString GetCalcType(CasioType nCalcType);

private:
	/** Ferme le fichier ouvert */
	void           CloseCalcFile();

	/** Ouvre un fichier de donnes
	 * @param[in] sCalcDir Rprtoire du fichier  ouvrir
	 * @param[in] sCalcFile Nom du fichier  ouvrir
	 * @return Vrai pour le succs, sinon faux
	 */
	bool           OpenCalcFile(const QString& sCalcDir, const QString& sCalcFile);

signals:
	/** Signal mit lorsque des informations sont modifies */
	void InformationChanged();

private:
	CasioModel m_nCalcModel;
	QString    m_sCalcFile;
	QString    m_sCalcDir;
	HND        m_hFile;
	int        m_nCounters[7];

};

#endif
