
#ifndef CHECKFILEWINDOW_H
#define CHECKFILEWINDOW_H

#include "ToolWindow.h"

#include <F100Qt.h>

#include <qdatetime.h>
#include <qmutex.h> 

class QTimer;
class QString;

/*!
	\class CheckFileWindow CheckFileWindow.h
	\brief Permet de crer un plug-in et de surveiller un fichier quelconque
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
*/
class DLLEXPORT CheckFileWindow : public CToolWindow
{
public:
	/** Constructeur
	 * @param[in] parent Parent de la fentre
	 * @param[in] pMainWindow Fentre principale
	 * @param[in] fl Flags Qt de la fentre
	 * @param[in] pTabWidget TabWidget pouvant contenir cette fentre ou NULL si non voulu
	 * @param[in] bAllowOpacityControl A vrai si on veut que le rglage de la transparence de la fentre soit possible, sinon  faux
	 * @param[in] bInTabWidget A vrai si on veut que la fentre apparaisse dans la TabWidget, sinon  faux
	 * @param[in] bForceReload A vrai si l'on veut que le rechargement soit automatique, sinon  faux pour qu'une boite de dialogue apparaisse
	 * @param[in] bAutoClose A vrai si l'on veut que l'outil se ferme, ou demande confirmation, si le fichier n'existe plus
	 * @remarks Dmarre la vrification du fichier
	 * @see StartChecking()
	 */
	CheckFileWindow(CONSTRUCTOR_PARAMDEF, bool bForceReload = TRUE, bool bAutoClose = TRUE);

	virtual int	 LoadFrom(QDomElement& element);

	virtual int	 SaveTo(QDomElement& element);
	
	/** Dfini le fichier  surveiller
	 * @param[in] sFileName Fichier  surveiller ou QString::null si aucun fichier  surveiller
	 * @param[in] bEmitStateChanged A vrai si le signal StateChanged est emit
	 * @param[in] bShowMsgErr A vrai si on veut afficher l'erreur dans une boite de dialogue
	 */
	bool         SetFileName(const QString& sFileName, bool bEmitStateChanged = TRUE, bool bShowMsgErr = TRUE);

	/** Filtre des vnements */
	virtual bool eventFilter(QObject* o, QEvent* e);

protected:
	/** Prend la dernire valeur de la date de modification du fichier.\n
	 * Permet d'viter un appel de DoReloadFile(bool) quand la classe elle-mme modifie le fichier
	 */
	void		 ResetTime();

	/** Permet de vrifier le fichier\n
	 * Appelle OnFileRemoved() ou OnInvalidFile() suivant l'erreur
	 */
	void         CheckFile();

	/** Permet de stopper la vrification du fichier */
	void         StopChecking();
	/** Permet de dmarrer la vrification du fichier */
	void         StartChecking();

	/** Appele automatiquement lorsque CheckFile() a dtect que le fichier n'existait plus.\n
	 * Par dfaut cette mthode affiche un message signalant la suppression du fichier et suivant les paramtres de construction de l'outil :
	 * - bAutoClose  faux : Ne ferme pas l'outil, affiche juste le message 
	 * - bAutoClose  vrai et bForceReload  vrai : Ferme aprs le message
	 * - bAutoClose  vrai et bForceReload  faux : Demande confirmation  l'utilisateur pour fermer ou non l'outil.\n
	 * Vous pouvez rimplmenter cette mthode pour changer son comportement.
	 */
	virtual void OnFileRemoved();

	/** Appele automatique lorsque CheckFile() a dtect que le fichier n'est plus valide.\n
	 * Si bAutoClose est  vrai alors ferme l'outil, sinon ne fait rien.\n
	 * Vous pouvez rimplmenter cette mthode pour changer son comportement.
	 */
	virtual void OnInvalidFile();

	/** Mthode appele automatiquement lorsque le fichier est modifi
	 * @param[in] bShowValidErr A vrai si on doit afficher un message d'erreur.\n
	 * Si le fichier n'est pas valide ou non ouvrable par exemple.\n
	 * Ceci peut tre utilis si on appelle cette mthode dans la mthode \a Init, car
	 * dans \a Init il ne faut pas forcement afficher des erreurs de validit (ex : fichier lec (ROM-DISK, BASIC, ...))
	 * @return > 0 Ok \n
	 * <= 0 Erreur qui provoque la fermeture de la fentre lors d'un auto-rechargement si bAutoClose est  vrai
	 */
	virtual int	 DoReloadFile(bool bShowValidErr) = 0;

	/** Rajoute le nom du fichier surveill dans la barre de titre.\n
	 * Du style : MONTITRE - Fichier.txt \n
	 * Il vous suffit de rimplmenter cette mthode pour soit dfinir vous mme la forme du titre,
	 * soit d'crire setCaption("votre titre") et d'appeler ensuite CheckFileWindow::DoShowCaption()
	 */
	virtual void DoShowCaption();

	/** Permet de vrifier par intervalle de temps le fichier  surveiller */
	virtual void timerEvent(QTimerEvent*);

protected:
	QString m_sFileName; ///< Fichier en cours de surveillance

private:
	int	    m_nTimerId;
	bool    m_bForceReload;
	bool    m_bFileDestroy;
	bool    m_bAutoClose;
	QTime   m_qTime;
};

#endif
