
#include "FileList.h"

#include <qdragobject.h>

#include "image_defs.h"

//-----------------------------------------------------------------------------------
CFileList::CFileList(QWidget* parent, bool bAcceptDrops)
	:QListView(parent)
{
	setAcceptDrops(bAcceptDrops);
}

//-----------------------------------------------------------------------------------
void CFileList::setAcceptDrops(bool on)
{
	QListView::setAcceptDrops(on);
	viewport()->setAcceptDrops(on);
}

//-----------------------------------------------------------------------------------
void CFileList::DeleteItem(QListViewItem* pItem, QListView* pList)
{
	if(pItem) {
		if(pItem->parent()) {
			pItem->parent()->takeItem(pItem);
		}
		else {
			if(pList) {
				pList->takeItem(pItem);
			}
		}
		delete pItem;
	}
}

//-----------------------------------------------------------------------------------
void CFileList::IncItem(QListViewItem*& pItem)
{
	if(pItem)
	{
		QListViewItem* pChild = pItem->firstChild();
		if(pChild)
		{
			pItem = pChild;
		}
		else
		{
			pChild = pItem->nextSibling();
			if(!pChild)
			{
				QListViewItem* pParent = pItem->parent();
				while(pParent && !(pItem = pParent->nextSibling()))
				{
					pParent = pParent->parent();
				}
				if(!pParent)
				{
					pItem = NULL;
				}
			}
			else
			{
				pItem = pChild;
			}
		}
	}
}

//-----------------------------------------------------------------------------------
QListViewItem* CFileList::LastItem(QListViewItem* pItem)
{
	QListViewItem* i = pItem;
	while(pItem) {
		i = pItem;
		pItem = pItem->nextSibling();
	}
	return i;
}

//-----------------------------------------------------------------------------------
void CFileList::contentsDragEnterEvent(QDragEnterEvent* e)
{
    if(QUriDrag::canDecode(e))
    {
		e->accept();
    }
}

//-----------------------------------------------------------------------------------
void CFileList::contentsDragMoveEvent(QDragMoveEvent* e)
{
    if(QUriDrag::canDecode(e))
    {
		e->accept();
		QPoint vp = contentsToViewport(e->pos());
		QListViewItem* pItem = itemAt(vp);
		if(pItem)
		{
			clearSelection();
			setSelected(pItem, TRUE);
		}
    }
}

//-----------------------------------------------------------------------------------
void CFileList::contentsDropEvent(QDropEvent* e)
{
	QStringList sList;
	if(QUriDrag::decodeLocalFiles(e, sList))
	{
		QPoint vp = contentsToViewport(e->pos());
		emit Dropped(sList, itemAt(vp));
	}
}


//------------------------CFileListItem--------------------------------------------

//-----------------------------------------------------------------------------------
CFileItem::CFileItem(Type nType, QListView* parent, QListViewItem* after)
	: QListViewItem(parent, after),
	m_nType(nType)
{
	setOpen(FALSE);
}

//-----------------------------------------------------------------------------------
CFileItem::CFileItem(Type nType, QListViewItem* parent, QListViewItem * after)
	: QListViewItem(parent, after),
	m_nType(nType)
{
	setOpen(FALSE);
}

//-----------------------------------------------------------------------------------
void CFileItem::SetName(const QString& sName)
{
	setText(0, sName);
}

//-----------------------------------------------------------------------------------
void CFileItem::SetSize(int nSize)
{
	setText(1, QString::number(nSize));
}

//-----------------------------------------------------------------------------------
void CFileItem::SetUnknowSize()
{
	setText(1, CFileList::tr("Inconnue"));
}

//-----------------------------------------------------------------------------------
QString CFileItem::GetName() const
{
	return text(0);
}

//-----------------------------------------------------------------------------------
int CFileItem::GetSize() const
{
	return text(1).toInt();
}

//-----------------------------------------------------------------------------------
int CFileItem::GetFirstColumnFree() const
{
	return 2;
}

//-----------------------------------------------------------------------------------
bool CFileItem::IsDir() const
{
	return m_nType == Directory;
}

//-----------------------------------------------------------------------------------
bool CFileItem::IsFile() const
{
	return m_nType == File;
}

//-----------------------------------------------------------------------------------
void CFileItem::setOpen(bool bOpen)
{
	QPixmap pixmap;
	if(m_nType == File) {
		pixmap = QPixmap::fromMimeSource(IMG_FILE);
	}
	else {
		if(bOpen) {
			pixmap = QPixmap::fromMimeSource(IMG_DIR_OPEN);
		}
		else {
			pixmap = QPixmap::fromMimeSource(IMG_DIR_CLOSED);
		}
	}
	setPixmap(0, pixmap);
	QListViewItem::setOpen(bOpen);
}

//-----------------------------------------------------------------------------------
int CFileItem::compare(QListViewItem* pItem, int nCol, bool bAscending) const
{
	CFileItem* pFileItem = (CFileItem*)pItem;
	if(pItem->rtti() == RTTI()) {
		if(m_nType == pFileItem->m_nType) {
			if(nCol != 1) {
				return text(nCol).compare(pItem->text(nCol));
			}
			int nDif = GetSize() - pFileItem->GetSize();
			if(nDif < 0) {
				return -1;
			}
			else if(nDif == 0) {
				return 0;
			}
		}
		else {
			if(IsDir()) {
				return -1;
			}
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CFileItem::rtti() const
{
	return RTTI();
}
