
#ifndef FILELIST_H
#define FILELIST_H

#include <F100Qt.h>

#include <qlistview.h>

class QStringList;

/*!
	\class CFileList FileList.h
	\brief Liste d'lments grant en interne le drag&drop
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt
	Permet de grer facilement le drag&drop dans une liste d'lment de type QListView
*/
class DLLEXPORT CFileList : public QListView
{
	Q_OBJECT
public:
	/** Constructeur
	 * @param[in] parent Fentre parente
	 * @param[in] bAcceptDrops A vrai si on veut que la liste accepte le drag&drop, sinon  faux
	 */
	CFileList(QWidget* parent, bool bAcceptDrops = FALSE);

	virtual void setAcceptDrops(bool on);

public:
	/** Permet de dtruire un lment d'une liste
	 * @param[in] pItem Elment  supprimer
	 * @param[in] pList Liste contenant l'lment
	 */
	static void           DeleteItem(QListViewItem* pItem, QListView* pList);

	/** Permet de parcourir une liste en rcuprant l'lment suivant.\n
	 * Priorit au parcours en profondeur.
	 * @param[in,out] pItem Elment de dpart
	 */
	static void           IncItem(QListViewItem*& pItem);

	/** Permet d'avoir le dernier lment d'une liste d'lments du mme niveau.
	 * @param[in,out] pItem Elment de dpart
	 * @return Retourne le dernier l'lment de la liste du mme niveau que \a pItem
	 */
	static QListViewItem* LastItem(QListViewItem* pItem);

protected:
	/** From Qt */
	void contentsDragEnterEvent(QDragEnterEvent*);
	/** From Qt */
	void contentsDragMoveEvent(QDragMoveEvent*);
	/** From Qt */
	void contentsDropEvent(QDropEvent*);

signals:
	/** Signal mit lorsque des fichiers ont t gliss dans la liste.
	 * Signal uniquement mit lorsque le drag&drop est activ.
	 * @param[in] sFiles Liste des fichiers
	 * @param[in] pDroppedItem Elment correspondant au relchement de la souris
	 */
	void Dropped(const QStringList& sFiles, QListViewItem* pDroppedItem);

};

/*!
	\class CFileItem FileList.h
	\brief Elment reprsentant un dossier ou un fichier
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt
*/
class DLLEXPORT CFileItem : public QListViewItem
{
public:
	/** Type de l'lment */
	enum Type{
		File = 0,  /*!< Fichier */
		Directory, /*!< Dossier */
	};

public:
	/** Constructeur
	 * @param[in] nType Type de l'lment
	 * @param[in] parent Liste contenant l'lment
	 * @param[in] after Elment prcdent cet lment
	 */
	CFileItem(Type nType, QListView* parent, QListViewItem* after = NULL);

	/** Constructeur
	 * @param[in] nType Type de l'lment
	 * @param[in] parent Elment parent
	 * @param[in] after Elment prcdent cet lment
	 */
	CFileItem(Type nType, QListViewItem* parent, QListViewItem * after = NULL);

	/** Permet de dfinir le nom de l'lment.\n
	 * Affiche le nom de l'objet dans la colonne 0.
	 * @param[in] sName Nom de l'lment
	 */
	void        SetName(const QString& sName);

	/** Permet de dfinir la taille de l'lment.\n
	 * Affiche la taille de l'objet dans la colonne 1.
	 * @param[in] nSize Taille de l'lment
	 */
	void        SetSize(int nSize);

	/** Permet de dfinir que taille de l'lment est inconnue.*/
	void        SetUnknowSize();
	/** @return Le nom de l'lment */
	QString     GetName() const;
	/** @return La taille de l'lment */
	int         GetSize() const;
	/** @return La premire colonne vide (ie. Non utilise par le nom et la taille de l'lment) */
	virtual int GetFirstColumnFree() const;
	/** @return Retourne vrai si l'lment est un dossier, sinon faux */
	bool        IsDir() const;
	/** @return Retourne vrai si l'lment est un fichier, sinon faux */
	bool        IsFile() const;
	/** From Qt */
    void    setOpen(bool);
	/** From Qt */
	int     compare(QListViewItem* i, int col, bool ascending) const;
	/** From Qt */
	int     rtti() const;

public:
	/** Permet de connaitre la valeur rtti de \a CFileItem
	 * @return Retourne la valeur du rtti de \a CFileItem
	 * @see rtti() const
	 */
	static int RTTI() { return 1001;}

private:
	Type m_nType; /*!< Type de l'lment */
};

#endif
