
#include "Label.h"

#include <qpainter.h>
#include <qpixmap.h>
#include <qcolordialog.h>


//-----------------------------------------------------------------------------------
CLabel::CLabel(QWidget* parent, const char* name, bool bDoubleBuffering)
	: QLabel(parent, name, bDoubleBuffering ? WNoAutoErase : 0)
	, m_bColorChooser(FALSE)
{
	m_bDoubleBuffering = bDoubleBuffering;
}

//-----------------------------------------------------------------------------------
void CLabel::SetDefaultStyle()
{
	if(m_bColorChooser) {
		setText(QString::null);
		m_bColorChooser = FALSE;
	}
	unsetPalette();
	setBackgroundMode(PaletteBase);
	setFrameStyle(QFrame::Box | QFrame::Plain);
	setAlignment(AlignAuto);
}

//-----------------------------------------------------------------------------------
void CLabel::SetColorChooser(const QColor& color)
{
	m_bColorChooser = TRUE;
	SetColor(color);
	setFrameStyle(QFrame::Box | QFrame::Plain);
	setAlignment(AlignCenter);
	languageChange();
}

//-----------------------------------------------------------------------------------
const QColor& CLabel::GetColor() const
{
	return palette().color(QPalette::Active, QColorGroup::Background);
}

//-----------------------------------------------------------------------------------
void CLabel::SetColor(const QColor& color)
{
	QColor invert(color.red()^255, color.green()^255, color.blue()^255);
	QPalette p = palette();
	p.setColor(QPalette::Active,   QColorGroup::Background, color);
	p.setColor(QPalette::Active,   QColorGroup::Foreground, invert);
	p.setColor(QPalette::Inactive, QColorGroup::Background, color);
	p.setColor(QPalette::Inactive, QColorGroup::Foreground, invert);
	setPalette(p);
}

//-----------------------------------------------------------------------------------
void CLabel::mouseReleaseEvent(QMouseEvent* e)
{
	if(m_bColorChooser) {
		QColor color = QColorDialog::getColor(GetColor(), this);
		if(color.isValid() && color != GetColor()) {
			SetColor(color);
			emit colorChanged(color);
		}
	}
	emit release(e->button());
}

//-----------------------------------------------------------------------------------
void CLabel::paintEvent(QPaintEvent* e)
{
	if(m_bDoubleBuffering) {
		QPixmap pm(size());
		pm.fill(this, 0, 0);
		QPainter p(&pm, this);
		drawContents(&p);
		drawFrame(&p);
		p.end();
		bitBlt(this, 0, 0, &pm);
	}
	else {
		QPainter p(this);
		drawContents(&p);
		drawFrame(&p);
	}
}

//-----------------------------------------------------------------------------------
void CLabel::languageChange()
{
	if(m_bColorChooser) {
		setText(tr("Changer la couleur"));
	}
}
