
#ifndef CLABEL_H
#define CLABEL_H

#include <qlabel.h>

#include <F100Qt.h>

/*!
	\class CLabel Label.h
	\brief Permet d'afficher du texte ou une image
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt
	Classe drivant de QLabel et proposant plusieurs autres fonctionnalits
	dont notamment le double buffering pour viter un mauvais affichage
	lors de changement de texte trop rapide
*/
class DLLEXPORT CLabel : public QLabel
{
	Q_OBJECT
public:
	/** Constructeur
	 * @param[in] parent Parent de l'objet
	 * @param[in] name Nom de l'objet
	 * @param[in] bDoubleBuffering A vrai si on veut activer le double buffering, sinon  faux
	 */
    CLabel(QWidget* parent = 0, const char* name = 0, bool bDoubleBuffering = FALSE);

	/** Permet de dfinir le style par dfaut  cet objet (ie. Flat) */
	void SetDefaultStyle();

	/** Permet de passer en mode 'ColorChooser', ie choix de couleur.\n
	 * La couleur slectionne est affiche dans le fond de l'objet, le label prend
	 * le texte 'Changer la couleur' et lorsque l'utilisateur clique sur le label alors une boite
	 * de slection apparait.
	 * @param[in] color Couleur par dfaut.
	 */
	void SetColorChooser(const QColor& color);

	/** @return La couleur slectionne en mode 'ColorChooser' */
	const QColor& GetColor() const;
	
	/** Permet de modifier la couleur slectionne
	 * @param[in] color Couleur souhaite
	 */
	void SetColor(const QColor& color);

protected:
	/** From Qt */
    void mouseReleaseEvent(QMouseEvent* e);
	/** From Qt */
	void paintEvent(QPaintEvent* e);

protected slots:
	/** From Qt */
	void  languageChange();

signals:
	/** Signal mit pour signaler le relchement d'un bouton de la souris
	 * @param[in] button Bouton relch. (Qt::ButtonState)
	 */
	void release(int button);

	/** Signal mit lorsque l'utilisateur change de couleur
	 * @param[in] color Couleur slectionne
	 */
	void colorChanged(const QColor& color);

private:
	bool m_bDoubleBuffering;
	bool m_bColorChooser;
};

#endif
