
#include "MessageBox.h"

#include <qapplication.h>
#include <qstring.h>
#include <qfiledialog.h>

#include <APIFunction.h>

#include "language.h"

//-----------------------------------------------------------------------------------
void CMessageBox::MsgCritical(QWidget* parent, const QString& sText)
{
	QMessageBox::critical(parent, GetCaption(parent), sText, QMessageBox::Ok, QMessageBox::NoButton);
}

//-----------------------------------------------------------------------------------
void CMessageBox::MsgExclamation(QWidget* parent, const QString& sText)
{
	QMessageBox::warning(parent, GetCaption(parent), sText, QMessageBox::Ok, QMessageBox::NoButton);
}

//-----------------------------------------------------------------------------------
void CMessageBox::MsgInformation(QWidget* parent, const QString& sText)
{
	QMessageBox::information(parent, GetCaption(parent), sText, QMessageBox::Ok, QMessageBox::NoButton);
}

//-----------------------------------------------------------------------------------
bool CMessageBox::MsgQuestion(QWidget* parent, const QString& sText)
{
#if QT_VERSION < 0x030200
	QMessageBox mb(GetCaption(parent), sText, QMessageBox::Information,
		QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, 0, parent);
	return (QMessageBox::Yes == mb.exec());
#else
	return QMessageBox::question(parent, GetCaption(parent), sText, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No) == QMessageBox::Yes;
#endif

}

//-----------------------------------------------------------------------------------
int CMessageBox::MsgYesNoCancel(QWidget* parent, const QString& sText)
{
#if QT_VERSION < 0x030200
	QMessageBox mb(GetCaption(parent), sText, QMessageBox::Information,
		QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel, parent);
	return mb.exec();
#else
	return QMessageBox::question(parent, GetCaption(parent), sText, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel);
#endif
}

//-----------------------------------------------------------------------------------
QProgressDialog* CMessageBox::CreateProgressDialog(QWidget* parent, const QString& sText, unsigned int nMaxValue, const QString& sCaption, bool bIsCancelable)
{
	QProgressDialog* pProgressDialog = new QProgressDialog(parent);
	if(pProgressDialog)
	{
		pProgressDialog->setTotalSteps(nMaxValue);
		pProgressDialog->setLabelText(sText);
		pProgressDialog->setAutoReset(FALSE);
		pProgressDialog->setMinimumDuration(2500);
		pProgressDialog->setCaption(sCaption);
		if(!bIsCancelable)
		{
			pProgressDialog->setCancelButton(0);
		}
		pProgressDialog->setProgress(0);
	}
	return pProgressDialog;
}

//-----------------------------------------------------------------------------------
bool CMessageBox::SavingFile(QWidget* parent, const QString& sFileName, const void* pBuffer, const unsigned int nBytes)
{
	HND h = OpenDesc(sFileName, GENERIC_WRITE, CREATE_ALWAYS);
	if(h == INVALID_HANDLE_VALUE)
	{
		CMessageBox::MsgCritical(parent, CLanguage::GetMsgNoOpenFile(sFileName, TRUE));
		return false;
	}
	QProgressDialog* pProgressDialog = CreateProgressDialog(parent, QObject::tr("Ecrit sur le fichier") + " " + sFileName, nBytes);
	unsigned int nBytesWritten = 0;
	unsigned int nStep = 32768;
	bool bOk = true;
	while(nBytesWritten < nBytes && bOk)
	{
		if(nBytesWritten + nStep > nBytes)
		{
			nStep = nBytes - nBytesWritten;
		}
		bOk = WriteDesc(h, (const char*)(pBuffer) + nBytesWritten, nStep);
		nBytesWritten += nStep;
		pProgressDialog->setProgress(nBytesWritten);
		qApp->processEvents();
	}
	CloseDesc(h);
	delete pProgressDialog;
	qApp->processEvents();

	if(!bOk)
	{
		CMessageBox::MsgCritical(parent, CLanguage::GetMsgNoWriteFile(sFileName));
	}
	return bOk;
}

//-----------------------------------------------------------------------------------
bool CMessageBox::ReadingFile(QWidget* parent, const QString& sFileName, void* pBuffer, unsigned int nByteMax, unsigned int nReadPos, unsigned char nFillValue)
{
	HND h = OpenDescModeRead(sFileName);
	if(h == INVALID_HANDLE_VALUE)
	{
		CMessageBox::MsgCritical(parent, CLanguage::GetMsgNoOpenFile(sFileName));
		return false;
	}
	SeekDesc(h, nReadPos);

	unsigned int nBytes     = FileSize(h);
	unsigned int nBytesRead = 0;
	unsigned int nStep      = 65536;
	if(nBytes > nByteMax)
	{
		nBytes = nByteMax;
	}

	QProgressDialog* pProgressDialog = CreateProgressDialog(parent, QObject::tr("Lit le fichier") + " " + sFileName, nBytes);
	bool bOk = true;
	while(nBytesRead < nBytes && bOk)
	{
		if(nBytesRead + nStep > nBytes)
		{
			nStep = nBytes - nBytesRead;
		}
		bOk = ReadDesc(h, (char*)(pBuffer) + nBytesRead, nStep);
		nBytesRead += nStep;
		pProgressDialog->setProgress(nBytesRead);
		qApp->processEvents();
	}
	CloseDesc(h);
	delete pProgressDialog;
	qApp->processEvents();

	if(!bOk)
	{
		CMessageBox::MsgCritical(parent, CLanguage::GetMsgNoReadFile(sFileName));
	}
	else
	{
		if(nBytes < nByteMax)
		{
			//Si le fichier est plus petit que nByteMax alors
			//on rempli le reste de 'nFillValue'
			memset((char*)pBuffer + nBytes, nFillValue, nByteMax - nBytes);
		}
	}
	return bOk;
}

//-----------------------------------------------------------------------------------
QString CMessageBox::GetCaption(QWidget* pWidget)
{
	while(pWidget != NULL) {
		if(!pWidget->caption().isEmpty()) {
			return pWidget->caption();
		}
		pWidget = pWidget->parentWidget();
	}
	return QString::null;
}

//-----------------------------------------------------------------------------------
QString CMessageBox::ChooseOpenFileName(const QString& sStartWith, const QString& sFilter, QWidget* pParent, const QString& sCaption, QString* pSelectedFilter, bool bResolveSymlinks)
{
	return QFileDialog::getOpenFileName(sStartWith, sFilter, pParent, NULL, sCaption, pSelectedFilter, bResolveSymlinks);
}

//-----------------------------------------------------------------------------------
QStringList CMessageBox::ChooseOpenFileNames(const QString& sFilter, const QString& sDir, QWidget* pParent, const QString& sCaption, QString* pSelectedFilter, bool bResolveSymlinks)
{
	return QFileDialog::getOpenFileNames(sFilter, sDir, pParent, NULL, sCaption, pSelectedFilter, bResolveSymlinks);
}

//-----------------------------------------------------------------------------------
QString CMessageBox::ChooseSaveFileName(const QString& sStartWith, const QString& sFilter, QWidget* pParent, const QString& sDefaultExt, bool bConfirmErase, const QString& sCaption, QString* pSelectedFilter, bool bResolveSymlinks)
{
	QString sName = sStartWith;
	while(true) {
		sName = QFileDialog::getSaveFileName(sName, sFilter, pParent, NULL, sCaption, pSelectedFilter, bResolveSymlinks);
		if(sName.isEmpty()) {
			return sName;
		}
		if(!sDefaultExt.isEmpty()) {
			if(sName.findRev('.', sName.length() - 1) <= sName.findRev('/', sName.length() - 1)) {
				sName += "." + sDefaultExt;
			}
		}
		if(bConfirmErase) {
			QFile file(sName);
			if(file.exists()) {
				if(!MsgQuestion(pParent, QObject::tr("Voulez-vous vraiment craser le fichier '%1' ?").arg(sName))) {
					continue;
				}
			}
		}
		return sName;
	}
}

//-----------------------------------------------------------------------------------
QString CMessageBox::ChooseExistingDirectory(const QString& sDir, QWidget* pParent, const QString& sCaption, bool bDirOnly, bool bResolveSymlinks)
{
	return QFileDialog::getExistingDirectory(sDir, pParent, NULL, sCaption, bDirOnly, bResolveSymlinks);
}
