
#ifndef MESSAGEBOX_H
#define MESSAGEBOX_H

#include <qmessagebox.h>
#include <qprogressdialog.h>

#include <F100Qt.h>

/*!
	\class CMessageBox MessageBox.h
	\brief Permet d'afficher des botes de dialogue
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
*/
class DLLEXPORT CMessageBox
{
public:
	/** Permet d'afficher une fentre d'erreur
	 * @param[in] parent Fentre parente
	 * @param[in] sText Texte  afficher dans la bote de dialogue
	 */
	static void MsgCritical(QWidget* parent, const QString& sText);

	/** Permet d'afficher une fentre d'avertissement
	 * @param[in] parent Fentre parente
	 * @param[in] sText Texte  afficher dans la bote de dialogue
	 */
	static void MsgExclamation(QWidget* parent, const QString& sText);

	/** Permet d'afficher une fentre d'information
	 * @param[in] parent Fentre parente
	 * @param[in] sText Texte  afficher dans la bote de dialogue
	 */
	static void MsgInformation(QWidget* parent, const QString& sText);

	/** Permet d'afficher une fentre avec deux boutons OUI et NON
	 * @param[in] parent Fentre parente
	 * @param[in] sText Texte  afficher dans la bote de dialogue
	 * @return Retourne vrai si l'utilisateur a choisi OUI, sinon faux
	 */
	static bool MsgQuestion(QWidget* parent, const QString& sText);

	/** Permet d'afficher une fentre avec trois boutons OUI, NON et ANNULER
	 * @param[in] parent Fentre parente
	 * @param[in] sText Texte  afficher dans la bote de dialogue
	 * @return Retourne QMessageBox::Yes ou QMessageBox::No ou QMessageBox::Cancel 
	 */
	static int  MsgYesNoCancel(QWidget* parent, const QString& sText);

	/** Permet d'crire sur un fichier en rcrant une boite de progression pour voir l'avancement
	 * @param[in] parent Parent de la fentre
	 * @param[in] sFileName Chemin du fichier
	 * @param[in] pBuffer Donnes  sauver
	 * @param[in] nBytes Taille de la zone mmoire
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	static bool SavingFile(QWidget* parent, const QString& sFileName, const void* pBuffer, const unsigned int nBytes);

	/** Permet de lire un fichier en rcrant une boite de progression pour voir l'avancement
	 * @param[in] parent Parent de la fentre
	 * @param[in] sFileName Chemin du fichier  lire
	 * @param[out] pBuffer Mmoire pour stocker les donnes
	 * @param[in] nByteMax Taille maximale lue (en octets)
	 * @param[in] nReadPos Position de dpart dans le fichier (en octets)
	 * @param[in] nFillValue Valeur de remplissage si le fichier lu est plus petit que \a nByteMax
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	static bool ReadingFile(QWidget* parent, const QString& sFileName, void* pBuffer, unsigned int nByteMax, unsigned int nReadPos = 0, unsigned char nFillValue = 0xFF);

	/** Permet de crer une boite de progression non modale.\n
	 * Temps minimum avant apparition environ 2.5 secondes.\n
 	 * Appelle QProgressDialog::setProgress(0).
	 * @param[in] parent Parent de la fentre
	 * @param[in] sText Texte  afficher
	 * @param[in] nMaxValue Nombre de valeur maximum
	 * @param[in] sCaption Titre de la fentre
	 * @param[in] bIsCancelable A vrai si l'on veut permet l'annulation du traitement, sinon faux
	 * @return Retourne la fentre de transfert, NULL si manque de mmoire
	 */
	static QProgressDialog* CreateProgressDialog(QWidget* parent, const QString& sText, unsigned int nMaxValue, const QString& sCaption = QString::null, bool bIsCancelable = FALSE);


	/** Permet d'ouvrir une bote de dialogue pour choisir un fichier  ouvrir
	 * @param[in] sStartWith Chemin du fichier par dfaut
	 * @param[in] sFilter Filtre pour les types de fichiers  afficher
	 * @param[in] pParent Parent de la fentre de la bote de dialogue
	 * @param[in] sCaption Titre de la fentre
	 * @param[in] pSelectedFilter Filtre slectionn
	 * @param[in] bResolveSymlinks A vrai si l'ouverture d'un lien entraine l'ouverture de la cible, faux pour ouvrir le lien et pas la cible
	 * @return Retourne le fichier slectionn. Une chaine vide si annul
	 */
	static QString     ChooseOpenFileName(const QString& sStartWith, const QString& sFilter, QWidget* pParent, const QString& sCaption = QString::null, QString* pSelectedFilter = 0, bool bResolveSymlinks = TRUE);

	/** Permet d'ouvrir une bote de dialogue pour choisir un fichier  ouvrir
	 * @param[in] sFilter Filtre pour les types de fichiers  afficher
	 * @param[in] sDir Chemin du rpertoire par dfaut
	 * @param[in] pParent Parent de la fentre de la bote de dialogue
	 * @param[in] sCaption Titre de la fentre
	 * @param[in] pSelectedFilter Filtre slectionn
	 * @param[in] bResolveSymlinks A vrai si l'ouverture d'un lien entraine l'ouverture de la cible, faux pour ouvrir le lien et pas la cible
	 * @return Retourne la liste des fichiers slectionns. La liste est vide si l'utilisateur annule
	 */
	static QStringList ChooseOpenFileNames(const QString& sFilter, const QString& sDir, QWidget* pParent, const QString& sCaption = QString::null, QString* pSelectedFilter = 0, bool bResolveSymlinks = TRUE);

	/** Permet d'ouvrir une bote de dialogue pour choisir un fichier  sauver
	 * @param[in] sStartWith Chemin du fichier par dfaut
	 * @param[in] sFilter Filtre pour les types de fichiers  afficher
	 * @param[in] pParent Parent de la fentre de la bote de dialogue
	 * @param[in] sDefaultExt Extension par dfaut si le fichier choisi n'a pas d'extension
	 * @param[in] bConfirmErase A vrai pour informer que le fichier existe,  faux pour ne pas afficher ce message
	 * @param[in] sCaption Titre de la fentre
	 * @param[in] pSelectedFilter Filtre slectionn
	 * @param[in] bResolveSymlinks A vrai si l'ouverture d'un lien entraine l'ouverture de la cible, faux pour ouvrir le lien et pas la cible
	 * @return Retourne le fichier slectionn. Une chaine vide si annul
	 */
	static QString     ChooseSaveFileName(const QString& sStartWith, const QString& sFilter, QWidget* pParent, const QString& sDefaultExt = QString::null, bool bConfirmErase = TRUE, const QString& sCaption = QString::null, QString* pSelectedFilter = 0, bool bResolveSymlinks = TRUE);

	/** Permet d'ouvrir une bote de dialogue pour choisir un rpertoire  ouvrir
	 * @param[in] sDir Chemin du rpertoire par dfaut
	 * @param[in] pParent Parent de la fentre de la bote de dialogue
	 * @param[in] sCaption Titre de la fentre
	 * @param[in] bDirOnly A vrai pour afficher seulement les rpertoires, sinon  faux pour afficher les fichiers et les rpertoires
	 * @param[in] bResolveSymlinks A vrai si l'ouverture d'un lien entraine l'ouverture de la cible, faux pour ouvrir le lien et pas la cible
	 * @return Retourne le fichier slectionn. Une chaine vide si annul
	 */
	static QString     ChooseExistingDirectory(const QString& sDir, QWidget* pParent, const QString& sCaption = QString::null, bool bDirOnly = TRUE, bool bResolveSymlinks = TRUE);

private:
	/** Permet de connaitre le titre d'une fentre
	 * @param pWidget Fentre, si nulle alors QString::null sera retourn
	 * @return Retourne le titre de la fentre.
	 */
	static QString GetCaption(QWidget* pWidget);
};

#endif
