
#include "OpacityButton.h"

#include <qpopupmenu.h>
#include <qtooltip.h>

#include "image_defs.h"

#define CALL(o, m) if(o) o->m

//-----------------------------------------------------------------------------------
COpacityButton::COpacityButton(QWidget* parent, QWidget* pMonitoredWidget)
: QToolButton(parent)
{
	setIconSet(QPixmap::fromMimeSource(IMG_OPACITY));

	m_pMonitoredWidget = pMonitoredWidget;
	m_pMenu            = new QPopupMenu(this);
	
	for(int i = 10; i <= 100; i += 10) {
		m_pMenu->insertItem(QString::number(i) + " %", i / 10);
	}
	setPopup(m_pMenu);
	setPopupDelay(1);
	
	CheckItem(true);

	connect(m_pMenu, SIGNAL(activated(int)), this, SLOT(OnItemActivated(int)));

	languageChange();
}

//-----------------------------------------------------------------------------------
double COpacityButton::GetOpacity() const
{
#if QT_VERSION >= 0x030300
	if(m_pMonitoredWidget != NULL) {
		return m_pMonitoredWidget->windowOpacity();
	}
#endif
	return 1.0;
}

//-----------------------------------------------------------------------------------
void COpacityButton::SetOpacity(double dOpacity)
{
#if QT_VERSION >= 0x030300
	CheckItem(false);
	CALL(m_pMonitoredWidget, setWindowOpacity(dOpacity));
	CheckItem(true);
#endif
}

//-----------------------------------------------------------------------------------
bool COpacityButton::OpacityIsAvalaible()
{
#if QT_VERSION < 0x030300 || (!defined Q_WS_WIN && !defined Q_WS_MAC)
	return false;
#else
#ifdef Q_WS_WIN
	int nWinVersion = qWinVersion();
	return ((nWinVersion&WV_NT_based) && nWinVersion >= WV_2000);
#endif
#ifdef Q_WS_MAC
	return true;
#endif
#endif
}

//-----------------------------------------------------------------------------------
void COpacityButton::OnItemActivated(int id)
{
	SetOpacity(id / 10.0);
	emit OpacityChanged();
}

//-----------------------------------------------------------------------------------
void COpacityButton::CheckItem(bool bCheck)
{
	int nOpacity = (int)(GetOpacity() * 10 + 0.5);
	if(nOpacity >= 0) {
		m_pMenu->setItemChecked(nOpacity, bCheck);
	}
}

//-----------------------------------------------------------------------------------
void COpacityButton::languageChange()
{
	QToolTip::add(this, tr("Rgle l'opacit de la fentre"));
}
