// OpacityButton.h: interface for the COpacityButton class.
//
//////////////////////////////////////////////////////////////////////

#ifndef OPACITYBUTTON_H
#define OPACITYBUTTON_H

#include <qtoolbutton.h>
#include <qdom.h>

#include <F100Qt.h>

class QPopupMenu;

/*!
	\class COpacityButton OpacityButton.h
	\brief Bouton permettant de dfinir l'opacit d'une fentre
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
*/
class DLLEXPORT COpacityButton : public QToolButton
{
	Q_OBJECT
public:
	/** Constructeur
	 * @param[in] parent Fentre parente
	 * @param[in] pMonitoredWidget Fentre sur laquelle on veut rgler l'opacit
	 */
	COpacityButton(QWidget* parent, QWidget* pMonitoredWidget);

	/** @return L'opacit de la fentre surveille [0, 1] */
	double GetOpacity() const;

	/** Peremet de dfinir l'opacit de la fentre surveille
	 * @param[in] dOpacity Opacit souhait [0, 1]
	 */
	void   SetOpacity(double dOpacity);

public:
	/** Permet de savoir si le systme gre l'opacit
	 * @return Retourne vrai si l'opacit est supporte, sinon faux
	 */
	static bool OpacityIsAvalaible();

protected slots:
	/** Appel lorsque la langue change */
    void languageChange();
	
	/** Slot pour capturer une slection d'opacit dans le menu
	 * @param[in] id Identifiant du menu
	 */
	void OnItemActivated(int id);

signals:
	/** Emit lorsque l'opacit change */
	void OpacityChanged();

private:
	/** Marque l'lment du menu correspondant  l'opacit de la fentre surveille
	 * @param[in] bCheck A vrai si on veut marquer l'lment,  faux pour le dselectionner
	 */
	void CheckItem(bool bCheck);

private:
	QWidget*    m_pMonitoredWidget;
	QPopupMenu* m_pMenu;
};

#endif
