// SendToMenu.cpp: implementation of the CSendToMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "SendToMenu.h"
#include "ToolWindow.h"

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CSendToMenu::CSendToMenu(QWidget* parent)
:QPopupMenu(parent)
{
}

//-----------------------------------------------------------------------------------
void CSendToMenu::AddSendToItem(CToolWindow* pTool, QPopupMenu* pItem)
{
	if(pTool != NULL && pItem != NULL) {
		int id;
		if(pTool->icon() != NULL) {
			id = insertItem((*pTool->icon()), pTool->caption(), pItem);
		}
		else {
			id = insertItem(pTool->caption(), pItem);
		}
		if(id != -1) {
			m_mItems[pTool] = id;
			pTool->installEventFilter(this);
			connect(pTool, SIGNAL(destroyed(QObject*)), this, SLOT(OnToolDestroyed(QObject*)));
		}
	}
}

//-----------------------------------------------------------------------------------
void CSendToMenu::AddSendToItem(CToolWindow* pTool, const char* member)
{
	if(pTool != NULL && member != NULL) {
		int id;
		if(pTool->icon() != NULL) {
			id = insertItem((*pTool->icon()), pTool->caption(), pTool, member);
		}
		else {
			id = insertItem(pTool->caption(), pTool, member);
		}
		if(id != -1) {
			m_mItems[pTool] = id;
			pTool->installEventFilter(this);
			connect(pTool, SIGNAL(destroyed(QObject*)), this, SLOT(OnToolDestroyed(QObject*)));
		}
	}
}

//-----------------------------------------------------------------------------------
void CSendToMenu::SetFileToSend(const QString& sFileName, int nNbFile)
{
	if(m_sFileName != sFileName || m_nNbFile != nNbFile) {
		m_sFileName = sFileName;
		m_nNbFile   = nNbFile;
		emit FileToSendChanged(sFileName, nNbFile);
	}
}

//-----------------------------------------------------------------------------------
void CSendToMenu::SetItemEnabled(CToolWindow* pTool, bool bEnabled)
{
	ItemMap::Iterator it = m_mItems.find(pTool);
	if(it != m_mItems.end()) {
		setItemEnabled(it.data(), bEnabled);
	}
}

//-----------------------------------------------------------------------------------
bool CSendToMenu::eventFilter(QObject* pWatched, QEvent* e)
{
	CToolWindow* pTool   = (CToolWindow*)pWatched;
	ItemMap::Iterator it = m_mItems.find(pTool);
	if(it != m_mItems.end()) {
		if(e->type() == QEvent::CaptionChange || e->type() == QEvent::IconChange) {
			if(pTool->icon() != NULL) {
				changeItem(it.data(), *pTool->icon(), pTool->caption());
			}
			else {
				changeItem(it.data(), pTool->caption());
			}
		}
	}
	return QPopupMenu::eventFilter(pWatched, e);
}

//-----------------------------------------------------------------------------------
const QString& CSendToMenu::GetFileName() const
{
	return m_sFileName;
}

//-----------------------------------------------------------------------------------
int CSendToMenu::GetNbFile() const
{
	return m_nNbFile;
}

//-----------------------------------------------------------------------------------
void CSendToMenu::OnToolDestroyed(QObject* pObject)
{
	assert(pObject != NULL);
	CToolWindow* pTool   = (CToolWindow*)pObject;
	ItemMap::Iterator it = m_mItems.find(pTool);
	_ASSERTE(it != m_mItems.end());
	removeItem(it.data());
	m_mItems.erase(it);
}
