
#ifndef SENDTOMENU_H
#define SENDTOMENU_H

#include <qpopupmenu.h>
#include <qmap.h>

#include <F100_Op.h>

class CToolWindow;

#ifdef _MSC_VER
#	pragma warning(disable:4251)
#endif

/*!
	\class CSendToMenu SendToMenu.h
	\brief Menu 'Envoyer vers'
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt
	Cette classe permet d'associer un outil au menu 'Envoyer vers' de l'application et aussi de permettre  un outil d'utiliser
	le menu 'Envoyer vers' pour permettre  l'utilisateur d'envoyer un fichier dans un autre outil
*/
class DLLEXPORT CSendToMenu : public QPopupMenu 
{
	Q_OBJECT
public:
	/** Constructeur
	 * @param[in] parent Parent du menu
	 */
	CSendToMenu(QWidget* parent = 0);

	/** Permet d'ajouter un lment dans le menu 'Envoyer vers'
	 * @param[in] pTool Outil associ  cet lment
	 * @param[in] pItem Menu  ajouter
	 */
	void           AddSendToItem(CToolWindow* pTool, QPopupMenu* pItem);

	/** Permet d'ajouter un lment dans le menu 'Envoyer vers'
	 * @param[in] pTool Outil associ  cet lment
	 * @param[in] member Slot qui va tre appel lorsque l'utilisateur cliquera sur cet lment ajout
	 */
	void           AddSendToItem(CToolWindow* pTool, const char* member);

	/** Permet de dfinir le fichier qui est suceptible d'tre envoy
	 * @param[in] sFileName Chemin complet du fichier
	 * @param[in] nNbFile Paramtre valable uniquement pour les lecteurs multiples. Indique combien de fichiers seront envoys.
	 */
	void           SetFileToSend(const QString& sFileName, int nNbFile = 1);

	/** Permet d'activer ou dsactiver un lment du menu 'Envoyer vers'
	 * @param[in] pTool Outil de l'lment concern
	 * @param[in] bEnabled A vrai pour activer l'lment du menu, sinon  faux
	 */
	void           SetItemEnabled(CToolWindow* pTool, bool bEnabled);

	/** Filtre d'vnements */
	bool           eventFilter(QObject* pWatched, QEvent* e);

	/** @return Le nom du fichier  envoyer */
	const QString& GetFileName() const;

	/** @return Le nombre de fichier  envoyer */
	int            GetNbFile() const;

signals:
	/** Signal mit lorsque les donnes du fichier  envoyer ont chang
	 * @param[in] sFileName Chemin complet du fichier suceptible d'tre envoy
	 * @param[in] nNbFile Paramtre valable uniquement pour les lecteurs multiples. Indique combien de fichiers seront envoys.
	 */
	void           FileToSendChanged(const QString& sFileName, int nNbFile);

private slots:
	/** Fonction appele lorsqu'un outil est dtruit
	 * @param[in] pObject Outil dtruit
	 */
	void           OnToolDestroyed(QObject* pObject);

private:
	typedef QMap<CToolWindow*, int> ItemMap;

private:
	ItemMap   m_mItems;
	QString   m_sFileName;
	int       m_nNbFile;
};

#endif
