// SettingTabWidget.cpp: implementation of the CSettingTabWidget class.
//
//////////////////////////////////////////////////////////////////////

#include "SettingTabWidget.h"

#include <IMainWindow.h>

#include <assert.h>

//-----------------------------------------------------------------------------------
CSettingTabWidget::CSettingTabWidget(IMainWindow* pMainWindow, const QString& sKey, bool bRemoveRecursive, bool bShowInList)
	: QWidget()
	, m_pMainWindow(pMainWindow)
	, m_sKey(sKey + "/")
	, m_pParent(NULL)
	, m_bRemoveRecursive(bRemoveRecursive)
	, m_bShowInList(bShowInList)
{
	m_pChildren = new SettingTabList();
}

//-----------------------------------------------------------------------------------
CSettingTabWidget::~CSettingTabWidget()
{
	close();
	SettingTabList::iterator it = m_pChildren->begin();
	for(; it != m_pChildren->end(); it++) {
		(*it)->SetParent(NULL);
	}
	delete m_pChildren;
}

//-----------------------------------------------------------------------------------
const CSettingTabWidget::SettingTabList* CSettingTabWidget::GetChildren() const
{
	return m_pChildren;
}

//-----------------------------------------------------------------------------------
CSettingTabWidget* CSettingTabWidget::GetParent() const
{
	return m_pParent;
}

//-----------------------------------------------------------------------------------
void CSettingTabWidget::SetParent(CSettingTabWidget* pParent)
{
	assert(m_pParent == NULL || pParent == NULL);
	m_pParent = pParent;
}

//-----------------------------------------------------------------------------------
void CSettingTabWidget::AddChild(CSettingTabWidget* pChild)
{
	if(!m_pChildren->contains(pChild)) {
		pChild->SetParent(this);
		m_pChildren->push_back(pChild);
		emit ChildAdded(pChild);
	}
}

//-----------------------------------------------------------------------------------
void CSettingTabWidget::RemoveChild(CSettingTabWidget* pChild)
{
	if(m_pChildren->remove(pChild) > 0) {
		pChild->SetParent(NULL);
		emit ChildRemoved(pChild);
	}
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::IsShownInList() const
{
	return m_bShowInList;
}

//-----------------------------------------------------------------------------------
IMainWindow* CSettingTabWidget::GetMainWindow() const
{
	return m_pMainWindow;
}

//-----------------------------------------------------------------------------------
const QString& CSettingTabWidget::GetKey() const
{
	return m_sKey;
}

//-----------------------------------------------------------------------------------
int CSettingTabWidget::RestoreDefaults()
{
	m_pMainWindow->RemoveSettings(m_sKey, m_bRemoveRecursive);
	DoRefresh();
	return 1;
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::SortChildrenInList() const
{
	return TRUE;
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::WriteSettings(const QString& sPath, const int& nNumber)
{
	return m_pMainWindow->WriteSettings(m_sKey + sPath, nNumber);
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::WriteSettings(const QString& sPath, const QString& sText)
{
	return m_pMainWindow->WriteSettings(m_sKey + sPath, sText);
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::ReadSettings(const QString& sPath, int& nNumber, const int& nDefault) const
{
	return m_pMainWindow->ReadSettings(m_sKey + sPath, nNumber, nDefault);
}

//-----------------------------------------------------------------------------------
bool CSettingTabWidget::ReadSettings(const QString& sPath, QString& sText, const QString& sDefault) const
{
	return m_pMainWindow->ReadSettings(m_sKey + sPath, sText, sDefault);
}

//-----------------------------------------------------------------------------------
QStringList CSettingTabWidget::SettingsList(const QString& sPath) const
{
	return m_pMainWindow->SettingsList(m_sKey + sPath);
}

//-----------------------------------------------------------------------------------
QStringList CSettingTabWidget::SubSettingsList(const QString& sPath) const
{
	return m_pMainWindow->SubSettingsList(m_sKey + sPath);
}
