// SettingsManager.cpp: implementation of the CSettingsManager class.
//
//////////////////////////////////////////////////////////////////////

#include "SettingsManager.h"
#include "SettingTabWidget.h"
#include "language.h"
#include "FileList.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qheader.h>
#include <qlabel.h>

#include <assert.h>

#define SEND_TO_ALL(a) 	ItemMap::iterator it  = m_pItems->begin(); \
						ItemMap::iterator end = m_pItems->end(); \
						for(; it != end; it++) { \
							it.key()->a; \
						}

//-----------------------------------------------------------------------------------
CSettingsManager::CSettingsManager(QWidget* parent)
	: QWidget(parent, 0)
	, m_pSelectedWidget(NULL)
{
	m_pLayout  = new QGridLayout(this, 5, 4, 5);
	m_pList    = new QListView(this);
	m_pTitle   = new QLabel(this);
	m_pApply   = new QPushButton(this);
	m_pRestore = new QPushButton(this);
	m_pItems   = new ItemMap();

	QLabel* pLine1 = new QLabel(this);
	QLabel* pLine2 = new QLabel(this);
 
	pLine1->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	pLine1->setMaximumHeight(2);

	pLine2->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	pLine2->setMaximumHeight(2);

	m_pLayout->addMultiCellWidget(m_pList,  0, 4, 0, 0);
	m_pLayout->addMultiCellWidget(m_pTitle, 0, 0, 1, 3);
	m_pLayout->addMultiCellWidget(pLine1,   1, 1, 1, 3, AlignTop);
	m_pLayout->addMultiCellWidget(pLine2,   4, 4, 1, 3, AlignBottom);
	m_pLayout->addWidget(m_pRestore, 3, 2, AlignBottom);
	m_pLayout->addWidget(m_pApply,   3, 3, AlignBottom);

	m_pLayout->setRowStretch(0, 0);
	m_pLayout->setRowStretch(1, 0);
	m_pLayout->setRowStretch(2, 8);
	m_pLayout->setRowStretch(3, 0);
	m_pLayout->setRowStretch(4, 0);

	m_pLayout->setColStretch(0, 2);
	m_pLayout->setColStretch(1, 3);
	m_pLayout->setColStretch(2, 0);
	m_pLayout->setColStretch(3, 0);

	m_pLayout->setRowSpacing(0, 200);

	m_pList->setFrameStyle(QFrame::Box | QFrame::Sunken);
	m_pList->setLineWidth(1);
	m_pList->setMidLineWidth(0);
	m_pList->header()->hide();
	m_pList->addColumn(QString::null);
	m_pList->setSorting(-1, TRUE);
	m_pList->setRootIsDecorated(TRUE);
	m_pList->setResizePolicy(QScrollView::AutoOneFit);

	connect(m_pApply,   SIGNAL(clicked()), SLOT(OnApply()));
	connect(m_pRestore, SIGNAL(clicked()), SLOT(OnRestore()));
	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(OnSelectionChanged(QListViewItem*)));
	
	languageChange();
}

//-----------------------------------------------------------------------------------
CSettingsManager::~CSettingsManager()
{
	delete m_pItems;
}

//-----------------------------------------------------------------------------------
void CSettingsManager::languageChange()
{
	m_pRestore->setText(tr("Par dfaut"));
	m_pApply->setText(CLanguage::GetWordTranslation(CLanguage::WO_APPLY));
	
	ItemMap::iterator it  = m_pItems->begin();
	ItemMap::iterator end = m_pItems->end();
	for(; it != end; it++) {
		it.key()->DoInitWords();
	}
	RefreshSelection();
}

//-----------------------------------------------------------------------------------
CSettingTabWidget* CSettingsManager::GetSelectedWidget() 
{
	return m_pSelectedWidget;
}

//-----------------------------------------------------------------------------------
bool CSettingsManager::AddItem(CSettingTabWidget* pWidget)
{
	return AddItem(pWidget, NULL);
}

//-----------------------------------------------------------------------------------
bool CSettingsManager::AddItem(CSettingTabWidget* pWidget, QListViewItem* parent)
{
	if(pWidget != NULL && m_pItems->find(pWidget) == m_pItems->end()) {
		QListViewItem* pItem;
		connect(pWidget, SIGNAL(ChildAdded(CSettingTabWidget*)),   this, SLOT(AddItem(CSettingTabWidget*)));
		connect(pWidget, SIGNAL(ChildRemoved(CSettingTabWidget*)), this, SLOT(AddItem(CSettingTabWidget*)));
		if(parent == NULL && pWidget->GetParent() != NULL) {
			ItemMap::iterator it = m_pItems->find(pWidget->GetParent());
			if(it != m_pItems->end()) {
				parent = it.data();
			}
		}
		if(parent != NULL) {
			pItem =	new QListViewItem(parent);
			if(pWidget->IsShownInList()) {
				parent->setOpen(TRUE);
			}
		}
		else {
			pItem =	new QListViewItem(m_pList, CFileList::LastItem(m_pList->firstChild()));
		}
		pWidget->installEventFilter(this);
		pWidget->reparent(this, QPoint(0, 0), TRUE);
		pWidget->hide();
		m_pItems->insert(pWidget, pItem);
		RefreshInfo(m_pItems->find(pWidget));
		if(m_pSelectedWidget == NULL) {
			Show(pWidget);
		}
		CSettingTabWidget::SettingTabList::const_iterator it  = pWidget->GetChildren()->begin();
		CSettingTabWidget::SettingTabList::const_iterator end = pWidget->GetChildren()->end();
		for(; it != end; it++) {
			AddItem(*it, pItem);
		}
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
bool CSettingsManager::RemoveItem(CSettingTabWidget* pWidget)
{
	return RemoveItem(pWidget, TRUE);
}

//-----------------------------------------------------------------------------------
bool CSettingsManager::RemoveItem(CSettingTabWidget* pWidget, bool bDeleteListViewItem)
{
	ItemMap::iterator it = m_pItems->find(pWidget);
	if(it != m_pItems->end()) {
		pWidget->removeEventFilter(this);
		disconnect(pWidget);
		if(bDeleteListViewItem) {
			delete it.data();
		}
		m_pItems->remove(it);
		CSettingTabWidget::SettingTabList::const_iterator it  = pWidget->GetChildren()->begin();
		CSettingTabWidget::SettingTabList::const_iterator end = pWidget->GetChildren()->end();
		for(; it != end; it++) {
			RemoveItem(*it, FALSE);
		}
		if(GetSelectedWidget() == pWidget) {
			Show(pWidget->GetParent());
		}
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
void CSettingsManager::Show(CSettingTabWidget* pWidget)
{
	if(m_pSelectedWidget != pWidget) {
		ItemMap::iterator it = m_pItems->find(pWidget);
		if(pWidget == NULL ||it != m_pItems->end()) {
			if(m_pSelectedWidget != NULL) {
				m_pLayout->remove(m_pSelectedWidget);
				m_pSelectedWidget->hide();
			}
			if(pWidget != NULL) {
				m_pLayout->addMultiCellWidget(pWidget, 2, 2, 1, 3);
				pWidget->show();
				m_pList->setSelected(it.data(), TRUE);
			}
			else {
				m_pList->clearSelection();
			}
			m_pSelectedWidget = pWidget;
			RefreshSelection();
		}
	}
}

//-----------------------------------------------------------------------------------
void CSettingsManager::RefreshInfo(CSettingTabWidget* pWidget)
{
	RefreshInfo(m_pItems->find(pWidget));
}

//-----------------------------------------------------------------------------------
void CSettingsManager::RefreshInfo(ItemMap::iterator it)
{
	if(it != m_pItems->end()) {
		QListViewItem* pItem = it.data();
		pItem->setText(0, it.key()->caption());
		if(pItem->parent() != NULL) {
			pItem->parent()->sortChildItems(0, Find(pItem->parent())->SortChildrenInList());
		}
	}
}

//-----------------------------------------------------------------------------------
void CSettingsManager::RefreshSelection()
{
	if(m_pSelectedWidget != NULL) {
		QString s = "<font size=+3><b>%1</b></font>";
		s = s.arg(m_pSelectedWidget->caption());
		if(!m_pSelectedWidget->GetDescription().isEmpty()) {
			s += "<br>";
			s += m_pSelectedWidget->GetDescription();
		}
		m_pTitle->setText(s);
	}
	else {
		m_pTitle->setText(QString::null);
	}
}

//-----------------------------------------------------------------------------------
CSettingTabWidget* CSettingsManager::Find(QListViewItem* pItem)
{
	ItemMap::iterator it  = m_pItems->begin();
	ItemMap::iterator end = m_pItems->end();
	for(; it != end; it++) {
		if(it.data() == pItem) {
			return it.key();
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
void CSettingsManager::OnSelectionChanged(QListViewItem* pItem)
{
	Show(Find(pItem));
}

//-----------------------------------------------------------------------------------
void CSettingsManager::OnApply()
{
	if(GetSelectedWidget() != NULL) {
		if(GetSelectedWidget()->DoSave() == -2) {
			emit NeedRestartApplication();
		}
	}
}

//-----------------------------------------------------------------------------------
void CSettingsManager::OnRestore()
{
	if(GetSelectedWidget() != NULL) {
		if(GetSelectedWidget()->RestoreDefaults() == -2) {
			emit NeedRestartApplication();
		}
	}
}

//-----------------------------------------------------------------------------------
void CSettingsManager::SaveAll()
{
	bool bNeedRestart = FALSE;
	ItemMap::iterator it = m_pItems->begin();
	ItemMap::iterator end = m_pItems->end();
	for(; it != end; it++) {
		if(it.key()->DoSave() == -2) {
			bNeedRestart = TRUE;
		}
	}
	if(bNeedRestart) {
		emit NeedRestartApplication();
	}
}

//-----------------------------------------------------------------------------------
void CSettingsManager::RefreshAll()
{
	SEND_TO_ALL(DoRefresh());
}

//-----------------------------------------------------------------------------------
bool CSettingsManager::eventFilter(QObject* o, QEvent* e)
{
	CSettingTabWidget* pWidget = (CSettingTabWidget*)o;
	switch(e->type())
	{
	case QEvent::Close:
		if(RemoveItem(pWidget)) {
			return FALSE;
		}
		break;
	case QEvent::CaptionChange:
		RefreshInfo(pWidget);
		break;
	}

	return QWidget::eventFilter(o, e);
}
