// SettingsManager.h: interface for the CSettingsManager class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _SETTINGSMANAGER_H_
#define _SETTINGSMANAGER_H_

#include <qwidget.h>
#include <qmap.h>

#include <F100_Op.h>

class CSettingTabWidget;
class QLabel;
class QListView;
class QListViewItem;
class QPushButton;
class QGridLayout;
class QBoxLayout;

/*!
	\class CSettingsManager SettingsManager.h
	\brief Permet d'afficher des pages de rglages
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
	Fentre qui affiche plusieurs pages d'options de type CSettingTabWidget.\n
	Les pages sont affiches  droite de la fentre et sur la gauche est prsente une liste affichant
	toutes les fentres de rglages disponibles.\n
	Le titre d'une fentre, caption(), est utilis pour tre mis dans la liste.
*/
class DLLEXPORT CSettingsManager : public QWidget
{
	Q_OBJECT
public:

	/** Constructeur
	 * @param[in] parent Fentre parente
	 */
	CSettingsManager(QWidget* parent);

	/** Destructeur */
	virtual ~CSettingsManager();

	/** @return La fentre en cours de visualisation, ou NULL si aucune */
	CSettingTabWidget* GetSelectedWidget();

	/** Permet de sauver tous les rglages de toutes les fentres */
	void SaveAll();

	/** Permet de recharger tous les rglages de toutes les fentres */
	void RefreshAll();

protected:
	bool eventFilter(QObject* o, QEvent* e);

public slots:
	/** Permet d'ajouter une fentre de rglages et ses enfants
	 * @param[in] pWidget Fentre  ajouter
	 * @return Vrai si ajoute, sinon faux
	 */
	bool AddItem(CSettingTabWidget* pWidget);

	/** Permet de supprimer une fentre de rglages et ses enfants
	 * @param[in] pWidget Fentre  supprimer
	 * @return Vrai si supprime, sinon faux
	 */
	bool RemoveItem(CSettingTabWidget* pWidget);

	/** Permet d'afficher une fentre de rglages
	 * @param[in] pWidget Fentre  afficher
	 */
	void Show(CSettingTabWidget* pWidget);

protected slots:
	void languageChange();

private slots:
	void OnSelectionChanged(QListViewItem* pItem);
	void OnApply();
	void OnRestore();

signals:
	/** Emit lors qu'une ou plusieurs fentres de rglages demande  redmarrer l'application */
	void NeedRestartApplication();

private:
	typedef QMap<CSettingTabWidget*, QListViewItem*> ItemMap;

private:
	bool AddItem(CSettingTabWidget* pWidget, QListViewItem* parent);
	bool RemoveItem(CSettingTabWidget* pWidget, bool bDeleteListViewItem);
	void RefreshInfo(CSettingTabWidget* pWidget);
	void RefreshInfo(ItemMap::iterator it);
	void RefreshSelection();
	CSettingTabWidget* Find(QListViewItem* pItem);

private:
	QGridLayout* m_pLayout;
	QListView*   m_pList;
	QLabel*      m_pTitle;
	QPushButton* m_pApply;
	QPushButton* m_pRestore;
	ItemMap*     m_pItems;
	CSettingTabWidget* m_pSelectedWidget;
};

#endif
