
#include "ToolWindow.h"

#include <qlayout.h>
#include <qtabwidget.h>
#include <qiconset.h>
#include <qtoolbutton.h>
#include <qapplication.h>

#include <DriveAccess.h>
#include <F100_Debug.h>

#include "MessageBox.h"
#include "language.h"
#include "OpacityButton.h"
#include "image_defs.h"

#define CALL(o, m) if(o) o->m
char appHeader[] = "[myapp]";

/** Classe interne reprsentant la fentre principale de l'outil */
class CMainWidget : public QWidget
{
public:
	CMainWidget(CToolWindow* pToolWindow, bool bAllowTabDock, bool bAllowOpacityControl)
		: QWidget(pToolWindow) {
		m_pToolWindow    = pToolWindow;
		m_pWidgetLayout  = new QBoxLayout(this,            QBoxLayout::LeftToRight);
		m_pButtonLayout  = new QBoxLayout(m_pWidgetLayout, QBoxLayout::TopToBottom);
		m_pCentralWidget = NULL;
		m_pDockButton    = NULL;
		m_pOpacityButton = NULL;
		m_nNbButton      = 0;
		
		m_pButtonLayout->addStretch(100);

		if(bAllowTabDock) {
			m_pDockButton  = new QToolButton(this);
			AddLeftWidget(m_pDockButton);
			m_pDockButton->setAutoRaise(TRUE);	
			connect(m_pDockButton,  SIGNAL(clicked()), m_pToolWindow, SLOT(ToggleDisplayMode()));
		}
		if(bAllowOpacityControl) {
			m_pOpacityButton = new COpacityButton(this, m_pToolWindow);
			m_pOpacityButton->setAutoRaise(TRUE);
			AddLeftWidget(m_pOpacityButton);
			connect(m_pOpacityButton, SIGNAL(OpacityChanged()), m_pToolWindow, SIGNAL(StateChanged()));
		}
		RefreshButtons(false);
	}
	bool close(bool alsoDelete) { return m_pToolWindow->close(); }
	void RefreshButtons(bool bInTabWidget) {
		if(bInTabWidget) {
			CALL(m_pOpacityButton, hide());
		}
		else {
			CALL(m_pOpacityButton, show());
		}
		if(m_pDockButton != NULL) {
			QPixmap pixmap;
			if(bInTabWidget) {
				pixmap = QPixmap::fromMimeSource(IMG_DOCK);
			}
			else {
				pixmap = QPixmap::fromMimeSource(IMG_UNDOCK);
			}
			m_pDockButton->setIconSet(pixmap);
			m_pDockButton->setTextLabel(CToolWindow::tr("Intgrer/Sortir la fentre de la fentre principale"), TRUE);
		}
	}
	void AddLeftWidget(QWidget* pWidget) {
		if(pWidget != NULL) {
			m_pButtonLayout->insertWidget(m_nNbButton++, pWidget);
		}
	}
	void SetCentralWidget(QWidget* pWidget) {
		if(m_pCentralWidget != NULL) {
			m_pWidgetLayout->remove(m_pCentralWidget);
		}
		m_pCentralWidget = pWidget;
		if(m_pCentralWidget != NULL) {
			m_pCentralWidget->reparent(this, QPoint(0, 0), TRUE);
			m_pWidgetLayout->insertWidget(1, m_pCentralWidget);
		}
	}
public:
	CToolWindow*    m_pToolWindow;
	QToolButton*    m_pDockButton;
	COpacityButton* m_pOpacityButton;
	QWidget*        m_pCentralWidget;
	QBoxLayout*     m_pWidgetLayout;
	QBoxLayout*     m_pButtonLayout;
	int             m_nNbButton;
};

//-----------------------------------------------------------------------------------
CToolWindow::CToolWindow(CONSTRUCTOR_PARAM)
	: QWidget(parent, 0, fl | WDestructiveClose)
{
	m_pMainLayout      = new QBoxLayout(this, QBoxLayout::LeftToRight);
	m_pTabWidget       = pTabWidget;
	m_pMainWidget	   = new CMainWidget(this, m_pTabWidget != NULL, bAllowOpacityControl && parent == NULL && COpacityButton::OpacityIsAvalaible());
	m_bInTabWidget	   = false;
	m_pMainWindow      = pMainWindow;
	
	m_pMainLayout->addWidget(m_pMainWidget);
	if(bInTabWidget) {
		SetInTabWidget(bInTabWidget, FALSE, FALSE);
	}
	if(!isMaximized() && !isMinimized()) {
		m_rNormalGeometry = geometry();
	}
}

//-----------------------------------------------------------------------------------
CToolWindow::~CToolWindow()
{
	//Si la fentre principale n'est pas dans la fentre de l'outil, on l'accroche pour qu'elle soit dtruit avec.
	SetInTabWidget(FALSE, FALSE, FALSE);
}

//-----------------------------------------------------------------------------------
void CToolWindow::SetCentralWidget(QWidget* pWidget)
{
	assert(m_pMainWidget != NULL);
	m_pMainWidget->SetCentralWidget(pWidget);
}

//-----------------------------------------------------------------------------------
IMainWindow* CToolWindow::GetMainWindow() const
{
	return m_pMainWindow;
}

//-----------------------------------------------------------------------------------
QWidget* CToolWindow::GetMainWidget() const
{
	return m_pMainWidget;
}

//-----------------------------------------------------------------------------------
QWidget* CToolWindow::GetCentralWidget() const
{
	return m_pMainWidget->m_pCentralWidget;
}

//-----------------------------------------------------------------------------------
void CToolWindow::closeEvent(QCloseEvent* e)
{
	if(m_pMainWidget->m_pCentralWidget == NULL || m_pMainWidget->m_pCentralWidget->close()) {
		e->accept();
		return;
	}
	e->ignore();
}

//-----------------------------------------------------------------------------------
void CToolWindow::resizeEvent(QResizeEvent* e)
{
	QWidget::resizeEvent(e);
	if(!isMaximized() && !isMinimized()) {
		if(m_rNormalGeometry != geometry())	{
			m_rNormalGeometry = geometry();
			emit StateChanged();
		}
	}
	else {
		emit StateChanged();
	}
}

//-----------------------------------------------------------------------------------
void CToolWindow::moveEvent(QMoveEvent* e)
{
	QWidget::moveEvent(e);
	if(!isMaximized() && !isMinimized() && !m_bInTabWidget) {
		if(m_rNormalGeometry != geometry())	{
			m_rNormalGeometry = geometry();
			emit StateChanged();
		}
	}
}

//-----------------------------------------------------------------------------------
void CToolWindow::SetInTabWidget(bool bInTabWidget, bool bShowIt, bool bEmitStateChanged)
{
	if(m_pTabWidget && bInTabWidget != m_bInTabWidget) {
		m_bInTabWidget = bInTabWidget;
		
		if(m_bInTabWidget) {
			if(icon()) {
				m_pTabWidget->addTab(m_pMainWidget, QIconSet(*icon()), caption());
			}
			else {
				m_pTabWidget->addTab(m_pMainWidget, caption());
			}
			hide();
		}
		else {
			m_pTabWidget->removePage(m_pMainWidget);
			m_pMainWidget->reparent(this, QPoint(0, 0), TRUE);
			m_pMainLayout->addWidget(m_pMainWidget);
			m_pMainLayout->setMargin(0);
		}
		m_pMainWidget->RefreshButtons(m_bInTabWidget);
		if(bEmitStateChanged) {
			emit StateChanged();
		}
	}
	if(bShowIt) {
		show();
	}
}

//-----------------------------------------------------------------------------------
void CToolWindow::ToggleDisplayMode()
{
	if(m_pTabWidget) {
		SetInTabWidget(!m_bInTabWidget);
	}
}

//-----------------------------------------------------------------------------------
void CToolWindow::show()
{
	if(m_bInTabWidget && m_pTabWidget) {
		m_pTabWidget->showPage(m_pMainWidget);
	}
	else {
		QWidget::show();
		setActiveWindow();
		QWidget::show();
	}
}

//-----------------------------------------------------------------------------------
int CToolWindow::LoadFrom(QDomElement& element)
{
	assert(m_pMainWidget != NULL);
	QDomElement e = element.namedItem("CToolWindow").toElement();
	if(e.isNull()) {
		return -1;
	}
	if(m_pMainWidget->m_pOpacityButton != NULL) {
		m_pMainWidget->m_pOpacityButton->SetOpacity(e.attribute("Opacity", "1.0").toDouble());

	}

	bool bInTabWidget;
	int nStyle;
	m_rNormalGeometry.setX(e.attribute("x").toInt());
	m_rNormalGeometry.setY(e.attribute("y").toInt());
	m_rNormalGeometry.setWidth(e.attribute("w").toInt());
	m_rNormalGeometry.setHeight(e.attribute("h").toInt());
	
	nStyle       = e.attribute("Style",     "0").toInt();
	bInTabWidget = e.attribute("TabWidget", "0").toInt();
	
	setGeometry(m_rNormalGeometry);
	SetInTabWidget(bInTabWidget, false, false);
#if QT_VERSION < 0x030300
	switch(nStyle)
	{
	case 1:
		showMinimized();
		break;
	case 2:
		showMaximized();
		break;
	}
	hide();
#else
	setWindowState(nStyle);
#endif
	return 1;
}

//-----------------------------------------------------------------------------------
int CToolWindow::SaveTo(QDomElement& element)
{
	assert(m_pMainWidget != NULL);
	QDomElement e = element.ownerDocument().createElement("CToolWindow");
	element.appendChild(e);
	e.setAttribute("x", QString::number(m_rNormalGeometry.x()));
	e.setAttribute("y", QString::number(m_rNormalGeometry.y()));
	e.setAttribute("w", QString::number(m_rNormalGeometry.width()));
	e.setAttribute("h", QString::number(m_rNormalGeometry.height()));

#if QT_VERSION < 0x030300
	e.setAttribute("Style", QString::number((isMaximized() << 1) + isMinimized()));
#else
	e.setAttribute("Style", windowState());
#endif
	e.setAttribute("TabWidget", QString::number(m_bInTabWidget));

	if(m_pMainWidget->m_pOpacityButton != NULL) {
		e.setAttribute("Opacity", QString::number(m_pMainWidget->m_pOpacityButton->GetOpacity()));
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CToolWindow::languageChange()
{
	DoInitWords();
	DoShowCaption();
	m_pMainWidget->RefreshButtons(m_bInTabWidget);
}

//-----------------------------------------------------------------------------------
bool CToolWindow::event(QEvent* e)
{
	int nType = e->type();
	if(nType == QEvent::CaptionChange || nType == QEvent::IconChange) {
		if(m_pTabWidget != NULL && m_bInTabWidget) {
			if(icon()) {
				m_pTabWidget->changeTab(m_pMainWidget, QIconSet(*icon()), caption());
			}
			else {
				m_pTabWidget->changeTab(m_pMainWidget, caption());
			}
		}
		m_pMainWidget->setCaption(caption());
	}
	return QWidget::event(e);
}

//-----------------------------------------------------------------------------------
const QString& CToolWindow::GetWindowType()
{
	return m_sWindowType;
}

//-----------------------------------------------------------------------------------
void CToolWindow::SetWindowType(const QString& sWindowType)
{
	m_sWindowType = sWindowType;
}

//-----------------------------------------------------------------------------------
void CToolWindow::AddLeftWidget(QWidget* pWidget)
{
	assert(m_pMainWidget != NULL);
	m_pMainWidget->AddLeftWidget(pWidget);
}

//-----------------------------------------------------------------------------------
void CToolWindow::AddLeftSeparator()
{
	QFrame* pLine = new QFrame(GetMainWidget());
	pLine->setFrameShape(QFrame::HLine);
    pLine->setFrameShadow(QFrame::Sunken);
	AddLeftWidget(pLine);
}

//-----------------------------------------------------------------------------------
int CToolWindow::GetIndexInTabWidget() const
{
	if(m_pTabWidget != NULL && m_bInTabWidget) {
		return m_pTabWidget->indexOf(GetMainWidget());
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CToolWindow::IsInTabWidget() const
{
	return m_bInTabWidget;
}

//-----------------------------------------------------------------------------------
QString CToolWindow::ConvertToRelativePath(const QString& sPath)
{
	QString s = sPath;
	if(s.startsWith(APP_DIR_PATH)) {
		s.remove(APP_DIR_PATH); 
		s.prepend(appHeader);
	}
	return s;
}

//-----------------------------------------------------------------------------------
QString CToolWindow::ConvertToFullPath(const QString& sPath)
{
	QString s = sPath;
	if(s.startsWith(appHeader)) {
		s.remove(0, sizeof(appHeader) - 1);
		s.prepend(APP_DIR_PATH);
	}
	return s;
}
