
#ifndef TOOLWINDOW_H
#define TOOLWINDOW_H

#include <qwidget.h>
#include <qdom.h>

#include "IMainWindow.h"

#include <F100Qt.h>

class QTabWidget;
class QBoxLayout;
class QToolButton;
class COpacityButton;
class CMainWidget;

/** Paramtre pour la construction de la classe (dans le header) */
#define CONSTRUCTOR_PARAMDEF QWidget* parent, IMainWindow* pMainWindow, WFlags fl = 0,  QTabWidget* pTabWidget = NULL, bool bAllowOpacityControl = true, bool bInTabWidget = false
/** Paramtre pour la construction de la classe (dans le source) */
#define CONSTRUCTOR_PARAM	 QWidget* parent, IMainWindow* pMainWindow, WFlags fl, QTabWidget* pTabWidget, bool bAllowOpacityControl, bool bInTabWidget
/** Paramtre pass au constructeur de CToolWindow */
#define CALL_PARAM           parent, pMainWindow, fl, pTabWidget, bAllowOpacityControl, bInTabWidget

/** Dfinition des paramtres pour la mthode AddFile() */
#define ADDFILE_PARAMDEF     const QString& sFileName = QString::null, int nShift = 0, void* pExtraParam = NULL
/** Paramtres pour la mthode AddFile() */
#define ADDFILE_PARAM        const QString& sFileName, int nShift, void* pExtraParam

/** Dfinition des paramtres pour la mthode Init() */
#define INIT_PARAMDEF        ADDFILE_PARAMDEF
/** Paramtres pour la mthode Init() */
#define INIT_PARAM           ADDFILE_PARAM

/*!
	\class CToolWindow ToolWindow.h
	\brief Permet de crer un plug-in pour l'application
	\author Quaranta David
	\version 1.01
	
	\ingroup F100Qt
	Classe abstraite permettant d'avoir une fentre dans le programme principal.\n
	Si vous crez un plug-in vous devez crer une classe qui drive de cette classe.
*/
class DLLEXPORT CToolWindow : public QWidget
{
	Q_OBJECT
public:
	/** Constructeur
	 * @param[in] parent Parent de la fentre
	 * @param[in] pMainWindow Fentre principale
	 * @param[in] fl Flags Qt de la fentre
	 * @param[in] pTabWidget TabWidget pouvant contenir cette fentre ou NULL si non voulu
	 * @param[in] bAllowOpacityControl A vrai si on veut que le rglage de la transparence de la fentre soit possible, sinon  faux
	 * @param[in] bInTabWidget A vrai si on veut que la fentre apparaisse dans la TabWidget, sinon  faux
	 */
	CToolWindow(CONSTRUCTOR_PARAMDEF);
	
	/** Destructeur */
	virtual ~CToolWindow();
	
	/** Dfini la fentre centrale du plug-in.
	 * C'est la fentre o sont affiches les informations.\n
	 * Le parent de cette fentre est automatiquement redirig
	 * sur GetMainWidget() (Utilise QObject::reparent())
	 * @param[in] pWidget Fentre centrale ou NULL pour supprimer la fentre centrale actuelle
	 * @remarks Par dfaut il n'y a pas de fentre centrale
	 */
	virtual void   SetCentralWidget(QWidget* pWidget);


	/** Permet d'ajouter un objet dans l'espace des boutons situs  gauche de la fentre.\n
	 * Les objets seront ajouts sous les boutons rservs.
	 * @param[in] pWidget Objet  ajouter
	 */
	virtual void   AddLeftWidget(QWidget* pWidget);

	/** Permet d'ajouter un sparateur entre les boutons situs  gauche de la fentre.\n
	 * Les sparateurs seront ajouts sous les boutons rservs.
	 */
	virtual void   AddLeftSeparator();

	/** @return Retourne la fentre principale du programme associe  cette classe */
	IMainWindow*   GetMainWindow() const;

	/** @return La fentre centrale actuelle
	 * @remarks Si cette fentre est ferme alors cela entrainera la suppression de cette fentre en tant que fentre centrale.
	 * @see SetCentralWidget(QWidget*)
	 */
	QWidget*       GetCentralWidget() const;

	/** @return La fentre principale de cette fentre.
	 * @remarks Cette fentre contient la fentre centrale plus les boutons.
	 * Utilisz cette fentre pour afficher des botes de dialogue par exemple.
	 * @remarks La demande de fermeture de cette fentre (close()) provoque la demande de fermeture
	 * de l'outil, si celui-ci accepte alors cette fentre ainsi que l'outil seront ferms, dans le
	 * cas contraire la fentre ne sera pas ferme.
	 */
	QWidget*       GetMainWidget() const;

	/** @return Retourne la position de cette fentre dans la QTabWidget
	 * ou -1 pour si la fentre n'est pas dans ou n'a pas de QTabWidget.
	 */
	int            GetIndexInTabWidget() const;

	/** @return Vrai si la fentre est dans la QTabWidget, sinon retourne faux */
	bool           IsInTabWidget() const;

	/** Bascule ou non la fentre dans la fentre principale de l'application
	 * @param[in] bInTabWidget Si vrai, alors bascule la fentre dans la fentre principale, sinon sort cette fentre de la fentre principale
	 * @param[in] bShowIt Si vrai, alors demande d'afficher la fentre
	 * @param[in] bEmitStateChanged Si vrai, alors envoi le signal StateChanged() pour signaler un changement
	 */
	virtual void   SetInTabWidget(bool bInTabWidget, bool bShowIt = true, bool bEmitStateChanged = true);

	/** Charge les donnes depuis un noeud XML
	 * @return -1 Erreur\n
	 * 0 Non implment\n
	 * 1 OK
	 */
	virtual int    LoadFrom(QDomElement& element);

	/** Sauve les donnes dans un noeud XML
	 * @return -1 Erreur\n
	 * 0 Non implment\n
	 * 1 OK
	 */
	virtual int    SaveTo(QDomElement& element);

	/** Capture les venements Qt
	 * @param[in] e Evnement Qt
	 * @return Retourne le rsultat de QWidget::event(e)
	 */
	virtual bool   event(QEvent* e);

	/** Initialisation de l'outil
	 * @param[in] sFileName Nom du fichier  ouvrir
	 * @param[in] nShift Dcalage en octets du fichier  lire
	 * @param[in] pExtraParam Paramtres optionnels propre  un outil
	 * @return -1 Fichier non valide (Pas forcement de message d'erreur car une
	 * extension de fichier peut avoir plusieurs type de donnes (ex:.lec (ROM-DISK, BACKUP, ...)) \n
	 * 0 Ne pas ouvrir car inutile (pas pu ouvrir le fichier, ou rien  afficher) \n
	 * 1 OK
	 * @remarks La mthode Init(INIT_PARAMDEF) n'est appele qu'une seule fois par l'application.
	 * @remarks L'application appelle cette mthode avec \a sFileName = QString::null lors du chargement d'un projet et
	 * ensuite elle appellera LoadFrom(QDomElement&)
	 */
	virtual int    Init(INIT_PARAMDEF) = 0;

	/** @return Le type de la fentre.
	 * @remarks Dfini par la fentre principale de l'application
	 * @see OP_WINDOWTYPESTR
	 */
	const QString& GetWindowType();

	/** Defini le type de la fentre
	 * @param[in] sWindowType Type de la fentre
	 * @remarks Dfini par la fentre principale de l'application
	 */
	void           SetWindowType(const QString& sWindowType);
	
	/** Permet d'ajouter un fichier  l'outil
	 * @remarks Par dfaut cette mthode retourne 0
	 * @pre OnlyOneInstance() doit retourner VRAI
	 * @return -1 Fichier non valide \n
	 * 0 Non implment \n
	 * 1 OK
	 */
	virtual int    AddFile(ADDFILE_PARAMDEF){return 0;}

	/** Permet de savoir si un outil ne peut avoir qu'une seule instance par application
	 * @remarks Par dfaut cette mthode retourne FAUX
	 * @return Retourne vrai si l'outil ne doit avoir qu'une seule instance dans une application, sinon faux
	 */
	virtual bool   OnlyOneInstance(){return  FALSE;}

public:
	/** Permet de convertir un chemin complet en un chemin relatif  l'application en rajoutant [myapp]/...
	 * @param[in] sPath Chemin  convertir
	 * @return Retourne le chemin relatif si possible, sinon retourne le chemin \a sPath
	 */
	static QString ConvertToRelativePath(const QString& sPath);

	/** Permet de convertir un chemin relatif  l'application en un chemin complet.
	 * @param[in] sPath Chemin  convertir
	 * @return Retourne le chemin complet
	 */
	static QString ConvertToFullPath(const QString& sPath);

protected:
	/** Capture l'vnement de fermeture de l'outil */
	virtual void closeEvent(QCloseEvent* e);

	/** Capture du redimensionnement de la fentre de l'outil pour signaler une modification 
	 * @see StateChanged()
	 */
	virtual void resizeEvent(QResizeEvent*);
	
	/** Capture du dplacement de la fentre de l'outil pour signaler une modification 
	 * @see StateChanged()
	 */
	virtual void moveEvent(QMoveEvent* e);

	/** Permet de mettre  jour le titre de la fentre de l'outil */
	virtual void DoShowCaption() = 0;

	/** Remet  jour les mots devant tre traduit */
	virtual void DoInitWords() = 0;

public slots:
	/** Bascule le mode d'affichage (Dans la fentre principale ou dehors) */
	void		 ToggleDisplayMode();

	/** Affiche correctement l'outil */
	virtual void show();

protected slots:
	/** Appel automatique lorsque la langue change.\n
	 * Vous pouvez appeler cette mthode pour mettre  jour vos textes  l'initialisation de votre outil
	 * @remarks Rimplmentez les mthodes DoShowCaption() et DoInitWords() pour grer plusieurs langues
	 */
    void         languageChange();

signals:	
	/** Emettre ce signal pour signaler la modification de la fentre
	 * pour que la fenetre principale sache que le projet est modifi
	 */
	void		 StateChanged();

	/** Emettre un warning
	 * @param[in] msg Message  afficher
	 */
	void         Warning(const QString& msg);

	/** Emettre une erreur
	 * @param[in] msg Message  afficher
	 */
	void         Error(const QString& msg);

	/** Emettre une information
	 * @param[in] msg Message  afficher
	 */
	void         Information(const QString& msg);
	
	/** Emettre une message dans la barre d'tat de la fentre principale de l'application
	 * @param[in] msg Message  afficher
	 * @param[in] ms Dure de l'affichage en millisecondes
	 */
	void         Message(const QString& msg, int ms);

private:
	IMainWindow* m_pMainWindow; ///< Fentre principale de l'application
	CMainWidget* m_pMainWidget;
	bool         m_bInTabWidget;
	QTabWidget*  m_pTabWidget;
	QBoxLayout*  m_pMainLayout;
	QRect        m_rNormalGeometry;
	QString      m_sWindowType;
};

#endif
