
#include "language.h"

#include <qtranslator.h>
#include <qobject.h>
#include <qapplication.h>
#include <qimage.h>

#include "MessageBox.h"
#include <APIFunction.h>


//-----------------------------------------------------------------------------------
QString CLanguage::GetWordTranslation(Words nWord)
{
	switch(nWord)
	{
	case WO_AUTHOR:
		return QObject::tr("Traduit par Quaranta David");
	case WO_INFO:
		return QObject::tr("email : david.quaranta@laposte.net");
	case WO_DRIVEFILTER:
		return GetWordTranslation(WO_DRIVES) + QString(" (*.lec *.cdr)\n") + GetWordTranslation(WO_ALLFILEFILTER);
	case WO_SYSTEMFILTER:
		return QObject::tr("Systmes") + QString(" (*.mems *.csf)\n") + GetWordTranslation(WO_ALLFILEFILTER);
	case WO_ALLFILEFILTER:
		return QObject::tr("Tous les fichiers") + QString(" (*)");
	case WO_EXEFILTER:
		return QObject::tr("Excutables") + QString(" (*.exe *.com)");
	case WO_SYSTEM:
		return QObject::tr("Systme");
	case WO_DRIVE:
		return QObject::tr("Lecteur");
	case WO_DRIVES:
		return QObject::tr("Lecteurs");
	case WO_OPTION:
		return QObject::tr("Options");
	case WO_REFRESH:
		return QObject::tr("Actualiser");
	case WO_NAME:
		return QObject::tr("Nom");
	case WO_SIZE:
		return QObject::tr("Taille");
	case WO_SENDTO:
		return QObject::tr("Envoyer vers");
	case WO_CANCEL:
		return QObject::tr("Annuler");
	case WO_APPLY:
		return QObject::tr("Appliquer");
	case WO_IMPORT:
		return QObject::tr("Importer");
	case WO_EXPORT:
		return QObject::tr("Exporter");
	case WO_PATH:
		return QObject::tr("Chemin");
	case WO_FILE:
		return QObject::tr("fichier");
	case WO_FILES:
		return QObject::tr("fichiers");
	case WO_EXTRACT:
		return QObject::tr("Extraire");
	case WO_RENAME:
		return QObject::tr("Renommer");
	case WO_FORMAT:
		return QObject::tr("Formater");
	case WO_WRITING:
		return QObject::tr("Ecriture en cours...");
	case WO_READING:
		return QObject::tr("Lecture en cours...");
	case WO_CLOSE:
		return QObject::tr("Fermer");
	case WO_INPUTIMAGEFILTER:
		return QObject::tr("Images") + " (" + ConvertStrListToFilter(&QImageIO::inputFormats())
			+ ")\n" + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER);
	case WO_OUTPUTIMAGEFILTER:
		return QObject::tr("Images") + " (" + ConvertStrListToFilter(&QImageIO::outputFormats())
			+ ")";
	}
	//Mot inconnu!
	return QString::null;
}

//-----------------------------------------------------------------------------------
QString CLanguage::ConvertStrListToFilter(QStrList* pList)
{
	QString sFilter;
	if(pList) {
		QStrList l = QImageIO::inputFormats();
		char* pFormat = pList->first();
		while(pFormat) {
			sFilter += QString("*.") + pFormat;
			if(pFormat != pList->getLast()) {
				sFilter += " ";
			}
			pFormat = pList->next();
		}
	}
	return sFilter;
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgNoReadFile(const QString& sFile)
{
	QString sMsg = QObject::tr("Impossible de lire sur le fichier%1!");
	if(sFile.isEmpty()) {
		return sMsg.arg(QString::null);
	}
	return sMsg.arg(" '" + sFile + '\'');
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgNoWriteFile(const QString& sFile)
{
	QString sMsg = QObject::tr("Impossible d'crire sur le fichier%1!\nVrifiez que le disque ne soit pas plein.");
	if(sFile.isEmpty()) {
		return sMsg.arg(QString::null);
	}
	return sMsg.arg(" '" + sFile + '\'');
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgNoOpenFile(const QString& sFile, bool bWriteMode)
{
	QString sMsg;
	if(bWriteMode) {
		sMsg = QObject::tr("Impossible d'ouvrir le fichier%1 en criture.\nVrifiez que :\n-Aucun autre processus utilise ce fichier\n-Vous avez les droits d'accs ncessaire");
	}
	else {
		sMsg = QObject::tr("Impossible d'ouvrir le fichier%1 en lecture.\nVrifiez que :\n-Le fichier existe\n-Aucun autre processus utilise ce fichier\n-Vous avez les droits d'accs ncessaire");
	}
	if(sFile.isEmpty()) {
		return sMsg.arg(QString::null);
	}
	return sMsg.arg(" '" + sFile + '\'');
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgNoValidFile(const QString& sFile)
{
	QString sMsg = QObject::tr("Le fichier%1 n'est pas valide.");
	if(sFile.isEmpty()) {
		return sMsg.arg(QString::null);
	}
	return sMsg.arg(" '" + sFile + '\'');
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgInternalError(const QString& sErr)
{
	return QObject::tr("Erreur interne : %1").arg(sErr);
}

//-----------------------------------------------------------------------------------
QString CLanguage::GetMsgOutOfMemory(unsigned int nSize)
{
	return QObject::tr("Manque de mmoire.\nLa demande d'allocation de %1 octets a echou.").arg(nSize);
}
