// language.h: interface for the CLanguage class.
//
// Classe offrant des mots dj traduit et/ou des phrases toutes prtes
//////////////////////////////////////////////////////////////////////////////////

#ifndef LANGUAGE_H
#define LANGUAGE_H

#include <qstring.h>

#include <F100Qt.h>

class QStrList;

/*!
	\class CLanguage language.h
	\brief Classe offrant des mots dj traduit et/ou des phrases toutes prtes
	\author Quaranta David
	\version 1.00

	\ingroup F100Qt

  Classe utilisant le systme de traduction de Qt (QTranslator et QObject::tr(const QString&))
*/
class DLLEXPORT CLanguage
{
public:
	/** Type de Mots/Phrases traduit */
	enum Words{
		WO_AUTHOR = 0,        /*!< Auteur de la traduction */
		WO_INFO,              /*!< Info sur l'auteur (email, etc.) */
		WO_DRIVEFILTER,       /*!< Filtre pour les lecteurs */
		WO_SYSTEMFILTER,      /*!< Filtre pour les systmes */
		WO_ALLFILEFILTER,     /*!< Filtre pour voir tous les fichiers */
		WO_EXEFILTER,         /*!< Filtre pour les fichiers excutables (exe;com) */
		WO_SYSTEM,            /*!< Systme */
		WO_DRIVE,             /*!< Lecteur */
		WO_DRIVES,            /*!< Lecteurs */
		WO_OPTION,            /*!< Options */
		WO_REFRESH,           /*!< Actualiser */
		WO_NAME,              /*!< Nom */
		WO_SIZE,              /*!< Taille */
		WO_SENDTO,            /*!< Envoyer vers */
		WO_CANCEL,            /*!< Annuler */
		WO_APPLY,             /*!< Appliquer */
		WO_IMPORT,            /*!< Importer */
		WO_EXPORT,            /*!< Exporter */
		WO_PATH,              /*!< Chemin */
		WO_FILE,              /*!< fichier */
		WO_FILES,             /*!< fichiers */
		WO_EXTRACT,           /*!< Extraire */
		WO_RENAME,            /*!< Renommer */
		WO_FORMAT,            /*!< Formater */
		WO_WRITING,           /*!< Ecriture en cours... */
		WO_READING,           /*!< Lecture en cours... */
		WO_CLOSE,             /*!< Fermer */
		WO_INPUTIMAGEFILTER,  /*!< Filtre pour charger des images (Filtre dynamique utilise QImageIO::inputFormats()) */
		WO_OUTPUTIMAGEFILTER, /*!< Filtre pour sauver des images (Filtre dynamique utilise QImageIO::outputFormats()) */
	};

public:
	/** Permet d'obtenir la traduction d'un type connu
	 * @param[in] nWord Type de la traduction
	 * @return Retourne la traduction, sinon QString::null si non trouv
	 */
	static QString GetWordTranslation(Words nWord);

	/** Permet d'obtenir un message pr-dfini sur l'chec de lecture d'un fichier
	 * @param[in] sFile Fichier non lu
	 * @return Retourne le message
	 */
	static QString GetMsgNoReadFile(const QString& sFile = QString::null);

	/** Permet d'obtenir un message pr-dfini sur l'chec d'criture sur un fichier
	 * @param[in] sFile Fichier non crit
	 * @return Retourne le message
	 */
	static QString GetMsgNoWriteFile(const QString& sFile = QString::null);

	/** Permet d'obtenir un message pr-dfini sur l'chec d'ouverture d'un fichier
	 * @param[in] sFile Fichier non ouvert
	 * @param[in] bWriteMode A vrai si le fichier est ouvert en criture, sinon  faux
	 * @return Retourne le message
	 */
	static QString GetMsgNoOpenFile(const QString& sFile = QString::null, bool bWriteMode = FALSE);

	/** Permet d'obtenir un message pr-dfini sur la non validit d'un fichier
	 * @param[in] sFile Fichier non valide
	 * @return Retourne le message
	 */
	static QString GetMsgNoValidFile(const QString& sFile = QString::null);

	/** Permet de formater un message d'erreur en message d'erreur interne
	 * @param[in] sErr Message d'erreur
	 * @return Retourne le message d'erreur
	 */
	static QString GetMsgInternalError(const QString& sErr);

	/** Permet d'obtenir un message indiquant un manque de mmoire
	 * @param[in] nSize Taille en octets indiquant la taille  allouer qui a chou
	 * @return Retourne le message d'erreur
	 */
	static QString GetMsgOutOfMemory(unsigned int nSize);

private:
	/** Converti une liste de QString en un QString o chaque lment est spar par un espace
	 * @param[in] pList Liste de QString
	 * @return QString contenant pour les lments
	 */
	static QString ConvertStrListToFilter(QStrList* pList);
};

#endif
