
#include "APIFunction.h"
#include <stdio.h>

#ifdef WIN32
	#include <process.h>
	#include <errno.h>
#else
	#include <unistd.h>
	#include <string.h>
	#include <sys/time.h>
	#include <sys/wait.h>
	#include <dirent.h>
	#include <errno.h>
	#include <stdlib.h>
#endif

//-----------------------------------------------------------------------------------
void Upper(char* s)
{
	if(s)
	{
		for(int i = 0; s[i] != '\0'; i++)
		{
			if(s[i] >= 'a' && s[i] <= 'z')
			{
				s[i] += 'A' - 'a';
			}
		}
	}
}

//-----------------------------------------------------------------------------------
int LaunchNavigator(const char* sPath, bool bHttpProtocol)
{
	char sTmp[300];
	const char* s = sPath;
	if(bHttpProtocol) {
		sprintf(sTmp,"http://%s", sPath);
		s = sTmp;
	}
#ifdef WIN32
	return ExecuteFile(s);
#else
	return ExecuteFile("mozilla", s);
#endif
}

//-----------------------------------------------------------------------------------
void OpenEMail(const char* name)
{
	char sFile[300];
	sprintf(sFile,"Mailto:%s", name);
#ifdef WIN32
	ExecuteFile(sFile);
#else
	ExecuteFile("mozilla", sFile);
	wait(0);
#endif
}

//-----------------------------------------------------------------------------------
int ExecuteFile(const char* name, const char* param, bool bWait)
{
#ifdef WIN32
	if(bWait)
	{
		int result = _spawnl(_P_WAIT, name, name, param, NULL);
		if(result < 0)
		{
			switch(errno)
			{
			case ENOENT:
				return -1;
			case ENOMEM:
				return 0;
			default:
				return -2;
			}
		}
		else
			return 1;
	}
	else
	{
		int result = (int)ShellExecute(NULL, NULL, name, param, NULL, SW_SHOWNORMAL);
		if(result < 32)
		{
			switch(result)
			{
			case ERROR_FILE_NOT_FOUND:
			case ERROR_PATH_NOT_FOUND:
				return -1;
			case 0://pas assez de mmoire
				return 0;
			default:
				return -2;
			}
		}
		else
			return 1;
	}
#else
	int r = fork();
	switch(r)
	{
	case -1 :
		return 0;
	case 0: //fils
	{
		//Dcoupage des paramtres
		const char* tab[50];
		char arg[1024];
		int i;
		for(i = 0; i < 1023 && param[i] != '\0'; i++)//fait la copie des paramtres
			arg[i] = param[i];
		arg[i] = '\0';	//caractre de fin
		tab[0] = name;	//premier nom de l'appli
		tab[1] = arg;	//pointe sur le premier paramtre
		int cpt = 2; 	// dpart  deux
		for(i = 0; arg[i] != '\0'; i++)
		{
			if(arg[i] == ' ')//si espace on coupe
			{
				arg[i] = '\0';
				tab[cpt] = arg + i + 1;
				cpt++;
			}
		}
		tab[cpt] = NULL;//dernier pointeur  null
		execvp(name, (char**) tab);
		exit(-1);
	}
	default:
		if(bWait)
			waitpid(r, NULL, 0);
	}
	return 1;
#endif
}

//-----------------------------------------------------------------------------------
void Wait(int dwMilliseconds)
{
#ifdef WIN32
	Sleep(dwMilliseconds);
#else
	struct timeval tv;
	tv.tv_sec  =  dwMilliseconds / 1000;
	tv.tv_usec = (dwMilliseconds % 1000) * 1000;
	select(0, NULL, NULL, NULL, &tv);
#endif
}

//-----------------------------------------------------------------------------------
HND OpenDesc(const char* name, int fa, int fc, int shared)
{
#ifdef WIN32
	return CreateFile(name, fa, shared, NULL, fc, 0, NULL);
#else
	return open(name, fa | fc, S_IRWXU);
#endif
}

//-----------------------------------------------------------------------------------
bool ReadDesc(HND h, void* buf, unsigned int nBytes)
{
#ifdef WIN32
	unsigned long nRead;
	ReadFile(h, buf, nBytes, &nRead, NULL);
	return nBytes == nRead;
#else
	return(read(h, buf, nBytes) == nBytes);
#endif
}

//-----------------------------------------------------------------------------------
bool WriteDesc(HND h, const void* buf, unsigned int nBytes)
{
	unsigned long nWrite;
#ifdef WIN32
	WriteFile(h, buf, nBytes, &nWrite, NULL);
#else
	nWrite = write(h, buf, nBytes);
#endif
	return nBytes == nWrite;
}

//-----------------------------------------------------------------------------------
void SeekDesc(HND h, unsigned int nPos)
{
#ifdef WIN32
	SetFilePointer(h, nPos, NULL, FILE_BEGIN);
#else
	lseek(h, nPos, SEEK_SET);
#endif
}

//-----------------------------------------------------------------------------------
unsigned int PointerPosition(HND h)
{
#ifdef WIN32
	return SetFilePointer(h, 0, NULL, FILE_CURRENT);
#else
	return lseek(h, 0, SEEK_CUR);
#endif
}

//-----------------------------------------------------------------------------------
void CloseDesc(HND& h)
{
	if(h != INVALID_HANDLE_VALUE)
	{
#	ifdef WIN32
		CloseHandle(h);
#	else
		close(h);
#	endif
		h = INVALID_HANDLE_VALUE;
	}
}

//-----------------------------------------------------------------------------------
unsigned int FileSize(const char* name, bool* pError)
{
	HND h = OpenDescModeRead(name);
	unsigned int n = 0;
	bool bErr = (h == INVALID_HANDLE_VALUE);
	if(!bErr) {
		n = FileSize(h);
		CloseDesc(h);
	}
	if(pError != NULL) {
		*pError = bErr;
	}
	return n;
}

//-----------------------------------------------------------------------------------
unsigned int FileSize( HND h )
{
#ifdef WIN32
	return GetFileSize(h, NULL);
#else
	struct stat buf;
	fstat(h, &buf);
	return buf.st_size;
#endif
}

//-----------------------------------------------------------------------------------
int SetFileSize(HND h, unsigned int nPos)
{
	if(h == INVALID_HANDLE_VALUE)
	{
		return -1;
	}
#ifdef WIN32
	SeekDesc(h, nPos);
	if(SetEndOfFile(h) < 1)
	{
		return -1;
	}
#else
	if(ftruncate(h, nPos) == -1)
	{
		return -1;
	}
#endif
	return 1;
}

//-----------------------------------------------------------------------------------
HND OpenDescModeRead(const char* sFile)
{
	return OpenDesc(sFile, GENERIC_READ, OPEN_EXISTING, FILE_SHARE_READ);
}

//-----------------------------------------------------------------------------------
HND OpenDescModeRdWr(const char* sFile)
{
	return OpenDesc(sFile, READWRITE, CREATE_ALWAYS);
}

//-----------------------------------------------------------------------------------
bool MoveOrRenameFile(const char* oldName, const char* newName)
{
	KillFile(newName);
#ifdef WIN32
	return (MoveFile(oldName, newName) != 0);
#else
	return (rename(oldName, newName) != -1);
#endif
}

//-----------------------------------------------------------------------------------
bool KillFile(const char* name)
{
#ifdef WIN32
	return (DeleteFile(name) != 0);
#else
	return (remove(name) == -1);
#endif
}

//-----------------------------------------------------------------------------------
int MkDir(const char* name)
{
#ifdef WIN32
	if(CreateDirectory(name, NULL))
	{
		return 1;
	}
	else
	{
		if(ERROR_ALREADY_EXISTS == GetLastError())
		{
			return 0;
		}
		return -1;
	}
#else
	if(mkdir(name, S_IRWXU) == -1)
	{
		if(errno == EEXIST)
		{
			return 0;
		}
		return -1;
	}
	return 1;
#endif
}

//-----------------------------------------------------------------------------------
void GetNameOfFile(const char* sFullPath, const char*& sName)
{
	sName = NULL;
	if(sFullPath)
	{
		for(int i = 0; sFullPath[i]; i++)
		{
			if(sFullPath[i] == '/' || sFullPath[i] == '\\')
			{
				sName = sFullPath + i + 1;
			}
		}
	}
	if(!sName)
	{
		sName = sFullPath;
	}
}
