
#include "CasioSystem.h"
#include "DriveAccess.h"
#include "utils.h"

#define SIZEBLOCKINFO 66

const unsigned char keys[][5] =
{
{ 32, 96, 32, 32, 32},	// 1
{240,  8,112,128,248},	// 2
{240,  8,112,  8,240},	// 3
{144,144,248, 16, 16},	// 4
{248,128,240,  8,240},	// 5
{112,128,240,136,112},	// 6
{248,  8, 16, 32, 32},	// 7
{112,136,112,136,112},	// 8
{112,136,120,  8,112},	// 9
{112,136,248,136,136},	// A
{240,136,240,136,240},	// B
{120,128,128,128,120},	// C
{240,136,136,136,240},	// D
{248,128,240,128,248},	// E
{248,128,240,128,128},	// F
{120,128,184,136,120},	// G
{136,136,248,136,136},	// H
{248, 32, 32, 32,248},	// I
{ 16, 16, 16,144, 96},	// J
{144,160,192,160,144},	// K
{128,128,128,128,248},	// L
{136,216,168,136,136},	// M
{136,200,168,152,136},	// N
{112,136,136,136,112},	// O
{248,136,248,128,128},	// P
{112,136,168,144,104},	// Q
{240,136,240,136,136},	// R
{120,128,112,  8,240},	// S
{248, 32, 32, 32, 32},	// T
{136,136,136,136,112},	// U
{136,136,136, 80, 32},	// V
{168,168,168,168, 80},	// W
{136, 80, 32, 80,136},	// X
{136, 80, 32, 32, 32},	// Y
{248, 16, 32, 64,248},	// Z
};

//-----------------------------------------------------------------------------------
CCasioSystem::CCasioSystem()
: CDriveFile(true, false)
{
	m_bIsValid  = false;
	m_nG100Type = CM_INVALID;
}

//-----------------------------------------------------------------------------------
CCasioSystem::~CCasioSystem()
{
	CloseDrive();
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetSizeInBytes() const
{
	return DRIVE_SIZE;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode)
{
	int nRet = CDriveFile::OpenDriveFile(sPath, nShift, bReadWriteMode);
	if(nRet > 0) {
		IsValid();
	}
	return nRet;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::OpenDriveFile(void* pData, unsigned int nMaxSize)
{
	int nRet = CDriveFile::OpenDriveFile(pData, nMaxSize);
	if(nRet > 0) {
		IsValid();
	}
	return nRet;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::CloseDrive()
{
	int nRet = CDriveFile::CloseDrive();
	m_bIsValid  = false;
	m_nG100Type = CM_INVALID;
	return nRet;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::IsValid()
{
	if(!m_bIsValid)
	{		
		if(m_hFile != INVALID_HANDLE_VALUE && (FileSize(m_hFile) - m_nShift) < 65536)
		{
			return false;
		}
		m_bIsValid = true;
		
		char buf[16];
		TInfo info;
		if(SetPosition(16384) < 1
			|| Read(&info, SIZEBLOCKINFO) < 1
			|| SetPosition(0x5FC0) < 1
			|| Read(buf, sizeof(buf)) < 1)
		{
			return false;
		}
		if(info.number == 0) {
			if(strcmp(buf, "B:\\SALTIRE1.EXE") == 0)
			{
				m_nG100Type = CM_G100;
				return true;
			}
			if(strcmp(buf, "G:\\SALTIRE.EXE") == 0)
			{
				m_nG100Type = CM_G100_PLUS;
				return true;
			}
		}
		m_nG100Type = CM_INVALID;
		m_bIsValid = false;
	}
	return m_bIsValid;
}

//-----------------------------------------------------------------------------------
CasioModel CCasioSystem::GetSystemType() const
{
	return m_nG100Type;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::GetMenus(void* pMenus)
{
	for(int i = 0; i < MAXMENU; i++){
		if(!GetMenu(((char*)pMenus) + SIZEBMP * i, i)){
			return false;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::GetMenu(void* pMenu, int nNumMenu)
{
	if(nNumMenu < 0 || nNumMenu >= MAXMENU){
		return false;
	}
	char pCasioBuf[SIZEBMP];
	if(SetPosition(24576 + (nNumMenu * SIZEBMP)) > 0)
	{
		if(Read(pCasioBuf, SIZEBMP) > 0){
			InvertBuffer(pMenu, pCasioBuf, true);
		}
		else{
			return false;
		}
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::GetTitle(void* pTitle, int nNumMenu)
{
	char menu[SIZEBMP];
	if(GetMenu(menu, nNumMenu)) {
		const void* title = GetTitle(menu);
		memcpy(pTitle, title, CCasioSystem::MENU_BYTE_PER_ROW * CCasioSystem::TITLE_HEIGHT);
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::GetInfo(TInfo* pInfo)
{
	return SetPosition(16384) > 0
		&& Read(pInfo, SIZEBLOCKINFO * MAXICON) > 0;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetMenus(const void* pMenus)
{
	for(int i = 0; i < MAXMENU; i++){
		if(!SetMenu(((char*)pMenus) + SIZEBMP * i, i)){
			return false;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetMenu(const void* pMenu, int nNumMenu)
{
	char pCasioBuf[SIZEBMP];
	if(nNumMenu < 0 || nNumMenu >= MAXMENU){
		return false;
	}
	if(SetPosition(24576 + (nNumMenu * SIZEBMP)) > 0)
	{
		InvertBuffer((void*)pMenu, pCasioBuf, false);
		if(Write(pCasioBuf, SIZEBMP) < 1){
			return false;
		}
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetTitle(const void* pTitle, int nNumMenu)
{
	char menu[SIZEBMP];
	if(GetMenu(menu, nNumMenu)) {
		SetTitle(menu, pTitle);
		return SetMenu(menu, nNumMenu);
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetInfo(const TInfo* pInfo)
{
	return SetPosition(16384) > 0
		&& Write(pInfo, SIZEBLOCKINFO * MAXICON) > 0;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetNbIcon()
{
	TInfo infos[MAXICON];
	if(GetInfo(infos))
	{
		return GetNbIcon(infos);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetNbMenu()
{
	TInfo infos[MAXICON];
	if(GetInfo(infos))
	{
		return GetNbMenu(GetNbIcon(infos));
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::AddIcon(const TInfo* pIconInfo, const void* pIconImage, bool bReportTitle, bool bCpyLastIcon, bool bShowKey)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	return GetInfo(infos) && GetMenus(menus)
		&& AddIcon(pIconInfo, pIconImage, infos, menus, bReportTitle, bCpyLastIcon, bShowKey) != NULL
		&& SetInfo(infos) && SetMenus(menus);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::RemoveIcon(int nNumIcon)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	return GetInfo(infos) && GetMenus(menus)
		&& RemoveIcon(infos, menus, nNumIcon)
		&& SetInfo(infos) && SetMenus(menus);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::ShiftIconLeft(int nNumIcon)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	return GetInfo(infos) && GetMenus(menus)
		&& ShiftIconLeft(infos, menus, nNumIcon)
		&& SetInfo(infos) && SetMenus(menus);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::ShiftIconRight(int nNumIcon)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	return GetInfo(infos) && GetMenus(menus)
		&& ShiftIconRight(infos, menus, nNumIcon)
		&& SetInfo(infos) && SetMenus(menus);
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetIconImage(void* pIconImage, int nNumIcon, int nInMenu)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	if(GetInfo(infos) && GetMenus(menus))
	{
		return GetIconImage(infos, menus, pIconImage, nNumIcon, nInMenu);
	}
	return -3;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::SetIconImage(const void* pIconImage, int nNumIcon, int nInMenu)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	if(GetInfo(infos) && GetMenus(menus))
	{
		int nRet = SetIconImage(infos, menus, pIconImage, nNumIcon, nInMenu);
		if(nRet > 0)
		{
			if(!SetInfo(infos) || !SetMenus(menus))
			{
				return -3;
			}
		}
		return nRet;
	}
	return -3;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetIconVisibility(int nNumIcon)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos)){
		return -1;
	}
	return GetIconVisibility(infos, nNumIcon);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconVisibility(int nNumIcon, int nState)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos))
	{
		return false;
	}
	return SetIconVisibility(infos, nNumIcon, nState);
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconPath(int nNumIcon)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos)) {
		return "";
	}
	return GetIconPath(infos, nNumIcon);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconPath(const string& sPath, int nNumIcon)
{
	TInfo infos[MAXICON];
	return GetInfo(infos)
		&& SetIconPath(infos, sPath, nNumIcon)
		&& SetInfo(infos);
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconArguments(int nNumIcon)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos)) {
		return "";
	}
	return GetIconArguments(infos, nNumIcon);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconArguments(const string& sArg, int nNumIcon)
{
	TInfo infos[MAXICON];
	return GetInfo(infos)
		&& SetIconArguments(infos, sArg, nNumIcon)
		&& SetInfo(infos);
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconImageName(int nNumIcon)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos)) {
		return "";
	}
	return GetIconImageName(infos, nNumIcon);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconImageName(const string& sImageName, int nNumIcon)
{
	TInfo infos[MAXICON];
	return GetInfo(infos)
		&& SetIconImageName(infos, sImageName, nNumIcon)
		&& SetInfo(infos);
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconVersion(int nNumIcon)
{
	TInfo infos[MAXICON];
	if(!GetInfo(infos)) {
		return "";
	}
	return GetIconVersion(infos, nNumIcon);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconVersion(const string& sVersion, int nNumIcon)
{
	TInfo infos[MAXICON];
	return GetInfo(infos)
		&& SetIconVersion(infos, sVersion, nNumIcon)
		&& SetInfo(infos);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::PutKey(int nNumIcon)
{
	TInfo infos[MAXICON];
	char menus[MAXMENU][SIZEBMP];
	if(GetInfo(infos) && GetMenus(menus) && nNumIcon < GetNbIcon(infos)) {
		int nNbMenu = GetNbMenu(GetNbIcon(infos));
		int nStart  = GetFirstMenuIndex(nNumIcon);
		int nEnd    = GetLastMenuIndex(nNumIcon, nNbMenu);
		for(; nStart <= nEnd; ++nStart){
			int n = (nNumIcon - MENU_ROW * nStart);
			PutKey(menus[nStart], nNumIcon, ICON_WIDTH * (n%4) + 23, ICON_HEIGHT * (n/4) + 12 + TITLE_HEIGHT, MENU_BYTE_PER_ROW);
		}
		return SetMenus(menus);
	}
	return false;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetFirstMenuIndex(int nNumIcon)
{
	return (((nNumIcon)/MENU_ROW) > 2) ? (((nNumIcon)/MENU_ROW) - 2) : (0);
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetLastMenuIndex(int nNumIcon, int nNbMenu)
{
	int nStart = GetFirstMenuIndex(nNumIcon);
	return F100MIN(nStart + F100MIN(2, nNumIcon / MENU_ROW), nNbMenu - 1);
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetDrivePosition()
{
	return 65536;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::IsValid(void* buffer)
{
	CCasioSystem system;
	if(system.OpenDriveFile(buffer, DRIVE_SIZE) < 0){
		return CM_INVALID;
	}
	if(system.IsValid()){
		return system.GetSystemType();
	}
	return CM_INVALID;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::IsValid(const string& sPath, int nShift)
{
	CCasioSystem system;
	if(system.OpenDriveFile(sPath, nShift, false) < 0){
		return CM_INVALID;
	}
	if(system.IsValid()){
		return system.GetSystemType();
	}
	return CM_INVALID;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetNbIcon(const TInfo* pInfo)
{
	if(pInfo != NULL)
	{
		int i = 0;
		for(; i < MAXICON && pInfo[i].number != -1; i++);
		return i;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetNbMenu(int nNbIcon)
{
	if(nNbIcon < 0 || nNbIcon > MAXICON)
	{
		return -1;
	}
    float f = (nNbIcon / 4.0f);
	int n = (int)f;
	if(f > n){
		n++;
	}
    if(n > 2){
		n -= 2;
	}
	else{
		n = 1;
	}
    return n;
}

//-----------------------------------------------------------------------------------
void CCasioSystem::ClearZoneInfo(TInfo& info)
{
	memset(&info, 0, SIZEBLOCKINFO);
	info.number = -1;
	info.bmp    = -1;
}

//-----------------------------------------------------------------------------------
void CCasioSystem::NewZoneInfo(TInfo& info, int nPos)
{
	ClearZoneInfo(info);
	info.number = nPos;
	info.bmp    = 1;
	memcpy(info.version, "1.00", 4);
	memcpy(info.lecteur, "L:\\", 3);
}

//-----------------------------------------------------------------------------------
void CCasioSystem::SetTitle(void* pMenu, const void* pTitle)
{
	memcpy(pMenu, pTitle, CCasioSystem::MENU_BYTE_PER_ROW * CCasioSystem::TITLE_HEIGHT);
}

//-----------------------------------------------------------------------------------
const void* CCasioSystem::GetTitle(const void* pMenu)
{
	return pMenu;
}

//-----------------------------------------------------------------------------------
void CCasioSystem::InvertBuffer(void* pPCBuf, void* pCasioBuf, bool bCasioToPC)
{
	int c = 0;
	int source = SIZEBMP - 1;
	for(int x = 0; x < 16; x++)
	{
		source = SIZEBMP - x -1;
		for(int y = 0; y < 64; y++)
		{
			if(bCasioToPC){
				((char*)pPCBuf)[source] = ((char*)pCasioBuf)[c];
			}
			else{
				((char*)pCasioBuf)[c] = ((char*)pPCBuf)[source];
			}
			source -= 16;
			c++;
		}
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::Cpy8LastIcon(void* pDest, const void* pSource)
{
	int x, y;
    for(x = 0; x < 16; x++){
        for(y = 26; y < 64; y++){
            ((char*)pDest)[x + 16*(y - 19)] = ((char*)pSource)[x + 16*y];
		}
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::CpyIcon(void* pDest, int nDestPos, const void* pSrc, int nSrcPos, int bInv, int nDestBytePerRow, int nSrcBytePerRow)
{
	int	xDest = 0, yDest = 0;
	if(nDestPos != -1)
	{
		xDest = 30*(nDestPos % 4);
		yDest = 19*(nDestPos / 4) + 7;
	}
	int xSource = 0, ySource = 0;
	if(nSrcPos != -1)
	{
		xSource = 30*(nSrcPos % 4);
		ySource = 19*(nSrcPos / 4) + 7;
	}
	unsigned char val;
	for(int x = 0; x < 30; x++)
	{
		for(int y = 0; y < 19; y++)
		{
			val = GetPixel(pSrc, x + xSource, y + ySource, nSrcBytePerRow) ^ bInv;
			PutPixel(pDest, x + xDest, y + yDest, nDestBytePerRow, val);
		}
	}
}

//-----------------------------------------------------------------------------------
CCasioSystem::TInfo* CCasioSystem::AddIcon(const TInfo* pIconInfo, const void* pIconImage, TInfo* pInfo, void* pMenus, bool bReportTitle, bool bCpyLastIcon, bool bShowKey)
{
	int nNbIcon = GetNbIcon(pInfo);
	int nNbMenu = GetNbMenu(nNbIcon);
	if(nNbIcon < 1 || nNbMenu < 1 || nNbIcon >= MAXICON)
	{
		return NULL;
	}
	if(pIconInfo){
		pInfo[nNbIcon] = *pIconInfo;
		pInfo[nNbIcon].number = nNbIcon;
	}
	else{
		NewZoneInfo(pInfo[nNbIcon], nNbIcon);
	}
	if(GetNbMenu(nNbIcon + 1) != nNbMenu)
	{
		char* pLastMenu = ((char*)pMenus) + (nNbMenu-1) * SIZEBMP;
		char* pNewMenu  = pLastMenu + SIZEBMP;

		memset(pNewMenu, 0, 1024); // efface l'cran
		
		if(bReportTitle){
			SetTitle(pNewMenu, GetTitle(pLastMenu));
		}
		if(bCpyLastIcon){
			Cpy8LastIcon(pNewMenu, pLastMenu);
		}
	}
	if(pIconImage != NULL || bShowKey) {
		char tmp[76];
		if(pIconImage != NULL) {
			memcpy(tmp, pIconImage, sizeof(tmp));
		}
		else{
			memset(tmp, 0, sizeof(tmp));
		}
		PutKey(tmp, nNbIcon);
		SetIconImage(pInfo, pMenus, tmp, nNbIcon, -1);
	}
	return pInfo + nNbIcon;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::RemoveIcon(TInfo* pInfo, void* pMenus, int nNumIcon)
{
	int nNbIcon = GetNbIcon(pInfo);
	int nNbMenu = GetNbMenu(nNbIcon);
	if(nNbIcon < 1 || nNbMenu < 1)
	{
		return false;
	}
	int n;
	for(n = nNumIcon + 1; n < nNbIcon; n++)
	{
		pInfo[n - 1] = pInfo[n];
		pInfo[n - 1].number = n - 1;
	}
	ClearZoneInfo(pInfo[nNbIcon - 1]);
	nNbIcon--;

	n = GetFirstMenuIndex(nNumIcon);

	for(; n < nNbMenu; n++)
	{
		int nStart = nNumIcon - 4 * n;
		if(nStart < 0){
			nStart = 0;
		}
		int i = nStart;
		char* pMenu = ((char*)pMenus) + n * SIZEBMP;
		for(; i < 11 && i < nNbIcon - 4 * n; i++){
			CpyIcon(pMenu, i, pMenu, i + 1);
		}
		if(n != nNbMenu - 1){
			CpyIcon(pMenu, 11, pMenu + SIZEBMP, 8);
		}
		else{
			ClearIcon(pMenu, i);
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::MoveIconLeft(TInfo* pInfo, void* pMenus, int nNumIcon)
{
	int nNbIcon = GetNbIcon(pInfo);
	int nNbMenu = GetNbMenu(nNbIcon);
	if(nNumIcon < 1 || nNbIcon <= nNumIcon || nNbMenu < 1)
	{
		return false;
	}
	//Permute les infos
	TInfo i;
	i = pInfo[nNumIcon];
	i.number--;
	pInfo[nNumIcon] = pInfo[nNumIcon - 1];
	pInfo[nNumIcon].number++;
	pInfo[nNumIcon - 1] = i;
	//Permute les icones
	int n1 = GetFirstMenuIndex(nNumIcon );
	int n2 = GetFirstMenuIndex(nNumIcon - 1);
	int cpt = n1;
	int a = 4 * cpt;
	char* menus = (char*)pMenus;
	if(n2 < n1){
		CpyIcon(menus + n2 * SIZEBMP, 11, menus + n1 * SIZEBMP, 8);
	}
	while(cpt < nNbMenu && a < nNumIcon)
	{
		PermuteIcon(menus + cpt * SIZEBMP, nNumIcon - a, nNumIcon - 1 - a);
		cpt++;
		a+= 4;
	}
	if(cpt < nNbMenu && a >= nNumIcon){
		CpyIcon(menus + cpt * SIZEBMP, 0, menus + (cpt-1) * SIZEBMP, 4);
	}
	return true;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::ShiftIconLeft(TInfo* pInfo, void* pMenus, int nNumIcon)
{
	if(MoveIconLeft(pInfo, pMenus, nNumIcon)){
		SetIconVisibility(pInfo, nNumIcon - 1, GetIconVisibility(pInfo, nNumIcon - 1));

		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::ShiftIconRight(TInfo* pInfo, void* pMenus, int nNumIcon)
{
	if(MoveIconLeft(pInfo, pMenus, nNumIcon + 1)){
		SetIconVisibility(pInfo, nNumIcon + 1, GetIconVisibility(pInfo, nNumIcon + 1));

		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetIconVisibility(const TInfo* pInfo, int nNumIcon)
{
	int nNbIcon = GetNbIcon(pInfo);
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return -1;
	}
	return pInfo[nNumIcon].bmp;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconVisibility(TInfo* pInfo, int nNumIcon, int nState)
{
	int nNbIcon = GetNbIcon(pInfo);
	if(nNumIcon < 0 || nNbIcon <= nNumIcon)
	{
		return false;
	}
	int found = 0;
	int i;
	if(nState == IV_SHOW)
	{
		pInfo[nNumIcon].bmp = IV_SHOW;
		for(i = 0; i < nNumIcon; i++)
		{
			if(found)
			{
				pInfo[i].bmp = IV_SHOW;
			}
			found = pInfo[i].bmp == IV_SHOW;
		}
	}
	else
	{
		i = nNumIcon;
		do
		{
			pInfo[i].bmp = IV_HIDE;
			i++;
		}while(nNumIcon && pInfo[nNumIcon - 1].bmp == IV_SHOW && i < 35 && pInfo[i].number != -1);
	}
	return true;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetIconImage(const TInfo* pInfo, const void* pMenus, void* pIconImage, int nNumIcon, int nInMenu)
{
	int nNbIcon = GetNbIcon(pInfo);
	int nNbMenu = GetNbMenu(nNbIcon);
	if(nNbIcon > 0 && nNbMenu > 0 && pIconImage != NULL && nNumIcon < nNbIcon && nInMenu >= 0)
	{
		int nStart = GetFirstMenuIndex(nNumIcon);
		int nEnd   = GetLastMenuIndex(nNumIcon, nNbMenu);
		nStart += nInMenu;
		if(nInMenu > 0 && nStart > nEnd){
			return -2;
		}
		CpyIcon(pIconImage, -1, ((char*)pMenus)+nStart*SIZEBMP, nNumIcon - 4 * nStart, 0, ICON_BYTE_PER_ROW, MENU_BYTE_PER_ROW);
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::SetIconImage(const TInfo* pInfo, void* pMenus, const void* pIconImage, int nNumIcon, int nInMenu)
{
	int nNbIcon = GetNbIcon(pInfo);
	int nNbMenu = GetNbMenu(nNbIcon);
	if(nNbIcon > 0 && nNbMenu > 0 && pIconImage != NULL && nNumIcon < nNbIcon)
	{
		int nStart = GetFirstMenuIndex(nNumIcon);
		int nEnd   = GetLastMenuIndex(nNumIcon, nNbMenu);
		if(nInMenu >= 0)
		{
			nStart += nInMenu;
			if(nStart > nEnd)
			{
				return -2;
			}
			nEnd = nStart;
		}
		for(; nStart <= nEnd; nStart++){
			CpyIcon(((char*)pMenus)+nStart*SIZEBMP, nNumIcon - 4 * nStart, pIconImage, -1, 0, MENU_BYTE_PER_ROW, ICON_BYTE_PER_ROW);
		}
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconPath(const TInfo* pInfo, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo)) {
		return "";
	}
	return string(pInfo[nNumIcon].lecteur) + string(pInfo[nNumIcon].name);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconPath(TInfo* pInfo, const string& sPath, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return false;
	}
	string sDrive = section(sPath, '\\', 0,  0) + string("\\");
	string sName  = section(sPath, '\\', 1, -1);

	CpyChar(pInfo[nNumIcon].lecteur, sDrive.c_str(), sizeof(pInfo[nNumIcon].lecteur));
	CpyChar(pInfo[nNumIcon].name,    sName.c_str(),  sizeof(pInfo[nNumIcon].name));
	return true;
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconArguments(const TInfo* pInfo, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return "";
	}
	return string(pInfo[nNumIcon].arg);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconArguments(TInfo* pInfo, const string& sArg, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return false;
	}
	CpyChar(pInfo[nNumIcon].arg, sArg.c_str(), sizeof(pInfo[nNumIcon].arg));
	return true;
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconImageName(const TInfo* pInfo, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return "";
	}
	return string(pInfo[nNumIcon].image);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconImageName(TInfo* pInfo, const string& sImageName, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo))
	{
		return false;
	}
	CpyChar(pInfo[nNumIcon].image, sImageName.c_str(), sizeof(pInfo[nNumIcon].image));
	return true;
}

//-----------------------------------------------------------------------------------
string CCasioSystem::GetIconVersion(const TInfo* pInfo, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo)) {
		return "";
	}
	return string(pInfo[nNumIcon].version);
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::SetIconVersion(TInfo* pInfo, const string& sVersion, int nNumIcon)
{
	if(nNumIcon < 0 || nNumIcon >= GetNbIcon(pInfo)) {
		return false;
	}
	CpyChar(pInfo[nNumIcon].version, sVersion.c_str(), sizeof(pInfo[nNumIcon].version));
	return true;
}

//-----------------------------------------------------------------------------------
int CCasioSystem::GetPixel(const void* pBuf, int x, int y, int nBytePerRow)
{
	int pX = x / 8;
	int mask = 1 << (7 - (x - 8 * pX));
	if(((const unsigned char*)pBuf)[nBytePerRow * y + pX] & mask){
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
void CCasioSystem::PutPixel(void* pBuf, int x, int y, int nBytePerRow, int nVal)
{
	int pX = x / 8;
	int mask = 1 << (7 - (x - 8 * pX));
	((unsigned char*)pBuf)[nBytePerRow * y + pX] &= (mask ^ 255);
	if(nVal){
		((unsigned char*)pBuf)[nBytePerRow * y + pX] |= mask;
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::vLine(void* pBuf, int y1, int y2, int x, int nBytePerRow, int nVal)
{
	for(int y = y1; y <= y2; y++){
		PutPixel(pBuf, x, y, nBytePerRow, nVal);
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::hLine(void* pBuf, int x1, int x2, int y, int nBytePerRow, int nVal)
{
	for(int x = x1; x <= x2; x++){
		PutPixel(pBuf, x, y, nBytePerRow, nVal);
	}
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::PutKey(void* pBuf, int nPosition, int nDx, int nDy, int nBytePerRow)
{
	for(int x = nDx; x < nDx + 7; x++){
		vLine(pBuf, nDy, nDy + 6, x, nBytePerRow, 1);
	}
	if(nPosition < 0 || nPosition >= MAXICON){
		return false;
	}

	for(int i = 0; i < 5; i++)
	{
		int n = 128;
		for(int j = 0; j < 5; j++)
		{
			PutPixel(pBuf, nDx + j + 1, nDy + i + 1, nBytePerRow, !(keys[nPosition][i] & n));
			n >>= 1;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
bool CCasioSystem::PutKey(void* pIconImage, int nPosition)
{
	return PutKey(pIconImage, nPosition, 23, 12, ICON_BYTE_PER_ROW);
}

//-----------------------------------------------------------------------------------
void CCasioSystem::ClearIcon(void* pDest, int nDestPos)
{
	int xDest = 30*(nDestPos % 4);
	int yDest = 19*(nDestPos / 4) + 7;
	for(int x = 0; x < 30; x++)
	{
		for(int y = 0; y < 19; y++)
		{
			PutPixel(pDest, x + xDest, y + yDest, 16, 0);
		}
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::PermuteIcon(void* pBuf, int n1, int n2)
{
	int xDest = 30*(n1 % 4);
	int yDest = 19*(n1 / 4) + 7;
	int xSource = 30*(n2 % 4);
	int ySource = 19*(n2 / 4) + 7;
	unsigned char val1, val2;
	for(int x = 0; x < 30; x++)
	{
		for(int y = 0; y < 19; y++)
		{
			val1 = GetPixel(pBuf, x + xDest, y + yDest, 16);
			val2 = GetPixel(pBuf, x + xSource, y + ySource, 16);
			PutPixel(pBuf, x + xDest, y + yDest, 16, val2);
			PutPixel(pBuf, x + xSource, y + ySource, 16, val1);
		}
	}
}

//-----------------------------------------------------------------------------------
void CCasioSystem::CpyChar(char* pBuf, const char* pSource, int nMaxSize)
{
	int i;
	if(pSource){
		for(i = 0; pSource[i] != '\0' && i < nMaxSize - 1; i++){
			pBuf[i] = pSource[i];
		}
	}
	for(; i < nMaxSize; i++){
		pBuf[i] = 0;
	}
}
