// DriveAccess.cpp: implementation of the CDriveAccess class.
//
//////////////////////////////////////////////////////////////////////

#include "DriveAccess.h"

#include <math.h>

//-----------------------------------------------------------------------------------
char* CDriveAccess::GetDriveName(int nDrive)
{
	static char* sName[] = {"A:\\", "L:\\", "M:\\", "N:\\", "O:\\", "P:\\", "Q:\\"};
	if(nDrive < 0 || nDrive > 6)
	{
		return 0;
	}
	return sName[nDrive];
}

//-----------------------------------------------------------------------------------
bool CDriveAccess::IsEmptyH(HND h, int nShift, int nSize)
{
	SeekDesc(h, nShift);
	bool bEmpty = true;
	unsigned int size;
	if(nSize < 0)
	{
		size = FileSize(h) - nShift;
	}
	else
	{
		size = nSize;
	}
	unsigned int i = 0;
	char* buf = new char[DRIVE_SIZE];
	unsigned nRead = DRIVE_SIZE;
	while(i < size && bEmpty)
	{
		if(size - i < nRead)
		{
			nRead = (size - i);
		}
		if(!ReadDesc(h, buf, nRead))
		{
			delete[] buf;
			return true; // Si erreur retourne vide
		}
		bEmpty = IsEmpty(buf, nRead);
	}
	delete[] buf;
	return bEmpty;
}

//-----------------------------------------------------------------------------------
bool CDriveAccess::IsEmpty(const void* buffer, int nSize)
{
	const unsigned char* buf = (const unsigned char*)buffer;
	int i;
	for(i = 0; i < nSize && buf[i] == 0xFF; i++);
	return (i == nSize);
}

//-----------------------------------------------------------------------------------
int CDriveAccess::GetNbFile(int nSize, bool bOneFile)
{
	if(bOneFile) {
		return 1;
	}
	else {
		int n = (int)ceil(nSize / (float)DRIVE_SIZE);
		if(n < 1) {
			return 1;
		}
		return n;
	}
}

//-----------------------------------------------------------------------------------
string& CDriveAccess::ChangeFileName(string& sFileName, int nNumDrive)
{
	int pos_L = GetPartionPos(sFileName);
	int i = sFileName.rfind('.');
	if(pos_L != -1 && i != -1)
	{
		//Ce nom contient dj un nom de lecteur dcoup
		//On vrifie si _L est bien plac
		if(pos_L < i) // _L est bien plac
		{
			//On supprime de "_L"  '.'
			sFileName.erase(pos_L, i - pos_L);
			i = pos_L;
		}
	}
	if(nNumDrive == 0)
	{
		return sFileName;
	}
	if(i == -1)
	{
		i = sFileName.length();
	}
	char buffer[20];
	sprintf(buffer, "_L%i", nNumDrive);
	sFileName.insert(i, buffer);
	return sFileName;
}

//-----------------------------------------------------------------------------------
bool CDriveAccess::AccessInDrive(HND& h, string& name, void* buffer, int nBytes, int& numDrive, bool WriteMode, bool bOneFile, unsigned int nHeaderSize)
{
	bool ok = true;
	char* buf = (char*)buffer;
	unsigned int wrote = PointerPosition(h);
	if(bOneFile)
	{
		wrote = wrote % DRIVE_SIZE;
	}
	while(nBytes && ok)
	{
		if((bOneFile && nHeaderSize == 0) || wrote + nBytes < DRIVE_SIZE)
		{
			if(WriteMode) {
				ok = WriteDesc(h, buf, nBytes);
			}
			else {
				ok = ReadDesc(h, buf, nBytes);
			}
			wrote = nBytes = 0;
		}
		else
		{
			int write = DRIVE_SIZE - wrote;
			if(write)
			{
				if(WriteMode)
				{
					ok = WriteDesc(h, buf, write);
				}
				else
				{
					ok = ReadDesc(h, buf, write);
				}
			}
			buf		+= write;
			nBytes	-= write;
			wrote	 = nHeaderSize;
			if(bOneFile)
			{
				SeekDesc(h, PointerPosition(h) + nHeaderSize);
			}
			else
			{
				if(nBytes)
				{
					CloseDesc(h);
					if(WriteMode && numDrive == 1)
					{
						string sNewName = name;
						ChangeFileName(sNewName, 1);
						if(sNewName != name)
						{
							ok = ok && MoveOrRenameFile(name.c_str(), sNewName.c_str());
						}
					}
					numDrive++;
					ChangeFileName(name, numDrive);
					if(WriteMode)
					{
						h = OpenDesc(name.c_str(), READWRITE, OPEN_ALWAYS);
					}
					else
					{
						h = OpenDescModeRead(name.c_str());
					}
					ok = ok && (h != INVALID_HANDLE_VALUE);
					if(ok)
					{
						SeekDesc(h, nHeaderSize);
					}
				}
			}
		}
	}
	return ok;
}

//-----------------------------------------------------------------------------------
int CDriveAccess::GetSizeUseful(int nDrive, const void* buffer)
{
	if(nDrive == 0 || buffer == NULL) {
		return DRIVE_SIZE;
	}
	unsigned int nCpt = DRIVE_SIZE - 1;
	for(; (nCpt > 0) && (((const unsigned char*)buffer)[nCpt] == 0xFF); nCpt--);
	if((((const unsigned char*)buffer)[nCpt] != 0xFF)) {
		return nCpt + 1;
	}
	return nCpt;
}

//-----------------------------------------------------------------------------------
bool CDriveAccess::IsOnlyOneFile(const string& sFileName)
{
	return GetPartionPos(sFileName) == -1;
}

//-----------------------------------------------------------------------------------
int CDriveAccess::GetNumDrive(const string& sFileName)
{
	int nPos = GetPartionPos(sFileName);
	int i    = sFileName.rfind('.');
	if(nPos == -1 || i < nPos) {
		return 0;
	}
	return atoi(sFileName.substr(nPos + 2, i - nPos - 2).c_str());
}

//-----------------------------------------------------------------------------------
int CDriveAccess::GetPartionPos(const string& sFileName)
{
	int nPos = sFileName.rfind("_L");
	if(nPos == -1) {
		nPos = sFileName.rfind("_l");
	}
	return nPos;
}
