
#ifndef _FILEINFO_H_
#define _FILEINFO_H_

#include <string>
using namespace std;

#include <F100_Op.h>

/*!
	\class CFileInfo FileInfo.h
	\brief Permet d'avoir des informations sur un fichier quelconque
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
	Contient les donnes d'un fichier en associant un compteur de rfrences pour connaitre
	le nombre de rfrences sur cet objet et savoir quand il peut tre dtruit
*/
class DLLEXPORT CFileInfo  
{
public:
	/** Constructeur */
	CFileInfo();

	/** @return Retourne le nom du fichier */
	virtual string       GetName() const;

	/** @return La position du fichier en octets dans le lecteur */
	virtual unsigned int GetAddress() const;

protected:
	/** Incrmente le compteur de rfrence
	 * @return La valeur du compteur de rfrence actuelle
	 */
	virtual int          IncCountRef();

	/** Dcrmente le compteur de rfrence
	 * @return La valeur du compteur de rfrence actuelle
	 */
	virtual int          DecCountRef();

protected:
	string         m_sName;     ///< Nom du fichier
	int            m_nCountRef; ///< Compteur de rfrence
	unsigned int   m_nAddress;  ///< Adresse du fichier
};

#endif
