
#include "FlashBackup.h"

#include "DriveAccess.h"

//-----------------------------------------------------------------------------------
CFlashBackup::CFlashBackup()
{
	m_hFile     = INVALID_HANDLE_VALUE;
	m_bValidFAT = false;
}

//-----------------------------------------------------------------------------------
CFlashBackup::~CFlashBackup()
{
	Close();
}

//-----------------------------------------------------------------------------------
bool CFlashBackup::Open(const char* pName, bool bWriteMode)
{
	Close();
	if(bWriteMode)
	{
		m_hFile = OpenDesc(pName, READWRITE, OPEN_ALWAYS);
	}
	else
	{
		m_hFile = OpenDescModeRead(pName);
	}
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		SeekDesc(m_hFile, 0);
		m_bValidFAT = ReadDesc(m_hFile, m_FAT, sizeof(m_FAT));
	}
	return m_hFile != INVALID_HANDLE_VALUE;
}

//-----------------------------------------------------------------------------------
void CFlashBackup::Close()
{
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		SeekDesc(m_hFile, 0);
		WriteDesc(m_hFile, m_FAT, sizeof(m_FAT));
	}
	CloseDesc(m_hFile);
	m_bValidFAT = false;
}

//-----------------------------------------------------------------------------------
int CFlashBackup::IsValid()
{
	if(m_hFile == INVALID_HANDLE_VALUE)
	{
		return -1;	
	}
	if(FileSize(m_hFile) <  DRIVE_SIZE + BACKUP_HEADER_SIZE)
	{
		//Taille invalide!
		return -2;
	}
	unsigned int nSize = 0;
	if(!m_bValidFAT)
	{
		return -1;	
	}
	if(m_FAT[0] == 0)
	{
		//Il doit tjrs avoir le systme
		return -1;
	}
	for(int i = 0; i < 7; i++)
	{
		if(m_FAT[i])
		{
			nSize += DRIVE_SIZE;
		}
	}
	if(FileSize(m_hFile) - BACKUP_HEADER_SIZE != nSize)
	{
		//Taille invalide!
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CFlashBackup::GetStartDrive(int nDrive)
{
	if(nDrive < 0 || nDrive > 6)
	{
		return -2;
	}
	else
	{
		if(m_FAT[nDrive])
		{
			int nStart = BACKUP_HEADER_SIZE;
			for(int i = 0; i < nDrive; i++)
			{
				if(m_FAT[i])
				{
					nStart += DRIVE_SIZE;
				}
			}
			return nStart;
		}
		return -1;
	}
}

//-----------------------------------------------------------------------------------
int CFlashBackup::RetrieveDrive(int nDrive, void* buffer)
{
	int nStart = GetStartDrive(nDrive);
	if(nStart != -2)
	{
		if(nStart != -1)
		{
			SeekDesc(m_hFile, nStart);
			if(buffer)
			{
				if(ReadDesc(m_hFile, buffer, DRIVE_SIZE))
				{
					if(CDriveAccess::IsEmpty(buffer, DRIVE_SIZE))
					{
						return 0;
					}
					else
					{
						return DRIVE_SIZE;
					}
				}
			}
			else
			{
				return DRIVE_SIZE;
			}
		}
		else
		{
			memset(buffer, 0xFF, DRIVE_SIZE);
			return 0;
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CFlashBackup::WriteDrive(int nDrive, const void* buffer)
{
	if(buffer && !CDriveAccess::IsEmpty(buffer, DRIVE_SIZE))
	{
		//Ecriture de donnes
		int nPos = GetStartDrive(nDrive);
		if(nPos != -1)
		{
			//Le lecteur exite dj, on l'crase simplement
			SeekDesc(m_hFile, nPos);
			return WriteDesc(m_hFile, buffer, DRIVE_SIZE);
		}
		else
		{
			//Le lecteur n'existe pas, on doit dcaler et ensuite l'insrer
			//Calcul de sa place thorique
			if(nDrive)
			{
				for(int i = nDrive - 1; i >= 0 && nPos == -1; i--)
				{
					nPos = GetStartDrive(i);
				}
			}
			if(nPos == -1)
			{
				nPos = BACKUP_HEADER_SIZE;
			}
			else
			{
				nPos += DRIVE_SIZE;
			}
			int nWrite = FileSize(m_hFile);
			char buf[DRIVE_SIZE];
			bool bOk = true;
			while(nWrite > nPos && nWrite > DRIVE_SIZE)
			{
				SeekDesc(m_hFile, nWrite - DRIVE_SIZE);
				bOk = ReadDesc(m_hFile, buf, DRIVE_SIZE);
				SeekDesc(m_hFile, nWrite);
				bOk = bOk && WriteDesc(m_hFile, buf, DRIVE_SIZE);
				nWrite -= DRIVE_SIZE;
			}
			if(bOk)
			{
				//Insre le lecteur et met  jour la FAT
				SeekDesc(m_hFile, nPos);
				m_FAT[nDrive] = 0xFF;
				return WriteDesc(m_hFile, buffer, DRIVE_SIZE);
			}
		}
		return false;
	}
	else
	{
		//Effacer le lecteur
		int nPos = GetStartDrive(nDrive);
		if(nPos != -1)
		{
			//Le lecteur existe donc on doit le supprimer
			char buf[DRIVE_SIZE];
			unsigned int nRead = FileSize(m_hFile) - nPos;
			bool bOk = true;
			while(nRead > DRIVE_SIZE && bOk)
			{
				SeekDesc(m_hFile, nPos + DRIVE_SIZE);
				bOk = ReadDesc(m_hFile, buf, DRIVE_SIZE);
				SeekDesc(m_hFile, nPos);
				bOk = bOk && WriteDesc(m_hFile, buf, DRIVE_SIZE);
				nPos  += DRIVE_SIZE;
				nRead -= DRIVE_SIZE;
			}
			//Diminue la taille du fichier et met  jour la FAT
			m_FAT[nDrive] = 0;
			return (bOk && SetFileSize(m_hFile, nPos) > 0);
		}
		else
		{
			//Le lecteur n'existe pas donc pas de changement
			return true;
		}
	}
}

//-----------------------------------------------------------------------------------
bool CFlashBackup::ClearBackup()
{
	memset(m_FAT, 0, sizeof(m_FAT));
	m_bValidFAT = true;
	return SetFileSize(m_hFile, sizeof(m_FAT)) > 0;
}

//-----------------------------------------------------------------------------------
bool CFlashBackup::Optimization()
{
	if(IsValid() > 0)
	{
		bool bOk = true;
		for(int i = 1; i < sizeof(m_FAT) && bOk; i++)
		{
			if(m_FAT[i] == 0)
			{
				bOk = WriteDrive(i, NULL);
			}
		}
		return bOk;
	}
	return false;
}
