
#ifndef FLASHBACKUP_H
#define FLASHBACKUP_H

#include <F100_Op.h>

#include "APIFunction.h"

#define BACKUP_HEADER_SIZE 7

/*!
	\class CFlashBackup FlashBackup.h
	\brief Permet de lire ou de crer un backup de la Flash
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
	Permet de grr les backups de Flash100. Ces backups contiennent le sytme et les 6 autres lecteurs de la flash.

*/
class DLLEXPORT CFlashBackup  
{
public:
	CFlashBackup();
	virtual ~CFlashBackup();

	/** Permet de savoir si le fichier est valide
	 * @return Retourne > 0 si valide \n
	 * -1 Invalide \n
	 * -2 Taille non valide
	 */
	int  IsValid();

	/** Permet d'ouvrir un fichier (ferme automatiquement l'ancien ouvert)
	 * @param[in] pName Chemin complet du fichier
	 * @param[in] bWriteMode A vrai si on veut crire sur le fichier
	 * @return Retourne vrai pour OK
	 */
	bool Open(const char* pName, bool bWriteMode);

	/** Ferme le fichier ouvert */
	void Close();
	

	/** Permet de rcuprer un lecteur
	 * @param[in] nDrive Numro du lecteur (0..6)
	 * @param[out] buffer Zone mmoire recevant les donnes
	 * @return Retourne la taille lu, -1 en cas d'erreur
	 * @remarks Si buffer == NULL, elle retourne juste la taille du lecteur
	 * @remarks La zone mmoire doit tre correctement cre (au moins DRIVE_SIZE octets)
	 */
	int  RetrieveDrive(int nDrive, void* buffer);

	/** Permet d'ajouter un lecteur
	 * @param[in] nDrive Numro du lecteur (0..6)
	 * @param[in] buffer Donnes  crire
	 * @return Retourne vrai en cas de succes
	 * @remarks Si buffer = NULL, ou vide (que 0xFF) supprime le lecteur
	 * @remarks la taille maximale est de SIZEDRIVE
	 */
	bool WriteDrive(int nDrive, const void* buffer);

	/** Efface toutes les donnes et cre la FAT
	 * @return Retourne vrai si l'opration a russi
	 * @remarks Si le fichier est vierge il faut appeler cette mthode avant d'ajouter un lecteur
	 */
	bool ClearBackup();

	/** Permet de supprimer les lecteurs vides
	 * @return Retourne vrai si l'opration a russi
	 */
	bool Optimization();

	/** Permet de connaitre la position d'un lecteur dans le backup
	 * @param[in] nDrive Numro du lecteur (0..6)
	 * @return Retourne (en octet) le dbut du lecteur \n
	 * -2 si erreur \n
	 * -1 si lecteur non trouv \n
	 */
	int  GetStartDrive(int nDrive);


private:
	HND           m_hFile;                  ///< Handle du fichier ouvert
	unsigned char m_FAT[BACKUP_HEADER_SIZE];///< FAT
	bool          m_bValidFAT;              ///< Indique si la FAT est valide
};

#endif
