
#include "Protocol.h"

#include <assert.h>

#include "APIFunction.h"

//-----------------------------------------------------------------------------------
CProtocol::CProtocol()
:IProtocol()
{
	m_nTransfert = TR_NONE;
	m_nError     = OK;
	m_nSpeeds    = 0;
	m_pSpeeds    = NULL;
	m_pComDevice = NULL;
}

//-----------------------------------------------------------------------------------
CProtocol::~CProtocol()
{
	CloseCom(0);
}

//-----------------------------------------------------------------------------------
int CProtocol::InitCom(IComDevice* pComDevice, int nSpeed)
{
	//Vrifie les attributs de vitesse qu'en mode DEBUG
	assert(m_nSpeeds > 0 && m_pSpeeds != NULL);
	m_pComDevice = pComDevice;
	if(nSpeed >= m_nSpeeds)
	{
		nSpeed = m_nSpeeds - 1;
	}
	if(m_pComDevice == NULL)
	{
		m_nError = BAD_CALL;
		return 0;
	}

	if(m_pComDevice->OpenPort(m_pSpeeds[nSpeed]) == 1)
	{
		m_nTransfert = TR_NONE;
		return 1;
	}
	m_nError = (ERRORS)m_pComDevice->GetLastError();
	return 0;
}

//-----------------------------------------------------------------------------------
void CProtocol::CloseCom(int nStatus)
{
	if(m_pComDevice)
	{
		m_nTransfert = TR_NONE;
		m_pComDevice->ClosePort();
	}
}

//-----------------------------------------------------------------------------------
int CProtocol::Receive(void* buffer, unsigned int nNbBytes, int nTimeOut)
{
	if(m_pComDevice == NULL)
	{
		m_nError = BAD_CALL;
		return 0;
	}
	if(m_pComDevice->Receive(buffer, nNbBytes, nTimeOut))
	{
		return 1;
	}
	m_nError = TIMEOUT;
	return 0;
}

//-----------------------------------------------------------------------------------
int CProtocol::Send(const void* buffer, unsigned int nNbBytes, int nTimeOut)
{
	if(m_pComDevice == NULL)
	{
		m_nError = BAD_CALL;
		return 0;
	}
	if(m_pComDevice->Send(buffer, nNbBytes, nTimeOut))
	{
		return 1;
	}
	m_nError = TIMEOUT;
	return 0;
}

//-----------------------------------------------------------------------------------
CProtocol::ERRORS CProtocol::GetLastError() const
{
	return m_nError;
}

//-----------------------------------------------------------------------------------
void CProtocol::SetError(ERRORS nError)
{
	m_nError = nError;
}

//-----------------------------------------------------------------------------------
int CProtocol::SetTransfertMode(TRANSFERT nTransfert)
{
	m_nTransfert = nTransfert;
	return 1;
}

//-----------------------------------------------------------------------------------
CProtocol::TRANSFERT CProtocol::GetTransfertMode() const
{
	return m_nTransfert;
}

//-----------------------------------------------------------------------------------
int* CProtocol::GetAvailableSpeeds(int& nNumber) const
{
	nNumber = m_nSpeeds;
	return m_pSpeeds;
}

//-----------------------------------------------------------------------------------
void CProtocol::SetAvailableSpeeds(int* pSpeeds, int nNbSpeed)
{
	m_pSpeeds = pSpeeds;
	m_nSpeeds = nNbSpeed;
}
