// SerialDevice.h: interface for the CSerialDevice class.
// Classe permettant d'envoyer/recevoir des donnes  un cble utilsant
// le port srie. (ou un cble  interface USB en utilisant un driver
// pour le grer comme un port srie)
// Cbles grs : Auto, FXI, SB87, MAX232
//////////////////////////////////////////////////////////////////////

#ifndef SERIALDEVICE_H
#define SERIALDEVICE_H

#include <IComDevice.h>
#include <F100_Op.h>

#include "APIFunction.h"

/*!
	\class CSerialDevice SerialDevice.h
	\brief Classe permettant d'envoyer/recevoir des donnes au port srie.
	\author Quaranta David
	\version 1.02
	Cette clase peut grer les cbles utilsant le port srie. (ou un cble  interface USB en utilisant un pilote
	pour le grer comme un port srie) \n
	Cbles grs : Auto, FXI, SB87, MAX232

	\ingroup F100TK

*/
class CSerialDevice : public IComDevice
{
public:
	CSerialDevice();
	virtual ~CSerialDevice();
	
	void         Init(int nPort, int nCable);

	int          OpenPort(int nSpeed);
	void         ClosePort();

	int          Send(const void* buffer, unsigned int nNbBytes, int nTimeOut = -1);
	int          Receive(void* buffer, unsigned int nNbBytes, int nTimeOut = -1);
	ERRORS       GetLastError() const;
	const char** GetCables(int& nNbCable) const;
	int          GetMaxSpeed() const;
	int          GetNbPort() const;
	void         GetPortName(int nPort, char* pBuffer, int nBufferSize) const;

private:
	/** Permet de dfinir le temps d'attente
	 * @param[in] nTimeOut Temps d'attente en milli-seconde
	 * @return Retourne non-zro pour Ok
	 */
	int          SetTimeOut(int nTimeOut);

private:
	int         m_nPort;   ///< Numro du port  ouvrir
	int         m_nCable;  ///< Numro du cble  utiliser
	HND         m_hCom;    ///< Handle du port srie ouvert
	ERRORS      m_nError;  ///< Dernire erreur
	int         m_nTimeOut;///< Temps d'attente
};

#endif
