// ApperanceTab.cpp: implementation of the CApperanceTab class.
//
//////////////////////////////////////////////////////////////////////

#include "ApperanceTab.h"
#include "GeneralTab.h"

#include <qapplication.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qstylefactory.h>
#include <qstyle.h>

#include <Label.h>

#define F100_REG_KEY_OPTIONWINDOW_USEDCUSTOMPALETTE "UsedCustomPalette"
#define F100_REG_KEY_OPTIONWINDOW_COLORS            "Colors"
#define F100_REG_KEY_OPTIONWINDOW_STYLE             "Style"

#define MAX_COLOR_ROLES 10

static QColorGroup::ColorRole nColorRoles[] = {QColorGroup::Background, QColorGroup::Foreground, QColorGroup::Base,
QColorGroup::Text, QColorGroup::Button, QColorGroup::ButtonText, QColorGroup::Highlight,
QColorGroup::HighlightedText, QColorGroup::Link, QColorGroup::LinkVisited};

//-----------------------------------------------------------------------------------
CApperanceTab::CApperanceTab(IMainWindow* pMainWindow)
	: CSettingTabWidget(pMainWindow, "OptionWindow/Appearance", TRUE)
{
	QBoxLayout* pBoxLayout = new QBoxLayout(this, QBoxLayout::TopToBottom, 10);

	m_pStyleGroup	= new QVGroupBox(this);
	m_pGroupPalette = new QVGroupBox(this);

	pBoxLayout->addWidget(m_pStyleGroup,   0, AlignVCenter);
	pBoxLayout->addWidget(m_pGroupPalette, 0, AlignVCenter);
	pBoxLayout->insertStretch(2, 1);
	
	m_pComboStyle = new QComboBox(m_pStyleGroup);
	QStringList lStyleList = QStyleFactory::keys();
	lStyleList.sort();
	for(QStringList::Iterator it = lStyleList.begin(); it != lStyleList.end(); ++it) {
		m_pComboStyle->insertItem(*it);
	}

	m_pChkUsedCustomPalette = new QCheckBox(m_pGroupPalette);
	m_pComboPalette			= new QComboBox(m_pGroupPalette);
	m_pComboColorRole       = new QComboBox(m_pGroupPalette);
	m_pCmdChangeColor	    = new CLabel(m_pGroupPalette);
	m_pCmdCopyPalette	    = new QPushButton(m_pGroupPalette);
	m_pCmdDefaultPalette    = new QPushButton(m_pGroupPalette);

	m_pCmdChangeColor->SetColorChooser(QColor());

	connect(m_pChkUsedCustomPalette, SIGNAL(clicked()),      SLOT(OnRefreshStyle()));
	connect(m_pComboPalette,         SIGNAL(activated(int)), SLOT(OnColorTypeChanged()));
	connect(m_pComboColorRole,       SIGNAL(activated(int)), SLOT(OnColorTypeChanged()));
	connect(m_pCmdCopyPalette,       SIGNAL(clicked()),      SLOT(OnCopySelectedPalette()));
	connect(m_pCmdDefaultPalette,    SIGNAL(clicked()),      SLOT(OnDefaultPalette()));
	connect(m_pCmdChangeColor, SIGNAL(colorChanged(const QColor&)), SLOT(OnColorChanged(const QColor&)));
	connect(m_pComboStyle, SIGNAL(activated(const QString&)), SLOT(OnRefreshStyle()));

	m_qCustomPalette = palette();
	
	DoInitWords();
	DoRefresh();
}

//-----------------------------------------------------------------------------------
int CApperanceTab::DoSave()
{
	QString sColors;
	for(int n = 0; n < 2; n++) {
		for(int i = 0; i < MAX_COLOR_ROLES; i++) {
			int r, g, b;
			QColor color = GetColor(n == 0, nColorRoles[i]);
			color.rgb(&r, &g, &b);
			sColors += QString::number(r) + ';'
				+ QString::number(g) + ';'
				+ QString::number(b) + ';';
		}
	}

	WriteSettings(F100_REG_KEY_OPTIONWINDOW_USEDCUSTOMPALETTE, m_bUsedCustomPalette);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_STYLE,  m_pComboStyle->currentText());
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_COLORS, sColors);

	return 1;
}

//-----------------------------------------------------------------------------------
void CApperanceTab::DoRefresh()
{
	int nRet;
	QString sColors, sStyle;
	m_bUsedCustomPalette = FALSE;

	ReadSettings(F100_REG_KEY_OPTIONWINDOW_USEDCUSTOMPALETTE, nRet, 0);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_COLORS, sColors);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_STYLE,  sStyle);

	m_bUsedCustomPalette = (nRet != 0);

	if(!sColors.isEmpty()) {
		int nCpt = 0;
		for(int n = 0; n < 2; n++) {
			QColor color;
			for(int i = 0; i < MAX_COLOR_ROLES; i++) {
				int r = sColors.section(';', nCpt, nCpt).toInt();
				int g = sColors.section(';', nCpt + 1, nCpt + 1).toInt();
				int b = sColors.section(';', nCpt + 2, nCpt + 2).toInt();
				color.setRgb(r, g, b);
				SetColor(n == 0, nColorRoles[i], color);
				nCpt += 3;
			}
		}
	}

	m_pChkUsedCustomPalette->setChecked(m_bUsedCustomPalette);

	if(!CGeneralTab::SetTextInComboBox(m_pComboStyle, sStyle)) {
		m_pComboStyle->setCurrentText(style().name());
	}

	RefreshStyle();

	if(sColors.isEmpty()) {
		m_qCustomPalette = palette();
	}
}

//-----------------------------------------------------------------------------------
void CApperanceTab::DoInitWords()
{
	setCaption(tr("Apparence"));

	m_pStyleGroup->setTitle(tr("Style"));

	m_pGroupPalette->setTitle(tr("Palettes de couleurs"));
	m_pChkUsedCustomPalette->setText(tr("Utiliser des palettes personnalises"));
	m_pCmdCopyPalette->setText(tr("Copier la palette slectionne"));
	m_pCmdDefaultPalette->setText(tr("Palettes par dfaut du style slectionn"));
	
	m_pComboPalette->clear();
	m_pComboPalette->insertItem(tr("Palette Active"),     0);
	m_pComboPalette->insertItem(tr("Palette Dsactive"), 1);

	m_pComboColorRole->clear();
	m_pComboColorRole->insertItem(tr("Fond"),                  0);
	m_pComboColorRole->insertItem(tr("Texte de fond"),         1);
	m_pComboColorRole->insertItem(tr("Fond du texte"),         2);
	m_pComboColorRole->insertItem(tr("Texte"),                 3);
	m_pComboColorRole->insertItem(tr("Bouton"),                4);
	m_pComboColorRole->insertItem(tr("Texte des boutons"),     5);
	m_pComboColorRole->insertItem(tr("Surbrillance"),          6);
	m_pComboColorRole->insertItem(tr("Texte en surbrillance"), 7);
	m_pComboColorRole->insertItem(tr("Lien"),                  8);
	m_pComboColorRole->insertItem(tr("Lien visit"),           9);
	
	OnColorTypeChanged();
}

//-----------------------------------------------------------------------------------
QString CApperanceTab::GetDescription() const
{
	return tr("Permet de modifier l'apparence graphique de l'application");
}

//-----------------------------------------------------------------------------------
void CApperanceTab::OnColorChanged(const QColor& color)
{
	int nColorRole = m_pComboColorRole->currentItem();
	bool bActivePalette = m_pComboPalette->currentItem() == 0;
	SetColor(bActivePalette, nColorRoles[nColorRole], color);
	OnRefreshStyle();
}

//-----------------------------------------------------------------------------------
void CApperanceTab::OnColorTypeChanged()
{
	bool bActivePalette = m_pComboPalette->currentItem() == 0;
	int  nColorRole     = m_pComboColorRole->currentItem();
	m_pCmdChangeColor->SetColor(GetColor(bActivePalette, nColorRoles[nColorRole]));
}

//-----------------------------------------------------------------------------------
void CApperanceTab::RefreshStyle()
{
	m_bUsedCustomPalette = m_pChkUsedCustomPalette->isChecked();
	OnColorTypeChanged();

	QApplication::setStyle(m_pComboStyle->currentText());

	bool bHaveTexture = FALSE;

	for(int i = 0; i < MAX_COLOR_ROLES && !bHaveTexture; i++) {
		bHaveTexture = palette().active().brush(nColorRoles[i]).pixmap() != NULL
			|| palette().inactive().brush(nColorRoles[i]).pixmap() != NULL
			|| palette().disabled().brush(nColorRoles[i]).pixmap() != NULL;
	}

	bool bEnabled = m_bUsedCustomPalette && !bHaveTexture;
	if(bEnabled) {
		QApplication::setPalette(m_qCustomPalette, TRUE);
	}

	m_pChkUsedCustomPalette->setEnabled(!bHaveTexture);

	m_pComboPalette->setEnabled(bEnabled);
	m_pComboColorRole->setEnabled(bEnabled);
	m_pCmdChangeColor->setEnabled(bEnabled);
	m_pCmdCopyPalette->setEnabled(bEnabled);
	m_pCmdDefaultPalette->setEnabled(bEnabled);
}

//-----------------------------------------------------------------------------------
void CApperanceTab::OnRefreshStyle()
{
	RefreshStyle();
	DoSave();
}

//-----------------------------------------------------------------------------------
void CApperanceTab::OnDefaultPalette()
{
	QApplication::setStyle(m_pComboStyle->currentText());
	m_qCustomPalette = palette();
	OnRefreshStyle();
}

//-----------------------------------------------------------------------------------
QColor CApperanceTab::GetColor(bool bActivePalette, QColorGroup::ColorRole nColorRole)
{
	if(bActivePalette) {
		return m_qCustomPalette.color(QPalette::Active, nColorRole);
	}
	return m_qCustomPalette.color(QPalette::Disabled, nColorRole);
}

//-----------------------------------------------------------------------------------
void CApperanceTab::SetColor(bool bActivePalette, QColorGroup::ColorRole nColorRole, const QColor &color)
{
	if(bActivePalette) {
		m_qCustomPalette.setColor(QPalette::Active,   nColorRole, color);
		m_qCustomPalette.setColor(QPalette::Inactive, nColorRole, color);
	}
	else {
		m_qCustomPalette.setColor(QPalette::Disabled, nColorRole, color);
	}
}

//-----------------------------------------------------------------------------------
void CApperanceTab::OnCopySelectedPalette()
{
	bool bActivePalette = m_pComboPalette->currentItem() == 0;
	for(int i = 0; i < MAX_COLOR_ROLES; i++) {
		if(nColorRoles[i] != QColorGroup::Text) {
			SetColor(!bActivePalette, nColorRoles[i], GetColor(bActivePalette, nColorRoles[i]));
		}
	}
	OnRefreshStyle();
}
