// CommunicationTab.cpp: implementation of the CCommunicationTab class.
//
//////////////////////////////////////////////////////////////////////

#include "CommunicationTab.h"
#include "GeneralTab.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qdir.h>
#include <qlibrary.h>

#include <F100Devices.h>
#include <F100Devices.h>
#include <TransfertWindow.h>
#include <MessageBox.h>

#ifndef PROTOCOL_DIR
#	define PROTOCOL_DIR (APP_DIR_PATH + "/Protocols")
#endif

#define F100_REG_KEY_OPTIONWINDOW_PROTOCOL "Protocol"
#define F100_REG_KEY_OPTIONWINDOW_SPEED    "Speed"
#define F100_REG_KEY_OPTIONWINDOW_PORT     "Port"
#define F100_REG_KEY_OPTIONWINDOW_CABLE    "Cable"
#define F100_REG_KEY_OPTIONWINDOW_DEVICE   "Device"

//-----------------------------------------------------------------------------------
CCommunicationTab::CCommunicationTab(IMainWindow* pMainWindow)
	: CSettingTabWidget(pMainWindow, "OptionWindow/Communication", TRUE)
{
	QGridLayout* pLayout = new QGridLayout(this, 6, 3, 10);

	m_pLblDevice     = new QLabel(this);
	m_pComboDevice   = new QComboBox(FALSE, this);
	m_pLblPort       = new QLabel(this);
	m_pComboPort     = new QComboBox(FALSE, this);
	m_pLblCable      = new QLabel(this);
	m_pComboCable    = new QComboBox(FALSE, this);
	m_pLblProtocol   = new QLabel(this);
	m_pComboProtocol = new QComboBox(FALSE, this);
	m_pLblSpeed      = new QLabel(this);
	m_pComboSpeed    = new QComboBox(FALSE, this);

	pLayout->addWidget(m_pLblDevice,     0, 0);
	pLayout->addWidget(m_pComboDevice,   0, 1);
	pLayout->addWidget(m_pLblPort,       1, 0);
	pLayout->addWidget(m_pComboPort,     1, 1);
	pLayout->addWidget(m_pLblCable,      2, 0);
	pLayout->addWidget(m_pComboCable,    2, 1);
	pLayout->addWidget(m_pLblProtocol,   3, 0);
	pLayout->addWidget(m_pComboProtocol, 3, 1);
	pLayout->addWidget(m_pLblSpeed,      4, 0);
	pLayout->addWidget(m_pComboSpeed,    4, 1);

	pLayout->setRowStretch(5, 1);
	pLayout->setColStretch(2, 1);

	m_pLblDevice->setBuddy(m_pComboDevice);
	m_pLblPort->setBuddy(m_pComboPort);
	m_pLblCable->setBuddy(m_pComboCable);
	m_pLblProtocol->setBuddy(m_pComboProtocol);
	m_pLblSpeed->setBuddy(m_pComboSpeed);

	for(int i = 0; i < CF100Devices::GetNbComDevices(); ++i) {
		m_pComboDevice->insertItem(CF100Devices::GetComDeviceName(i));
	}

	connect(m_pComboDevice,   SIGNAL(activated(int)), SLOT(OnDeviceChanged()));
	connect(m_pComboProtocol, SIGNAL(activated(int)), SLOT(OnProtocolChanged()));

	LoadProtocols();
	DoRefresh();
	DoInitWords();
}

//-----------------------------------------------------------------------------------
int CCommunicationTab::DoSave()
{
	QString sProtocol;

	m_nDevice   = m_pComboDevice->currentItem();
	m_nPort     = m_pComboPort->currentItem();
	m_nCable    = m_pComboCable->currentItem();
	m_nProtocol = m_pComboProtocol->currentItem();
	m_nSpeed    = m_pComboSpeed->currentItem();

	if(!m_vProtocols.isEmpty()) {
		sProtocol = m_vProtocols[m_nProtocol].sText;
	}

	WriteSettings(F100_REG_KEY_OPTIONWINDOW_PROTOCOL, sProtocol);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_SPEED,    m_nSpeed);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_PORT,     m_nPort);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_CABLE,    m_nCable);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_DEVICE,   m_nDevice);

	return 1;
}

//-----------------------------------------------------------------------------------
void CCommunicationTab::DoRefresh()
{
	QString sProtocol;

	ReadSettings(F100_REG_KEY_OPTIONWINDOW_PROTOCOL,   sProtocol);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_SPEED,      m_nSpeed,  0);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_PORT,       m_nPort,   0);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_DEVICE,     m_nDevice, 0);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_CABLE,      m_nCable,  0);

	ProtocolsVector::Iterator it = m_vProtocols.begin();
	for(int i = 0; it != m_vProtocols.end(); ++i) {
		if((*it).sText == sProtocol) {
			m_nProtocol = i;
			break;
		}
		++it;
	}

	CGeneralTab::SetIndexInComboBox(m_pComboDevice, m_nDevice);
	OnDeviceChanged();
	CGeneralTab::SetIndexInComboBox(m_pComboPort,   m_nPort);
	CGeneralTab::SetIndexInComboBox(m_pComboCable,  m_nCable);

	CGeneralTab::SetIndexInComboBox(m_pComboProtocol, m_nProtocol);
	OnProtocolChanged();

	CGeneralTab::SetIndexInComboBox(m_pComboSpeed, m_nSpeed);
}

//-----------------------------------------------------------------------------------
void CCommunicationTab::DoInitWords()
{
	setCaption(tr("Communication"));
	QString s(" :");
	m_pLblDevice->setText(tr("P&riphrique") + s);
	m_pLblPort->setText(tr("&Port") + s);
	m_pLblCable->setText(tr("&Cble") + s);
	m_pLblProtocol->setText(tr("Pr&otocole") + s);
	m_pLblSpeed->setText(tr("&Vitesse") + s);

	for(int i = 0; i < m_pComboProtocol->count(); i++) {
		TProtocol tp = m_vProtocols[i];
		m_pComboProtocol->changeItem(tp.sText.arg(CCalcInfo::GetCalcType(tp.ct)), i);
	}

}

//-----------------------------------------------------------------------------------
QString CCommunicationTab::GetDescription() const
{
	return tr("Permet de rgler les paramtres de communication entre le PC et la calculatrice");
}

//-----------------------------------------------------------------------------------
void CCommunicationTab::OnDeviceChanged()
{
	IComDevice* pComDevice = CF100Devices::GetComDevice(m_pComboDevice->currentItem());
	m_pComboPort->clear();
	m_pComboCable->clear();
	if(pComDevice != NULL) {
		int nNbCable;
		const char** pCables = pComDevice->GetCables(nNbCable);
		int i;
		for(i = 0; i < nNbCable; ++i) {
			m_pComboCable->insertItem(pCables[i]);
		}
		int nMax = pComDevice->GetNbPort();
		for(i = 0; i < nMax; ++i) {
			char name[128];
			pComDevice->GetPortName(i, name, sizeof(name));
			m_pComboPort->insertItem(name);
		}
	}
}

//-----------------------------------------------------------------------------------
void CCommunicationTab::OnProtocolChanged()
{	
	IProtocol* pProtocol = NULL;
	Protocol_Op op;
	if(GetProtocolOps(op, true)) {
		pProtocol = CreateProtocol(op);
	}

	m_pComboSpeed->clear();
	if(pProtocol != NULL) {
		int nCpt;
		int* pSpeeds = pProtocol->GetAvailableSpeeds(nCpt);
		for(int i = 0; i < nCpt; i++) {
			m_pComboSpeed->insertItem(QString::number(pSpeeds[i]) + " bps", i);
		}
		DeleteProtocol(op, pProtocol);
		m_pComboSpeed->setCurrentItem(nCpt - 1);
	}
}

//-----------------------------------------------------------------------------------
IProtocol* CCommunicationTab::CreateProtocol(Protocol_Op& op)
{
	return (IProtocol*)op(OP_CREATEINSTANCE, NULL);
}

//-----------------------------------------------------------------------------------
bool CCommunicationTab::DeleteProtocol(Protocol_Op& op, IProtocol* pProtocol)
{
	if(pProtocol != NULL) {
		op(OP_DELETEINSTANCE, pProtocol);
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCommunicationTab::GetProtocolOps(Protocol_Op& op, bool bCurrent)
{
	if(!bCurrent) {
		CGeneralTab::SetIndexInComboBox(m_pComboProtocol, m_nProtocol);
	}
	if(m_vProtocols.size() != 0) {
		int nProtocol = m_pComboProtocol->currentItem();
		op = m_vProtocols[nProtocol].op;
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
void CCommunicationTab::LoadProtocols()
{
	m_vProtocols.clear();
	m_pComboProtocol->clear();

	QString sPath = PROTOCOL_DIR;
	QDir d(sPath);
	if(d.exists())
	{
		d.setFilter(QDir::Files);
		d.setNameFilter(DLL_EXTENSION);
		const QFileInfoList* pList = d.entryInfoList();
		QFileInfoListIterator it(*pList);
		QFileInfo* pFileInfo;
		int nCpt = 0;
		for(; (pFileInfo = it.current()) != NULL; ++it)
		{
			Protocol_Op op = (Protocol_Op)QLibrary::resolve(pFileInfo->absFilePath(), EP_PROTOCOL_REQUEST);
			if(op != NULL)
			{
				const char* sName;
				const char* sVersion;
				sName    = (const char*)op(OP_TOOLNAME, NULL);
				sVersion = (const char*)op(OP_VERSION,  NULL);

				QString sText = QString("%1 (%3) v%2").arg(sName, sVersion);

				IProtocol* pProtocol = CreateProtocol(op);
				if(pProtocol != NULL) {
					int n;
					if(pProtocol->GetAvailableSpeeds(n) != NULL && n > 0) {
						TProtocol tp;
						tp.op    = op;
						tp.sText = sText;
						tp.ct    = pProtocol->GetCasioType();
						m_vProtocols.push_back(tp);
						m_pComboProtocol->insertItem(QString::null, nCpt++);
					}
					else {
						qWarning(tr("Le protocole '%1' a t refus car l'accs aux vitesses supportes a chou.")
							.arg(sText));
					}
					
					DeleteProtocol(op, pProtocol);
				}
				else {
					qWarning(tr("Le protocole '%1' a t refus car la cration du protocole a chou.")
						.arg(sText));
				}
			}
			else {
				qWarning(tr("Le fichier '%1' n'a pas pu tre charg.")
						.arg(pFileInfo->absFilePath()));
			}
		}
	}
	if(m_pComboProtocol->count() == 0) {
		m_pComboProtocol->setEnabled(FALSE);
		m_pComboSpeed->setEnabled(FALSE);
		m_pLblProtocol->setEnabled(FALSE);
		m_pLblSpeed->setEnabled(FALSE);
	}
	OnProtocolChanged();
}

//-----------------------------------------------------------------------------------
CTransfertWindow* CCommunicationTab::InitTransfertWindow(CTransfertWindow* pTransfertWindow, int nTotalSize)
{
	Protocol_Op op;
	if(GetProtocolOps(op)) {
		IComDevice* pComDevice = CF100Devices::GetComDevice(m_nDevice);
		pComDevice->Init(m_nPort, m_nCable);
		if(pTransfertWindow->SetProtocol(op, pComDevice, m_nSpeed, nTotalSize) < 1) {
			delete pTransfertWindow;
			pTransfertWindow = NULL;
		}
	}
	else {
		CMessageBox::MsgCritical(pTransfertWindow->parentWidget(), tr("Impossible de lancer de communication car aucun protocole n'a t trouv."));
		delete pTransfertWindow;
		pTransfertWindow = NULL;
	}
	return pTransfertWindow;
}
