// CommunicationTab.h: interface for the CCommunicationTab class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _COMMUNICATIONTAB_H_
#define _COMMUNICATIONTAB_H_

#include <qvaluevector.h>

#include <SettingTabWidget.h>
#include <IProtocol.h>
#include <F100_Op.h>

class QComboBox;
class QLabel;
class CTransfertWindow;

class CCommunicationTab : public CSettingTabWidget
{
	Q_OBJECT
public:
	CCommunicationTab(IMainWindow* pMainWindow);

	int     DoSave();
	void    DoRefresh();
	void    DoInitWords();
	QString GetDescription() const;

	CTransfertWindow* InitTransfertWindow(CTransfertWindow* pTransfertWindow, int nTotalSize);

private:
	bool              GetProtocolOps(Protocol_Op& op, bool bCurrent = false);
	IProtocol*        CreateProtocol(Protocol_Op& op);
	bool              DeleteProtocol(Protocol_Op& op, IProtocol* pProtocol);
	void              LoadProtocols();

private slots:
	void OnDeviceChanged();
	void OnProtocolChanged();
	
private:
	struct TProtocol {
		Protocol_Op op;
		CasioType   ct;
		QString     sText;
	};
	
	typedef QValueVector<TProtocol> ProtocolsVector;
	
private:
	QComboBox* m_pComboProtocol;
	QComboBox* m_pComboSpeed;
	QComboBox* m_pComboCable;
	QComboBox* m_pComboPort;
	QComboBox* m_pComboDevice;
	QLabel*    m_pLblDevice;
	QLabel*    m_pLblPort;
	QLabel*    m_pLblCable;
	QLabel*    m_pLblProtocol;
	QLabel*    m_pLblSpeed;

	int        m_nProtocol;
	int		   m_nSpeed;
	int        m_nPort;
	int        m_nCable;
	int        m_nDevice;

	ProtocolsVector m_vProtocols;
};

#endif
