// EditLinkTab.cpp: implementation of the CEditLinkTab class.
//
//////////////////////////////////////////////////////////////////////

#include "EditLinkTab.h"
#include "registry.h"

#include <qlistview.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qaction.h>

#include <language.h>
#include <IMainWindow.h>

CEditLinkTab* CEditLinkTab::m_pInstance = NULL;

//-----------------------------------------------------------------------------------
CEditLinkTab::CEditLinkTab(IMainWindow* pMainWindow)
	: CSettingTabWidget(pMainWindow, F100_REG_KEY_MAINWINDOW_LINKS, TRUE)
{
	m_pInstance = this;
	QBoxLayout* pLayout = new QBoxLayout(this, QBoxLayout::TopToBottom);
	m_pLinksList = new QListView(this);

	pLayout->addWidget(m_pLinksList);

	m_pLinksList->setAllColumnsShowFocus(TRUE);
	m_pLinksList->setRootIsDecorated(TRUE);
	m_pLinksList->setSorting(0);

	m_pLinksList->addColumn( QString::null );
	m_pLinksList->addColumn( QString::null );

	connect(m_pLinksList, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)), SLOT(OnRightButtonClicked(QListViewItem*, const QPoint&)));
	connect(m_pLinksList, SIGNAL(itemRenamed(QListViewItem*, int)), SLOT(OnItemRenamed(QListViewItem*, int)));

	DoInitWords();
}
//-----------------------------------------------------------------------------------
int CEditLinkTab::DoSave()
{
	if(IsDifferent()) {
		GetMainWindow()->RemoveSettings(GetKey(), TRUE);
		CreateLinks(QString::null, m_pLinksList->firstChild());
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::DoRefresh()
{
	m_pLinksList->clear();
	LoadLinks(QString::null, NULL);
	if(m_pLinksList->childCount() == 0) {
		WriteSettings("Forums/Graph100 (Franais)", "http://www.graph100.com/forum");
		WriteSettings("Forums/UCF", "http://www.casiocalc.org");
		WriteSettings("Flash100",   "http://flash100.free.fr/xoops");
		WriteSettings("Sites/CasioWorld", "http://www.casioworld.fr.st");
		WriteSettings("Sites/TeamG100",   "http://teamg100.earthforge.com");
		LoadLinks(QString::null, NULL);
	}
	m_sLastLinks = QString::null;
	Serialize(m_sLastLinks, QString::null, NULL);
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::DoInitWords()
{
	m_pLinksList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pLinksList->setColumnText(1, tr("Adresse"));
	setCaption(tr("Liens"));
}

//-----------------------------------------------------------------------------------
QString CEditLinkTab::GetDescription() const
{
	return tr("Permet d'ajouter/enlever des liens");
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::LoadLinks(const QString& sKey, QListViewItem* pItem)
{
	QStringList sSubKeys = SubSettingsList(sKey);
	QListViewItem* pChildItem = NULL;
	if(!sSubKeys.isEmpty()) {
		sSubKeys.sort();
		QStringList::Iterator it    = sSubKeys.begin();
		QStringList::Iterator itEnd = sSubKeys.end();
		for(; it != itEnd; ++it) {
			if(pItem != NULL) {
				pChildItem = new QListViewItem(pItem, pChildItem);
			}
			else {
				pChildItem = new QListViewItem(m_pLinksList, pChildItem);
			}
			pChildItem->setText(0, *it);
			pChildItem->setRenameEnabled(0, TRUE);
			LoadLinks(sKey + (*it) + '/', pChildItem);
		}
	}
	QStringList sLinks = SettingsList(sKey);
	if(!sLinks.isEmpty()) {
		sLinks.sort();
		QStringList::Iterator it    = sLinks.begin();
		QStringList::Iterator itEnd = sLinks.end();
		for(; it != itEnd; ++it) {
			if(pItem != NULL) {
				pChildItem = new QListViewItem(pItem, pChildItem);
			}
			else {
				pChildItem = new QListViewItem(m_pLinksList, pChildItem);
			}
			QString sPath = sKey + (*it);
			if(pChildItem != NULL) {
				QString sValue;
				ReadSettings(sPath, sValue);
				pChildItem->setText(0, *it);
				pChildItem->setText(1, sValue);
				pChildItem->setRenameEnabled(0, TRUE);
				pChildItem->setRenameEnabled(1, TRUE);
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::Serialize(QString& sLinks, const QString& sKey, QListViewItem* pItem)
{
	QListViewItem* pChildItem;
	if(pItem != NULL) {
		pChildItem = pItem->firstChild();
	}
	else {
		pChildItem = m_pLinksList->firstChild();
	}

	while(pChildItem != NULL) {
		if(pChildItem->childCount() > 0) {
			Serialize(sLinks, sKey + pChildItem->text(0) + '/', pChildItem);
		}
		else {
			sLinks += sKey + pChildItem->text(0) + ':' + pChildItem->text(1) + '\n';
		}
		pChildItem = pChildItem->nextSibling();
	}
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::OnRightButtonClicked(QListViewItem* pItem, const QPoint& pos)
{
	QPopupMenu menu(this);
	menu.insertItem(tr("Ajouter un lment"), 0);
	menu.insertSeparator();
	menu.insertItem(CLanguage::GetWordTranslation(CLanguage::WO_RENAME), 1);
	menu.insertItem(tr("Modifier l'adresse"), 2);
	menu.insertSeparator();
	menu.insertItem(tr("Supprimer"), 3);

	
	menu.setItemEnabled(1, pItem);
	menu.setItemEnabled(2, pItem && pItem->childCount() == 0);
	menu.setItemEnabled(3, pItem);

	switch(menu.exec(pos))
	{
	case 0://Ajouter un lment
		{
			QListViewItem* pChildItem;
			if(pItem != NULL) {
				pChildItem = new QListViewItem(pItem, tr("Nouveau"));
				//pItem devient un parent
				pItem->setText(1, QString::null);
				pItem->setRenameEnabled(1, FALSE);
			}
			else {
				pChildItem = new QListViewItem(m_pLinksList, tr("Nouveau"));
			}
			pChildItem->setRenameEnabled(0, TRUE);
			pChildItem->setRenameEnabled(1, TRUE);
			pChildItem->setText(1, "http://");
			pChildItem->startRename(0);
		}
		break;
	case 1://Renommer
		pItem->startRename(0);
		break;
	case 2://Modifier l'adresse
		pItem->startRename(1);
		break;
	case 3://Supprimer l'lment
		{
			QListViewItem* pParentItem = pItem->parent();
			if(pParentItem != NULL) {
				pParentItem->takeItem(pItem);
				if(pParentItem->childCount() == 0) {
					pParentItem->setRenameEnabled(1, TRUE);//r-active la modification de l'adresse
				}
			}
			else {
				m_pLinksList->takeItem(pItem);
			}
			delete pItem;
		}
		break;
	}
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::OnItemRenamed(QListViewItem* pItem, int nCol)
{
	if(pItem != NULL && nCol == 0) {
		QListViewItem* p;
		int nCpt = 0;
		QString sText = pItem->text(0);
		do {
			nCpt++;
			if(pItem->parent() != NULL) {
				p = pItem->parent()->firstChild();
			}
			else {
				p = m_pLinksList->firstChild();
			}
			QString sLower = pItem->text(0).lower();
			while(p != NULL) {
				if(p != pItem && sLower == p->text(0).lower()) {
					pItem->setText(0, sText + QString("(%1)").arg(nCpt));
					break;
				}
				p = p->nextSibling();
			}
		} while(p != NULL);
		m_pLinksList->sort();
	}
}

//-----------------------------------------------------------------------------------
QString CEditLinkTab::GetPath(QListViewItem* pItem) const
{
	if(pItem != NULL) {
		return GetPath(pItem->parent()) + '/' + pItem->text(0);
	}
	return QString::null;
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::CreateLinks(const QString& sKey, QListViewItem* pItem)
{
	while(pItem != NULL) {
		if(pItem->childCount() == 0) {
			WriteSettings(sKey + pItem->text(0), pItem->text(1));
		}
		else {
			CreateLinks(sKey + pItem->text(0) + '/', pItem->firstChild());
		}
		pItem = pItem->nextSibling();
	}
}

//-----------------------------------------------------------------------------------
bool CEditLinkTab::IsDifferent()
{
	QString s;
	Serialize(s, QString::null, NULL);
	if(s != m_sLastLinks) {
		m_sLastLinks = s;
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
QPopupMenu* CEditLinkTab::CreateMenu(QWidget* parent, QObject* receiver, const char* member)
{
	DoRefresh();
	QPopupMenu* pMenu = new QPopupMenu(parent);
	UpdateMenu(pMenu, NULL, receiver, member);
	return pMenu;
}

//-----------------------------------------------------------------------------------
void CEditLinkTab::UpdateMenu(QPopupMenu* pMenu, QListViewItem* pItem, QObject* receiver, const char* member) const
{
	QListViewItem* pChildItem;
	if(pItem != NULL) {
		pChildItem = pItem->firstChild();
	}
	else {
		pChildItem = m_pLinksList->firstChild();
	}

	while(pChildItem != NULL) {
		if(pChildItem->childCount() > 0) {
			QPopupMenu* pChildMenu = new QPopupMenu(pMenu);
			pMenu->insertItem(pChildItem->text(0), pChildMenu);
			UpdateMenu(pChildMenu, pChildItem, receiver, member);
		}
		else {
			QAction* pAction = new QAction(receiver);
			connect(pAction, SIGNAL(activated()), receiver, member);
			pAction->addTo(pMenu);
			pAction->setMenuText(pChildItem->text(0));
			pAction->setText(pChildItem->text(1));
		}
		pChildItem = pChildItem->nextSibling();
	}
}

//-----------------------------------------------------------------------------------
CEditLinkTab* CEditLinkTab::GetInstance()
{
	return m_pInstance;
}
