// GeneralTab.cpp: implementation of the CGeneralTab class.
//
//////////////////////////////////////////////////////////////////////

#include "GeneralTab.h"
#include "ApperanceTab.h"
#include "CommunicationTab.h"
#include "EditLinkTab.h"
#include "assocfile.h"

#include <language.h>
#include <MessageBox.h>
#include <ToolWindow.h>
#include <TransfertWindow.h>

#include <qlayout.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qtranslator.h>
#include <qdir.h>
#include <qinputdialog.h>
#include <qapplication.h>
#include <qtextcodec.h>

#ifndef LANGUAGE_DIR
#	define LANGUAGE_PATH (APP_DIR_PATH + "/Languages")
#else
#	define LANGUAGE_PATH (QString(LANGUAGE_DIR))
#endif


#define F100_REG_KEY_OPTIONWINDOW_LANGUAGE          "Language"
#define F100_REG_KEY_OPTIONWINDOW_AUTOLAUNCH        "AutoLaunch"
#define F100_REG_KEY_OPTIONWINDOW_AUTOSAVEPROJECT   "AutoSaveProject"
#define F100_REG_KEY_OPTIONWINDOW_DRIVESIZE         "DriveSize"
#define F100_REG_KEY_OPTIONWINDOW_CALCNAME          "CalcName"
#define F100_REG_KEY_OPTIONWINDOW_CALCDIR           "CalcDir"
#define F100_REG_KEY_OPTIONWINDOW_MULTIPLE_INSTANCE "MultipleInstance"


//-----------------------------------------------------------------------------------
CGeneralTab::CGeneralTab(IMainWindow* pMainWindow)
	: CSettingTabWidget(pMainWindow, "OptionWindow", TRUE)
	, m_calcInfo(this)
#ifdef WIN32
	, m_pAssocController(NULL)
#endif
{
	AddChild(new CApperanceTab(pMainWindow));
	m_pCommunication = new CCommunicationTab(pMainWindow);
	AddChild(m_pCommunication);
	AddChild(new CEditLinkTab(pMainWindow));

	QBoxLayout* pBoxLayout = new QBoxLayout(this, QBoxLayout::TopToBottom, 5);

	m_pLanguageGroup = new QHGroupBox(this);
	m_pCalcGroup     = new QHGroupBox(this);
	m_pPrefGroup     = new QHGroupBox(this);
	m_pLblInfo       = new QLabel(this);
	m_pPrefList      = new QListView(m_pPrefGroup);

	pBoxLayout->addWidget(m_pPrefGroup);
	pBoxLayout->addWidget(m_pCalcGroup);
	pBoxLayout->addWidget(m_pLanguageGroup);
	pBoxLayout->addWidget(m_pLblInfo,  0, AlignBottom);

	m_pChkAutoLoad    = new QCheckListItem(m_pPrefList, QString::null, QCheckListItem::CheckBox);
	m_pChkMultiInst   = new QCheckListItem(m_pPrefList, m_pChkAutoLoad,  QString::null, QCheckListItem::CheckBox);
	m_pChkAutoSave    = new QCheckListItem(m_pPrefList, m_pChkMultiInst, QString::null, QCheckListItem::CheckBox);
	m_pSizeController = new QCheckListItem(m_pPrefList, m_pChkAutoSave,  QString::null, QCheckListItem::RadioButtonController);

	m_pDriveSize[0] = new QCheckListItem(m_pSizeController, QString::null, QCheckListItem::RadioButton);
	m_pDriveSize[1] = new QCheckListItem(m_pSizeController, m_pDriveSize[0], QString::null, QCheckListItem::RadioButton);

	m_pPrefList->addColumn(QString::null);
	m_pPrefList->header()->hide();
	m_pPrefList->setResizeMode(QListView::LastColumn);
	m_pPrefList->setSorting(-1);

	m_pSizeController->setOpen(TRUE);
	
	m_pComboLanguage = new QComboBox(FALSE, m_pLanguageGroup);
	
	m_pChangeCalc    = new QPushButton(m_pCalcGroup);
	m_pChangeCalcDir = new QPushButton(m_pCalcGroup);
	m_pCalcType      = new QComboBox(FALSE, m_pCalcGroup);

	connect(&m_calcInfo,      SIGNAL(InformationChanged()), SLOT(OnShowCalcName()));
	connect(m_pChangeCalc,    SIGNAL(clicked()), SLOT(OnChangeCalc()));
	connect(m_pChangeCalcDir, SIGNAL(clicked()), SLOT(OnChangeCalcDir()));
	connect(m_pCalcType,      SIGNAL(activated(const QString&)), SLOT(OnCalcTypeChanged()));

	Init();
	DoRefresh();
	DoInitWords();
#ifdef WIN32
	if(IntallF100Reg(tr("Fichier Flash100"))) {
		m_pAssocController = new QCheckListItem(m_pPrefList, m_pSizeController, QString::null, QCheckListItem::CheckBoxController);		
	}
#endif
}

//-----------------------------------------------------------------------------------
void CGeneralTab::LoadExtAssociations()
{
#ifdef WIN32
	if(m_pAssocController != NULL && m_pAssocController->childCount() == 0) {
		QString sFiles = GetMainWindow()->GetSupportedFiles();
		QString sExt;
		for(int nSection = 0; !((sExt = sFiles.section(';', nSection, nSection)).isEmpty()); nSection++) {
			QCheckListItem* pItem = new QCheckListItem(m_pAssocController, sExt, QCheckListItem::CheckBox);
			pItem->setOn(IsAssociateWith(sExt));
		}
		m_pAssocController->sortChildItems(0, TRUE);
		m_pAssocController->setOpen(TRUE);
	}
#endif
}

//-----------------------------------------------------------------------------------
void CGeneralTab::Init()
{
	m_pComboLanguage->clear();
	QDir d(LANGUAGE_PATH);
	if(d.exists()) {
		d.setFilter(QDir::Dirs);
		d.setSorting(QDir::Name);
		const QFileInfoList* pList = d.entryInfoList();
		if(pList != NULL) {
			QFileInfoListIterator it(*pList);
			QFileInfo* pFileInfo;
			for(int i = 0; (pFileInfo = it.current()) != 0; i++) {
				QString s = pFileInfo->fileName();
				if(s != "." && s != "..") {
					m_pComboLanguage->insertItem(s, -1);
				}
				++it;
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CGeneralTab::LoadLanguage()
{
	if(m_pComboLanguage->count() != 0) {
		int i = m_pComboLanguage->currentItem();
		if(OpenLanguageFile(m_pComboLanguage->currentText())) {
			m_sLanguageName = m_pComboLanguage->currentText();
		}
		else {
			CMessageBox::MsgCritical(this, tr("Impossible de charger ce langage!\nVrifiez que le dossier contient des fichiers au format *.qm"));
			SetTextInComboBox(m_pComboLanguage, m_sLanguageName);
		}
	}
}

//-----------------------------------------------------------------------------------
int CGeneralTab::DoSave()
{
	bool bDoRestart;
	//Sauvegarde de l'auto start
	m_nAutoLaunch = m_pChkAutoLoad->isOn();
	int nMultIns  = m_pChkMultiInst->isOn();
	bDoRestart    = (m_nMultIns != nMultIns);
	m_nMultIns    = nMultIns;
	m_nAutoSaveProject = m_pChkAutoSave->isOn();
	m_nDriveSetting = m_pDriveSize[1]->isOn();

	if(m_sLanguageName != m_pComboLanguage->currentText()) {
		LoadLanguage();
		WriteSettings(F100_REG_KEY_OPTIONWINDOW_LANGUAGE, m_sLanguageName);
	}
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_AUTOLAUNCH,        m_nAutoLaunch);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_MULTIPLE_INSTANCE, m_nMultIns);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_AUTOSAVEPROJECT,   m_nAutoSaveProject);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_DRIVESIZE,         m_nDriveSetting);
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_CALCNAME, m_calcInfo.GetCalcFile());
	WriteSettings(F100_REG_KEY_OPTIONWINDOW_CALCDIR,  CToolWindow::ConvertToRelativePath(m_calcInfo.GetCalcDir()));

#ifdef WIN32
	if(m_pAssocController != NULL) {
		//Sauve les associations
		QCheckListItem* pItem = (QCheckListItem*)m_pAssocController->firstChild();
		while(pItem != NULL) {
			if(pItem->isOn()) {
				AddAssocFile(pItem->text(0));
			}
			else {
				RemoveAssocFile(pItem->text(0));
			}
			pItem = (QCheckListItem*)pItem->nextSibling();
		}
	}
#endif
	if(bDoRestart) {
		return -2;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CGeneralTab::RestoreDefaults()
{
	int nMultIns = m_nMultIns;
	int nRet = CSettingTabWidget::RestoreDefaults();
	if(nMultIns != m_nMultIns) {
		return -2;
	}
	return nRet;
}

//-----------------------------------------------------------------------------------
void CGeneralTab::DoRefresh()
{
	QString sCalcName;
	QString sCalcDir;
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_CALCNAME, sCalcName);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_CALCDIR,  sCalcDir, "[myapp]/Calcs");

	sCalcDir = CToolWindow::ConvertToFullPath(sCalcDir);

	if(!m_calcInfo.SetCalcDir(sCalcDir) || !m_calcInfo.SetCalcFile(sCalcName)) {
		qWarning(tr("Le chargement des donnes de la calculatrice a chou. Fichier : %1").arg(sCalcDir + '/' + sCalcName));
	}

	QString sLanguageName;

	ReadSettings(F100_REG_KEY_OPTIONWINDOW_LANGUAGE,          sLanguageName);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_AUTOLAUNCH,        m_nAutoLaunch, 1);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_MULTIPLE_INSTANCE, m_nMultIns);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_AUTOSAVEPROJECT,   m_nAutoSaveProject, 0);
	ReadSettings(F100_REG_KEY_OPTIONWINDOW_DRIVESIZE,         m_nDriveSetting, 0);

	if(sLanguageName.isEmpty()) {
		//Tente de trouver la langue la mieux approprie
		//Si pas trouve dans la liste, choisi l'anglais
		QString sZone = QTextCodec::locale();
		static const char* sCountry[] = {"fr", "de", "es", "en"};
		static QString sFile[]		  = {"Francais", "Deutsch", "Espanol", "English"};
		int i;
		for(i = 0; i < 3 && sZone != sCountry[i]; i++);
		if(i < 4) {
			sLanguageName = sFile[i];
		}
	}
	
	if(m_sLanguageName.isEmpty() || sLanguageName != m_sLanguageName) {
		SetTextInComboBox(m_pComboLanguage, sLanguageName);
		LoadLanguage();
	}
	else {
		SetTextInComboBox(m_pComboLanguage, m_sLanguageName);
	}

	m_pChkAutoLoad->setOn(m_nAutoLaunch);
	m_pChkMultiInst->setOn(m_nMultIns);
	m_pDriveSize[m_nDriveSetting & 1]->setOn(true);
	m_pChkAutoSave->setOn(m_nAutoSaveProject);
#ifdef WIN32
	if(m_pAssocController != NULL) {
		QCheckListItem* pItem = (QCheckListItem*)m_pAssocController->firstChild();
		while(pItem != NULL) {
			pItem->setOn(IsAssociateWith(pItem->text(0)));
			pItem = (QCheckListItem*)pItem->nextSibling();
		}
	}
#endif
	OnShowCalcName();
}

//-----------------------------------------------------------------------------------
void CGeneralTab::DoInitWords()
{
	setCaption(tr("Gnral"));
	m_pLblInfo->setText(CLanguage::GetWordTranslation(CLanguage::WO_AUTHOR) + "\n" + CLanguage::GetWordTranslation(CLanguage::WO_INFO));

	m_pLanguageGroup->setTitle(tr("Langue"));

	m_pPrefGroup->setTitle(tr("Prfrences"));
	m_pSizeController->setText(0, tr("Taille des fichiers '.lec'"));
	m_pChkAutoLoad->setText(0,    tr("Charger le dernier projet ouvert au dmarrage"));
	m_pChkMultiInst->setText(0,   tr("Permettre plusieurs instances"));
	m_pChkAutoSave->setText(0,    tr("Sauvegarde automatique des projets"));
	m_pDriveSize[0]->setText(0,   tr("Fichier de taille variable"));
	m_pDriveSize[1]->setText(0,   tr("Fichier compatible avec Flash Editor"));

#ifdef WIN32
	if(m_pAssocController != NULL) {
		m_pAssocController->setText(0, tr("Associer Flash100 avec"));
	}

#endif
	m_pChangeCalc->setText(tr("Changer de calculatrice"));
	m_pChangeCalcDir->setText(tr("Changer de rpertoire"));

	m_pCalcType->clear();
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_G100));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_G100_PLUS));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_GRAPH25));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_GRAPH30));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_GRAPH60));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_GRAPH65));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_GRAPH80));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_CLASSPAD));
	m_pCalcType->insertItem(CCalcInfo::GetCalcName(CM_OTHER));

	OnShowCalcName();
}

//-----------------------------------------------------------------------------------
QString CGeneralTab::GetDescription() const
{
	return tr("Permet de rgler les paramtres de Flash100");
}

//-----------------------------------------------------------------------------------
void CGeneralTab::OnShowCalcName()
{
	QString s = m_calcInfo.GetCalcFile();
	if(s.isEmpty()) {
		s = tr("Non dfini");
		m_pCalcType->setEnabled(FALSE);
	}
	else {
		m_pCalcType->setEnabled(TRUE);
	}
	m_pCalcGroup->setTitle(tr("Calculatrice : %1").arg(s));
	QString sCalcName = m_calcInfo.GetCalcName();
	SetTextInComboBox(m_pCalcType, sCalcName);
}

//-----------------------------------------------------------------------------------
bool CGeneralTab::SetTextInComboBox(QComboBox* pCombo, QString& sText)
{
	if(pCombo != NULL) {
		int nMax = pCombo->count();
		int i;
		for(i = 0; i < nMax && sText != pCombo->text(i); i++);
		if(i < nMax) {
			pCombo->setCurrentText(sText);
			return TRUE;
		}
		sText = pCombo->currentText();
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
bool CGeneralTab::SetIndexInComboBox(QComboBox* pCombo, int& nIndex)
{
	if(pCombo != NULL) {
		int nMax = pCombo->count();
		if(nIndex < nMax) {
			pCombo->setCurrentItem(nIndex);
			return TRUE;
		}
		nIndex = pCombo->currentItem();
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
void CGeneralTab::OnChangeCalc()
{
	bool bOk;
	QDir d(m_calcInfo.GetCalcDir());
	d.setFilter(QDir::Files);
	d.setSorting(QDir::Name);
	QStringList lCalcs;
	if(d.exists()) {
		lCalcs = d.entryList();
	}

	int nCurrent = lCalcs.findIndex(m_calcInfo.GetCalcFile());
	QString sCalc = QInputDialog::getItem(tr("Changer de calculatrice"), tr("Entrez l'identifiant de cette calculatrice (Ex : MaGraph100)"), lCalcs, nCurrent, TRUE, &bOk, this);

	if(!bOk){
		return;
	}
	if(!m_calcInfo.SetCalcFile(sCalc)) {
		CMessageBox::MsgCritical(this, tr("Une erreur s'est produite lors du changement du fichier."));
	}
}

//-----------------------------------------------------------------------------------
void CGeneralTab::OnChangeCalcDir()
{
	QString sName = CMessageBox::ChooseExistingDirectory(m_calcInfo.GetCalcDir(), this);
	if(!sName.isEmpty()) {
		if(!m_calcInfo.SetCalcDir(sName)) {
			CMessageBox::MsgCritical(this, tr("Une erreur s'est produite lors du changement de rpertoire."));
		}
	}
}

//-----------------------------------------------------------------------------------
void CGeneralTab::OnCalcTypeChanged()
{
	CasioModel cm = CM_OTHER;
	switch(m_pCalcType->currentItem())
	{
	case 0:
		cm = CM_G100;
		break;
	case 1:
		cm = CM_G100_PLUS;
		break;
	case 2:
		cm = CM_GRAPH25;
		break;
	case 3:
		cm = CM_GRAPH30;
		break;
	case 4:
		cm = CM_GRAPH60;
		break;
	case 5:
		cm = CM_GRAPH65;
		break;
	case 6:
		cm = CM_GRAPH80;
		break;
	case 7:
		cm = CM_CLASSPAD;
		break;
	}
	if(!m_calcInfo.SetCalcModel(cm)) {
		QString sCalcName = m_calcInfo.GetCalcName();
		SetTextInComboBox(m_pCalcType, sCalcName);
	}
}

//-----------------------------------------------------------------------------------
bool CGeneralTab::OpenLanguageFile(const QString& sLanguageName)
{
	QValueList<QTranslator*>::iterator it = m_lTranslators.begin();
	for(;it != m_lTranslators.end(); it++) {
		QTranslator* pTranslator = *it;
		qApp->removeTranslator(pTranslator);
		delete pTranslator;
	}
	m_lTranslators.clear();

	QDir d(LANGUAGE_PATH + "/" + sLanguageName);
	if(d.exists()) {
		d.setNameFilter("*.qm");
		d.setSorting(QDir::Name);
		const QFileInfoList* pList = d.entryInfoList();
		QFileInfoListIterator it(*pList);
		QFileInfo* pFileInfo;
		for(; (pFileInfo = it.current()) != 0; ++it) {
			QTranslator* pTranslator = new QTranslator(this);
			if(pTranslator != NULL) {
				pTranslator->load(pFileInfo->absFilePath());
				qApp->installTranslator(pTranslator);
				m_lTranslators.append(pTranslator);
			}
		}
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
CCalcInfo* CGeneralTab::GetCalcInfo()
{
	return &m_calcInfo;
}

//-----------------------------------------------------------------------------------
int CGeneralTab::GetPreference(IOptionWindow::Preference nPref) const
{
	switch(nPref)
	{
	case IOptionWindow::PREF_AUTOLAUNCH:
		return m_nAutoLaunch;
	case IOptionWindow::PREF_DRIVE_VARIABLE_SIZE:
		return (1 - (m_nDriveSetting&1));
	case IOptionWindow::PREF_MULTI_INSTANCE:
		return m_nMultIns;
	case IOptionWindow::PREF_AUTO_SAVE_PROJECT:
		return m_nAutoSaveProject;
	}
	return -1; //Pas trouv.
}

//-----------------------------------------------------------------------------------
CTransfertWindow* CGeneralTab::CreateTransfertWindow(QWidget* pParent, int nTotalSize)
{
	CTransfertWindow* pTransfertWindow = new CTransfertWindow(GetCalcInfo(), pParent);
	if(pTransfertWindow != NULL) {
		pTransfertWindow = m_pCommunication->InitTransfertWindow(pTransfertWindow, nTotalSize);
	}
	return pTransfertWindow;
}
