// GeneralTab.h: interface for the CGeneralTab class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _GENERALTAB_H_
#define _GENERALTAB_H_

#include <qvaluelist.h>

#include <SettingTabWidget.h>
#include <CalcInfo.h>
#include <IOptionWindow.h>

class QLabel;
class QGroupBox;
class QListView;
class QCheckListItem;
class QPushButton;
class QComboBox;
class QTranslator;
class CTransfertWindow;
class CCommunicationTab;

class CGeneralTab : public CSettingTabWidget
{
	Q_OBJECT
public:
	CGeneralTab(IMainWindow* pMainWindow);
	int     DoSave();
	void    DoRefresh();
	void    DoInitWords();
	QString GetDescription() const;
	int     RestoreDefaults();

	CCalcInfo* GetCalcInfo();
	int        GetPreference(IOptionWindow::Preference nPref) const;
	void       LoadExtAssociations();

	CTransfertWindow* CreateTransfertWindow(QWidget* pParent, int nTotalSize);

private:
	void Init();
	void LoadLanguage();
	bool OpenLanguageFile(const QString& sLanguageName);

public:
	static bool SetTextInComboBox(QComboBox* pCombo, QString& sText);
	static bool SetIndexInComboBox(QComboBox* pCombo, int& nIndex);

private slots:
	void   OnChangeCalc();
	void   OnChangeCalcDir();
	void   OnShowCalcName();
	void   OnCalcTypeChanged();

private:
	CCommunicationTab* m_pCommunication;

	QLabel*		  m_pLblInfo;
	QGroupBox*	  m_pPrefGroup;
	QGroupBox*	  m_pLanguageGroup;
	QComboBox*	  m_pComboLanguage;
	QValueList<QTranslator*> m_lTranslators;

	QListView*      m_pPrefList;
	QCheckListItem* m_pChkAutoLoad;
	QCheckListItem* m_pChkMultiInst;
	QCheckListItem* m_pDriveSize[2];
	QCheckListItem* m_pSizeController;
	QCheckListItem* m_pChkAutoSave;
#ifdef WIN32
	QCheckListItem* m_pAssocController;	
#endif

	QString       m_sLanguageName;	
	
	CCalcInfo     m_calcInfo;
	QGroupBox*    m_pCalcGroup;
	QPushButton*  m_pChangeCalc;
	QPushButton*  m_pChangeCalcDir;
	QComboBox*    m_pCalcType;

	int           m_nAutoLaunch;
	int           m_nAutoSaveProject;
	int           m_nMultIns;
	int           m_nDriveSetting;
};

#endif
