
#include "OptionWindow.h"

#include <qpushbutton.h>
#include <qlayout.h>

#include <language.h>
#include <SettingsManager.h>
#include <SettingsWidget.h>
#include <IMainWindow.h>
#include <MessageBox.h>

#include "GeneralTab.h"
#include "ToolTab.h"
#include "registry.h"

//-----------------------------------------------------------------------------------
OptionWindow::OptionWindow(IMainWindow* pMainWindow, QWidget* parent)
    : QWidget(parent, 0, WStyle_StaysOnTop | WStyle_Customize | WStyle_Tool | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu)
	, m_pMainWindow(pMainWindow)
	, m_pGeneral(NULL)
	, m_pTools(NULL)
{
	QGridLayout* pGridLayout = new QGridLayout(this, 2, 3, 5);
	m_pManager               = new CSettingsManager(this);
    m_pCmdCancel			 = new QPushButton(this);
   	QPushButton* pCmdOk		 = new QPushButton(this);

	pGridLayout->addMultiCellWidget(m_pManager, 0, 0, 0, 2);
	pGridLayout->addWidget(pCmdOk,          1, 1);
	pGridLayout->addWidget(m_pCmdCancel,    1, 2);
	
	pGridLayout->setResizeMode(QLayout::Minimum);
	pGridLayout->setColStretch(0, 1);

    pCmdOk->setText("OK");
	pCmdOk->setDefault(true);
	
	connect(m_pCmdCancel, SIGNAL(clicked()), SLOT(close()));
	connect(pCmdOk,       SIGNAL(clicked()), SLOT(OnSaveConfig()));
	connect(m_pManager,   SIGNAL(NeedRestartApplication()), SLOT(OnRestartApplication()));
}

//-----------------------------------------------------------------------------------
void OptionWindow::languageChange()
{
	m_pCmdCancel->setText(CLanguage::GetWordTranslation(CLanguage::WO_CANCEL));
	setCaption(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));
}

//-----------------------------------------------------------------------------------
int	OptionWindow::DoLoad()
{
	m_pGeneral = new CGeneralTab(m_pMainWindow);
	m_pManager->AddItem(m_pGeneral);
	languageChange();

	int nX, nY, nW, nH;
	resize(minimumSizeHint());
	geometry().rect(&nX, &nY, &nW, &nH);
	m_pMainWindow->ReadSettings(F100_REG_KEY_OPTIONWINDOW_X, nX, nX);
	m_pMainWindow->ReadSettings(F100_REG_KEY_OPTIONWINDOW_Y, nY, nY);
	m_pMainWindow->ReadSettings(F100_REG_KEY_OPTIONWINDOW_WIDTH,  nW, nW);
	m_pMainWindow->ReadSettings(F100_REG_KEY_OPTIONWINDOW_HEIGHT, nH, nH);
	setGeometry(nX, nY, nW, nH);

	return 1;
}

//-----------------------------------------------------------------------------------
int OptionWindow::DoSave()
{
	QRect rect = geometry();
	m_pMainWindow->WriteSettings(F100_REG_KEY_OPTIONWINDOW_X, rect.x());
	m_pMainWindow->WriteSettings(F100_REG_KEY_OPTIONWINDOW_Y, rect.y());
	m_pMainWindow->WriteSettings(F100_REG_KEY_OPTIONWINDOW_WIDTH,  rect.width());
	m_pMainWindow->WriteSettings(F100_REG_KEY_OPTIONWINDOW_HEIGHT, rect.height());
	return 1;
}

//-----------------------------------------------------------------------------------
void OptionWindow::OnSaveConfig()
{
	close();
	m_pManager->SaveAll();
}

//-----------------------------------------------------------------------------------
void OptionWindow::show()
{
	if(!isShown()) {
		m_pManager->RefreshAll();
	}
	QWidget::show();
}

//-----------------------------------------------------------------------------------
int OptionWindow::GetPreference(Preference nPref) const
{
	if(m_pGeneral != NULL) {
		return m_pGeneral->GetPreference(nPref);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void OptionWindow::AddSettingsWidget(CSettingsWidget* pWidget)
{
	if(pWidget != NULL) {
		if(m_pTools == NULL) {
			m_pTools = new CToolTab(m_pMainWindow);
			m_pManager->AddItem(m_pTools);
		}
		m_pTools->AddChild(pWidget);
	}
}

//-----------------------------------------------------------------------------------
void OptionWindow::ShowSettingsWidget(CSettingsWidget* pWidget)
{
	if(pWidget == NULL) {
		m_pManager->Show(m_pGeneral);
	}
	else {
		m_pManager->Show(pWidget);
	}
	setActiveWindow();
	show();
}

//-----------------------------------------------------------------------------------
CSettingsWidget* OptionWindow::GetSettingsWidget(const QString& sToolName) const
{
	if(m_pTools != NULL) {
		return m_pTools->GetSettingsWidget(sToolName);
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CCalcInfo* OptionWindow::GetCalcInfo()
{
	if(m_pGeneral != NULL) {
		return m_pGeneral->GetCalcInfo();
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CTransfertWindow* OptionWindow::CreateTransfertWindow(QWidget* pParent, int nTotalSize)
{
	if(m_pGeneral != NULL) {
		return m_pGeneral->CreateTransfertWindow(pParent, nTotalSize);
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
void OptionWindow::OnRestartApplication()
{
	CMessageBox::MsgInformation(this, tr("Pour que les changements soient pris en compte Flash100 doit redmarrer."));
	m_pMainWindow->ReStartApplication();
}

//-----------------------------------------------------------------------------------
void OptionWindow::OnToolsLoaded()
{
	if(m_pGeneral != NULL) {
		m_pGeneral->LoadExtAssociations();
	}
}
