// StartUp.cpp: implementation of the StartUp class.
//
//////////////////////////////////////////////////////////////////////

#include "StartUp.h"

#include <qsettings.h>
#include <qapplication.h>
#include <qsocketdevice.h>

#include <assert.h>

#include "IPCServer.h"
#include "mainwindow.h"
#include "registry.h"

//-----------------------------------------------------------------------------------
StartUp::StartUp(QSettings* pSettings)
	: m_pSettings(pSettings)
	, m_pMainWindow(NULL)
	, m_pServer(NULL)
	, m_bFirstCall(TRUE)
	, m_bQuitEnable(TRUE)
{	
}

//-----------------------------------------------------------------------------------
StartUp::~StartUp()
{
	if(m_pMainWindow) {
		delete m_pMainWindow;
	}
}

//-----------------------------------------------------------------------------------
void StartUp::Quit()
{
	if(m_bQuitEnable) {
		qApp->quit();
	}
}

//-----------------------------------------------------------------------------------
bool StartUp::Start()
{
	assert(m_pSettings != NULL);
	if(m_pSettings->readNumEntry(F100_REG_KEY_MULTIPLE_INSTANCE)) {
		return CreateMainWindow();
	}
	//Lance le serveur
	m_pServer = new IPCServer(this);
	if(m_pServer->ok()) {
		return CreateMainWindow();
	}
	//Plus besoin du serveur
	delete m_pServer;
	m_pServer = NULL;

	QHostAddress adr((127 << 24) + 0 + 0 + 1);
	QSocketDevice socket;
	if(m_bFirstCall && qApp->argc() > 1) {
		if(socket.connect(adr, IPCPORT)) {
			//Envoi les donnes au serveur
			unsigned int nSize;
			for(int i = 1; i < qApp->argc(); i++) {
				nSize = strlen(qApp->argv()[i]);
				socket.writeBlock((char*)&nSize, sizeof(nSize));
				socket.writeBlock(qApp->argv()[i], nSize);
			}
			m_bFirstCall = FALSE;
			return false;
		}
		return CreateMainWindow();
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool StartUp::ReStart()
{
	m_bQuitEnable = FALSE;
	bool bRet     = FALSE;
	if(m_pMainWindow != NULL) {
		if(m_pMainWindow->close()) {
			if(m_pServer != NULL) {
				delete m_pServer;
				m_pServer = NULL;
			}
			delete m_pMainWindow;
			m_pMainWindow = NULL;
			if(!Start()) {
				qApp->quit();
				return TRUE;
			}
			bRet = TRUE;
		}
	}
	m_bQuitEnable = TRUE;
	return bRet;
}

//-----------------------------------------------------------------------------------
void StartUp::ConvertArgToQSL(QStringList& lFiles)
{
	lFiles.clear();
	if(m_bFirstCall && qApp->argc() > 1) {
		for(int i = 1; i < qApp->argc(); i++) {
			lFiles << qApp->argv()[i];
		}
	}
	m_bFirstCall = FALSE;
}

//-----------------------------------------------------------------------------------
bool StartUp::CreateMainWindow()
{
	if(m_pMainWindow == NULL) {
		m_pMainWindow = new MainWindow(this, m_pSettings);
		if(m_pMainWindow != NULL) {
			if(m_pServer != NULL) {
				m_pServer->SetMainWindow(m_pMainWindow);
			}
			QStringList lFiles;
			ConvertArgToQSL(lFiles);
			return m_pMainWindow->Init(lFiles);
		}
		return false;
	}
	return true;
}
