// ToolBar.cpp: implementation of the CToolBar class.
//
//////////////////////////////////////////////////////////////////////

#include "ToolBar.h"

#include <qaction.h>

#include <F100_Debug.h>

#include "ToolBarSettings.h"

//-----------------------------------------------------------------------------------
CToolBar::CToolBar(QMainWindow* parent)
: QToolBar(parent)
{
	m_pToolsMap = NULL;
}

//-----------------------------------------------------------------------------------
void CToolBar::SetToolList(ToolsMap* pToolsMap)
{
	m_pToolsMap = pToolsMap;
	Reset();
}

//-----------------------------------------------------------------------------------
void CToolBar::Clear()
{
	if(m_pToolsMap != NULL) {
		ToolList::Iterator it = m_lToolsShown.begin();
		for(; it != m_lToolsShown.end(); ++it) {
			(*m_pToolsMap)[*it].pAction->removeFrom(this);
		}
	}
	m_lToolsShown.clear();
	m_lToolsHidden.clear();
}

//-----------------------------------------------------------------------------------
void CToolBar::ShowTool(const QString& sToolName)
{
	if(m_pToolsMap != NULL) {
		ToolsMap::Iterator it = m_pToolsMap->find(sToolName);
		if(it != m_pToolsMap->end() && !m_lToolsShown.contains(sToolName)) {
			m_lToolsShown.push_back(sToolName);
			m_lToolsHidden.remove(sToolName);
			it.data().pAction->addTo(this);
		}
	}
}

//-----------------------------------------------------------------------------------
void CToolBar::SetLastTool(const QString& sLastToolShown, const QString& sLastToolHidden)
{
	m_sLastToolShown  = sLastToolShown;
	m_sLastToolHidden = sLastToolHidden;
	Reset();
}

//-----------------------------------------------------------------------------------
QString CToolBar::ConvertToString(const ToolList& lTools)
{
	QString s;
	ToolList::const_iterator it = lTools.constBegin();
	for(; it != lTools.end(); ++it) {
		if(s.isEmpty()) {
			s += *it;
		}
		else {
			s += ";" + *it;
		}
	}
	return s;
}

//-----------------------------------------------------------------------------------
QString CToolBar::GetToolShown() const
{
	return ConvertToString(m_lToolsShown);
}

//-----------------------------------------------------------------------------------
QString CToolBar::GetToolHidden() const
{
	return ConvertToString(m_lToolsHidden);
}

//-----------------------------------------------------------------------------------
int CToolBar::GetNbSection(const QString& sText)
{
	int i = 0;
	for(; GetSection(sText, i) != QString::null; ++i);
	return i;
}

//-----------------------------------------------------------------------------------
QString CToolBar::GetSection(const QString& sText, int nNum)
{
	return sText.section(";", nNum, nNum);
}

//-----------------------------------------------------------------------------------
bool CToolBar::IncludeIn(const QString& sText, const QString& sToolName)
{
	int nMax = GetNbSection(sText);
	for(int i = 0; i < nMax; i++) {
		if(GetSection(sText, i) == sToolName) {
			return true;
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
void CToolBar::Reset()
{
	Clear();
	int nMaxShow = GetNbSection(m_sLastToolShown);
	for(int i = 0; i < nMaxShow; i++) {
		QString sToolName = GetSection(m_sLastToolShown, i);
		if(!IncludeIn(m_sLastToolHidden, sToolName)) {
			ShowTool(sToolName);
		}
	}
	if(m_pToolsMap != NULL) {
		ToolsMap::Iterator it = m_pToolsMap->begin();
		for(; it != m_pToolsMap->end(); ++it) {
			bool bAddTo = FALSE;
			if(!it.data().pAction->iconSet().isNull()) {
				QString sToolName = it.key();
				it.data().op(OP_ADDTOTOOLBAR, (void**)&bAddTo, NULL, NULL, NULL, NULL);
				if(bAddTo && !IncludeIn(m_sLastToolHidden, sToolName)) {
					ShowTool(sToolName);
				}
				else {
					if(!m_lToolsShown.contains(sToolName)) {
						m_lToolsHidden.push_back(sToolName);
					}
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CToolBar::ShowCustomize()
{
	CToolBarSettings settings(parentWidget(), this);
	settings.exec();
}

//-----------------------------------------------------------------------------------
QAction* CToolBar::GetToolAction(const QString& sToolName)
{
	if(m_pToolsMap != NULL) {
		ToolsMap::Iterator it = m_pToolsMap->find(sToolName);
		if(it != m_pToolsMap->end()) {
			return it.data().pAction;
		}
	}
	return NULL;
}
