// ToolBarSettings.cpp: implementation of the CToolBarSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "ToolBarSettings.h"
#include "ToolBar.h"

#include <qpushbutton.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qheader.h>
#include <qaction.h>

//-----------------------------------------------------------------------------------
CToolBarSettings::CToolBarSettings(QWidget* parent, CToolBar* pToolBar)
: QDialog(parent, 0, TRUE)
{
	setCaption(pToolBar->caption());
	m_pToolBar = pToolBar;
	QVGroupBox* pShowTool = new QVGroupBox(tr("Outils affichs"), this);
	QVGroupBox* pHideTool = new QVGroupBox(tr("Outils cachs"), this);
	QGridLayout* pLayout  = new QGridLayout(this, 5, 3, 5);

	m_pShowList  = new QListView(pShowTool);
	m_pHideList  = new QListView(pHideTool);
	m_pUp        = new QPushButton(tr("Monter"), this);
	m_pDown      = new QPushButton(tr("Descendre"), this);
	m_pTransfert = new QPushButton("<-->", this);

	pLayout->addMultiCellWidget(pShowTool, 0, 4, 0, 0);
	pLayout->addMultiCellWidget(pHideTool, 0, 4, 2, 2);
	pLayout->addWidget(m_pUp,        1, 1);
	pLayout->addWidget(m_pDown,      2, 1);
	pLayout->addWidget(m_pTransfert, 3, 1);

	m_pShowList->header()->hide();
	m_pShowList->addColumn(QString::null);
	m_pShowList->setSorting(-1);
	m_pHideList->header()->hide();
	m_pHideList->addColumn(QString::null);
	m_pHideList->setSorting(-1);

	m_pUp->setEnabled(FALSE);
	m_pDown->setEnabled(FALSE);
	m_pTransfert->setEnabled(FALSE);

	ShowTools(m_pShowList, m_pToolBar->GetToolShown());
	ShowTools(m_pHideList, m_pToolBar->GetToolHidden());

	connect(m_pShowList,  SIGNAL(clicked(QListViewItem*)), SLOT(OnShowChanged(QListViewItem*)));
	connect(m_pHideList,  SIGNAL(clicked(QListViewItem*)), SLOT(OnHideChanged(QListViewItem*)));
	connect(m_pUp,        SIGNAL(clicked()),               SLOT(OnUp()));
	connect(m_pDown,      SIGNAL(clicked()),               SLOT(OnDown()));
	connect(m_pTransfert, SIGNAL(clicked()),               SLOT(OnTransfert()));
	m_bShow = TRUE;
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::ShowTools(QListView* pList, const QString& sText)
{
	int nMax = CToolBar::GetNbSection(sText);
	QListViewItem* pItem = NULL;
	for(int i = 0; i < nMax; i++) {
		QString sToolName = CToolBar::GetSection(sText, i);
		QAction* pAction  = m_pToolBar->GetToolAction(sToolName);
		pItem = new QListViewItem(pList, pItem, pAction->menuText(), sToolName);
		pItem->setPixmap(0, pAction->iconSet().pixmap());
	}
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::OnShowChanged(QListViewItem* pItem)
{
	m_bShow = pItem != NULL;
	m_pUp->setEnabled(m_bShow);
	m_pDown->setEnabled(m_bShow);
	m_pTransfert->setEnabled(m_bShow);
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::OnHideChanged(QListViewItem* pItem)
{
	m_bShow = FALSE;
	m_pUp->setEnabled(FALSE);
	m_pDown->setEnabled(FALSE);
	m_pTransfert->setEnabled(pItem != NULL);
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::OnTransfert()
{
	QListViewItem* pItem;
	if(m_bShow) {
		pItem = m_pShowList->currentItem();
		m_pShowList->takeItem(pItem);
		m_pHideList->insertItem(pItem);
	}
	else {
		pItem = m_pHideList->currentItem();
		if(pItem != NULL) {
			m_pHideList->takeItem(pItem);
			m_pShowList->insertItem(pItem);
			pItem->moveItem(m_pShowList->lastItem());
		}
	}
	FlushData();
}

//-----------------------------------------------------------------------------------
QString CToolBarSettings::ConvertToString(QListView* pList)
{
	QString s;
	QListViewItem* pItem = pList->firstChild();
	if(pItem != NULL) {
		s = pItem->text(1);
		pItem = pItem->nextSibling();
	}
	while(pItem != NULL) {
		s += ";" + pItem->text(1);
		pItem = pItem->nextSibling();
	}
	return s;
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::OnUp()
{
	QListViewItem* pItem = m_pShowList->selectedItem();
	if(pItem != NULL && pItem->itemAbove() != NULL) {
		QListViewItem* pAfter = pItem->itemAbove()->itemAbove();
		if(pAfter != NULL) {
			pItem->moveItem(pAfter);
		}
		else {
			m_pShowList->takeItem(pItem);
			m_pShowList->insertItem(pItem);
		
		}
		m_pShowList->setSelected(pItem, TRUE);
		FlushData();
	}
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::OnDown()
{
	QListViewItem* pItem = m_pShowList->selectedItem();
	if(pItem && pItem->itemBelow()) {
		pItem->moveItem(pItem->itemBelow());
		m_pShowList->setSelected(pItem, TRUE);
		FlushData();
	}
}

//-----------------------------------------------------------------------------------
void CToolBarSettings::FlushData()
{
	m_pToolBar->SetLastTool(ConvertToString(m_pShowList), ConvertToString(m_pHideList));
}
