// ToolTab.cpp: implementation of the CToolTab class.
//
//////////////////////////////////////////////////////////////////////

#include "ToolTab.h"

#include <qlayout.h>
#include <qlistview.h>

#include <SettingsWidget.h>
#include <language.h>
#include <IMainWindow.h>

#define ID 6

//-----------------------------------------------------------------------------------
CToolTab::CToolTab(IMainWindow* pMainWindow)
	: CSettingTabWidget(pMainWindow, "Tools")
{
	QBoxLayout* pLayout = new QBoxLayout(this, QBoxLayout::TopToBottom);
	m_pList = new QListView(this);

	pLayout->addWidget(m_pList);

	for(int i = 0; i < 7; i++) {
		m_pList->addColumn(QString::null);
	}

	m_pList->setAllColumnsShowFocus(TRUE);
	m_pList->setSorting(0, TRUE);
	m_pList->setShowSortIndicator(TRUE);
}

//-----------------------------------------------------------------------------------
int CToolTab::DoSave()
{
	return 1;
}

//-----------------------------------------------------------------------------------
void CToolTab::DoRefresh()
{
	//DoRefresh() est appele par la fenetre des options aprs que les outils aient t chargs
	//donc la liste des outils trouvs ne doit pas tre modifie
	if(m_pList->childCount() == 0) {
		QStringList lTools = GetMainWindow()->GetToolNames();
		QStringList::const_iterator it = lTools.begin();
		for(; it != lTools.end(); it++) {
			(new QListViewItem(m_pList))->setText(ID, *it);
		}
		DoInitWords();
	}
}

//-----------------------------------------------------------------------------------
void CToolTab::DoInitWords()
{
	setCaption(CSettingTabWidget::tr("Outils"));
	m_pList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pList->setColumnText(1, CSettingTabWidget::tr("Extension"));
	m_pList->setColumnText(2, CSettingTabWidget::tr("Version"));
	m_pList->setColumnText(3, CSettingTabWidget::tr("Auteur"));
	m_pList->setColumnText(4, CSettingTabWidget::tr("Date de cration"));
	m_pList->setColumnText(5, CSettingTabWidget::tr("Description"));
	m_pList->setColumnText(6, CSettingTabWidget::tr("Id"));

	QListViewItem* pItem = m_pList->firstChild();
	while(pItem != NULL) {
		QString sToolName, sDesc, sAuthor, sVersion, sBuildDate, sFileExt;
		if(GetMainWindow()->GetToolInfos(pItem->text(ID), sToolName, sDesc, sFileExt, sAuthor, sVersion, sBuildDate)) {
			pItem->setText(0, sToolName);
			pItem->setText(1, sFileExt);
			pItem->setText(2, sVersion);
			pItem->setText(3, sAuthor);
			pItem->setText(4, sBuildDate);
			pItem->setText(5, sDesc);
		}
		pItem = pItem->nextSibling();
	}
	m_pList->sort();
}

//-----------------------------------------------------------------------------------
QString CToolTab::GetDescription() const
{
	return CSettingTabWidget::tr("Permet d'afficher les informations sur les outils chargs et de proposer une fentre de rglages pour certains outils");
}

//-----------------------------------------------------------------------------------
CSettingsWidget* CToolTab::GetSettingsWidget(const QString& sToolName) const
{
	SettingTabList::const_iterator it  = GetChildren()->begin();
	SettingTabList::const_iterator end = GetChildren()->end();
	for(; it != end; it++) {
		CSettingsWidget* pWidget = (CSettingsWidget*)*it;
		if(pWidget->GetToolName() == sToolName) {
			return pWidget;
		}
	}
	return NULL;
}
