// ToolTabWidget.cpp: implementation of the ToolTabWidget class.
//
//////////////////////////////////////////////////////////////////////

#include "ToolTabWidget.h"

#include <qtoolbutton.h>
#include <qstyle.h>
#include <qtooltip.h>
#include <qlayout.h>

#include <F100_Debug.h>
#include <image_defs.h>

#include <qbitmap.h>
#include <qimage.h>

/*
void save(const QString& name)
{
	QPixmap pm = QPixmap::fromMimeSource(name + ".bmp");

	pm.setMask(pm.createHeuristicMask());

	QImage i = pm.convertToImage();

	i = i.convertDepth(8);
	i.save(QString("d:/programmation/c++/flash100/images/")+name + ".png", "PNG");
}
*/
//-----------------------------------------------------------------------------------
ToolTabWidget::ToolTabWidget(QWidget* parent)
: QTabWidget(parent),
m_bForceToHide(FALSE),
m_qCornerWidget(this),
m_qCloseButton(&m_qCornerWidget),
m_qShiftLeftButton(&m_qCornerWidget),
m_qShiftRightButton(&m_qCornerWidget)
{
//	save("rammanager/file_selected");

	QBoxLayout* pLayout = new QBoxLayout(&m_qCornerWidget, QBoxLayout::LeftToRight);

	pLayout->addWidget(&m_qShiftLeftButton);
	pLayout->addWidget(&m_qShiftRightButton);
	pLayout->addWidget(&m_qCloseButton);

	m_qCloseButton.setAutoRaise(TRUE);
	m_qShiftLeftButton.setAutoRaise(TRUE);
	m_qShiftRightButton.setAutoRaise(TRUE);

	m_qShiftLeftButton.setIconSet(QPixmap::fromMimeSource(IMG_LEFT));
	m_qShiftRightButton.setIconSet(QPixmap::fromMimeSource(IMG_RIGHT));

	setCornerWidget(&m_qCornerWidget, Qt::TopRight);

	hide();

	connect(&m_qCloseButton,      SIGNAL(clicked()), SLOT(OnCloseCurrentPage()));
	connect(&m_qShiftLeftButton,  SIGNAL(clicked()), SLOT(OnLeftShift()));
	connect(&m_qShiftRightButton, SIGNAL(clicked()), SLOT(OnRightShift()));
	connect(&m_qShiftLeftButton,  SIGNAL(clicked()), parent, SLOT(ProjectModified()));
	connect(&m_qShiftRightButton, SIGNAL(clicked()), parent, SLOT(ProjectModified()));
	connect(this, SIGNAL(currentChanged(QWidget*)), SLOT(RefreshButtons()));

	languageChange();
	RefreshButtons();
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::OnCloseCurrentPage()
{
	QWidget* pTab = currentPage();
	if(pTab != NULL) {
		pTab->close();
	}
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::OnLeftShift()
{
	QWidget* pTab = currentPage();
	if(pTab != NULL) {
		QIconSet icon  = tabIconSet(pTab);
		QString sLabel = tabLabel(pTab);
		int nIndex     = indexOf(pTab);
		_ASSERTE(nIndex != -1);
		removePage(pTab);
		insertTab(pTab, icon, sLabel, nIndex - 1);
		showPage(pTab);
	}
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::OnRightShift()
{
	QWidget* pTab = currentPage();
	if(pTab != NULL) {
		QIconSet icon  = tabIconSet(pTab);
		QString sLabel = tabLabel(pTab);
		int nIndex     = currentPageIndex();
		_ASSERTE(nIndex != -1);
		removePage(pTab);
		insertTab(pTab, icon, sLabel, nIndex + 1);
		showPage(pTab);
	}
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::languageChange()
{
	m_qCloseButton.setIconSet(style().stylePixmap(QStyle::SP_TitleBarCloseButton, this));
#if QT_VERSION < 0x030300
	m_qCloseButton.setMaximumSize(20, 20);
#endif
	QToolTip::add(&m_qCloseButton, tr("Ferme l'onglet slectionn"));
	QToolTip::add(&m_qShiftLeftButton, tr("Dcale l'onglet slectionn vers la gauche"));
	QToolTip::add(&m_qShiftRightButton, tr("Dcale l'onglet slectionn vers la droite"));
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::removePage(QWidget* w)
{
	QTabWidget::removePage(w);
	if(count() == 0) {
		hide();
	}
	RefreshButtons();
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::showPage(QWidget* w)
{
	show();
	QTabWidget::showPage(w);
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::show()
{
	if(!m_bForceToHide && count() > 0) {
		QTabWidget::show();
	}
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::SetForceToHide(bool bForceToHide)
{
	m_bForceToHide = bForceToHide;
	if(m_bForceToHide) {
		hide();
	}
	else {
		show();
	}
}

//-----------------------------------------------------------------------------------
void ToolTabWidget::RefreshButtons()
{
	QWidget* pTab = currentPage();
	m_qShiftLeftButton.setEnabled(pTab != NULL && indexOf(pTab) > 0);
	m_qShiftRightButton.setEnabled(pTab != NULL && indexOf(pTab) < count() - 1);
}
