
#include "assocfile.h"

#include <qapplication.h>

#ifdef WIN32
#include <windows.h>
static char sF100Path[] = "Flash100.File";
#endif

#define BACK "Flash100_Back"

#ifdef WIN32
//-----------------------------------------------------------------------------------
void OpenRegExt(HKEY& h, const QString& sExtension, LPBYTE pData, DWORD& nType, DWORD& nSize)
{
	QString sExt = '.' + sExtension;
	RegOpenKeyEx(HKEY_CLASSES_ROOT, sExt, 0, KEY_ALL_ACCESS, &h);
	if(h != NULL) {
		RegQueryValueEx(h, 0, 0, &nType, pData, &nSize);
	}
}

//-----------------------------------------------------------------------------------
void DeleteKeys(HKEY h, LPCTSTR lpSubKey)
{
	char name[2048];
	DWORD nSize = sizeof(name);
	if(h != NULL) {
		HKEY hKey;
		RegOpenKeyEx(h, lpSubKey, 0, KEY_ALL_ACCESS, &hKey);
		if(hKey != NULL) {
			for(DWORD nIndex = 0; RegEnumKeyEx(hKey, nIndex, name, &nSize, 0, 0, 0, 0) == ERROR_SUCCESS; nIndex++) {
				DeleteKeys(hKey, name);
			}
		}
		RegCloseKey(hKey);
		RegDeleteKey(h, lpSubKey);
	}
}

#endif

//-----------------------------------------------------------------------------------
bool IntallF100Reg(const QString& sF100Desc)
{
#ifdef WIN32
	HKEY hRoot;
	HKEY hCommand;
	DWORD dwDisposition = REG_CREATED_NEW_KEY;
	RegCreateKeyEx(HKEY_CLASSES_ROOT, sF100Path, 0, "REG_SZ", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hRoot, &dwDisposition);
	if(hRoot == NULL) {
		return false;
	}
	RegSetValueEx(hRoot, 0, 0, REG_SZ, (CONST BYTE*)sF100Desc.latin1(), sF100Desc.length() + 1);

	RegCreateKeyEx(hRoot, "shell\\open\\command", 0, "REG_SZ", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hCommand, &dwDisposition);
	if(hCommand != NULL) {
		QString sCommand = '\"' + qApp->applicationFilePath() + "\" \"%1\"";
		RegSetValueEx(hCommand, 0, 0, REG_SZ, (CONST BYTE*)(sCommand.latin1()), sCommand.length() + 1);
		RegCloseKey(hCommand);
	}
	RegCreateKeyEx(hRoot, "DefaultIcon", 0, "REG_SZ", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hCommand, &dwDisposition);
	if(hCommand != NULL) {
		QString sCommand = qApp->applicationFilePath();
		RegSetValueEx(hCommand, 0, 0, REG_SZ, (CONST BYTE*)(sCommand.latin1()), sCommand.length() + 1);
		RegCloseKey(hCommand);
	}
	RegCloseKey(hRoot);
	return true;
#else
	return false;
#endif
}

//-----------------------------------------------------------------------------------
void UninstallAssocFile(const QString& sExtensions)
{
#ifdef WIN32
	DeleteKeys(HKEY_CLASSES_ROOT, sF100Path);
	QString sExt;
	for(int nSection = 0; (sExt=sExtensions.section(';', nSection, nSection)) && !sExt.isEmpty(); nSection++) {
		RemoveAssocFile(sExt);
	}
#endif
}

//-----------------------------------------------------------------------------------
bool AddAssocFile(const QString& sExtension)
{
#ifdef WIN32
	HKEY hRoot;
	DWORD dwDisposition = REG_OPENED_EXISTING_KEY;
	QString sExt = '.' + sExtension;
	RegCreateKeyEx(HKEY_CLASSES_ROOT, sExt, 0, "REG_SZ", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hRoot, &dwDisposition);
	if(hRoot == NULL) {
		return false;
	}
	BYTE  pData[2048];
	DWORD nType, nSize = sizeof(pData);

	LONG nRes = RegQueryValueEx(hRoot, 0, 0, &nType, pData, &nSize);
	if(nType != REG_SZ || QString((const char*)pData) != sF100Path) {
		//Place Flash100 et sauve l'ancienne association
		if(nRes == ERROR_SUCCESS) {
			RegSetValueEx(hRoot, BACK, 0, nType, pData, nSize);
		}
		RegSetValueEx(hRoot, 0, 0, REG_SZ, (CONST BYTE*)sF100Path, sizeof(sF100Path));
	}

	RegCloseKey(hRoot);
	return true;
#else
	return false;
#endif
}

//-----------------------------------------------------------------------------------
void RemoveAssocFile(const QString& sExtension)
{
#ifdef WIN32
	HKEY h;
	BYTE  pData[2048];
	DWORD nType, nSize = sizeof(pData);
	OpenRegExt(h, sExtension, pData, nType, nSize);
	if(h == NULL) {
		return;
	}

	if(nType == REG_SZ && QString((const char*)pData) == sF100Path) {
		nSize = sizeof(pData);
		if(RegQueryValueEx(h, BACK, 0, &nType, pData, &nSize)  == ERROR_SUCCESS) {
			RegSetValueEx(h, 0, 0, nType, pData, nSize);
		}
		else {
			RegDeleteValue(h, 0);
		}
	}
	RegDeleteValue(h, BACK);
	RegCloseKey(h);
#endif
}

//-----------------------------------------------------------------------------------
bool IsAssociateWith(const QString& sExtension)
{
#ifdef WIN32
	HKEY h;
	BYTE  pData[2048];
	DWORD nType, nSize = sizeof(pData);
	OpenRegExt(h, sExtension, pData, nType, nSize);
	if(h == NULL) {
		return false;
	}

	RegCloseKey(h);

	return nType == REG_SZ && QString((const char*)pData) == sF100Path;
#else
	return false;
#endif
}
