
#include "mainwindow.h"

#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qlayout.h>
#include <qapplication.h>
#include <qlibrary.h>
#include <qstatusbar.h>
#include <qdockarea.h>
#include <qdragobject.h>
#include <qsettings.h>
#include <qdom.h>
#include <qcursor.h>
#include <qdir.h>
#include <qfiledialog.h>

#include <assert.h>

#include <language.h>
#include <MessageBox.h>
#include <ToolWindow.h>
#include <image_defs.h>
#include <APIFunction.h>
#include <SendToMenu.h>

#include "frmabout.h"
#include "ConsoleWindow.h"
#include "registry.h"
#include "assocfile.h"
#include "StartUp.h"
#include "ToolTabWidget.h"
#include "ToolBar.h"
#include "EditLinkTab.h"

#ifndef PLUGIN_DIR
#	define PLUGIN_DIR (APP_DIR_PATH + "/Tools")
#endif

#define DRIVE_EXT  "lec"
#define SYSTEM_EXT "mems"

#define FLAGS  WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu | WStyle_MinMax

static char* sDockName[] = {"qt_left_dock", "qt_right_dock", "qt_bottom_dock","qt_top_dock"};
static Qt::ToolBarDock pToolBarDock[4] = {Qt::DockLeft, Qt::DockRight, Qt::DockBottom, Qt::DockTop};

#define NO_VALID_FILE(s) (MainWindow::tr("Le fichier '%1' est soit un fichier non valide, soit les outils chargs ne connaissent pas ce type de fichier!").arg(s))

#define PROJECT_FILTER MainWindow::tr("Projets (*.prj)")

//-----------------------------------------------------------------------------------
MainWindow::MainWindow(StartUp* pStartUp, QSettings* pSettings, QWidget* parent)
	: QMainWindow(parent, 0)
	, m_wOptions(this)
	, m_pLinksMenu(NULL)
{	
	assert(pStartUp != NULL && pSettings != NULL);

	m_pSettings = pSettings;
	m_pStartUp  = pStartUp;

	setIcon(QPixmap::fromMimeSource(IMG_MAINICON));

	//Gestion du Drag&Drop
	setAcceptDrops(TRUE);

	//Cr une barre d'tat
	statusBar();

	m_pOutput     = ConsoleWindow::GetMainInstance(this);
	m_pToolBar    = new CToolBar(this);
	m_pTab        = new ToolTabWidget(this);
	m_pSendToMenu = new CSendToMenu(this);

	m_pToolBar->setMovingEnabled(TRUE);
	m_pOutput->setMovingEnabled(TRUE);

	setCentralWidget(m_pTab);	
	if(layout()) {
		layout()->setResizeMode(QLayout::FreeResize);
	}

	int i;

	//Actions
	for(i = 0; i < 5; i++) {
		fileAction[i] = new QAction(this);
	}

	fileAction[0]->setIconSet(QPixmap::fromMimeSource(IMG_NEW));
	fileAction[0]->setAccel(4194382);
	
	fileAction[1]->setIconSet(QPixmap::fromMimeSource(IMG_OPEN));
	fileAction[1]->setAccel(4194383);
	
	fileAction[2]->setIconSet(QPixmap::fromMimeSource(IMG_SAVE));
	fileAction[2]->setAccel(4194387);
		
	fileAction[4]->setAccel(8392755);
		
	helpAction[0] = new QAction(this);
	helpAction[0]->setAccel(Qt::Key_F1);
	
	helpAction[1] = new QAction(this);
	helpAction[1]->setIconSet(QPixmap::fromMimeSource(IMG_HELP));
			
	fileAction[0]->addTo(m_pToolBar);
	fileAction[1]->addTo(m_pToolBar);
	fileAction[2]->addTo(m_pToolBar);
	m_pToolBar->addSeparator();
	
	m_pFileMenu   = new QPopupMenu(this);
	m_pToolMenu   = new QPopupMenu(this);
	m_pWindowMenu = new QPopupMenu(this);
	m_pHelpMenu   = new QPopupMenu(this);
	
	m_pListWindowShowMenu  = new QPopupMenu(m_pWindowMenu);
	m_pListWindowCloseMenu = new QPopupMenu(m_pWindowMenu);

	//Connexion pour la gestion des fentres
	connect(m_pListWindowCloseMenu, SIGNAL(activated(int)), SLOT(CloseWindow(int)));
	connect(m_pListWindowShowMenu,  SIGNAL(activated(int)), SLOT(ShowWindow(int)));

	//signals and slots connections
	connect(fileAction[0], SIGNAL(activated()), SLOT(fileNew()));
	connect(fileAction[1], SIGNAL(activated()), SLOT(fileOpen()));
	connect(fileAction[2], SIGNAL(activated()), SLOT(fileSave()));
	connect(fileAction[3], SIGNAL(activated()), SLOT(fileSaveAs()));
	connect(fileAction[4], SIGNAL(activated()), SLOT(close()));
	connect(helpAction[0], SIGNAL(activated()), SLOT(OnShowHelp()));
	connect(helpAction[1], SIGNAL(activated()), SLOT(OnShowAbout()));

	connect(m_pHelpMenu, SIGNAL(aboutToShow()), SLOT(OnAboutToShowHelpMenu()));
	
	connect(m_pToolMenu ,SIGNAL(activated(int)), SLOT(OnToolMenuClick(int)));
	
	m_pOpenFilesAction      = new QAction(this);
	m_pOpenFileWithTool     = new QAction(this);
	m_pOpenToolMenu         = new QPopupMenu(this);

	m_pOpenFilesAction->setAccel(Key_F3);

	connect(m_pOpenFilesAction,  SIGNAL(activated()), SLOT(OnOpenFiles()));
	connect(m_pOpenFileWithTool, SIGNAL(activated()), SLOT(OnOpenFile()));
	connect(m_pWindowMenu,       SIGNAL(aboutToShow()), SLOT(OnRefreshWindowList()));
}

//-----------------------------------------------------------------------------------
void MainWindow::LoadDockWindow(QDockWindow* pDockWindow, const QString& sArea, ToolBarDock defaultDock)
{
	bool bToolBarIsDocked = TRUE;
	int i;
	for(i = 0; i < 4 && sArea != sDockName[i]; i++);
	if(i < 4) {
		defaultDock = pToolBarDock[i];
	}
	else {
		bToolBarIsDocked = FALSE;
	}
	moveDockWindow(pDockWindow, defaultDock);
	if(!bToolBarIsDocked) {
		pDockWindow->undock();
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::Init(QStringList& lFiles)
{
	int x, y, width, height, nStyle;
	QString sLastOpenedProject;
	QString sOpacity;
	ReadSettings(F100_REG_KEY_MAINWINDOW_X, x, 50);
	ReadSettings(F100_REG_KEY_MAINWINDOW_Y, y, 50);
	ReadSettings(F100_REG_KEY_MAINWINDOW_HEIGHT, height, 332);
	ReadSettings(F100_REG_KEY_MAINWINDOW_WIDTH, width, 633);
	ReadSettings(F100_REG_KEY_MAINWINDOW_STYLE, nStyle, 0);
	ReadSettings(F100_REG_KEY_MAINWINDOW_LASTOPENEDPROJECT,  sLastOpenedProject);

	QRect geo = QRect(x, y, width, height);

	QString sArea, sToolHidden, sToolShow;

	int nToolBarX, nToolBarY;
	ReadSettings(F100_REG_KEY_TOOLBAR_AREA, sArea, sDockName[3]);
	ReadSettings(F100_REG_KEY_TOOLBAR_X, nToolBarX);
	ReadSettings(F100_REG_KEY_TOOLBAR_Y, nToolBarY);
	ReadSettings(F100_REG_KEY_TOOLBAR_TOOLSSHOW,   sToolShow);
	ReadSettings(F100_REG_KEY_TOOLBAR_TOOLSHIDDEN, sToolHidden);

	m_pToolBar->SetLastTool(sToolShow, sToolHidden);

	LoadDockWindow(m_pToolBar, sArea, DockTop);

	int nOutputX, nOutputY,
		nOutputW, nOutputH,
		nIsShown;
	ReadSettings(F100_REG_KEY_OUTPUT_AREA,    sArea, sDockName[2]);
	ReadSettings(F100_REG_KEY_OUTPUT_X,       nOutputX);
	ReadSettings(F100_REG_KEY_OUTPUT_Y,       nOutputY);
	ReadSettings(F100_REG_KEY_OUTPUT_WIDTH,   nOutputW);
	ReadSettings(F100_REG_KEY_OUTPUT_HEIGHT,  nOutputH);
	ReadSettings(F100_REG_KEY_OUTPUT_ISSHOWN, nIsShown, TRUE);
	ReadSettings(F100_REG_KEY_OUTPUT_OPACITY, sOpacity, "1");

	m_pOutput->SetOpacity(sOpacity.toDouble());
	LoadDockWindow(m_pOutput, sArea, DockBottom);
	
	//Initialise la fentre des options
	m_wOptions.DoLoad();

	languageChange();

	//Charge les outils dynamiquement
	LoadTools();

	m_wOptions.OnToolsLoaded();

	if(CheckArgs(lFiles)) {
		return false;//Stoppe
	}

	setGeometry(geo);
#if QT_VERSION < 0x030300
	switch(nStyle)
	{
	case 1:
		showMinimized();
		break;
	case 2:
		showMaximized();
		break;
	default:
		show();
	}
#else
	setWindowState(nStyle);
	show();
#endif

	m_pToolBar->move(nToolBarX, nToolBarY);
	m_pToolBar->show();

	m_pOutput->move(nOutputX, nOutputY);
	m_pOutput->resize(nOutputW, nOutputH);
	if(nIsShown){
		m_pOutput->show();
	}
	else{
		m_pOutput->hide();
	}

	CloseProject();

	//Charge d'abord le premier projet pass en paramtre
	//les autres seront supprims
	if(LoadFiles(lFiles, TRUE) == 0 && m_wOptions.GetPreference(OptionWindow::PREF_AUTOLAUNCH)) {
		//Si aucun projet pass en paramtre alors charger le dernier
		//projet ouvert (si option coche)
		LoadProject(CToolWindow::ConvertToFullPath(sLastOpenedProject));
	}
	m_pTab->show();

	//Charge des fichiers (diffrent d'un projet de Flash100)
	LoadFiles(lFiles);
	return true;
}

//-----------------------------------------------------------------------------------
void MainWindow::languageChange()
{
	//Barre de menu
	menuBar()->clear();
	menuBar()->insertItem(tr("&Fichier"), m_pFileMenu);
	menuBar()->insertItem(tr("&Outils"), m_pToolMenu);
	menuBar()->insertItem(tr("F&entre"), m_pWindowMenu, 0);
	menuBar()->insertItem("&?", m_pHelpMenu);
	
	if(m_pFileMenu->count() == 0) {
		fileAction[0]->addTo(m_pFileMenu);
		fileAction[1]->addTo(m_pFileMenu);
		m_pFileMenu->insertSeparator();
		fileAction[2]->addTo(m_pFileMenu);
		fileAction[3]->addTo(m_pFileMenu);
		m_pFileMenu->insertSeparator();
		fileAction[4]->addTo(m_pFileMenu);
	}
	
	if(m_pToolMenu->count() == 0) {
		m_pOpenFilesAction->addTo(m_pToolMenu);
		m_pOpenFileWithTool->addTo(m_pToolMenu);
		m_pToolMenu->insertSeparator();
		m_pToolMenu->insertItem(QString::null, m_pOpenToolMenu, 2);
		m_pToolMenu->insertSeparator();
		m_pToolMenu->insertItem(QString::null, 1);
		m_pToolMenu->setAccel(Key_F10, 1);
	}
		
	if(m_pWindowMenu->count() == 0) {
		m_pWindowMenu->insertItem(QString::null, this, SLOT(CloseAllWindows()), 0, 0);
		m_pWindowMenu->insertItem(QString::null, m_pListWindowCloseMenu, 1);
		m_pWindowMenu->insertSeparator();
		m_pWindowMenu->insertItem(QString::null, m_pListWindowShowMenu, 2);
		m_pWindowMenu->insertSeparator();
		m_pWindowMenu->insertItem(QString::null, m_pOutput, SLOT(show()), 0, 3);
	}

	//ToolBar
	m_pToolBar->setCaption(tr("Barre d'outils"));

	//Console
	m_pOutput->setCaption(tr("Sortie"));

	//Menu Fichier
	fileAction[0]->setMenuText(tr("&Nouveau projet"));
	fileAction[0]->setToolTip(tr("Charger un projet vierge"));

	fileAction[1]->setMenuText(tr("&Ouvrir un projet..."));
	fileAction[1]->setToolTip(tr("Ouvrir un projet existant"));

	fileAction[2]->setMenuText(tr("&Sauver le projet"));
	fileAction[2]->setToolTip(tr("Sauver le projet courant"));

	fileAction[3]->setMenuText(tr("S&auver le projet sous..."));
	fileAction[3]->setToolTip(tr("Sauver le projet dans un fichier choisi"));

	fileAction[4]->setMenuText(tr("&Quitter"));
	fileAction[4]->setToolTip(tr("Quitter Flash100"));
		
	//Menu Fenetre
	m_pWindowMenu->changeItem(0, tr("Fermer &Tout"));
	m_pWindowMenu->changeItem(1, tr("&Fermer"));
	m_pWindowMenu->changeItem(2, tr("&Afficher"));
	m_pWindowMenu->changeItem(3, tr("Sortie"));
	
	//Menu Outils
	m_pOpenFilesAction->setMenuText(tr("Ouvrir des &fichiers..."));
	m_pOpenFilesAction->setToolTip(tr("Permet d'ouvrir diffrents fichiers en choisissant les outils appropris"));

	m_pOpenFileWithTool->setMenuText(tr("Ouvrir un fichier avec un outil..."));
	m_pOpenFileWithTool->setToolTip(tr("Permet d'ouvrir un fichier en vous laissant choisir l'outil"));

	RefreshToolMenu();
	m_pToolMenu->changeItem(1, tr("&Options..."));
	m_pToolMenu->changeItem(2, tr("Ouvrir un outil"));
}

//-----------------------------------------------------------------------------------
void MainWindow::OnShowHelp()
{
	LaunchNavigator("flash100.free.fr/Help/index.htm", TRUE);
}

//-----------------------------------------------------------------------------------
void MainWindow::OnShowAbout()
{
	frmAbout c(this);
	c.exec();
}

//-----------------------------------------------------------------------------------
void MainWindow::ProjectModified()
{
	m_bModified = true;
}

//-----------------------------------------------------------------------------------
void MainWindow::fileNew()
{
	if(AskToSaveProject()) {
		CloseProject();
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::AskToSaveProject()
{
	bool bClose = true;
	if(m_bModified) {
		if(m_wOptions.GetPreference(IOptionWindow::PREF_AUTO_SAVE_PROJECT) && !m_sProject.isEmpty()) {
			fileSave();
		}
		else {
			switch(CMessageBox::MsgYesNoCancel(this, tr("Voulez-vous sauver le projet ouvert avant de le fermer ?")))
			{
			case QMessageBox::Yes:
				//Sauve le projet
				fileSave();
				break;
			case QMessageBox::Cancel:
				bClose = false;
			}
		}
	}
	return bClose;
}

//-----------------------------------------------------------------------------------
void MainWindow::CloseProject()
{
	//Effacer les paramtres
	m_sProject = QString::null;
	CloseAllWindows();
	setCaption(F100_VERSION);
	m_bModified = false;
}

//-----------------------------------------------------------------------------------
void MainWindow::fileOpen()
{
	if(AskToSaveProject()) {
		QString sName = CMessageBox::ChooseOpenFileName(m_sProject, PROJECT_FILTER + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), this);
		if(!sName.isEmpty()) {
			LoadProject(sName);
		}
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::LoadProject(const QString& sName)
{
	if(!sName.isEmpty())
	{
		CloseProject();
		QDomDocument doc;
		QFile f(sName);
		if(!f.open(IO_ReadOnly)) {
			CMessageBox::MsgCritical(this, CLanguage::GetMsgNoOpenFile(sName));
			return;
		}
		if(!doc.setContent(&f) || doc.doctype().name() != "Flash100") {
			f.close();
			CMessageBox::MsgCritical(this, CLanguage::GetMsgNoValidFile(sName));
			return;
		}
		f.close();
		
		m_pTab->SetForceToHide(TRUE);
		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
		QDomElement root = doc.documentElement();
		
		QDomNode n = root.namedItem("Tools");
		if(!n.isNull())
		{
			QDomElement e = n.toElement();
			n = e.firstChild();
			while(!n.isNull()) {
				QString sType = n.nodeName();
				QDomElement e = n.toElement();
				QString s = tr("(Chargement d'un projet) La cration de l'outil '%1' a chou. (%2)").arg(sType);
				if(IsLoaded(sType)) {
					if(CreateToolWindowByType(sType, QString::null, 0, FALSE, NULL, NULL, NULL, &e) == NULL) {
						m_pOutput->OnError(s.arg(tr("Donnes non valides)")));
					}
				}
				else {
					m_pOutput->OnWarning(s.arg(tr("Type d'diteur non connu")));
				}
				n = n.nextSibling();
			}
		}
		
		n = root.namedItem("MainWindow");
		if(!n.isNull()) {
			QDomElement e = n.toElement();
			m_sLastFileOpened = CToolWindow::ConvertToFullPath(e.attribute("LastFileOpened"));
			m_sLastSelectedFile = CToolWindow::ConvertToFullPath(e.attribute("LastSelectedFile"));
			m_sLastSelectedFilter = CToolWindow::ConvertToFullPath(e.attribute("LastSelectedFilter"));
			m_pTab->setCurrentPage(e.attribute("CurrentPage", "0").toInt());
		}
		QApplication::restoreOverrideCursor();
		m_pTab->SetForceToHide(FALSE);
		
		m_sProject = sName;
		m_bModified = false;
		RefreshCaption();
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::fileSave()
{
	if(m_sProject.isEmpty()) {
		fileSaveAs();
	}
	else {
		SaveProject(m_sProject);
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::SortWindowList(WindowsList& out, WindowsList& in)
{
	WindowsList::iterator it = in.begin();
	CToolWindow* pWidget;
	int nLastIndex = -1;
	for(; it != in.end(); ++it) {
		pWidget = *it;
		int index = pWidget->GetIndexInTabWidget();
		if(index == -1 || (nLastIndex != -1 && index >= nLastIndex)) {
			out.push_back(pWidget);
			nLastIndex = index;
		}
		else {
			WindowsList::iterator it2 = out.begin();
			for(; it2 != out.end(); ++it2) {
				int i = (*it2)->GetIndexInTabWidget();
				if(i == -1 || index < i) {
					out.insert(it2, 1, pWidget);
					break;
				}
			}
			if(it2 == out.end()) {
				out.push_back(pWidget);
				nLastIndex = index;
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::SaveProject(const QString& sName)
{
	QDomDocument doc("Flash100");
	
	QDomElement root = doc.createElement("Project");
	doc.appendChild(root);
	QDomElement mainwindow = doc.createElement("MainWindow");
	root.appendChild(mainwindow);
	mainwindow.setAttribute("CurrentPage", QString::number(m_pTab->currentPageIndex()));
	mainwindow.setAttribute("LastFileOpened", CToolWindow::ConvertToRelativePath(m_sLastFileOpened));
	mainwindow.setAttribute("LastSelectedFile", CToolWindow::ConvertToRelativePath(m_sLastSelectedFile));
	mainwindow.setAttribute("LastSelectedFilter", CToolWindow::ConvertToRelativePath(m_sLastSelectedFilter));
		
	QDomElement tools = doc.createElement("Tools");
	root.appendChild(tools);
	WindowsList lTmpList;
	SortWindowList(lTmpList, m_lWindows);
	WindowsList::iterator it = lTmpList.begin();
	QString sType;
	while(it != lTmpList.end()) {
		CToolWindow* pWidget = *it;
		sType = pWidget->GetWindowType();
		QDomElement tool = doc.createElement(sType);
		if(pWidget->SaveTo(tool) != 1) {
			m_pOutput->OnWarning(tr("L'outil '%1' ne s'est pas sauv correctement dans le projet '%2'")
				.arg(sType)
				.arg(sName));
		}
		else {
			tools.appendChild(tool);
		}
		it++;
	}

	QFile file(sName);
	if(file.open(IO_WriteOnly)) {
		QString s = doc.toString();
		file.writeBlock(s, s.length());
		file.close();

		m_sProject  = sName;
		m_bModified = false;
		RefreshCaption();
	}
	else {
		CMessageBox::MsgCritical(this, CLanguage::GetMsgNoOpenFile(sName, TRUE));
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::fileSaveAs()
{
	QString sName = CMessageBox::ChooseSaveFileName(m_sProject, PROJECT_FILTER + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), this, "prj");
	if(!sName.isEmpty()) {
		SaveProject(sName);
	}
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::OpenToolByType(const QString& sType, const QString& sFileName, int nStartBytes, bool bShowMsgError, QWidget* pParent, void* pExtraParam)
{
	bool bIsValid;
	CToolWindow* pDialog = CreateToolWindowByType(sType, sFileName, nStartBytes, bShowMsgError, pParent, pExtraParam, &bIsValid);
	if(!bIsValid && bShowMsgError) {
		CMessageBox::MsgCritical(this, NO_VALID_FILE(sFileName));
	}
	return pDialog;
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::OpenTool(const QString& sSupportedFile, const QString& sFileName, int nStartBytes, bool bShowMsgError, QWidget* pParent)
{
	ToolsMap::Iterator it  = m_lTools.begin();
	ToolsMap::Iterator end = m_lTools.end();
	CToolWindow* pDialog = NULL;
	bool bIsValid = TRUE;
	for(; it != end; it++) {
		if(QDir::match(it.data().sSupportedExt, sSupportedFile)) {
			pDialog = CreateToolWindow(it, sFileName, nStartBytes, bShowMsgError, pParent, NULL, &bIsValid);
			if(bIsValid) {
				break;
			}
		}
	}
	if(bShowMsgError) {
		if(!bIsValid){
			CMessageBox::MsgCritical(this, NO_VALID_FILE(sFileName));
		}
		else if(it == end) {
			if(m_lTools.size() == 0) {
				CMessageBox::MsgInformation(this, tr("Impossible d'ouvrir des fichiers car aucun outil n'a t trouv!"));
			}
			else {
				CMessageBox::MsgCritical(this, tr("Aucun outil charg ne peut ouvrir les fichiers '%1'!").arg(sSupportedFile));
			}
		}
	}
	return pDialog;
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::OpenFileDrive(const QString& sFileName, int nStartBytes, bool bShowMsgError, QWidget* pParent)
{
	return OpenTool(DRIVE_EXT, sFileName, nStartBytes, bShowMsgError, pParent);
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::OpenFileSystem(const QString& sFileName, int nStartBytes, bool bShowMsgError, QWidget* pParent)
{
	return OpenTool(SYSTEM_EXT, sFileName, nStartBytes, bShowMsgError, pParent);
}

//-----------------------------------------------------------------------------------
void MainWindow::closeEvent(QCloseEvent* e)
{
	//Ferme le projet ouvert pour savoir si on doit sauver en quittant ou ne pas quitter
	//Ferme toutes les fentres (s'il y en a)
	if(!AskToSaveProject() || !CloseAllWindows()) {
		e->ignore();
		return;
	}

	//Sauve les donnes de la barre d'outils
	if(m_pToolBar->area()) {
		WriteSettings(F100_REG_KEY_TOOLBAR_AREA, m_pToolBar->area()->name());
	}
	else {
		WriteSettings(F100_REG_KEY_TOOLBAR_AREA, QString::null);
	}
	WriteSettings(F100_REG_KEY_TOOLBAR_X, m_pToolBar->x());
	WriteSettings(F100_REG_KEY_TOOLBAR_Y, m_pToolBar->y());
	WriteSettings(F100_REG_KEY_TOOLBAR_TOOLSSHOW,   m_pToolBar->GetToolShown());
	WriteSettings(F100_REG_KEY_TOOLBAR_TOOLSHIDDEN, m_pToolBar->GetToolHidden());

	
	if(m_pOutput->area()) {
		WriteSettings(F100_REG_KEY_OUTPUT_AREA, m_pOutput->area()->name());
	}
	else {
		WriteSettings(F100_REG_KEY_OUTPUT_AREA, QString::null);
	}
	WriteSettings(F100_REG_KEY_OUTPUT_X,       m_pOutput->x());
	WriteSettings(F100_REG_KEY_OUTPUT_Y,       m_pOutput->y());
	WriteSettings(F100_REG_KEY_OUTPUT_WIDTH,   m_pOutput->width());
	WriteSettings(F100_REG_KEY_OUTPUT_HEIGHT,  m_pOutput->height());
	WriteSettings(F100_REG_KEY_OUTPUT_ISSHOWN, m_pOutput->isShown());
	WriteSettings(F100_REG_KEY_OUTPUT_OPACITY, QString::number(m_pOutput->GetOpacity()));
	
	//Sauvegarde les donnes
	QRect geo = geometry();
	int nStyle;
#if QT_VERSION < 0x030300
	nStyle = (isMaximized() << 1) + isMinimized();
#else
	nStyle = windowState();
#endif
	WriteSettings(F100_REG_KEY_MAINWINDOW_X, geo.x());
	WriteSettings(F100_REG_KEY_MAINWINDOW_Y, geo.y());
	WriteSettings(F100_REG_KEY_MAINWINDOW_HEIGHT, geo.height());
	WriteSettings(F100_REG_KEY_MAINWINDOW_WIDTH, geo.width());
	WriteSettings(F100_REG_KEY_MAINWINDOW_STYLE, nStyle);
	WriteSettings(F100_REG_KEY_MAINWINDOW_LASTOPENEDPROJECT,  CToolWindow::ConvertToRelativePath(m_sProject));

	m_wOptions.DoSave();
	e->accept();
	m_pStartUp->Quit();
}

//-----------------------------------------------------------------------------------
void MainWindow::OnLinkClick()
{
	QAction* pAction = (QAction*)sender();
	if(pAction != NULL) {
		LaunchNavigator(pAction->text(), FALSE);
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::AddWindow(CToolWindow* w)
{
	//Vrifie que la fentre ajoute ne soit pas dj prsente dans la liste
	if(w && m_lWindows.find(w) == m_lWindows.end())	{
		m_lWindows.push_back(w);
		connect(w, SIGNAL(StateChanged()), SLOT(ProjectModified()));
		connect(w, SIGNAL(destroyed(QObject*)), SLOT(OnObjectRemoved(QObject*)));
		connect(w, SIGNAL(Message(const QString&, int)), statusBar(), SLOT(message(const QString&, int)));

		connect(w, SIGNAL(Warning(const QString&)),     m_pOutput, SLOT(OnWarning(const QString&)));
		connect(w, SIGNAL(Error(const QString&)),       m_pOutput, SLOT(OnError(const QString&)));
		connect(w, SIGNAL(Information(const QString&)), m_pOutput, SLOT(OnInformation(const QString&)));
		
		ProjectModified();
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::ShowWindow(int num)
{
	CToolWindow* w = m_lWindows[num];
	if(w != NULL) {
		w->show();
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::CloseWindow(int num)
{
	CToolWindow* w = m_lWindows[num];
	if(w != NULL) {
		w->close(TRUE);
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::CloseAllWindows()
{
	while(!m_lWindows.isEmpty() && (*m_lWindows.begin())->close(TRUE));
	return m_lWindows.isEmpty();
}

//-----------------------------------------------------------------------------------
void MainWindow::OnRefreshWindowList()
{
	m_pListWindowShowMenu->clear();
	m_pListWindowCloseMenu->clear();
	WindowsList::iterator it = m_lWindows.begin();
	int i = 0;
	for(;it != m_lWindows.end(); i++) {
		CToolWindow* w = *it;
		it++;
		if(w->icon()) {
			QIconSet icon(*w->icon());
			m_pListWindowCloseMenu->insertItem(icon, w->caption(), i);
			m_pListWindowShowMenu->insertItem(icon, w->caption(), i);
		}
		else {
			m_pListWindowCloseMenu->insertItem(w->caption(), i);
			m_pListWindowShowMenu->insertItem(w->caption(), i);
		}
	}
	m_pListWindowCloseMenu->setEnabled(i != 0);
	m_pListWindowShowMenu->setEnabled(i != 0);
	m_pWindowMenu->setItemEnabled(0, i != 0);
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::CreateToolWindow(ToolsMap::Iterator& it, const QString& sName, int nStartBytes, bool bShowMsgError, QWidget* pParent, void* pExtraParam, bool* bIsValid, QDomElement* pElement)
{
	CToolWindow* pDialog = NULL;
	bool bValid = TRUE;
	if(it != m_lTools.end())
	{
		QString sType = it.key();
		WindowsList::iterator it2   = m_lWindows.begin();
		WindowsList::iterator itEnd = m_lWindows.end();
		for(; it2 != itEnd && (*it2)->GetWindowType() != sType; ++it2);
		if(it2 != itEnd)
		{
			pDialog = (*it2);
			if(pDialog->OnlyOneInstance())
			{
				if(!sName.isEmpty()) {
					bValid = pDialog->AddFile(sName, nStartBytes, pExtraParam) >= 0;
				}
				if(bIsValid) {
					*bIsValid = bValid;
				}
				if(bValid) {
					pDialog->show();
					return pDialog;
				}
				return NULL;
			}
		}
		if(pParent){
			(it.data().op)(OP_CREATEINSTANCE, (void**)&pDialog, this, 0, 0, pParent);
		}
		else{
			(it.data().op)(OP_CREATEINSTANCE, (void**)&pDialog, this, FLAGS, m_pTab, pParent);
			AddWindow(pDialog);
		}
		if(pDialog) {
			pDialog->SetWindowType(sType);
			int r = pDialog->Init(sName, nStartBytes, pExtraParam);
			bValid = (r >= 0);
			if(r < 1) {
				pDialog->close(TRUE);
				pDialog = NULL;
			}
		}
		if(pDialog != NULL && pElement != NULL) {
			if(pDialog->LoadFrom(*pElement) < 1) {
				pDialog->close(TRUE);
				pDialog = NULL;
			}
		}
	}
	else {
		if(bShowMsgError) {
			CMessageBox::MsgCritical(this, tr("L'outil demand n'a pas t trouv!"));
		}
	}
	if(bIsValid) {
		*bIsValid = bValid;
	}
	if(pDialog != NULL) {
		pDialog->show();
	}
	return pDialog;
}

//-----------------------------------------------------------------------------------
CToolWindow* MainWindow::CreateToolWindowByType(const QString& sType, const QString& sName, int nStartBytes, bool bShowMsgError, QWidget* pParent, void* pExtraParam, bool* bIsValid, QDomElement* pElement)
{
	QMapIterator<QString, Tools> tmp = m_lTools.find(sType);
	return CreateToolWindow(tmp, sName, nStartBytes, bShowMsgError, pParent, pExtraParam, bIsValid, pElement);
}

//-----------------------------------------------------------------------------------
void MainWindow::LoadTools()
{
	m_lTools.clear();
	m_sSupportedFiles = QString::null;

	QString sPath = PLUGIN_DIR;
	QDir d(sPath);
	if(d.exists())
	{
		d.setFilter(QDir::Files);
		d.setNameFilter(DLL_EXTENSION);
		const QFileInfoList* pList = d.entryInfoList();
		QFileInfoListIterator it(*pList);
		QFileInfo* pFileInfo;
		for(; (pFileInfo = it.current()) != NULL; ++it)
		{
			Tools_Op pFunction = (Tools_Op)QLibrary::resolve(pFileInfo->absFilePath(), "Tool_Request");
			if(pFunction)
			{
				const char* pType = NULL;
				pFunction(OP_WINDOWTYPESTR, (void**)&pType, NULL, NULL, NULL, NULL);
				if(pType != NULL)
				{
					if(m_lTools.find(pType) == m_lTools.end())
					{
						QPixmap pixmap;
						QString sFilter;
						CSettingsWidget* pWidget = NULL;
						const char* pFiles = NULL;
						QAction* pAction = NULL;
						pFunction(OP_TOOLBARICON, (void**)&pixmap, this, NULL, NULL, NULL);
						pFunction(OP_SETTINGSWIDGET, (void**)&pWidget, this, NULL, NULL, NULL);
						pFunction(OP_SUPPORTEDFILE, (void**)&pFiles, this, NULL, NULL, NULL);						
						pFunction(OP_FILEFILTER, (void**)&sFilter, NULL, NULL, NULL, NULL);
						QString sSupportedFiles(pFiles);
						QString sExt;
						for(int nSection = 0; !((sExt=sSupportedFiles.section(';', nSection, nSection)).isEmpty()); nSection++)
						{
							if(!IsSupported(sExt)) {
								if(!m_sSupportedFiles.isEmpty()) {
									m_sSupportedFiles += ";";
								}
								m_sSupportedFiles += sExt;
							}
						}
						pAction = new QAction(this);
						connect(pAction, SIGNAL(activated()), SLOT(OnActionTool()));
						pAction->setIconSet(pixmap);						
						if(sFilter.isEmpty()) {
							pAction->addTo(m_pOpenToolMenu);
						}
						Tools tool;
						tool.op      = pFunction;
						tool.pAction = pAction;
						tool.sSupportedExt = sSupportedFiles;
						m_lTools[pType] = tool;
						m_wOptions.AddSettingsWidget(pWidget);
					}
					else
					{
						CMessageBox::MsgExclamation(this, tr("Le type de cet diteur a dj t charg. (%1)").arg(pFileInfo->absFilePath()));
					}
				}
			}
			else
			{
				m_pOutput->OnWarning(tr("Le fichier '%1' n'a pas pu tre charg en tant qu'outil.").arg(pFileInfo->absFilePath()));
			}
		}
	}
	m_pToolBar->SetToolList(&m_lTools);

	m_pToolMenu->setItemEnabled(2, m_pOpenToolMenu->count());

	//Rafraichit les textes dans les menus
	RefreshToolMenu();
}

//-----------------------------------------------------------------------------------
void MainWindow::RefreshToolMenu()
{
	ToolsMap::const_iterator it = m_lTools.begin();
	QString sMenuText;
	QString sDescription;
	QString sAuthor;
	QString sVersion;
	QString sBuildDate;
	QString sFileExt;
	while(it != m_lTools.end()) {
		Tools tool = it.data();
		if(tool.pAction != NULL && GetToolInfos(it, sMenuText, sDescription, sFileExt, sAuthor, sVersion, sBuildDate)) {
			tool.pAction->setMenuText(sMenuText);
			tool.pAction->setToolTip(sDescription);
		}
		++it;
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::OnActionTool()
{
	QAction* pAction = (QAction*)sender();
	ToolsMap::Iterator it = m_lTools.begin();
	for(; it != m_lTools.end() && it.data().pAction != pAction; ++it);
	if(it != m_lTools.end()) {
		Tools tool = it.data();
		Tools_Op pFunction = tool.op;
		QString sFilter;
		QString sName;
		pFunction(OP_FILEFILTER, (void**)&sFilter, NULL, NULL, NULL, NULL);
		if(!sFilter.isEmpty()) {
			sName = CMessageBox::ChooseOpenFileName(QString::null, sFilter, this);
			if(sName.isEmpty()) {
				return;
			}
		}
		OpenToolByType(it.key(), sName);
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::OnToolMenuClick(int id)
{
	if(id == 1) {
		m_wOptions.setActiveWindow();
		m_wOptions.show();
		return;
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::WriteSettings(const QString& sPath, const int& nNumber)
{
	assert(m_pSettings != NULL);
	return m_pSettings->writeEntry(sPath, nNumber);
}

//-----------------------------------------------------------------------------------
bool MainWindow::WriteSettings(const QString& sPath, const QString& sText)
{
	assert(m_pSettings != NULL);
	if(sText.isNull()) {
		return m_pSettings->writeEntry(sPath, "");
	}
	return m_pSettings->writeEntry(sPath, sText);
}

//-----------------------------------------------------------------------------------
bool MainWindow::ReadSettings(const QString& sPath, int& nNumber, const int& nDefault) const
{
	assert(m_pSettings != NULL);
	bool bOk;
	nNumber = m_pSettings->readNumEntry(sPath, nDefault, &bOk);
	return bOk;
}

//-----------------------------------------------------------------------------------
bool MainWindow::ReadSettings(const QString& sPath, QString& sText, const QString& sDefault) const
{
	assert(m_pSettings != NULL);
	bool bOk;
	sText = m_pSettings->readEntry(sPath, sDefault, &bOk);
	return bOk;
}

//-----------------------------------------------------------------------------------
QStringList MainWindow::SettingsList(const QString& sPath) const
{
	assert(m_pSettings != NULL);
	return m_pSettings->entryList(sPath);
}

//-----------------------------------------------------------------------------------
QStringList MainWindow::SubSettingsList(const QString& sPath) const
{
	assert(m_pSettings != NULL);
	return m_pSettings->subkeyList(sPath);
}

//-----------------------------------------------------------------------------------
bool MainWindow::RemoveSettings(const QString& sPath, bool bRemoveSubKeys)
{
	assert(m_pSettings != NULL);
	QStringList sList = m_pSettings->entryList(sPath);
	QStringList::iterator it  = sList.begin();
	QStringList::iterator end = sList.end();
	bool bRet = true;
	for(; it != end && bRet; ++it) {
		bRet = m_pSettings->removeEntry(sPath + '/' + *it);
	}
	if(bRemoveSubKeys) {
		sList = m_pSettings->subkeyList(sPath);
		it  = sList.begin();
		end = sList.end();
		for(; it != end && bRet; ++it) {
			bRet = RemoveSettings(sPath + '/' + *it);
		}
	}
	return bRet && m_pSettings->removeEntry(sPath);
}

//-----------------------------------------------------------------------------------
QString MainWindow::ConvertSupportedExtToFilter(const QString& sSupportedExt)
{
	QString sExts;
	QString sExt;
	for(int nSection = 0; (sExt=sSupportedExt.section(';', nSection, nSection)) && !sExt.isEmpty(); nSection++) {
		if(!sExts.isEmpty()) {
			sExts += ";";
		}
		sExts += "*." + sExt;
	}
	return sExts;
}

//-----------------------------------------------------------------------------------
void MainWindow::OnOpenFiles()
{
	QString sFilters = PROJECT_FILTER;
	QString sExts = ConvertSupportedExtToFilter(m_sSupportedFiles);

	if(!sExts.isEmpty()) {
		sFilters = tr("Fichiers Supports (%1)").arg(sExts) + '\n' + sFilters;
	}
	QStringList lFiles = CMessageBox::ChooseOpenFileNames(sFilters, m_sLastFileOpened, this);
	if(lFiles.count()) {
		m_sLastFileOpened = lFiles[0];
		LoadFiles(lFiles);
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::OnOpenFile()
{
	QStringList lTools;
	QString sFilters;

	QString sToolName, sDescription;
	QString sAuthor, sVersion;
	QString sBuildDate, sFileExt;

	ToolsMap::const_iterator it  = m_lTools.begin();
	ToolsMap::const_iterator end = m_lTools.end();
	for(; it != end; ++it) {
		if(!it.data().sSupportedExt.isEmpty()) {
			GetToolInfos(it, sToolName, sDescription, sFileExt, sAuthor, sVersion, sBuildDate);
			QString sFilter = ConvertSupportedExtToFilter(it.data().sSupportedExt);
			if(!sFilters.isEmpty()) {
				sFilters += '\n';
			}
			sFilter   = sToolName + " (" + sFilter + ")";
			sFilters += sFilter;
			lTools.append(sFilter);
		}
		else {
			lTools.append(it.key());
		}
	}

	QFileDialog fd(this, NULL, TRUE);
	fd.setMode(QFileDialog::ExistingFile);
	fd.setFilters(sFilters);
	fd.setCaption(m_pOpenFileWithTool->menuText());
	if(!m_sLastSelectedFilter.isEmpty()) {
		fd.setSelectedFilter(m_sLastSelectedFilter);
	}
	fd.setSelection(m_sLastSelectedFile);
	if(fd.exec() == QDialog::Accepted) {
		QString key = m_lTools.keys()[lTools.findIndex(fd.selectedFilter())];
		if(CreateToolWindowByType(key, fd.selectedFile(), 0, TRUE, NULL) == NULL) {
			CMessageBox::MsgCritical(this, NO_VALID_FILE(fd.selectedFile()));
		}
		else {
			m_sLastSelectedFile   = fd.selectedFile();
			m_sLastSelectedFilter = fd.selectedFilter();
		}
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::CheckArgs(QStringList& lFiles)
{
	QStringList::iterator it    = lFiles.begin();
	QStringList::iterator itEnd = lFiles.end();
	while(it != itEnd) {
		QString s(*it);	
		if(s == "-uninstall") {
			UninstallAssocFile(m_sSupportedFiles);
			return true;
		}			
		++it;
	}
	return false;
}

//-----------------------------------------------------------------------------------
int MainWindow::LoadFiles(QStringList& lFiles, bool bDoLoadOnlyProject)
{
	int nLoadedFiles = 0;
	if(CheckArgs(lFiles)) {
		qApp->exit(0);
		return 0;
	}
	//Charge d'abord le premier projet pass en paramtre
	//les autres sont ignors
	QStringList::iterator it    = lFiles.begin();
	QStringList::iterator itEnd = lFiles.end();
	while(it != itEnd)
	{
		QString s(*it);
		if(QDir::match(QString("*.prj"), s))
		{
			if(!nLoadedFiles)
			{
				if(AskToSaveProject())
				{
					LoadProject(s);
				}
				nLoadedFiles++;
			}
			it    = lFiles.erase(it);
			itEnd = lFiles.end();
		}
		else {
			++it;
		}
	}
	if(!bDoLoadOnlyProject)
	{
		it = lFiles.begin();
		while(it != itEnd)
		{
			QString s(*it);
			QFileInfo fi(s);
			if(fi.isFile() && OpenTool(fi.extension(), s))
			{
				nLoadedFiles++;
			}
			++it;
		}
	}
	return nLoadedFiles;
}

//-----------------------------------------------------------------------------------
void MainWindow::dragEnterEvent(QDragEnterEvent* e)
{
	if(QUriDrag::canDecode(e)) {
		e->accept();
    }
}

//-----------------------------------------------------------------------------------
void MainWindow::dropEvent(QDropEvent* e)
{
	QStringList sList;
	if(QUriDrag::decodeLocalFiles(e, sList)) {
		LoadFiles(sList);
	}
}

//-----------------------------------------------------------------------------------
void MainWindow::RefreshCaption()
{
	const char* s;
	GetNameOfFile(m_sProject, s);
	setCaption(QString(F100_VERSION) + QString(" - ") + s);
}

//-----------------------------------------------------------------------------------
bool MainWindow::IsLoaded(const QString& sToolType)
{
	return m_lTools.end() != m_lTools.find(sToolType);
}

//-----------------------------------------------------------------------------------
bool MainWindow::IsSupported(const QString& sFileExt)
{
	return QDir::match(m_sSupportedFiles, sFileExt);
}

//-----------------------------------------------------------------------------------
QString MainWindow::GetSupportedFiles()
{
	return m_sSupportedFiles;
}

//-----------------------------------------------------------------------------------
void MainWindow::OnObjectRemoved(QObject* o)
{
	CToolWindow* pDialog = (CToolWindow*)o;
	WindowsList::iterator it = m_lWindows.find(pDialog);
	if(it != m_lWindows.end()) {
		m_lWindows.remove(pDialog);
		ProjectModified();
	}
}

//-----------------------------------------------------------------------------------
bool MainWindow::ReStartApplication()
{
	return m_pStartUp->ReStart();
}

//-----------------------------------------------------------------------------------
bool MainWindow::CloseApplication(bool bForce)
{
	if(bForce) {
		WindowsList::iterator it = m_lWindows.begin();
		while(m_lWindows.size() != 0) {
			delete *(m_lWindows.begin());
		}
		m_bModified = FALSE;
	}
	return close();
}

//-----------------------------------------------------------------------------------
IOptionWindow* MainWindow::GetOptionWindow()
{
	return &m_wOptions;
}

//-----------------------------------------------------------------------------------
CSendToMenu* MainWindow::GetSendToMenu()
{
	return m_pSendToMenu;
}

//-----------------------------------------------------------------------------------
bool MainWindow::isCustomizable() const
{
	return TRUE;
}

//-----------------------------------------------------------------------------------
void MainWindow::customize()
{
	m_pToolBar->ShowCustomize();
}

//-----------------------------------------------------------------------------------
QStringList MainWindow::GetToolNames() const
{
	return m_lTools.keys();
}

//-----------------------------------------------------------------------------------
bool MainWindow::GetToolInfos(const QString& sToolType, QString& sToolName, QString& sDesc, QString& sFileExt, QString& sAuthor, QString& sVersion, QString& sBuildDate) const
{
	QMapConstIterator<QString, Tools> tmp = m_lTools.find(sToolType);
	return GetToolInfos(tmp, sToolName, sDesc, sFileExt, sAuthor, sVersion, sBuildDate);
}

//-----------------------------------------------------------------------------------
bool MainWindow::GetToolInfos(ToolsMap::const_iterator& it, QString& sToolName, QString& sDesc, QString& sFileExt, QString& sAuthor, QString& sVersion, QString& sBuildDate) const
{
	char* pAuthor, *pVersion, *pDate;
	if(it != m_lTools.end()) {
		Tools tool = it.data();
		Tools_Op pFunction = tool.op;
		pAuthor = pVersion = pDate = "?";

		pFunction(OP_TOOLNAME, (void**)&sToolName, NULL, NULL, NULL, NULL);
		pFunction(OP_DESCRIPTION, (void**)&sDesc,  NULL, NULL, NULL, NULL);
		pFunction(OP_AUTHOR, (void**)&pAuthor,     NULL, NULL, NULL, NULL);
		pFunction(OP_VERSION, (void**)&pVersion,   NULL, NULL, NULL, NULL);
		pFunction(OP_BUILDDATE, (void**)&pDate,    NULL, NULL, NULL, NULL);

		sFileExt   = it.data().sSupportedExt;
		sAuthor    = pAuthor;
		sVersion   = pVersion;
		sBuildDate = pDate;
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
void MainWindow::OnAboutToShowHelpMenu()
{
	helpAction[0]->setMenuText(tr("&Aide de Flash100"));
	helpAction[1]->setMenuText(tr("A propos de &Flash100..."));

	if(m_pHelpMenu->count() == 0) {
		helpAction[0]->addTo(m_pHelpMenu);
		m_pHelpMenu->insertSeparator();
		m_pHelpMenu->insertSeparator();
		helpAction[1]->addTo(m_pHelpMenu);
	}

	if(m_pLinksMenu != NULL) {
		delete m_pLinksMenu;
	}

	m_pLinksMenu = CEditLinkTab::GetInstance()->CreateMenu(m_pHelpMenu, this, SLOT(OnLinkClick()));

	m_pHelpMenu->insertItem(tr("Liens"), m_pLinksMenu, -1, 2);
}
