
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <qdatetime.h>
#include <qmap.h>
#include <qmainwindow.h>
#include <qvaluelist.h>

#include <IMainWindow.h>
#include <F100_Op.h>

#include "OptionWindow.h"
#include "defs.h"

class CToolWindow;
class QTabWidget;
class QDomElement;
class ToolTabWidget;
class CToolBar;
class QPopupMenu;
class QSettings;
class ConsoleWindow;
class StartUp;
class CSendToMenu;

typedef QValueList<CToolWindow*> WindowsList;

class MainWindow : public QMainWindow, public IMainWindow
{
	Q_OBJECT
public:
	MainWindow(StartUp* pStartUp, QSettings* pSettings, QWidget* parent = 0);

	bool           Init(QStringList& lFiles);
	CToolWindow*   OpenToolByType(const QString& sType, const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL, void* pExtraParam = NULL);
	CToolWindow*   OpenTool(const QString& sSupportedFile, const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL);
	CToolWindow*   OpenFileDrive(const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL);
	CToolWindow*   OpenFileSystem(const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL);
	bool           IsLoaded(const QString& sToolType);
	bool           IsSupported(const QString& sFileExt);
	QString        GetSupportedFiles();
	int            LoadFiles(QStringList& lFiles, bool bDoLoadOnlyProject = FALSE);
	QStringList    GetToolNames() const;
	bool           GetToolInfos(const QString& sToolType, QString& sToolName, QString& sDesc, QString& sFileExt, QString& sAuthor, QString& sVersion, QString& sBuildDate) const;

	bool           WriteSettings(const QString& sPath, const int& nNumber);
	bool           WriteSettings(const QString& sPath, const QString& sText);
	bool           ReadSettings(const QString& sPath, int& nNumber, const int& nDefault = 0) const;
	bool           ReadSettings(const QString& sPath, QString& sText, const QString& sDefault = QString::null) const;
	QStringList    SettingsList(const QString& sPath) const;
	QStringList    SubSettingsList(const QString& sPath) const;
	bool           RemoveSettings(const QString& sPath, bool bRemoveSubKeys = TRUE);
	bool           ReStartApplication();
	bool           CloseApplication(bool bForce = FALSE);
	IOptionWindow* GetOptionWindow();
	CSendToMenu*   GetSendToMenu();
	bool           isCustomizable() const;

protected:
	void       LoadTools();
	void       RefreshToolMenu();
	void       RefreshCaption();
	void       SortWindowList(WindowsList& out, WindowsList& in);
	void       LoadDockWindow(QDockWindow* pDockWindow, const QString& sArea, ToolBarDock defaultDock);
	/** Retourne vrai pour spcifier un arrt du programme */
	bool       CheckArgs(QStringList& lFiles);
	/** From Qt */
	void       closeEvent(QCloseEvent* e);
	void       dragEnterEvent(QDragEnterEvent* );
	void       dropEvent(QDropEvent* );

private slots:
	void       fileNew();
	void       fileOpen();
	void       fileSave();
	void       fileSaveAs();
	void       OnShowAbout();
	void       OnShowHelp();
	void       OnLinkClick();
	bool       CloseAllWindows();
	void       CloseWindow(int);
	void       ShowWindow(int);
	void       ProjectModified();
	void       OnToolMenuClick(int);
	void       OnActionTool();
	void       OnOpenFiles();
	void       OnOpenFile();
	void       OnObjectRemoved(QObject* o);
	void       languageChange();
	void       OnRefreshWindowList();
	void       customize();
	void       OnAboutToShowHelpMenu();

private:
	void       LoadProject(const QString& sName);
	void       SaveProject(const QString& sName);
	void       CloseProject();
	bool       AskToSaveProject();//retourne false si l'utilisateur annule
	void       AddWindow(CToolWindow* w);
	CToolWindow* CreateToolWindowByType(const QString& sType, const QString& sName, int nStartBytes, bool bShowMsgError, QWidget* pParent, void* pExtraParam = NULL, bool* bIsValid = NULL, QDomElement* pElement = NULL);
	CToolWindow* CreateToolWindow(ToolsMap::Iterator& it, const QString& sName, int nStartBytes, bool bShowMsgError, QWidget* pParent, void* pExtraParam = NULL, bool* bIsValid = NULL, QDomElement* pElement = NULL);
	bool       GetToolInfos(ToolsMap::const_iterator& it, QString& sToolName, QString& sDesc, QString& sFileExt, QString& sAuthor, QString& sVersion, QString& sBuildDate) const;
	QString    ConvertSupportedExtToFilter(const QString& sSupportedExt);

private:
	ConsoleWindow* m_pOutput;
	ToolTabWidget* m_pTab;
	
	QPopupMenu*   m_pFileMenu;
	QPopupMenu*   m_pToolMenu;
	QPopupMenu*   m_pHelpMenu;
	QPopupMenu*   m_pWindowMenu;
	QPopupMenu*   m_pListWindowCloseMenu;
	QPopupMenu*   m_pListWindowShowMenu;
	QPopupMenu*   m_pLinksMenu;
	CToolBar*     m_pToolBar;
	QAction*      fileAction[5];
	QAction*      helpAction[2];
	QAction*      m_pOpenFilesAction;
	QAction*      m_pOpenFileWithTool;
	QPopupMenu*   m_pOpenToolMenu;
	OptionWindow  m_wOptions;
	CSendToMenu*  m_pSendToMenu;

	QString	      m_sProject;	//Nom du projet ouvert
	WindowsList   m_lWindows;
	bool          m_bModified;	//A vrai si le fichier a t modifi.
	ToolsMap      m_lTools;		//Liste des outils chargs (dll)
	QSettings*    m_pSettings;

	QString       m_sLastFileOpened;

	QString       m_sLastSelectedFile;  //Menu ouvrir un fichier avec un outil
	QString       m_sLastSelectedFilter;//Menu ouvrir un fichier avec un outil

	QString       m_sSupportedFiles;
	StartUp*      m_pStartUp;
};

#endif // MAINWINDOW_H
