
#ifdef _MSC_VER
//Enlve les warning sur les listes avec les string (nom trop long)
#	pragma warning(disable:4786)
#endif

#include <iostream>
#include <stdio.h>
#include <list>
#include <string>
#include <signal.h>

#include <APIFunction.h>
#include <IProtocol.h>
#include <F100_Op.h>
#include <F100_Debug.h>
#include <F100Devices.h>
#include <CasioSystem.h>

#ifdef FRANCAIS
#	include "Languages/francais.h"
#else
#	include "Languages/english.h"
#endif

#ifndef PROTOCOL_DIR
#	define PROTOCOL_DIR "./Protocols"
#endif

using namespace std;

char* m_sVersion = "1.30";

#ifdef WIN32

#	define _CRTDBG_MAP_ALLOC
#	include <crtdbg.h>

#	define HLIB HMODULE

#	define COLOR_ERROR   FOREGROUND_RED   | FOREGROUND_INTENSITY
#	define COLOR_OK      FOREGROUND_GREEN | FOREGROUND_INTENSITY
#	define COLOR_WARNING FOREGROUND_RED   | FOREGROUND_BLUE
#	define COLOR_INFO    FOREGROUND_GREEN

HND hOut;
HND hIn;

#else

#	include <dlfcn.h>

#	define HLIB void*
#	define WORD short
#	define strnicmp strncmp

#	define COLOR_ERROR   31
#	define COLOR_OK      32
#	define COLOR_WARNING 35
#	define COLOR_INFO    34

#	define hOut stdout
#	define hIn  stdin

#endif

#ifndef TRUE
#	define TRUE true
#endif
#ifndef FALSE
#	define FALSE false
#endif

bool bDoExit = false;

char* sZonesName[] = {NZ_VARIABLES, NZ_BASIC, NZ_GRAPH, NZ_SYSTEM, NZ_CHAR, NZ_LIST, NZ_MATRIX,
							NZ_IMAGE, NZ_XLECT, NZ_LCL, "DT8", "DT9", NZ_ADDIN};

#define OTHER "other"
char* sZonesDir[] = {"vars", "basic", "graph", "system", "char", "lists", "matrix",
							"images", "xlect", "local", "dt8", "dt9", "addins", OTHER, OTHER, OTHER};


void ShowText(const char* msg, WORD attribute);
void ShowWarning(const char* msg);
void ShowError(const char* msg);
void ShowInformation(const char* msg);
void ShowCanceled();
void ShowHelp();
void ShowOK();
void ShowFailed();
void ShowProtocolError(IProtocol* pProtocol);
void ShowAvailableDevices();
int  LoadProtocol(HLIB& h, const char* sName, IProtocol*& pProtocol, bool bShowInfos = FALSE);
void DisplayLastError();
void WritePercent(unsigned short nPercent);
const char* GetCalcType(CasioType ct);

void FreeProtocol(HLIB& h);
void DeleteProtocol(HLIB& h, IProtocol*& pProtocol);
IProtocol* CreateProtocol(HLIB& h);

typedef list<string> FileList;
/** Retourne une liste de fichiers qui ont pour path sPath */
void   GetFileList(const char* sPath, FileList& filelist);
/** Retourne le rpertoire du chemin indiqu avec '/' comme dernier caractre */
string GetPath(const char* sFile);

//signaux CTRL-C, SIGV9
void SIGHandler(int sig);

unsigned short nDefaultRamSizes[] = {24, 14};


int SendRAMFile(IProtocol* pProtocol, unsigned short nZone, const char* path)
{
	unsigned char buffer[65536];
	unsigned short nMaxSize = 65535 - nDefaultRamSizes[nZone != 1];
	HND h = OpenDescModeRead(path);
	if(h == INVALID_HANDLE_VALUE)
	{
		char buf[1024];
		sprintf(buf, STR_CANT_OPEN_FILE, path);
		ShowError(buf);
		return -1;
	}
	unsigned int nSize = FileSize(h);
	if(nSize > nMaxSize)
	{
		nSize = nMaxSize;
		ShowWarning(STR_WNG_MAX_RAM_SIZE);
	}

	if(!ReadDesc(h, buffer, nSize))
	{
		ShowError(STR_CANT_READ_FILE);
		CloseDesc(h);
		return -1;
	}
	CloseDesc(h);
	cout << STR_ASKING_SEND << flush;
	const char* name;
	GetNameOfFile(path, name);
	if(pProtocol->SendRAMFile(nZone, name, nSize) == -1)
	{
		ShowFailed();
		ShowProtocolError(pProtocol);
		return -1;
	}
	ShowOK();
	cout << STR_WAIT_CONFIRMATION << flush;
	int result;
	while((result=pProtocol->WaitRAMFileConfirmation(nZone)) == 0)
	{
		if(bDoExit)
		{
			ShowCanceled();
			return 0;
		}
	}
	switch(result)
	{
	case -1:
		ShowOK();
		ShowInformation(STR_SEND_CANCELED);
		return 0;
	case -2:
		ShowFailed();
		ShowProtocolError(pProtocol);
		return -1;
	}
	ShowOK();
	cout << STR_SENDING_FILE << " " << name << "..." << flush;
	unsigned int nCpt = 0;

	int nRet = 1;
	while(nRet > 0 && nCpt < nSize)
	{
		nRet = pProtocol->SendData(buffer + nCpt, nSize - nCpt);
		if(nRet > 0)
		{
			nCpt += nRet;
		}
		WritePercent(100*nCpt / nSize);
	}
	if(nRet < 1)
	{
		ShowFailed();
		return -1;
	}
	ShowOK();
	return 0; 
}

int ReceiveRAMFiles(IProtocol* pProtocol, const char* pDirectory)
{
	char sName[9];//Nom du fichier recu
	int nRet = -1;
	unsigned short nZone;
	char sFile[1024];
	char buffer[65535];
	while(1)
	{
		cout << STR_WAITING_REQUESTS << flush;
		buffer[0] = '\0';
		while((nRet = pProtocol->ReceiveRAMFile(nZone, sName)) == -3)
		{
			if(bDoExit)
			{
				ShowCanceled();
				return 0;
			}
		}
		if(nRet == -1)
		{
			ShowFailed();
			ShowProtocolError(pProtocol);
			break;
		}
		ShowOK();
		if(nRet == -2)//Fin de rception
		{
			break;
		}
		sprintf(buffer, "%s/%s", pDirectory, sZonesDir[nZone]);
		MkDir(buffer);
		sprintf(sFile, "%s/%s", buffer, sName);
		cout << STR_CREATING_FILE << sFile << "..." << flush;

		HND h = OpenDescModeRdWr(sFile);
		if(h == INVALID_HANDLE_VALUE)
		{
			ShowFailed();
			sprintf(buffer, STR_CANT_OPEN_FILE, sFile);
			ShowError(buffer);
			break;
		}
		ShowOK();

		cout << STR_SENDING_CONFIRMATION << flush;
		if(pProtocol->SendRAMFileConfirmation(TRUE, nZone) < 1)
		{
			ShowFailed();
			ShowProtocolError(pProtocol);
			CloseDesc(h);
			break;
		}
		ShowOK();

		//Reception des donnees
		cout << STR_RECEIVING << sName << " (" << sZonesName[nZone] << ")..." << flush;
		int nBytesReceived = 0;
		int r = 1;
		while(nBytesReceived < nRet && r > 0)
		{
			r = pProtocol->ReceiveData(buffer + nBytesReceived, nRet - nBytesReceived);
			if(r > 0)
			{
				nBytesReceived += r;
			}
			WritePercent(100*nBytesReceived / nRet);
		}
		if(r > 0)
		{
			r = WriteDesc(h, buffer, nBytesReceived);
		}
		CloseDesc(h);
		if(!r)
		{
			ShowFailed();
			break;
		}
		ShowOK();
	}
	return nRet != -2;
}

int SendFlash(IProtocol* pProtocol, int nDrive, const char* path)
{
	unsigned char buffer[131072];
	memset(buffer, 0xFF, sizeof(buffer));
	if(nDrive == 0 && !CCasioSystem::IsValid(path, 0))
	{
		ShowError(STR_INVALID_SYSTEM);
		return -1;
	}
	HND h = OpenDescModeRead(path);
	if(h == INVALID_HANDLE_VALUE)
	{
		char buf[1024];
		sprintf(buf, STR_CANT_OPEN_FILE, path);
		ShowError(buf);
		return -1;
	}
	unsigned int nSize = FileSize(h);
	if(nSize > 131072)
	{
		nSize = 131072;
		ShowWarning(STR_WNG_MAX_DRIVE_SIZE);
	}
	
	if(!ReadDesc(h, buffer, nSize))
	{
		ShowError(STR_CANT_READ_FILE);
		CloseDesc(h);
		return -1;
	}
	CloseDesc(h);
	
	const char* sName;
	GetNameOfFile(path, sName);

	nSize = pProtocol->GetUsefulSize(buffer);

	cout << STR_EXPORT_DRIVE << flush;
	if(pProtocol->ExportDrive(nDrive, buffer) <= 0)
	{
		ShowFailed();
		ShowProtocolError(pProtocol);
		return -1;
	}
	ShowOK();
	
	cout << STR_SENDING_FILE << " " << sName << "..." << flush;

	unsigned int nSend = 0;
	int nRet = 1;
	while(nRet > 0 && nSend < nSize)
	{
		nRet = pProtocol->SendData(buffer + nSend, nSize - nSend);
		if(nRet > 0)
		{
			nSend	+= nRet;
		}
		WritePercent((100*nSend) / nSize);
	}
	if(nRet < 1)
	{
		ShowFailed();
		ShowProtocolError(pProtocol);
		return -1;
	}
	ShowOK();
	return 0;
}

int ReceiveFlash(IProtocol* pProtocol, int nDrive, const char* path)
{
	unsigned char buffer[131072];
	cout << STR_IMPORT_DRIVE << flush;
	int nReceive = pProtocol->ImportDrive(nDrive);
	int nByteReadDrive = 0;
	if(nReceive < 0)
	{
		ShowFailed();
		ShowProtocolError(pProtocol);
		return -1;
	}
	ShowOK();
	cout << STR_RECEIVING_DRIVE << flush;
	//Dpart du transfert
	while(nByteReadDrive < nReceive)
	{
		if(bDoExit)
		{
			ShowCanceled();
			return 0;
		}

		int n = pProtocol->ReceiveData(buffer + nByteReadDrive, nReceive - nByteReadDrive);
		if(n < 1)
		{
			ShowFailed();
			ShowProtocolError(pProtocol);
			return -1;
		}
		nByteReadDrive	+= n;
		WritePercent((100*nByteReadDrive) / nReceive);
	}
	ShowOK();


	cout << STR_CREATING_FILE << path << "..." << flush;
	HND h = OpenDescModeRdWr(path);
	if(h == INVALID_HANDLE_VALUE)
	{
		char buf[1024];
		ShowFailed();
		sprintf(buf, STR_CANT_OPEN_FILE, path);
		ShowError(buf);
	}
	ShowOK();
	if(nDrive == 0 && nReceive < 131072)//Systme
	{
		memset(buffer + nReceive, 0xFF, 131072 - nReceive);
		nReceive = 131072;
	}
	WriteDesc(h, buffer, nReceive);
	CloseDesc(h);

	return 0;
}

int main(int argc, char** argv)
{
	//Pour dtecter les fuites de mmoire en mode debug sous windows
#ifdef WIN32
#	ifdef _DEBUG
	_CrtSetReportMode(_CRT_WARN,   _CRTDBG_MODE_DEBUG);
	_CrtSetReportMode(_CRT_ERROR,  _CRTDBG_MODE_DEBUG);
	_CrtSetReportMode(_CRT_ASSERT, _CRTDBG_MODE_DEBUG);
	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
	_crtBreakAlloc = 0;
#	endif
	signal(SIGINT,  SIGHandler);
	signal(SIGSEGV, SIGHandler);
	signal(SIGTERM, SIGHandler);
#endif
	HLIB h = 0;
	IProtocol* pProtocol   = NULL;
	IComDevice* pComDevice = NULL;

#ifdef WIN32
	CONSOLE_CURSOR_INFO ConCurInf;

	hOut = GetStdHandle(STD_OUTPUT_HANDLE);
	hIn  = GetStdHandle(STD_INPUT_HANDLE);

	ConCurInf.dwSize = 40;
	ConCurInf.bVisible = TRUE;
	SetConsoleCursorInfo(hOut, &ConCurInf);
#endif

	int nRet = -1;

	if(argc <= 7)
	{
		if(argc == 2)
		{
			if(strcmp(argv[1], "-devices") == 0)
			{
				ShowAvailableDevices();
				return 0;
			}
			if(LoadProtocol(h, argv[1], pProtocol, TRUE) != -1)
			{
				int nCpt;
				int* pSpeeds = pProtocol->GetAvailableSpeeds(nCpt);
				if(nCpt < 2)
				{
					cout << nCpt << " " << STR_SUPPORTED_SPEED;
				}
				else
				{
					cout << nCpt << " " << STR_SUPPORTED_SPEEDS;
				}
				cout << " " << STR_BY_THIS_PROTOCOL << endl;
				for(int i = 0; i < nCpt; i++)
				{
					cout << i << " => " << pSpeeds[i] << " bps" << endl;
				}
				DeleteProtocol(h, pProtocol);
				FreeProtocol(h);
				return 0;
			}
			return -1;
		}
		ShowHelp();
		return argc > 1;
	}
	else
	{
		if(LoadProtocol(h, argv[1], pProtocol) == -1) {
			FreeProtocol(h);
			return -1;
		}
		int nDevice = atoi(argv[2]);
		int nPort   = atoi(argv[3]);
		int nCable  = atoi(argv[4]);
		int nSpeed  = atoi(argv[5]);

		IComDevice* pComDevice = CF100Devices::GetComDevice(nDevice);

		if(pComDevice == NULL)
		{
			ShowError(STR_DEVICE_ERROR);
			DeleteProtocol(h, pProtocol);
			FreeProtocol(h);
			return -1;
		}

		int nNbCable;
		int nNbPort  = pComDevice->GetNbPort();
		const char** cables = pComDevice->GetCables(nNbCable);

		if(nPort >= nNbPort)
		{
			nPort = 0;
			ShowWarning(STR_WNG_BAD_PORT_NUM);
		}
		if(nCable >= nNbCable)
		{
			nCable = 0;
			ShowWarning(STR_WNG_BAD_CABLE_NUM);
		}

		int nCpt;
		int* pSpeeds = pProtocol->GetAvailableSpeeds(nCpt);
		if(nSpeed >= nCpt)
		{
			ShowWarning(STR_WNG_BAD_SPEED);
			nSpeed = nCpt - 1;
		}
		pComDevice->Init(nPort, nCable);

		char sPort[128];
		pComDevice->GetPortName(nPort, sPort, sizeof(sPort));
		cout << STR_INIT_PORT << " " << sPort << STR_TO << pSpeeds[nSpeed] << " bps..." << flush;
		if(pProtocol->InitCom(pComDevice, nSpeed) <= 0)
		{
			ShowFailed();
			ShowProtocolError(pProtocol);
			DeleteProtocol(h, pProtocol);
			return -1;
		}
		ShowOK();

		int nCount = 6;		
		nRet = 0;
		while(nRet == 0 && !bDoExit && nCount + 1 < argc)
		{
			char* sOrder   = argv[nCount];
			char* sPath    = argv[nCount + 1];
			if(strnicmp(sOrder, "Flash", 5) == 0)
			{
				if(pProtocol->SetTransfertMode(IProtocol::TR_FLASH) == 1)
				{
					unsigned int nDrive = sOrder[5] - '0';
					if(nDrive > 6)
					{
						ShowError(STR_INVALID_DRIVE);
						nRet = -1;
					}
					else
					{
						nRet = SendFlash(pProtocol, nDrive, sPath);
					}
				}
				else
				{
					ShowProtocolError(pProtocol);
					nRet = -1;
				}
			}
			else if(strnicmp(sOrder, "RFlash", 6) == 0)
			{
				if(pProtocol->SetTransfertMode(IProtocol::TR_FLASH) == 1)
				{
					unsigned int nDrive = sOrder[6] - '0';
					if(nDrive > 6)
					{
						ShowError(STR_INVALID_DRIVE);
						nRet = -1;
					}
					else
					{
						nRet = ReceiveFlash(pProtocol, nDrive, sPath);
					}
				}
				else
				{
					ShowProtocolError(pProtocol);
					nRet = -1;
				}
			}
			else
			{
				if(strnicmp(sOrder, "RAM", 3) == 0)
				{
					if(sOrder[3] == '\0')
					{
						if(pProtocol->SetTransfertMode(IProtocol::TR_RECEIVERAM) == 1)
						{
							nRet = ReceiveRAMFiles(pProtocol, sPath);
						}
						else
						{
							ShowProtocolError(pProtocol);
							nRet = -1;
						}
					}
					else
					{
						if(pProtocol->SetTransfertMode(IProtocol::TR_RECEIVERAM) == 1)
						{
							unsigned int nZone = sOrder[3] - '0';
							if(sOrder[4] != '\0')
							{
								nZone = nZone * 10 + sOrder[4] - '0';
							}
							if(nZone == 0 || nZone > 12)
							{
								ShowError(STR_INVALID_ZONE_NUM);
								nRet = -1;
							}
							else
							{
								FileList filelist;
								GetFileList(sPath, filelist);
								FileList::iterator it  = filelist.begin();
								FileList::iterator end = filelist.end();
								for(; it != end && nRet == 0 && !bDoExit; ++it)
								{
									cout << (*it).c_str() << endl;
									nRet = SendRAMFile(pProtocol, nZone, (*it).c_str());
								}
								
							}
						}
						else
						{
							ShowProtocolError(pProtocol);
							nRet = -1;
						}
					}
				}
				else
				{
					ShowError(STR_UNKNOWORDER);
					nRet = -1;
				}
			}
			nCount += 2;
		}
		pProtocol->CloseCom(nRet == 0);
		DeleteProtocol(h, pProtocol);
		FreeProtocol(h);
	}
	return nRet;
}

//--------------------------------------------------------------------
void ShowText(const char* msg, WORD attribute)
{
#ifdef WIN32
	CONSOLE_SCREEN_BUFFER_INFO infos;
	GetConsoleScreenBufferInfo(hOut, &infos);
	SetConsoleTextAttribute(hOut, attribute);
	cout << msg << flush;
	SetConsoleTextAttribute(hOut, infos.wAttributes);
#else
	cout << "\e[1;" << attribute << "m" << msg << "\e[0m" << flush;
#endif
}

//--------------------------------------------------------------------
void ShowCanceled()
{
	ShowInformation(STR_CANCELED);
}

//--------------------------------------------------------------------
void ShowWarning(const char* msg)
{
	ShowText(STR_WARNING, COLOR_WARNING);
	ShowText(msg, COLOR_WARNING);
	cout << endl;
}

//--------------------------------------------------------------------
void ShowError(const char* msg)
{
	ShowText(STR_ERROR, COLOR_ERROR);
	ShowText(msg, COLOR_ERROR);
	cout << endl;
}

//--------------------------------------------------------------------
void ShowInformation(const char* msg)
{
	ShowText(msg, COLOR_INFO);
	cout << endl;
}


//--------------------------------------------------------------------
void ShowOK()
{
	ShowText(OK, COLOR_OK);
	cout << endl;
}

//--------------------------------------------------------------------
void ShowFailed()
{
	ShowText(ECHEC, COLOR_ERROR);
	cout << endl;
}

//--------------------------------------------------------------------
void ShowTextCenter(const char* sText, WORD attribute)
{
#ifdef WIN32
	CONSOLE_SCREEN_BUFFER_INFO infos;
	GetConsoleScreenBufferInfo(hOut, &infos);

	unsigned int nWidth = infos.dwSize.X;

	unsigned int nTextLength = strlen(sText);
	if(infos.dwCursorPosition.X != 0)
	{
		cout << endl;
		infos.dwCursorPosition.Y++;
	}
	int nPos = 0;
	if(nTextLength < nWidth)
	{
		nPos = (nWidth - nTextLength) >> 1;
	}
	infos.dwCursorPosition.X = nPos;
	SetConsoleCursorPosition(hOut, infos.dwCursorPosition);
#else
#endif
	ShowText(sText, attribute);
	cout << endl;
}

//--------------------------------------------------------------------
void ShowHelp()
{
	char buf[512];
	sprintf(buf, "Flash100 Console v%s\t%s", m_sVersion, __DATE__);
	ShowTextCenter(buf, COLOR_INFO);
	ShowTextCenter("Copyright (C) 2004-2005, Quaranta David (david.quaranta@laposte.net)", COLOR_INFO);

	ShowText(STR_PRESENTATION, COLOR_INFO);
	cout << STR_HELP << endl;
	ShowTextCenter(STR_THANKS, COLOR_INFO);
}

//--------------------------------------------------------------------
void ShowAvailableDevices()
{
	cout << STR_DEVICE_START << endl;
	IComDevice* pComDevice;
	int nMax = CF100Devices::GetNbComDevices();
	for(int i = 0; i < nMax; i++)
	{
		cout << i << ": ";
		ShowText(CF100Devices::GetComDeviceName(i), COLOR_INFO);
		cout << endl;
		pComDevice = CF100Devices::GetComDevice(i);
		if(pComDevice)
		{
			int nNbPort = pComDevice->GetNbPort();
			cout << STR_DEVICE_PORT;
			for(int nPort = 0; nPort < nNbPort; nPort++)
			{
				char buffer[128];
				pComDevice->GetPortName(nPort, buffer, sizeof(buffer));
				cout << buffer << ", ";
			}
			cout << endl;
			cout << STR_DEVICE_CABLE;
			int nNbCable;
			const char** pCables = pComDevice->GetCables(nNbCable);
			for(int nCable = 0; nCable < nNbCable; nCable++)
			{
				cout << pCables[nCable] << ", ";
			}
			cout << endl;
		}
		cout << STR_DEVICE_END << endl;
	}
}

//--------------------------------------------------------------------
void ShowProtocolError(IProtocol* pProtocol)
{
	if(pProtocol)
	{
		char* s = STR_UNKNOW_ERROR;
		switch(pProtocol->GetLastError())
		{
		case IProtocol::OPENPORT:
			s = STR_NO_OPENPORT;
			break;
		case IProtocol::GETPORT:
			s = STR_NO_GETPORT;
			break;
		case IProtocol::SETPORT:
			s = STR_NO_SETPORT;
			break;
		case IProtocol::TIMEOUT:
			s = STR_TIME_OUT;
			break;
		case IProtocol::NOINIT:
			s = STR_NO_INIT;
			break;
		case IProtocol::NOSEND:
			s = STR_NO_SEND;
			break;
		case IProtocol::NOSUPPORTED:
			s = STR_NO_SUPPORTED;
			break;
		case IProtocol::INVALID_VALUE:
			s = STR_INVALID_VALUE;
			break;
		case IProtocol::CANT_CHANGE_TRANSFERT_MODE:
			s = STR_CANT_CHANGE_TRANSFERT_MODE;
			break;
		case IProtocol::BAD_CALL:
			s = STR_BAD_CALL;
			break;
		}
		ShowText(STR_PROTOCOL, COLOR_ERROR);
		ShowText(s, COLOR_ERROR);
		cout << endl;
	}
}

//--------------------------------------------------------------------
void* GetOp(HLIB& h, const char* name)
{
	void* ret = NULL;
#ifdef WIN32
	if(h)
	{
		ret = GetProcAddress(h, name);
	}
	if(ret == NULL)
#else
	if(h)
	{
		ret = dlsym(h, name);
	}
	if(dlerror() != NULL)
#endif

	{
		if(h)
		{
			char buf[1024];
			sprintf(buf, STR_ENTRY_POINT_NOT_FOUND, name); 
			ShowError(buf);
		}
		else
		{
			ShowError("Internal error : In function GetOp(const HLIB& h, const char* name) h == NULL");
		}
	}
	return ret;
}

//--------------------------------------------------------------------
IProtocol* CreateProtocol(HLIB& h)
{
	Protocol_Op pFunction = (Protocol_Op)GetOp(h, EP_PROTOCOL_REQUEST);
	if(pFunction)
	{
		return (IProtocol*)pFunction(OP_CREATEINSTANCE, NULL);
	}
	return NULL;
}

//--------------------------------------------------------------------
void DeleteProtocol(HLIB& h, IProtocol*& pProtocol)
{
	Protocol_Op pDelete = (Protocol_Op)GetOp(h, EP_PROTOCOL_REQUEST);
	if(pDelete)
	{
		pDelete(OP_DELETEINSTANCE, pProtocol);
	}
}

//--------------------------------------------------------------------
int LoadProtocol(HLIB& h, const char* sName, IProtocol*& pProtocol, bool bShowInfos)
{
	char path[512];
	char* pExt = strrchr(sName, '.');
	if(pExt) {
		sprintf(path, "%s/%s", PROTOCOL_DIR, sName);
	}
	else {
		sprintf(path, "%s/%s%s", PROTOCOL_DIR, sName, DLL_EXTENSION+1);
	}
#ifdef WIN32
	h = LoadLibrary(path);
	if(!h) {
		ShowError(STR_NO_LOAD_PROTOCOL);
		return -1;
	}
#else
	FreeProtocol(h);
	h = dlopen(path, RTLD_LAZY);
	if(h == NULL) {
		ShowError(STR_NO_LOAD_PROTOCOL);
		return -1;
	}
#endif
	Protocol_Op pFunction = (Protocol_Op)GetOp(h, EP_PROTOCOL_REQUEST);
	if(!pFunction) {
		FreeProtocol(h);
		return -1;
	}
	const char* pName;
	const char* pVersion;
	const char* pAuthor;
	const char* pBuildDate;
	pName      = pFunction(OP_TOOLNAME, NULL);
	pVersion   = pFunction(OP_VERSION, NULL);
	pAuthor    = pFunction(OP_AUTHOR, NULL);
	pBuildDate = pFunction(OP_BUILDDATE, NULL);

	char buf[512];
	sprintf(buf, STR_PROTOCOL_LOAD_SUCCESS, pName, pVersion);
	ShowText(buf, COLOR_OK);
	cout << endl;

	pProtocol = CreateProtocol(h);
	if(pProtocol == NULL) {
		return -1;
	}
	if(bShowInfos)
	{
		cout << STR_INFO_START   << endl
			 << STR_INFO_NAME    << pName      << endl
			 << STR_CALC_TYPE    << GetCalcType(pProtocol->GetCasioType()) << endl
			 << STR_INFO_VERSION << pVersion   << endl
			 << STR_INFO_AUTHOR  << pAuthor    << endl
			 << STR_INFO_DATE    << pBuildDate << endl
			 << STR_INFO_END     << endl;
	}
	return 1;
}

//--------------------------------------------------------------------
const char* GetCalcType(CasioType ct)
{
	switch(ct)
	{
	case CT_G100_LIKE:
		return STR_G100_LIKE;
	case CT_GRAPH_LIKE:
		return STR_GRAPH_LIKE;
	case CT_CLASSPAD:
		return STR_CLASSPAD;
	default:
		return STR_CALC_UNKNOW;
	}
}

//--------------------------------------------------------------------
void DisplayLastError()
{
#ifdef WIN32
	LPVOID lpMsgBuf;
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL
		);
	cout << (LPCTSTR)lpMsgBuf << endl;
	// Free the buffer.
	LocalFree(lpMsgBuf);
#endif
}

//--------------------------------------------------------------------
void SIGHandler(int sig)
{
	bDoExit = true;
}

//--------------------------------------------------------------------
void FreeProtocol(HLIB& h)
{
#ifdef WIN32
	FreeLibrary(h);
#else
	if(h)
	{
		dlclose(h);
		h = NULL;
	}
#endif
}

//--------------------------------------------------------------------
void WritePercent(unsigned short nPercent)
{
	assert(nPercent <= 100);
#ifdef WIN32
	static CONSOLE_SCREEN_BUFFER_INFO infos;
	GetConsoleScreenBufferInfo(hOut, &infos);
	cout << nPercent << "%" << flush;
	SetConsoleCursorPosition(hOut, infos.dwCursorPosition);
#else
	int nWrite = fprintf(hOut, "%i%%", nPercent);
	fflush(hOut);
	for(int i = 0; i < nWrite; i++)
	{
		fprintf(hOut, "\10");
	}
#endif
}

//--------------------------------------------------------------------
string GetPath(const char* sFile)
{
	string s(sFile);
	unsigned int nPos = s.rfind('/');
	if(nPos == string::npos)
	{
		nPos = s.rfind('\\');
	}
	if(nPos == string::npos)
	{
		s.resize(0);
	}
	else
	{
		s.resize(nPos+1);
	}
	return s;
}

//--------------------------------------------------------------------
void GetFileList(const char* sPath, FileList& filelist)
{
	filelist.clear();
	if(sPath)
	{
		string path = GetPath(sPath);
#ifdef WIN32
		WIN32_FIND_DATA find_data;
		HANDLE h = FindFirstFile(sPath, &find_data);
		if(h != INVALID_HANDLE_VALUE)
		{
			do
			{
				if((find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY)
				{
					string filepath = path;
					filepath += string(find_data.cFileName);
					filelist.push_back(filepath);
				}
			}while(FindNextFile(h, &find_data));
		}
		FindClose(h);
#else
#endif
	}
}
