
#include "CasioG100.h"

#include <APIFunction.h>

static unsigned char close_com[] = {58,69,78,68,49,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,26};


//-----------------------------------------------------------------------------------
CasioG100Protocol::CasioG100Protocol()
 : CProtocol()
{
	static int nSpeeds[] = {38400};
	SetAvailableSpeeds(nSpeeds, 1);
	bCommInit = false;
}

//-----------------------------------------------------------------------------------
void CasioG100Protocol::CloseCom(int nStatus)
{
	bCommInit = false;
	if(nStatus > 0)
	{
		if(GetTransfertMode() == TR_FLASH)
		{
			Wait(400);
			Send(close_com, 40, 2000);
			Wait(200);
		}
	}
	CProtocol::CloseCom(nStatus);
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::ImportDrive(int nLect)
{
	Wait(400);
	unsigned char chaine[] = {0x3A, 0x52, 0x45, 0x51, 0x31, 0x49, 0x4E, 0x46, 0x31, 0xFF,
			 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
			 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
			 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF7};
	if(nLect)
	{
		chaine[5] = 'F';
		chaine[6] = 'R';
		chaine[7] = '0';
		chaine[8] = nLect + 48;
		chaine[39] = 13 - nLect;
	}
	Send(chaine, 40, 2000);
	if(!Receive((unsigned char*)chaine, 40, 4000))
	{
		SetError(TIMEOUT);
		return -1;
	}
	if(SendStart() == 1)
	{
		return 131072;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::ExportDrive(int nLect, const void* pBuffer)
{
	Wait(400);
	unsigned char chaine[] = {58,65,68,78,49,73,78,70,49,0,0,4,0,
		0,0,0,128,0,2,0,0,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,122};
	if(nLect)
	{
		chaine[5] = 'F';
		chaine[6] = 'R';
		chaine[7] = '0';
		chaine[8] = nLect + 48;
		chaine[39] = 144 - nLect;
	}

	Send(chaine, 40, 2000);

	unsigned char buffer;
	if(!Receive(&buffer, 1, 10000))
	{
		SetError(TIMEOUT);
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SendAck()
{
	unsigned char buf = 6;
	if(Send(&buf, 1, 1500)) {
		return 1;
	}
	SetError(TIMEOUT);
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SendStart()
{
	unsigned char buf;
	if(SendAck() == 1 && Receive(&buf, 1)) {
		return 1;
	}
	SetError(TIMEOUT);
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::ReceiveData(void* buffer, unsigned int nRemainBytes)
{
	switch(GetTransfertMode())
	{
	case TR_FLASH:		
		{
			unsigned int nReceive = 2048;
			char buf;
			if(nRemainBytes < nReceive)
			{
				nReceive = nReceive;
			}
			if(Receive(buffer, nReceive, 1500)
				&& (nReceive != nRemainBytes || Receive(&buf, 1, 1500)))
			{
				return nReceive;
			}
			SetError(TIMEOUT);
			return -1;
		}
	case TR_RECEIVERAM:
		{
			unsigned int nStep = 2048;
			bool bReadChecksum = false;
			if(nRemainBytes <= nStep)
			{
				nStep = nRemainBytes;
				bReadChecksum = true;
			}
			if(Receive(buffer, nStep, 1500))
			{
				if(bReadChecksum)
				{
					char buf;
					Receive(&buf, 1, 1500);
					SendAck();
				}
				return nStep;
			}
			SetError(TIMEOUT);
			return -1;
		}
	}
	SetError(BAD_CALL);
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SendData(const void* buffer, unsigned int nRemainBytes)
{
	unsigned char outBuf;
	unsigned char sum;
	static const unsigned int nSend = 1024;
	if(nRemainBytes < nSend)
	{
		SetError(BAD_CALL);
		return -1;
	}
	unsigned char* buf = (unsigned char*)buffer;
	
	outBuf = 58;
	Send(&outBuf, 1);
	sum = 0;
	for(unsigned int i = 0; i < nSend; i++)
	{
		sum += buf[i];
	}
	outBuf = 256 - sum;
	
	if(Send(buf, nSend))
	{
		Send(&outBuf, 1);
		Receive(&outBuf, 1, 3000);
		if(outBuf != 6)
		{
			SetError(NOSEND);
			return -1;
		}
		return nSend;
	}
	SetError(NOSEND);
	return -1;
}

//-----------------------------------------------------------------------------------
unsigned int CasioG100Protocol::GetUsefulSize(const void* pBuffer)
{
	return 131072;
}

//-----------------------------------------------------------------------------------
unsigned int CasioG100Protocol::GetSizeOfDrives(bool bSystem, bool bDriveL, bool bDriveM, bool bDriveN,
			bool bDriveO, bool bDriveP, bool bDriveQ)
{
	return 131072 * bSystem + 131072 * bDriveL + 131072 * bDriveM + 131072 * bDriveN
		+ 131072 * bDriveO + 131072 * bDriveP + 131072 * bDriveQ;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SendRAMFile(unsigned short nZone, const char* name, unsigned int nSize, const char* password)
{
	SetError(NOSUPPORTED);
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::WaitRAMFileConfirmation(unsigned short nZone, const char* password)
{
	SetError(NOSUPPORTED);
	return -2;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::ReceiveRAMFile(unsigned short& nZone, char* name, char* password)
{
	unsigned char buffer[40];
	if(bCommInit == false)
	{
		char buf;
		//Attend le premier packet (cas particulier)
		if(Receive(&buf, 1, 300) == 0) {
			return -3;
		}
		if(buf != 0x16)	{
			return -3;
		}
		//Envoi la confirmation
		buf = 0x13;
		if(!Send(&buf, 1, 2000)
			|| !Receive(buffer, 40, 2000)
			|| !Send(buffer, 40, 2000)
			|| !Receive(&buf, 1, 2000))
		{
			SetError(TIMEOUT);
			return -1;
		}
		if(buf != 0x6)
		{
			SetError(INVALID_VALUE);
			return -1;
		}
		if(SendAck() == -1)
		{
			return -1;
		}
		bCommInit = true;
	}
	
	if(bCommInit)
	{
		static char header[] = ":MCS1";
		//Recoit le fichier
		if(!Receive(buffer, 40, 300))
		{
			return -3;
		}
		//Analyse le packet reu pour voir si c le packet de fin
		if(memcmp(buffer, close_com, sizeof(buffer)) == 0)
		{
			return -2;
		}
		if(memcmp(buffer, header, sizeof(header)) == 0)
		{
			int nSize = ((buffer[6] * 256 + buffer[7]) * 256 + buffer[8]) * 256 + buffer[9];

			if(buffer[10] >= 0 && buffer[10] <= 11)
			{
				memset(name, 0, 9);
				for(int i = 0; i < 8 && buffer[i+11] != 0xFF; i++)
				{
					name[i] = buffer[i+11];
				}
				nZone = buffer[10];
				if(nZone == 1)
				{
					return nSize - 10;
				}
				if(nSize)
				{
					return nSize;
				}
				if(SendAck() == -1)
				{
					return -1;
				}
				return -3;
			}
			else
			{ //Zap ce type de fichier
				char buf;
				SendAck();
				if(nSize)
				{
					nSize+=2;//checksum et 3Ah
				}
				while(nSize && Receive(&buf, 1, 1000))
				{
					nSize--;
				}
				SendAck();
				return -3;
			}
		}
		else
		{
			SendAck();
			return -3;
		}
	}
	SetError(NOSUPPORTED);
	return -1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SendRAMFileConfirmation(bool bOk, unsigned short nZone, char* password)
{
	char buffer[11];
	int nRead;
	if(nZone == 1) {
		nRead = 11;
	}
	else {
		nRead = 1;
	}

	if(SendAck() == -1
		|| !Receive(buffer, nRead, 1000)) {
		SetError(TIMEOUT);
		return -1;
	}
	if(nZone == 1 && password) {
		memset(password, 0, 9);
		for(int i = 0; i < 8 && buffer[i+1] != 0xFF; i++) {
			password[i] = buffer[i+1];
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CasioG100Protocol::SetTransfertMode(TRANSFERT nTransfert)
{
	if(GetTransfertMode() == nTransfert)
	{
		//Rien  changer
		return 1;
	}
	if(GetTransfertMode() != TR_NONE)
	{
		//Rend tout changement impossible
		SetError(CANT_CHANGE_TRANSFERT_MODE);
		return -1;
	}
	if(CProtocol::SetTransfertMode(nTransfert) < 1)
	{
		return -1;
	}
	if(nTransfert == TR_FLASH)
	{
		unsigned char Buf[40];
		static unsigned char Chaine[] = {58,77,68,76,49,90,88,57,52,53,255,48,
			51,56,52,48,48,78,49,46,48,48,0,0,16,0,0,0,4,0,0,0,1,0,48,120,48,
			55,255,64};
		
		unsigned char nHello = 0x16;
		
		Send(&nHello, 1, 4000);
		
		Receive(&nHello, 1);
		
		if(nHello != 19)
		{
			SetError(NOINIT);
			return -1;
		}

		Send(Chaine, 40);
		if(!Receive(Buf, 40))
		{
			SetError(TIMEOUT);
			return -1;
		}
		return SendStart();
	}
	if(nTransfert == TR_SENDRAM)
	{
		SetError(NOSUPPORTED);
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
CasioType CasioG100Protocol::GetCasioType() const
{
	return CT_G100_LIKE;
}
