
#include "GComm.h"

#include <APIFunction.h>

#define PACKETSIZE		1024

/** Valeur retourne par SendPacket en cas de succes */
#define GLINK_OK 2

/** Valeurs retournes par SendPacket et ReceivePacket en cas d'erreur */
#define GLINK_TIMEOUT      -1 //Temps d'attente coul
#define GLINK_HEADER_ERROR -2 //La rponse du destinataire n'est pas correcte
#define GLINK_ARQ_TIMEOUT  -3 //Temps d'attente de l'accus de rception dpass
#define GLINK_ARQ_ERROR    -4 //La valeur recue pour l'accus de rception est mauvaise
#define GLINK_MAX_RESEND   -5 //Le nombre maximal de renvoi du packet a t atteint
#define GLINK_BADSIZE      -6 //Valeur retourne par SendPacket pour signaler un taille trop grande (ie. > MAX_PACKETSIZE)

#define GLINK_HEADER   3
#define GLINK_RESEND   1
#define GLINK_FINISHED 2

#define TRANSFERT_RAMFILE		128
#define TRANSFERT_GO			2
#define TRANSFERT_STOP			3
#define TRANSFERT_WAIT			4
#define TRANSFERT_IMPORTDRIVE	5
#define TRANSFERT_EXPORTDRIVE	6
#define TRANSFERT_GETSIZE		7
#define TRANSFERT_EXIT			50

#if GCOMM_NUM_VERSION < 110
#	pragma pack(1) //Alignement des structures sur 1 octet
	struct THeader{
		unsigned char type;
		unsigned long size;
		char name[9];
	};
	struct GLinkHeader{
		unsigned short size;
		unsigned char  checksum;
	};
#	define GCOMM_MAX_PACKETSIZE 1024
#	define GCOMM_DATA_SIZE       512
#else
	struct THeader{
		unsigned char type;
		char name[9];
		unsigned short size;
	};
	struct GLinkHeader{
		unsigned short nSize;
		unsigned short nChecksum;
	};
#	define MAX_PACKETSIZE       32767
#	define GCOMM_MAX_PACKETSIZE  2048
#	define GCOMM_DATA_SIZE         16
#endif

//-----------------------------------------------------------------------------------
CGComm::CGComm()
:CProtocol()
{
	static int nSpeeds[] = {14400, 38400, 57600, 115200};
	SetAvailableSpeeds(nSpeeds, 4);
}

//-----------------------------------------------------------------------------------
void CGComm::CloseCom(int nStatus)
{
	if(nStatus > 0)
	{
		THeader header;
		header.type = TRANSFERT_EXIT;
		SendPacket(&header, sizeof(header));
	}
	CProtocol::CloseCom(nStatus);
}

//-----------------------------------------------------------------------------------
int CGComm::ImportDrive(int nLect)
{
	THeader header;
	header.type = TRANSFERT_EXPORTDRIVE;
	header.name[0] = nLect;
	//Demande l'importation
	if(SendPacket(&header, sizeof(header), 2000) <= 0)
	{
		return -1;
	}

	//Attend le retour, pour savoir la taille du lecteur
	if(ReceivePacket(&header, sizeof(header), 2000) <= 0)
	{
		return -1;
	}
	//Demande le dbut du transfert
	header.type = TRANSFERT_GO;
	if(SendPacket(&header, 1) <= 0)
	{
		return -1;
	}
	if(header.size > 131072 / GCOMM_DATA_SIZE)
	{//La taille spcifie est suprieur  un lecteur
		SetError(INVALID_VALUE);
		return -1;
	}
	return header.size * GCOMM_DATA_SIZE;
}

//-----------------------------------------------------------------------------------
int CGComm::ExportDrive(int nLect, const void* pBuffer)
{
	THeader header;
	header.type = TRANSFERT_IMPORTDRIVE;
	header.name[0] = nLect;
	header.size = GetUsefulSize(pBuffer) / GCOMM_DATA_SIZE;
	if(SendPacket(&header, sizeof(header), 1500) <= 0)
	{
		return -1;
	}
	
	if(ReceivePacket(&header, sizeof(header), 5500) <= 0)
	{
		return -1;
	}
	if(header.type == TRANSFERT_GO)
	{
		return 1;
	}
	SetError(INVALID_VALUE);
	return -1;
}

#if GCOMM_NUM_VERSION < 110
//-----------------------------------------------------------------------------------
int CGComm::SendPacket(const void* buffer, unsigned int nNbBytes, int time_out)
{
	GLinkHeader header;
	header.size = PACKETSIZE;
	char val;
	char code = GLINK_HEADER;
	unsigned char* buf = (unsigned char*)buffer;
	int count;
	while(nNbBytes)
	{
		count = 0;
		if(nNbBytes < header.size)
		{
			header.size = nNbBytes;
		}
		header.checksum = 0;
		for(unsigned int i = 0; i < header.size; i++)
		{
			header.checksum += buf[i];
		}
send:
		count++;
		Send(&code, 1, time_out);
		if(!Receive(&code, 1, time_out))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		if(code != GLINK_HEADER)
		{
			SetError(INVALID_VALUE);
			return GLINK_HEADER_ERROR;
		}
		Send(&header, sizeof(header));
		Send(buf, header.size);
		if(!Receive(&val, 1, 1000)) //Attend l'accus de rception
		{
			SetError(TIMEOUT);
			return GLINK_ARQ_TIMEOUT;
		}
		else
		{
			if(val == GLINK_RESEND)
			{
				//D'apres le recepteur les infos recues ne sont pas correctes
				//Renvoi les infos
				if(count == 5)
				{
					SetError(TIMEOUT);
					return GLINK_MAX_RESEND;
				}
				goto send;
			}
			if(val != GLINK_OK)
			{
				//L'octet recu n'est pas valide
				//Attend la fin de l'envoi des octets
				SetError(INVALID_VALUE);
				return GLINK_ARQ_ERROR;
			}
		}
		nNbBytes -= header.size;
		buf      += header.size;
	}
	return GLINK_OK;
}

//-----------------------------------------------------------------------------------
int CGComm::ReceivePacket(void* buffer, unsigned int nMaxBytes, int time_out)
{
	unsigned char mychecksum;
	unsigned int i;
	GLinkHeader header;
	char val;
	char code;
	unsigned char* buf = (unsigned char*)buffer;
	int result = 1;
	unsigned short nSize;
	while(result)
	{
		code = 0;
		if(!Receive(&code, 1, time_out))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		if(code != GLINK_HEADER)
		{
			SetError(INVALID_VALUE);
			return GLINK_HEADER_ERROR;
		}
		Send(&code, 1);
		if(!Receive(&header, sizeof(header), 500))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		nSize = header.size;
		result = nSize > PACKETSIZE || nSize > nMaxBytes || !Receive(buf, header.size, 600);
		if(!result)
		{
			//Verifie le checksum si on a recu les donnees
			mychecksum = 0;
			for(i = 0; i < header.size; i++)
			{
				mychecksum += buf[i];
			}
			result = (mychecksum != header.checksum);
		}

		val = GLINK_OK;//Reception valide
		if(result)
		{
			val = GLINK_RESEND;//Reception invalide
		}
		Send(&val, 1);//Envoie l'accuse de reception
	}
	return nSize;
}
#else

//-----------------------------------------------------------------------------------
unsigned short CGComm::CalcCheckSum(const unsigned char* buffer, unsigned short nSize)
{
	unsigned short nCheckSum = 0;
	for(unsigned short i = 0; i < nSize; i++)
	{
		nCheckSum ^= buffer[i];
	}
	return nCheckSum;
}

//-----------------------------------------------------------------------------------
int CGComm::SendPacket(const void* buffer, unsigned int nNbBytes, int time_out)
{
	GLinkHeader header;
	char code = GLINK_HEADER;
	char val;
	if(nNbBytes > MAX_PACKETSIZE)
	{
		SetError(INVALID_VALUE);
		return GLINK_BADSIZE;
	}
	int count = 0;
	header.nSize = nNbBytes;
	header.nChecksum = CalcCheckSum((unsigned char*)buffer, nNbBytes);
	do
	{
		if(count == 5)
		{
			SetError(TIMEOUT);
			return GLINK_MAX_RESEND;
		}
		count++;
		Send(&code, 1);	//Envoi l'identifiant
		//Attend la confirmation
		if(!Receive(&code, 1, time_out))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		if(code != GLINK_HEADER) //Si l'octet recu n'est pas valide erreur
		{
			SetError(INVALID_VALUE);
			return GLINK_HEADER_ERROR;
		}
		Send(&header, sizeof(header));//Envoi le header
		Send(buffer, header.nSize);   //Envoi les infos
		if(!Receive(&val, 1, 1000))  //Attend l'accuse de reception.
		{
			SetError(TIMEOUT);
			return GLINK_ARQ_TIMEOUT;
		}

	}while(val == GLINK_RESEND);
	if(val != GLINK_FINISHED)
	{
		SetError(INVALID_VALUE);
		return GLINK_ARQ_ERROR;
	}
	return GLINK_OK;
}

//-----------------------------------------------------------------------------------
int CGComm::ReceivePacket(void* buffer, unsigned int nMaxBytes, int time_out)
{
	GLinkHeader header;
	int result;
	char code;
	do
	{
		if(!Receive(&code, 1, time_out))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		if(code != GLINK_HEADER)
		{
			SetError(INVALID_VALUE);
			return GLINK_HEADER_ERROR;
		}
		Send(&code, 1);//Envoi la confirmation de dpart
		if(!Receive(&header, sizeof(header), 500))
		{
			SetError(TIMEOUT);
			return GLINK_TIMEOUT;
		}
		result = header.nSize > MAX_PACKETSIZE
				|| header.nSize > nMaxBytes
				|| !Receive(buffer, header.nSize, 500);
		if(!result) //Verifie le packet recu par l'intermediaire du checksum
		{
			result = header.nChecksum - CalcCheckSum((unsigned char*)buffer, header.nSize);
		}

		code = GLINK_FINISHED;//Reception valide
		if(result)
		{
			code = GLINK_RESEND;//Reception invalide
		}
		Send(&code, 1);//Envoie l'accuse de reception
	}while(result);
	return header.nSize;
}
#endif

//-----------------------------------------------------------------------------------
int CGComm::SendRAMFile(unsigned short nZone, const char* name, unsigned int nSize, const char* password)
{	
	THeader header;
	header.type = TRANSFERT_RAMFILE + nZone;
	header.size = nSize;
	memset(header.name, 0, sizeof(header.name));
	for(int i = 0; i < 8 && name[0] != '\0'; i++)
	{
		header.name[i] = name[i];
	}
	
	if(SendPacket(&header, sizeof(header)) <= 0)
	{
		SetError(TIMEOUT);
		return -1;
	}
#if GCOMM_NUM_VERSION == 100
	//Envoi la taille reelle
	unsigned short realsize = nSize + 14;
	if(nZone == 1)
	{
		realsize += 10;
	}
	if(SendPacket(&realsize, sizeof(realsize)) <= 0)
	{
		SetError(TIMEOUT);
		return -1;
	}
#endif
	return 1;
}

//-----------------------------------------------------------------------------------
int CGComm::WaitRAMFileConfirmation(unsigned short nZone, const char* password)
{
	THeader header;
	if(ReceivePacket(&header, sizeof(header), 100) == GLINK_TIMEOUT)
	{
		return 0;
	}
	if(header.type == TRANSFERT_GO)
	{
		if(nZone == 1)
		{
			if(password)
			{
				if(SendPacket((void*)password, 8) <= 0)
				{
					return -2;
				}
			}
			else
			{
				char tmp[8];
				memset(tmp, 0, 8);
				if(SendPacket(tmp, 8) <= 0)
				{
					return -2;
				}
			}
		}
		return 1;
	}
	else
	{
		SetError(INVALID_VALUE);
		return -1;
	}
}

//-----------------------------------------------------------------------------------
int CGComm::ReceiveRAMFile(unsigned short& nZone, char* name, char* password)
{
	THeader header;
	int nRet = ReceivePacket(&header, sizeof(header), 200);
	if(nRet < 1)
	{
		return -3;//Rien de recu
	}
	if(nRet != sizeof(header))
	{
		SetError(INVALID_VALUE);
		return -1;
	}
	if(header.type == TRANSFERT_EXIT)
	{
		return -2;
	}
	nZone = header.type&15;
	if(!(header.type & TRANSFERT_RAMFILE))
	{
		SetError(INVALID_VALUE);
		return -1;
	}
	strcpy(name, header.name);
#if GCOMM_NUM_VERSION == 100
	//Recoit la taille reelle
	unsigned short nRealsize;
	if(ReceivePacket(&nRealsize, sizeof(nRealsize), 500) != sizeof(nRealsize))
	{
		SetError(TIMEOUT);
		return -1;
	}
#endif
	return header.size;
}

//-----------------------------------------------------------------------------------
int CGComm::SendRAMFileConfirmation(bool bOk, unsigned short nZone, char* password)
{
	THeader header;
	if(bOk)
	{
		header.type = TRANSFERT_GO;
	}
	else
	{
		header.type = TRANSFERT_STOP;
	}
	int nRet = SendPacket(&header, sizeof(header), 1500) > 0;
	if(nRet && bOk && nZone == 1)
	{
		nRet = ReceivePacket(header.name, 8, 1500) > 0;
		header.name[8] = '\0';
		if(nRet && password)
		{
			strcpy(password, header.name);
		}
	}
	return nRet;
}

#if GCOMM_NUM_VERSION > 101
//-----------------------------------------------------------------------------------
unsigned int CGComm::GetUsefulSize(const void* pBuffer)
{
	if(pBuffer == NULL)
	{
		return 8192;
	}
	unsigned int nCpt = 131071;
	for(; (nCpt > 0) && (((const unsigned char*)pBuffer)[nCpt] == 0xFF); nCpt--);
	int nNbSeg = 0;
	if(nCpt & 0xF)
	{
		nNbSeg++;
	}
	nNbSeg += (nCpt >> 4); //Divis par 16
	return nNbSeg << 4;
}
#else
//-----------------------------------------------------------------------------------
unsigned int CGComm::GetUsefulSize(const void* pBuffer)
{
	unsigned int nCpt = 131071;
	for(; (nCpt > 0) && (((const unsigned char*)pBuffer)[nCpt] == 0xFF); nCpt--);
	int nNbSector = 0;
	if(nCpt & 511)
	{
		nNbSector++;
	}
	nNbSector += (nCpt >> 9); //Divis par 512
	return (nNbSector << 9);
}
#endif

//-----------------------------------------------------------------------------------
unsigned int CGComm::GetSizeOfDrives(bool bSystem, bool bDriveL, bool bDriveM, bool bDriveN,
			bool bDriveO, bool bDriveP, bool bDriveQ)
{
	THeader header;
	header.type = TRANSFERT_GETSIZE;
	header.size = bSystem + (bDriveL<<1) + (bDriveM<<2) + (bDriveN<<3) + (bDriveO<<4) + (bDriveP<<5) + (bDriveQ<<6);
	short size;
	if(SendPacket(&header, sizeof(header)) > 0 && ReceivePacket(&size, sizeof(size), 3000) > 0)
	{
		return size * GCOMM_DATA_SIZE;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
int CGComm::ReceiveData(void* buffer, unsigned int nRemainBytes)
{
	return ReceivePacket(buffer, nRemainBytes, 700);
}

//-----------------------------------------------------------------------------------
int CGComm::SendData(const void* buffer, unsigned int nRemainBytes)
{
	unsigned int nStep = GCOMM_MAX_PACKETSIZE;
	if(nRemainBytes < nStep)
	{
		nStep = nRemainBytes;
	}
	if(SendPacket(buffer, nStep, 700) > 0)
	{
		return nStep;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
CasioType CGComm::GetCasioType() const
{
	return CT_G100_LIKE;
}
