
#include "BackupWindow.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qpopupmenu.h>
#include <qlayout.h>

#include <language.h>
#include <FlashBackup.h>
#include <ROMDISKDrive.h>
#include <MessageBox.h>
#include <DriveAccess.h>
#include <CasioSystem.h>
#include <image_defs.h>
#include <BASICDrive.h>
#include <CasioBackup.h>
#include <CalcInfo.h>

#define TYPE_SYSTEM  1
#define TYPE_ROMDISK 2
#define TYPE_BASIC   3
#define TYPE_BACKUP  4

//-----------------------------------------------------------------------------------
BackupWindow::BackupWindow(CONSTRUCTOR_PARAM)
:CheckFileWindow(CALL_PARAM)
{
	resize(400, 220);

	setIcon(QPixmap::fromMimeSource(IMG_SEE));
	
	QWidget* pWidget = new QWidget(0);
	QGridLayout* pLayout = new QGridLayout(pWidget, 1, 1, 15);
	driveList = new QListView(pWidget);
	SetCentralWidget(pWidget);
	
	pLayout->addWidget(driveList, 0, 0);

	int i;
	for(i = 0; i < 3; i++)
	{
		driveList->addColumn(QString::null);
		driveList->header()->setClickEnabled(FALSE, i);
		driveList->header()->setResizeEnabled(TRUE, i);
	}
	driveList->setResizeMode(QListView::AllColumns);
	driveList->setColumnWidth(0, (int)(driveList->width() * 0.3f));
	driveList->setColumnWidth(1, (int)(driveList->width() * 0.2f));
	driveList->setColumnWidth(2, (int)(driveList->width() * 0.5f - 10));
	driveList->setSorting(-1);// dsactive le tri
	driveList->setAllColumnsShowFocus( TRUE );
	
	//Cr les lecteurs dans la liste
	QListViewItem* last = NULL;
	for(i = 0; i < 7; i++)
	{
		items[i] = new QListViewItem(driveList, last);
		last = items[i];
	}

	//Menu flottant
	popupmenu = new QPopupMenu(pWidget);
	for(i = 0; i < 4; i++)
	{
		popupmenu->insertItem(QString::null, i);
	}
	popupmenu->insertSeparator(1);
	popupmenu->insertSeparator(4);

	//Connexions
	connect(driveList, SIGNAL(mouseButtonPressed(int, QListViewItem *, const QPoint &, int )), SLOT(ListClicked( int, QListViewItem *, const QPoint &)));
	connect(driveList, SIGNAL(doubleClicked(QListViewItem*)), SLOT(ListDoubleClicked(QListViewItem*)));	
}

//-----------------------------------------------------------------------------------
int BackupWindow::Init(INIT_PARAM)
{
	DoInitWords();
	if(sFileName.isEmpty()) {
		return 1;
	}
	if(!SetFileName(sFileName, FALSE)) {
		return 0;
	}
	return DoReloadFile(false);
}

//-----------------------------------------------------------------------------------
void BackupWindow::DoInitWords()
{
	driveList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_DRIVE));
	driveList->setColumnText(1, tr("Type"));
	driveList->setColumnText(2, tr("Information"));
	
	//Met  jour la liste des noms
	items[0]->setText(0, CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM));
	char drive = 'L';
	for(int i = 1; i < 7; i++)
	{
		items[i]->setText(0, CLanguage::GetWordTranslation(CLanguage::WO_DRIVE) + " " + QChar(drive));
		drive++;
	}
	popupmenu->changeItem(0, tr("Editer"));
	popupmenu->changeItem(1, CLanguage::GetWordTranslation(CLanguage::WO_IMPORT));
	popupmenu->changeItem(2, CLanguage::GetWordTranslation(CLanguage::WO_EXPORT));
	popupmenu->changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_FORMAT));
	RefreshInfos();
}

//-----------------------------------------------------------------------------------
int BackupWindow::DoReloadFile(bool bShowValidErr)
{
	DoShowCaption();

	CFlashBackup backup;

	if(!backup.Open(m_sFileName, false))
	{
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_sFileName, true));
		return 0;
	}
	if(backup.IsValid() < 1)
	{
		if(bShowValidErr) {
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(m_sFileName));
		}
		return -1;
	}

	int i = 0;
	CCasioSystem system;
	CROMDISKDrive drive(TRUE);
	CCasioBackup cbackup(TRUE);
	CBASICDrive basic(TRUE);
	while(i < 7)
	{
		infos[i].nStart = backup.GetStartDrive(i);
		infos[i].type   = 0xFF;//Type 'Autre'
		infos[i].nPart  = 0;   //Aucune Partition
		if(infos[i].nStart == -1) {
			infos[i].nSize = 0;
			i++;
			continue;
		}
		if(infos[i].nStart < 0) {
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoReadFile(m_sFileName));
			return 0;
		}
		CDriveFile* pDrive = NULL;

		system.OpenDriveFile(m_sFileName.latin1(),  infos[i].nStart, false);
		drive.OpenDriveFile(m_sFileName.latin1(),   infos[i].nStart, false);
		cbackup.OpenDriveFile(m_sFileName.latin1(), infos[i].nStart, false);
		basic.OpenDriveFile(m_sFileName.latin1(),   infos[i].nStart, false);

		if(system.IsValid()) {
			infos[i].type  = TYPE_SYSTEM;
			infos[i].extra = system.GetSystemType();
			pDrive = &system;
		}
		else if(drive.IsValid()) {
			infos[i].type = TYPE_ROMDISK;
			pDrive = &drive;
		}
		else if(cbackup.IsValid()) {
			infos[i].type = TYPE_BACKUP;
			pDrive = &cbackup;
		}
		else if(basic.IsValid()) {
			infos[i].type = TYPE_BASIC;
			pDrive = &basic;
		}
		
		if(pDrive != NULL) {
			int nNbDrive   = pDrive->GetNumDrive();
			infos[i].nSize = pDrive->GetSizeInBytes();
			if(nNbDrive > 1) {
				infos[i].nPart = 1;
			}
			for(int j = i+1; j < i + nNbDrive; ++j) {
				infos[j].type  = infos[i].type;
				infos[j].extra = infos[i].extra;
				infos[j].nPart = infos[j-1].nPart + 1;
			}
			i += nNbDrive;	
		}
		else {
			infos[i].nSize = DRIVE_SIZE;
			i++;
		}
	}

	backup.Close();
	ResetTime();
	RefreshInfos();
	return 1;
}

//-----------------------------------------------------------------------------------
void BackupWindow::RefreshInfos()
{
	for(int i = 0; i < 7; i++)
	{
		QString s;
		switch(infos[i].type)
		{
		case TYPE_SYSTEM:
			s = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM) + " (" + CCalcInfo::GetCalcName((CasioModel)infos[i].extra) + ")";
			break;
		case TYPE_ROMDISK: 
			s = "Rom-Disk";
			break;
		case TYPE_BASIC:
			s = "Basic";
			break;
		case TYPE_BACKUP:
			s = "Backup";
			break;
		default:
			s = tr("Autre");
		}
		items[i]->setText(1, s);
		if(infos[i].nSize == 0)
		{
			s = tr("Lecteur vide");
		}
		else
		{
			if(infos[i].nPart)
			{
				s = tr("Partition n ") + QString::number(infos[i].nPart);
			}
			else
			{
				s = tr("Lecteur complet");
			}
		}
		items[i]->setText(2, s);
	}
}

//-----------------------------------------------------------------------------------
void BackupWindow::ListDoubleClicked(QListViewItem*)
{
	ShowPopup(0);
}

//-----------------------------------------------------------------------------------
void BackupWindow::ListClicked(int button, QListViewItem* i, const QPoint& p)
{
	if(button == RightButton)
	{
		bool bEnabledVisu   = FALSE;
		bool bEnabledFormat = FALSE;
		bool bEnabledExport = FALSE;
		if(i)
		{
			int n = 0;
			QListViewItem* j = driveList->firstChild();
			for(; j && j != i; j= j->nextSibling(), n++);
			if(j)
			{
				bEnabledVisu   = (infos[n].type != 0xFF);
				bEnabledFormat = (n != 0 && infos[n].nSize != 0);
			}
			bEnabledExport = infos[n].nSize;
		}
		popupmenu->setItemEnabled(0, bEnabledVisu);
		popupmenu->setItemEnabled(1, i);
		popupmenu->setItemEnabled(2, bEnabledExport);
		popupmenu->setItemEnabled(3, bEnabledFormat);
		ShowPopup(popupmenu->exec(p));
	}
}

//-----------------------------------------------------------------------------------
void BackupWindow::ShowPopup(int id)
{
	int n = 0;
	QListViewItem* i = driveList->selectedItem();
	QListViewItem* j = driveList->firstChild();
	for(; j && j != i; j = j->nextSibling(), n++);
	if(!j)
	{
		return;
	}
	switch(id)
	{
	case 0: //Edition
		{
			if(infos[n].nPart) {
				int k = n - (infos[n].nPart - 1); 
				if(infos[k].type == infos[n].type) {
					n = k;
				}
			}
			if(infos[n].type == TYPE_SYSTEM) {
				//Systme
				GetMainWindow()->OpenFileSystem(m_sFileName, infos[n].nStart, NULL);
			}
			else if(infos[n].type != 0xFF) {
				//Lecteur (ROM-DISK, Backup, BasicDrive)
				GetMainWindow()->OpenFileDrive(m_sFileName, infos[n].nStart, NULL);
			}
		}
		break;
	case 1: //Importer
		{
			QString sFilter;
			if(n)
			{
				sFilter = CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER);
			}
			else
			{
				sFilter = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEMFILTER);
			}
			QString s = CMessageBox::ChooseOpenFileName(QString::null, sFilter, GetMainWidget());
			if(!s.isEmpty())
			{
				char buffer[DRIVE_SIZE];
				CMessageBox::ReadingFile(GetMainWidget(), s, buffer, DRIVE_SIZE);
				
				CFlashBackup backup;
				if(!backup.Open(m_sFileName, true) || !backup.WriteDrive(n, buffer))
				{
					CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_sFileName, true));
				}
				
				ResetTime();
			}
		}
		break;
	case 2://Exporter
		{
			QString sFilter;
			if(n)
			{
				sFilter = CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER);
			}
			else
			{
				sFilter = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEMFILTER);
			}
			QString sExt = "mems";
			if(n != 0) {
				sExt = "lec";
			}
			QString s = CMessageBox::ChooseSaveFileName(QString::null, sFilter, GetMainWidget(), sExt);
			if(!s.isEmpty())
			{
				CFlashBackup backup;
				if(!backup.Open(m_sFileName, false)) {
					CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_sFileName));
					return;
				}
				char buffer[DRIVE_SIZE];

				if(backup.RetrieveDrive(n, buffer) == -1) {
					CMessageBox::MsgCritical(GetMainWidget(), tr("Impossible de lire le lecteur  extraire!"));
				}
				else {
					CMessageBox::SavingFile(GetMainWidget(), s, buffer, DRIVE_SIZE);
				}
			}
		}
			break;
	case 3: //Formater
		if(n != 0) //Ne doit pas formater le systme
		{
			CFlashBackup backup;
			if(!backup.Open(m_sFileName, true))
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_sFileName, true));
				return;
			}
			if(!backup.WriteDrive(n, NULL))
			{
				CMessageBox::MsgCritical(GetMainWidget(), tr("Impossible de formater le lecteur slectionn!"));
				return;
			}
		
			ResetTime();
		}
		break;
	}
	DoReloadFile(true);
	RefreshInfos();
}

//-----------------------------------------------------------------------------------
int BackupWindow::LoadFrom(QDomElement& element)
{
	if(CheckFileWindow::LoadFrom(element) == 1 && !m_sFileName.isEmpty()) {
		return DoReloadFile(true);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void BackupWindow::DoShowCaption()
{
	setCaption(tr("Editeur de backup"));
	CheckFileWindow::DoShowCaption();
}

