
#include "BuildDriveWindow.h"

#include <qtoolbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qheader.h>
#include <qpopupmenu.h>
#include <qtimer.h>
#include <qprocess.h>
#include <qapplication.h>
#include <qcursor.h>

#include <assert.h>

#include <Label.h>
#include <FileList.h>
#include <language.h>
#include <FileList.h>
#include <MessageBox.h>
#include <DriveAccess.h>
#include <SendToMenu.h>
#include <IOptionWindow.h>
#include <image_defs.h>

#include "BuildSettings.h"

#define LIST_BEGIN  "<LIST>"
#define LIST_END    "</LIST>"
#define LIST_DIR    "<DIR>"
#define LIST_ENDDIR "</DIR>"
#define LIST_OPEN   "<OPEN>"

#define PATH_COL    2

#define IMG_BUILD   "romdiskmaker/build.png"
#define IMG_REFRESH "romdiskmaker/refresh.png"

//-----------------------------------------------------------------------------------
BuildDriveWindow::BuildDriveWindow(CBuildSettings* pBuildSettings, CONSTRUCTOR_PARAM)
:CToolWindow(CALL_PARAM)
{
	resize(350, 350);
	m_pBuildSettings = pBuildSettings;
	setIcon(GetDefaultIcon());

	m_nSize			 = 0;
	m_nNumDrive		 = 0;
	m_nTimerInterval = 0;
	m_bTimerOk		 = true;

	QWidget* pWidget = new QWidget(0);
	QBoxLayout* pBuildLayout = new QBoxLayout(pWidget, QBoxLayout::TopToBottom, 2);
	
	m_pFileList		 = new CFileList(pWidget, TRUE);
	m_pLblSize		 = new CLabel(pWidget);
	m_pLblDriveName  = new CLabel(pWidget);
	m_pLblAutoBuild  = new CLabel(pWidget);
	m_pTimer		 = new QTimer(pWidget);
	m_pListMenu		 = new QPopupMenu(pWidget);
	m_pSendToButton  = new QToolButton(GetMainWidget());
	m_pBuildButton   = new QToolButton(GetMainWidget());
	m_pRefreshButton = new QToolButton(GetMainWidget());

	SetCentralWidget(pWidget);

	m_pBuildButton->setAutoRaise(TRUE);
	m_pBuildButton->setIconSet(GetDefaultIcon());
	m_pRefreshButton->setAutoRaise(TRUE);
	m_pRefreshButton->setIconSet(QPixmap::fromMimeSource(IMG_REFRESH));
	m_pSendToButton->setAutoRaise(TRUE);
	m_pSendToButton->setIconSet(QPixmap::fromMimeSource(IMG_SENDTO));

	AddLeftSeparator();
	AddLeftWidget(m_pBuildButton);
	AddLeftWidget(m_pRefreshButton);
	AddLeftWidget(m_pSendToButton);

	pBuildLayout->addWidget(m_pFileList);
	pBuildLayout->addWidget(m_pLblSize);
	pBuildLayout->addWidget(m_pLblDriveName);
	pBuildLayout->addWidget(m_pLblAutoBuild);
	
	m_pLblSize->SetDefaultStyle();
	m_pLblDriveName->SetDefaultStyle();
	m_pLblAutoBuild->SetDefaultStyle();
	
	m_pLblDriveName->setAlignment(AlignCenter);
	m_pLblAutoBuild->setAlignment(AlignCenter);
		
	int i;
	for(i = 0; i < 3; i++) {
		m_pFileList->addColumn( NULL );
		m_pFileList->header()->setClickEnabled( TRUE, i);
	}
	m_pFileList->setColumnWidth(0, 90);
	m_pFileList->setColumnWidth(1, 50);
	m_pFileList->setColumnWidth(2, 150);
	m_pFileList->setColumnWidthMode(1, QListView::Manual);//taille de la colonne manuelle
	m_pFileList->setColumnAlignment(1, AlignRight);
	m_pFileList->setSelectionMode(QListView::Extended);
	m_pFileList->setAllColumnsShowFocus(TRUE);
	m_pFileList->setSorting(0, TRUE);
	m_pFileList->setRootIsDecorated(TRUE);
	m_pFileList->setMinimumSize(100, 100);
	m_pFileList->setShowSortIndicator(TRUE);

	for(i = 0; i < 8; i++) {
		m_pListMenu->insertItem(QString::null, i);
	}
	m_pListMenu->insertItem(QString::null, 8, 2);
	m_pListMenu->insertSeparator();
	m_pListMenu->insertItem(QString::null, 9);

	m_pListMenu->setAccel(CTRL + Key_O, 0);
	m_pListMenu->setAccel(Key_Delete,   1);
	m_pListMenu->setAccel(CTRL + Key_D, 8);

	m_pListMenu->setAccel(CTRL + Key_L, 2);
	m_pListMenu->setAccel(CTRL + Key_S, 3);

	m_pListMenu->setAccel(CTRL + Key_A, 6);
	m_pListMenu->setAccel(CTRL + Key_E, 7);

	m_pListMenu->insertSeparator(2);
	m_pListMenu->insertSeparator(4);
	m_pListMenu->insertSeparator(7);
	m_pListMenu->insertSeparator(10);
	

	connect(m_pFileList, SIGNAL(mouseButtonPressed( int, QListViewItem *, const QPoint &, int )), SLOT(OnListClicked(int, QListViewItem*, const QPoint&)));
	connect(m_pFileList, SIGNAL(selectionChanged()), SLOT(OnSelectionChanged()));
	connect(m_pBuildButton,   SIGNAL(clicked()), SLOT(OnBuildDrive()));
	connect(m_pRefreshButton, SIGNAL(clicked()), SLOT(RefreshList()));
	connect(m_pSendToButton,  SIGNAL(clicked()), SLOT(OnSendTo()));
	connect(m_pLblAutoBuild, SIGNAL(release(int)), SLOT(ToggleTimer()));
	connect(m_pTimer, SIGNAL(timeout()), SLOT(TimerDone()));
	connect(m_pListMenu, SIGNAL(activated(int)), SLOT(OnPopupActivated(int)));
	connect(m_pFileList, SIGNAL(Dropped(const QStringList&, QListViewItem*)), SLOT(OnFilesDropped(const QStringList&, QListViewItem*)));
	connect(m_pFileList, SIGNAL(itemRenamed(QListViewItem*, int)), SLOT(OnItemRenamed(QListViewItem*)));	
	connect(GetMainWindow()->GetSendToMenu(), SIGNAL(FileToSendChanged(const QString&, int)), SLOT(OnRefreshMenu(const QString&, int)));

	GetMainWindow()->GetSendToMenu()->AddSendToItem(this, SLOT(OnFileSent()));
	RefreshInfoDrive();
	OnRefreshMenu(GetMainWindow()->GetSendToMenu()->GetFileName(), GetMainWindow()->GetSendToMenu()->GetNbFile());
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::Init(INIT_PARAM)
{
	DoInitWords();
	return LoadFileList(sFileName, false);
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::DoInitWords()
{
	m_pBuildButton->setTextLabel(tr("Crer lecteur"));
	m_pRefreshButton->setTextLabel(CLanguage::GetWordTranslation(CLanguage::WO_REFRESH));
	m_pSendToButton->setTextLabel(CLanguage::GetWordTranslation(CLanguage::WO_SENDTO));

	m_pListMenu->changeItem(0, tr("Ajouter un ou des fichier(s)..."));
	m_pListMenu->changeItem(1, tr("Supprimer la slection"));
	m_pListMenu->changeItem(2, tr("Charger une liste..."));
	m_pListMenu->changeItem(3, tr("Sauver la liste"));
	m_pListMenu->changeItem(4, tr("Compresser avec UPX"));
	m_pListMenu->changeItem(5, tr("Dcompresser avec UPX"));
	m_pListMenu->changeItem(6, tr("Slectionner tout"));
	m_pListMenu->changeItem(7, tr("Effacer la liste"));
	m_pListMenu->changeItem(8, tr("Nouveau dossier"));
	m_pListMenu->changeItem(9, tr("Ouvrir"));

	m_pFileList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pFileList->setColumnText(1, CLanguage::GetWordTranslation(CLanguage::WO_SIZE));
	m_pFileList->setColumnText(2, CLanguage::GetWordTranslation(CLanguage::WO_PATH));

	RefreshList();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnSendTo()
{
	GetMainWindow()->GetSendToMenu()->SetFileToSend(m_sFileName.c_str(), m_nNumDrive);
	GetMainWindow()->GetSendToMenu()->popup(m_pSendToButton->mapToGlobal(QPoint(0, m_pSendToButton->height())));
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::RefreshInfoDrive()
{
	QString sUnit = tr("Ko");
	double dSize = m_nSize / 1024.0;
	if(dSize > 1024.0)
	{
		sUnit = tr("Mo");
		dSize /= 1024.0;
	}
	m_pLblSize->setText(tr("Taille approximative : ") + QString::number(dSize, 'f', 2) + " " + sUnit);


	const char* c;
	GetNameOfFile(m_sFileName.c_str(), c);
	QString s;
	if(c && c[0] != '\0')
	{
		if(m_nNumDrive > 1)
		{
			s = CLanguage::GetWordTranslation(CLanguage::WO_FILES);
			s = QString(" ") + tr("sur") + " " + QString::number(m_nNumDrive) + " " + s;
		}
		s = c + s;
		m_pSendToButton->setEnabled(TRUE);
		setCaption(tr("Cration de lecteur") + " - " + c);
	}
	else
	{
		s = tr("Pas de lecteur cr");
		m_pSendToButton->setEnabled(FALSE);
		setCaption(tr("Cration de lecteur"));
	}
	m_pLblDriveName->setText(s);
	DisplaySurveillance();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::DisplaySurveillance()
{
	if(m_pTimer->isActive() && !m_sFileName.empty())
	{
		QFileInfo qDriveInfo(m_sFileName.c_str());
		QDateTime qTimeBuild = qDriveInfo.lastModified();
		m_pLblAutoBuild->setText(tr("Surveillance active. Modifi le") + " " + qTimeBuild.toString());
	}
	else
	{
		m_pLblAutoBuild->setText(tr("Aucune surveillance"));
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::ToggleTimer()
{
	if(m_sFileName.empty() && !m_pTimer->isActive())
	{
		return;
	}
	if(m_pTimer->isActive())
	{
		m_pTimer->stop();
	}
	else
	{
		StartTimerOrChangeInterval();
	}
	DisplaySurveillance();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::StartTimerOrChangeInterval()
{
	static int nTimes[] = {100, 200, 500, 1000, 2000, 5000};
	int nVal = m_pBuildSettings->GetPreference(CBuildSettings::PREF_BUILD_STEP);
	int nTime = nTimes[nVal];
	if(m_pTimer->isActive() && nTime != m_nTimerInterval)
	{
		m_pTimer->changeInterval(nTime);
		m_nTimerInterval = nTime;
	}
	else
	{
		m_pTimer->start(nTime);
		m_nTimerInterval = nTime;
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnPopupActivated(int id)
{
	QListViewItem* pCurrentItem = m_pFileList->currentItem();
	if(pCurrentItem && !pCurrentItem->isSelected())
	{
		pCurrentItem = NULL;
	}
	//couper le timer si il est activ
	if(pCurrentItem && !pCurrentItem->text(PATH_COL).isEmpty())
	{
		pCurrentItem = pCurrentItem->parent();
	}
	switch(id)
	{
	case 0: //Ajouter un ou des fichiers
		{
			QString s = QString(CLanguage::GetWordTranslation(CLanguage::WO_EXEFILTER)) + '\n'
					+ tr("Fichiers BAT (*.bat)\nFichiers Bitmap (*.bmp)\nFichiers Texte (*.txt)\nFichiers Map (*.map)\nFichiers RFI (*.rfi)\n")
					+ CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER);
			QStringList list = CMessageBox::ChooseOpenFileNames(s, m_sOpenFile, GetMainWidget());
			if(list.count() > 0)
			{
				m_sOpenFile = *(list.begin());
				AddDirsOrFiles(pCurrentItem, list);
				RefreshList();
				ForceRebuild();
				emit StateChanged();
			}
		}
		break;
	case 1: //Supprimer la slection
		{
			QListViewItem* i = m_pFileList->firstChild();
			while(i)
			{
				if(i->isSelected())
				{
					CFileList::DeleteItem(i, m_pFileList);
					i = m_pFileList->firstChild();
				}
				else
				{
					CFileList::IncItem(i);
				}
			}
		}
		emit StateChanged();
		RefreshList();
		ForceRebuild();
		break;
	case 2: //Charger une liste
		{
			QString filter = "Casio File List (*.cfl)\n";
			filter += CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER);
			QString s = CMessageBox::ChooseOpenFileName(m_sLoadListFile, filter, GetMainWidget());			
			if(!s.isEmpty() && LoadFileList(s) == 1)
			{						
				m_sLoadListFile = s;
				ForceRebuild();
				emit StateChanged();
			}
		}
		break;
	case 3: //Sauver la liste des fichiers
		{
			QString filter = "Casio File List (*.cfl)\n";
			filter += CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER);
			QString s = CMessageBox::ChooseSaveFileName(m_sSaveListFile, filter, GetMainWidget(), "cfl");
			if(s.isEmpty()){
				return;
			}
			QDomDocument doc("FileList");
			
			QDomElement root = doc.createElement("List");
			doc.appendChild(root);

			SaveList(root);
			
			QFile file(s);
			if(file.open(IO_WriteOnly))
			{
				QString sDoc = doc.toString();
				file.writeBlock(sDoc, sDoc.length());
				file.close();
				m_sSaveListFile = s;
				emit StateChanged();				
			}
			else
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(s, TRUE));
			}
		}
		break;
	case 4: //Compresser
	case 5: //Dcompresser
		{
			QListViewItem* pItem = m_pFileList->firstChild();
			int r = 1;
			while(pItem && r > 0)
			{
				if(pItem->isSelected() && ((CFileItem*)pItem)->IsFile()) //si c'est un fichier
				{
					r = LaunchUpx(id == 4, pItem->text(PATH_COL));
				}
				CFileList::IncItem(pItem);
			}
			RefreshList();
			break;
		}
	case 6: //Slectionner tout
		m_pFileList->selectAll(TRUE);
		break;
	case 7: //Effacer la liste
		m_pFileList->clear();
		RefreshList();
		ForceRebuild();
		emit StateChanged();
		break;
	case 8: //Nouveau dossier
		{
			QString sName = tr("Nouveau");
			AddFolder(pCurrentItem, sName)->startRename(0);
			RefreshList();
		}
		break;
	case 9:
		{
			CFileItem* pFileItem = (CFileItem*)m_pFileList->currentItem();
			if(pFileItem != NULL && pFileItem->IsFile()) {
				QFileInfo fi(pFileItem->text(pFileItem->GetFirstColumnFree()));
				GetMainWindow()->OpenTool(fi.extension(), fi.filePath());
			}
		}
		break;
	}
	RefreshPopupMenu(m_pFileList->currentItem());
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::LaunchUpx(bool bCompress, const QString& sFile)
{
	assert(m_pBuildSettings != NULL);
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	QString sUpxPath = m_pBuildSettings->GetUpxPath();
	QProcess qProcess(sUpxPath, NULL);
	if(bCompress)
	{
		QString sArg(m_pBuildSettings->GetUpxArg());
		int i = 0;
		bool bOk = true;
		while(bOk)
		{
			QString s = sArg.section(' ', i, i);
			bOk = !s.isEmpty();
			if(bOk)
			{
				qProcess.addArgument(s);				
			}
			i++;
		}

	}
	else 
	{
		qProcess.addArgument("-d");
	}
	qProcess.addArgument(sFile);

	bool bStarted = qProcess.start();

	while(qProcess.isRunning())
	{
		qApp->processEvents();
	}

	QApplication::restoreOverrideCursor();
	
	if(!bStarted)
	{
		if(QFile::exists(sUpxPath))
		{
			CMessageBox::MsgCritical(GetMainWidget(), tr("Impossible d'excuter upx!\nVrifiez vos droits d'accs, la RAM libre,..."));
		}
		else
		{
			CMessageBox::MsgCritical(GetMainWidget(), tr("Upx n'a pas t trouv.\nVeuillez vrifier le chemin indiqu dans la fentre des options."));
			GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(m_pBuildSettings);
		}
		return -1;
	}
	if(qProcess.exitStatus() == 0)
	{
		emit Information(qProcess.readStdout());
		return 1;
	}
	emit Error(qProcess.readStderr());
	return 2;
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::RefreshList()
{
	QListViewItem* pItem = m_pFileList->firstChild();
	bool bErr;
	unsigned int n;
	m_nNbElement = m_nSize = 0;
	while(pItem)
	{
		CFileItem* pFileItem = (CFileItem*)pItem;
		if(pFileItem->IsDir())
		{
			//Rpertoire
			n = CROMDISKDrive::GetDirSize(pItem->childCount());
			bErr = false;
		}
		else
		{
			n = FileSize(pFileItem->text(pFileItem->GetFirstColumnFree()), &bErr);
		}
		if(bErr) {
			pFileItem->SetUnknowSize();//Signale une erreur sur le fichier
		}
		else {
			pFileItem->SetSize(n);
			m_nSize += CROMDISKDrive::ConvertFileSizeToRealSize(n);
		}
		m_nNbElement++;
		CFileList::IncItem(pItem);
	}
	RefreshInfoDrive();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::LoadList(QDomElement& e, QListViewItem* parent)
{
	if(!e.isNull())
	{
		QListViewItem* pItem;
		QDomNode n = e.firstChild();
		while(!n.isNull())
		{
			QString sName = n.nodeName();
			QDomElement child = n.toElement();
			if(sName == "File")
			{
				QString sPath = child.attribute("Path");
				if(!sPath.isEmpty())
				{
					AddFile(parent, sPath);
				}
			}
			if(sName == "Dir")
			{
				QString sDirName = child.attribute("Name");
				if(!sDirName.isEmpty())
				{					
					pItem = AddFolder(parent, sDirName);
					if(pItem)
					{
						LoadList(child, pItem);
						if(child.attribute("Open").toInt() == 1)
						{
							pItem->setOpen(TRUE);
						}
					}
				}
			}
			n = n.nextSibling();
		}
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::SaveList(QDomElement& e, QListViewItem* parent)
{
	if(!e.isNull())
	{
		CFileItem* pItem;
		if(parent)
		{
			pItem = (CFileItem*)parent->firstChild();
		}
		else
		{
			pItem= (CFileItem*)m_pFileList->firstChild();
		}
		QDomElement child;
		QDomDocument doc = e.ownerDocument();
		while(pItem)
		{		
			if(pItem->IsFile())
			{
				child = doc.createElement("File");
				e.appendChild(child);
				child.setAttribute("Path", pItem->text(pItem->GetFirstColumnFree()));
			}
			else
			{
				child = doc.createElement("Dir");
				e.appendChild(child);
				child.setAttribute("Name", pItem->GetName());
				if(pItem->isOpen())
				{
					child.setAttribute("Open", "1");
				}
				SaveList(child, pItem);
			}
			pItem = (CFileItem*)pItem->nextSibling();
		}
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::RefreshPopupMenu(QListViewItem* pItemSelected)
{
	//Active ou dsactive les options possible suivant la slection et le nbre de fichiers
	CFileItem* pFileItem = (CFileItem*)pItemSelected;
	m_pListMenu->setItemEnabled(3, m_pFileList->childCount() > 0);
	m_pListMenu->setItemEnabled(1, pItemSelected != NULL);
	m_pListMenu->setItemEnabled(4, pItemSelected != NULL);
	m_pListMenu->setItemEnabled(5, pItemSelected != NULL);
	if(pFileItem != NULL && pFileItem->IsFile()) {
		QFileInfo fi(pFileItem->text(pFileItem->GetFirstColumnFree()));
		m_pListMenu->setItemEnabled(9, GetMainWindow()->IsSupported(fi.extension()));
	}
	else {
		m_pListMenu->setItemEnabled(9, FALSE);
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnSelectionChanged()
{
	RefreshPopupMenu(m_pFileList->currentItem());
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnListClicked(int button, QListViewItem* i, const QPoint& p)
{
	if(button == RightButton) {
		RefreshPopupMenu(i);
		m_pListMenu->exec(p);
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnBuildDrive()
{
	QString sPath = CMessageBox::ChooseSaveFileName(m_sFileName.c_str(), CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER), GetMainWidget(), "lec");
	if(!sPath.isEmpty())
	{
		string sName = sPath;
		int n =	BuildDrive(sName);
		if(n > 1)
		{
			QString s = CLanguage::GetWordTranslation(CLanguage::WO_FILES);
		}
		if(n > 0) // Si cration russi
		{
			m_nNumDrive = n; // Affectation du nombre de fichier pour le lecteur
			if(sName != m_sFileName.c_str())
			{
				emit StateChanged();
				m_sFileName = sName;
			}
			//Active autobuild si l'option est coche
			if(m_pBuildSettings->GetPreference(CBuildSettings::PREF_AUTO_BUILD))
			{
				if(!m_pTimer->isActive())
				{
					ToggleTimer();
				}
			}
		}
	}
	RefreshInfoDrive();
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::BuildDrive(string& sName)
{
	RefreshList();
	int nNumDrive = 1;

	int nRootFile = m_pFileList->childCount();
	int nNumOp    = m_nNbElement + 1;//plus le formatage
	CROMDISKDrive drive(FALSE, GetMainWindow()->GetOptionWindow()->GetPreference(IOptionWindow::PREF_DRIVE_VARIABLE_SIZE));
	if(drive.OpenDriveFile(sName, 0, TRUE) < 1)
	{
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName.c_str(), TRUE));
		return 0;
	}
	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	m_pBuildButton->setEnabled(FALSE);

	QProgressDialog* pProgressDialog = CMessageBox::CreateProgressDialog(GetMainWidget(), tr("Formatage en cours..."), nNumOp, tr("Cration du lecteur"));
	CROMDISKFileInfo* pRoot;
	bool bOk = drive.FormatDrive(nRootFile) > 0;
	pRoot = drive.OpenRoot();
	bOk = bOk && pRoot != NULL;

	int nProgress = 1;
	pProgressDialog->setProgress(nProgress);
	qApp->processEvents();

	CFileItem* pItem = (CFileItem*)m_pFileList->firstChild();
	bOk = bOk && CreateDrive(drive, pProgressDialog, pRoot, pItem, nProgress);

	drive.FreeFileInfo(pRoot);

	pProgressDialog->setLabelText(tr("Fermeture du lecteur"));
	qApp->processEvents();
	nNumDrive = drive.GetNumDrive();
	drive.CloseDrive();
	delete pProgressDialog;

	QApplication::restoreOverrideCursor();
	m_pBuildButton->setEnabled(TRUE);

	if(!bOk) {
		CMessageBox::MsgCritical(GetMainWidget(), tr("Une erreur s'est produite lors de la cration du lecteur."));
		return 0;
	}
	if(nNumDrive > 1)
	{
		CDriveAccess::ChangeFileName(sName, 1);
	}
	else
	{
		string sTemp = sName;
		CDriveAccess::ChangeFileName(sTemp, 0);
		if(sTemp != sName)
		{
			if(MoveOrRenameFile(sName.c_str(), sTemp.c_str()))
			{
				sName = sTemp;
			}
		}
	}
	return nNumDrive;
}

//-----------------------------------------------------------------------------------
bool BuildDriveWindow::CreateDrive(CROMDISKDrive& drive, QProgressDialog* pProgressDialog, CROMDISKFileInfo* pParent, CFileItem* pItem, int& nProgress)
{
	bool bOk = TRUE;
	while(bOk && pItem)
	{
		if(pItem->IsDir()) {
			string sDirName = pItem->GetName();
			pProgressDialog->setLabelText(tr("Cration du rpertoire") + QString(" ") + pItem->GetName());
			CROMDISKFileInfo* pDir;
			int nRet = drive.CreateDirectory(pParent, pDir, sDirName, pItem->childCount());
			pProgressDialog->setProgress(++nProgress);
			qApp->processEvents();
			if(nRet == 1) {
				pDir = drive.OpenDir(pDir, true);
				bOk = pDir != NULL && CreateDrive(drive, pProgressDialog, pDir, (CFileItem*)pItem->firstChild(), nProgress);
				drive.FreeFileInfo(pDir);
			}
			else if(nRet == CROMDISKDrive::ER_ALREADY_EXISTS) {
				Warning(tr("Duplication de nom dtect sur le dossier '%1'.").arg(sDirName.c_str()));
			}
		}
		else {
			string sFileName = pItem->GetName();
			string sFilePath = pItem->text(PATH_COL);
			pProgressDialog->setLabelText(tr("Cration du fichier") + QString(" ") + pItem->GetName());
			CROMDISKFileInfo* pFile;
			int nRet = drive.CreateFile(pParent, pFile, sFileName, sFilePath);
			drive.FreeFileInfo(pFile);
			pProgressDialog->setProgress(++nProgress);
			qApp->processEvents();
			if(nRet == CROMDISKDrive::ER_CANT_READ) {
				Warning(tr("Le fichier '%1' n'a pas t cr car le fichier source '%2' n'a pas pu tre lu.").arg(sFileName.c_str(), sFilePath.c_str()));
			}
			if(nRet == CROMDISKDrive::ER_ALREADY_EXISTS) {
				Warning(tr("Duplication de nom dtect sur le fichier '%1'.").arg(sFileName.c_str()));
			}
			bOk = (nRet > 0) || (nRet == CROMDISKDrive::ER_CANT_READ) || (nRet == CROMDISKDrive::ER_ALREADY_EXISTS);
		}
		pItem = (CFileItem*)pItem->nextSibling();
	}
	return bOk;
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1)
	{
		m_sFileName     = ConvertToFullPath(element.attribute("m_sFileName")).latin1();
		m_sOpenFile     = ConvertToFullPath(element.attribute("m_sOpenFile"));
		m_sLoadListFile = ConvertToFullPath(element.attribute("m_sLoadListFile"));
		m_sSaveListFile = ConvertToFullPath(element.attribute("m_sSaveListFile"));
		int nActived    = element.attribute("Actived").toInt();
		m_nNumDrive     = element.attribute("m_nNumDrive").toInt();
		m_pFileList->setColumnWidth(0,
			element.attribute("WidthCol0").toInt());
		m_pFileList->setColumnWidth(1,
			element.attribute("WidthCol1").toInt());
		m_pFileList->setColumnWidth(2,
			element.attribute("WidthCol2").toInt());
		QDomElement files = element.namedItem("Files").toElement();
		
		if(nActived)
		{
			StartTimerOrChangeInterval();
		}
		RefreshInfoDrive();
		
		//Charger la liste des fichiers et dossiers
		m_pFileList->clear();//On efface l'ancienne avant
		LoadList(files);
		RefreshList();
		
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1)
	{		
		element.setAttribute("m_sFileName",     ConvertToRelativePath(m_sFileName.c_str()));
		element.setAttribute("m_sOpenFile",     ConvertToRelativePath(m_sOpenFile));
		element.setAttribute("m_sLoadListFile", ConvertToRelativePath(m_sLoadListFile));
		element.setAttribute("m_sSaveListFile", ConvertToRelativePath(m_sSaveListFile));
		element.setAttribute("Actived",         QString::number(m_pTimer->isActive()));
		element.setAttribute("m_nNumDrive",     QString::number(m_nNumDrive));
		element.setAttribute("WidthCol0",       QString::number(m_pFileList->columnWidth(0)));
		element.setAttribute("WidthCol1",       QString::number(m_pFileList->columnWidth(1)));
		element.setAttribute("WidthCol2",       QString::number(m_pFileList->columnWidth(2)));

		QDomElement files = element.ownerDocument().createElement("Files");
		element.appendChild(files);
		SaveList(files);
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::TimerDone()
{
	if(!m_bTimerOk)
	{
		return;
	}

	m_bTimerOk = FALSE;

	QFileInfo qDriveInfo(m_sFileName.c_str());
	QDateTime qDriveTime = qDriveInfo.lastModified();
	QDateTime qDriveCreatedTime = qDriveInfo.created();
	if(qDriveTime < qDriveCreatedTime)
	{
		qDriveTime = qDriveCreatedTime;
	}

	bool bDoRebuild  = !qDriveInfo.exists();
	bool bDoCompress = m_pBuildSettings->GetPreference(CBuildSettings::PREF_AUTO_COMPRESS);

	QListViewItem* pItem = m_pFileList->firstChild();

	//Parcours de la liste des fichiers, compare la dernire modification du lecteur 
	//celle des fichiers
	while(pItem && (!bDoRebuild || bDoCompress))
	{
		CFileItem* pFileItem = (CFileItem*)pItem;
		if(pFileItem->IsFile()) //pour viter les dossiers
		{
			QString sFilePath = pItem->text(PATH_COL);
			QFileInfo qFileInfo(sFilePath);
			QDateTime tFileLastModifiedTime = qFileInfo.lastModified();
			QDateTime tFileCreatedTime		= qFileInfo.created();
			if(tFileLastModifiedTime > qDriveTime || tFileCreatedTime > qDriveTime)
			{
				bDoRebuild = TRUE;
				if(bDoCompress && QDir::match(QString("*.exe;*.com;*.dll"), sFilePath))
				{
					if(LaunchUpx(TRUE, sFilePath) < 1)
					{
						bDoCompress = FALSE;//arrete la compression si une erreur se produit
					}
				}
			}
		}
		CFileList::IncItem(pItem);
	}
	if(bDoRebuild)
	{
		ForceRebuild();
	}
	else
	{
		DisplaySurveillance();
		//Actualise l'intervalle entre chaque vrification
		StartTimerOrChangeInterval();
	}
	m_bTimerOk = TRUE;
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::DoShowCaption()
{
	RefreshInfoDrive();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::AddDirsOrFiles(QListViewItem* pParentItem, const QStringList& sFiles)
{
	QListViewItem* pItem;

	if(pParentItem && !((CFileItem*)pParentItem)->IsDir())
	{
		pParentItem = pParentItem->parent();
	}

	const char* c;
	QString sName;
	for(QStringList::const_iterator it = sFiles.begin(); it != sFiles.end(); ++it)
	{
		QString sFile = *it;
		QDir dir(sFile);
		if(dir.exists())
		{
			GetNameOfFile(sFile, c);
			sName = c;
			pItem = AddFolder(pParentItem, sName);
			QStringList sSubList = dir.entryList(QDir::Files | QDir::Dirs);
			QStringList::Iterator it = sSubList.begin();
			while(it != sSubList.end())
			{
				QString s = *it;
				if(s == "." || s == "..")
				{
					it = sSubList.remove(it);
				}
				else
				{
					(*it) = sFile + QDir::separator() + s;
					++it;
				}
			}
			AddDirsOrFiles(pItem, sSubList);
		}
		else
		{
			pItem = AddFile(pParentItem, sFile);
		}
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnItemRenamed(QListViewItem* pItem)
{
	//Renome un dossier
	CFileItem* pDir = (CFileItem*)pItem;
	if(pDir && pDir->IsDir())
	{
		QString sDirName = pDir->GetName();
		RenameDirectory(pDir, sDirName);
		ForceRebuild();
		emit StateChanged();
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::RenameDirectory(CFileItem* pDirectory, QString& sName)
{
	assert(pDirectory != NULL && pDirectory->IsDir());
	sName.truncate(8);
	sName = sName.upper();
	pDirectory->SetName(sName);
	m_pFileList->sort();
}

//-----------------------------------------------------------------------------------
QListViewItem* BuildDriveWindow::AddFolder(QListViewItem* pParentItem, QString& sName)
{	
	CFileItem* pItem = NULL;
	if(pParentItem)
	{
		pItem = new CFileItem(CFileItem::Directory, pParentItem);
	}
	else
	{
		pItem = new CFileItem(CFileItem::Directory, m_pFileList);
	}
	RenameDirectory(pItem, sName);
	pItem->setRenameEnabled(0, TRUE);
	return pItem;
}

//-----------------------------------------------------------------------------------
QListViewItem* BuildDriveWindow::AddFile(QListViewItem* pParentItem, QString& sPath)
{	
	CFileItem* pItem = NULL;
	if(pParentItem) {
		pItem = new CFileItem(CFileItem::File, pParentItem);
	}
	else {
		pItem = new CFileItem(CFileItem::File, m_pFileList);
	}
	QFileInfo fi(sPath);
	pItem->setText(pItem->GetFirstColumnFree(), sPath);
	pItem->SetName(fi.fileName().upper());
	return pItem;
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem)
{
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	AddDirsOrFiles(pDroppedItem, sFiles);
	RefreshList();
	ForceRebuild();
	QApplication::restoreOverrideCursor();
	emit StateChanged();
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::ForceRebuild()
{
	if(m_pTimer->isActive())
	{
		if(m_pFileList->childCount())
		{
			int n =	BuildDrive(m_sFileName);
			if(n > 0) // Si cration russi
			{
				m_nNumDrive = n; // Affectation du nombre de fichier pour le lecteur
				RefreshInfoDrive();
				//Actualise l'intervalle entre chaque vrification
				StartTimerOrChangeInterval();
				emit Information(tr("(Auto Recration) Le fichier '%1' a t recr  %2.").arg(m_sFileName.c_str()).arg(QTime::currentTime().toString()));
				return;
			}
			emit Error(tr("(Auto Recration) Le fichier '%1' n'a pas pu tre recr  %2.").arg(m_sFileName.c_str()).arg(QTime::currentTime().toString()));
		}
		ToggleTimer();
	}
}

//-----------------------------------------------------------------------------------
QPixmap BuildDriveWindow::GetDefaultIcon()
{
	return QPixmap::fromMimeSource(IMG_BUILD);
}

//-----------------------------------------------------------------------------------
int BuildDriveWindow::LoadFileList(const QString& sFileName, bool bShowErr)
{
	if(!sFileName.isEmpty())
	{
		QDomDocument doc;
		QFile f(sFileName);
		if(!f.open(IO_ReadOnly))
		{
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName));
			return 0;
		}
		if(!doc.setContent(&f) || doc.doctype().name() != "FileList")
		{
			f.close();
			if(bShowErr)
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(sFileName));
			}
			return -1;
		}
		f.close();
		QDomElement e = doc.documentElement();
		LoadList(e);
		RefreshList();
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnFileSent()
{
	QString s = GetMainWindow()->GetSendToMenu()->GetFileName();
	if(!s.isEmpty()) {
		AddFile(NULL, s);
		RefreshList();
	}
}

//-----------------------------------------------------------------------------------
void BuildDriveWindow::OnRefreshMenu(const QString& sFileName, int nNbFile)
{
	GetMainWindow()->GetSendToMenu()->SetItemEnabled(this, !QDir::match(QString("*.mems;*.cfs;*.lec;*.cdr;*.cfx;*.bck;*.prj"), sFileName)
		&& nNbFile == 1);
}
