// BuildDriveWindow.h: interface for the BuildDriveWindow class.
//
//////////////////////////////////////////////////////////////////////

#ifndef BUILDDRIVEWINDOW_H
#define BUILDDRIVEWINDOW_H

#include <ToolWindow.h>
#include <qdatetime.h>

#include <ROMDISKDrive.h>

class QToolButton;
class CLabel;
class CFileList;
class QPopupMenu;
class QColor;
class QListViewItem;
class CBuildSettings;
class QStringList;
class QProgressDialog;
class CFileItem;

class BuildDriveWindow : public CToolWindow
{
	Q_OBJECT

public:
	BuildDriveWindow(CBuildSettings* pBuildSettings, CONSTRUCTOR_PARAMDEF);
	int            Init(INIT_PARAMDEF);
	int            LoadFrom(QDomElement& element);
	int            SaveTo(QDomElement& element);

public:
	static QPixmap GetDefaultIcon();

protected:
	void           RefreshInfoDrive();
	void           DisplaySurveillance();
	/** Force la recration du lecteur si en mode surveillance */
	void           ForceRebuild();

	/** Launch Upx, lance Upx.exe
	  * Retourne 1 si Ok
	  * Retourne 2 si le retour d'upx est diffrent de 0
	  * Retourne -1 erreur (fichier manquant, ...)
	  */
	int            LaunchUpx(bool bCompress, const QString& sFile);
	int            LoadFileList(const QString& sFileName, bool bShowErr = TRUE);
	void           LoadList(QDomElement& e, QListViewItem* parent = NULL);
	void           SaveList(QDomElement& e, QListViewItem* parent = NULL);
	int            BuildDrive(string& sName);
	void		   DoShowCaption();
	void           DoInitWords();
	void           AddDirsOrFiles(QListViewItem* pParentItem, const QStringList& sFiles);
	QListViewItem* AddFolder(QListViewItem* pParentItem, QString& sName);
	QListViewItem* AddFile(QListViewItem* pParentItem, QString& sPath);
	bool           CreateDrive(CROMDISKDrive& drive, QProgressDialog* pProgressDialog, CROMDISKFileInfo* pParent, CFileItem* pItem, int& nProgress);

private slots:
	void        ToggleTimer();
	void        OnSendTo();
	void        OnListClicked(int button, QListViewItem* i, const QPoint& p);
	void		OnSelectionChanged();
	void        RefreshList();
	void		OnBuildDrive();
	void        TimerDone();
	void		OnPopupActivated(int id);
	void		OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem);
	void        OnItemRenamed(QListViewItem*);
	void        OnFileSent();
	void        OnRefreshMenu(const QString& sFileName, int nNbFile);

private:
	void        StartTimerOrChangeInterval();
	void		RefreshPopupMenu(QListViewItem* pItemSelected);
	void        RenameDirectory(CFileItem* pDirectory, QString& sName);

private:
	CFileList*	    m_pFileList;
	QToolButton*    m_pSendToButton;
	QToolButton*    m_pBuildButton;
	QToolButton*    m_pRefreshButton;
	CLabel*		    m_pLblSize;
	CLabel*		    m_pLblDriveName;
	CLabel*		    m_pLblAutoBuild;
	QPopupMenu*	    m_pListMenu;
	string		    m_sFileName;
	QString		    m_sOpenFile;
	QString		    m_sLoadListFile;
	QString		    m_sSaveListFile;
	int			    m_nSize;
	int             m_nNbElement;
	int			    m_nNumDrive;
	QTimer*		    m_pTimer;
	int			    m_nTimerInterval;
	bool            m_bTimerOk;
	CBuildSettings* m_pBuildSettings;
};

#endif
