// BuildSettings.cpp: implementation of the CBuildSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "BuildSettings.h"

#include <qapplication.h>

#include <ToolWindow.h>

#define KEY_BUILDDRIVE	"Drive"
#define KEY_UPXPATH		"UPXPath"
#define KEY_UPXARG		"UPXArg"

#ifdef WIN32
#	define UPX_PATH "upx.exe"
#else
#	define UPX_PATH "upx"
#endif

#define UPX_DEFAULT_ARG "--best --crp-ms=100000 --8086"

#define UPX_DEFAULT_PATH (APP_DIR_PATH + '/' + QString(UPX_PATH))

//-----------------------------------------------------------------------------------
CBuildSettings::CBuildSettings(IMainWindow* pMainWindow)
:	CSettingsWidget(pMainWindow, TOOL_NAME),
	m_qVBoxLayout(this, 10),
	m_qBuildGroup(this),
	m_qUPXGroup(this),
	m_chkAutoBuild(&m_qBuildGroup),
	m_chkAutoCompress(&m_qBuildGroup),
	m_pWidget(&m_qBuildGroup),
	m_pHLayout(&m_pWidget, 0, 5),
	m_txtInterval(&m_pWidget),
	m_cmbAutoBuild(&m_pWidget),
	m_txtUpxPath(&m_qUPXGroup),
	m_lblUpxPath(&m_qUPXGroup),
	m_txtUpxArg(&m_qUPXGroup),
	m_lblUpxArg(&m_qUPXGroup)
{
	m_qUPXGroup.setColumns(2);

	m_qVBoxLayout.addWidget(&m_qBuildGroup, 0, AlignVCenter);
	m_qVBoxLayout.addWidget(&m_qUPXGroup,   0, AlignVCenter);
	m_qVBoxLayout.addStretch(1); 

	m_pHLayout.addWidget(&m_txtInterval);
	m_pHLayout.addWidget(&m_cmbAutoBuild);

	int i;
	for(i = 0; i < 6; i++)
	{
		m_cmbAutoBuild.insertItem(QString::null, i);
	}

	DoInitWords();
	DoRefresh();
}

//-----------------------------------------------------------------------------------
int CBuildSettings::DoSave()
{
	m_nBuildPreferences  = m_chkAutoBuild.isChecked();
	m_nBuildPreferences += (m_chkAutoCompress.isChecked()  << 1);
	m_nBuildPreferences += (m_cmbAutoBuild.currentItem() << 2);

	m_sUpxPath = m_txtUpxPath.text();
	m_sUpxArg  = m_txtUpxArg.text().simplifyWhiteSpace();

	WriteSettings(KEY_BUILDDRIVE, m_nBuildPreferences);
	WriteSettings(KEY_UPXPATH,    CToolWindow::ConvertToRelativePath(m_sUpxPath));
	WriteSettings(KEY_UPXARG,     m_sUpxArg);
	return 1;
}

//-----------------------------------------------------------------------------------
void CBuildSettings::DoRefresh()
{
	ReadSettings(KEY_BUILDDRIVE, m_nBuildPreferences, 18);
	ReadSettings(KEY_UPXPATH,    m_sUpxPath, UPX_DEFAULT_PATH);
	ReadSettings(KEY_UPXARG,     m_sUpxArg,  UPX_DEFAULT_ARG);

	m_sUpxPath = CToolWindow::ConvertToFullPath(m_sUpxPath);
	m_sUpxArg  = m_sUpxArg.simplifyWhiteSpace();

	m_chkAutoBuild.setChecked(GetPreference(PREF_AUTO_BUILD));
	m_chkAutoCompress.setChecked(GetPreference(PREF_AUTO_COMPRESS));
	m_cmbAutoBuild.setCurrentItem(GetPreference(PREF_BUILD_STEP));

	m_txtUpxPath.setText(m_sUpxPath);
	m_txtUpxArg.setText(m_sUpxArg);
}

//-----------------------------------------------------------------------------------
QString CBuildSettings::GetDescription() const
{
	return QString::null;
}

//-----------------------------------------------------------------------------------
int CBuildSettings::GetPreference(int nPref)
{
	switch(nPref)
	{
	case PREF_AUTO_BUILD:
		return m_nBuildPreferences&1;
	case PREF_AUTO_COMPRESS:
		return m_nBuildPreferences&2;
	case PREF_BUILD_STEP:
		return m_nBuildPreferences>>2;
	}
	return -1;//Pas trouv
}

//-----------------------------------------------------------------------------------
void CBuildSettings::DoInitWords()
{
	setCaption(tr("Cration d'un lecteur"));
	m_qUPXGroup.setTitle(tr("Options pour UPX"));
	m_qBuildGroup.setTitle(tr("Options sur la surveillance du lecteur cr"));

	m_chkAutoBuild.setText(tr("Activer lors de la cration d'un lecteur"));
	m_chkAutoCompress.setText(tr("Compresser automatiquement les exe"));
	m_txtInterval.setText(tr("Intervalle entre chaque vrification :"));

	m_lblUpxPath.setText(tr("Chemin d'UPX :"));
	m_lblUpxArg.setText(tr("Paramtres d'UPX :"));

	QString sSecond (" " + tr("seconde"));
	QString sSeconds(" " + tr("secondes"));
	m_cmbAutoBuild.changeItem(QString::number(0.1f) + sSecond,  0);
	m_cmbAutoBuild.changeItem(QString::number(0.2f) + sSecond,  1);
	m_cmbAutoBuild.changeItem(QString::number(0.5f) + sSecond,  2);
	m_cmbAutoBuild.changeItem(QString::number(1)	+ sSecond,  3);
	m_cmbAutoBuild.changeItem(QString::number(2)	+ sSeconds, 4);
	m_cmbAutoBuild.changeItem(QString::number(5)	+ sSeconds, 5);
}

//-----------------------------------------------------------------------------------
QString CBuildSettings::GetUpxPath()
{
	return m_sUpxPath;
}

//-----------------------------------------------------------------------------------
QString CBuildSettings::GetUpxArg()
{
	return m_sUpxArg;
}
