
#include "CFXWindow.h"

#include <qgroupbox.h>
#include <qheader.h>
#include <qlistview.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qiconview.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qdir.h>
#include <qvgroupbox.h> 
#include <qinputdialog.h>
#include <qapplication.h>

#include <Label.h>
#include <language.h>
#include <MessageBox.h>
#include <DriveAccess.h>
#include <TransfertWindow.h>
#include <FileList.h>
#include <ROMDISKDrive.h>
#include <CasioSystem.h>
#include <image_defs.h>
#include <CasioBackup.h>
#include <IOptionWindow.h>

#include "CFXSettings.h"
#include "../DriveWindow/defs.h"

#define TMPCFX "/F100_tmpcfx"

#define IMG_COMM "cfx/comm.png"

//-----------------------------------------------------------------------------------
CFXWindow::CFXWindow(CCFXSettings* pCFXSettings, CONSTRUCTOR_PARAM)
:CToolWindow(CALL_PARAM)
{
	resize(550, 288);
	m_pCFXSettings = pCFXSettings;
#ifdef WIN32
	m_sTmpFile = QDir::homeDirPath() + TMPCFX;
#endif
	nTotalSize = 0;

	QWidget* pWidget     = new QWidget(0);
	QBoxLayout* pLayout  = new QBoxLayout(pWidget, QBoxLayout::LeftToRight, 5);

	SetCentralWidget(pWidget);

	setIcon(GetDefaultCFXPixmap());

	frame[0] = new QVGroupBox(pWidget);
	frame[1] = new QVGroupBox(pWidget);
	m_pCommButton	 = new QToolButton(GetMainWidget());
	m_pOptionsButton = new QToolButton(GetMainWidget());

	AddLeftSeparator();
	AddLeftWidget(m_pCommButton);
	AddLeftWidget(m_pOptionsButton);

	m_pCommButton->setIconSet(QPixmap::fromMimeSource(IMG_COMM));
	m_pCommButton->setAutoRaise(TRUE);
	m_pOptionsButton->setIconSet(QPixmap::fromMimeSource(IMG_OPTION));
	m_pOptionsButton->setAutoRaise(TRUE);
	
	pLayout->addWidget(frame[0], 2);
	pLayout->addWidget(frame[1], 3);

	m_pList = new CFileList(frame[0], TRUE);

	m_pLblInfo = new CLabel(frame[0]);
	m_pLblInfo->SetDefaultStyle();
	m_pLblInfo->setAlignment(AlignCenter);
	
	int i;
	for(i = 0; i < 7; i++) {
		tab[i].item = NULL;
	}
	for(i = 0; i < 2; i++) {
		m_pList->addColumn(QString::null);
		m_pList->header()->setClickEnabled(FALSE, i);
		m_pList->header()->setResizeEnabled(TRUE, i);
	}
	m_pList->setColumnWidth(0, 80);
	m_pList->setColumnWidth(1, 50);
	m_pList->setSorting(-1);// dsactive le tri
	m_pList->setAllColumnsShowFocus(TRUE);
	m_pList->setColumnAlignment(1, AlignRight);
			
	m_pIconView = new QIconView(frame[1]);
	m_pIconView->setItemsMovable(false);
		
	m_pCommButton->setEnabled(false);

	m_pDriveWindow = NULL;
	if(GetMainWindow()->IsLoaded(DRIVEEDIT_WINDOWTYPESTR)) {
		m_pDriveWindow = GetMainWindow()->OpenToolByType(DRIVEEDIT_WINDOWTYPESTR, QString::null, 0, FALSE, frame[1]);
	}
	
	m_pPopupMenu = new QPopupMenu(pWidget);
	m_pIconMenu  = new QPopupMenu(pWidget);

	m_pPopupMenu->insertItem(QString::null, 0);
	m_pPopupMenu->insertItem(QString::null, 1);
	m_pPopupMenu->insertItem(QString::null, m_pIconMenu, 2);
	m_pPopupMenu->insertItem(QString::null, 3);
	m_pPopupMenu->insertSeparator(2);
	m_pPopupMenu->insertItem(QString::null, 4);
	m_pPopupMenu->insertItem(QString::null, 5);
	m_pPopupMenu->insertSeparator(5);
	
	m_pPopupMenu->setAccel(CTRL + Key_A, 0);
	m_pPopupMenu->setAccel(Key_Delete,   1);
	m_pPopupMenu->setAccel(CTRL + Key_E, 3);
	m_pPopupMenu->setAccel(CTRL + Key_U, 4);
	m_pPopupMenu->setAccel(CTRL + Key_D, 5);

	//Connexions
	connect(m_pOptionsButton, SIGNAL(clicked()), SLOT(ShowOption()));
	connect(m_pCommButton,    SIGNAL(clicked()), SLOT(StartComm()));
	connect(m_pList, SIGNAL(mouseButtonPressed(int, QListViewItem *, const QPoint &, int )), SLOT(OnListClicked( int, QListViewItem *, const QPoint &)));
	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(OnSelectionChanged(QListViewItem*)));
	connect(m_pList, SIGNAL(Dropped(const QStringList&, QListViewItem*)), SLOT(OnFilesDropped(const QStringList&, QListViewItem*)));
	connect(m_pPopupMenu, SIGNAL(activated(int)), SLOT(OnMenuActivated(int)));
	connect(m_pIconMenu,  SIGNAL(activated(int)), SLOT(OnExtractIcon(int)));
	languageChange();
	RefreshPopupMenu(NULL);
}

//-----------------------------------------------------------------------------------
CFXWindow::~CFXWindow()
{
#ifdef WIN32
	KillFile(m_sTmpFile);
#endif
	for(int i = 0; i < 7; i++) {
		DeleteTCFX(tab[i]);
	}
}

//-----------------------------------------------------------------------------------
void CFXWindow::DeleteTCFX(TCFX& cfx)
{
	if(cfx.item != NULL) {
		delete[] cfx.pDrive;
		for(int i = 0; i < cfx.nNbIcon; i++) {
			delete[] cfx.pIcon[i];
			delete[] cfx.pInfo[i];
		}
		delete[] cfx.pIcon;
		delete[] cfx.pInfo;
	}
	cfx.item = NULL;
}

//-----------------------------------------------------------------------------------
int CFXWindow::Init(INIT_PARAM)
{
	if(!sFileName.isEmpty()) {
		return AddFile(sFileName);
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CFXWindow::DoInitWords()
{
	m_pList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pList->setColumnText(1, CLanguage::GetWordTranslation(CLanguage::WO_SIZE));
	frame[0]->setTitle(tr("Projets"));
	frame[1]->setTitle(tr("Dtails"));
	m_pCommButton->setTextLabel(tr("Communiquer"));
	m_pOptionsButton->setTextLabel(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));
	m_pPopupMenu->changeItem(0, tr("Ajouter un fichier..."));
	m_pPopupMenu->changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_EXTRACT) + tr(" le lecteur"));
	m_pPopupMenu->changeItem(4, tr("Monter"));
	m_pPopupMenu->changeItem(5, tr("Descendre"));
	Refresh();
}

//-----------------------------------------------------------------------------------
int CFXWindow::AddFile(ADDFILE_PARAM)
{
#ifdef WIN32
	OFSTRUCT infos;
	int src  = LZOpenFile((char*)sFileName.latin1(),  &infos, OF_READ);
	int dest = LZOpenFile((char*)m_sTmpFile.latin1(), &infos, OF_CREATE | OF_WRITE);
	if(src >= 0 && src >= 0) {
		LZCopy(src, dest);
	}
	LZClose(src);
	LZClose(dest);
	HND h2 = OpenDescModeRead(m_sTmpFile);
	if(h2 == INVALID_HANDLE_VALUE) {
		CMessageBox::MsgCritical(GetMainWidget(), tr("La copie du cfx a choue."));
		return -1;
	}
#else
	HND h2 = OpenDescModeRead(sFileName);
	if(h2 == INVALID_HANDLE_VALUE) {
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName));
		return -1;
	}		
#endif
	SeekDesc(h2, 4);
	char buf[10];
	ReadDesc(h2, buf, 6);
	QString s = buf;
	if(s == "CASIO")
	{
		SeekDesc(h2, 0x81);
		ReadDesc(h2, buf, 8);
		buf[8] = '\0';
		int size = 0;
		
		//Recherche du nombre de lecteur (taille)
		int j;char c = 1;
		for(j = 0x90; c == 1 && j <= 0xD0; j+= 0x10)
		{
			size += DRIVE_SIZE;
			SeekDesc(h2, j);
			ReadDesc(h2, &c, 1);
		}
		if(nTotalSize + size > DRIVE_SIZE * 6)
		{
			CMessageBox::MsgCritical(GetMainWidget(), tr("Pas assez de place pour ajouter ce fichier."));
			CloseDesc(h2);
			return -1;
		}
		QListViewItem* i = new QListViewItem(m_pList, CFileList::LastItem(m_pList->firstChild()));
		i->setText(0, buf);
		i->setText(1, QString::number(size));
		i->setText(2, sFileName);//Sauvegarde le chemin du fichier
		nTotalSize += size;
		for(j = 0; j < 7 && tab[j].item;j++);
		tab[j].item = i;
		
		//Recherche de l'emplacement de l'icne principale
		SeekDesc(h2, 0x20);
		ReadDesc(h2, buf, 4);
		unsigned int nStartIcon = ConvertToInt((unsigned char*)buf);
		
		//Recherche de l'emplacement de dpart du lecteur
		SeekDesc(h2, 0x1C);
		ReadDesc(h2, buf, 4);
		unsigned int nStartDrive = ConvertToInt((unsigned char*)buf);
		
		//Calcul du nbre d'icnes supplmentaire
		unsigned int nNbIcon = (nStartIcon - nStartDrive - size) / (138 + 22);
		if(nNbIcon) //Vrifie qu'il y a bien des icnes supplmentaire
		{
			//On se place juste  la fin du lecteur
			//Decalage de 8 pour le header de Calipso
			SeekDesc(h2, nStartDrive + size + 8);
			ReadDesc(h2, buf, 1);
			if((unsigned char)(buf[0]) != nNbIcon)
			{
				nNbIcon = 0;
			}
		}
		tab[j].nNbIcon = nNbIcon + 1;
		
		//Allocation mmoire
		tab[j].pIcon = new uchar* [tab[j].nNbIcon];
		tab[j].pInfo = new char*  [tab[j].nNbIcon];
		
		//Extraction de l'icone principale
		tab[j].pIcon[0] = new uchar[138];
		tab[j].pInfo[0] = new char[13];
		SeekDesc(h2, nStartIcon);
		ReadDesc(h2, tab[j].pInfo[0], 12);//Copie le nom
		tab[j].pInfo[0][12] = '\0';
		SeekDesc(h2, nStartIcon + 22);
		ReadDesc(h2, tab[j].pIcon[0], 138);//Copie la bitmap
		
		//Extraction des icnes supplmentaire
		nStartIcon = nStartDrive + size + 10;
		for(int num = 1; num < tab[j].nNbIcon; num++)
		{
			tab[j].pIcon[num] = new uchar[138];
			tab[j].pInfo[num] = new char[13];
			SeekDesc(h2, nStartIcon); //Lit le nom
			ReadDesc(h2, tab[j].pInfo[num], 12);
			tab[j].pInfo[num][12] = '\0';
			SeekDesc(h2, nStartIcon + 22);
			ReadDesc(h2, tab[j].pIcon[num], 138);//Lit la bitmap
			nStartIcon += 160; //(138 + 22)
		}
		CloseDesc(h2);	
		
		//Extraction du lecteur (affiche une boite de progression si trop long)
		//tant donn que l'on peut avoir 700Ko de donnes, le tps d'extraction peut tre long
		tab[j].pDrive = new uchar[size];
		CMessageBox::ReadingFile(GetMainWidget(), m_sTmpFile, tab[j].pDrive, size, nStartDrive);
		
		//Le rend slectionn pour voir ses infos
		m_pList->setSelected(i, true);
		m_pCommButton->setEnabled(true);
		emit StateChanged();
	}
	else
	{
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(sFileName));
		CloseDesc(h2);
	}
	Refresh();
	return 1;
}

//-----------------------------------------------------------------------------------
void CFXWindow::OnSelectionChanged(QListViewItem* item)
{
	RefreshPopupMenu(item);
	if(!item) {
		return;
	}
	int j;
	for(j = 0; j < 7 && tab[j].item != item; j++);
	
	//Supprime tous les lments de la liste
	m_pIconView->clear();
	
	//Affichage des l'icones disponibles
	QIconViewItem* lastItem = NULL;
	for(int i = 0; i < tab[j].nNbIcon; i++)
	{
		QPixmap pixmap;
		pixmap.loadFromData(tab[j].pIcon[i], 138);
		QIconViewItem* item = new QIconViewItem(m_pIconView, lastItem, tab[j].pInfo[i], pixmap);
		lastItem = item;
	}
	
	int size = item->text(1).toInt();

	if(m_pDriveWindow)
	{
		ExtraParam param;
		param.nMaxSize         = -1;
		param.bReadOnly        = TRUE;
		param.bShowGraphWidget = FALSE;
		param.pBuffer          = tab[j].pDrive;
		param.bShowOpenButton  = FALSE;
		if(m_pDriveWindow->Init(QString::null, 0, &param) == -1) {
			CMessageBox::MsgInformation(GetMainWidget(), tr("Ce fichier ne contient pas un lecteur ROM-DISK valide."));
		}
	}
}

//-----------------------------------------------------------------------------------
void CFXWindow::ShowOption()
{
	GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(m_pCFXSettings);
}

//-----------------------------------------------------------------------------------
void CFXWindow::RefreshPopupMenu(QListViewItem* pItemSelected)
{
	QString s;
	QString sIcon(CLanguage::GetWordTranslation(CLanguage::WO_EXTRACT));
	m_pIconMenu->clear();
	if(pItemSelected)
	{
		s = " '" + pItemSelected->text(0) + "'";
		int j;
		for(j = 0; tab[j].item != pItemSelected; j++);
		if(tab[j].nNbIcon > 1)
		{
			sIcon += tr(" les icnes");
		}
		else
		{
			sIcon += tr(" l'icne");
		}
		for(int n = 0; n < tab[j].nNbIcon; n++)
		{
			m_pIconMenu->insertItem(tab[j].pInfo[n], n);
		}
	}
	else
	{
		sIcon += tr(" l'icne");
	}
	m_pPopupMenu->changeItem(1, tr("Supprimer") + s);
	m_pPopupMenu->changeItem(2, sIcon);
	m_pPopupMenu->setItemEnabled(1, pItemSelected);
	m_pPopupMenu->setItemEnabled(2, pItemSelected);
	m_pPopupMenu->setItemEnabled(3, pItemSelected);
	m_pPopupMenu->setItemEnabled(4, pItemSelected && pItemSelected->itemAbove());
	m_pPopupMenu->setItemEnabled(5, pItemSelected && pItemSelected->itemBelow());
}

//-----------------------------------------------------------------------------------
void CFXWindow::OnListClicked(int button, QListViewItem* i, const QPoint & p)
{
	if(button == RightButton)
	{
		RefreshPopupMenu(i);
		m_pPopupMenu->exec(p);
	}
}

//-----------------------------------------------------------------------------------
unsigned int CFXWindow::ConvertToInt(unsigned char* buf)
{
	unsigned int n = 0;
	for(int i = 0; i < 4; i++)
	{
		n = (n << 8) + buf[i];
	}
	return n;
}

//-----------------------------------------------------------------------------------
void CFXWindow::OnMenuActivated(int id)
{
	switch(id)
	{
	case 0: // Ajouter un fichier
		{
			QString filter = QString("CFX (*.cfx)") + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER);
			QString s = CMessageBox::ChooseOpenFileName(m_sLastOpenedFile, filter, GetMainWidget());
			if(!s.isEmpty())
			{
				AddFile(s);
				m_sLastOpenedFile = s;
			}
		}
		break;
	case 1: // Supprimer le fichier de la liste
		{
			QListViewItem* i = m_pList->selectedItem();
			int j;
			for(j = 0; j < 7 && tab[j].item != i; j++);
			DeleteTCFX(tab[j]);
			m_pList->takeItem(i);
			m_pList->setSelected(m_pList->firstChild(), true);
			nTotalSize -= i->text(1).toInt();
			delete i;
			Refresh();
			emit StateChanged();
		}
		break;
	case 3: //Extraire le lecteur
		ExtractDrive();
		break;
	case 4: //Monter
		{
			QListViewItem* pItem = m_pList->selectedItem();
			if(pItem && pItem->itemAbove())
			{
				QListViewItem* pAfter = pItem->itemAbove()->itemAbove();
				if(pAfter)
				{
					pItem->moveItem(pAfter);
				}
				else
				{
					m_pList->takeItem(pItem);
					m_pList->insertItem(pItem);
				}
				m_pList->setSelected(pItem, true);
				emit StateChanged();
			}
		}
		break;
	case 5: //Descendre
		{
			QListViewItem* pItem = m_pList->selectedItem();
			if(pItem && pItem->itemBelow())
			{
				pItem->moveItem(pItem->itemBelow());
				emit StateChanged();
			}
		}
	}

	RefreshPopupMenu(m_pList->currentItem());
	m_pCommButton->setEnabled(m_pList->childCount());
}

//-----------------------------------------------------------------------------------
void CFXWindow::OnExtractIcon(int id)
{
	QListViewItem* item = m_pList->selectedItem();
	if(!item)
	{
		return;
	}
	int pos;
	for(pos = 0; pos < 7 && tab[pos].item != item; pos++);
	if(id < 0 || tab[pos].nNbIcon <= id)
	{
		return;
	}

	QString s = CMessageBox::ChooseSaveFileName(QString::null, "*.bmp", GetMainWidget(), "bmp");
	if(s.isEmpty()) {
		return;
	}
	HND h = OpenDescModeRdWr(s);
	if(h == INVALID_HANDLE_VALUE) {
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(s, TRUE));
	}
	else {
		WriteDesc(h, tab[pos].pIcon[id], 138);
	}
	CloseDesc(h);
}

//-----------------------------------------------------------------------------------
void CFXWindow::ExtractDrive()
{
	QListViewItem* i = m_pList->selectedItem();
	if(!i)
	{
		return;
	}
	QString name = CMessageBox::ChooseSaveFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER), GetMainWidget(), "lec");
	if(name.isEmpty()){
		return;
	}

	unsigned int nSize = i->text(1).toInt();
	string filepath = name;
	if(nSize > DRIVE_SIZE){
		filepath = CDriveAccess::ChangeFileName(filepath, 1);
	}
	HND h = OpenDescModeRdWr(filepath.c_str());
	if(h == INVALID_HANDLE_VALUE){
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(filepath.c_str(), TRUE));
	}
	else{
		int nNumDrive = 1;
		int pos;
		for(pos = 0; pos < 7 && tab[pos].item != i; pos++);

		QProgressDialog* pProgressDialog = CMessageBox::CreateProgressDialog(GetMainWidget(), CLanguage::GetWordTranslation(CLanguage::WO_WRITING), nSize, CLanguage::GetWordTranslation(CLanguage::WO_EXTRACT));
		unsigned int nBytesWrote = 0;
		unsigned int nStep       = 32768;
		bool bOk = TRUE;
		while(bOk && nBytesWrote < nSize)
		{
			if(nBytesWrote + nStep > nSize)
			{
				nStep = nSize - nBytesWrote;
			}
			bOk = CDriveAccess::AccessInDrive(h, filepath, tab[pos].pDrive + nBytesWrote, nStep, nNumDrive, TRUE);
			nBytesWrote += nStep;
			pProgressDialog->setProgress(nBytesWrote);
			qApp->processEvents();
		}
		CloseDesc(h);
		delete pProgressDialog;

	}
}

//-----------------------------------------------------------------------------------
void CFXWindow::Refresh()
{
	if(m_pList->childCount() == 0) {
		if(m_pDriveWindow) {
			ExtraParam param;
			param.nMaxSize         = -1;
			param.bReadOnly        = TRUE;
			param.bShowGraphWidget = FALSE;
			param.pBuffer          = NULL;
			param.bShowOpenButton  = FALSE;
			m_pDriveWindow->Init(QString::null, 0, &param);
		}
		m_pIconView->clear();
	}
	QString s = tr("Ncessite ");
	int nSize = nTotalSize / DRIVE_SIZE;
	if(nSize)
	{
		s += QString::number(nSize) + " ";
		if(nSize > 1)
		{
			s += CLanguage::GetWordTranslation(CLanguage::WO_DRIVES).lower();
		}
		else
		{
			s += CLanguage::GetWordTranslation(CLanguage::WO_DRIVE).lower();
		}
	}
	else
	{
		s += tr("aucun lecteur de libre");
	}
	m_pLblInfo->setText(s);
}

//-----------------------------------------------------------------------------------
int CFXWindow::ChooseDrive(QWidget* parent, const QString& sCaption, const QString& sLabel, int nCurrent)
{
	bool bOk;
	QStringList lDrives;
	for(int i = 1; i < 7; i++)
	{
		lDrives << CDriveAccess::GetDriveName(i);
	}
	QString sDrive = QInputDialog::getItem(sCaption, sLabel, lDrives, nCurrent - 1, FALSE, &bOk, parent);
	if(!bOk)
	{
		return -1;
	}
	return lDrives.findIndex(sDrive) + 1;
}

//-----------------------------------------------------------------------------------
void CFXWindow::StartComm()
{
	unsigned char buffer[DRIVE_SIZE];
	int i;
	CTransfertWindow* pTransfert = GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(GetMainWidget());
	if(!pTransfert){
		return;
	}

	//nBytes : nbre d'octets recus et envoys
	int nBytes = pTransfert->Receiving(0, buffer);//Recoit le systeme
	if(nBytes == -1){
		delete pTransfert;
		return;
	}	
	//Fin reception du systeme

	int nDrive   = 1; //Lecteur L: lecteur physique
	int nLogical = 1; //Lecteur L: lecteur logique
	unsigned int options = m_pCFXSettings->GetPreference();
	CCasioSystem system;
	system.OpenDriveFile(buffer, sizeof(buffer));
	int cpt = 0;
	for(i = 0; i < 7; i++){
		if(tab[i].item != NULL) {
			cpt += tab[i].nNbIcon;
		}
	}
	if(system.GetNbIcon() + cpt > CCasioSystem::MAXICON) {
		CMessageBox::MsgCritical(GetMainWidget(), tr("Vous n'avez pas assez de place dans votre menu!"));
	}
	else
	{
		bool stopped = false;
		if(options != 2) // si =2 ne rien vrifier
		{
			uchar* buf = new uchar[DRIVE_SIZE];
			int nD = -1;
			int nL;
			while(nDrive < 7 && !stopped)
			{
				int r = pTransfert->Receiving(nDrive, buf);
				if(r == -1) {
					stopped = true;
				}
				else
				{
					nBytes += r;
					//Verification du contenu
					if(CDriveAccess::IsEmpty(buf, DRIVE_SIZE)) {
						if(nD == -1) {
							nD = nDrive;
							nL = nLogical;
						}
						if(options)	{//Si s'arrter au premier lecteur vide
							break;
						}
					}
					else {
						//Effacement des lecteurs potentiellement trouvs
						//Option tout vrifier
						nD = -1;
					}
					CROMDISKDrive drive;
					CCasioBackup backup;
					if(drive.OpenDriveFile(buf, DRIVE_SIZE) > 0 && drive.IsValid()) {
						nDrive += drive.GetNumDrive();
						nLogical++;
					}
					else
					{
						if(backup.OpenDriveFile(buf, DRIVE_SIZE) > 0 && backup.IsValid()) {
							nDrive += backup.GetNumDrive();
						}
						else {
							nDrive++;
						}
					}
				}
			}
			delete buf;
			if(!options)
			{
				if(nD == -1) {
					nDrive = 7; //Pas de place
				}
				else {
					 nDrive   = nD;
					 nLogical = nL;
				}
			}
		}
		else
		{			
			nDrive = ChooseDrive(GetMainWidget(), tr("Choisissez"), tr("Lecteur physique"), 0);
			if(nDrive > 0){
				nLogical = ChooseDrive(GetMainWidget(), tr("Choisissez"), tr("Lecteur logique"), nDrive);
			}
			stopped = (nDrive < 1 || nLogical < 1);
		}
		if(!stopped)
		{
			if(nDrive + nTotalSize / DRIVE_SIZE > 7)
			{
				CMessageBox::MsgCritical(GetMainWidget(), tr("Vous n'avez pas assez de lecteur libre!"));
			}
			else
			{
				//Modification des Infos
				QListViewItem* j = m_pList->firstChild();
				char m_pDriveList[] = "L:\\";
				m_pDriveList[0] = nLogical + 'K';
				int nNbIcon = system.GetNbIcon();
				int nNumIcon = 0;
				CCasioSystem::TInfo info;
				for(i = 0; j; i++)
				{
					int pos;
					uchar buf[76];
					for(pos = 0; pos < 7 && tab[pos].item != j; pos++);
					CCasioSystem::ClearZoneInfo(info);
					info.bmp = CCasioSystem::IV_SHOW;
					CCasioSystem::CpyChar(info.lecteur, m_pDriveList, sizeof(info.lecteur));
					QString s(tab[pos].pInfo[nNumIcon]);
					s = s.stripWhiteSpace();
					QString bmp;
					if(s.findRev('.') != -1){
						int n = s.findRev('.');
						bmp = s.left(n) + ".BMP";
					}
					else{
						bmp = s + ".BMP";
						s  += ".EXE";
					}
					CCasioSystem::CpyChar(info.name, s, sizeof(info.name));
					CCasioSystem::CpyChar(info.image, bmp, sizeof(info.image));
					CCasioSystem::CpyChar(info.version, "1.00", sizeof(info.version));
					ConvertBmpToBuf(tab[pos].pIcon[nNumIcon], buf, 138, 30, 19);
					system.AddIcon(&info, buf, true, true, true);
					nNumIcon++;
					if(nNumIcon >= tab[pos].nNbIcon){
						nNumIcon = 0;
						j = j->nextSibling();
						m_pDriveList[0]++;
					}
				}			
				//Ajoute l'envoi du menu + les lecteurs
				nBytes += pTransfert->GetUsefulSize(buffer);
				j = m_pList->firstChild();
				while(j)
				{
					int size = j->text(1).toInt();
					int pos;
					for(pos = 0; pos < 7 && tab[pos].item != j; pos++);
					for(int n = 0; n < size; n += DRIVE_SIZE)
					{
						nBytes += pTransfert->GetUsefulSize(tab[pos].pDrive + n);
					}
					j = j->nextSibling();
				}
				pTransfert->SetData(nBytes, false);

				//Envoie le menu
				if(pTransfert->Sending(0, buffer) < 1)
				{
					delete pTransfert;
					return;
				}

				//Envoie les lecteurs
				j = m_pList->firstChild();
				while(j)
				{
					int size = j->text(1).toInt();
					int step = 0;
					int pos;
					for(pos = 0; pos < 7 && tab[pos].item != j; pos++);
					while(size > step)
					{
						if(pTransfert->Sending(nDrive, tab[pos].pDrive + step) < 1)
						{
							delete pTransfert;
							return;
						}
						step += DRIVE_SIZE;
						nDrive++;
					}
					j = j->nextSibling();
				}
			}
		}
		pTransfert->CloseComm(1);
	}
	delete pTransfert;
}

//-----------------------------------------------------------------------------------
int CFXWindow::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1)
	{
		m_pList->setColumnWidth(0, element.attribute("WidthCol0", "100").toInt());
		m_pList->setColumnWidth(1, element.attribute("WidthCol1", "80").toInt());
		m_sLastOpenedFile = ConvertToFullPath(element.attribute("m_sLastOpenedFile"));

		QDomNode n = element.firstChild();
		while(!n.isNull())
		{
			QDomElement e = n.toElement();
			if(e.nodeName() == "File")
			{
				QString sPath = ConvertToFullPath(e.attribute("Path"));
				if(!sPath.isEmpty()) {
					AddFile(sPath);
				}
			}
			n = n.nextSibling();
		}
		return 1;
	}
	return -1;	
}

//-----------------------------------------------------------------------------------
int CFXWindow::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1)
	{
		element.setAttribute("WidthCol0", QString::number(m_pList->columnWidth(0)));
		element.setAttribute("WidthCol1", QString::number(m_pList->columnWidth(1)));
		element.setAttribute("m_sLastOpenedFile", ConvertToRelativePath(m_sLastOpenedFile));
	
		QListViewItem* pItem = m_pList->firstChild();
		while(pItem)
		{
			QDomElement file = element.ownerDocument().createElement("File");
			element.appendChild(file);
			file.setAttribute("Path", ConvertToRelativePath(pItem->text(2)));
			pItem = pItem->nextSibling();
		}		
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void CFXWindow::DoShowCaption()
{
	setCaption(CAPTION);
}

//-----------------------------------------------------------------------------------
QPixmap CFXWindow::GetDefaultCFXPixmap()
{
	return QPixmap::fromMimeSource(IMG_CFX);
}

//-----------------------------------------------------------------------------------
void CFXWindow::OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem)
{
	for(QStringList::const_iterator it = sFiles.begin(); it != sFiles.end() && AddFile(*it) > 0; ++it);
}

//-----------------------------------------------------------------------------------
bool CFXWindow::ConvertBmpToBuf(const uchar* bmp, uchar* buf, const int& nSize, int width, int height)
{
	int w,h;
	if(!buf)
	{
		return false;
	}
	memcpy(&w, bmp + 18, 4);
	memcpy(&h, bmp + 22, 4);
	uchar mask;
	memcpy(&mask, bmp + 58, 1); //Couleur
	w = ConvertWidth(w);
	int nRead = nSize - 62;// Enleve le header
	if(nRead <=0)
	{
		return false;
	}
	if(nRead > w * h)
	{
		nRead = w * h;
	}
	const uchar* temp = bmp + 62;
	if(height < 0)
	{
		height = h;
	}
	if(width < 0)
	{
		width = w;
	}
	width = ConvertWidth(width);
	int minWidth = width;
	int minHeight = height;
	if(w < minWidth)
	{
		minWidth = w;
	}
	if(h < minHeight)
	{
		minHeight = h;
	}
	for(int i = 0; i < minHeight; i++)
	{
		for(int k = 0; k < minWidth; k++)
		{
			buf[k + i * width] = temp[k + (h - i - 1) * w] ^ mask;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
int CFXWindow::ConvertWidth(int w)
{
	if(w / 8 < w / 8.0f){
		 return w / 8 + 1;
	}
	else{
		return  w / 8;
	}
}
