
#ifndef CFXWINDOW_H
#define CFXWINDOW_H

#include <ToolWindow.h>

class QGroupBox;
class QToolButton;
class QPopupMenu;
class CLabel;
class QIconView;
class QIconViewItem;
class QTabWidget;
class CCFXSettings;
class QListViewItem;
class QListView;

#define CAPTION "Add-in Installer Emulator"

class CFXWindow : public CToolWindow
{ 
    Q_OBJECT
protected:
	struct TCFX
	{
		QListViewItem* item;
		uchar** pIcon;
		char** pInfo;
		uchar* pDrive;
		int nNbIcon;
	};

public:
	CFXWindow(CCFXSettings* pCFXSettings, CONSTRUCTOR_PARAMDEF);
	virtual ~CFXWindow();
	int                 Init(INIT_PARAMDEF);
	int					AddFile(ADDFILE_PARAMDEF);
	int					LoadFrom(QDomElement& element);
	int					SaveTo(QDomElement& element);
	static QPixmap		GetDefaultCFXPixmap();
	bool				OnlyOneInstance(){return  TRUE;}

private:
	static unsigned int ConvertToInt(unsigned char* buf);
	static void			DeleteTCFX(TCFX& cfx);
	static bool         ConvertBmpToBuf(const uchar* bmp, uchar* buf, const int& nSize, int width, int height);
	static int          ConvertWidth(int w);
	static int          ChooseDrive(QWidget* parent, const QString& sCaption, const QString& sLabel, int nCurrent);

protected:
	void				DoShowCaption();
	void				DoInitWords();
	void				RefreshPopupMenu(QListViewItem* pItemSelected);

protected slots:
	void				OnMenuActivated(int id);
	void				OnExtractIcon(int id);
	void				OnSelectionChanged(QListViewItem*);
	void				OnListClicked(int button, QListViewItem* i, const QPoint & p);
	void				OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem);
	
private slots:
	void				ShowOption();
	void				StartComm();

private:
	void				Refresh();
	void				ExtractDrive();

private:
	CCFXSettings* m_pCFXSettings;
	QGroupBox*	  frame[2];
	QListView*	  m_pList;
	QToolButton*  m_pCommButton;
	QToolButton*  m_pOptionsButton;
	QPopupMenu*   m_pPopupMenu;
	QPopupMenu*	  m_pIconMenu;
	CLabel*		  m_pLblInfo;
	QString		  m_sPortName;
	QString		  m_sLastOpenedFile;
	int			  nTotalSize;
	int			  nCable;
	TCFX		  tab[7];
	QIconView*	  m_pIconView;
	QString		  m_sTmpFile;
	CToolWindow*  m_pDriveWindow;
};

#endif // CFXWINDOW_H
