// BASICEditor.cpp: implementation of the CBASICEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "BASICEditor.h"
#include "DriveWindow.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qheader.h>
#include <qinputdialog.h>

#include <language.h>
#include <MessageBox.h>
#include <BASICDrive.h>
#include <DriveAccess.h>
#include <IOptionWindow.h>

//-----------------------------------------------------------------------------------
CBASICEditor::CBASICEditor(DriveWindow* parent)
:CEditor(parent)
{
	m_bViewAllFile = FALSE;
	QGridLayout* pLayout = new QGridLayout(this, 3, 2, 5);

	m_pList         = new QListView(this);
	m_pOptimization = new QPushButton(this);
	m_pToggleView   = new QPushButton(this);
	m_pPopupMenu    = new QPopupMenu(this);

	pLayout->addMultiCellWidget(m_pList, 0, 2, 0, 0);

	pLayout->addWidget(m_pOptimization, 0, 1);
	pLayout->addWidget(m_pToggleView,   1, 1);

	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);
	m_pList->header()->setClickEnabled (FALSE, 0);
	m_pList->header()->setClickEnabled (FALSE, 1);
	m_pList->header()->setClickEnabled (FALSE, 2);
	m_pList->setColumnWidth(0, 100);
	m_pList->setColumnWidth(1, 50);
	m_pList->setColumnWidth(2, 100);
	m_pList->setSorting(-1);// dsactive le tri
	m_pList->setAllColumnsShowFocus( TRUE );
	m_pList->setColumnAlignment(1, AlignRight);
	m_pList->setColumnAlignment(2, AlignCenter);

	for(int i = 0; i < 9; i++) {
		m_pPopupMenu->insertItem(QString::null, i);
	}
	
	RefreshPopupMenu(NULL);

	m_pPopupMenu->insertSeparator(3);
	m_pPopupMenu->insertSeparator(6);
	m_pPopupMenu->insertSeparator(8);
	m_pPopupMenu->insertSeparator(10);

	m_pPopupMenu->setAccel(CTRL  + Key_R,		0);
	m_pPopupMenu->setAccel(CTRL  + Key_P,		1);
	m_pPopupMenu->setAccel(CTRL  + Key_Delete,  2);
	m_pPopupMenu->setAccel(Key_Delete,			3);
	m_pPopupMenu->setAccel(CTRL  + Key_B,		4);
	m_pPopupMenu->setAccel(SHIFT + Key_Delete,  5);
	m_pPopupMenu->setAccel(CTRL  + Key_A,       6);
	m_pPopupMenu->setAccel(CTRL  + Key_E,       7);
	m_pPopupMenu->setAccel(CTRL  + Key_I,       8);

	DoInitWords();

	connect(m_pList, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)), SLOT(OnListClicked(QListViewItem*, const QPoint&)));
	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(RefreshPopupMenu(QListViewItem*)));
	connect(m_pList, SIGNAL(itemRenamed(QListViewItem*, int)), SLOT(OnItemRenamed(QListViewItem*, int)));
	connect(m_pToggleView,   SIGNAL(clicked()), SLOT(OnToggleView()));
	connect(m_pOptimization, SIGNAL(clicked()), SLOT(OnOptimization()));

	connect(m_pPopupMenu, SIGNAL(activated(int)), SLOT(OnPopupMenuActivated(int)));
}

//-----------------------------------------------------------------------------------
int	CBASICEditor::DoReloadFile()
{
	m_pList->clear();
	CBASICDrive* pDrive = NULL;
	int nRet = CreateDrive(pDrive);
	if(nRet < 1) {
		return nRet;
	}

	CBASICFileInfo* pInfo = pDrive->GetFirstFile();
	QListViewItem* pItem = NULL;
	
	while(pInfo){
		if(m_bViewAllFile || pInfo->IsVisible()) {
			pItem = new QListViewItem(m_pList, pItem);
			pItem->setText(0, pInfo->GetName().c_str());
			pItem->setText(1, QString::number(pInfo->GetSize()));
			pItem->setText(2, pInfo->GetPassword().c_str());
			pItem->setText(3, QString::number(pInfo->GetAddress()));
			pItem->setRenameEnabled(0, TRUE);
			pItem->setRenameEnabled(2, TRUE);
			if(!pInfo->IsVisible()) {
				SetGarbageTo(pItem);
			}
		}
		pInfo = pDrive->GetNextFile(pInfo, TRUE);
	}
	bool bOptimizable = pDrive->IsOptimizable();
	m_pToggleView->setEnabled(bOptimizable);
	m_pOptimization->setEnabled(bOptimizable && !m_pParent->m_bReadOnly);
	DeleteDrive(pDrive);

	return nRet;
}

//-----------------------------------------------------------------------------------
int CBASICEditor::CreateDrive(CBASICDrive*& pDrive, bool bWriteMode)
{
	bool bVariableSize = m_pParent->GetMainWindow()->GetOptionWindow()->GetPreference(IOptionWindow::PREF_DRIVE_VARIABLE_SIZE)
		&& m_pParent->m_nShift == 0;
	pDrive = new CBASICDrive(m_pParent->m_bOneFile, bVariableSize);

	if(m_pParent->m_pDriveBuffer) {
		pDrive->OpenDriveFile(m_pParent->m_pDriveBuffer, m_pParent->m_nMaxSize);
	}
	else {
		string sFileName = m_pParent->m_sFileName;
		if(pDrive->OpenDriveFile(sFileName, m_pParent->m_nShift, bWriteMode) < 1)
		{
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_pParent->m_sFileName, bWriteMode));//L'ouverture a echoue
			delete pDrive;
			pDrive = NULL;
			return 0;//Pour viter de recharger le lecteur
		}
	}
	if(!pDrive->IsValid()) {
		delete pDrive;
		pDrive = NULL;
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CBASICEditor::DoInitWords()
{
	m_pList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pList->setColumnText(1, CLanguage::GetWordTranslation(CLanguage::WO_SIZE));
	m_pList->setColumnText(2, tr("Mot de passe"));
	m_pOptimization->setText(tr("Optimiser"));
	if(m_bViewAllFile){
		m_pToggleView->setText(tr("Masquer"));	
	}
	else {
		m_pToggleView->setText(tr("Voir tout"));
	}
}

//-----------------------------------------------------------------------------------
void CBASICEditor::RefreshPopupMenu(QListViewItem* pItemSelected)
{
	QString s;
	m_pPopupMenu->setItemEnabled(0, !m_pParent->m_bReadOnly);
	m_pPopupMenu->setItemEnabled(1, !m_pParent->m_bReadOnly);
	m_pPopupMenu->setItemEnabled(5, !m_pParent->m_bReadOnly);
	m_pPopupMenu->setItemEnabled(6, !m_pParent->m_bReadOnly);
	if(pItemSelected)
	{
		s = " '" + pItemSelected->text(0) + "'";
		m_pPopupMenu->setItemEnabled(2, !pItemSelected->text(2).isEmpty() && !m_pParent->m_bReadOnly);
		m_pPopupMenu->setItemEnabled(3, !pItemSelected->pixmap(0) && !m_pParent->m_bReadOnly);
		m_pPopupMenu->setItemEnabled(4,  pItemSelected->pixmap(0) && !m_pParent->m_bReadOnly);
		m_pPopupMenu->setItemEnabled(7, pItemSelected->text(1).toInt() > CBASICFileInfo::MIN_FILE_SIZE);
		m_pPopupMenu->setItemEnabled(8, !m_pParent->m_bReadOnly);
	}
	else
	{
		m_pPopupMenu->setItemEnabled(0, FALSE);
		m_pPopupMenu->setItemEnabled(1, FALSE);
		m_pPopupMenu->setItemEnabled(2, FALSE);
		m_pPopupMenu->setItemEnabled(3, FALSE);
		m_pPopupMenu->setItemEnabled(4, FALSE);
		m_pPopupMenu->setItemEnabled(5, FALSE);
		m_pPopupMenu->setItemEnabled(7, FALSE);
		m_pPopupMenu->setItemEnabled(8, FALSE);
	}
	m_pPopupMenu->changeItem(0, CLanguage::GetWordTranslation(CLanguage::WO_RENAME) + s);
	m_pPopupMenu->changeItem(1, tr("Changer le mot de passe"));
	m_pPopupMenu->changeItem(2, tr("Enlever le mot de passe"));
	m_pPopupMenu->changeItem(3, tr("Supprimer partiellement") + s);
	m_pPopupMenu->changeItem(4, tr("Rcuprer") + s);
	m_pPopupMenu->changeItem(5, tr("Supprimer dfinitivement") + s);
	m_pPopupMenu->changeItem(6, tr("Ajouter un fichier"));
	m_pPopupMenu->changeItem(7, tr("Exporter les donnes du fichier") + s);
	m_pPopupMenu->changeItem(8, tr("Importer des donnes dans le fichier") + s);
}

//-----------------------------------------------------------------------------------
void CBASICEditor::OnOptimization()
{
	m_pParent->StopChecking();
	CBASICDrive* pDrive = NULL;
	if(CreateDrive(pDrive, TRUE) == 1){
		pDrive->Optimization();
		DeleteDrive(pDrive);
		DoReloadFile();
	}
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
void CBASICEditor::OnItemRenamed(QListViewItem* pItem, int nCol)
{
	EditDrive(pItem, nCol >> 1);
}

//-----------------------------------------------------------------------------------
void CBASICEditor::OnPopupMenuActivated(int id)
{
	QListViewItem* pItem = m_pList->selectedItem();
	if(pItem && (id == 0 || id == 1)) {
		pItem->startRename(id << 1);
	}
	else {
		EditDrive(pItem, id);
	}
}

//-----------------------------------------------------------------------------------
void CBASICEditor::OnListClicked(QListViewItem* pItem, const QPoint& p)
{
	RefreshPopupMenu(pItem);
	m_pPopupMenu->exec(p);
}

//-----------------------------------------------------------------------------------
void CBASICEditor::OnToggleView()
{
	m_bViewAllFile = !m_bViewAllFile;
	DoInitWords();
	DoReloadFile();
}

//-----------------------------------------------------------------------------------
void CBASICEditor::SetGarbageTo(QListViewItem* pItem)
{
	static const unsigned char garbage[] = {137, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 15, 0, 0, 0, 16, 8, 3, 0, 0, 0, 254, 136, 213, 54, 0, 0, 0, 45, 80, 76, 84, 69, 12, 30, 225, 43, 169, 255, 45, 46, 78, 112, 112, 130, 251, 244, 240, 207, 206, 207, 177, 177, 177, 255, 248, 207, 207, 200, 255, 140, 140, 211, 239, 208, 192, 130, 200, 255, 218, 225, 234, 149, 145, 146, 255, 255, 255, 199, 47, 119, 233, 0, 0, 0, 15, 116, 82, 78, 83, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 0, 212, 220, 152, 161, 0, 0, 0, 25, 116, 69, 88, 116, 83, 111, 102, 116, 119, 97, 114, 101, 0, 65, 100, 111, 98, 101, 32, 73, 109, 97, 103, 101, 82, 101, 97, 100, 121, 113, 201, 101, 60, 0, 0, 0, 131, 73, 68, 65, 84, 120, 156, 77, 143, 65, 146, 4, 33, 8, 4, 65, 176, 84, 180, 252, 255, 115, 7, 122, 123, 39, 166, 78, 36, 25, 16, 32, 55, 99, 111, 178, 148, 123, 73, 144, 52, 2, 217, 145, 203, 209, 90, 51, 91, 192, 162, 53, 121, 145, 92, 68, 103, 177, 21, 27, 123, 159, 40, 207, 24, 149, 0, 58, 202, 175, 62, 207, 57, 93, 69, 182, 148, 199, 60, 195, 93, 117, 137, 60, 156, 222, 61, 52, 60, 68, 248, 120, 31, 91, 143, 251, 143, 223, 170, 42, 30, 203, 190, 94, 115, 96, 9, 255, 253, 206, 245, 16, 228, 189, 175, 143, 106, 88, 253, 243, 236, 63, 30, 147, 252, 251, 15, 189, 130, 162, 123, 63, 58, 22, 6, 116, 15, 130, 216, 132, 0, 0, 0, 0, 73, 69, 78, 68, 174, 66, 96, 130, 0};
	QPixmap pixmap;
	pixmap.loadFromData(garbage, sizeof(garbage));
	pItem->setPixmap(0, pixmap);
}

//-----------------------------------------------------------------------------------
void CBASICEditor::EditDrive(QListViewItem* pItem, int nAction)
{
	CBASICDrive* pDrive = NULL;
	bool bNeedReload    = false;
	if(CreateDrive(pDrive, nAction != 7) < 1 || pDrive == NULL){
		return;
	}
	CBASICFileInfo* pInfo = NULL;
	if(nAction != 6) {
		pInfo = pDrive->FindFile(pItem->text(3).toInt());
	}
	else{
		QString s = QInputDialog::getText(tr("Ajouter un fichier"), tr("Entrez le nom du fichier"));
		if(!s.isEmpty()) {
			pInfo = pDrive->AddFile(s.latin1());
		}
	}
	if(pInfo != NULL) {
		switch(nAction)
		{
		case 3: //Marquer supprim
			pInfo->SetVisibility(false);
			break;
		case 5: //Supprimer dfinitivement
			pDrive->RemoveFile(pInfo);
			break;
		case 4: //Rcuprer
			pInfo->SetVisibility(true);
			break;
		case 2: //Enlever le mot de passe
			pInfo->SetPassword("");
			break;
		case 1: //Mot de passe
			pInfo->SetPassword(pItem->text(2).latin1());
			break;
		case 0: //Renommer le programme
			pInfo->Rename(pItem->text(0).latin1());
			break;
		case 7: //Exporter des donnes
			{
				QString sFile = CMessageBox::ChooseSaveFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), this);
				if(!sFile.isEmpty()){
					HND h = OpenDescModeRdWr(sFile);
					if(h != INVALID_HANDLE_VALUE) {
						char* pData = new char[pInfo->GetDataSize()];
						if(pInfo->GetData(pData) == 1) {
							WriteDesc(h, pData, pInfo->GetDataSize());
						}
						delete[] pData;
						CloseDesc(h);
					}
					else{
						CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFile, true));
					}
				}
			}
			break;
		case 8: //Importer des donnes
			{
				QString sFile = CMessageBox::ChooseOpenFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), this);
				if(!sFile.isEmpty()){
					HND h = OpenDescModeRead(sFile);
					if(h != INVALID_HANDLE_VALUE) {
						unsigned int nSize = FileSize(h);
						if(nSize > CBASICFileInfo::MAX_DATA_SIZE) {
							CMessageBox::MsgInformation(m_pParent->GetMainWidget(), tr("La taille du fichier '%1' a t rduit  la limite qui est de %2 octets.").arg(pInfo->GetName().c_str()).arg(CBASICFileInfo::MAX_FILE_SIZE));
							nSize = CBASICFileInfo::MAX_DATA_SIZE;
						}
						char* pData = new char[nSize];
						if(ReadDesc(h, pData, nSize)) {
							pInfo->SetData(pData, nSize);
						}
						delete[] pData;
						CloseDesc(h);
					}
					else{
						CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFile));
					}
				}
			}
			break;
		}
		m_pParent->StopChecking();
		RefreshPopupMenu(m_pList->currentItem());
		bNeedReload = nAction > 2 && nAction != 7;
		if(!bNeedReload) {
			pItem->setText(0, pInfo->GetName().c_str());
			pItem->setText(1, QString::number(pInfo->GetSize()));
			pItem->setText(2, pInfo->GetPassword().c_str());
		}
		pDrive->FreeFileInfo(pInfo);
	}
	else {
		m_pParent->StopChecking();
	}
	DeleteDrive(pDrive);

	if(bNeedReload) {
		DoReloadFile();
	}
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
QString CBASICEditor::GetCaption() const
{
	return tr("Lecteur BASIC");
}
