
#include "DriveWindow.h"

#include <qdir.h>
#include <qtoolbutton.h>

#include <language.h>
#include <MessageBox.h>
#include <image_defs.h>
#include <DriveAccess.h>
#include <SendToMenu.h>

#include "Editor.h"
#include "ROMDISKEditor.h"
#include "BASICEditor.h"
#include "defs.h"

#define	CALL_MSG_ERR CMessageBox::MsgCritical(GetMainWidget(), DriveWindow::tr("Le fichier '%1' n'est pas un lecteur valide ou son format n'est pas connu.").arg(m_sFileName))

//-----------------------------------------------------------------------------------
DriveWindow::DriveWindow(CONSTRUCTOR_PARAM)
:CheckFileWindow(CALL_PARAM, TRUE, FALSE)
{
	m_pDefaultWidget = new QWidget(this);

	SetCentralWidget(m_pDefaultWidget);
	m_pDefaultWidget->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

	setIcon(QPixmap::fromMimeSource(IMG_DRIVE));
	resize(530, 270);
	m_pEditor      = NULL;
	m_pDriveBuffer = NULL;
	m_pSendTo      = new QToolButton(GetMainWidget());
	m_pOpen        = new QToolButton(GetMainWidget());

	m_pOpen->setIconSet(QPixmap::fromMimeSource(IMG_OPEN));
	m_pOpen->setAutoRaise(TRUE);
	m_pSendTo->setIconSet(QPixmap::fromMimeSource(IMG_SENDTO));
	m_pSendTo->setAutoRaise(TRUE);

	if(parentWidget() == NULL) {
		AddLeftSeparator();
	}
	AddLeftWidget(m_pOpen);
	AddLeftWidget(m_pSendTo);

	connect(m_pOpen,   SIGNAL(clicked()), SLOT(OnOpen()));
	connect(m_pSendTo, SIGNAL(clicked()), SLOT(OnSendTo()));

	DoInitWords();
}

//-----------------------------------------------------------------------------------
int DriveWindow::Init(INIT_PARAM)
{
	m_nShift = nShift;
	if(pExtraParam) {
		ExtraParam* param  = (ExtraParam*)pExtraParam;
		m_pDriveBuffer     = param->pBuffer;
		m_bReadOnly        = param->bReadOnly;
		m_nMaxSize         = param->nMaxSize;
		m_bShowGraphWidget = param->bShowGraphWidget;
		m_bOneFile         = TRUE;
		if(!param->bShowOpenButton) {
			m_pOpen->hide();
		}
		else {
			m_pOpen->show();
		}
	}
	else {
		m_nMaxSize         = -1;
		m_pDriveBuffer     = NULL;
		m_bReadOnly        = FALSE;		
		m_bShowGraphWidget = TRUE;
	}

	if(m_pDriveBuffer != NULL) {
		return DoReloadFile(false);
	}
	if(!sFileName.isEmpty())
	{
		if(!SetFileName(sFileName, FALSE)) {
			return 0;
		}
		string s   = sFileName;
		m_bOneFile = CDriveAccess::IsOnlyOneFile(s) && FileSize(s.c_str(), NULL) > DRIVE_SIZE;
		return DoReloadFile(false);
	}
	CloseEditor();
	m_pSendTo->hide();
	return 1;
}

//-----------------------------------------------------------------------------------
void DriveWindow::CloseEditor()
{
	if(m_pEditor != NULL) {
		m_pEditor->close(TRUE);
		SetCentralWidget(m_pDefaultWidget);
		m_pDefaultWidget->show();
		m_pEditor = NULL;
	}
}

//-----------------------------------------------------------------------------------
int DriveWindow::DoReloadFile(bool bShowValidErr)
{
	if(m_pDriveBuffer != NULL) {
		m_pSendTo->hide();
	}
	else{
		m_pSendTo->show();
	}
	DoShowCaption();
	int nRet;
	if(m_pEditor != NULL) {
		nRet = m_pEditor->DoReloadFile();
		if(nRet >= 0) {
			return nRet;
		}
		CloseEditor();
	}
	for(int i = 0; i < 2; i++) {
		switch(i)
		{
		case 0:
			m_pEditor = new CROMDISKEditor(this);
			break;
		case 1:
			m_pEditor = new CBASICEditor(this);
			break;
		}
		nRet = m_pEditor->DoReloadFile();
		if(nRet > 0) {
			SetCentralWidget(m_pEditor);
			m_pEditor->show();
			m_pDefaultWidget->hide();
			DoShowCaption();
			return nRet;
		}
		if(nRet == 0) {
			m_pSendTo->hide();
			return nRet;
		}
		m_pEditor->close(TRUE);
	}
	m_pSendTo->hide();
	m_pEditor = NULL;
	if(bShowValidErr) {
		CALL_MSG_ERR;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void DriveWindow::DoShowCaption()
{
	if(m_pEditor != NULL) {
		setCaption(m_pEditor->GetCaption());
	}
	else {
		setCaption(CLanguage::GetWordTranslation(CLanguage::WO_DRIVE));
	}
	CheckFileWindow::DoShowCaption();
	if(m_bReadOnly) {
		setCaption(caption() + " [" + tr("Lecture seule") + "]");
	}
}

//-----------------------------------------------------------------------------------
void DriveWindow::DoInitWords()
{
	m_pSendTo->setTextLabel(CLanguage::GetWordTranslation(CLanguage::WO_SENDTO), TRUE);
	m_pOpen->setTextLabel(tr("Ouvrir un fichier"), TRUE);
	if(m_pEditor != NULL) {
		m_pEditor->DoInitWords();
	}
}

//-----------------------------------------------------------------------------------
int DriveWindow::LoadFrom(QDomElement& element)
{
	if(CheckFileWindow::LoadFrom(element) == 1 && !m_sFileName.isEmpty()) {
		m_nShift    = element.attribute("Shift",    "0").toInt();
		m_bOneFile  = element.attribute("OneFile",  "0").toInt();
		m_nMaxSize  = element.attribute("MaxSize", "-1").toInt();
		m_bReadOnly = element.attribute("ReadOnly", "0").toInt();
		return DoReloadFile(true);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int DriveWindow::SaveTo(QDomElement& element)
{
	if(CheckFileWindow::SaveTo(element) == 1) {
		element.setAttribute("Shift",    QString::number(m_nShift));
		element.setAttribute("OneFile",  QString::number(m_bOneFile));
		element.setAttribute("MaxSize",  QString::number(m_nMaxSize));
		element.setAttribute("ReadOnly", QString::number(m_bReadOnly));
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void DriveWindow::OnSendTo()
{
	GetMainWindow()->GetSendToMenu()->SetFileToSend(m_sFileName, 1);
	GetMainWindow()->GetSendToMenu()->popup(m_pSendTo->mapToGlobal(QPoint(0, m_pSendTo->height())));
}

//-----------------------------------------------------------------------------------
void DriveWindow::OnOpen()
{
	QString sFile = CMessageBox::ChooseOpenFileName(m_sFileName, CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER), this);
	if(!sFile.isEmpty()) {
		if(Init(sFile) < 0) {
			CALL_MSG_ERR;
		}
	}
}

//-----------------------------------------------------------------------------------
void DriveWindow::OnFileRemoved()
{
	CheckFileWindow::OnFileRemoved();
	Init(QString::null);
}
