// GraphWidget.cpp: implementation of the CGraphWidget class.
//
//////////////////////////////////////////////////////////////////////

#include "GraphWidget.h"

#include <qpainter.h>
#include <qfont.h>
#include <qfontmetrics.h>

//-----------------------------------------------------------------------------------
CGraphWidget::CGraphWidget(QWidget* parent)
:QWidget(parent)
{
	m_nSelectedBegin = -1;
	m_nUsed			 = 0;
	m_nMax			 = 0;
	m_nPartSize		 = 0;
	setMinimumSize(152, 100);
}

//-----------------------------------------------------------------------------------
void CGraphWidget::SetText(const QString& sUsed, const QString& sFree, const QString& sUnit)
{
	m_sUsed = sUsed;
	m_sFree = sFree;
	m_sUnit = sUnit;
	update();
}

//-----------------------------------------------------------------------------------
void CGraphWidget::DrawText(QPainter& p, int x, int y, const QString& sText, bool bUsedForeGroundColor)
{
	QPen qPen = p.pen();
	if(bUsedForeGroundColor) {
		p.setPen(colorGroup().foreground());
	}
	p.drawText(x, y,sText);
	if(bUsedForeGroundColor) {
		p.setPen(qPen);
	}
}

//-----------------------------------------------------------------------------------
void CGraphWidget::UpdateGraph(QPainter& p)
{
	if(m_nMax && m_nPartSize)
	{
		int nY = 68;
		float fHeight = height() - nY;
		int nWidth = width() >> 1;
		int nX = (width() - nWidth) >> 1;

		float r = ((m_nUsed / (float)m_nMax)) * fHeight;
		
		//Affiche zone utilise
		p.setBrush(QColor(255, 0, 0));
		p.drawRect(nX, nY, nWidth, r + 1);
		
		//Affiche zone libre
		p.setBrush(QColor(0, 0, 255));
		p.drawRect(nX, r + nY, nWidth, fHeight - r);

		if(m_nSelectedBegin != -1)
		{
			//Affiche la slection
			p.setBrush(QColor(255, 255, 0));
			float fStart  = fHeight * (m_nSelectedBegin  / (float)m_nMax);
			float fLength = fHeight * (m_nSelectedLength / (float)m_nMax);
			p.drawRect(nX, fStart + nY, nWidth, fLength);		
		}
		
		float step = (m_nPartSize / (float)m_nMax) * fHeight;
		
		int nCpt = m_nPartSize;
		
		if(m_nMax < m_nPartSize)
		{
			step   = fHeight;
			nCpt = m_nMax;
		}
		
		float y = step + nY;
		int i = 1;
		for(; nCpt <= m_nMax; nCpt += m_nPartSize)
		{
			p.drawLine(nX, (int)y, nWidth + nX, (int)y);
			DrawText(p, nX + (nWidth >> 1), (int)(y - (step / 2.0f)) + 5, QString::number(i));
			DrawText(p, nX + nWidth,(int)y, QString::number(nCpt) + "Ko", TRUE);
			y += step;
			i++;
		}
	}
}

//-----------------------------------------------------------------------------------
void CGraphWidget::paintEvent(QPaintEvent* e)
{
	QWidget::paintEvent(e);
	QString space(" ");

	QPainter p(this);

	InitPainterFont(p);

	//Affiche zone utilise
	p.setBrush(QColor(255, 0, 0));
	p.drawRect(0, 10, 20, 20);
	QString sText = m_sUsed + space + QString::number(m_nUsed) + space + m_sUnit;
	int nWidth = GetWidth(p, sText);
	DrawText(p, 25, 24, sText, TRUE);
	
	//Affiche zone libre
	p.setBrush(QColor(0, 0, 255));
	p.drawRect(0, 40, 20, 20);
	sText = m_sFree + space + QString::number(m_nMax - m_nUsed) + space + m_sUnit;
	int nWidth2 = GetWidth(p, sText);
	DrawText(p, 25, 54, sText, TRUE);

	UpdateGraph(p);
	p.end();

	if(nWidth < nWidth2)
	{
		nWidth = nWidth2;
	}
	setMinimumWidth(25 + nWidth);
}

//-----------------------------------------------------------------------------------
void CGraphWidget::UpdateData(int nUsed, int nMax, int nPartSize)
{
	m_nUsed     = nUsed;
	m_nMax      = nMax;
	m_nPartSize = nPartSize;
	update();
}

//-----------------------------------------------------------------------------------
void CGraphWidget::SetSelectedZone(int nSelectedBegin, int nSelectedLength)
{
	m_nSelectedBegin  = nSelectedBegin;
	m_nSelectedLength = nSelectedLength;
	QPainter p(this);
	InitPainterFont(p);
	UpdateGraph(p);
	p.end();
}

//-----------------------------------------------------------------------------------
void CGraphWidget::InitPainterFont(QPainter& p)
{
	//Couleur du texte
	p.setPen(QColor(0, 0, 0));
	QFont font = p.font();
	font.setPointSize(8);
}

//-----------------------------------------------------------------------------------
int CGraphWidget::GetWidth(QPainter& p, const QString& sText)
{
	QFontMetrics fontMetrics(p.font());
	return fontMetrics.width(sText);	
}