// ROMDISKEditor.cpp: implementation of the CROMDISKEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "ROMDISKEditor.h"

#include <qheader.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qinputdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qdir.h>

#include <language.h>
#include <MessageBox.h>
#include <DriveAccess.h>
#include <FileList.h>
#include <image_defs.h>
#include <IOptionWindow.h>
#include <ROMDISKFileInfo.h>
#include <ROMDISKDrive.h>
#include <F100_Debug.h>
#include <FileList.h>

#include "GraphWidget.h"
#include "defs.h"
#include "DriveWindow.h"


#define ERR_FOLDER(a, b) CROMDISKEditor::tr("Impossible de crer le dossier '%1': %2").arg(a).arg(TranslateError(b))
#define ERR_FILE(a, b)   CROMDISKEditor::tr("Impossible de crer le fichier '%1': %2").arg(a).arg(TranslateError(b))

//-----------------------------------------------------------------------------------
CROMDISKEditor::CROMDISKEditor(DriveWindow* parent)
:CEditor(parent)
{
	m_pDrawingWidget = new CGraphWidget(this);
	m_pList			 = new CFileList(this);
	m_pVolumeLbl     = new QLabel(this);
	m_pVolume        = new QLineEdit(this);

	QGridLayout* pLayout = new QGridLayout(this, 2, 4, 5);

	pLayout->addMultiCellWidget(m_pDrawingWidget, 0, 1, 0, 0);
	pLayout->addMultiCellWidget(m_pList,          1, 1, 1, 3);
	pLayout->addWidget(m_pVolumeLbl, 0, 1);
	pLayout->addWidget(m_pVolume,    0, 2);
	
	pLayout->setColStretch(3, 10);

	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);
	m_pList->setAllColumnsShowFocus(TRUE);
	m_pList->setColumnAlignment(1, AlignRight);
	m_pList->setRootIsDecorated(TRUE);
	m_pList->setShowSortIndicator(TRUE);

	m_pVolume->setMaxLength(8);

	//Connection
	connect(m_pList, SIGNAL(mouseButtonPressed( int, QListViewItem *, const QPoint &, int )), SLOT(OnListClicked( int, QListViewItem *, const QPoint &)));
	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(UpdateDraw(QListViewItem*)));
	connect(m_pList, SIGNAL(itemRenamed(QListViewItem*, int)), SLOT(OnItemRenamed(QListViewItem*)));
	connect(m_pList, SIGNAL(Dropped(const QStringList&, QListViewItem*)), SLOT(OnFilesDropped(const QStringList&, QListViewItem*)));
	connect(m_pVolume, SIGNAL(textChanged(const QString&)), SLOT(OnVolumeNameChanged(const QString&)));

	DoInitWords();
}

//-----------------------------------------------------------------------------------
int CROMDISKEditor::CreateDrive(CROMDISKDrive*& pDrive, bool bWriteMode)
{
	bool bVariableSize = m_pParent->GetMainWindow()->GetOptionWindow()->GetPreference(IOptionWindow::PREF_DRIVE_VARIABLE_SIZE)
		&& m_pParent->m_nShift == 0;
	pDrive = new CROMDISKDrive(m_pParent->m_bOneFile, bVariableSize);
	if(pDrive == NULL) {
		CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgOutOfMemory(sizeof(CROMDISKDrive)));
		return -1;
	}
	if(m_pParent->m_pDriveBuffer) {
		pDrive->OpenDriveFile(m_pParent->m_pDriveBuffer, m_pParent->m_nMaxSize);
	}
	else {
		string sFileName = m_pParent->m_sFileName;
		if(pDrive->OpenDriveFile(sFileName, m_pParent->m_nShift, bWriteMode) < 1) {
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_pParent->m_sFileName, bWriteMode));//L'ouverture a echoue
			delete pDrive;
			pDrive = NULL;
			return 0;//Pour viter de recharger le lecteur
		}
	}
	if(pDrive->IsEmpty(m_pParent->m_nMaxSize)) {
		CMessageBox::MsgInformation(m_pParent->GetMainWidget(), tr("Lecteur vierge."));
		delete pDrive;
		pDrive = NULL;
		return 0;//Pour viter de recharger le lecteur, rien d'autre  afficher
	}
	if(!pDrive->IsValid()) {
		delete pDrive;
		pDrive = NULL;
		return -1;
	}
	return 1;
}


//-----------------------------------------------------------------------------------
void CROMDISKEditor::ClearData()
{
	//Suppression des lments existant
	m_pList->clear();
	m_pDrawingWidget->UpdateData(0, 0, DRIVE_SIZE);
	m_pDrawingWidget->SetSelectedZone(-1, 0);
}

//-----------------------------------------------------------------------------------
int CROMDISKEditor::DoReloadFile()
{
	ClearData();
	m_pVolume->setReadOnly(m_pParent->m_bReadOnly);
	m_pList->setAcceptDrops(!m_pParent->m_bReadOnly);

	if(m_pParent->m_bShowGraphWidget) {
		m_pDrawingWidget->show();
	}
	else {
		m_pDrawingWidget->hide();
	}

	CROMDISKDrive* pDrive;
	int nRet = CreateDrive(pDrive, FALSE);
	if(nRet < 1) {
		return nRet;
	}

	m_sVolumeName = pDrive->GetVolumeName().c_str();
	m_pVolume->setText(m_sVolumeName);

	CROMDISKFileInfo* pRoot = pDrive->OpenRoot();
	ShowDrive(pDrive, m_pList, NULL, pDrive->FindNext(pRoot));

	DeleteDrive(pDrive);

	return 1;
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::DeleteDrive(CDriveFile* pDrive)
{
	if(pDrive != NULL) {
		update();
		int nSize = pDrive->GetSizeInBytes();
		int nMax = m_pParent->m_nMaxSize;
		if(nMax <= 0) {
			nMax = CDriveAccess::GetNbFile(nSize) * DRIVE_SIZE;
		}
		nMax >>= 10;
		m_pDrawingWidget->UpdateData(nSize >> 10, nMax, DRIVE_SIZE);
		CEditor::DeleteDrive(pDrive);
		m_pParent->ResetTime();
	}
}

//-----------------------------------------------------------------------------------
QString CROMDISKEditor::FullName(QListViewItem* pItem)
{
	QString sName = pItem->text(3);
	while((pItem = pItem->parent())) {
		sName = pItem->text(3) + '/' + sName;
	}
	return sName;
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::ShowDrive(CROMDISKDrive* pDrive, QListView* pList, QListViewItem* pParent, CROMDISKFileInfo* pFile)
{
	CFileItem* pLastItem = NULL;
	CFileItem* pItem;
	while(pFile != NULL)
	{
		if(pFile->IsFile() || (pFile->GetName() != "." && pFile->GetName() != ".."))
		{	
			if(pFile->IsDir())
			{
				if(pParent) {
					pItem = new CFileItem(CFileItem::Directory, pParent, pLastItem);
				}
				else {
					pItem = new CFileItem(CFileItem::Directory, pList, pLastItem);
				}
				CROMDISKFileInfo* pDir = pDrive->OpenDir(pFile, false);
				if(pDir != NULL) {
					ShowDrive(pDrive, pList, pItem, pDir);
				}
				else {
					emit m_pParent->Error(tr("Impossible d'ouvrir le rpertoire %1 dans le fichier ROM-DISK '%2'.").arg(pFile->GetName().c_str()).arg(m_pParent->m_sFileName));
				}
			}
			else
			{
				if(pParent)	{
					pItem = new CFileItem(CFileItem::File, pParent, pLastItem);
				}
				else {
					pItem = new CFileItem(CFileItem::File, pList, pLastItem);
				}
			}
			SetInformations(pItem, pFile);
			pLastItem = pItem;
		}
		pFile = pDrive->FindNext(pFile);
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::DoInitWords()
{
	m_pList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pList->setColumnText(1, CLanguage::GetWordTranslation(CLanguage::WO_SIZE));
	m_pList->setColumnText(2, tr("Modifi le"));
	m_pDrawingWidget->SetText(tr("Espace utilis :"), tr("Espace libre :"), tr("Ko"));
	m_pVolumeLbl->setText(tr("Nom de volume :"));
}

//-----------------------------------------------------------------------------------
int CROMDISKEditor::ExtractFiles(CROMDISKDrive* pDrive, CROMDISKFileInfo* pFile, const QString& sPath)
{
	QString sFilePath = sPath + '/' + QString(pFile->GetFullName().c_str());
	if(pFile->IsDir())
	{
		if(MkDir(sFilePath) < 0) {
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), tr("Impossible de crer le dossier '%1'!").arg(sFilePath));
			return -1;
		}
		CROMDISKFileInfo* pDir = pDrive->OpenDir(pFile, false);
		if(pDir == NULL) {
			emit m_pParent->Error(tr("Le dossier '%1' n'a pas pu tre ouvert.").arg(pFile->GetName().c_str()));
			return -1;
		}
		/* Saute les rpertoires '.' et '..' */
		pDir = pDrive->FindNext(pDir);
		pDir = pDrive->FindNext(pDir);

		bool bOk       = true;
		bool bContinue = true;
		while(bOk && pDir != NULL) {
			bOk  = (ExtractFiles(pDrive, pDir, sFilePath) > 0);
			pDir = pDrive->FindNext(pDir);
		}
		pDrive->FreeFileInfo(pDir);
		if(bOk) {
			return 1;
		}
		return -1;
	}
	if(pFile->IsFile())
	{
		char buffer[65536];
		HND h = OpenDescModeRdWr(sFilePath);
		if(h == INVALID_HANDLE_VALUE) {
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFilePath, TRUE));
			return -1;
		}
		int nFileSize = pFile->GetSize();
		int nWrote = 0, nRet = 1;
		while(nRet > 0 && nWrote < nFileSize)
		{
			nRet = pFile->ReadFile(buffer, nWrote, sizeof(buffer));
			if(nRet < 1) {
				CMessageBox::MsgCritical(m_pParent->GetMainWidget(), tr("Impossible de lire le fichier '%1' dans le lecteur!\nErreur retourne : %2.").arg(sFilePath).arg(nRet));
				break;
			}
			nWrote += nRet;
			if(!WriteDesc(h, buffer, nRet)) {
				CMessageBox::MsgCritical(m_pParent->GetMainWidget(), CLanguage::GetMsgNoWriteFile(sFilePath));
				break;
			}
		}
		CloseDesc(h);
		if(nWrote == nFileSize) {
			return 1;
		}
	}
	if(pFile->IsRoot()) {
		CROMDISKFileInfo* pNext = pDrive->FindNext(pFile, false);
		while(pNext != NULL) {
			ExtractFiles(pDrive, pNext, sPath);
			pNext = pDrive->FindNext(pNext);
		}
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::Extract(QListViewItem* pItem)
{
	QString sName = CMessageBox::ChooseExistingDirectory(m_sLastExtractDir, m_pParent->GetMainWidget(), QString::null);
	if(sName.isEmpty()) {
		return;
	}
	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, FALSE) < 1) {
		return;
	}
	CROMDISKFileInfo* pFile = GetFileInfo(pDrive, pItem);
	if(pFile == NULL) {
		DeleteDrive(pDrive);
		return;
	}
	if(ExtractFiles(pDrive, pFile, sName) > 0) {
		m_sLastExtractDir = sName;
	}
	pDrive->FreeFileInfo(pFile);
	DeleteDrive(pDrive);
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::OnItemRenamed(QListViewItem* pItem)
{
	if(pItem == NULL || pItem->rtti() != CFileItem::RTTI())
	{
		return;
	}
	CFileItem* pFileItem = (CFileItem*)pItem;
	
	QString sFileName = FullName(pItem);
	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, TRUE) < 1) {
		pFileItem->SetName(pItem->text(3));
		return;
	}

	string sName = pFileItem->GetName().latin1();
	if(pDrive->Rename(sFileName.latin1(), sName) < 1) {
		CMessageBox::MsgCritical(m_pParent->GetMainWidget(), tr("Impossible de renommer cet lment!"));		
		pFileItem->SetName(pItem->text(3));
	}
	else {
		pFileItem->SetName(sName.c_str());
		pFileItem->setText(3, sName.c_str());
	}
	DeleteDrive(pDrive);
	m_pList->sort();
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::OnListClicked(int button, QListViewItem* pItem, const QPoint & p)
{
	if(button == RightButton)
	{
		QString s;
		QPopupMenu* pMenu = new QPopupMenu(this);
		if(pItem != NULL) {
			s = " '" + pItem->text(0) + "'";
		}
		pMenu->insertItem(CLanguage::GetWordTranslation(CLanguage::WO_RENAME)  + s, 0);
		pMenu->insertItem(tr("Supprimer") + s,                                      1);
		pMenu->insertSeparator();
		pMenu->insertItem(tr("Ajouter un dossier..."),                              2);
		pMenu->insertItem(tr("Ajouter un fichier..."),                              3);
		pMenu->insertSeparator();
		pMenu->insertItem(CLanguage::GetWordTranslation(CLanguage::WO_EXTRACT) + s, 4);
		pMenu->insertItem(tr("%1 tout le lecteur").arg(CLanguage::GetWordTranslation(CLanguage::WO_EXTRACT)), 5);

		pMenu->setItemEnabled(0, !m_pParent->m_bReadOnly && pItem != NULL);
		pMenu->setItemEnabled(1, !m_pParent->m_bReadOnly && pItem != NULL);
		pMenu->setItemEnabled(2, !m_pParent->m_bReadOnly);
		pMenu->setItemEnabled(3, !m_pParent->m_bReadOnly);
		pMenu->setItemEnabled(4, pItem != NULL);
		pMenu->setItemEnabled(5, m_pList->childCount() != 0);
		switch(pMenu->exec(p))
		{
		case 0: // Renomer le fichier ou dossier
			m_pList->selectedItem()->startRename(0);
			break;
		case 1: //Supprimer
			//TODO
			Kill(pItem);
			break;
		case 2: //Ajout d'un dossier
			AddFolder(pItem);
			break;
		case 3: //Ajout d'un fichier
			AddFile(pItem);
			break;
		case 4: // Extraire le fichier
			Extract(m_pList->selectedItem());
			break;
		case 5: // Extraire tout
			Extract(NULL);
			break;
		}
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::UpdateDraw(QListViewItem* pItem)
{
	if(pItem) {
		m_pDrawingWidget->SetSelectedZone(pItem->text(4).toInt() >> 10, CROMDISKDrive::ConvertFileSizeToRealSize(pItem->text(1).toInt()) >> 10);
	}
	else {
		m_pDrawingWidget->SetSelectedZone(-1, -1);
	}
}

//-----------------------------------------------------------------------------------
QString CROMDISKEditor::GetCaption() const
{
	return tr("Lecteur ROM-DISK");
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKEditor::GetFileInfo(CROMDISKDrive* pDrive, QListViewItem* pItem)
{
	CROMDISKFileInfo* pRoot = pDrive->OpenRoot();
	if(pRoot == NULL) {
		emit m_pParent->Error(tr("Impossible d'accder  la racine du lecteur"));
		return NULL;
	}
	if(pItem == NULL) {
		return pRoot;
	}
	QString sFullName = FullName(pItem);
	CROMDISKFileInfo* pStart = pDrive->FindFileOrDir(sFullName.latin1(), pRoot);
	pDrive->FreeFileInfo(pRoot);
	if(pStart == NULL) {
		emit m_pParent->Error(tr("Le fichier '%1' est introuvable dans le lecteur").arg(sFullName));
		return NULL;
	}
	return pStart;
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::ShowItem(const QString &sName, QListViewItem* pItem)
{
	QListViewItem* pNext;
	if(pItem != NULL) {
		pNext = pItem->firstChild();
	}
	else {
		pNext = m_pList->firstChild();
	}
	QString s = sName.section('/', 0, 0);
	while(pNext != NULL) {
		if(s == pNext->text(3)) {
			if(s == sName) {
				m_pList->ensureItemVisible(pNext);
				m_pList->setSelected(pNext, TRUE);
			}
			else {
				ShowItem(sName.section('/', 1), pNext);
			}
			return;
		}
		pNext = pNext->nextSibling();
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::AddFolder(QListViewItem* pItem)
{
	QString s;
	if(pItem != NULL) {
		s = FullName(pItem) + '/';
	}

	m_pParent->StopChecking();
	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, TRUE) < 1) {
		return;
	}
	string sName;
	CROMDISKFileInfo* pParent = GetFileInfo(pDrive, pItem);
	if(pParent != NULL) {
		if(pParent->IsDir()) {
			pParent = pDrive->OpenDir(pParent, true);
		}
		CROMDISKFileInfo* pDir;
		QString sDirName = QInputDialog::getText(QString::null, tr("Entrez le nom du dossier"), QLineEdit::Normal, QString::null, 0, m_pParent->GetMainWidget());
		if(sDirName.isEmpty()) {
			DeleteDrive(pDrive);
			return;
		}
		sName = sDirName.latin1();
		int nRet = pDrive->CreateDirectory(pParent, pDir, sName, 0);
		if(nRet < 1) {
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), ERR_FOLDER(sDirName, nRet));
			DeleteDrive(pDrive);
			return;
		}
		pDrive->FreeFileInfo(pParent);
		pDrive->FreeFileInfo(pDir);
	}
	DeleteDrive(pDrive);
	DoReloadFile();
	if(!sName.empty()) {
		s += sName.c_str();
		ShowItem(s, NULL);
	}
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::AddFile(QListViewItem* pItem)
{
	QString s;
	if(pItem != NULL) {
		s = FullName(pItem) + '/';
	}

	m_pParent->StopChecking();
	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, TRUE) < 1) {
		return;
	}
	string sName;
	CROMDISKFileInfo* pParent = GetFileInfo(pDrive, pItem);
	if(pParent != NULL) {
		if(pParent->IsDir()) {
			pParent = pDrive->OpenDir(pParent, true);
		}
		CROMDISKFileInfo* pFile;
		QString sFileName = CMessageBox::ChooseOpenFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), m_pParent->GetMainWidget());
		if(sFileName.isEmpty()) {
			DeleteDrive(pDrive);
			return;
		}
		const char* pName;
		GetNameOfFile(sFileName, pName);
		sName = pName;
		int nRet = pDrive->CreateFile(pParent, pFile, sName, sFileName.latin1());
		if(nRet < 1) {
			CMessageBox::MsgCritical(m_pParent->GetMainWidget(), ERR_FILE(sName.c_str(), nRet));
			DeleteDrive(pDrive);
			return;
		}
		pDrive->FreeFileInfo(pParent);
		pDrive->FreeFileInfo(pFile);
	}
	DeleteDrive(pDrive);
	DoReloadFile();
	if(!sName.empty()) {
		s += sName.c_str();
		ShowItem(s, NULL);
	}
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::Kill(QListViewItem* pItem)
{
	assert(pItem != NULL);

	QString s = FullName(pItem);

	m_pParent->StopChecking();
	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, TRUE) < 1) {
		return;
	}
	string sName;
	CROMDISKFileInfo* pInfo = GetFileInfo(pDrive, pItem);
	if(pInfo != NULL) {
		pDrive->FreeFileInfo(pInfo);
	}
	int nRet = Kill(pDrive, pItem);
	if(nRet != 1) {
		CMessageBox::MsgCritical(m_pParent->GetMainWidget(), tr("Impossible de supprimer '%1' : %2").arg(s, TranslateError(nRet)));
	}

	DeleteDrive(pDrive);
	DoReloadFile();
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
int CROMDISKEditor::Kill(CROMDISKDrive* pDrive, QListViewItem* pItem)
{
	assert(pDrive != NULL && pItem != NULL);
	QListViewItem* pChild = pItem->firstChild();
	while(pChild != NULL) {
		int nRet = Kill(pDrive, pChild);
		if(nRet < 1) {
			return nRet;
		}
		pChild = pChild->nextSibling();
	}
	CROMDISKFileInfo* pInfo = GetFileInfo(pDrive, pItem);
	if(pInfo != NULL) {
		return pDrive->Kill(pInfo);
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::SetInformations(CFileItem* pItem, CROMDISKFileInfo* pFile)
{
	pItem->setRenameEnabled(0, !m_pParent->m_bReadOnly);
	pItem->setRenameEnabled(1, FALSE);
	pItem->SetName(pFile->GetFullName().c_str());
	pItem->SetSize(pFile->GetSize());
	pItem->setText(3, pItem->GetName());
	pItem->setText(4, QString::number(pFile->GetAddress()));
	CROMDISKFileInfo::FileTime ft = pFile->GetFileTime();
	QString sDay, sMonth, sHour, sMin;
	sDay.sprintf("%.2i",   ft.day);
	sMonth.sprintf("%.2i", ft.month);
	sHour.sprintf("%.2i",  ft.hour);
	sMin.sprintf("%.2i",   ft.min);
	
	pItem->setText(2, tr("%1/%2/%3 %4:%5", "Date and Time format").arg(sDay)
		.arg(sMonth)
		.arg(ft.year)
		.arg(sHour)
		.arg(sMin));	
}

//-----------------------------------------------------------------------------------
QString CROMDISKEditor::TranslateError(int nRet)
{
	switch(nRet)
	{
	case CROMDISKDrive::ER_ALREADY_EXISTS   : return tr("Le fichier ou dossier existe dj.");
	case CROMDISKDrive::ER_OUT_OF_MEM       : return tr("Manque de mmoire.");
	case CROMDISKDrive::ER_NOT_A_FILE       : return tr("La zone n'est pas un fichier.");
	case CROMDISKDrive::ER_FILE_TOO_SMALL   : return tr("Le fichier est trop petit.");
	case CROMDISKDrive::ER_POS_TOO_BIG      : return tr("La position de dpart est suprieure  la taille du fichier.");
	case CROMDISKDrive::ER_CANT_READ        : return tr("Erreur de lecture sur le fichier.");
	case CROMDISKDrive::ER_FILE_NOT_FOUND   : return tr("Fichier ou dossier non trouv.");		
	case CROMDISKDrive::ER_FILE_USED        : return tr("Fichier ou dossier en cours d'utilisation.");
	case CROMDISKDrive::ER_FOLDER_NOT_EMPTY : return tr("Dossier non vide.");
	}
	return tr("Erreur inconnue");
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::OnVolumeNameChanged(const QString& sVolumeName)
{
	if(m_sVolumeName != sVolumeName) {
		CROMDISKDrive* pDrive;
		if(CreateDrive(pDrive, TRUE) < 1) {
			return;
		}
		if(pDrive->SetVolumeName(sVolumeName.latin1()) < 1) {
			m_pVolume->setText(m_sVolumeName);
		}
		m_sVolumeName = sVolumeName;
		DeleteDrive(pDrive);
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem)
{
	m_pParent->StopChecking();

	CROMDISKDrive* pDrive;
	if(CreateDrive(pDrive, TRUE) < 1) {
		return;
	}
	CROMDISKFileInfo* pParent = GetFileInfo(pDrive, pDroppedItem);
	if(pParent->IsDir()) {
		pParent = pDrive->OpenDir(pParent, true);
	}
	AddDirsOrFiles(pDrive, pParent, sFiles);
	pDrive->FreeFileInfo(pParent);
	DeleteDrive(pDrive);

	DoReloadFile();
	m_pParent->StartChecking();
}

//-----------------------------------------------------------------------------------
void CROMDISKEditor::AddDirsOrFiles(CROMDISKDrive* pDrive, CROMDISKFileInfo* pParent, const QStringList& sFiles)
{
	const char* c;
	if(pParent == NULL) {
		return;
	}
	for(QStringList::const_iterator it = sFiles.begin(); it != sFiles.end(); ++it)
	{
		QString sFile = *it;
		QDir dir(sFile);
		GetNameOfFile(sFile, c);
		string s = c;
		if(dir.exists())
		{
			CROMDISKFileInfo::CheckName(s, false);
			CROMDISKFileInfo* pDir = pDrive->FindFileOrDir(s, pParent);
			if(pDir != NULL) {
				if(!CMessageBox::MsgQuestion(m_pParent->GetMainWidget(), tr("Voulez-vous modifier le contenu du dossier '%1' ?").arg(s.c_str()))) {
					pDrive->FreeFileInfo(pDir);
					continue;
				}
			}
			else {
				int nRet = pDrive->CreateDirectory(pParent, pDir, s, 0);
				if(nRet < 1) {
					emit m_pParent->Error(ERR_FOLDER(c, nRet));
					continue;
				}
			}
			QStringList sSubList = dir.entryList(QDir::Files | QDir::Dirs);
			QStringList::Iterator it = sSubList.begin();
			while(it != sSubList.end())
			{
				QString s = *it;
				if(s == "." || s == "..") {
					it = sSubList.remove(it);
				}
				else {
					(*it) = sFile + QDir::separator() + s;
					++it;
				}
			}
			pDir = pDrive->OpenDir(pDir, true);
			AddDirsOrFiles(pDrive, pDir, sSubList);
			pDrive->FreeFileInfo(pDir);
		}
		else {
			CROMDISKFileInfo::CheckName(s, true);
			CROMDISKFileInfo* pFile = pDrive->FindFileOrDir(s, pParent);
			if(pFile != NULL) {
				if(!CMessageBox::MsgQuestion(m_pParent->GetMainWidget(), tr("Voulez-vous remplacer le fichier '%1' ?").arg(s.c_str()))) {
					pDrive->FreeFileInfo(pFile);
					continue;
				}
				pDrive->Kill(pFile);
			}
			int nRet = pDrive->CreateFile(pParent, pFile, s, sFile.latin1());
			if(nRet < 1) {
				emit m_pParent->Error(ERR_FILE(c, nRet));
			}
			pDrive->FreeFileInfo(pFile);
		}
	}
}
