
#ifndef ROMDISKEDITOR_H
#define ROMDISKEDITOR_H

#include "Editor.h"

class QListView;
class CFileList;
class QListViewItem;
class QPushButton;
class QLabel;
class QLineEdit;
class CGraphWidget;
class CFileItem;
class CROMDISKDrive;
class CROMDISKFileInfo;

class CROMDISKEditor : public CEditor  
{
	Q_OBJECT
public:
	CROMDISKEditor(DriveWindow* parent=0);
	int			DoReloadFile();
	void        DoInitWords();
	QString     GetCaption() const;
	void        DeleteDrive(CDriveFile* pDrive);

protected:
	int         CreateDrive(CROMDISKDrive*& pDrive, bool bWriteMode);
	void        ShowItem(const QString &sName, QListViewItem* pItem);
	void        AddDirsOrFiles(CROMDISKDrive* pDrive, CROMDISKFileInfo* pParent, const QStringList& sFiles);

private:
	void        Extract(QListViewItem* pItem);
	int         ExtractFiles(CROMDISKDrive* pDrive, CROMDISKFileInfo* pFile,  const QString& sPath);
	void	    ShowDrive(CROMDISKDrive* pDrive, QListView* pList, QListViewItem* pParent, CROMDISKFileInfo* pFile);
	void        ClearData();
	CROMDISKFileInfo* GetFileInfo(CROMDISKDrive* pDrive, QListViewItem* pItem);
	void        AddFolder(QListViewItem* pItem);
	void        AddFile(QListViewItem* pItem);
	void        Kill(QListViewItem* pItem);
	int         Kill(CROMDISKDrive* pDrive, QListViewItem* pItem);
	QString     TranslateError(int nRet);
	void        SetInformations(CFileItem* pItem, CROMDISKFileInfo* pFile);

private:
	static QString FullName(QListViewItem* pItem);

private slots:
	void        OnListClicked(int, QListViewItem*, const QPoint&);
	void        UpdateDraw(QListViewItem*);
	void        OnItemRenamed(QListViewItem*);
	void        OnVolumeNameChanged(const QString&);
	void        OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem);

private:
	CFileList*    m_pList;
	CGraphWidget* m_pDrawingWidget;
	QLabel*       m_pVolumeLbl;
	QLineEdit*    m_pVolume;
	QString       m_sLastExtractDir;
	QString       m_sVolumeName;
};

#endif
