// FlashWidget.cpp: implementation of the FlashWidget class.
//
//////////////////////////////////////////////////////////////////////

#include "FlashWidget.h"

#include <qaction.h>
#include <qmainwindow.h>
#include <qlistview.h>
#include <qfile.h>
#include <qdir.h>
#include <qpopupmenu.h>
#include <qheader.h>
#include <qinputdialog.h>
#include <qpopupmenu.h>

#include <language.h>
#include <FixedToolBar.h>
#include <DriveAccess.h>
#include <MessageBox.h>
#include <TransfertWindow.h>
#include <F100_Debug.h>
#include <FlashBackup.h>
#include <IOptionWindow.h>
#include <CalcInfo.h>
#include <SendToMenu.h>
#include <image_defs.h>

#define DRIVE_EXT  "lec"
#define SYSTEM_EXT "mems"

#define IMG_ICON       "flashmanager/icon.png"
#define IMG_SEND       "flashmanager/send.png"
#define IMG_RECEIVE    "flashmanager/receive.png"
#define IMG_SENDB      "flashmanager/send_backup.png"
#define IMG_RECEIVEB   "flashmanager/receive_backup.png"

//-----------------------------------------------------------------------------------
FlashWidget::FlashWidget(CONSTRUCTOR_PARAM)
:CToolWindow(CALL_PARAM)
{
	setIcon(GetDefaultIcon());

	resize(530, 270);
	QMainWindow* pCentralWidget = new QMainWindow(GetMainWidget());
	CFixedToolBar*  pToolBar    = new CFixedToolBar(pCentralWidget);
	m_pDriveList = new QListView(pCentralWidget);
	m_pMenu      = new QPopupMenu(this);

	SetCentralWidget(pCentralWidget);

	pCentralWidget->setUsesTextLabel(TRUE);
	pCentralWidget->setCentralWidget(m_pDriveList);

	m_pDriveList->setSorting(-1);
	m_pDriveList->addColumn(QString::null);
	m_pDriveList->addColumn(QString::null);
	m_pDriveList->addColumn(QString::null);
	m_pDriveList->setAllColumnsShowFocus(TRUE);
	m_pDriveList->setItemMargin(4);
	m_pDriveList->setColumnAlignment(1, AlignRight);

	m_pSend          = new QAction(pCentralWidget);
	m_pReceive       = new QAction(pCentralWidget);
	m_pSendBackup    = new QAction(pCentralWidget);
	m_pReceiveBackup = new QAction(pCentralWidget);
	m_pOption        = new QAction(pCentralWidget);

	m_pSend->setIconSet(QPixmap::fromMimeSource(IMG_SEND));
	m_pSend->setAccel(Qt::Key_F5);

	m_pReceive->setIconSet(QPixmap::fromMimeSource(IMG_RECEIVE));
	m_pReceive->setAccel(Qt::Key_F6);

	m_pSendBackup->setIconSet(QPixmap::fromMimeSource(IMG_SENDB));
	m_pSendBackup->setAccel(Qt::Key_F7);

	m_pReceiveBackup->setIconSet(QPixmap::fromMimeSource(IMG_RECEIVEB));
	m_pReceiveBackup->setAccel(Qt::Key_F8);

	m_pOption->setIconSet(QPixmap::fromMimeSource(IMG_OPTION));
	m_pOption->setAccel(Qt::Key_F10);

	m_pSend->addTo(pToolBar);
	m_pReceive->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pSendBackup->addTo(pToolBar);
	m_pReceiveBackup->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pOption->addTo(pToolBar);

	pToolBar->AddStretchableWidget();

	QCheckListItem* pItem = m_pDrives[0] = new QCheckListItem(m_pDriveList, QString::null, QCheckListItem::CheckBox);	
	for(int i = 1; i < 7; i++) {
		m_pDrives[i] = pItem = new QCheckListItem(m_pDriveList, pItem, QString::null, QCheckListItem::CheckBox);		
	}

	GetMainWindow()->GetSendToMenu()->AddSendToItem(this, m_pMenu);
	
	languageChange();
	connect(m_pSend,          SIGNAL(activated()), SLOT(OnSendDrives()));
	connect(m_pReceive,       SIGNAL(activated()), SLOT(OnReceiveDrives()));
	connect(m_pSendBackup,    SIGNAL(activated()), SLOT(OnSendBackup()));
	connect(m_pReceiveBackup, SIGNAL(activated()), SLOT(OnReceiveBackup()));		
	connect(m_pOption,        SIGNAL(activated()), SLOT(OnShowOption()));		
	connect(m_pMenu,          SIGNAL(activated(int)), SLOT(OnItemActivated(int))); 

	connect(m_pDriveList, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)), SLOT(OnRightButtonClicked(QListViewItem*, const QPoint&)));
	connect(m_pDriveList, SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), SLOT(OnDoubleClicked(QListViewItem*)));
	connect(GetMainWindow()->GetOptionWindow()->GetCalcInfo(), SIGNAL(InformationChanged()), SLOT(OnCalcInfoChanged()));

	connect(GetMainWindow()->GetSendToMenu(), SIGNAL(FileToSendChanged(const QString&, int)), SLOT(OnRefreshMenu(const QString&, int)));
	OnCalcInfoChanged();
	OnRefreshMenu(GetMainWindow()->GetSendToMenu()->GetFileName(), GetMainWindow()->GetSendToMenu()->GetNbFile());
}

//-----------------------------------------------------------------------------------
QPixmap FlashWidget::GetDefaultIcon()
{
	return QPixmap::fromMimeSource(IMG_ICON);
}


//-----------------------------------------------------------------------------------
int FlashWidget::Init(INIT_PARAM)
{
	return 1;
}

//-----------------------------------------------------------------------------------
void FlashWidget::DoShowCaption()
{
	QString s = tr("Flash");
	if(!m_sName.isEmpty()) {
		s += " - " + m_sName;
	}
	setCaption(s);
}

//-----------------------------------------------------------------------------------
void FlashWidget::DoInitWords()
{
	m_pSend->setText(tr("Envoyer"));
	m_pSend->setToolTip(tr("Envoyer les lecteurs slectionns"));

	m_pReceive->setText(tr("Recevoir"));
	m_pReceive->setToolTip(tr("Recevoir les lecteurs slectionns"));

	m_pSendBackup->setText(tr("Envoyer un backup"));
	m_pSendBackup->setToolTip(tr("Modifie tout le contenu de votre flash"));

	m_pReceiveBackup->setText(tr("Recevoir un backup"));
	m_pReceiveBackup->setToolTip(tr("Sauvegarde tout le contenu de la flash de votre G100"));

	m_pOption->setText(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));

	m_pDriveList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_DRIVE));
	m_pDriveList->setColumnText(1, tr("Compteur"));
	m_pDriveList->setColumnText(2, CLanguage::GetWordTranslation(CLanguage::WO_PATH));
	m_pDrives[0]->setText(0, CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM));	
	m_pDriveList->adjustColumn(0);
	m_pDriveList->adjustColumn(1);
	m_pDriveList->adjustColumn(2);
	for(int i = 1; i < 7; i++) {
		m_pDrives[i]->setText(0, CLanguage::GetWordTranslation(CLanguage::WO_DRIVE) + QString(" ") + QString(CDriveAccess::GetDriveName(i)));
	}
	RefreshDriveNames();
}

//-----------------------------------------------------------------------------------
int FlashWidget::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1) {
		int i;
		for(i = 0; i < 7; i++) {
			QString sNum = QString::number(i);
			m_pDrives[i]->setOn(element.attribute("Check_" + sNum).toInt());
			m_sFileNames[i] = ConvertToFullPath(element.attribute("FileName_" + sNum));
		}
		QString sDriveCol = element.attribute("Columns");
		m_sName = element.attribute("m_sName");
		for(i = 0; i < 3; i++){
			QString s(sDriveCol.section(';', i, i));
			if(!s.isNull()) {
				m_pDriveList->header()->moveSection(i, s.toInt());
			}
		}
		DoShowCaption();
		RefreshDriveNames();
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int FlashWidget::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1) {
		for(int i = 0; i < 7; i++) {
			QString sNum = QString::number(i);
			element.setAttribute("Check_"    + sNum, QString::number(m_pDrives[i]->isOn()));
			element.setAttribute("FileName_" + sNum, ConvertToRelativePath(m_sFileNames[i]));
		}
		element.setAttribute("Columns", QString("%1;%2;%3")
		.arg(m_pDriveList->header()->mapToIndex(0))
		.arg(m_pDriveList->header()->mapToIndex(1))
		.arg(m_pDriveList->header()->mapToIndex(2)));
		element.setAttribute("m_sName", m_sName);
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void FlashWidget::RefreshDriveNames()
{	
	for(int i = 0; i < 7; i++) {
		if(m_sFileNames[i].isEmpty()) {
			if(i != 0) {
				m_pDrives[i]->setText(2, tr("Formatage du lecteur"));
			}
			else {
				m_pDrives[i]->setText(2, tr("Aucun fichier associ"));
			}
		}
		else {
			m_pDrives[i]->setText(2, m_sFileNames[i]);
		}
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnSendDrives()
{
	CTransfertWindow* pTransfert = GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(GetMainWidget());
	if(pTransfert == NULL) {
		return;
	}
	unsigned char buffer[7][DRIVE_SIZE];
	int nSize = 0;
	bool bSel = FALSE;
	int i;
	if(m_pDrives[0]->isOn() && m_sFileNames[0].isEmpty()) {
		//Empche le formatage du systme
		m_pDrives[0]->setOn(false);
	}
	for(i = 0 ; i < 7; i++)
    {
		if(m_pDrives[i]->isOn() && (m_sFileNames[i].isEmpty() || CMessageBox::ReadingFile(GetMainWidget(), m_sFileNames[i], buffer[i], DRIVE_SIZE)))
		{
			if(m_sFileNames[i].isEmpty()) {
				memset(buffer[i], 0xFF, DRIVE_SIZE);
			}
			bSel   = TRUE;
			nSize += pTransfert->GetUsefulSize(buffer[i]);
		}
		else
		{
			m_pDrives[i]->setOn(false);
		}
	}
	
	if(!bSel)
    {
		delete pTransfert;
		CMessageBox::MsgExclamation(GetMainWidget(), tr("Slectionnez au moins un lecteur  envoyer"));
		return;
    }

	pTransfert->SetData(nSize, false);

	for(i = 0; i < 7; i++)
    {
		if(m_pDrives[i]->isOn()) {
			if(pTransfert->Sending(i, buffer[i]) < 1) {
				delete pTransfert;
				return;
			}
		}
    }
	pTransfert->CloseComm(1);
	delete pTransfert;
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnReceiveDrives()
{
	unsigned char buffer[DRIVE_SIZE];
	bool bOneCheck = FALSE;
	int i;
	for(i = 0; i < 7; i++)
    {
		if(m_pDrives[i]->isOn() && m_sFileNames[i].isEmpty())
		{
			m_pDrives[i]->setOn(false);
		}
		if(!bOneCheck)
		{
			bOneCheck = m_pDrives[i]->isOn();
		}
    }
	if(!bOneCheck) {
		CMessageBox::MsgExclamation(GetMainWidget(), tr("Slectionnez au moins un lecteur  recevoir"));
		return;
    }
	
	CTransfertWindow* pTransfert = GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(GetMainWidget());
	if(!pTransfert) {
		return;
	}

	pTransfert->SetData(pTransfert->GetSizeReceive(m_pDrives[0]->isOn(), m_pDrives[1]->isOn(), m_pDrives[2]->isOn(), m_pDrives[3]->isOn()
		, m_pDrives[4]->isOn(), m_pDrives[5]->isOn(), m_pDrives[6]->isOn()));

	for(i = 0; i < 7; i++)
    {
		if(m_pDrives[i]->isOn())
		{
			int nWrite = pTransfert->Receiving(i, buffer);
			if(nWrite == -1)
			{
				delete pTransfert;
				return;
			}
			if(!GetMainWindow()->GetOptionWindow()->GetPreference(IOptionWindow::PREF_DRIVE_VARIABLE_SIZE))
			{
				nWrite = DRIVE_SIZE;
			}
			else
			{
				nWrite = CDriveAccess::GetSizeUseful(i, buffer);
			}
			while(!CMessageBox::SavingFile(pTransfert, m_sFileNames[i], buffer, nWrite))
			{
				if(!OpenDialog(i))
				{
					//L'utilisateur a press annuler, donc on annule la sauvegarde de ce fichier
					break;
				}
			}
		}
    }
	pTransfert->CloseComm(1);
	delete pTransfert;
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnSendBackup()
{
	QString sName = CMessageBox::ChooseOpenFileName(NULL, tr("Fichiers de sauvegarde (*.bck)") + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget());
	if(!sName.isEmpty())
	{
		CFlashBackup backup;
		if(!backup.Open(sName, false))
		{
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName));
			return;
		}
		if(backup.IsValid() < 1)
		{
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(sName));
			return;
		}
		CTransfertWindow* pTransfert = GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(GetMainWidget());
		if(!pTransfert)
		{
			return;
		}

		unsigned char* pBuffer[7];
		
		//Calcul la taille de l'envoi en octet (dpend du protocole)
		int nSize  = 0;
		int result = 0;
		int k;
		int i;

		for(i = 0; i < 7 && result != -1; i++)
		{
			pBuffer[i] = new unsigned char[DRIVE_SIZE];
			result = backup.RetrieveDrive(i, pBuffer[i]);
			if(result != -1)
			{
				nSize += pTransfert->GetUsefulSize(pBuffer[i]);
			}
		}

		pTransfert->SetData(nSize, false);

		for(k = 0; k < 7 && result != -1; k++)
		{
			if(pTransfert->Sending(k, pBuffer[k]) < 1)
			{
				break;
			}
		}

		pTransfert->CloseComm(result);
		delete pTransfert;

		for(k = 0; k < i; k++)
		{
			delete[] pBuffer[k];
		}
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnReceiveBackup()
{
	QString sName = CMessageBox::ChooseSaveFileName(QString::null, tr("Fichiers de sauvegarde (*.bck)") + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget(), "bck");
	if(!sName.isEmpty())
	{
		CFlashBackup backup;
		
		if(!backup.Open(sName, true) || !backup.ClearBackup())
		{
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName, true));
			return;
		}
		
		CTransfertWindow* pTransfert = GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(GetMainWidget());
		if(!pTransfert)
		{
			return;
		}
		
		pTransfert->SetData(pTransfert->GetSizeReceive(true, true, true, true, true, true, true));
		unsigned char* buffer = new unsigned char[DRIVE_SIZE];
		int i;
		for(i = 0; i < 7; i++)
		{
			if(pTransfert->Receiving(i, buffer) == -1)
			{
				backup.ClearBackup();
				break;
			}
			if(!backup.WriteDrive(i, buffer))
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoWriteFile(sName));
				break;
			}
		}
		pTransfert->CloseComm(i == 7);
		delete pTransfert;
		delete[] buffer;
	}
}

//-----------------------------------------------------------------------------------
int FlashWidget::OpenDialog(int nDrive)
{
	QString s;
	QString sCaption;
	QString sExt;
	if(nDrive)
	{
		s = CLanguage::GetWordTranslation(CLanguage::WO_DRIVEFILTER);
		sCaption = CLanguage::GetWordTranslation(CLanguage::WO_DRIVE) + QString(" ") + QChar((char)(nDrive + 'K'));
		sExt = DRIVE_EXT;
	}
	else
	{
		s = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEMFILTER);
		sCaption = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM);
		sExt = SYSTEM_EXT;
	}
	QString sName = CMessageBox::ChooseSaveFileName(m_sFileNames[nDrive], s, GetMainWidget(), sExt, FALSE, sCaption);
	if(!sName.isEmpty())
	{
		m_sFileNames[nDrive] = sName;
		RefreshDriveNames();
		emit StateChanged();
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
int FlashWidget::GetNumDrive(QListViewItem* pItem)
{
	if(pItem != NULL){
		int nDrive = 0;
		for(; nDrive < 6 && pItem != m_pDrives[nDrive]; nDrive++);
		return nDrive;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnRightButtonClicked(QListViewItem* pItem, const QPoint& pos)
{
	int nDrive = GetNumDrive(pItem);
	QPopupMenu menu(this);
	menu.insertItem(tr("&Editer"), 0);
	menu.insertItem(tr("Choisir un &fichier..."), 1);
	menu.insertSeparator();
	menu.insertItem(tr("&Modifier le compteur d'envoi"), 2);
	menu.insertSeparator();
	if(nDrive == 0)	{
		menu.insertItem(tr("&Supprimer l'association"), 3);
	}
	else {
		menu.insertItem(tr(CLanguage::GetWordTranslation(CLanguage::WO_FORMAT)), 3);
	}
	menu.insertSeparator();
	menu.insertItem(tr("Modifier le &titre de l'outil"), 4);
	
	menu.setItemEnabled(0, pItem != NULL);
	menu.setItemEnabled(1, pItem != NULL);
	menu.setItemEnabled(2, pItem != NULL);
	menu.setItemEnabled(3, pItem != NULL);
	switch(menu.exec(pos))
	{
	case 0://Editer
		OpenZone(nDrive);
		break;
	case 1://Choisir l'association
		OpenDialog(nDrive);
		break;
	case 2://Modifier le compteur d'envoi
		{
			int nValue = QInputDialog::getInteger(pItem->text(0), tr("Entrez la nouvelle valeur"), pItem->text(1).toInt(), 0);
			GetMainWindow()->GetOptionWindow()->GetCalcInfo()->SetCounter(nDrive, nValue);
		}
		break;
	case 3://Supprimer
		m_sFileNames[nDrive] = QString::null;
		emit StateChanged();
		RefreshDriveNames();
		break;
	case 4:
		{
			bool bOk;
			QString s = QInputDialog::getText(caption(), tr("Entrez un nom pour le titre de l'outil"), QLineEdit::Normal, m_sName, &bOk, GetMainWidget());
			if(bOk) {
				m_sName = s;
				DoShowCaption();
				emit StateChanged();
			}
		}
		break;
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnDoubleClicked(QListViewItem* pItem)
{
	if(pItem) {
		OpenZone(GetNumDrive(pItem));
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OpenZone(int nDrive)
{
	if(nDrive)
	{
		if(m_sFileNames[nDrive].isEmpty()) {
			CMessageBox::MsgInformation(GetMainWidget(), tr("Rien  afficher. Ceci formatera votre lecteur."));
		}
		else {
			GetMainWindow()->OpenFileDrive(m_sFileNames[nDrive]);
		}
	}
	else
	{
		if(m_sFileNames[nDrive].isEmpty()) {
			if(!OpenDialog(0)) {
				return;
			}
		}
		GetMainWindow()->OpenFileSystem(m_sFileNames[nDrive]);
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnCalcInfoChanged()
{
	for(int i = 0; i < 7; i++) {
		m_pDrives[i]->setText(1, QString::number(GetMainWindow()->GetOptionWindow()->GetCalcInfo()->GetCounter(i)));
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnRefreshMenu(const QString& sFileName, int nNbFile)
{
	m_pMenu->clear();
	GetMainWindow()->GetSendToMenu()->SetItemEnabled(this, TRUE);
	if(nNbFile > 0 && !sFileName.isEmpty()) {
		if(QDir::match(QString("*.mems;*.csf"), sFileName)) {
			m_pMenu->insertItem(CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM), 0);
			return;
		}
		if(QDir::match(QString("*.lec;*.cdr"), sFileName)) {
			for(int i = 0; i < 7 - nNbFile; i++) {
				m_pMenu->insertItem(CDriveAccess::GetDriveName(i + 1), i + 1);
			}
			return;
		}
	}
	GetMainWindow()->GetSendToMenu()->SetItemEnabled(this, FALSE);
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnItemActivated(int id)
{
	if(!GetMainWindow()->GetSendToMenu()->GetFileName().isEmpty()) {
		string s = GetMainWindow()->GetSendToMenu()->GetFileName().latin1();
		if(GetMainWindow()->GetSendToMenu()->GetNbFile() == 1) {
			m_sFileNames[id] = s.c_str();
			m_pDrives[id]->setOn(TRUE);
		}
		else {
			for(int i = 0; i < GetMainWindow()->GetSendToMenu()->GetNbFile(); i++) {
				m_sFileNames[id + i] = CDriveAccess::ChangeFileName(s, i + 1).c_str();
				m_pDrives[id + i]->setOn(TRUE);
			}
		}
		emit StateChanged();
		RefreshDriveNames();
	}
}

//-----------------------------------------------------------------------------------
void FlashWidget::OnShowOption()
{
	GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(NULL);
}
