// ImagEditor.cpp: implementation of the CImagEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "ImagEditor.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qscrollview.h>
#include <qapplication.h>

#include <language.h>
#include <MessageBox.h>
#include <image_defs.h>

#include "PainterDevice.h"

//-----------------------------------------------------------------------------------
CImagEditor::CImagEditor(QWidget* parent)
 :QWidget(parent)
{
	QGridLayout* pLayout = new QGridLayout(this, 1, 2, 5);

	QButtonGroup* g = new QButtonGroup(this);

	m_pView       = new QScrollView(this);
    m_pImage      = new CPainterDevice(m_pView->viewport());
	m_pApply      = new QPushButton(this);
    m_pOptions    = new QPushButton(this);
	m_pZoom       = new QLabel(this);
	m_pLocation   = new QLabel(this);
	m_pOptionMenu = new QPopupMenu(this);

	m_pView->addChild(m_pImage);

	m_pView->clipper()->installEventFilter(this);
	m_pImage->installEventFilter(this);

	m_pOptions->setPopup(m_pOptionMenu);

	QGridLayout* pButtonGroupLayout = new QGridLayout(0, 5, 3);

	pLayout->addWidget(m_pView, 0, 0);
	pLayout->addMultiCellLayout(pButtonGroupLayout, 0, 0, 1, 1, AlignVCenter);
	pLayout->setColStretch(0, 1);

	g->hide();
	g->setExclusive(true);

	int i;
	for(i = 0; i < NB_TOOL; i++)
	{
		m_pTools[i] = new QPushButton(this);
		m_pTools[i]->setToggleButton(true);
		if(i % 2) {
			pButtonGroupLayout->addWidget(m_pTools[i], i / 2, i % 2, AlignTop | AlignLeft);
		}
		else {
			pButtonGroupLayout->addWidget(m_pTools[i], i / 2, i % 2, AlignTop | AlignRight);
		}
		g->insert(m_pTools[i]);
	}
	pButtonGroupLayout->setRowSpacing(2, 20);
	pButtonGroupLayout->addMultiCellWidget(m_pOptions,  3, 3, 0, 2);
	pButtonGroupLayout->addMultiCellWidget(m_pApply,    4, 4, 0, 2);
	pButtonGroupLayout->addMultiCellWidget(m_pZoom,     5, 5, 0, 2);
	pButtonGroupLayout->addMultiCellWidget(m_pLocation, 6, 6, 0, 2);

	m_pTools[0]->setPixmap(QPixmap::fromMimeSource("menuedit/zoom.png"));
	m_pTools[1]->setPixmap(QPixmap::fromMimeSource("menuedit/paint.png"));
	m_pTools[2]->setPixmap(QPixmap::fromMimeSource("menuedit/pen.png"));
	m_pTools[3]->setPixmap(QPixmap::fromMimeSource("menuedit/line.png"));
//	m_pTools[4]->setPixmap(QPixmap::fromMimeSource(IMG_OPACITY));
	
	m_pTools[2]->setOn(true);

	for(i = 0; i < 5; i++)
	{
		m_pOptionMenu->insertItem(QString::null, i);
	}
	m_pOptionMenu->insertSeparator(2);
	m_pOptionMenu->insertSeparator(4);
	
	m_pImage->setMouseTracking(true);

	//Connections...
	connect(m_pImage, SIGNAL(mouseReleased(int, int, int)), SLOT(OnImageMouseRelease(int, int, int)));
	connect(m_pImage, SIGNAL(mouseMoved(int, int, int)),    SLOT(OnImageMouseMove(int, int, int)));
	connect(m_pImage, SIGNAL(mousePressed(int, int, int)),  SLOT(OnImageMousePressed(int, int, int)));
	connect(m_pApply, SIGNAL(clicked()), SLOT(OnApply()));
	connect(m_pOptionMenu, SIGNAL(activated(int)), SLOT(OnOptionClicked(int)));
	connect(g, SIGNAL(clicked(int)), SLOT(OnToolCliked(int)));

	//Init
	OnToolCliked(2);
	DoInitWords();
	SetZoom(1);
}

//-----------------------------------------------------------------------------------
void CImagEditor::DoInitWords()
{
    m_pOptions->setText(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));
	m_pApply->setText(CLanguage::GetWordTranslation(CLanguage::WO_APPLY));
	m_pOptionMenu->changeItem(0, tr("Effacer tout"));
	m_pOptionMenu->changeItem(1, tr("Remplir tout"));
	m_pOptionMenu->changeItem(2, tr("Inverser l'image"));
	m_pOptionMenu->changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_IMPORT));
	m_pOptionMenu->changeItem(4, CLanguage::GetWordTranslation(CLanguage::WO_EXPORT));

}

//-----------------------------------------------------------------------------------
void CImagEditor::SetImageData(uchar* pData, unsigned int nSize)
{
	m_pImage->SetData(pData, nSize);
}

//-----------------------------------------------------------------------------------
void CImagEditor::SetImageBackgroundColor(const QColor& color)
{
	m_pView->setBackgroundColor(color);
	m_pImage->setBackgroundColor(color);
}

//-----------------------------------------------------------------------------------
void CImagEditor::SetImageMinSize(int nWidth, int nHeight)
{
	m_pView->setMinimumSize(nWidth, nHeight);
}

//-----------------------------------------------------------------------------------
void CImagEditor::SetImageSize(int nWidth, int nHeight)
{
	m_pImage->SetSize(nWidth, nHeight);
	SetZoom(m_nZoom);
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnImageMouseRelease(int button, int x, int y)
{
	int pX, pY;
	uchar val;
	m_pImage->GetWorldPos(x, y, pX, pY);
	QSize size = m_pImage->GetSize();
	if((MinMax(pX, 0, size.width() - 1) | MinMax(pY, 0, size.height() - 1)) && (m_nMode != 3 && m_nMode != 0))
	{
		return;
	}
	switch(m_nMode)
	{
	case 0: // Loupe
		{
			if(button == RightButton) {
				SetZoom(m_nZoom >> 1);
			}
			else if(button == LeftButton) {
				SetZoom(m_nZoom << 1);
			}
		}
		break;
	case 1: // Mode Remplissage
		val = (button == LeftButton);
		FillZone(pX, pY, val);
		m_pImage->update();
		break;
	case 2: // Mode rayon
		m_pImage->SetPixel(pX, pY, button == LeftButton);
		break;
	case 3: // Mode ligne
		m_pImage->DrawLine(m_nCurX, m_nCurY, pX, pY, button == LeftButton);
		m_pImage->ClearTempLine();
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnImageMouseMove(int button, int x, int y)
{
	int pX, pY;
	QSize size = m_pImage->GetSize();
	m_pImage->GetWorldPos(x, y, pX, pY);
	if((MinMax(pX, 0, size.width() - 1) | MinMax(pY, 0, size.height() - 1)) && m_nMode != 3)
	{
		return;
	}
	m_pLocation->setText(QString("(%1, %2)").arg(pX).arg(pY));
	if(button == NoButton){
		return;
	}
	if(m_nMode == 2) // Rayon
	{
		m_pImage->SetPixel(pX, pY, button == LeftButton);
	}
	if(m_nMode == 3) //Ligne
	{
		m_pImage->SetTempLine(m_nCurX, m_nCurY, pX, pY, button == LeftButton);
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnImageMousePressed(int button, int x, int y)
{
	m_pImage->GetWorldPos(x, y, m_nCurX, m_nCurY);	
	OnImageMouseMove(button, x, y);
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnApply()
{
	if(m_pImage->GetData() != NULL) {
		emit Apply(m_pImage->GetData());
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnOptionClicked(int id)
{
	switch(id)
	{
		case 0:
		case 1: //Effacer et Remplir Tout
			m_pImage->Fill(id == 1);
			break;
		case 2: // Inverser l'image
			m_pImage->InvertPixels();
			break;
		case 3://Importer
			ImportImag(this, m_pImage);
			break;
		case 4://Exporter
			ExportImag(this, m_pImage);
			break;
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::OnToolCliked(int id)
{
	QBitmap bitmap;
	m_nMode = id;
	switch(m_nMode)
	{
	case 0: // Loupe
		bitmap = QPixmap::fromMimeSource("menuedit/zoom32.png");
		break;
	case 1: // Remplissage
		bitmap = QPixmap::fromMimeSource("menuedit/paint32.png");
		break;
	case 2: // Mode rayon
		bitmap = QPixmap::fromMimeSource("menuedit/pen32.png");
		break;
	case 3: // Mode ligne
		QCursor c(CrossCursor);	
		m_pImage->setCursor(c);
	}
	if(id < 3)
	{
		QCursor c(bitmap, bitmap.createHeuristicMask());	
		m_pImage->setCursor(c);
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::FillZone(int x, int y, uchar &val)
{
	QSize size = m_pImage->GetSize();
	if(MinMax(x, 0, size.width() - 1) || MinMax(y, 0, size.height() - 1))
	{
		return;
	}
	if(val != m_pImage->GetPixel(x, y))
	{
		m_pImage->SetPixel(x, y, val);
		FillZone(x + 1, y, val);
		FillZone(x, y + 1, val);
		FillZone(x - 1, y, val);
		FillZone(x, y - 1, val);
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::SetZoom(int nZoom)
{
	if(nZoom < 1 || nZoom > 16) {
		return;
	}
	m_nZoom = nZoom;
	UpdateView();
	m_pZoom->setText(QString("Zoom %1x").arg(m_nZoom));
}

//-----------------------------------------------------------------------------------
bool CImagEditor::MinMax(int& val, int min, int max)
{
	if(val < min) {
		val = min;
	}
	else {
		if(val > max) {
			val = max;
		}
		else {
			return false;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------------
void CImagEditor::ImportImag(QWidget* parent, CPainterDevice* device)
{
	QString sFilter = CLanguage::GetWordTranslation(CLanguage::WO_INPUTIMAGEFILTER);
	QString s = CMessageBox::ChooseOpenFileName(QString::null, sFilter, parent);
	if(!s.isEmpty()) {
		if(!device->LoadFile(s)) {
			CMessageBox::MsgCritical(parent, tr("Le chargement de l'image '%1' a chou.").arg(s));
		}
	}
}

//-----------------------------------------------------------------------------------
void CImagEditor::ExportImag(QWidget* parent, CPainterDevice* device)
{
	QString sFilter = CLanguage::GetWordTranslation(CLanguage::WO_OUTPUTIMAGEFILTER);
	QString s = CMessageBox::ChooseSaveFileName(QString::null, sFilter, parent, "bmp");
	if(!s.isEmpty()) {
		if(!device->SaveFile(s)) {
			CMessageBox::MsgCritical(parent, tr("La sauvegarde du fichier '%1' a chou.").arg(s));
		}
	}
}

//-----------------------------------------------------------------------------------
int CImagEditor::AddOptionItem(const QString& text, const QObject* receiver, const char* member, const QKeySequence& accel, int index)
{
	return m_pOptionMenu->insertItem(text, receiver, member, accel, -1, index);
}

//-----------------------------------------------------------------------------------
int CImagEditor::AddOptionItem(const QString& text, QPopupMenu* popup, int index)
{
	return m_pOptionMenu->insertItem(text, popup, -1, index);
}

//-----------------------------------------------------------------------------------
void CImagEditor::AddOptionSeparator(int index)
{
	m_pOptionMenu->insertSeparator(index);
}

//-----------------------------------------------------------------------------------
void CImagEditor::ChangeOptionItem(int id, const QString& text)
{
	if(id < 0 || id > 4){
		m_pOptionMenu->changeItem(id, text);
	}
}

//-----------------------------------------------------------------------------------
CPainterDevice* CImagEditor::GetPainterDevice()
{
	return m_pImage;
}

//-----------------------------------------------------------------------------------
bool CImagEditor::eventFilter(QObject* o, QEvent* e)
{
	if(o == m_pImage && e->type() == QEvent::Leave) {
		m_pLocation->clear();
	}
	else if(o == m_pView->clipper() && e->type() ==  QEvent::Resize) {
		UpdateView();
	}
	return QWidget::eventFilter(o, e);
}

//-----------------------------------------------------------------------------------
void CImagEditor::UpdateView()
{
	QSize size = m_pImage->GetSize();
	int width  = size.width()  * m_nZoom;
	int height = size.height() * m_nZoom;
	QWidget* clipper = m_pView->viewport();
	int x = 0;
	int y = 0;
	if(clipper->height() > height) {
		y = (clipper->height() - height) >> 1;
	}
	if(clipper->width() > width) {
		x = (clipper->width() - width) >> 1;
	}
	m_pView->moveChild(m_pImage, x, y);
	m_pImage->resize(width, height);
	m_pView->updateContents();
}
