// ImagEditor.h: interface for the CImagEditor class.
//
//////////////////////////////////////////////////////////////////////

#ifndef IMAGEDITOR_H
#define IMAGEDITOR_H

#include <qwidget.h>
#include <qkeysequence.h> 

class CPainterDevice;
class QPushButton;
class QPopupMenu;
class QLabel;
class QScrollView;

#define NB_TOOL 4

class CImagEditor : public QWidget
{
	Q_OBJECT
public:
	CImagEditor(QWidget* parent);

	void SetImageBackgroundColor(const QColor& color);
	void SetImageMinSize(int nWidth, int nHeight);
	void SetImageSize(int nWidth, int nHeight);
	void DoInitWords();
	void SetImageData(uchar* pData, unsigned int nSize);
	int  AddOptionItem(const QString& text, const QObject* receiver, const char* member, const QKeySequence& accel = 0, int index = -1);
	int  AddOptionItem(const QString& text, QPopupMenu* popup, int index = -1);
	void AddOptionSeparator(int index = -1);
	void ChangeOptionItem(int id, const QString& text);
	CPainterDevice* GetPainterDevice();

protected:
	bool        eventFilter(QObject* o, QEvent* e);
	void        UpdateView();

public:
	static void ImportImag(QWidget* parent, CPainterDevice* device);
	static void ExportImag(QWidget* parent, CPainterDevice* device);

private:
	bool MinMax(int& val, int min, int max);
	void SetZoom(int nZoom);
	void FillZone(int x, int y, uchar &val);

protected slots:
	void OnImageMouseRelease(int, int, int);
	void OnImageMouseMove(int, int, int);
	void OnImageMousePressed(int, int, int);
	void OnApply();
	void OnOptionClicked(int);
	void OnToolCliked(int);

signals:
	void Apply(uchar*);

private:
	QScrollView*    m_pView;
	CPainterDevice* m_pImage;
	QPushButton*    m_pApply;
	QLabel*         m_pZoom;
	QLabel*         m_pLocation;
	QPushButton*    m_pOptions;
	QPushButton*    m_pTools[NB_TOOL];
	QPopupMenu*     m_pOptionMenu;
	int             m_nMode;
	int             m_nZoom;
	int             m_nCurX;
	int             m_nCurY;
};

#endif
