// MenuSettings.cpp: implementation of the CMenuSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "MenuSettings.h"

#include <language.h>

#include <qheader.h>

//-----------------------------------------------------------------------------------
CMenuSettings::CMenuSettings(IMainWindow* pMainWindow)
	: CSettingsWidget(pMainWindow, TOOL_NAME)
	, m_qVBoxLayout(this, 10)
	, m_qList(this)
{
	m_qList.addColumn(QString::null);
	m_qList.header()->hide();
	m_qList.setSorting(-1);

	m_qVBoxLayout.addWidget(&m_qList);

	int i;

	m_pControllers[0] = new QCheckListItem(&m_qList, QString::null, QCheckListItem::RadioButtonController);
	for(i = 1; i < 5; i++)
	{
		m_pControllers[i] = new QCheckListItem(&m_qList, m_pControllers[i-1], QString::null, QCheckListItem::RadioButtonController);
	}

	for(i = 0; i < 5; i++)
	{
		m_pRadioButton[3*i+2] = new QCheckListItem(m_pControllers[i], QString::null, QCheckListItem::RadioButton);
		m_pRadioButton[3*i+1] = new QCheckListItem(m_pControllers[i], QString::null, QCheckListItem::RadioButton);
		m_pRadioButton[3*i]   = new QCheckListItem(m_pControllers[i], QString::null, QCheckListItem::RadioButton);
		m_qList.ensureItemVisible(m_pRadioButton[3*i]);
	}

	DoInitWords();
	DoRefresh();
}

//-----------------------------------------------------------------------------------
void CMenuSettings::DoRefresh()
{
	ReadSettings("Preference", m_nPreference, 0xFFFF);

	int nPref = m_nPreference;
	for(int i = 0; i < 5; i++)
	{
		int n = nPref&3;
		m_pRadioButton[3*i]->setOn(n&1);
		m_pRadioButton[3*i+1]->setOn(n == 2);
		m_pRadioButton[3*i+2]->setOn(n == 0);
		nPref >>= 2;
	}
}

//-----------------------------------------------------------------------------------
int CMenuSettings::DoSave()
{
	m_nPreference = 0;
	int j = 0;
	for(int i = 0; i < 5; i++)
	{
		m_nPreference += (m_pRadioButton[3*i+1]->isOn()*2 + m_pRadioButton[3*i]->isOn()) << j;
		j += 2;
	}
	WriteSettings("Preference", m_nPreference);
	return 1;
}

//-----------------------------------------------------------------------------------
QString CMenuSettings::GetDescription() const
{
	return QString::null;
}

//-----------------------------------------------------------------------------------
int CMenuSettings::GetPreference(int nPref)
{
	switch(nPref)
	{	
	case PREF_SYSTEM_MODIF_ICON_ON_ALL_MENU:
		return m_nPreference & 3;
	case PREF_SYSTEM_REMOVE_ICON:
		return (m_nPreference & 12) >> 2;
	case PREF_SYSTEM_REPORT_TITLE:
		return (m_nPreference & 48) >> 4;
	case PREF_SYSTEM_REPORT_LAST_ICON:
		return (m_nPreference & 192) >> 6;
	case PREF_SYSTEM_MODIF_TITLE_ON_ALL_MENU:
		return (m_nPreference & 768) >> 8;
	}
	return -1; //Pas trouv.
}

//-----------------------------------------------------------------------------------
void CMenuSettings::DoInitWords()
{
	setCaption(CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM));
	m_pControllers[0]->setText(0, tr("Modifier l'icne dite dans tous les menus"));
	m_pControllers[1]->setText(0, tr("Mettre  jour les touches de raccourcis"));
	m_pControllers[2]->setText(0, tr("Reporter le titre sur le nouveau menu cr"));
	m_pControllers[3]->setText(0, tr("Reporter les huit dernires icnes sur le nouveau menu cr"));
	m_pControllers[4]->setText(0, tr("Modifier le titre dit dans tous les menus"));

	for(int i = 0; i < 15; i+=3)
	{
		m_pRadioButton[i]->setText(0,   tr("Demander"));
		m_pRadioButton[i+1]->setText(0, tr("Toujours"));
		m_pRadioButton[i+2]->setText(0, tr("Jamais"));
	}
}
