
#include "MenuWindow.h"

#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qscrollbar.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtabwidget.h>

#include <language.h>
#include <IOptionWindow.h>
#include <MessageBox.h>
#include <utils.h>
#include <image_defs.h>
#include <SendToMenu.h>
#include <CalcInfo.h>

#include "PainterDevice.h"
#include "MenuSettings.h"
#include "ImagEditor.h"

#include "../DriveWindow/defs.h"

#define SB_X      120
#define SB_Y       16
#define SB_WIDTH    8
#define SB_HEIGHT  39

//-----------------------------------------------------------------------------------
MenuWindow::MenuWindow(CMenuSettings* pMenuSetting, CONSTRUCTOR_PARAM)
	: CheckFileWindow(CALL_PARAM, FALSE, FALSE)
{
	m_pMenuSetting = pMenuSetting;
	setIcon(QPixmap::fromMimeSource(IMG_SEE));
	
	QWidget* pWidget     = new QWidget(0);
	QGridLayout* pLayout = new QGridLayout(pWidget, 2, 2, 5);

	SetCentralWidget(pWidget);

	m_pMenuGroup   = new QGroupBox(pWidget);
	m_pInfoGroup   = new QGroupBox(pWidget);
	m_pEditors     = new QTabWidget(pWidget);
	m_pOptionMenu  = new QPopupMenu(this);
	m_pMenuKey     = new QPopupMenu(this);
	m_pActionMenu  = new QPopupMenu(this);
	char c[] = "1";
	int i;
	for(i = 0; i < 35; i++) {
		m_pMenuKey->insertItem(c, i);
		if(i == 8) {
			c[0] = 'A';
		}
		else {
			c[0]++;
		}
	}

	pLayout->addWidget(m_pMenuGroup, 0, 0);
	pLayout->addWidget(m_pEditors,   0, 1);
	pLayout->addMultiCellWidget(m_pInfoGroup, 1, 1, 0, 1, AlignBottom);

	pLayout->setColStretch(0, 10);
	pLayout->setColStretch(1, 18);

	//Zone Menu
	pMenu        = new CPainterDevice(m_pMenuGroup, TRUE);
    m_pCmdOption = new QPushButton(m_pMenuGroup);

	pMenu->SetSize(CCasioSystem::MENU_WIDTH, CCasioSystem::MENU_HEIGHT);
	pMenu->setMinimumSize(CCasioSystem::MENU_WIDTH, CCasioSystem::MENU_HEIGHT);

	pLayout = new QGridLayout(m_pMenuGroup, 1, 1, 15);
	pLayout->addWidget(pMenu,        0, 0);
	pLayout->addWidget(m_pCmdOption, 1, 0);
		
	//Zone Editeurs
    m_pIconEditor      = new CImagEditor(m_pEditors);
	m_pTitleEditor     = new CImagEditor(m_pEditors);
	m_pScrollBarEditor = new CImagEditor(m_pEditors);;
	
	m_pEditors->addTab(m_pIconEditor,      QString::null);
	m_pEditors->addTab(m_pTitleEditor,     QString::null);
	m_pEditors->addTab(m_pScrollBarEditor, QString::null);

	m_nBorderId = m_pIconEditor->AddOptionItem(QString::null, this, SLOT(OnAddBorder()), 0, 0);
	m_nKeyId    = m_pIconEditor->AddOptionItem(QString::null, m_pMenuKey, 1);
	m_pIconEditor->AddOptionSeparator(2);
	m_pIconEditor->SetImageSize(CCasioSystem::ICON_WIDTH, CCasioSystem::ICON_HEIGHT);
	m_pIconEditor->SetImageBackgroundColor(QColor(128, 128, 128));

	m_pTitleEditor->SetImageSize(CCasioSystem::MENU_WIDTH, CCasioSystem::TITLE_HEIGHT);
	m_pTitleEditor->SetImageBackgroundColor(QColor(128, 128, 128));

	m_pScrollBarEditor->SetImageSize(SB_WIDTH, SB_HEIGHT);
	m_pScrollBarEditor->SetImageBackgroundColor(QColor(128, 128, 128));

	m_pIconEditor->SetImageMinSize(240, 152);
	m_pTitleEditor->SetImageMinSize(240, 152);
	m_pScrollBarEditor->SetImageMinSize(240, 152);

	//Zone info
	QGridLayout* pInfoLayout = new QGridLayout(m_pInfoGroup, 1, 1, 15);
	pLayout = new QGridLayout(pInfoLayout, 4, 4, 2);
	for(i = 0; i < 4; i++)
	{
		label[i] = new QLabel(m_pInfoGroup);
		label[i]->setAlignment(AlignRight);
	}

	m_pCmdActions = new QPushButton(m_pInfoGroup);

	for(i = 0; i < 4; i++)
	{
		edit[i] = new QLineEdit(m_pInfoGroup);
		edit[i]->setAlignment(AlignHCenter);
		edit[i]->setFrameStyle(QFrame::Box | QFrame::Plain);
		edit[i]->setLineWidth(1);
	}

	edit[0]->setMaxLength(12 + 12);//nom du fichier
	edit[1]->setMaxLength(12);//nom de l'image
	edit[2]->setMaxLength(19);//arguments
	edit[3]->setMaxLength(4);//version

	scrollIco = new QScrollBar(m_pInfoGroup);
	chkBmp    = new QCheckBox(m_pInfoGroup);

	scrollIco->setOrientation(Horizontal);

	//Chemin
	pLayout->addWidget(label[0], 0, 0, AlignRight | AlignVCenter);
	pLayout->addWidget(edit[0], 0, 1);
	//BMP
	pLayout->addWidget(label[1], 1, 0, AlignRight | AlignVCenter);
	pLayout->addWidget(edit[1], 1, 1);
	//Paramtres
	pLayout->addWidget(label[2], 2, 0, AlignRight | AlignVCenter);
	pLayout->addWidget(edit[2], 2, 1);
	//Version
	pLayout->addWidget(label[3], 2, 2, AlignRight | AlignVCenter);
	pLayout->addWidget(edit[3],  2, 3, AlignRight | AlignVCenter);

	pLayout->addMultiCellWidget(m_pCmdActions, 0, 0, 2, 3);
	pLayout->addMultiCellWidget(chkBmp,        1, 1, 2, 3);

	pLayout->addMultiCellWidget(scrollIco, 3, 3, 0, 3, AlignBottom);

	m_pOptionMenu->insertItem(QString::null, 0);
	m_pOptionMenu->insertItem(QString::null, 1);
	m_pOptionMenu->insertItem(QString::null, 2);
	m_pOptionMenu->insertItem(QString::null, 3);
	m_pOptionMenu->insertSeparator(1);
	m_pOptionMenu->insertSeparator(3);

	m_pCmdOption->setPopup(m_pOptionMenu);

	for(i = 0; i < 9; ++i) {
		m_pActionMenu->insertItem(QString::null, i);
	}
	m_pActionMenu->insertItem(QPixmap::fromMimeSource(IMG_SENDTO), QString::null, GetMainWindow()->GetSendToMenu(), 9, 8);


	m_pActionMenu->insertSeparator(1);
	m_pActionMenu->insertSeparator(5);
	m_pActionMenu->insertSeparator(9);
	m_pActionMenu->insertSeparator(12);
	m_pCmdActions->setPopup(m_pActionMenu);

	m_pActionMenu->setAccel(CTRL + Key_A,      0);	
	m_pActionMenu->setAccel(ALT  + Key_Left,   1);
	m_pActionMenu->setAccel(ALT  + Key_Right,  2);
	m_pActionMenu->setAccel(CTRL + Key_Delete, 3);
	m_pActionMenu->setAccel(CTRL + Key_S,      4);
	m_pActionMenu->setAccel(CTRL + Key_R,      6);

//Connection
	connect(scrollIco, SIGNAL(valueChanged(int)),SLOT(ScrollIco(int)));
	connect(pMenu, SIGNAL(mouseReleased(int, int, int)), SLOT(MenuClick(int, int, int)));
	connect(chkBmp, SIGNAL(stateChanged(int)), SLOT(ChkChange()));
	connect(edit[0], SIGNAL(textChanged(const QString&)), SLOT(OnPathChanged(const QString&)));
	connect(edit[1], SIGNAL(textChanged(const QString&)), SLOT(BmpChanged(const QString&)));
	connect(edit[2], SIGNAL(textChanged(const QString&)), SLOT(ArgsChanged(const QString&)));
	connect(edit[3], SIGNAL(textChanged(const QString&)), SLOT(VersionChanged(const QString&)));
	connect(m_pMenuKey, SIGNAL(activated(int)), SLOT(OnAddKey(int)));
	connect(m_pOptionMenu, SIGNAL(activated(int)), SLOT(OptionMenuClick(int)));
	connect(m_pActionMenu, SIGNAL(activated(int)), SLOT(OnAction(int)));
	connect(m_pIconEditor, SIGNAL(Apply(uchar*)),  SLOT(OnIconApply(uchar*)));
	connect(m_pTitleEditor, SIGNAL(Apply(uchar*)),  SLOT(OnTitleApply(uchar*)));
	connect(m_pScrollBarEditor, SIGNAL(Apply(uchar*)),  SLOT(OnScrollBarApply(uchar*)));
	connect(m_pActionMenu, SIGNAL(aboutToShow()), SLOT(OnSendTo()));

	m_bModified			 = FALSE;
	m_bLockSignalChanged = FALSE;
}

//-----------------------------------------------------------------------------------
int MenuWindow::Init(INIT_PARAM)
{
	m_nShift = nShift;
	m_pActionMenu->setItemEnabled(7 , GetMainWindow()->IsLoaded(DRIVEEDIT_WINDOWTYPESTR));
	DoInitWords();
	if(sFileName.isEmpty()) {
		return 1;
	}
	if(!SetFileName(sFileName, FALSE)) {
		return 0;
	}
	return DoReloadFile(false);
}

//-----------------------------------------------------------------------------------
int MenuWindow::DoReloadFile(bool bShowValidErr)
{
	string path = m_sFileName;
	HND h = OpenDescModeRead(m_sFileName);

	if(h == INVALID_HANDLE_VALUE) {
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(m_sFileName));
		return 0;
	}
	memset(m_pBuffer, 0xFD, DRIVE_SIZE);
	SeekDesc(h, m_nShift);
	ReadDesc(h, m_pBuffer, F100MIN(DRIVE_SIZE, FileSize(h)));
	CloseDesc(h);
	m_cSystem.OpenDriveFile(m_pBuffer, sizeof(m_pBuffer));
	if(!m_cSystem.IsValid()) {
		if(bShowValidErr) {
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(m_sFileName));
		}
		return -1;
	}
	m_nNbIcon = m_cSystem.GetNbIcon();
	m_nNbMenu = m_cSystem.GetNbMenu();
	scrollIco->setMaxValue(m_nNbIcon  - 1);
	m_nNumIcon = m_nNumMenu = 0;
	SetMenu(0);
	scrollIco->setValue(0);
	RefreshIco();
	RefreshMenu();
	SetFileModified(false);
	return 1;
}

//-----------------------------------------------------------------------------------
void MenuWindow::SaveSystemAs()
{
	QString s = CMessageBox::ChooseSaveFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_SYSTEMFILTER), GetMainWidget(), "mems");
	if(!s.isEmpty()) {
		if(SaveSystem(s, 0)) {
			m_nShift = 0; //Enleve le dcalage
			SetFileName(s);
			ResetTime();
			SetFileModified(false);
			emit StateChanged();
		}
	}
}

//-----------------------------------------------------------------------------------
bool MenuWindow::SaveSystem(const QString& sFileName, int nShift)
{
	HND h = OpenDesc(sFileName, GENERIC_WRITE, OPEN_ALWAYS);
	if(h == INVALID_HANDLE_VALUE){
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName, TRUE));
		return false;
	}
	SeekDesc(h, nShift);
	WriteDesc(h, m_pBuffer, m_cSystem.GetSizeInBytes());
	CloseDesc(h);
	return true;
}

//-----------------------------------------------------------------------------------
bool MenuWindow::SaveSystem()
{
	if(SaveSystem(m_sFileName, m_nShift)){
		ResetTime();
		SetFileModified(false);
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
void MenuWindow::DoInitWords()
{
    m_pCmdOption->setText(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));
    m_pMenuGroup->setTitle(tr("Menu ") + QString::number(m_nNumMenu + 1) + " / " + QString::number(m_nNbMenu));
	QString s = " :";
	label[0]->setText(CLanguage::GetWordTranslation(CLanguage::WO_PATH) + s);
	label[1]->setText("BMP" + s);
	label[2]->setText(tr("Paramtres") + s);
	label[3]->setText(tr("Version") + s);

	chkBmp->setText(tr("Afficher dans System.exe"));
	
	m_pIconEditor->DoInitWords();
	m_pTitleEditor->DoInitWords();
	m_pScrollBarEditor->DoInitWords();

	m_pEditors->changeTab(m_pIconEditor,      tr("Editeur d'icne"));
	m_pEditors->changeTab(m_pTitleEditor,     tr("Editeur de titre"));
	m_pEditors->changeTab(m_pScrollBarEditor, tr("Editeur de barre"));

	m_pIconEditor->ChangeOptionItem(m_nBorderId, tr("Insrer bordure"));
	m_pIconEditor->ChangeOptionItem(m_nKeyId,    tr("Insrer touche"));

	m_pOptionMenu->changeItem(0, tr("Ajouter les touches de raccourcis sur tous les menus"));
	m_pOptionMenu->changeItem(1, tr("Inverser l'image"));
	m_pOptionMenu->changeItem(2, CLanguage::GetWordTranslation(CLanguage::WO_IMPORT));
	m_pOptionMenu->changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_EXPORT));

	m_pCmdActions->setText(tr("Actions"));
	m_pActionMenu->changeItem(0, tr("Ajouter une icne"));
	m_pActionMenu->changeItem(1, tr("Dcaler l'icne vers la gauche"));
	m_pActionMenu->changeItem(2, tr("Dcaler l'icne vers la droite"));
	m_pActionMenu->changeItem(3, tr("Supprimer l'icne"));
	m_pActionMenu->changeItem(4, tr("Sauver le systme"));
	m_pActionMenu->changeItem(5, tr("Sauver le systme sous..."));
	m_pActionMenu->changeItem(6, tr("Recharger le systme"));
	m_pActionMenu->changeItem(7, tr("Voir le lecteur"));
	m_pActionMenu->changeItem(9, CLanguage::GetWordTranslation(CLanguage::WO_SENDTO));
	m_pActionMenu->changeItem(8, tr("Ajouter l'icone de Tutor"));
	
	RefreshMenuText();
	RefreshInformationTitle();
}

//-----------------------------------------------------------------------------------
void MenuWindow::SetMenu(int val)
{
	if(val < 0 || val >= m_nNbMenu) {
		return;
	}
	if(val == m_nNumMenu) {
		RefreshMenu();
		return;
	}
	m_nNumMenu = val;
	if(m_nNumIcon / 4 > 2 + m_nNumMenu)
	{
		m_nNumIcon -= 4 * (m_nNumIcon / 4 - 2 - m_nNumMenu);
		scrollIco->setValue(m_nNumIcon);
	}	
	if(m_nNumIcon / 4 <  m_nNumMenu)
	{
		m_nNumIcon += 4 * (m_nNumMenu - m_nNumIcon / 4);
		scrollIco->setValue(m_nNumIcon);
	}
	RefreshMenu();
}

//-----------------------------------------------------------------------------------
void MenuWindow::ScrollIco(int val)
{
	if(val == m_nNumIcon)
	{
		RefreshIco();
		return;
	}
	InvertSelection();
	m_nNumIcon = scrollIco->value();
	if(m_nNumIcon / 4 > 2 + m_nNumMenu)
	{
		SetMenu(m_nNumIcon / 4 - 2);
	}
	else if(m_nNumIcon <  4 * m_nNumMenu)
	{
		SetMenu(m_nNumIcon / 4);
	}
	else{
		InvertSelection();
	}
	RefreshIco();
}

//-----------------------------------------------------------------------------------
void MenuWindow::RefreshMenu()
{
	m_cSystem.GetMenu(pMenu->GetData(), m_nNumMenu);
	DrawArrows(true);
	InvertSelection();
	RefreshMenuText();
}

//-----------------------------------------------------------------------------------
void MenuWindow::RefreshMenuText()
{
	QString sText = tr("Menu %1 / %2")
		.arg(m_nNumMenu + 1)
		.arg(m_nNbMenu);
	m_pMenuGroup->setTitle(sText);
}

//-----------------------------------------------------------------------------------
void MenuWindow::RefreshIco()
{
	m_bLockSignalChanged = true;
	m_pActionMenu->setItemEnabled(1, m_nNumIcon);
	m_pActionMenu->setItemEnabled(2, m_nNumIcon != m_nNbIcon - 1);

	edit[0]->setText(m_cSystem.GetIconPath(m_nNumIcon).c_str());
	edit[1]->setText(m_cSystem.GetIconImageName(m_nNumIcon).c_str());
	edit[2]->setText(m_cSystem.GetIconArguments(m_nNumIcon).c_str());
	edit[3]->setText(m_cSystem.GetIconVersion(m_nNumIcon).c_str());
	chkBmp->setChecked(m_cSystem.GetIconVisibility(m_nNumIcon) == CCasioSystem::IV_SHOW);
	RefreshInformationTitle();

	m_bLockSignalChanged = false;
}

//-----------------------------------------------------------------------------------
void MenuWindow::InvertSelection()
{
	int nMinIco = 4 * m_nNumMenu;
	int x = (m_nNumIcon - nMinIco) % 4;
	int y = (m_nNumIcon - nMinIco) / 4;
	pMenu->InvertPixels(30 * x, 19 * y + 7, 30, 19); 
}

//-----------------------------------------------------------------------------------
void MenuWindow::MenuClick(int button, int nScreenX, int nScreenY)
{
	int x, y;
	pMenu->GetWorldPos(nScreenX, nScreenY, x, y);
	bool bOk = (x < CCasioSystem::MENU_WIDTH && y < CCasioSystem::MENU_HEIGHT && y >= 0 && x >= 0);
	bool bInTitle = bOk && y < CCasioSystem::TITLE_HEIGHT;
	bool bInIcon  = bOk && x <= 119 && y >= CCasioSystem::TITLE_HEIGHT;
	bool bInScroolBar = bOk && x >= SB_X && y >= SB_Y && x < SB_X + SB_WIDTH && y < SB_Y + SB_HEIGHT;
	bool bInTopArrow    = bOk && x >= 121 && x <= 125 && y >=  8 && y <= 14; 
	bool bInBottomArrow = bOk && x >= 121 && x <= 125 && y >= 56 && y <= 62;
    if((button == LeftButton || button == RightButton) && bInIcon)
	{
		int nIco = x / 30 + 4 * ((y - CCasioSystem::TITLE_HEIGHT) / 19) + 4 * m_nNumMenu;
		if(nIco >= m_nNbIcon) {
			return;
		}
		scrollIco->setValue(nIco);
	}
	if(button == LeftButton) {
		if(bInTopArrow) {
			SetMenu(m_nNumMenu - 1);
		}
		else if(bInBottomArrow) {
			SetMenu(m_nNumMenu + 1);
		}
	}
    if(button == RightButton)
	{
		if(bInTitle) {
			m_cSystem.GetTitle(m_pTitleEditor->GetPainterDevice()->GetData(), m_nNumMenu);
			m_pTitleEditor->GetPainterDevice()->update();
			m_pEditors->showPage(m_pTitleEditor);
		}else if(bInIcon) {
			m_cSystem.GetIconImage(m_pIconEditor->GetPainterDevice()->GetData(), m_nNumIcon, m_nNumMenu - CCasioSystem::GetFirstMenuIndex(m_nNumIcon));
			m_pIconEditor->GetPainterDevice()->update();
			m_pEditors->showPage(m_pIconEditor);
		}else if(bInScroolBar){
			char menu[CCasioSystem::SIZEBMP];
			m_cSystem.GetMenu(menu, m_nNumMenu);
			for(int x = 0; x < SB_WIDTH; ++x){
				for(int y = 0; y < SB_HEIGHT; ++y){
					m_pScrollBarEditor->GetPainterDevice()->SetPixel(x, y, CCasioSystem::GetPixel(menu, x + SB_X, y + SB_Y, CCasioSystem::MENU_BYTE_PER_ROW));
				}
			}
			m_pEditors->showPage(m_pScrollBarEditor);
		}
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::ChkChange()
{
	if(chkBmp->isChecked()){
		m_cSystem.SetIconVisibility(m_nNumIcon, CCasioSystem::IV_SHOW);
	}
	else{
		m_cSystem.SetIconVisibility(m_nNumIcon, CCasioSystem::IV_HIDE);
	}
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnPathChanged(const QString& s)
{
	m_cSystem.SetIconPath(s.upper().latin1(), m_nNumIcon);
	SetFileModified(true);
	int nPos = edit[0]->cursorPosition();
	edit[0]->setText(m_cSystem.GetIconPath(m_nNumIcon).c_str());
	edit[0]->setCursorPosition(nPos);
}

//-----------------------------------------------------------------------------------
void MenuWindow::BmpChanged(const QString& s)
{
	m_cSystem.SetIconImageName(s.latin1(), m_nNumIcon);
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::ArgsChanged(const QString& s)
{
	m_cSystem.SetIconArguments(s.latin1(), m_nNumIcon);
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::VersionChanged(const QString& s)
{
	m_cSystem.SetIconVersion(s.latin1(), m_nNumIcon);
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::AddIcon()
{
	if(m_nNbIcon < CCasioSystem::MAXICON)
	{
		bool bReportTitle  = false;
		bool bReportIcon = false;
		if(CCasioSystem::GetNbMenu(m_nNbIcon + 1) != m_nNbMenu)
		{
			int nSetting = m_pMenuSetting->GetPreference(CMenuSettings::PREF_SYSTEM_REPORT_TITLE);
			bReportTitle  = nSetting&2;
			QString s = tr("En ajoutant une icne vous avez cr un menu supplmentaire.") + '\n';
			if(nSetting & 1)
			{
				QString msg = s + tr("Voulez-vous reporter le titre du dernier menu sur le nouveau ?");
				bReportTitle = CMessageBox::MsgQuestion(GetMainWidget(), msg);
			}
			nSetting = m_pMenuSetting->GetPreference(CMenuSettings::PREF_SYSTEM_REPORT_LAST_ICON);
			bReportIcon = nSetting&2;
			if(nSetting & 1)
			{
				QString msg = s + tr("Voulez-vous copier les 8 dernires icnes sur le nouveau menu ?");
				bReportIcon = CMessageBox::MsgQuestion(GetMainWidget(), msg);
			}
		}
		m_cSystem.AddIcon(NULL, NULL, bReportTitle, bReportIcon, false);
		m_nNbIcon = m_cSystem.GetNbIcon();
		m_nNbMenu = m_cSystem.GetNbMenu();

		scrollIco->setMaxValue(m_nNbIcon - 1);
		scrollIco->setValue(m_nNbIcon - 1);
		if(m_nNbIcon == 1) {
			RefreshIco();
		}
		RefreshMenu();
		SetFileModified(true);
	}
	else {
		CMessageBox::MsgExclamation(GetMainWidget(), tr("Vous avez atteint le nombre maximum d'icne!"));
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::RemoveIcon()
{
	if(m_nNbIcon > 1) {
		m_cSystem.RemoveIcon(m_nNumIcon);
		m_nNbIcon = m_cSystem.GetNbIcon();
		m_nNbMenu = m_cSystem.GetNbMenu();
		if(scrollIco->value() == m_nNbIcon)	{
			scrollIco->setValue(m_nNbIcon - 1);
			SetMenu(m_nNbMenu - 1);
		}
		else {
			if(m_nNumMenu >= m_nNbMenu)	{
				SetMenu(m_nNbMenu - 1);
			}
			RefreshIco();
			ShowQuestionRefreshKey();
		}
		scrollIco->setMaxValue(m_nNbIcon - 1);
		RefreshMenu();
		SetFileModified(true);
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnIconApply(uchar* pData)
{	
	int nSetting = m_pMenuSetting->GetPreference(CMenuSettings::PREF_SYSTEM_MODIF_ICON_ON_ALL_MENU);
	bool bAll = false;
	if(nSetting & 1) {
		if(m_nNbMenu > 1 && m_nNumIcon < m_nNbIcon - CCasioSystem::MENU_ROW) {
			bAll = CMessageBox::MsgQuestion(GetMainWidget(), tr("Voulez-vous modifier cette icne dans tous les menus ?"));
		}
	}
	else {
		bAll = nSetting & 2;
	}
	if(bAll) {
		m_cSystem.SetIconImage(pData, m_nNumIcon, -1);
	}
	else {
		m_cSystem.SetIconImage(pData, m_nNumIcon, m_nNumMenu - CCasioSystem::GetFirstMenuIndex(m_nNumIcon));
	}
	RefreshMenu();
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnTitleApply(uchar* pData)
{
	int nSetting = m_pMenuSetting->GetPreference(CMenuSettings::PREF_SYSTEM_MODIF_TITLE_ON_ALL_MENU);
	bool bAll;
	if(nSetting & 1) {
		bAll = CMessageBox::MsgQuestion(GetMainWidget(), tr("Voulez-vous appliquer ce titre  tous les menus ?"));
	}
	else {
		bAll = nSetting & 2;
	}
	if(bAll) {
		for(int i = 0; i < m_nNbMenu; i++) {
			m_cSystem.SetTitle(m_pTitleEditor->GetPainterDevice()->GetData(), i);
		}
	}
	else {
		m_cSystem.SetTitle(m_pTitleEditor->GetPainterDevice()->GetData(), m_nNumMenu);
	}
	RefreshMenu();
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnScrollBarApply(uchar*)
{
	char menu[CCasioSystem::SIZEBMP];
	m_cSystem.GetMenu(menu, m_nNumMenu);
	for(int x = 0; x < SB_WIDTH; ++x){
		for(int y = 0; y < SB_HEIGHT; ++y){
			CCasioSystem::PutPixel(menu, x + SB_X, y + SB_Y, CCasioSystem::MENU_BYTE_PER_ROW, m_pScrollBarEditor->GetPainterDevice()->GetPixel(x, y));
		}
	}
	m_cSystem.SetMenu(menu, m_nNumMenu);
	RefreshMenu();
	SetFileModified(true);
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnAddKey(int id)
{
	CCasioSystem::PutKey(m_pIconEditor->GetPainterDevice()->GetData(), id);
	m_pIconEditor->GetPainterDevice()->update();
}

//-----------------------------------------------------------------------------------
void MenuWindow::OptionMenuClick(int id)
{
	switch(id)
	{
	case 0://Met  jour les touches de raccourci
		{
			for(int i = 0; i < m_nNbIcon; ++i){
				m_cSystem.PutKey(i);
			}
			RefreshMenu();
			SetFileModified(true);
		}
		break;
	case 1://Inverse l'image
		InvertSelection();
		DrawArrows(false);
		pMenu->InvertPixels();
		m_cSystem.SetMenu(pMenu->GetData(), m_nNumMenu);
		RefreshMenu();
		SetFileModified(true);
		break;
	case 2://Import une image
		CImagEditor::ImportImag(GetMainWidget(), pMenu);
		m_cSystem.SetMenu(pMenu->GetData(), m_nNumMenu);
		RefreshMenu();
		SetFileModified(true);
		break;
	case 3://Exporte une image
		InvertSelection();
		DrawArrows(false);
		CImagEditor::ExportImag(GetMainWidget(), pMenu);
		InvertSelection();
		DrawArrows(true);
		break;
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::ShowQuestionRefreshKey()
{
	int	nSetting = m_pMenuSetting->GetPreference(CMenuSettings::PREF_SYSTEM_REMOVE_ICON);
	bool b = nSetting & 2;
	if(nSetting & 1) {
		b = CMessageBox::MsgQuestion(GetMainWidget(), tr("Voulez-vous mettre  jour les touches de raccourcis ?"));
	}
	if(b) {
		OptionMenuClick(0);
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::DoShowCaption()
{
	QString sCaption = CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM)
		+ " (" + CCalcInfo::GetCalcName(m_cSystem.GetSystemType()) + ")";
	setCaption(sCaption);
	CheckFileWindow::DoShowCaption();
	if(m_bModified)	{
		setCaption(caption() + " *");
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::SetFileModified(bool bModified)
{
	if(!m_bLockSignalChanged) {
		m_bModified = bModified;
		DoShowCaption();
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::closeEvent(QCloseEvent* e)
{
	if(m_bModified)
	{
		switch(CMessageBox::MsgYesNoCancel(GetMainWidget(), tr("Voulez-vous sauver les changements apports avant de quitter ?")))
		{
		case QMessageBox::Cancel:
			e->ignore();
			return;
		case QMessageBox::Yes:
			SaveSystem();
		}
	}
	CheckFileWindow::closeEvent(e);
}

//-----------------------------------------------------------------------------------
int MenuWindow::LoadFrom(QDomElement& element)
{
	if(CheckFileWindow::LoadFrom(element) == 1 && !m_sFileName.isEmpty()) {
		m_nShift = element.attribute("Shift", "0").toInt();
		m_pActionMenu->setItemEnabled(5 , element.attribute("Associate", "0").toInt());
		return DoReloadFile(true);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int MenuWindow::SaveTo(QDomElement& element)
{
	if(CheckFileWindow::SaveTo(element) == 1) {
		element.setAttribute("Shift",  QString::number(m_nShift));
		element.setAttribute("Associate", QString::number(m_pActionMenu->isItemEnabled(5)));
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnAction(int id)
{
	switch(id)
	{
	case 0://Ajouter une icone
		AddIcon();
		break;
	case 1://Dcaler l'icone  gauche
			m_cSystem.ShiftIconLeft(m_nNumIcon);
			scrollIco->setValue(m_nNumIcon - 1);
			RefreshMenu();
		break;
	case 2://Dcaler l'icone  droite
			m_cSystem.ShiftIconRight(m_nNumIcon);
			scrollIco->setValue(m_nNumIcon + 1);
			RefreshMenu();
		break;
	case 3://Supprimer l'icone
		RemoveIcon();
		break;
	case 4://Sauver
		SaveSystem();
		break;
	case 5://Sauver sous
		SaveSystemAs();
		break;
	case 6://Recharger le systeme
		DoReloadFile(true);
		break;
	case 7://Voir le lecteur A
		{
			ExtraParam param;
			param.nMaxSize         = 65536;
			param.bReadOnly        = TRUE;
			param.bShowGraphWidget = TRUE;
			param.bShowOpenButton  = TRUE;
			param.pBuffer          = NULL;
			GetMainWindow()->OpenToolByType(DRIVEEDIT_WINDOWTYPESTR, m_sFileName, m_nShift + 65536, TRUE, NULL, &param);
		}
		break;
	case 8://Ajouter tutor
		{
			static unsigned char tutor_icon[] = {0, 0, 0, 3, 0, 0, 0, 7, 62, 151, 204, 231, 8, 145, 18, 151, 8, 145, 18, 231, 8, 145, 18, 167, 8, 97, 12, 151, 0, 0, 0, 7, 0, 0, 0, 7, 12, 2, 164, 167, 30, 4, 78, 71, 27, 8, 164, 71, 30, 48, 1, 255, 12, 112, 1, 255, 31, 225, 81, 255, 31, 192, 1, 255, 30, 1, 81, 255, 30, 0, 1, 255, 127, 255, 255, 255, 0};
			bool bAdd = m_nNbIcon < CCasioSystem::MAXICON;
			AddIcon();
			if(bAdd)
			{
				int n = m_nNbIcon - 1;
				m_cSystem.SetIconImageName("TUTOR", n);
				m_cSystem.SetIconImage(tutor_icon, n, -1);
				switch(m_cSystem.GetSystemType())
				{
				case CM_G100:
					m_cSystem.SetIconPath("D:\\TUTOR.EXE", n);
					break;
				case CM_G100_PLUS:
					m_cSystem.SetIconPath("F:\\JOINT2.EXE", n);
					m_cSystem.SetIconArguments("TUTOR", n);
					break;
				}
				RefreshIco();
				RefreshMenu();
			}
		}
		break;
	}
}

//-----------------------------------------------------------------------------------
void MenuWindow::RefreshInformationTitle()
{
	QString sKey;
	if(m_nNumIcon < 9) {
		sKey = QString::number(m_nNumIcon + 1);
	}
	else {
		sKey = 'A' + m_nNumIcon - 9;
	}
	m_pInfoGroup->setTitle(tr("Icne %1/%2 (Touche %3)")
		.arg(m_nNumIcon + 1)
		.arg(m_nNbIcon)
		.arg(sKey));
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnAddBorder()
{
	m_pIconEditor->GetPainterDevice()->DrawLine(1, 18, 29, 18, 1);
	m_pIconEditor->GetPainterDevice()->DrawLine(29, 1, 29, 18, 1);
}

//-----------------------------------------------------------------------------------
void MenuWindow::DrawArrows(bool bColor)
{
	if(m_nNumMenu > 0) {
		pMenu->DrawLine(123.0f, 8.0f, 123.0f, 14.0f, bColor);
		pMenu->DrawLine(121.0f, 10.0f, 123.0f, 8.0f, bColor);
		pMenu->DrawLine(125.0f, 10.0f, 123.0f, 8.0f, bColor);
	}
	if((m_nNumMenu + 1) < m_nNbMenu){
		pMenu->DrawLine(123.0f, 62.0f, 123.0f, 56.0f, bColor);
		pMenu->DrawLine(121.0f, 60.0f, 123.0f, 62.0f, bColor);
		pMenu->DrawLine(125.0f, 60.0f, 123.0f, 62.0f, bColor);
	}
	pMenu->update();
}

//-----------------------------------------------------------------------------------
void MenuWindow::OnSendTo()
{
	GetMainWindow()->GetSendToMenu()->SetFileToSend(m_sFileName);
}
