
#ifndef MENUWINDOW_H
#define MENUWINDOW_H

class QGroupBox;
class QPushButton;
class CPainterDevice;
class QLabel;
class QScrollBar;
class QLineEdit;
class QCheckBox;
class QString;
class QAction;
class QPopupMenu;
class QFrame;
class CMenuSettings;
class CImagEditor;
class QTabWidget;


#include <CheckFileWindow.h>
#include <CasioSystem.h>
#include <DriveAccess.h>

class MenuWindow : public CheckFileWindow
{ 
    Q_OBJECT

public:
    MenuWindow(CMenuSettings* pMenuSetting, CONSTRUCTOR_PARAMDEF);

	int         Init(INIT_PARAMDEF);
	int			LoadFrom(QDomElement& element);
	int			SaveTo(QDomElement& element);

protected:
	int			DoReloadFile(bool bShowValidErr);
	void		SetFileModified(bool bModified);
	void		closeEvent(QCloseEvent* e);
	void		DoShowCaption();
	void        DoInitWords();	// Recharge les mots

private:
	void		InvertSelection();
	void		RefreshMenu();
	void        RefreshMenuText();
	void        RefreshInformationTitle();
	void		RefreshIco();
	void		ShowQuestionRefreshKey();
	void		AddIcon();
	bool		SaveSystem();
	bool        SaveSystem(const QString& sFileName, int nShift);
	void		SaveSystemAs();
	void		RemoveIcon();
	void        DrawArrows(bool bColor);
	void		SetMenu(int val);

protected slots:
	void		ChkChange();
	void		ScrollIco(int val);
	void		MenuClick(int button, int nScreenX, int nScreenY);
	void		OnPathChanged(const QString&);
	void		BmpChanged(const QString&);
	void		ArgsChanged(const QString&);
	void		VersionChanged(const QString&);
	void		OptionMenuClick(int id);
	void        OnAction(int id);
	void		OnIconApply(uchar*);
	void        OnAddBorder();
	void		OnAddKey(int id);
	void		OnTitleApply(uchar*);
	void		OnScrollBarApply(uchar*);
	void        OnSendTo();

private:
	CMenuSettings* m_pMenuSetting;
	QGroupBox*     m_pMenuGroup;
	QGroupBox*     m_pInfoGroup;
	QPushButton*   m_pCmdActions;
	QPopupMenu*    m_pActionMenu;

	QLabel* label[4];
	QPopupMenu*  m_pOptionMenu;
	QPopupMenu*  m_pMenuKey;
    QPushButton* m_pCmdOption;
	QLineEdit* edit[4];
	QScrollBar* scrollIco;
	QCheckBox* chkBmp;
    CPainterDevice* pMenu;
	CImagEditor* m_pIconEditor;
	CImagEditor* m_pTitleEditor;
	CImagEditor* m_pScrollBarEditor;
	CCasioSystem m_cSystem;
	QTabWidget*  m_pEditors;
	int          m_nBorderId;
	int          m_nKeyId;
	char         m_pBuffer[DRIVE_SIZE];
	int          m_nNbMenu;
	int          m_nNbIcon;
	int          m_nNumIcon;
	int          m_nNumMenu;
	unsigned int m_nShift;
	int          m_nZoom;
	bool         m_bLockSignalChanged;
	bool         m_bModified;
};

#endif // MENUWINDOW_H
