// PainterDevice.h: interface for the CPainterDevice class.
// Classe permettant d'afficher une image en N&B
//////////////////////////////////////////////////////////////////////

#ifndef PAINTERDEVICE_H
#define PAINTERDEVICE_H


#include <qimage.h>
#include <qwidget.h>

#include <F100Qt.h>

class QBitmap;

class CPainterDevice : public QWidget
{
	Q_OBJECT
public:
	CPainterDevice(QWidget* parent = 0, bool bEnableSmoothScale = FALSE);
	void		SetSize(int w, int h);
	void		SetData(uchar* pData, unsigned int nSize);
	uchar*		GetData();
	void		SetPixel(int x, int y, int state);
	int  		GetPixel(int x, int y);
	void		InvertPixels();
	void		InvertPixels(int x, int y, int w, int h);
	void		Fill(int value);
	bool		LoadFile(const QString& sFileName);
	bool		SaveFile(const QString& sFileName);
	void		DrawLine(int x1, int y1, int x2, int y2, bool bColor);
	QSize       GetSize() const;
	void		GetWorldPos(int x, int y, int& nWorldX, int& nWorldY);
	int         GetStartX() const;
	int         GetStartY() const;
	void        SetTempText(int x, int y, const QString& text);
	void		SetTempLine(int x1, int y1, int x2, int y2, bool bColor);
	void        ClearTempLine();
	void        ClearTempText();

protected:
	void		paintEvent(QPaintEvent*);
	void        resizeEvent(QResizeEvent*);
	void		mousePressEvent(QMouseEvent*);
	void		mouseMoveEvent(QMouseEvent*);
	void		mouseReleaseEvent(QMouseEvent*);
	void		DrawImage(QPainter& p, const QImage& image);
	void        DrawGrid(QPainter& p);
	void        UpdateRatio();

protected:
	static void   DrawLineInImage(QImage& image, float x1, float y1, float x2, float y2, bool bColor);
	static QImage ConvertStringToImage(const QString& text, const QFont& font);

signals:
	void		mouseReleased(int button, int x, int y);
	void		mousePressed(int button, int x, int y);
	void		mouseMoved(int button, int x, int y);

private:
	QImage  pix;
	bool    m_bEnableSmoothScale;
	float   m_fHRatio;
	float   m_fVRatio;
	QString m_sTmpText;
	int     m_nTmpTextX;
	int     m_nTmpTextY;
	QPoint  m_qP1;
	QPoint  m_qP2;
	bool    m_bLineColor;
};

#endif
