// BASICProperties.cpp: implementation of the CBASICProperties class.
//
//////////////////////////////////////////////////////////////////////

#include "BASICProperties.h"
#include "BASICSyntaxHightlighter.h"
#include "RamManagerOption.h"

//-----------------------------------------------------------------------------------
CBASICProperties::CBASICProperties()
	: CProgSyntaxProperties()
{
	AddCommentProperty();
	QFont f;
	if(f.fromString("FA123ALG,10")) {
		SetCustomFont(&f);
	}
}

//-----------------------------------------------------------------------------------
CBASICProperties::CBASICProperties(const CBASICProperties* pSource)
	: CProgSyntaxProperties(pSource)
{

}

//-----------------------------------------------------------------------------------
QString CBASICProperties::GetName() const
{
	return "BASIC";
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CBASICProperties::CreateSyntax(QTextEdit* pEdit)
{
	return new CBASICSyntaxHightlighter(pEdit, this);
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CBASICProperties::CreateCompatibleSyntax(const void* pData, unsigned int nSize, QTextEdit* pEdit)
{
	if(pData != NULL) {
		int nMaxSize = QMIN(nSize, 500);
		const uchar* pCharData = (const uchar*)pData;
		for(int i = 0; i < nMaxSize; i++) {
			if(pCharData[i] != '\n' && (pCharData[i] < 0x20 || pCharData[i] > 0x7E)) {
				return CreateSyntax(pEdit);
			}
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CSyntaxProperties* CBASICProperties::Clone()
{
	return new CBASICProperties(this);
}

//-----------------------------------------------------------------------------------
void CBASICProperties::GetKeywords(QStringList& lKeywords)
{
	CProgSyntaxProperties::GetKeywords(lKeywords);
	for(unsigned int i = 1; i < CBASICWrapper::SYMB_SIZE; i++) {
		if(CBASICWrapper::symb_table[i][0] != '\n') {
			lKeywords.push_back(CBASICWrapper::symb_table[i]);
		}
	}
	ShowKeywords(lKeywords, CBASICWrapper::basic_table,  CBASICWrapper::TABLE_SIZE);
	ShowKeywords(lKeywords, CBASICWrapper::basic_table2, CBASICWrapper::TABLE_SIZE2);
}

//-----------------------------------------------------------------------------------
void CBASICProperties::ShowKeywords(QStringList& lKeywords, const char* pTable[], unsigned int nSize)
{
	for(unsigned int i = 0; i < nSize; i++) {
		QString s = pTable[i];
		if(s != IGNORE_STR && s.length() > 0) {
			lKeywords.push_back(s);
		}
	}
}
