// BASICSyntaxHightlighter.cpp: implementation of the CBASICSyntaxHightlighter class.
//
//////////////////////////////////////////////////////////////////////

#include "BASICSyntaxHightlighter.h"
#include "BASICProperties.h"

//-----------------------------------------------------------------------------------
CBASICSyntaxHightlighter::CBASICSyntaxHightlighter(QTextEdit* pTextEdit, CProgSyntaxProperties* pProperties)
	: CSyntaxHighlighter(pTextEdit, pProperties)
	, m_comments("^'.*")
	, m_numbers("([0-9]+|\\.)[0-9]*(exp[0-9+-])?")
	, m_vars("([^A-Z]|^)[A-Z][^A-Z]")
{
}

//-----------------------------------------------------------------------------------
int CBASICSyntaxHightlighter::highlightParagraph(const QString& text, int endStateOfLastPara)
{
	QFont f = GetFont();
	setFormat(0, text.length(), f, GetEditorForeColor());
	QColor c;

	GetFont(f, c, KEY_VAR);	
	SearchExp(&m_vars, text, f, c);

	GetFont(f, c, KEY_NUMBER);
	SearchExp(&m_numbers, text, f, c);

	HighLightKeywords(text, f);

	GetFont(f, c, KEY_COMMENT);	
	SearchExp(&m_comments, text, f, c);

	GetFont(f, c, KEY_STRING);
	if(c.isValid()) {
		int nPos = 0;
		if(endStateOfLastPara != 1) {
			nPos = text.find('\"');
		}
		if(nPos != -1) {
			int nNextPos = text.find('\"', nPos + 1);
			if(nNextPos == -1) {
				setFormat(nPos, text.length() - nPos, f, c);
				return 1;
			}
			setFormat(nPos, nNextPos - nPos + 1, f, c);
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------------
const CWrapper* CBASICSyntaxHightlighter::GetWrapper() const
{
	return &m_wrapper;
}

//-----------------------------------------------------------------------------------
void CBASICSyntaxHightlighter::HighLightKeywords(const QString& sText, QFont& f)
{
	QColor c;
	GetFont(f, c, KEY_KEYWORD);
	if(c.isValid()) {
		HighLightKeywords(sText, CBASICWrapper::basic_table,  CBASICWrapper::TABLE_SIZE,  f, c);
		HighLightKeywords(sText, CBASICWrapper::basic_table2, CBASICWrapper::TABLE_SIZE2, f, c);
		HighLightKeywords(sText, CBASICWrapper::symb_table,   CBASICWrapper::SYMB_SIZE,   f, c);
	}
}

//-----------------------------------------------------------------------------------
void CBASICSyntaxHightlighter::HighLightKeywords(const QString& sText, const char** pText, unsigned int nSize, const QFont& f, const QColor& c)
{	
	for(unsigned int i = 0; i < nSize; i++) {
		unsigned int nLength = strlen(pText[i]);
		if(nLength > 1) {
			int nPos = CBASICWrapper::FindString(sText, pText[i], nLength);
			if(nPos != -1) {
				do {
					setFormat(nPos, nLength, f, c);
					nPos = CBASICWrapper::FindString(sText, pText[i], nLength, nPos + nLength);
				}while(nPos != -1);
			}
		}
	}
}
