// BASICWrapper.cpp: implementation of the CBASICWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "BASICWrapper.h"
#include "CasioIO.h"

#include <qtextstream.h>

const char* CBASICWrapper::basic_table[] = {"Pol(", "sin ", "cos ", "tan ", "hex", "ln ", "\xCB", "\xBA", "nPr",
							"+", "xnor", "\x92", "\xD0", "\xBF(", "Mod", "\xE2x\x92", "@@",
							"sin\xCE ", "cos\xCE ", "tan\xCE ", "dec", "log ",
							"\x93\xCB", "Abs ", "nCr", "\xD6", "xor", "\xCE", "\xD1", "@@",
							"Med", "\xE2x", "Rec(", "sinh ", "cosh ", "tanh ", "oct", "\xC6",
							"Int ", "not ", "^", "*", "or", "!", "\xD2", "minY", "minX", "\"n\"",
							"@@", "sinh\xCE ", "cosh\xCE ", "tanh\xCE ", "bin", "\xCC", "Frac ",
							"Neg ", "\xDE\xCB", "/", "and", "\xBB", "\xD3", "maxY", "maxX", "\xE2y\x92",
							"Ans", "Ran# ", "\xAB", "\xAC", "xon", "xon\xCE ", "yon", "yon\xCE", 
							"Ra", "Rb", "Rr", "\xAD", "\xAE", "\xAA", "\xE4", "\xE2y", 
							"\xE1", "Cls", "@@", "Rnd", "Dec", "Hex", "Bin", "Oct", "@@", "Norm ",
							"Deg", "Rad", "Gra", "Eng", "Intg ", "\xE2xy", "Plot ", "Line", "Lbl ",
							"Fix ", "Sci ", "@@", "@@", "@@", "Dsz ", "Isz ", "Factor ", "ViewWindow ",
							"Goto ", "Prog ", "Graph Y=", "Graph f", "Graph Y>", "Graph Y<", 
							"Graph Y\xCA", "Graph Y\xCA", "Graph r=", "Graph(X,Y)=(", "\xB5", "", "", "", "",
							"P(", "Q(", "R(", "t("};

const char* CBASICWrapper::basic_table2[] = {"Xmin", "Xmax", "Xscl", "@@", "Ymin", "Ymax", "Yscl", "@@",
							"Tmin", "Tmax", "Tptch", "Xfact", "Yfact", "D Start", "D End", "D pitch",
							"RightXMin", "RightXmin", "RightXmax", "RightXscl", "@@", "RightYmin", "RightYmax",
							"RightYscl", "@@", "RightTmin", "RightTmax", "RightTptch", "@@",
							"c(Reg)", "d(Reg)", "e(Reg)", "Max(", "Det ", "Arg ", "Conjg ", "ReP ", "ImP ",
							"d/dx(", "d2/dx2(", "Solve(", "Sum(", "FMin(", "FMax(", "Seq(", "Min(", "Mean", "Median",
							"@@", "P/L-Orange", "P/L-Blue", "P/L-Green", "Orange ", "Blue ", "Green ", "OrangeG ",
							"BlueG ", "GreenG ", "@@", "@@", "@@", "@@", "@@", "@@", "Mat ", "Trn ",
							"*Row(", "*Row+(", "Row+(", "Swap(", "Dim ", "Fill(", "Identity ", "Augment(", "List->Mat(",
							"Mat->List", "Sum ", "Prod ", "Percent ", "Cuml ", "@@", "List ", "@@", "@@", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "Sim Coef",
							"Ply Coef", "Sim Result", "Ply Result", "@@", "@@", "@@", "@@", "@@", "@@", "List1",
							"List2", "List3", "List4", "List5", "List6", "@@", "@@", "@@", "@@", "@@", "@@", "Q\xA1",
							"Q\xA3", "x\xA1", "y\xA1", "x\xA2", "y\xA2", "x\xA3", "y\xA3",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "Getkey", "F Result", "F Start", "F End", "F pitch", "R Result", "R Start", "R End", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "a\xD4", "a\xD4+\xA1", "a\xD4+\xA2", "\xD4", "a\xA0", "a\xA1",
							"a\xA2", "b\xD4", "b\xD4+\xA1", "b\xD4+\xA2", "b\xA0", "b\xA1", "b\xA2", "a\xD4Start", "b\xD4Start", "@@", " And ",
							" Or ", "@@", "Not ", "@@", "@@", "@@", "@@", "Simp ", "@@", "@@", "@@", " Int/ ", " Rmdr ", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "\xEA", "\xEB", "\xECt", "\xEAt ", "\xEC",
							//SHIFT2
							"If ", "Then ", "Else ", "IfEnd", "For ", " To ", " Step ", "Next", "While ", "WhileEnd", "Do",
							"LpWhile ", "Return", "Break", "Stop", "@@", "Locate ", "Send(", "Receive(",
							"@@", "@@", "@@", "@@", "@@", "ClrText", "ClrGraph", "ClrList ", "@@", "@@", "@@", "@@", "@@",
							"DrawGraph", "@@", "DrawDyna", "DrawStat", "DrawFTG-Con", "DrawFTG-Plt", "DrawR-Con", "DrawR-Plt",
							"DrawR-SumCon", "DrawR-SumPlt", "DrawWeb ", "@@", "@@", "@@", "DispF-Tbl", "DispR-Tbl", "AutoSimp", "ManSimp",
							"@@", "@@", "@@", "@@", "@@", "@@", "MixedFrac", "ImproperFrac", "@@", "@@", "@@", "@@", "@@", "@@",
							"1-Variable ", "2-Variable ", "LinearReg ", "Med-MedLine ", "QuadReg ", "CubicReg ", "QuartReg ", "LogReg ",
							"ExpReg ", "PowerReg ", "S-Gph1 ", "S-Gph2 ", "S-Gph3 ", "Square", "Cross", "Dot", "Scatter", "xyLine", "Hist",
							"MedBox", "MeanBox", "N-Dist", "Broken", "Linear", "Med-Med", "Quad", "Cubic", "Quart", "Log", "Exp", "Power", "@@",
							"S-WindAuto", "S-WindMan", "Graph X=", "Y=Type", "r=Type", "ParamType", "@@", "X-cType", "@@", "@@", "Y>Type", "Y<Type",
							"Y\xCAType", "Y\xCAType", "@@", "@@", "G-Connect", "G-Plot", "DualGraph", "DualGtoT", "DualT+G", "DualOff", "@@", "@@",
							"BG-None", "BG-Pict ", "GridOff", "GridDot", "GridLine", "GridOn", "@@", "@@", "D Var ", "NrmSpeed", "FastSpeed", "SlowSpeed",
							"Stop&Go", "DynaCon", "DynaStop", "@@", "VarRange", "@@", "Simul ", "Poly ", "@@", "@@", "@@", "@@", "aRanType", "aRan1Type",
							"aRan2Type", "StoPict ", "RclPict ", "StoGMEM ", "RclGMEM ", "StoV-Win ", "RclV-Win ", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "Tangent ", "Normal ", "Inverse ", "Vertical ", "Horizontal ", "Text ", "Circle ", "F-Line ", "PlotOn ", "PlotOff ",
							"PlotChg ", "PxlOn ", "PxlOff ", "PxlChg ", "PxlLine ", "PxlTest(", "SortA(", "SortD(", "VarList1", "VarList2", "VarList3",
							"VarList4", "VarList5", "VarList6", "File1", "File2", "File3", "File4", "File5", "File6", "@@", "@@", "FuncOn", "SimulOn",
							"AxesOn", "CoordOn", "LabelOn", "DerivOn", "LocusOn", "SumDispOn", "G SelOn ", "T SelOn ", "D SelOn ", "R SelOn ", "DrawOn",
							"@@", "@@", "@@", "FuncOff", "SimulOff", "AxesOff", "CoordOff", "LabelOff", "DerivOff", "LocusOff", "SumDispOff", "G SelOff ",
							"T SelOff ", "D SelOff ", "R SelOff ", "DrawOff",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@",
							"@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "@@", "\xE1("
};

const char* CBASICWrapper::symb_table[] = {"\x0", "femto", "pico", "\x9A", "\xE5", "milli", "kilo", "\x9B",
"\x9C", "\x9D", "\x9E", "\x9F", "{Disp}", "\n", "->", "exp", "<=", "!=", ">=", "=>", "f\xA1", "f\xA2", "f\xA3", "f\xA4", "f\xA5", "f\xA6", "\x80", "\x81", "\x82", "\x83", "\x84", "\x85"};

unsigned int CBASICWrapper::TABLE_SIZE  = sizeof(CBASICWrapper::basic_table)  / sizeof(CBASICWrapper::basic_table[0]);
unsigned int CBASICWrapper::TABLE_SIZE2 = sizeof(CBASICWrapper::basic_table2) / sizeof(CBASICWrapper::basic_table2[0]);
unsigned int CBASICWrapper::SYMB_SIZE   = sizeof(CBASICWrapper::symb_table)   / sizeof(CBASICWrapper::symb_table[0]);

#define CHAR_SHIFT 0x80

#define TABLE2_SEPARATOR 245

//-----------------------------------------------------------------------------------
CBASICWrapper::CBASICWrapper()
	: CWrapper()
{
}

//-----------------------------------------------------------------------------------
QString CBASICWrapper::ConvertCasioDataToText(const void* pData, unsigned int nSize) const
{
	QString s;
	unsigned int nIndex = 0;
	const uchar* pChar = (const uchar*)pData;
	while(nIndex < nSize) {
		if(pChar[nIndex] < CHAR_SHIFT || pChar[nIndex] == MAGIC_CHAR1 || pChar[nIndex] == MAGIC_CHAR2) {
			if(pChar[nIndex] < (int)SYMB_SIZE) {
				s += symb_table[pChar[nIndex]];
			}
			else if(pChar[nIndex] == MAGIC_CHAR1 && ((nIndex + 1) < nSize)) {
				uchar c = pChar[++nIndex];
				if(c < TABLE2_SEPARATOR) {
					s += basic_table2[c];
				}
				else {
					s += MAGIC_CHAR1;
					s += c;
				}
			}
			else if(pChar[nIndex] == MAGIC_CHAR2 && ((nIndex + 1) < nSize)) {
				uchar c = pChar[++nIndex];
				if(c < TABLE_SIZE2 - TABLE2_SEPARATOR) {
					s += basic_table2[c + TABLE2_SEPARATOR];
				}
				else {
					s += (uchar)MAGIC_CHAR2;
					s += c;
				}

			}
			else {
				s += pChar[nIndex];
			}
		}
		else {
			if(pChar[nIndex] - CHAR_SHIFT < (int)TABLE_SIZE) {
				s += basic_table[pChar[nIndex] - CHAR_SHIFT];
			}
			else {
				s += pChar[nIndex];
			}
		}
		nIndex++;
	}
	return s;
}

//-----------------------------------------------------------------------------------
int CBASICWrapper::FindFunction(const QString& sLine, int nIndex) const
{
	return FindFunction(sLine, nIndex, basic_table,  TABLE_SIZE)
		|| FindFunction(sLine, nIndex, basic_table2, TABLE_SIZE2);
}

//-----------------------------------------------------------------------------------
int CBASICWrapper::FindFunction(const QString& sLine, int nIndex, const char** table, unsigned int nSize)
{
	for(unsigned int i = 0; i < nSize; i++) {
		unsigned int nLength = strlen(table[i]);
		if(nLength > 1) {
			int nPos = FindString(sLine, table[i], nLength, 0, nIndex);
			while(nPos != -1) {
				if(nIndex < nPos + (int)nLength) {
					return TRUE;
				}
				nPos = FindString(sLine, table[i], nLength, nPos + 1, nIndex);
			}
		}
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertSymbole(QString& sLine) const
{
	ConvertSymbole(sLine, 11, SYMB_SIZE);
	ConvertSymbole(sLine,  1,        11);
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertSymbole(QString& sLine, unsigned int nStart, unsigned int nEnd) const
{
	for(unsigned int i = nStart; i < nEnd; i++) {
		unsigned int nLength = strlen(symb_table[i]);
		int nPos = FindString(sLine, symb_table[i], nLength);
		while(nPos != -1) {
			sLine.replace(nPos, nLength, QChar(i));
			nPos = FindString(sLine, symb_table[i], nLength, nPos + 1);
		}
	}
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertChar(CharMap& map, QString& sLine, const char* pTable[], unsigned int nSize) const
{
	for(unsigned int i = 0; i < nSize; i++) {
		if(pTable[i][1] == '\0') {
			int nPos = sLine.find(pTable[i][0]);
			while(nPos != -1) {
				if(map.find(nPos) == map.end() && !FindFunction(sLine, nPos)) {
					sLine.replace(nPos, 1, QChar(i+CHAR_SHIFT));
					map[nPos] = TRUE;
				}
				nPos = sLine.find(pTable[i][0], nPos + 1);
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertChar(QString& sLine) const
{
	CharMap map;
	ConvertChar(map, sLine, basic_table,  TABLE_SIZE);
	ConvertChar(map, sLine, basic_table2, TABLE_SIZE2);
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertFunction(QString& sLine) const
{
	for(unsigned int i = 0; i < TABLE_SIZE; i++) {
		unsigned int nLength = strlen(basic_table[i]);
		if(nLength > 1) {
			int nPos = FindString(sLine, basic_table[i], nLength);
			while(nPos != -1) {
				if(FindLongerFunction(sLine, nPos, nLength, basic_table, i + 1, TABLE_SIZE) == 0
					&& FindLongerFunction(sLine, nPos, nLength, basic_table2, 0, TABLE_SIZE2) == 0) {
					sLine.replace(nPos, nLength, QChar(i+CHAR_SHIFT));
				}
				nPos = FindString(sLine, basic_table[i], nLength, nPos + 1);
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CBASICWrapper::ConvertFunction2(QString& sLine) const
{
	for(unsigned int i = 0; i < TABLE_SIZE2; i++) {
		unsigned int nLength = strlen(basic_table2[i]);
		if(nLength > 1) {
			int nPos = FindString(sLine, basic_table2[i], nLength);
			while(nPos != -1) {
				if(FindLongerFunction(sLine, nPos, nLength, basic_table2, i + 1, TABLE_SIZE2) == 0) {
					if(i < TABLE2_SEPARATOR) {
						sLine.replace(nPos, nLength, QChar(i));
						sLine.insert(nPos, MAGIC_CHAR1);
					}
					else {
						sLine.replace(nPos, nLength, QChar(i - TABLE2_SEPARATOR));
						sLine.insert(nPos, (uchar)MAGIC_CHAR2);
					}
				}
				nPos = FindString(sLine, basic_table2[i], nLength, nPos + 1);
			}
		}
	}
}

//-----------------------------------------------------------------------------------
QString CBASICWrapper::ConvertTextToCasioData(const QString& sText) const
{
	QString s;
	QTextStream stream((QString*)&sText, IO_ReadOnly);
	while(!stream.atEnd()) {
		QString sLine = stream.readLine();
		if(!s.isEmpty()) {
			s += (char)0x0D;
		}
		ConvertSymbole(sLine);
		ConvertChar(sLine);
		ConvertFunction(sLine);
		ConvertFunction2(sLine);
		s += sLine;
	}
	return s;
}

//-----------------------------------------------------------------------------------
int CBASICWrapper::FindAllString(const QString& s, const char* pText, unsigned int nIndex, unsigned int nMaxIndex)
{
	return FindString(s, pText, strlen(pText), nIndex, nMaxIndex);
}

//-----------------------------------------------------------------------------------
int CBASICWrapper::FindString(const QString& s, const char* pText, unsigned int nTextLength, unsigned int nIndex, unsigned int nMaxIndex)
{
	const char* pLine = s.latin1();
	unsigned int nCpt = 0;
	unsigned int nPos = nIndex;
	unsigned int nLineLength = s.length();
	if(nLineLength <= nMaxIndex) {
		nMaxIndex = nLineLength - 1;
	}
	while(nPos < nLineLength) {
		if(pText[nCpt] == pLine[nPos]) {
			nCpt++; nPos++;
			if(pText[nCpt] == '\0') {
				return nIndex;
			}
		}
		else {
			nPos = ++nIndex;
			nCpt = 0;
			if(nIndex + nTextLength > nLineLength || nIndex > nMaxIndex) {
				break;
			}
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICWrapper::FindLongerFunction(const QString& sLine, int nPos, unsigned int nFunctionLength, const char** table, unsigned int nIndex, unsigned int nSize)
{
	unsigned int nPosMax = nPos + nFunctionLength;
	for(unsigned int i = nIndex; i < nSize; i++) {
		unsigned int nLength = strlen(table[i]);
		if(nLength > nFunctionLength) {
			int nNewPos;
			if(nPosMax > nLength) {
				nNewPos = FindString(sLine, table[i], nLength, nPosMax - nLength, nPos);
			}
			else {
				nNewPos = FindString(sLine, table[i], nLength, 0, nPos);
			}
			if(nNewPos != -1 && nNewPos <= nPos && (nNewPos + nLength >= nPosMax)) {
				return 1;
			}
		}
	}
	return 0;
}
