// BasicZone.cpp: implementation of the CBasicZone class.
//
//////////////////////////////////////////////////////////////////////

#include "BasicZone.h"
#include "CasioIO.h"

//-----------------------------------------------------------------------------------
CBasicZone::CBasicZone()
	: CFileZone(1)
{

}

class TextStream {
	const char*  m_pData;
	unsigned int m_nSize;
	unsigned int m_nPos;
public:
	TextStream(const char* pData, unsigned int nSize) {
		 m_pData = pData; m_nSize = nSize;
		 m_nPos = 0;
	}
	
	bool AtEnd() {
		return m_nPos == m_nSize;
	}

	QString ReadLine() {
		if(AtEnd()) {
			return QString::null;
		}
		unsigned int nPos = m_nPos;
		for(; nPos < m_nSize; nPos++) {
			if(m_pData[nPos] == 0xD
				&& (nPos == 0 || ((uchar)(m_pData[nPos-1]) != MAGIC_CHAR1 && (uchar)(m_pData[nPos-1]) != MAGIC_CHAR2))) {
				break;
			}
		}
		QString s;
		s.setLatin1(m_pData+m_nPos, nPos - m_nPos);
		if(m_nSize == nPos) {
			m_nPos = m_nSize;
		}
		else {
			m_nPos = nPos + 1;
		}
		return s;
	}
};

//-----------------------------------------------------------------------------------
void CBasicZone::WriteData(QTextStream& stream)
{
	stream << "Password:" << GetPassword() << "\nBaseN:0\n";
	if(GetData() != NULL) {
		TextStream ts((const char*)GetData(), GetSize());
		stream << CCasioIO::ConvertCasioDataToCat(ts.ReadLine());
		while(!ts.AtEnd()) {
			stream << '\n' << CCasioIO::ConvertCasioDataToCat(ts.ReadLine());
		}
	}
	stream << "\\@00\n";
}

//-----------------------------------------------------------------------------------
QString CBasicZone::GetDataType() const
{
	return "PG";
}

//-----------------------------------------------------------------------------------
unsigned int CBasicZone::GetCapacity()
{
	return GetSize() + 11;
}

//-----------------------------------------------------------------------------------
QString CBasicZone::ReadLine(QTextStream& stream, bool& bAtEnd)
{
	QString s = stream.readLine();
	bAtEnd = stream.atEnd();
	if(s.find("\\@00") != -1) {
		bAtEnd = true;
	}
	return CCasioIO::ConvertCatToCasioData(s);
}

//-----------------------------------------------------------------------------------
bool CBasicZone::ReadDataFrom(QTextStream& stream, unsigned int nCapacity)
{
	QString sPassword = stream.readLine();
	QString sBase     = stream.readLine();
	if(sPassword.section(':', 0, 0) == "Password"
		&& sBase.section(':', 0, 0) == "BaseN") {
		SetPassword(sPassword.section(':', 1, 1));
		bool bAtEnd;
		QString sData = ReadLine(stream, bAtEnd);
		QString s;
		while(!bAtEnd) {
			s = ReadLine(stream, bAtEnd);
			sData += (char)0x0D + s;
		}
		unsigned int nSize = sData.length() - 1;
		char* pData = new char[nSize];
		if(pData != NULL) {
			memcpy(pData, sData.latin1(), nSize);
			SetData(pData, nSize, TRUE);
		}
		return true;
	}
	return false;
}
