// CasioIO.cpp: implementation of the CCasioIO class.
//
//////////////////////////////////////////////////////////////////////

#include "CasioIO.h"

#include <math.h>

#if defined(_MSC_VER) && (_MSC_VER <= 1200)
#	define INT64 __int64
#else
#	define INT64 long long
#endif

extern char* tab_conv[];

//-----------------------------------------------------------------------------------
void CCasioIO::WriteCasioNumberFormat(char* pBuf, double dNumber, int nExp)
{
	memset(pBuf, 0, VAR_SIZE);
	bool bExpNeg = nExp < 0;
	pBuf[1] = 0x10;
	if(dNumber < 0.0) {
		if(bExpNeg) {
			pBuf[1] = 0x50;
		}
		else {
			pBuf[1] = 0x60;
		}
	}
	else {
		if(bExpNeg) {
			pBuf[1] = 0x00;
		}
	}
	if(bExpNeg) {
		WriteCasioNumberFormat(pBuf, 100 + nExp, 2, 3);
	}
	else {
		WriteCasioNumberFormat(pBuf, nExp, 2, 3);
	}

	WriteCasioNumberFormat(pBuf, dNumber*1e12, 13, 5);
}

//-----------------------------------------------------------------------------------
void CCasioIO::WriteCasioNumberFormat(char* pBuf, double dNumber, int nSize, int nHalfByteIndex)
{
	dNumber = fabs(dNumber);
	INT64 nDiv = pow(10, nSize);
	for(int i = 0; i < nSize && dNumber > 0.0; i++) {
		nDiv /= 10;
		char n = (dNumber / nDiv);
		if(nHalfByteIndex % 2) {
			pBuf[nHalfByteIndex >> 1] |= n;
		}
		else {
			pBuf[nHalfByteIndex >> 1] = n << 4;
		}
		dNumber -= n*nDiv;
		nHalfByteIndex++;
	}
}
//-----------------------------------------------------------------------------------
double CCasioIO::ConvertCasioDataToDouble(const unsigned char* pBuf)
{
	int nExp    = ReadCasioNumberFormat(pBuf,  2, 3);
	double dNum = ReadCasioNumberFormat(pBuf, 13, 5) / 1e12;
	char c = pBuf[1] >> 4;
	if(c == 5 || c == 0) {
		nExp = nExp - 100;
	}
	if(c == 6 || c == 5) {
		dNum = -dNum;
	}
	return dNum * pow(10, nExp);
}

//-----------------------------------------------------------------------------------
double CCasioIO::ReadCasioNumberFormat(const unsigned char* pBuf, int nSize, int nHalfByteIndex)
{
	double dNumber = 0.0;
	INT64 nMult = 1;
	for(int i = 0; i < nSize; i++) {
		dNumber *= 10;
		if(nHalfByteIndex % 2) {
			dNumber += pBuf[nHalfByteIndex >> 1]&15;
		}
		else {
			dNumber += pBuf[nHalfByteIndex >> 1] >> 4;
		}
		nHalfByteIndex++;
	}
	return dNumber;
}

//-----------------------------------------------------------------------------------
void CCasioIO::WriteCasioNumberFormat(char* pBuf, double dNumber)
{
	int nExp = (int)floor(log10(fabs(dNumber)));
	WriteCasioNumberFormat(pBuf, dNumber / pow(10, nExp), nExp);
}

//-----------------------------------------------------------------------------------
QString CCasioIO::ConvertDoubleToCasioString(double v)
{
	int nExp = (int)floor(log10(fabs(v)));

	if(nExp != 0) {
		return QString::number(v / pow(10, nExp), 'g', 15) + "\\EE" + QString::number(nExp);
	}
	return QString::number(v, 'g', 15);
}

//-----------------------------------------------------------------------------------
int CCasioIO::StartsWith(const char* pText, const char* pFind, unsigned int nTextLength)
{
	int i = 0;
	for(unsigned int nCpt = 0; nCpt < nTextLength; nCpt++) {
		if(pFind[i++] == pText[nCpt]) {
			if(pFind[i] == '\0') {
				if(pText[nCpt + 1] == '\0' || (pText[nCpt] != ' ' && pText[nCpt + 1] == '\\')) {
					return 2;
				}
				return 1;
			}
		}
		else {
			return 0;
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------------
int CCasioIO::StartsWith(const QString& s, int nFind, int nIndex)
{	
	const char* c = s.latin1() + nIndex;
	int max = s.length() - nIndex;
	int nRet = StartsWith(c, tab_conv[nFind], max);
	if(nRet > 0) {
		if(nRet == 1) {
			unsigned int nLength = strlen(tab_conv[nFind]);
			for(nFind++; nFind < 768; nFind++) {
				if(nLength < strlen(tab_conv[nFind])) {
					if(StartsWith(c, tab_conv[nFind], max)) {
						return false;
					}
				}
			}
			return true;
		}
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
QString CCasioIO::ConvertCatToCasioData(const QString& sText)
{
	QString sRet;
	int nIndex = 0;
	while(nIndex < (int)sText.length()) {
		int i;
		for(i = 0; i < 768; ++i) {
			if(StartsWith(sText, i, nIndex)) {
				if(i < 256) {
					sRet += QChar(i);
				}
				else {
					if(i < 512) {
						sRet += QChar(MAGIC_CHAR1);
						sRet += QChar(i - 256);
					}
					else {
						sRet += QChar(MAGIC_CHAR2);
						sRet += QChar(i - 512);
					}
				}
				nIndex += strlen(tab_conv[i]);
				break;
			}
		}
		if(i == 768) {
			sRet += sText.at(nIndex++);
		}
	}
	return sRet;
}

//-----------------------------------------------------------------------------------
QString CCasioIO::ConvertCasioDataToCat(const QString& sData)
{
	QString sRet;
	unsigned int i = 0;
	while(i < sData.length()) {
		unsigned int c = (unsigned int)(unsigned char)(sData.at(i).latin1());
		if(c == MAGIC_CHAR1) {
			c = (unsigned int)(unsigned char)sData.at(++i).latin1();
			sRet += tab_conv[c + 256];
		}
		else if(c == MAGIC_CHAR2) {
			c = (unsigned int)(unsigned char)sData.at(++i).latin1();
			sRet += tab_conv[c + 512];
		}
		else {
			sRet += tab_conv[c];
		}
		i++;
	}
	return sRet;
}

char* tab_conv[]=
{
"\\@00",  //0
"\\femto",
"\\pico",
"\\nano",
"\\micro",
"\\milli",
"\\kilo",
"\\Mega",
"\\Giga",
"\\Tera",
"\\Peta", //10
"\\Exa",
"\\Disp",
"\n",   //voir plus loin:
"\\->",
"\\EE",
"\\<=",
"\\<>",
"\\>=",
"\\=>",
"\\f1",   //20
"\\f2",
"\\f3",
"\\f4",
"\\f5",
"\\f6",
"\\hA",
"\\hB",
"\\hC",
"\\hD",
"\\hE",   //30
"\\hF",
" ",
"!mark",
"\"",
"#",
"$",//"\\@24",
"%",//"\\@25",
"&",//"\\@26",
"'",
"(",      //40
")",
"\\aster",
"\\@2B",
",",
"\\@2D",
".",
"\\slash",
"0",
"1",
"2",       //50
"3",
"4",
"5",
"6",
"7",
"8",
"9",
":",
";",
"<",       //60
"=",
">",
"?",
"@",
"A",
"B",
"C",
"D",
"E",
"F",       //70
"G",
"H",
"I",
"J",
"K",
"L",
"M",
"N",
"O",
"P",       //80
"Q",
"R",
"S",
"T",
"U",
"V",
"W",
"X",
"Y",
"Z",       //90
"[",
"\\@5C",
"]",
"\\@5E",
"\\@5F",
"\\@60",
"a",
"b",
"c",
"d",       //100
"e",
"f",
"g",
"h",
"i",
"j",
"k",
"l",
"m",
"n",       //110
"o",
"p",
"q",
"r",
"s",
"t",
"u",
"v",
"w",
"x",       //120
"y",
"z",
"{",
"\\@7C",
"}",
"~",
"\\@7F__",         //127=H'7F'=Shift code n1
"\\Pol(",
"\\sin ",
"\\cos ",   //130
"\\tan ",
"\\Hex>",
"\\In ",
"\\sqrt",
"\\(-)",
"\\nPr",
"+",
"\\xnor",
"\\x^2",
"\\Dms",    //140
"\\Integral(",
"\\Mo",
"\\Sumx2",
"\\@90",
"\\asin ",
"\\acos ",
"\\atan ",
"\\Dec>",
"\\log ",
"\\curt",  //150
"\\Abs ",
"\\nCr",
"-",
"\\xor",
"\\x^-1",
"\\Deg>",
"\\@9D",
"\\Med",
"\\Sumx",
"\\Rec(",   //160
"\\sinh ",
"\\cosh ",
"\\tanh ",
"\\Oct>",
"\\e^x",
"\\Int ",
"\\Not ",
"^",
"*",
"\\or",     //170
"!",
"\\Rad>",
"\\minY",
"\\minX",
"\\Cnt",
"\\@B0",
"\\asinh ",
"\\acosh ",
"\\atanh ",
"\\Bin>",   //180
"\\10^x",
"\\Frac ",
"\\Neg ",
"\\xrt",
"/",
"\\and",
"\\ab/c",
"\\Gra>",
"\\maxY",
"\\maxX",   //190
"\\Sumy2",
"\\Ans",
"\\Ran#",
"\\Meanx",
"\\Meany",
"\\Sdx",
"\\Sdxn",
"\\Sdy",
"\\Sdyn",
"\\Ra",     //200
"\\Rb",
"\\Cor",
"\\Eox",
"\\Eoy",
"\\r",
"\\theta",
"\\Sumy",
"\\Pi",
"\\Cls",
"\\@D2",    //210
"\\Rnd",
"\\Dec",
"\\Hex",
"\\Bin",
"\\Oct",
"\\@D8",
"\\Norm",
"\\Deg",
"\\Rad",
"\\Gra",    //220
"\\Eng",
"\\Intg ",
"\\Sumxy",
"\\Plot ",
"\\Line",
"\\Lbl ",
"\\Fix ",
"\\Sci ",
"\\@E5",
"\\@E6",    //230
"\\@E7",
"\\Dsz ",
"\\Isz ",
"\\Factor ",
"\\ViewWindow ",
"\\Goto",
"\\Prog ",
"\\Graph Y=",
"\\Graph Integral ",
"\\Graph Y>", //240
"\\Graph Y<",
"\\Graph Y>=",
"\\Graph Y<=",
"\\Graph r=",
"\\Graph (X,Y)=(",
"\\,",
"\\@F7__",        //247=H'F7'=Shift code n2
"\\@F8",
"\\@F9",
"\\@FA",    //250
"\\ProbP(",
"\\ProbQ(",
"\\ProbR(",
"\\ProbT(",
"\\@FF",     //fin de fichier???

//a partir d'ici, les caracteres sont preceds du caractere H'7F'
"\\Xmin",
"\\Xmax",
"\\Xscl",
"\\@7F03",
"\\Ymin",         //260
"\\Ymax",
"\\Yscl",
"\\@7F07",
"\\Tmin",
"\\Tmax",
"\\Tptch",
"\\Xfct",
"\\Yfct",
"\\D Start",
"\\D End",       //270
"\\D pitch",
"\\RightXmin",
"\\RightXmax",
"\\RightXscl",
"\\@7F13",
"\\RightYmin",
"\\RightYmax",
"\\RightYscl",
"\\@7F17",
"\\RightTmin",   //280
"\\RightTmax",
"\\RightTptch",
"\\@7F1F",
"\\@7F1C",
"\\Rc",
"\\Rd",
"\\Re",
"\\Max(",
"\\Det ",
"\\Arg ",        //290
"\\Conjg ",
"\\ReP ",
"\\ImP ",
"\\d/dx(",
"\\d2/dx2(",
"\\Solve(",
"\\Sum(",
"\\FMin(",
"\\FMax(",
"\\Seq(",        //300
"\\Min(",
"\\Mean",
"\\Median",
"\\@7F30",
"\\P/L-Orange",
"\\P/L-Blue",
"\\P/L-Green",
"\\Orange ",
"\\Blue ",
"\\Green ",      //310
"\\OrangeG ",
"\\BlueG ",
"\\GreenG ",
"\\@7F3A",
"\\@7F3B",
"\\@7F3C",
"\\@7F3D",
"\\@7F3E",
"\\@7F3F",
"\\Mat ",        //320
"\\Trn ",
"\\*Row(",
"\\*Row+(",
"\\Row+(",
"\\Swap(",
"\\Dim ",
"\\Fill(",
"\\Identity ",
"\\Augment(",
"\\List->Mat(",  //330
"\\Mat->List",
"\\Sum ",
"\\Prod ",
"\\Percent ",
"\\Cuml ",
"\\{i}",
"\\List ",
"\\@7F52",
"\\@7F53",
"\\@7F54",       //340
"\\@7F55",
"\\@7F56",
"\\@7F57",
"\\@7F58",
"\\@7F59",
"\\@7F5A",
"\\@7F5B",
"\\@7F5C",
"\\@7F5D",
"\\@7F5E",       //350
"\\@7F5F",
"\\Sim Coef",
"\\Ply Coef",
"\\Sim Result",
"\\Ply Result",
"\\@7F64",
"\\@7F65",
"\\@7F66",
"\\@7F67",
"\\@7F68",       //360
"\\@7F69",
"\\List1",
"\\List2",
"\\List3",
"\\List4",
"\\List5",
"\\List6",
"\\@7F70",
"\\@7F71",
"\\@7F72",       //370
"\\@7F73",
"\\@7F74",
"\\@7F75",
"\\Q1",
"\\Q3",
"\\x1",
"\\y1",
"\\x2",
"\\y2",
"\\x3",          //380
"\\y3",
"\\@7F7E",
"\\@7F7F",               //Shift Shift code n1
"\\@7F80",
"\\@7F81",
"\\@7F82",
"\\@7F83",
"\\@7F84",
"\\@7F85",
"\\@7F86",       //390
"\\@7F87",
"\\@7F88",
"\\@7F89",
"\\@7F8A",
"\\@7F8B",
"\\@7F8C",
"\\@7F8D",
"\\@7F8E",
"\\Getkey",
"\\F Result",    //400
"\\F Start",
"\\F End",
"\\F pitch",
"\\R Result",
"\\R Start",
"\\R End",
"\\@7F97",
"\\@7F98",
"\\@7F99",
"\\@7F9A",       //410
"\\@7F9B",
"\\@7F9C",
"\\@7F9D",
"\\@7F9E",
"\\@7F9F",
"\\R an",
"\\R an1",
"\\R an2",
"\\R n",
"\\R a0",        //420
"\\R a1",
"\\R a2",
"\\R bn",
"\\R bn1",
"\\R bn2",
"\\R b0",
"\\R b1",
"\\R b2",
"\\anStart",
"\\bnStart",     //430
"\\@7FAF",
"\\ And ",
"\\ Or ",
"\\@7FB2",
"\\ Not ",
"\\@7FB4",
"\\@7FB5",
"\\@7FB6",
"\\@7FB7",
"\\Simp ",       //440
"\\@7FB9",
"\\@7FBA",
"\\@7FBB",
"\\Int/ ",
"\\Rmdr ",
"\\@7FBE",
"\\@7FBF",
"\\@7FC0",
"\\@7FC1",
"\\@7FC2",       //450
"\\@7FC3",
"\\@7FC4",
"\\@7FC5",
"\\@7FC6",
"\\@7FC7",
"\\@7FC8",
"\\@7FC9",
"\\@7FCA",
"\\@7FCB",
"\\@7FCC",       //460
"\\@7FCD",
"\\@7FCE",
"\\@7FCF",
"\\@7FD0",
"\\@7FD1",
"\\@7FD2",
"\\@7FD3",
"\\@7FD4",
"\\@7FD5",
"\\@7FD6",       //470
"\\@7FD7",
"\\@7FD8",
"\\@7FD9",
"\\@7FDA",
"\\@7FDB",
"\\@7FDC",
"\\@7FDD",
"\\@7FDE",
"\\@7FDF",
"\\@7FE0",       //480
"\\@7FE1",
"\\@7FE2",
"\\@7FE3",
"\\@7FE4",
"\\@7FE5",
"\\@7FE6",
"\\@7FE7",
"\\@7FE8",
"\\@7FE9",
"\\@7FEA",       //490
"\\@7FEB",
"\\@7FEC",
"\\@7FED",
"\\@7FEE",
"\\@7FEF",
"\\Y ",
"\\{r} ",
"\\Xt ",
"\\Yt ",
"\\X ",          //500
"\\@7FF5",
"\\@7FF6",
"\\@7FF7",               //Shift n1 Shift n2
"\\@7FF8",
"\\@7FF9",
"\\@7FFA",
"\\@7FFB",
"\\@7FFC",
"\\@7FFD",
"\\@7FFE",       //510
"\\@7FFF",

//a partir d'ici, les caracteres sont preceds du caractere H'F7'
"\\If ",
"\\Then ",
"\\Else ",
"\\IfEnd",
"\\For ",
" \\To ",
" \\Step ",
"\\Next ",
"\\While ",      //520
"\\WhileEnd",
"\\Do",
"\\LpWhile ",
"\\Return",
"\\Break",
"\\Stop",
"\\@F70F",
"\\Locate ",
"\\Send(",
"\\Receive(",    //530
"\\@F713",
"\\@F714",
"\\@F715",
"\\@F716",
"\\@F717",
"\\ClrText",
"\\ClrGraph",
"\\ClrList",
"\\@F71B",
"\\@F71C",       //540
"\\@F71D",
"\\@F71E",
"\\@F71F",
"\\DrawGraph",
"\\@F721",
"\\DrawDyna",
"\\DrawStat",
"\\DrawFTG-Con",
"\\DrawFTG-Plt",
"\\DrawR-Con",   //550
"\\DrawR-Plt",
"\\DrawR-SumCon",
"\\DrawR-SumPlt",
"\\DrawWeb ",
"\\@F72B",
"\\@F72C",
"\\@F72D",
"\\DispF-Tbl",
"\\DispR-Tbl",
"\\AutoSimp",    //560
"\\ManSimp",
"\\@F732",
"\\@F733",
"\\@F734",
"\\@F735",
"\\@F736",
"\\@F737",
"\\MixedFrac",
"\\ImproperFrac",
"\\@F73A",       //570
"\\@F73B",
"\\@F73C",
"\\@F73D",
"\\@F73E",
"\\@F73F",
"\\1-Variable ",
"\\2-Variable ",
"\\LinearReg ",
"\\Med-MedLine ",
"\\QuadReg ",    //580
"\\CubicReg ",
"\\QuartReg ",
"\\LogReg ",
"\\ExpReg ",
"\\PowerReg ",
"\\S-Gph1 ",
"\\S-Gph2 ",
"\\S-Gph3 ",
"\\Square",
"\\Cross",       //590
"\\Dot",
"\\Scatter",
"\\xyLine",
"\\Hist",
"\\MedBox",
"\\MeanBox",
"\\N-Dist",
"\\Broken",
"\\Linear",
"\\Med-Med",     //600
"\\Quad",
"\\Cubic",
"\\Quart",
"\\Log",
"\\Exp",
"\\Power",
"\\@F75F",
"\\S-WindAuto",
"\\S-WindMan",
"\\Graph X=",    //610
"\\Y=Type",
"\\r=Type",
"\\ParamType",
"\\@F766",
"\\X-cType",
"\\@F768",
"\\@F769",
"\\Y>Type",
"\\Y<Type",
"\\Y>=Type",     //620
"\\Y<=Type",
"\\@F76E",
"\\@F76F",
"\\G-Connect",
"\\G-Plot",
"\\DualGraph",
"\\DualGtoT",
"\\DualT+G",
"\\DualOff",
"\\@F776",       //630
"\\@F777",
"\\BG-None",
"\\BG-Pict ",
"\\GridOff",
"\\GridDot",
"\\GridLine",
"\\GridOn",
"\\@F77E",
"\\@F77F",               //Shift n2 Shift n1
"\\D Var ",      //640
"\\NrmSpeed",
"\\FastSpeed",
"\\SlowSpeed",
"\\Stop&Go",
"\\DynaCon",
"\\DynaStop",
"\\@F787",
"\\VarRange",
"\\@F789",
"\\Simul ",      //650
"\\Poly ",
"\\@F78C",
"\\@F78D",
"\\@F78E",
"\\@F78F",
"\\aRanType",
"\\aRan1Type",
"\\aRan2Type",
"\\StoPict ",
"\\RclPict ",    //660
"\\StoGMEM ",
"\\RclGMEM ",
"\\StoV-Win ",
"\\RclV-Win ",
"\\@F799",
"\\@F79A",
"\\@F79B",
"\\@F79C",
"\\@F79D",
"\\@F79E",       //670
"\\@F79F",
"\\Tangent ",
"\\Normal ",
"\\Inverse ",
"\\Vertical ",
"\\Horizontal ",
"\\Text ",
"\\Circle ",
"\\F-Line ",
"\\PlotOn ",     //680
"\\PlotOff ",
"\\PlotChg ",
"\\PxlOn ",
"\\PxlOff ",
"\\PxlChg ",
"\\PxlLine ",
"\\PxlTest ",
"\\SortA(",
"\\SortD(",
"\\VarList1",    //690
"\\VarList2",
"\\VarList3",
"\\VarList4",
"\\VarList5",
"\\VarList6",
"\\File1",
"\\File2",
"\\File3",
"\\File4",
"\\File5",       //700
"\\File6",
"\\@F7BE",
"\\@F7BF",
"\\FuncOn",
"\\SimulOn",
"\\AxesOn",
"\\CoordOn",
"\\LabelOn",
"\\DerivOn",
"\\LocusOn",     //710
"\\SumDispOn",
"\\G SelOn ",
"\\T SelOn ",
"\\D SelOn ",
"\\R SelOn ",
"\\DrawOn",
"\\@F7CD",
"\\@F7CE",
"\\@F7CF",
"\\FuncOff",     //720
"\\SimulOff",
"\\AxesOff",
"\\CoordOff",
"\\LabelOff",
"\\DerivOff",
"\\LocusOff",
"\\SumDispOff",
"\\G SelOff ",
"\\T SelOff ",
"\\D SelOff ",   //730
"\\R SelOff ",
"\\DrawOff",
"\\@F7DD",
"\\@F7DE",
"\\@F7DF",
"\\@F7E0",
"\\@F7E1",
"\\@F7E2",
"\\@F7E3",
"\\@F7E4",       //740
"\\@F7E5",
"\\@F7E6",
"\\@F7E7",
"\\@F7E8",
"\\@F7E9",
"\\@F7EA",
"\\@F7EB",
"\\@F7EC",
"\\@F7ED",
"\\@F7EE",       //750
"\\@F7EF",
"\\@F7F0",
"\\@F7F1",
"\\@F7F2",
"\\@F7F3",
"\\@F7F4",
"\\@F7F5",
"\\@F7F6",
"\\@F7F7",               //Shift n2 Shift n2
"\\@F7F8",       //760
"\\@F7F9",
"\\@F7FA",
"\\@F7FB",
"\\@F7FC",
"\\@F7FD",
"\\@F7FE",
"\\@F7FF"        //767
};
