// FileLoader.cpp: implementation of the CFileLoader class.
//
//////////////////////////////////////////////////////////////////////

#include "FileLoader.h"
#include "BasicZone.h"
#include "ListZone.h"
#include "ListEditor.h"
#include "MatrixEditor.h"
#include "MatrixZone.h"
#include "ImageZone.h"
#include "ImageEditor.h"
#include "BasicEditor.h"

//-----------------------------------------------------------------------------------
CFileZone* CFileLoader::LoadFileZone(QTextStream& ts, bool& bOk)
{
	QString sName;
	unsigned short nZone;
	unsigned int nCapacity;
	if(ts.atEnd()) {
		bOk = true;
		return NULL;
	}
	bOk = CFileZone::ReadHeader(ts, nZone, sName, nCapacity);
	if(bOk) {
		CFileZone* pFile = LoadFileZone(nZone);
		if(pFile != NULL) {
			pFile->SetName(sName);
			if(pFile->ReadDataFrom(ts, nCapacity) && CFileZone::ReadFooter(ts)) {
				return pFile;
			}
			bOk = false;
			delete pFile;
		}
		else {
			bOk = CFileZone::SeekToFooter(ts);
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CFileZone* CFileLoader::LoadFileZone(unsigned int nZone)
{
	switch(nZone)
	{
	case 1:
		return new CBasicZone();
	case 5:
		return new CListZone();
	case 6:
		return new CMatrixZone();
	case 7:
		return new CImageZone();
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CEditor* CFileLoader::CreateEditor(CFileZone* pFile, QWidget* parent)
{
	CEditor* pEditor = NULL;
	if(pFile != NULL) {
		switch(pFile->GetZone())
		{
		case 1:
			pEditor = new CBasicEditor(parent);
			break;
		case 5:
			pEditor = new CListEditor(parent);
			break;
		case 6:
			pEditor = new CMatrixEditor(parent);
			break;
		case 7:
			pEditor = new CImageEditor(parent);
			break;
		}
	}
	if(pEditor != NULL) {
		if(!pEditor->SetFileZone(pFile)) {
			delete pEditor;
			pEditor = NULL;
		}
	}
	return pEditor;
}

//-----------------------------------------------------------------------------------
QString CFileLoader::ChooseFileName(unsigned short nZone, const QString& sCaption, const QString& sDefaultText, QWidget* parent)
{
	switch(nZone)
	{
	case 1:
		return CBasicZone::ChooseFileName(sCaption, sDefaultText, parent);
	case 5:
		return CListZone::ChooseFileName(sCaption, sDefaultText, parent);
	case 6:
		return CMatrixZone::ChooseFileName(sCaption, sDefaultText, parent);
	case 7:
		return CImageZone::ChooseFileName(sCaption, sDefaultText, parent);
	}
	return QString::null;
}
