// FileZone.cpp: implementation of the CFileZone class.
//
//////////////////////////////////////////////////////////////////////

#include "FileZone.h"

#include <qinputdialog.h>

CasioType m_ct;

//-----------------------------------------------------------------------------------
CFileZone::CFileZone(unsigned short nZone)
	: m_nZone(nZone)
	, m_nSize(0)
	, m_pData(NULL)
{
}

//-----------------------------------------------------------------------------------
CFileZone::~CFileZone()
{
	SetData(NULL, 0, FALSE);
}

//-----------------------------------------------------------------------------------
unsigned short CFileZone::GetZone() const
{
	return m_nZone;
}

//-----------------------------------------------------------------------------------
const QString& CFileZone::GetPassword() const
{
	return m_sPassword;
}

//-----------------------------------------------------------------------------------
const QString& CFileZone::GetName() const
{
	return m_sName;
}

//-----------------------------------------------------------------------------------
unsigned int CFileZone::GetSize()
{
	FlushData();
	return m_nSize;	
}

//-----------------------------------------------------------------------------------
unsigned int CFileZone::GetCapacity()
{
	if(GetSize() == 0) {
		return 0;
	}
	return GetSize() + 1;
}

//-----------------------------------------------------------------------------------
void* CFileZone::GetData()
{
	FlushData();
	return m_pData;	
}

//-----------------------------------------------------------------------------------
void CFileZone::SetName(const QString& sName)
{
	if(sName != m_sName) {
		m_sName = sName;
		m_sName.truncate(8);
		emit Renamed();
	}
}

//-----------------------------------------------------------------------------------
void CFileZone::SetPassword(const QString& sPassword)
{
	if(m_nZone == 1) {
		m_sPassword = sPassword;
		m_sPassword.truncate(8);
	}
}

//-----------------------------------------------------------------------------------
void CFileZone::SetData(void* pData, unsigned int nSize, bool bEmitDataChanged)
{
	if(m_pData != NULL && pData != m_pData) {
		delete[] m_pData;
	}
	m_pData = pData;
	m_nSize = nSize;
	if(bEmitDataChanged) {
		emit DataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CFileZone::SetCasioType(CasioType ct)
{
	m_ct = ct;
}

//-----------------------------------------------------------------------------------
CasioType CFileZone::GetCasioType()
{
	return m_ct;
}

//-----------------------------------------------------------------------------------
void CFileZone::Save(QTextStream& stream)
{
	WriteHeader(stream);
	WriteData(stream);
	WriteFooter(stream);
}

//-----------------------------------------------------------------------------------
void CFileZone::WriteHeader(QTextStream& stream)
{
	stream << "%Header Record\n";
	stream << "Format:";
	stream << "MCS1\n";
	stream << "Type Number:" << m_nZone << '\n';
	stream << "File Name:" << GetName() << '\n';
	stream << "Option Name:\nCommunication SW:0\n";
	stream << "Capacity:" << GetCapacity() << '\n';
	stream << "Data Type:" << GetDataType() << '\n';
	stream << "%Data Record\n";
}

//-----------------------------------------------------------------------------------
void CFileZone::WriteFooter(QTextStream& stream)
{
	stream << "%End\n";
}

//-----------------------------------------------------------------------------------
bool CFileZone::ReadHeader(QTextStream& stream, unsigned short& nZone, QString& sName, unsigned int& nCapacity)
{
	if(stream.readLine() == "%Header Record") {
		QString sFormat   = stream.readLine().section(':', 1, 1);
		if(sFormat != "MCS1") {
			return false;
		}
		nZone = stream.readLine().section(':', 1, 1).toInt();
		sName = stream.readLine().section(':', 1, 1);
		stream.readLine();stream.readLine();//Zap Option Name et Communication SW
		nCapacity = stream.readLine().section(':', 1, 1).toInt();
		stream.readLine();//Zap Data Type
		return stream.readLine() == "%Data Record";
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CFileZone::ReadFooter(QTextStream& stream)
{
	return stream.readLine() == "%End";
}

//-----------------------------------------------------------------------------------
bool CFileZone::SeekToFooter(QTextStream& stream)
{
	while(!stream.atEnd()) {
		if(ReadFooter(stream)) {
			return true;
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CFileZone::IsRenameable() const
{
	return TRUE;
}

//-----------------------------------------------------------------------------------
QString CFileZone::ChooseFileName(const QString& sCaption, const QString& sDefaultText, QWidget* parent)
{
	QString s = QInputDialog::getText(sCaption, tr("Entrez le nom du fichier"), QLineEdit::Normal, sDefaultText, 0, parent);
	s.truncate(8);
	return s;
}
