// FileZone.h: interface for the CFileZone class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FILEZONE_H_
#define _FILEZONE_H_

#include <qstring.h>
#include <qtextstream.h>
#include <qobject.h>

#include <casio.h>

using namespace Casio;

class CFileZone : public QObject
{
	Q_OBJECT
protected:
	CFileZone(unsigned short nZone);

public:
	virtual ~CFileZone();

	unsigned short    GetZone() const;
	const QString&    GetPassword() const;
	const QString&    GetName() const;
	unsigned int      GetSize();
	void*             GetData();

	virtual void      SetName(const QString& sName);
	virtual void      SetPassword(const QString& sPassword);
	virtual void      SetData(void* pData, unsigned int nSize, bool bEmitDataChanged);

	void              Save(QTextStream& stream);
	virtual bool      ReadDataFrom(QTextStream& stream, unsigned int nCapacity) = 0;
	virtual bool      IsRenameable() const;

public:
	static  QString   ChooseFileName(const QString& sCaption, const QString& sDefaultText = QString::null, QWidget* parent = 0);

protected:
	virtual void         WriteHeader(QTextStream& stream);
	virtual void         WriteFooter(QTextStream& stream);
	virtual void         WriteData(QTextStream& stream) = 0;
	virtual QString      GetDataType() const = 0;
	
	/** @return Retourne la capacit affiche dans le fichier CAT. Par dfaut
	 * retourne la GetSize() + 1 ou 0.
	 */
	virtual unsigned int GetCapacity();
	virtual void         FlushData() {};

public:
	static void          SetCasioType(CasioType ct);
	static CasioType     GetCasioType();
	static bool          ReadHeader(QTextStream& stream, unsigned short& nZone, QString& sName, unsigned int& nCapacity);
	static bool          ReadFooter(QTextStream& stream);
	static bool          SeekToFooter(QTextStream& stream);

signals:
	void DataChanged();
	void Renamed();

private:
	unsigned short    m_nZone;
	unsigned int      m_nSize;
	QString           m_sName;
	QString           m_sPassword;
	void*             m_pData;
};

#endif
