// FindDialog.cpp: implementation of the CFindDialog class.
//
//////////////////////////////////////////////////////////////////////

#include "FindDialog.h"

#include <qpushbutton.h>
#include <qlayout.h>
#include <qtextedit.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qhbuttongroup.h>
#include <qradiobutton.h>

#include <language.h>
#include <MessageBox.h>

#ifdef WIN32
#	define BORDER WStyle_NoBorder
#else
#	define BORDER WStyle_NormalBorder
#endif

//-----------------------------------------------------------------------------------
CFindDialog::CFindDialog(QTextEdit* pTextEdit)
	: QDialog(pTextEdit, 0, FALSE, WStyle_Customize | BORDER | WStyle_Title | WStyle_SysMenu)
	, m_pTextEdit(pTextEdit)
{
	QGridLayout* pLayout = new QGridLayout(this, 4, 4, 5);

	m_pLineEdit  = new QComboBox(this);
	m_pLabel     = new QLabel(this);
	m_pWordOnly  = new QCheckBox(this);
	m_pCase      = new QCheckBox(this);
	m_pNext      = new QPushButton(this);
	m_pCancel    = new QPushButton(this);
	m_pDirection = new QHButtonGroup(this);
	m_pUp        = new QRadioButton(m_pDirection);
	m_pDown      = new QRadioButton(m_pDirection);

	pLayout->addWidget(m_pLabel,  0, 0);
	pLayout->addWidget(m_pNext,   0, 3);
	pLayout->addMultiCellWidget(m_pLineEdit,  0, 0, 1, 2);
	pLayout->addMultiCellWidget(m_pWordOnly,  1, 1, 0, 1);
	pLayout->addMultiCellWidget(m_pCase,      2, 2, 0, 1);
	pLayout->addMultiCellWidget(m_pDirection, 1, 3, 2, 2);
	pLayout->addMultiCellWidget(m_pCancel,    1, 2, 3, 3, AlignTop);

	pLayout->setColStretch(2, 1);
	pLayout->setResizeMode(QLayout::Fixed );

	m_pNext->setDefault(TRUE);

	m_pDown->setChecked(TRUE);

	m_pLineEdit->setEditable(TRUE);
	m_pLineEdit->setInsertionPolicy(QComboBox::NoInsertion);
	m_pLineEdit->setFocus();

	connect(m_pNext,   SIGNAL(clicked()), SLOT(OnFind()));
	connect(m_pCancel, SIGNAL(clicked()), SLOT(reject()));

	languageChange();
}

//-----------------------------------------------------------------------------------
void CFindDialog::languageChange()
{
	m_pLabel->setText(tr("Rechercher :"));
	m_pWordOnly->setText(tr("Mot entier seulement"));
	m_pCase->setText(tr("Respecter la casse"));
	m_pNext->setText(tr("Suivant"));
	m_pCancel->setText(CLanguage::GetWordTranslation(CLanguage::WO_CANCEL));
	m_pDirection->setTitle(tr("Direction"));

	m_pUp->setText(tr("Haut"));
	m_pDown->setText(tr("Bas"));

	setCaption(tr("Rechercher"));
}

//-----------------------------------------------------------------------------------
void CFindDialog::OnFind()
{
	QString s = m_pLineEdit->currentText();
	if(!s.isEmpty()) {
		if(!m_pTextEdit->find(s, m_pCase->isChecked(), m_pWordOnly->isChecked(), m_pDown->isChecked())) {
			CMessageBox::MsgInformation(this, tr("Recherche du document termine."));
		}
		else {
			int i = 0;
			for(; i < m_pLineEdit->count() && m_pLineEdit->text(i) != s; i++);
			if(i == m_pLineEdit->count()) {
				m_pLineEdit->insertItem(s, 0);
			}
		}
	}
}
