// ImageEditor.cpp: implementation of the CImageEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "ImageEditor.h"
#include "ImageZone.h"

#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include <language.h>
#include <MessageBox.h>

//-----------------------------------------------------------------------------------
CImageEditor::CImageEditor(QWidget* parent)
	: CEditor(parent)
	, m_pImageZone(NULL)
{
	QWidget* pCentralWidget = new QWidget(this);
	QGridLayout* pLayout    = new QGridLayout(pCentralWidget, 2, 2, 5);

	m_pImage        = new QLabel(pCentralWidget);
	m_pImportButton = new QPushButton(pCentralWidget);
	m_pExportButton = new QPushButton(pCentralWidget);

	m_pImage->setScaledContents(TRUE);

	pLayout->addMultiCellWidget(m_pImage, 0, 0, 0, 1);
	pLayout->addWidget(m_pImportButton, 1, 0);
	pLayout->addWidget(m_pExportButton, 1, 1);

	setCentralWidget(pCentralWidget);

	RefreshData();
	DoInitWords();

	connect(m_pImportButton, SIGNAL(clicked()), SLOT(OnImportImage()));
	connect(m_pExportButton, SIGNAL(clicked()), SLOT(OnExportImage()));
}

//-----------------------------------------------------------------------------------
bool CImageEditor::SetFileZone(CFileZone* pFile)
{
	CImageZone* pImageZone = dynamic_cast<CImageZone*>(pFile);
	if(pImageZone != NULL) {
		if(m_pImageZone != NULL) {
			disconnect(m_pImageZone);
		}
		m_pImageZone = pImageZone;
		connect(m_pImageZone, SIGNAL(DataChanged()), SLOT(OnDataChanged()));
		connect(m_pImageZone, SIGNAL(Renamed()),     SLOT(OnFileRenamed()));
		RefreshData();
		UpdateCaption();
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
void CImageEditor::RefreshData()
{
	if(m_pImageZone != NULL) {
		m_pImage->setPixmap(m_pImageZone->GetImage());
	}
	else {
		m_pImage->setPixmap(QPixmap());
	}
}

//-----------------------------------------------------------------------------------
void CImageEditor::DoInitWords()
{
	CEditor::DoInitWords();
	m_pImportButton->setText(CLanguage::GetWordTranslation(CLanguage::WO_IMPORT));
	m_pExportButton->setText(CLanguage::GetWordTranslation(CLanguage::WO_EXPORT));
}

//-----------------------------------------------------------------------------------
void CImageEditor::UpdateCaption()
{
	if(m_pImageZone != NULL) {
		setCaption(tr("Editeur d'image") + " - " + m_pImageZone->GetName());
	}
}

//-----------------------------------------------------------------------------------
void CImageEditor::OnDataChanged()
{
	RefreshData();
}

//-----------------------------------------------------------------------------------
void CImageEditor::OnFileRenamed()
{
	UpdateCaption();
}

//-----------------------------------------------------------------------------------
void CImageEditor::OnImportImage()
{
	QString sFilter = CLanguage::GetWordTranslation(CLanguage::WO_INPUTIMAGEFILTER);
	QString s = CMessageBox::ChooseOpenFileName(QString::null, sFilter, this);
	if(!s.isEmpty()) {
		QImage image;
		if(image.load(s)) {
			m_pImageZone->SetImage(image);
		}
		else {
			CMessageBox::MsgCritical(this, tr("Le chargement de l'image '%1' a chou.").arg(s));
		}
	}
}

//-----------------------------------------------------------------------------------
void CImageEditor::OnExportImage()
{
	QString sFilter = CLanguage::GetWordTranslation(CLanguage::WO_OUTPUTIMAGEFILTER);
	QString s = CMessageBox::ChooseSaveFileName(QString::null, sFilter, this, "bmp");
	if(!s.isEmpty()) {
		QString sExt = s.section('.', -1);
		if(sExt == s) {
			sExt = "BMP";
		}
		if(!m_pImageZone->GetImage().save(s, sExt.upper())) {
			CMessageBox::MsgCritical(this, tr("La sauvegarde du fichier '%1' a chou.").arg(s));
		}
	}
}
