// ImageZone.cpp: implementation of the CImageZone class.
//
//////////////////////////////////////////////////////////////////////

#include "ImageZone.h"

#include <qinputdialog.h>

#include <CasioSystem.h>
#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CImageZone::CImageZone()
	: CFileZone(7)
	, m_image(128, 64, 1, 2, QImage::BigEndian)
	, m_bModified(TRUE)
{
	m_image.setColor(0, qRgb(255,255,255));
	m_image.setColor(1, 0);
	m_image.fill(0);
}

//-----------------------------------------------------------------------------------
bool CImageZone::ReadDataFrom(QTextStream& stream, unsigned int nCapacity)
{
	if(nCapacity == 2048) {
		unsigned char* pData = new unsigned char[2047];
		if(pData != NULL) {
			memset(pData, 0, 2047);
			int i = 0;
			for(int l = 0; l < 64; l++) {
				QString sLine = stream.readLine();
				for(int c = 0; c < 16; c++) {
					pData[i++] = sLine.mid(c*2, 2).toInt(NULL, 16);
				}
			}
			for(i = 0; i < 65; i++) {
				stream.readLine();
			}
			SetData(pData, 2047, TRUE);
			return true;
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
void CImageZone::WriteData(QTextStream& stream)
{
	QString sLine;
	unsigned char* pData = (unsigned char*)GetData();
	assert(pData != NULL);
	int nCpt;
	for(int i = 0; i < 2; i++) {
		nCpt = 0;
		for(int r = 0; r < 64; r++) {
			sLine = QString::null;
			for(int c = 0; c < 16; c++) {
				QString str = QString::number(pData[nCpt++], 16).upper();
				if(str.length() < 2) {
					str = '0' + str;
				}
				sLine += str;
			}
			stream << sLine << '\n';
		}
		if(i == 0) {
			stream << '\n';
		}
	}
}

//-----------------------------------------------------------------------------------
QString CImageZone::GetDataType() const
{
	return "PC";
}

//-----------------------------------------------------------------------------------
QString CImageZone::ChooseFileName(const QString& sCaption, const QString& sDefaultText, QWidget* parent)
{
	bool bOk;
	QStringList lList;
	for(int i = 1; i <= 20; i++) {
		lList.push_back("PICT" + QString::number(i));
	}
	int nCurrent = lList.findIndex(sDefaultText);
	QString s = QInputDialog::getItem(sCaption, CFileZone::tr("Choisissez une image possible"), lList, nCurrent, FALSE, &bOk, parent);
	if(bOk) {
		return s;
	}
	return QString::null;
}

//-----------------------------------------------------------------------------------
const QImage& CImageZone::GetImage() const
{
	return m_image;
}

//-----------------------------------------------------------------------------------
void CImageZone::SetImage(const QImage& image)
{
	QImage i = image.smoothScale(128, 64);
	i = i.convertDepth(1);
	i = i.convertBitOrder(QImage::BigEndian);
	i.setColor(0, qRgb(255, 255, 255));
	i.setColor(1, 0);
	m_image = i;
	OnDataChanged();
}

//-----------------------------------------------------------------------------------
void CImageZone::FlushData()
{
	if(m_bModified) {
		m_bModified = FALSE;
		unsigned char* pData = (unsigned char*)GetData();
		if(pData == NULL) {
			pData = new unsigned char[2047];
			memset(pData, 0, 2047);
			CFileZone::SetData(pData, 2047, FALSE);
		}
		if(pData != NULL) {
			CCasioSystem::InvertBuffer(m_image.bits(), pData, FALSE);
		}
	}
}

//-----------------------------------------------------------------------------------
void CImageZone::OnDataChanged()
{
	m_bModified = TRUE;
	emit DataChanged();
}

//-----------------------------------------------------------------------------------
void CImageZone::SetData(void* pData, unsigned int nSize, bool bEmitDataChanged)
{
	if(pData == NULL || nSize == 0) {
		m_image.fill(0);
		CFileZone::SetData(NULL, 0, bEmitDataChanged);
		return;
	}
	assert(nSize == 2047);
	CCasioSystem::InvertBuffer(m_image.bits(), pData, TRUE);
	CFileZone::SetData(pData, nSize, bEmitDataChanged);
}
