// ListZone.h: interface for the CListZone class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _LISTZONE_H_
#define _LISTZONE_H_

#include "FileZone.h"

#include <qvaluelist.h>

typedef QValueList<double> List;

class CListZone : public CFileZone  
{
public:
	CListZone();
	bool      ReadDataFrom(QTextStream& stream, unsigned int nCapacity);
	void      SetName(const QString& sName);
	void      SetData(void* pData, unsigned int nSize, bool bEmitDataChanged);

	int       GetNbElements() const;
	double    GetElement(int nIndex) const;

	void      AddElement(double dValue, int nIndex = -1);
	void      RemoveElement(int nIndex);
	void      SetElement(int nIndex, double dValue);
	bool      IsRenameable() const;
	QString   GetLabel(unsigned int nIndex);

public:
	static QString ChooseFileName(const QString& sCaption, const QString& sDefaultText = QString::null, QWidget* parent = 0);

protected:
	void         WriteData(QTextStream& stream);
	QString      GetDataType() const;
	unsigned int CalculateSize(int nElements) const;
	void         FlushData();
	void         OnDataChanged();

protected:
	static QString GetText(int nFlags, int nIndex);
	static void    AddSpecialLists(QStringList& lList);
	static int     FindIndex(int nFlags, const QString& sText);

private:
	enum DataType{
		DT_LIST = 0,
		DT_WD,
		DT_GFA,
	};

private:
	DataType m_nDataType;
	List     m_lElements;
	bool     m_bModified;
};

#endif
