// MLC1Properties.cpp: implementation of the CMLC1Properties class.
//
//////////////////////////////////////////////////////////////////////

#include "MLC1Properties.h"
#include "MLCSyntaxHighlighter.h"
#include "SyntaxProperty.h"

#include <qobject.h>

class CFunctionProperty : public CSyntaxProperty {
public:
	CFunctionProperty() : CSyntaxProperty() { Reset(); }
	CFunctionProperty(CFunctionProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Fonction"); }
	QString GetId()   const  { return KEY_FUNCTION; }
	CSyntaxProperty* Clone() { return new CFunctionProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor(0, 170, 255)); }
};

class CSpriteProperty : public CSyntaxProperty {
public:
	CSpriteProperty() : CSyntaxProperty() { Reset(); }
	CSpriteProperty(CSpriteProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Sprite"); }
	QString GetId()   const  { return KEY_SPRITE; }
	CSyntaxProperty* Clone() { return new CSpriteProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor()); }
};

//-----------------------------------------------------------------------------------
CMLC1Properties::CMLC1Properties()
	: CProgSyntaxProperties()
{
	GetProperties().push_back(new CFunctionProperty());
	GetProperties().push_back(new CSpriteProperty());
}

//-----------------------------------------------------------------------------------
CMLC1Properties::CMLC1Properties(const CMLC1Properties* pSource)
	: CProgSyntaxProperties(pSource)
{
}

//-----------------------------------------------------------------------------------
QString CMLC1Properties::GetName() const
{
	return "MLC 1";
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CMLC1Properties::CreateSyntax(QTextEdit* pEdit)
{
	return new CMLC1SyntaxHighlighter(pEdit, this);
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CMLC1Properties::CreateCompatibleSyntax(const void* pData, unsigned int nSize, QTextEdit* pEdit)
{
	if(nSize > 0 && memcmp(pData, "MLC:", QMIN(nSize, 4)) == 0) {
		return CreateSyntax(pEdit);
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CSyntaxProperties* CMLC1Properties::Clone()
{
	return new CMLC1Properties(this);
}

//-----------------------------------------------------------------------------------
void CMLC1Properties::GetKeywords(QStringList& lKeywords)
{
	CProgSyntaxProperties::GetKeywords(lKeywords);
	unsigned i;
	for(i = 0; i < CMLC1SyntaxHighlighter::m_nNbFunctions; i++) {
		lKeywords.push_back(CMLC1SyntaxHighlighter::m_pFunctions[i]);
	}
	for(i = 0; i < CMLC1SyntaxHighlighter::m_nNbKeywords; i++) {
		lKeywords.push_back(CMLC1SyntaxHighlighter::m_pKeywords[i]);
	}
}
