// MLCSyntaxHighlighter.cpp: implementation of the CMLCSyntaxHighlighter class.
//
//////////////////////////////////////////////////////////////////////

#include "MLCSyntaxHighlighter.h"
#include "MLC1Properties.h"

const char* CMLC1SyntaxHighlighter::m_pFunctions[] = {"#IIFF", "#DATA", "#TCHK", "#BITM", "#ARRY", "#SIZE", "#CLRS"
									, "#TEXT", "#LINE", "#PIXL", "#PIXT", "#SPRT", "#DISP", "#DBMP"
									, "#CLSN", "#RECT", "#DRAW", "#STOP", "#RCLP", "#SHFT", "#GKEY"
									, "#WKEY", "#PAUS", "#FRUN", "#FGOB", "#RNDM", "#DMAP", "#CMAP"
									, "#HALT", "#WAIT", "#WRTE", "#READ", "#POLR", "#SUBS", "#RSET"
									, "#CINT", "#ABSL", "#INPT", "#NMAP", "#TILE"};

const char* CMLC1SyntaxHighlighter::m_pKeywords[] = {"MLC:", "#FNCT", "#FEND", "#DEND"};

unsigned int CMLC1SyntaxHighlighter::m_nNbFunctions  = sizeof(m_pFunctions) / sizeof(m_pFunctions[0]);
unsigned int CMLC1SyntaxHighlighter::m_nNbKeywords   = sizeof(m_pKeywords)  / sizeof(m_pKeywords[0]);

//-----------------------------------------------------------------------------------
CMLCSyntaxHighlighter::CMLCSyntaxHighlighter(QTextEdit* pTextEdit, CProgSyntaxProperties* pProperties)
	: CSyntaxHighlighter(pTextEdit, pProperties)
	, m_strings("\"[^\"]*[\" ]")
	, m_numbers("\\.(\\d|((\\+|\\-)\\d))\\d*")
	, m_vars("(@|%|$)[a-zA-Z_]\\w*")
	, m_sprites("\\^[a-zA-Z_]\\w*")
{
}

//-----------------------------------------------------------------------------------
const CWrapper* CMLCSyntaxHighlighter::GetWrapper() const
{
	return &m_wrapper;
}

//-----------------------------------------------------------------------------------
int CMLCSyntaxHighlighter::highlightParagraph(const QString& text, int endStateOfLastPara)
{
	QFont f = GetFont();
	setFormat(0, text.length(), f, GetEditorForeColor());

	QColor c;
	GetFont(f, c, KEY_KEYWORD);
	if(c.isValid()) {
		Highlight(text, GetKeyWords(), GetNbKeyWords(), f, c);
	}
	GetFont(f, c, KEY_FUNCTION);
	if(c.isValid()) {
		Highlight(text, GetFunctions(), GetNbFunctions(), f, c);
	}

	GetFont(f, c, KEY_NUMBER);	
	SearchExp(&m_numbers, text, f, c);
	
	GetFont(f, c, KEY_STRING);	
	SearchExp(&m_strings, text, f, c);
	
	GetFont(f, c, KEY_VAR);	
	SearchExp(&m_vars, text, f, c);

	GetFont(f, c, KEY_SPRITE);	
	SearchExp(&m_sprites, text, f, c);

	return 0;
}

//-----------------------------------------------------------------------------------
void CMLCSyntaxHighlighter::Highlight(const QString& sText, const char** words, unsigned int nNbWords, const QFont& font, const QColor& color)
{
	for(unsigned int i = 0; i < nNbWords; i++) {
		QString s = words[i];
		if(sText.startsWith(s)) {
			QChar c = sText[(int)s.length()];
			if(c == ' ' || c == '\n') {
				setFormat(0, s.length(), font, color);
			}
		}
	}
}

//-----------------------------------------------------------------------------------
CMLC1SyntaxHighlighter::CMLC1SyntaxHighlighter(QTextEdit* pTextEdit, CProgSyntaxProperties* pProperties)
	: CMLCSyntaxHighlighter(pTextEdit, pProperties)
{

}

//-----------------------------------------------------------------------------------
const char** CMLC1SyntaxHighlighter::GetKeyWords()
{
	return m_pKeywords;
}

//-----------------------------------------------------------------------------------
unsigned int CMLC1SyntaxHighlighter::GetNbKeyWords()
{
	return m_nNbKeywords;
}

//-----------------------------------------------------------------------------------
const char** CMLC1SyntaxHighlighter::GetFunctions()
{
	return m_pFunctions;
}

//-----------------------------------------------------------------------------------
unsigned int CMLC1SyntaxHighlighter::GetNbFunctions()
{
	return m_nNbFunctions;
}
