// MatrixZone.cpp: implementation of the CMatrixZone class.
//
//////////////////////////////////////////////////////////////////////

#include "MatrixZone.h"
#include "CasioIO.h"

#include <math.h>

#include <qinputdialog.h>

#define HEADER_SIZE 15

//-----------------------------------------------------------------------------------
CMatrixZone::CMatrixZone()
	: CFileZone(6)
	, m_bModified(FALSE)
	, m_nRows(0)
	, m_nCols(0)
{

}

//-----------------------------------------------------------------------------------
bool CMatrixZone::ReadDataFrom(QTextStream& stream, unsigned int nCapacity)
{
	if(stream.readLine() != "File Name:") {
		return false;
	}
	QString sRow = stream.readLine();
	QString sCol = stream.readLine();
	if(sRow.section(':', 0, 0) != "Rows" 
		|| sCol.section(':', 0, 0) != "Columns") {
		return false;
	}
	m_nRows = sRow.section(':', 1, 1).toInt();
	m_nCols = sCol.section(':', 1, 1).toInt();
	m_lElements.clear();
	for(int r = 0; r < m_nRows; r++) {
		for(int c = 0; c < m_nCols; c++) {
			QString s = stream.readLine();
			s = s.section(':', 1, 1).section(' ', 3, 3);
			double n = s.section("\\EE", 0, 0).toDouble();
			int e    = s.section("\\EE", 1, 1).toInt();
			m_lElements.push_back(n * pow(10, e));
		}
	}
	m_bModified = true;
	return true;
}

//-----------------------------------------------------------------------------------
void CMatrixZone::WriteData(QTextStream& stream)
{
	stream << "File Name:\nRows:" << m_nRows << "\nColumns:" << m_nCols << '\n';

	for(int r = 0; r < m_nRows; r++) {
		for(int c = 0; c < m_nCols; c++) {
			stream << "Value : " << r << " " << c << " " << CCasioIO::ConvertDoubleToCasioString(GetElement(r, c)) << '\n';
		}
	}
}

//-----------------------------------------------------------------------------------
QString CMatrixZone::GetDataType() const
{
	return "MT";
}

//-----------------------------------------------------------------------------------
void CMatrixZone::FlushData()
{
	if(m_bModified) {
		int nElements = m_lElements.size();
		unsigned int nSize = CalculateSize(m_nRows, m_nCols);
		unsigned char* pData = new unsigned char[nSize];
		char var[VAR_SIZE];
		if(pData != NULL) {
			int pos = HEADER_SIZE;
			memset(pData, 0, HEADER_SIZE);
			pData[ 8] = m_nRows;
			pData[10] = m_nCols;
			List::Iterator it = m_lElements.begin();
			for(int r = 0; r < m_nRows; r++) {
				for(int c = 0; c < m_nCols; c++) {
					CCasioIO::WriteCasioNumberFormat(var, *it);
					memcpy(pData + pos, var, sizeof(var));
					pos += sizeof(var);
					++it;
				}
			}
			m_bModified = FALSE;
			CFileZone::SetData(pData, nSize, FALSE);
		}
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::OnDataChanged()
{
	m_bModified = TRUE;
	emit DataChanged();
}

//-----------------------------------------------------------------------------------
int CMatrixZone::GetNbRows() const
{
	return m_nRows;
}

//-----------------------------------------------------------------------------------
int CMatrixZone::GetNbCols() const
{
	return m_nCols;
}

//-----------------------------------------------------------------------------------
double CMatrixZone::GetElement(int nRow, int nCol) const
{
	if(nRow >= 0 && nCol >= 0 && nRow < m_nRows && nCol < m_nCols) {
		return m_lElements[nCol + nRow * m_nCols];
	}
	return 0.0;
}

//-----------------------------------------------------------------------------------
void CMatrixZone::SetElement(int nRow, int nCol, double dValue)
{
	if(nRow >= 0 && nCol >= 0 && nRow < m_nRows && nCol < m_nCols) {
		m_lElements[nCol + nRow * m_nCols] = dValue;
		OnDataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::AddRow(int nRow)
{
	if(nRow < m_nRows && m_nRows < 255 && CalculateSize(m_nRows + 1, m_nCols) + 14 <= 65535) {
		List::Iterator it = m_lElements.end();
		if(nRow >= 0) {
			it = m_lElements.begin() + (nRow * m_nCols);
		}
		for(int c = 0; c < m_nCols; c++) {
			it = m_lElements.insert(it, 0.0);
		}
		m_nRows++;
		DataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::AddCol(int nCol)
{
	if(nCol < m_nCols && m_nCols < 255 && CalculateSize(m_nRows, m_nCols + 1) + 14 <= 65535) {
		List::Iterator it = m_lElements.begin();
		if(nCol < 0) {
			it += m_nCols;
		}
		else {
			it += nCol;
		}
		m_nCols++;
		for(int r = 0; r < m_nRows; r++) {
			it = m_lElements.insert(it, 0.0) + m_nCols;
		}
		DataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::RemoveRow(int nRow)
{
	if(nRow >= 0 && nRow < m_nRows && m_nRows > 0) {
		List::Iterator it = m_lElements.begin() + (nRow * m_nCols);
		for(int c = 0; c < m_nCols; c++) {
			it = m_lElements.erase(it);
		}
		m_nRows--;
		DataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::RemoveCol(int nCol)
{
	if(nCol >= 0 && nCol < m_nCols && m_nCols > 0) {
		List::Iterator it = m_lElements.begin() +  nCol;
		m_nCols--;
		for(int r = 0; r < m_nRows; r++) {
			it = m_lElements.erase(it) + m_nCols;
		}
		DataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CMatrixZone::SetData(void* pData, unsigned int nSize, bool bEmitDataChanged)
{
	m_lElements.clear();
	if(pData == NULL || nSize == 0) {
		m_lElements.clear();
		m_nRows = m_nCols = 0;
		CFileZone::SetData(NULL, 0, bEmitDataChanged);
		return;
	}

	m_nRows = ((unsigned char*)pData)[8];
	m_nCols = ((unsigned char*)pData)[10];
	
	int nPos = HEADER_SIZE;
	for(int r = 0; r < m_nRows; r++) {
		for(int c = 0; c < m_nCols; c++) {			
			unsigned char var[VAR_SIZE];
			memcpy(var, (char*)pData + nPos, sizeof(var));
			nPos += sizeof(var);
			m_lElements.push_back(CCasioIO::ConvertCasioDataToDouble(var));
		}
	}
	m_bModified = FALSE;
	CFileZone::SetData(pData, nSize, bEmitDataChanged);
}

//-----------------------------------------------------------------------------------
unsigned int CMatrixZone::CalculateSize(int nRows, int nCols)
{
	return HEADER_SIZE + nRows * nCols * VAR_SIZE;
}

//-----------------------------------------------------------------------------------
QString CMatrixZone::ChooseFileName(const QString& sCaption, const QString& sDefaultText, QWidget* parent)
{
	bool bOk;
	QStringList lList;
	for(int i = 0; i < 26; i++) {
		lList.push_back(QString("MAT_") + (char)('A' + i));
	}
	lList.push_back("MAT_Ans");
	int nCurrent = lList.findIndex(sDefaultText);
	QString s = QInputDialog::getItem(sCaption, CFileZone::tr("Choisissez une matrice possible"), lList, nCurrent, FALSE, &bOk, parent);
	if(bOk) {
		return s;
	}
	return QString::null;
}
