// ProgSyntaxProperties.cpp: implementation of the CProgSyntaxProperties class.
//
//////////////////////////////////////////////////////////////////////

#include "ProgSyntaxProperties.h"
#include "SyntaxProperty.h"

#include <qobject.h>

class CCommentProperty : public CSyntaxProperty {
public:
	CCommentProperty() : CSyntaxProperty() { Reset(); }
	CCommentProperty(CCommentProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Commentaire"); }
	QString GetId()   const  { return KEY_COMMENT; }
	CSyntaxProperty* Clone() { return new CCommentProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor(0, 150, 100)); }
};

class CNumberProperty : public CSyntaxProperty {
public:
	CNumberProperty() : CSyntaxProperty() { Reset(); }
	CNumberProperty(CNumberProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Nombre"); }
	QString GetId()   const  { return KEY_NUMBER; }
	CSyntaxProperty* Clone() { return new CNumberProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor(170, 0, 0)); }
};

class CVarProperty : public CSyntaxProperty {
public:
	CVarProperty() : CSyntaxProperty() { Reset(); }
	CVarProperty(CVarProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Variable"); }
	QString GetId()   const  { return KEY_VAR; }
	CSyntaxProperty* Clone() { return new CVarProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor()); }
};

class CStringProperty : public CSyntaxProperty {
public:
	CStringProperty() : CSyntaxProperty() {	Reset(); }
	CStringProperty(CStringProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Chaine"); }
	QString GetId()   const  { return KEY_STRING; }
	CSyntaxProperty* Clone() { return new CStringProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor(0, 100, 100)); }
};

class CKeywordProperty : public CSyntaxProperty {
public:
	CKeywordProperty() : CSyntaxProperty() { Reset(); }
	CKeywordProperty(CKeywordProperty* pSource) : CSyntaxProperty(pSource) { }
	QString GetName() const  { return QObject::tr("Mot cl"); }
	QString GetId()   const  { return KEY_KEYWORD; }
	CSyntaxProperty* Clone() { return new CKeywordProperty(this); }
	void Reset() { CSyntaxProperty::Reset(); SetColor(QColor(0, 170, 255)); }
};

//-----------------------------------------------------------------------------------
CProgSyntaxProperties::CProgSyntaxProperties()
	: CSyntaxProperties()
{
	GetProperties().push_back(new CNumberProperty());
	GetProperties().push_back(new CVarProperty());
	GetProperties().push_back(new CStringProperty());
	GetProperties().push_back(new CKeywordProperty());
}

//-----------------------------------------------------------------------------------
CProgSyntaxProperties::CProgSyntaxProperties(const CProgSyntaxProperties* pSource)
: CSyntaxProperties(pSource)
{
}

//-----------------------------------------------------------------------------------
void CProgSyntaxProperties::AddCommentProperty()
{
	GetProperties().push_back(new CCommentProperty());
}
