// RAMManager.cpp: implementation of the RAMManager class.
//
//////////////////////////////////////////////////////////////////////

#include "RAMManager.h"
#include "RAMManagerPlugIn.h"
#include "LeftPanel.h"
#include "FileZone.h"
#include "FileLoader.h"
#include "Editor.h"
#include "RamManagerOption.h"

#include <qworkspace.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qfile.h>

#include <image_defs.h>
#include <language.h>
#include <IOptionWindow.h>
#include <TransfertWindow.h>
#include <MessageBox.h>
#include <APIFunction.h>
#include <F100_Debug.h>

#define IMG_SEND         "flashmanager/send.png"
#define IMG_RECEIVE      "flashmanager/receive.png"
#define IMG_FILESELECTED "rammanager/file_selected.png"

//-----------------------------------------------------------------------------------
RAMManager::RAMManager(RAMManagerPlugIn* pPlugIn)
	: QMainWindow()
	, m_bModified(FALSE)
{
	menuBar();
	m_pPlugIn        = pPlugIn;
	m_pWorkSpace     = new QWorkspace(this);
	m_pToolBar       = new QToolBar(this);
	m_pLeftPanel     = new LeftPanel(this, CT_G100_LIKE);
	m_pFileMenu      = new QPopupMenu(this);
	m_pSendMenu      = new QPopupMenu(this);
	m_pWindowMenu    = new QPopupMenu(this);
	m_pNewAction     = new QAction(this);
	m_pLoadAction    = new QAction(this);
	m_pSaveAction    = new QAction(this);
	m_pSaveAsAction  = new QAction(this);
	m_pSendAction    = new QAction(this);
	m_pReceiveAction = new QAction(this);
	m_pOptionAction  = new QAction(this);
	m_pSelectAction  = new QAction(this);

	setIcon(QPixmap::fromMimeSource(IMG_ICON));

    m_pWindowMenu->setCheckable(TRUE);

	setCentralWidget(m_pWorkSpace);
	moveDockWindow(m_pLeftPanel, Qt::DockLeft);

	m_pNewAction->addTo(m_pFileMenu);
	m_pLoadAction->addTo(m_pFileMenu);
	m_pFileMenu->insertSeparator();
	m_pSaveAction->addTo(m_pFileMenu);
	m_pSaveAsAction->addTo(m_pFileMenu);
	m_pSendAction->addTo(m_pSendMenu);
	m_pReceiveAction->addTo(m_pSendMenu);
	m_pSendMenu->insertSeparator();
	m_pOptionAction->addTo(m_pSendMenu);

	m_pNewAction->addTo(m_pToolBar);
	m_pLoadAction->addTo(m_pToolBar);
	m_pSaveAction->addTo(m_pToolBar);
	m_pToolBar->addSeparator();
	m_pSendAction->addTo(m_pToolBar);
	m_pReceiveAction->addTo(m_pToolBar);
	m_pToolBar->addSeparator();
	m_pSelectAction->addTo(m_pToolBar);
	m_pToolBar->addSeparator();
	m_pOptionAction->addTo(m_pToolBar);

	m_pNewAction->setIconSet(QPixmap::fromMimeSource(IMG_NEW));
	m_pLoadAction->setIconSet(QPixmap::fromMimeSource(IMG_OPEN));
	m_pSaveAction->setIconSet(QPixmap::fromMimeSource(IMG_SAVE));
	m_pSendAction->setIconSet(QPixmap::fromMimeSource(IMG_SEND));
	m_pReceiveAction->setIconSet(QPixmap::fromMimeSource(IMG_RECEIVE));
	m_pSelectAction->setIconSet(QPixmap::fromMimeSource(IMG_FILESELECTED));
	m_pOptionAction->setIconSet(QPixmap::fromMimeSource(IMG_OPTION));

	m_pNewAction->setAccel(CTRL  + Key_N);
	m_pLoadAction->setAccel(CTRL + Key_O);
	m_pSaveAction->setAccel(CTRL + Key_S);
	m_pSendAction->setAccel(Key_F5);
	m_pReceiveAction->setAccel(Key_F6);
	m_pOptionAction->setAccel(Key_F10);
	
	connect(m_pNewAction,     SIGNAL(activated()), SLOT(OnNew()));
	connect(m_pLoadAction,    SIGNAL(activated()), SLOT(OnOpen()));
	connect(m_pSaveAction,    SIGNAL(activated()), SLOT(OnSave()));
	connect(m_pSaveAsAction,  SIGNAL(activated()), SLOT(OnSaveAs()));
	connect(m_pSendAction,    SIGNAL(activated()), SLOT(OnSend()));
	connect(m_pReceiveAction, SIGNAL(activated()), SLOT(OnReceive()));
	connect(m_pOptionAction,  SIGNAL(activated()), SLOT(OnShowOption()));
	connect(m_pSelectAction,  SIGNAL(activated()), m_pLeftPanel, SLOT(OnSelect()));
	connect(m_pWindowMenu,    SIGNAL(aboutToShow()), SLOT(OnWindowsMenuAboutToShow()));
}

//-----------------------------------------------------------------------------------
RAMManager::~RAMManager()
{
	FileList::Iterator it = m_lFiles.begin();
	for(; it != m_lFiles.end(); ++it) {
		delete *it;
	}
	m_lFiles.clear();
}

//-----------------------------------------------------------------------------------
void RAMManager::DoInitWords()
{
	menuBar()->clear();
	menuBar()->insertItem(tr("&Fichier"),       m_pFileMenu);
	menuBar()->insertItem(tr("&Communication"), m_pSendMenu);
	menuBar()->insertItem(tr("&Fentre"),       m_pWindowMenu);

	m_pToolBar->setCaption(tr("Barre d'outils"));

	m_pNewAction->setMenuText(tr("&Nouveau projet"));
	m_pNewAction->setToolTip(tr("Charge un projet vierge"));

	m_pLoadAction->setMenuText(tr("&Ouvrir un projet"));
	m_pLoadAction->setToolTip(tr("Ouvre un fichier .cat"));

	m_pSaveAction->setMenuText(tr("&Sauver"));
	m_pSaveAction->setToolTip(tr("Sauve le projet courant"));

	m_pSaveAsAction->setMenuText(tr("S&auver sous..."));
	m_pSaveAsAction->setToolTip(tr("Sauve le projet courant sous un autre nom"));

	m_pSendAction->setMenuText(tr("&Envoyer"));
	m_pSendAction->setToolTip(tr("Envoie les fichiers slectionns"));

	m_pReceiveAction->setMenuText(tr("&Recevoir"));
	m_pReceiveAction->setToolTip(tr("Recevoir des fichiers"));

	m_pOptionAction->setMenuText(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));
	m_pOptionAction->setToolTip(tr("Affiche les options pour rgler les paramtres de communication"));

	m_pSelectAction->setToolTip(tr("Slectionne/dselectionne un fichier"));

	m_pLeftPanel->DoInitWords();

	EditorMap::Iterator it = m_mEditors.begin();
	const CEditor* pEditor = (const CEditor*)(sender());
	for(; it != m_mEditors.end(); ++it) {
		it.data()->DoInitWords();
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnShowOption()
{
	m_pPlugIn->GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(NULL);
}

//-----------------------------------------------------------------------------------
void RAMManager::SetProjectState(bool bModified)
{
	m_bModified = bModified;
	UpdateCaption();
}

//-----------------------------------------------------------------------------------
bool RAMManager::AskToSaveProject()
{
	bool bClose = true;
	if(m_bModified) {
		switch(CMessageBox::MsgYesNoCancel(m_pPlugIn->GetMainWidget(), tr("Voulez-vous sauver le projet modifi avant de le fermer ?")))
		{
		case QMessageBox::Yes:
			OnSave();
			break;
		case QMessageBox::Cancel:
			bClose = false;
		}
	}
	return bClose;
}

//-----------------------------------------------------------------------------------
bool RAMManager::CloseProject()
{
	bool bLastState = m_bModified;
	m_bModified = FALSE;

	while(m_mEditors.begin() != m_mEditors.end() && m_mEditors.begin().data()->close(TRUE));
	if(!m_mEditors.empty()) {
		m_bModified |= bLastState;
		return FALSE;
	}
	if(!AskToSaveProject()) {
		return FALSE;
	}
	m_pLeftPanel->RemoveAllFiles();

	FileList::Iterator it = m_lFiles.begin();
	for(; it != m_lFiles.end(); ++it) {
		delete *it;
	}
	m_lFiles.clear();
	if(!m_sFileName.isEmpty()) {
		emit m_pPlugIn->StateChanged();
		m_sFileName = QString::null;
	}
	SetProjectState(FALSE);
	return TRUE;
}

//-----------------------------------------------------------------------------------
void RAMManager::UpdateCaption()
{
	m_pPlugIn->setCaption(tr("Gestionnaire de RAM"));
	if(!m_sFileName.isEmpty()) {
		const char* pName;
		GetNameOfFile(m_sFileName, pName);
		m_pPlugIn->setCaption(m_pPlugIn->caption() + " - " + QString(pName) + (m_bModified ? "*" : QString::null));
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnNew()
{
	if(AskToSaveProject()) {
		CloseProject();
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnOpen()
{
	if(AskToSaveProject()) {
		QString sName = CMessageBox::ChooseOpenFileName(m_sFileName, tr("Projets (*.cat)"), this);
		LoadProject(sName);
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnSave()
{
	if(m_sFileName.isEmpty()) {
		OnSaveAs();
	}
	else {
		SaveProject(m_sFileName);
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnSaveAs()
{
	QString sName = CMessageBox::ChooseSaveFileName(m_sFileName, tr("Projets (*.cat)"), this, "cat");
	SaveProject(sName);
}

//-----------------------------------------------------------------------------------
bool RAMManager::CheckCalcType(CTransfertWindow* pTransfert)
{
	if(pTransfert->GetCasioType() != CFileZone::GetCasioType()) {
		CMessageBox::MsgCritical(pTransfert->parentWidget(), tr("Le protocole utilis n'est pas compatible avec le modle de calculatrice utilis."));
		return false;
	}
	return true;
}

//-----------------------------------------------------------------------------------
void RAMManager::OnSend()
{
	FileList list;
	unsigned int nSize = m_pLeftPanel->GetFileSelected(list);
	if(list.isEmpty()) {
		return;
	}
	CTransfertWindow* pTransfert = m_pPlugIn->GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(m_pPlugIn->GetMainWidget());
	if(!pTransfert) {
		return;
	}
	if(!CheckCalcType(pTransfert)) {
		delete pTransfert;
		return;
	}
	pTransfert->SetData(nSize);
	int nRet = 1;

	FileList::Iterator it = list.begin();
	for(; it != list.end() && nRet > 0; ++it) {
		CFileZone* pFile = *it;
		nRet = pTransfert->SendingRAM(pFile->GetZone(), pFile->GetName(), pFile->GetData(), pFile->GetSize(), pFile->GetPassword());
	}
	if(nRet != -1) {	
		pTransfert->CloseComm(1);
	}
	delete pTransfert;
}

//-----------------------------------------------------------------------------------
void RAMManager::OnReceive()
{
	CTransfertWindow* pTransfert = m_pPlugIn->GetMainWindow()->GetOptionWindow()->CreateTransfertWindow(m_pPlugIn->GetMainWidget());
	if(!pTransfert) {
		return;
	}
	if(!CheckCalcType(pTransfert)) {
		delete pTransfert;
		return;
	}

	int nRet = 1;
	while(nRet >= 0) {
		unsigned short nZone;
		char name[9];
		char password[9];
		nRet = pTransfert->WaitingRAM(nZone, name, password);
		if(nRet < 0) {
			break;
		}
		if(m_pLeftPanel->IsSupported(nZone) && (GetFile(nZone, name) == NULL || CMessageBox::MsgQuestion(m_pPlugIn->GetMainWidget(), tr("Voulez-vous craser le fichier %1 ?").arg(name)))) {
			void* pBuffer = new char[nRet];
			if(pBuffer == NULL) {
				CMessageBox::MsgCritical(m_pPlugIn->GetMainWidget(), CLanguage::GetMsgOutOfMemory(nRet));
				nRet = pTransfert->ReceivingRAM(NULL, nZone, nRet, NULL);
			}
			else {
				nRet = pTransfert->ReceivingRAM(pBuffer, nZone, nRet, password);
				if(nRet >= 0) {
					AddFile(nZone, pBuffer, (unsigned int)nRet, name, password);					
				}
				else {
					delete[] pBuffer;
				}
			}
		}
		else {
			nRet = pTransfert->ReceivingRAM(NULL, nZone, nRet, NULL);
		}
	}
	if(nRet != -1) {
		pTransfert->CloseComm(1);
	}
	delete pTransfert;
}

//-----------------------------------------------------------------------------------
bool RAMManager::AddFile(unsigned int nZone, void* pData, unsigned int nSize, const char* pName, const char* pPassword, bool bErase)
{
	CFileZone* pFile = GetFile(nZone, pName);
	if(pFile == NULL) {
		pFile = CFileLoader::LoadFileZone(nZone);
		if(pFile != NULL) {
			RecordFileZone(pFile);
		}
	}
	else {
		if(!bErase) {
			return false;
		}
	}
	if(pFile != NULL) {
		pFile->SetName(pName);
		pFile->SetPassword(pPassword);
		pFile->SetData(pData, nSize, TRUE);
		m_pLeftPanel->AddFile(pFile);
	}
	return true;
}

//-----------------------------------------------------------------------------------
CFileZone* RAMManager::GetFile(unsigned int nZone, const char* pName)
{
	FileList::Iterator it = m_lFiles.begin();
	for(; it != m_lFiles.end(); ++it) {
		CFileZone* pFile = *it;
		if(pFile->GetZone() == nZone && pFile->GetName() == pName) {
			return pFile;
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
void RAMManager::RecordFileZone(CFileZone* pFile)
{
	_ASSERTE(pFile != NULL && !m_lFiles.contains(pFile));
	m_lFiles.push_back(pFile);
	connect(pFile, SIGNAL(DataChanged()), SLOT(OnProjetModified()));
	connect(pFile, SIGNAL(Renamed()),     SLOT(OnProjetModified()));
}

//-----------------------------------------------------------------------------------
int RAMManager::LoadProject(const QString& sFileName, bool bShowMsgErr)
{
	if(!sFileName.isEmpty()) {
		QFile file(sFileName);
		if(!file.open(IO_ReadOnly)) {
			CMessageBox::MsgCritical(m_pPlugIn->GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName, FALSE));
			return false;
		}
		if(CloseProject()) {
			QTextStream ts(&file);
			CFileZone* pFile = NULL;
			bool bOk;
			do{
				pFile = CFileLoader::LoadFileZone(ts, bOk);
				if(pFile != NULL) {
					RecordFileZone(pFile);
					m_pLeftPanel->AddFile(pFile);
				}
			}while(bOk && !ts.atEnd());
			if(!bOk) {
				if(bShowMsgErr) {
					CMessageBox::MsgCritical(m_pPlugIn->GetMainWidget(), CLanguage::GetMsgNoValidFile(sFileName));
				}
				return -1;
			}
			emit m_pPlugIn->StateChanged();
			m_sFileName = sFileName;
			SetProjectState(FALSE);
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
bool RAMManager::SaveProject(const QString& sFileName)
{
	if(!sFileName.isEmpty()) {
		QFile file(sFileName);	
		if(!file.open(IO_Translate | IO_WriteOnly)) {
			CMessageBox::MsgCritical(m_pPlugIn->GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName, TRUE));
			return false;
		}
		QTextStream ts(&file);
		EditorMap::iterator eit = m_mEditors.begin();
		for(; eit != m_mEditors.end(); eit++) {
			eit.data()->Save();
		}
		FileList::Iterator it = m_lFiles.begin();
		for(; it != m_lFiles.end(); ++it) {
			(*it)->Save(ts);
		}
		if(m_sFileName != sFileName) {
			emit m_pPlugIn->StateChanged();
		}
		m_sFileName = sFileName;
		SetProjectState(FALSE);
	}
	return true;
}

//-----------------------------------------------------------------------------------
void RAMManager::closeEvent(QCloseEvent* e)
{
	if(!AskToSaveProject() || !CloseProject()) {
		e->ignore();
	}
	else {
		e->accept();
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::CreateEditor(CFileZone* pFile)
{
	EditorMap::Iterator it = m_mEditors.find(pFile);
	CEditor* pEditor = NULL;
	if(it == m_mEditors.end()) {
		pEditor = CFileLoader::CreateEditor(pFile, m_pWorkSpace);
		if(pEditor != NULL) {
			m_mEditors[pFile] = pEditor;
			connect(pEditor, SIGNAL(destroyed()), this, SLOT(OnEditorDestroyed()));
			m_pPlugIn->StateChanged();
		}
	}
	else {
		pEditor = it.data();
	}
	if(pEditor != NULL) {
		pEditor->show();
		pEditor->setFocus();
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnEditorDestroyed()
{
	EditorMap::Iterator it = m_mEditors.begin();
	const CEditor* pEditor = (const CEditor*)(sender());
	for(; it != m_mEditors.end(); ++it) {
		if(it.data() == pEditor) {
			m_mEditors.remove(it);
			m_pPlugIn->StateChanged();
			break;
		}
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::CreateEditor(const QString& sFileName, unsigned short nZone)
{
	CFileZone* pFile = GetFile(nZone, sFileName);
	if(pFile != NULL) {
		CreateEditor(pFile);
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnProjetModified()
{
	SetProjectState(TRUE);
}

//-----------------------------------------------------------------------------------
void RAMManager::OnWindowsMenuAboutToShow()
{
	m_pWindowMenu->clear();
	int nCascadeId = m_pWindowMenu->insertItem(tr("&Cascade"), m_pWorkSpace, SLOT(cascade()));
	int nTileId    = m_pWindowMenu->insertItem(tr("C&arreau"), m_pWorkSpace, SLOT(tile()));
	QWidgetList lWindows = m_pWorkSpace->windowList(QWorkspace::CreationOrder);
	if(lWindows.isEmpty()) {
		m_pWindowMenu->setItemEnabled(nCascadeId, FALSE);
		m_pWindowMenu->setItemEnabled(nTileId,    FALSE);
	}
	else {
		m_pWindowMenu->insertSeparator();
		m_pWindowMenu->insertItem(tr("Fermer &Tout"), m_pWorkSpace, SLOT(closeAllWindows()));
		m_pWindowMenu->insertSeparator();
	}
	for(unsigned int i = 0; i < lWindows.count(); i++) {
		int id = m_pWindowMenu->insertItem(lWindows.at(i)->caption(),
			this, SLOT(OnWindowsMenuActivated(int)));
		m_pWindowMenu->setItemParameter(id, i);
		m_pWindowMenu->setItemChecked(id, m_pWorkSpace->activeWindow() == lWindows.at(i));
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::OnWindowsMenuActivated(int id)
{
    QWidget* w = m_pWorkSpace->windowList(QWorkspace::CreationOrder).at(id);
    if(w) {
		w->showNormal();
		w->setFocus();
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::AddFile(const QString& sFileName, unsigned short nZone)
{
	if(!AddFile(nZone, NULL, 0, sFileName, NULL, FALSE)) {
		CMessageBox::MsgInformation(m_pPlugIn->GetMainWidget(), tr("Le fichier %1 existe dj.").arg(sFileName));
	}
}

//-----------------------------------------------------------------------------------
void RAMManager::RenameFile(CFileZone* pFileToRename, const QString& sFileName)
{
	FileList::Iterator it = m_lFiles.begin();
	for(; it != m_lFiles.end(); ++it) {
		CFileZone* pFZ = *it;
		if(pFZ->GetZone() == pFileToRename->GetZone() && pFZ->GetName() == sFileName) {
			CMessageBox::MsgInformation(m_pPlugIn->GetMainWidget(), tr("Le fichier %1 existe dj.").arg(sFileName));
			return;
		}
	}
	pFileToRename->SetName(sFileName);
}

//-----------------------------------------------------------------------------------
void RAMManager::RemoveFile(CFileZone* pFileToRemove)
{
	EditorMap::iterator it = m_mEditors.find(pFileToRemove);
	if(it != m_mEditors.end()) {
		it.data()->close();
	}
	m_pLeftPanel->RemoveFile(pFileToRemove);
	delete pFileToRemove;
	m_lFiles.remove(pFileToRemove);
	OnProjetModified();
}
