// RAMManager.h: interface for the RAMManager class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _RAMMANAGER_H_
#define _RAMMANAGER_H_

#include <qmainwindow.h>
#include <qdom.h>
#include <qvaluelist.h>

class RAMManagerPlugIn;
class QWorkspace;
class LeftPanel;
class QAction;
class QPopupMenu;
class QToolBar;
class CFileZone;
class CTransfertWindow;
class CEditor;

typedef QValueList<CFileZone*> FileList;
typedef QMap<CFileZone*, CEditor*> EditorMap;

#define IMG_ICON "rammanager/icon.png"

class RAMManager : public QMainWindow
{
	Q_OBJECT
	friend class RAMManagerPlugIn;
public:
	RAMManager(RAMManagerPlugIn* pPlugIn);
	virtual ~RAMManager();

	void DoInitWords();
	void closeEvent(QCloseEvent* e);
	void CreateEditor(CFileZone* pFile);
	void AddFile(const QString& sFileName, unsigned short nZone);
	void RemoveFile(CFileZone* pFileToRemove);
	void RenameFile(CFileZone* pFileToRename, const QString& sFileName);

private:
	bool       AddFile(unsigned int nZone, void* pData, unsigned int nSize, const char* pName, const char* pPassword = NULL, bool bErase = TRUE);
	CFileZone* GetFile(unsigned int nZone, const char* pName);

	bool  SaveProject(const QString& sFileName);
	int   LoadProject(const QString& sFileName, bool bShowMsgErr = TRUE);
	bool  CloseProject();
	void  UpdateCaption();
	bool  AskToSaveProject();
	void  SetProjectState(bool bModified);
	void  CreateEditor(const QString& sFileName, unsigned short nZone);
	void  RecordFileZone(CFileZone* pFile);

private:
	static bool CheckCalcType(CTransfertWindow* pTransfert);

private slots:
	void OnShowOption();
	void OnNew();
	void OnOpen();
	void OnSave();
	void OnSaveAs();
	void OnSend();
	void OnReceive();
	void OnEditorDestroyed();
	void OnProjetModified();
	void OnWindowsMenuAboutToShow();
	void OnWindowsMenuActivated(int id);

private:
	RAMManagerPlugIn* m_pPlugIn;
	QWorkspace*  m_pWorkSpace;
	LeftPanel*   m_pLeftPanel;
	QString      m_sFileName;
	QAction*     m_pNewAction;
	QAction*     m_pLoadAction;
	QAction*     m_pSaveAction;
	QAction*     m_pSaveAsAction;
	QAction*     m_pSendAction;
	QAction*     m_pReceiveAction;
	QAction*     m_pOptionAction;
	QAction*     m_pSelectAction;
	QPopupMenu*  m_pFileMenu;
	QPopupMenu*  m_pSendMenu;
	QPopupMenu*  m_pWindowMenu;
	QToolBar*    m_pToolBar;
	FileList     m_lFiles;
	EditorMap    m_mEditors;
	bool         m_bModified;
};

#endif
