
#include "RAMManagerPlugIn.h"
#include "RAMManager.h"
#include "FileZone.h"

#include <qstatusbar.h>

#include <assert.h>

//-----------------------------------------------------------------------------------
RAMManagerPlugIn::RAMManagerPlugIn(CONSTRUCTOR_PARAM)
	: CToolWindow(CALL_PARAM)
{
	m_pCentralWidget = new RAMManager(this);
	SetCentralWidget(m_pCentralWidget);
	if(m_pCentralWidget->icon() != NULL) {
		setIcon(*m_pCentralWidget->icon());
	}
	languageChange();
}

//-----------------------------------------------------------------------------------
int RAMManagerPlugIn::Init(INIT_PARAM)
{
	return m_pCentralWidget->LoadProject(sFileName, FALSE);
}

//-----------------------------------------------------------------------------------
void RAMManagerPlugIn::DoShowCaption()
{
	m_pCentralWidget->UpdateCaption();
}

//-----------------------------------------------------------------------------------
void RAMManagerPlugIn::DoInitWords()
{
	((RAMManager*)GetCentralWidget())->DoInitWords();
}

//-----------------------------------------------------------------------------------
int RAMManagerPlugIn::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1) {
		int nRet = m_pCentralWidget->LoadProject(ConvertToFullPath(element.attribute("m_sFileName")));
		QDomElement files = element.namedItem("Files").toElement();
		if(nRet > 0 && !files.isNull()) {
			QDomElement file = files.namedItem("File").toElement();
			while(!file.isNull()) {
				m_pCentralWidget->CreateEditor(file.attribute("name"), file.attribute("zone").toUShort());
				file = file.nextSibling().toElement();;
			}
		}
		return nRet;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int RAMManagerPlugIn::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1) {
		element.setAttribute("m_sFileName", ConvertToRelativePath(m_pCentralWidget->m_sFileName));
		QDomElement files = element.ownerDocument().createElement("Files");
		element.appendChild(files);
		EditorMap::Iterator it = m_pCentralWidget->m_mEditors.begin();
		for(; it != m_pCentralWidget->m_mEditors.end(); ++it) {
			QDomElement e = files.ownerDocument().createElement("File");
			files.appendChild(e);
			e.setAttribute("zone", it.key()->GetZone());
			e.setAttribute("name", it.key()->GetName());
		}
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void RAMManagerPlugIn::SetInTabWidget(bool bInTabWidget, bool bShowIt, bool bEmitStateChanged)
{
	CToolWindow::SetInTabWidget(bInTabWidget, bShowIt, bEmitStateChanged);
	if(IsInTabWidget()) {
		delete m_pCentralWidget->statusBar();
	}
	else {
		m_pCentralWidget->statusBar();
	}
}

//-----------------------------------------------------------------------------------
bool RAMManagerPlugIn::close(bool bAlsoDelete)
{
	assert(m_pCentralWidget != NULL);
	if(m_pCentralWidget->close()) {
		return CToolWindow::close(bAlsoDelete);
	}
	return FALSE;
}
