// SyntaxHighlighter.cpp: implementation of the CSyntaxHighlighter class.
//
//////////////////////////////////////////////////////////////////////

#include "SyntaxHighlighter.h"
#include "RamManagerOption.h"
#include "SyntaxProperty.h"
#include "SyntaxProperties.h"

#include <qregexp.h>

//-----------------------------------------------------------------------------------
CSyntaxHighlighter::CSyntaxHighlighter(QTextEdit* pTextEdit, CSyntaxProperties* pProperties)
	: QSyntaxHighlighter(pTextEdit)
	, m_pProperties(pProperties)
{

}

//-----------------------------------------------------------------------------------
const QFont& CSyntaxHighlighter::GetFont() const
{
	if(m_pProperties->GetCustomFont() != NULL) {
		return *(m_pProperties->GetCustomFont());
	}
	return CRamManagerOption::GetInstance()->GetFontEditor();
}

//-----------------------------------------------------------------------------------
const QColor& CSyntaxHighlighter::GetEditorForeColor() const
{
	return CRamManagerOption::GetInstance()->GetForeColor();
}

//-----------------------------------------------------------------------------------
void CSyntaxHighlighter::GetFont(QFont& font, QColor& color, const QString& sProperty)
{
	CSyntaxProperty* pProperty = GetProperties()->GetProperty(sProperty);
	if(pProperty != NULL) {
		color = pProperty->GetColor();
		if(!color.isValid()) {
			color = GetEditorForeColor();
		}
		font.setBold(pProperty->IsBold());
		font.setItalic(pProperty->IsItalic());
		font.setUnderline(pProperty->IsUnderline());
	}
	else {
		color = QColor();
	}
}

//-----------------------------------------------------------------------------------
void CSyntaxHighlighter::SearchExp(QRegExp* pRegExp, const QString& sText, const QFont& font, const QColor& color)
{
	int nPos = 0;
	if(color.isValid()) {
		do{
			nPos = pRegExp->search(sText, nPos);
			if(nPos != -1) {
				int nLength = pRegExp->matchedLength();
				setFormat(nPos, nLength, font, color);
				nPos += nLength;
			}
			
		}while(nPos != -1);
	}
}
