// SyntaxHighligterFactory.cpp: implementation of the CSyntaxHighligterFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "SyntaxHighligterFactory.h"
#include "TextSyntaxHighligter.h"
#include "SyntaxProperties.h"
#include "MLC1Properties.h"
#include "BASICProperties.h"

#include <qobject.h>

CSyntaxHighligterFactory CSyntaxHighligterFactory::m_instance;

class CTextProperties : public CSyntaxProperties
{
public:
	CTextProperties() : CSyntaxProperties() {}
	CTextProperties(const CTextProperties* pSource) : CSyntaxProperties(pSource) {}
	QString GetName() const { return QObject::tr("Texte"); }
	CSyntaxHighlighter* CreateSyntax(QTextEdit* pEdit) {
		return new CTextSyntaxHighligter(pEdit, this);
	}
	CSyntaxHighlighter* CreateCompatibleSyntax(const void* pData, unsigned int nSize, QTextEdit* pEdit) {
		return CreateSyntax(pEdit);
	}
	CSyntaxProperties* Clone() { return new CTextProperties(this); }
};

//-----------------------------------------------------------------------------------
CSyntaxHighligterFactory::CSyntaxHighligterFactory()
{
	m_lSyntaxP.push_back(new CTextProperties());
	m_lSyntaxP.push_back(new CMLC1Properties());
	m_lSyntaxP.push_back(new CBASICProperties());
}

//-----------------------------------------------------------------------------------
CSyntaxHighligterFactory::~CSyntaxHighligterFactory()
{
	SyntaxPropertiesList::iterator it = m_lSyntaxP.begin();
	for(; it != m_lSyntaxP.end(); it++) {
		delete *it;
	}
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CSyntaxHighligterFactory::CreateSyntax(unsigned int nIndex, QTextEdit* pEdit)
{
	if(nIndex < m_lSyntaxP.size()) {
		CSyntaxProperties* pProperties = m_lSyntaxP[nIndex];		
		return pProperties->CreateSyntax(pEdit);
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CSyntaxHighlighter* CSyntaxHighligterFactory::CreateSyntax(const void* pData, unsigned int nSize, QTextEdit* pEdit)
{
	SyntaxPropertiesList::iterator it = ++m_lSyntaxP.begin();
	for(; it != m_lSyntaxP.end(); it++) {
		CSyntaxHighlighter* pSyntax = (*it)->CreateCompatibleSyntax(pData, nSize, pEdit);
		if(pSyntax != NULL) {
			return pSyntax;
		}
	}
	return CreateSyntax(0, pEdit);
}

//-----------------------------------------------------------------------------------
CSyntaxHighligterFactory* CSyntaxHighligterFactory::GetInstance()
{
	return &m_instance;
}

//-----------------------------------------------------------------------------------
const SyntaxPropertiesList& CSyntaxHighligterFactory::GetSyntaxPropertiesList() const
{
	return m_lSyntaxP;
}
