// SyntaxProperties.cpp: implementation of the CSyntaxProperties class.
//
//////////////////////////////////////////////////////////////////////

#include "SyntaxProperties.h"
#include "SyntaxProperty.h"

#include <qfont.h>

//-----------------------------------------------------------------------------------
CSyntaxProperties::CSyntaxProperties()
	: m_pCustomFont(NULL)
{

}

//-----------------------------------------------------------------------------------
CSyntaxProperties::CSyntaxProperties(const CSyntaxProperties* pSource)
	: m_pCustomFont(NULL)
{
	*this = *pSource;
}

//-----------------------------------------------------------------------------------
CSyntaxProperties::~CSyntaxProperties()
{
	if(m_pCustomFont != NULL) {
		delete m_pCustomFont;
	}
	CleanUp();
}

//-----------------------------------------------------------------------------------
void CSyntaxProperties::CleanUp()
{
	PropertyList::iterator it = m_lProperties.begin();
	for(; it != m_lProperties.end(); it++) {
		delete *it;
	}
	m_lProperties.clear();
}

//-----------------------------------------------------------------------------------
PropertyList& CSyntaxProperties::GetProperties()
{
	return m_lProperties;
}

//-----------------------------------------------------------------------------------
CSyntaxProperty* CSyntaxProperties::GetProperty(const QString& sId)
{
	PropertyList::iterator it = m_lProperties.begin();
	for(; it != m_lProperties.end(); it++) {
		if((*it)->GetId() == sId) {
			return *it;
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CSyntaxProperties& CSyntaxProperties::operator=(const CSyntaxProperties& p)
{
	CleanUp();
	SetCustomFont(p.m_pCustomFont);
	PropertyList::const_iterator it = p.m_lProperties.begin();
	for(; it != p.m_lProperties.end(); it++) {
		m_lProperties.push_back((*it)->Clone());
	}
	return *this;
}

//-----------------------------------------------------------------------------------
bool CSyntaxProperties::operator==(const CSyntaxProperties& p) const
{
	if(GetName() == p.GetName()
		&& m_lProperties.size() == p.m_lProperties.size()
		&& (m_pCustomFont == p.m_pCustomFont
			|| (m_pCustomFont != NULL && p.m_pCustomFont != NULL && *m_pCustomFont == *p.m_pCustomFont))) {

		PropertyList::const_iterator it1 = m_lProperties.begin();
		PropertyList::const_iterator it2 = p.m_lProperties.begin();
		for(; it1 != m_lProperties.end(); it1++, it2++) {
			if(**it1 != **it2) {
				return FALSE;
			}
		}
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------------
bool CSyntaxProperties::operator!=(const CSyntaxProperties& p) const
{
	return !(*this == p);
}

//-----------------------------------------------------------------------------------
const QFont* CSyntaxProperties::GetCustomFont() const
{
	return m_pCustomFont;
}

//-----------------------------------------------------------------------------------
void CSyntaxProperties::SetCustomFont(const QFont* pFont)
{
	if(pFont != NULL) {
		if(m_pCustomFont != NULL) {
			*m_pCustomFont = *pFont;
		}
		else {
			m_pCustomFont = new QFont(*pFont);
		}
	}
	else {
		if(m_pCustomFont != NULL) {
			delete m_pCustomFont;
		}
		m_pCustomFont = NULL;
	}
}

//-----------------------------------------------------------------------------------
void CSyntaxProperties::GetKeywords(QStringList& lKeywords)
{
	lKeywords.clear();
}

//-----------------------------------------------------------------------------------
void CSyntaxProperties::Reset()
{
	PropertyList::const_iterator it = m_lProperties.begin();
	for(; it != m_lProperties.end(); it++) {
		(*it)->Reset();
	}
}
