// SyntaxProperties.h: interface for the CSyntaxProperties class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _SYNTAXPROPERTIES_H_
#define _SYNTAXPROPERTIES_H_

#include <qvaluelist.h>
#include <qstringlist.h>

class CSyntaxHighlighter;
class QTextEdit;
class CSyntaxProperty;
class QFont;

typedef QValueList<CSyntaxProperty*> PropertyList;

class CSyntaxProperties  
{
public:
	CSyntaxProperties();
	CSyntaxProperties(const CSyntaxProperties* pSource);
	virtual ~CSyntaxProperties();

	PropertyList&        GetProperties();
	CSyntaxProperty*     GetProperty(const QString& sId);
	virtual const QFont* GetCustomFont() const;
	virtual QString      GetName() const = 0;

	virtual void         SetCustomFont(const QFont* pFont);

	virtual CSyntaxHighlighter* CreateSyntax(QTextEdit* pEdit) = 0;

	/** Permet de savoir si une donne casio correspond  ce type de donnes casio
	 * @param[in] pData Donnes casio brutes
	 * @param[in] nSize Taille des donnes en octets
	 * @param[in] pEdit Zone d'dition
	 * @param[in] pOptionWindow Fentre des options
	 * @return La syntaxe si les donnes correspondent, sinon NULL
	 */
	virtual CSyntaxHighlighter* CreateCompatibleSyntax(const void* pData, unsigned int nSize, QTextEdit* pEdit) = 0;

	virtual CSyntaxProperties* Clone() = 0;

	/** Permet de remettre les paramtres  l'tat initial */
	virtual void Reset();

	CSyntaxProperties& operator=(const CSyntaxProperties& p);
	
	bool operator==(const CSyntaxProperties& p) const;
	bool operator!=(const CSyntaxProperties& p) const;

	/** Permet de rcuprer la liste des mots du langage.\n
	 * Par dfaut, la liste est vide.
	 * @param[out] lKeywords Liste des mots cls
	 */
	virtual void GetKeywords(QStringList& lKeywords);

protected:
	void CleanUp();

private:
	PropertyList m_lProperties;
	QFont*       m_pCustomFont;
};

#endif
