
#include "RFIWindow.h"

#include <qlayout.h>
#include <qinputdialog.h>
#include <qregexp.h>

#include <language.h>
#include <MessageBox.h>
#include <APIFunction.h>
#include <image_defs.h>
#include <SendToMenu.h>

static char rfi_header[] = "RFI";

#define CAPTION    "Ram File Installer"
#define RFI_FILTER "Ram File Installer (*.rfi)\n"
#define MAX_SIZE 65535

//-----------------------------------------------------------------------------------
RFIWindow::RFIWindow(CONSTRUCTOR_PARAM)
:CheckFileWindow(CALL_PARAM, FALSE, FALSE),
 m_qMenu(this)
{
	QWidget* pCentralWidget = new QWidget(0);
	setIcon(QPixmap::fromMimeSource("rfimaker/rfi.png"));

	QBoxLayout* pLayout = new QBoxLayout(pCentralWidget, QBoxLayout::TopToBottom);
	m_pList     = new QListView(pCentralWidget);
	m_pMenuBar  = new QMenuBar(pCentralWidget);
	m_pFileMenu = new QPopupMenu(pCentralWidget);

	SetCentralWidget(pCentralWidget);

	pLayout->setMenuBar(m_pMenuBar);
	pLayout->add(m_pList);

	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);
	m_pList->addColumn(QString::null);

	m_pList->setAllColumnsShowFocus(TRUE);
	m_pList->setColumnAlignment(1, AlignRight);
	m_pList->setColumnAlignment(2, AlignRight);
	m_pList->setColumnWidth(0, 100);
	m_pList->setColumnWidth(1, 50);
	m_pList->setColumnWidth(2, 50);
	m_pList->setColumnWidthMode(0, QListView::Maximum);
	m_pList->setColumnWidthMode(1, QListView::Maximum);
	m_pList->setColumnWidthMode(2, QListView::Maximum);

	m_pList->setSorting(-1);//Dsactive le tri

	m_pMenuBar->insertItem(QString::null, m_pFileMenu, 0);

	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_NEW), QString::null, 0);
	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_OPEN), QString::null, 1);
	m_pFileMenu->insertSeparator();
	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_SAVE), QString::null, 2);
	m_pFileMenu->insertItem(QString::null, 3);
	m_pFileMenu->insertSeparator();
	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_SENDTO), QString::null, GetMainWindow()->GetSendToMenu(), 4);

	for(int i = 0; i < 7; i++) {
		m_qMenu.insertItem(QString::null, i);
	}
	m_qMenu.insertSeparator(1);
	m_qMenu.insertSeparator(5);

	m_qMenu.setAccel(CTRL + Key_A, 0);//Nouveau
	m_qMenu.setAccel(CTRL + Key_I, 1);//Importer
	m_qMenu.setAccel(CTRL + Key_T, 2);//Importer Sources
	m_qMenu.setAccel(CTRL + Key_E, 3);//Exporter
	m_qMenu.setAccel(CTRL + Key_R, 4);//Renommer
	m_qMenu.setAccel(CTRL + Key_Z, 5);//Changer zone
	m_qMenu.setAccel(Key_Delete,   6);//Supprimer

	//Connexion
	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(RefreshPopup(QListViewItem*)));
	connect(m_pList, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)), SLOT(OnRightClick(QListViewItem*, const QPoint&)));
	connect(m_pList, SIGNAL(itemRenamed(QListViewItem*, int)), SLOT(OnItemRenamed(QListViewItem*)));
	connect(&m_qMenu, SIGNAL(activated(int)), SLOT(OnPopupMenuSelected(int)));
	connect(m_pFileMenu, SIGNAL(activated(int)), SLOT(OnMenuActivated(int)));
	connect(m_pFileMenu, SIGNAL(aboutToShow()), SLOT(OnSendTo()));
	
	RefreshSaveButtons();

	m_bFileModified = false;
}

//-----------------------------------------------------------------------------------
RFIWindow::~RFIWindow()
{
	ClearFileList();
}

//-----------------------------------------------------------------------------------
void RFIWindow::closeEvent(QCloseEvent* e)
{
	if(CloseFile() == 1) {
		CheckFileWindow::closeEvent(e);
	}
	else {
		e->ignore();
	}
}

//-----------------------------------------------------------------------------------
int RFIWindow::CloseFile()
{
	if(m_bFileModified)
	{
		switch(CMessageBox::MsgYesNoCancel(GetMainWidget(), tr("Voulez-vous sauver les changements apports avant de quitter ?")))
		{
		case QMessageBox::Cancel:
			return -1;
		case QMessageBox::Yes:
			Save();
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void RFIWindow::ClearFileList()
{
	FileMap::iterator it  = m_mFile.begin();
	FileMap::iterator end = m_mFile.end();
	for(; it != end; ++it)
	{
		delete[] (it.data().data);
	}
	m_mFile.clear();
	m_pList->clear();
	m_pFileMenu->setItemEnabled(4, FALSE);
	RefreshSaveButtons();
}

//-----------------------------------------------------------------------------------
void RFIWindow::DeleteFileItem(QListViewItem* pItem)
{
	if(pItem)
	{
		FileMap::iterator it = m_mFile.find(pItem);
		if(it != m_mFile.end())
		{
			delete[] (it.data().data);
			m_pList->takeItem(pItem);
			m_mFile.remove(it);
			delete pItem;
		}
	}
}

//-----------------------------------------------------------------------------------
int RFIWindow::Init(INIT_PARAM)
{
	DoInitWords();
	if(!SetFileName(sFileName, FALSE)) {
		return 0;
	}
	return DoReloadFile(FALSE);
}
	
//-----------------------------------------------------------------------------------
void RFIWindow::DoShowCaption()
{
	QString s(CAPTION);
	if(!m_sFileName.isEmpty()) {
		const char* pName;
		GetNameOfFile(m_sFileName, pName);
		s += QString(" - ") + pName;
	}
	if(m_bFileModified) {
		s += " *";
	}
	setCaption(s);
}
	
//-----------------------------------------------------------------------------------
void RFIWindow::DoInitWords()
{
	m_pMenuBar->changeItem(0, tr("&Fichier"));

	m_pFileMenu->changeItem(0, tr("&Nouveau"));
	m_pFileMenu->changeItem(1, tr("&Ouvrir..."));
	m_pFileMenu->changeItem(2, tr("&Sauver"));
	m_pFileMenu->changeItem(3, tr("S&auver Sous..."));
	m_pFileMenu->changeItem(4, CLanguage::GetWordTranslation(CLanguage::WO_SENDTO));
	
	m_pList->setColumnText(0, CLanguage::GetWordTranslation(CLanguage::WO_NAME));
	m_pList->setColumnText(1, tr("Zone"));
	m_pList->setColumnText(2, CLanguage::GetWordTranslation(CLanguage::WO_SIZE));

	m_qMenu.changeItem(0, tr("Ajouter un fichier"));
	m_qMenu.changeItem(1, CLanguage::GetWordTranslation(CLanguage::WO_IMPORT));
	m_qMenu.changeItem(2, tr("Importer des donnes textes"));
	m_qMenu.changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_EXPORT));
	m_qMenu.changeItem(4, CLanguage::GetWordTranslation(CLanguage::WO_RENAME));
	m_qMenu.changeItem(5, tr("Changer le numro de zone"));
	m_qMenu.changeItem(6, tr("Supprimer"));
}

//-----------------------------------------------------------------------------------
int RFIWindow::LoadFile(const QString& sFileName)
{
	if(!sFileName.isEmpty())
	{
		HND h = OpenDescModeRead(sFileName);
		if(h != INVALID_HANDLE_VALUE)
		{
			char header[4];
			ReadDesc(h, header, sizeof(header));
			if(QString(header) != rfi_header) {
				CloseDesc(h);
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(sFileName));
				return -1;
			}
			unsigned short nNbFile;
			ReadDesc(h, &nNbFile, sizeof(nNbFile));
			ClearFileList();
			RFI rfi;
			while(nNbFile) {
				ReadDesc(h, &rfi, sizeof(rfi));
				char* pData = new char[rfi.size];
				ReadDesc(h, pData, rfi.size);
				AddFileItem(rfi, pData);
				nNbFile--;
			}
			CloseDesc(h);
			SetFileName(sFileName);
			OnFileSaved();
			RefreshSaveButtons();
			m_pFileMenu->setItemEnabled(4, TRUE);
			return 1;
		}
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName));
		return -1;
	}
	SetFileName(sFileName);
	DoShowCaption();
	m_pFileMenu->setItemEnabled(4, !m_sFileName.isEmpty());
	return 1;
}

//-----------------------------------------------------------------------------------
int RFIWindow::SaveFile(const QString& sFileName)
{
	HND h = OpenDescModeRdWr(sFileName);
	if(h != INVALID_HANDLE_VALUE)
	{
		StopChecking();
		WriteDesc(h, rfi_header, sizeof(rfi_header));
		unsigned short nNbFile = m_mFile.size();
		WriteDesc(h, &nNbFile, sizeof(nNbFile));
		
		FileMap::iterator it  = m_mFile.begin();
		FileMap::iterator end = m_mFile.end();
		for(; it != end; ++it)
		{
			TFILE file = it.data();
			WriteDesc(h, &(file.rfi), sizeof(RFI));
			WriteDesc(h, file.data, file.rfi.size);
		}
		CloseDesc(h);
		SetFileName(sFileName);
		OnFileSaved();
		StartChecking();
		return 1;
	}
	CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sFileName, TRUE));
	return -1;
}

//-----------------------------------------------------------------------------------
void RFIWindow::Save()
{
	if(m_sFileName.isEmpty()) {
		SaveAs();
	}
	else {
		SaveFile(m_sFileName);
	}
}

//-----------------------------------------------------------------------------------
void RFIWindow::SaveAs()
{
	QString sName = CMessageBox::ChooseSaveFileName(QString::null, RFI_FILTER + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget(), "rfi");
	if(!sName.isEmpty()) {
		SaveFile(sName);
		m_pFileMenu->setItemEnabled(4, TRUE);
		emit StateChanged();
	}
}

//-----------------------------------------------------------------------------------
QListViewItem* RFIWindow::AddFileItem(const RFI& rfi, void* data)
{
	TFILE file;
	file.data = data;
	file.rfi  = rfi;
	QListViewItem* pItem = new QListViewItem(m_pList, rfi.name,
									QString::number(rfi.zone),
									QString::number(rfi.size));
	pItem->setRenameEnabled(0, TRUE);
	m_mFile[pItem] = file;
	return pItem;
}

//-----------------------------------------------------------------------------------
void RFIWindow::RefreshSaveButtons()
{
	m_pFileMenu->setItemEnabled(2, m_mFile.size());
	m_pFileMenu->setItemEnabled(3, m_mFile.size());
}

//-----------------------------------------------------------------------------------
void RFIWindow::RefreshPopup(QListViewItem* pItem)
{
	for(int i = 1; i < 7; i++) {
		m_qMenu.setItemEnabled(i, pItem != NULL);
	}
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnRightClick(QListViewItem* pItem, const QPoint& pos)
{
	RefreshPopup(pItem);
	m_qMenu.popup(pos);
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnPopupMenuSelected(int id)
{
	QListViewItem* pSelectedItem = m_pList->selectedItem();
	switch(id)
	{
	case 0://Nouveau
		{
			RFI rfi;
			strcpy(rfi.name, "FILE");
			rfi.zone = 1; rfi.size = 0;
			QListViewItem* pItem = AddFileItem(rfi, NULL);
			if(AskZone(pItem)) {
				pItem->startRename(0);
				RefreshSaveButtons();
			}
			else {
				m_pList->takeItem(pItem);
				delete pItem;
			}
		}
		break;
	case 1://Importer
		ImportData(pSelectedItem);
		break;
	case 2://Importer Sources
		{
			QString s = QInputDialog::getText(caption(), tr("Entrez une chane de caractre ou un tableau C/C++.\nSyntaxe : \"MACHAINE\" ou (TYPE){1,2,...} (TYPE = char, short, int ou long)")
						, QLineEdit::Normal, QString::null, NULL, GetMainWidget());
			QRegExp rx("^\"([^\"]*)\"");
			if(s.isEmpty())	{
				return;
			}
			if(rx.search(s) != -1 )
			{
				QString sText = rx.cap( 1 );
				unsigned int nSize = sText.length() + 1;
				char* buf = new char[nSize];
				memcpy(buf, sText.latin1(), nSize);
				FileMap::iterator it = m_mFile.find(pSelectedItem);
				ChangeData(it, buf, nSize);
			}
			else
			{
				rx.setPattern("^\\((int|char|short|long)\\)");
				QString sSynError = tr("Erreur de syntaxe : %1");
				if(rx.search(s) != -1 )
				{
					int nSize = 2;
					QString sType = rx.cap( 1 );
					if(sType == "char")
					{
						nSize = 1;
					}
					else if(sType == "long")
					{
						nSize = 4;
					}
					rx.setPattern("\\{[\\s\t]*(\\d+)(?:[\\s\t]*,[\\s\t]*(\\d+))*\\}");
					QValueList<int> lValues;
					if(rx.search(s) != -1)
					{
						int i;
						for(i = 1; rx.cap(i) != QString::null; i++)
						{
							lValues.push_back(rx.cap(i).toInt());
						}
						QValueList<int>::iterator it  = lValues.begin();
						QValueList<int>::iterator end = lValues.end();
						nSize *= lValues.size();
						char* buf = new char[nSize];
						for(i = 0; it != end; ++it)
						{
							if(sType == "char")
							{
								buf[i] = (char)*it;
							}
							else if(sType == "short" || sType == "int")
							{
								*((short*)buf+i) = (short)*it;
							}
							else if(sType == "long")
							{
								*((long*)buf+i) = (long)*it;
							}
							i++;
						}
						FileMap::iterator itSelectedItem = m_mFile.find(pSelectedItem);
						ChangeData(itSelectedItem, buf, nSize);
					}
					else
					{
						//erreur de syntaxe
						CMessageBox::MsgCritical(GetMainWidget(), sSynError.arg(tr("Tableau de valeur incorrect.")));
					}
				}
				else
				{
					//erreur de syntaxe
					CMessageBox::MsgCritical(GetMainWidget(), sSynError.arg(tr("Type non valide.")));
				}
			}
		}
		break;
	case 3://Exporter
		ExportData(pSelectedItem);
		break;
	case 4://Renommer
		if(pSelectedItem)
		{
			pSelectedItem->startRename(0);
		}
		break;
	case 5://Changer zone
		if(AskZone(pSelectedItem))
		{
			OnFileModified();
		}
		break;
	case 6://Supprimer
		DeleteFileItem(pSelectedItem);
		OnFileModified();
		RefreshSaveButtons();
		break;
	}
}

//-----------------------------------------------------------------------------------
bool RFIWindow::AskZone(QListViewItem* pItem)
{
	if(pItem)
	{
		FileMap::iterator it = m_mFile.find(pItem);
		if(it != m_mFile.end())
		{
			bool bOk;
			QStringList lZones;
			for(int nZone = 1; nZone < 16; nZone++)
			{
				lZones.push_back(QString::number(nZone));
			}
			QString sZone = QInputDialog::getItem(CAPTION, tr("Choisissez le numro de zone"), lZones, it.data().rfi.zone - 1, FALSE, &bOk, GetMainWidget());
			if(bOk)
			{
				it.data().rfi.zone = (char)sZone.toShort();
				unsigned int nMaxSize = GetMaxSize(it.data().rfi);
				if(it.data().rfi.size > nMaxSize)
				{
					CMessageBox::MsgInformation(GetMainWidget(), tr("La taille du fichier '%1' a t rduit  la limite qui est de %2 octets.")
						.arg(it.data().rfi.name)
						.arg(nMaxSize));
					it.data().rfi.size = nMaxSize;
				}
				
				UpdateItem(it);
				return true;
			}
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool RFIWindow::ImportData(QListViewItem* pItem)
{
	if(pItem)
	{
		FileMap::iterator it = m_mFile.find(pItem);
		if(it != m_mFile.end())
		{
			QString sName = CMessageBox::ChooseOpenFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget());
			if(!sName.isEmpty())
			{
				HND h = OpenDescModeRead(sName);
				if(h == INVALID_HANDLE_VALUE)
				{
					CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName));
					return false;
				}
				unsigned int nSize = FileSize(h);
				unsigned int nMaxSize = GetMaxSize(it.data().rfi);
				if(nSize > nMaxSize)
				{
					CMessageBox::MsgInformation(GetMainWidget(), tr("La taille du fichier '%1' dpasse la limite qui est de %2 octets.\nLa taille du fichier cr sera gale  la limite.")
						.arg(sName)
						.arg(nMaxSize));
					nSize = nMaxSize;
				}
				char* buf = new char[nSize];
				ReadDesc(h, buf, nSize);
				ChangeData(it, buf, nSize);
				CloseDesc(h);
				return true;
			}
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
void RFIWindow::ChangeData(FileMap::iterator& it, void* pData, unsigned int nSize)
{
	if(it != m_mFile.end())
	{
		it.data().rfi.size = nSize;
		delete[] it.data().data;
		it.data().data = pData;
		UpdateItem(it);
		OnFileModified();
	}
}

//-----------------------------------------------------------------------------------
bool RFIWindow::ExportData(QListViewItem* pItem)
{
	if(pItem)
	{
		FileMap::iterator it = m_mFile.find(pItem);
		if(it != m_mFile.end())
		{
			QString sName = CMessageBox::ChooseSaveFileName(QString::null, CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget());
			if(!sName.isEmpty())
			{
				HND h = OpenDescModeRdWr(sName);
				if(h == INVALID_HANDLE_VALUE)
				{
					CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName, TRUE));
					return false;
				}
				WriteDesc(h, it.data().data, it.data().rfi.size);
				CloseDesc(h);
			}
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
void RFIWindow::UpdateItem(FileMap::iterator& it)
{
	if(it != m_mFile.end())
	{
		QListViewItem* pItem = it.key();
		pItem->setText(0, it.data().rfi.name);
		pItem->setText(1, QString::number(it.data().rfi.zone));
		pItem->setText(2, QString::number(it.data().rfi.size));
	}
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnItemRenamed(QListViewItem* pItem)
{
	if(pItem)
	{
		FileMap::iterator it = m_mFile.find(pItem);
		if(it != m_mFile.end())
		{
			QString s(pItem->text(0));
			s.truncate(8);
			strcpy(it.data().rfi.name, s);
			pItem->setText(0, s);
			OnFileModified();
		}
	}
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnFileModified()
{
	m_bFileModified = true;
	DoShowCaption();
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnFileSaved()
{
	m_bFileModified = false;
	DoShowCaption();
}

//-----------------------------------------------------------------------------------
unsigned int RFIWindow::GetMaxSize(const RFI& rfi)
{
	if(rfi.zone == 1) {
		return MAX_SIZE - 24;
	}
	return MAX_SIZE - 14;
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnMenuActivated(int id)
{
	switch(id)
	{
	case 0://Nouveau
		if(CloseFile() == 1) {
			SetFileName(QString::null);
			ClearFileList();
			OnFileSaved();
		}
		break;
	case 1://Charger
		if(CloseFile() == 1) {
			QString sName = CMessageBox::ChooseOpenFileName(QString::null, RFI_FILTER + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget());
			if(!sName.isEmpty()) {
				LoadFile(sName);
			}
		}
		break;
	case 2://Sauver
		Save();
		break;
	case 3://Sauver sous
		SaveAs();
		break;
	}
}

//-----------------------------------------------------------------------------------
void RFIWindow::OnSendTo()
{
	GetMainWindow()->GetSendToMenu()->SetFileToSend(m_sFileName);
}

//-----------------------------------------------------------------------------------
int	RFIWindow::DoReloadFile(bool bShowValidErr)
{
	return LoadFile(m_sFileName);
}

//-----------------------------------------------------------------------------------
int RFIWindow::LoadFrom(QDomElement& element)
{
	if(CheckFileWindow::LoadFrom(element) == 1) {
		return DoReloadFile(TRUE);
	}
	return -1;
}
