// RFIWindow.h: interface for the RFIWindow class.
//
//////////////////////////////////////////////////////////////////////

#ifndef RFIWINDOW_H
#define RFIWINDOW_H

#include <CheckFileWindow.h>

#include <qpushbutton.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qmap.h>
#include <qmenubar.h>

class RFIWindow : public CheckFileWindow 
{
	Q_OBJECT
public:
	RFIWindow(CONSTRUCTOR_PARAMDEF);
	virtual ~RFIWindow();

	int  Init(INIT_PARAMDEF);
	int  LoadFrom(QDomElement& element);

protected:
	int  LoadFile(const QString& sFileName);
	int  SaveFile(const QString& sFileName);
	int  CloseFile();
	void DoShowCaption();
	void DoInitWords();
	void RefreshSaveButtons();
	void ClearFileList();
	void DeleteFileItem(QListViewItem* pItem);
	bool AskZone(QListViewItem* pItem);
	bool ImportData(QListViewItem* pItem);
	bool ExportData(QListViewItem* pItem);
	void OnFileModified();
	void OnFileSaved();
	void Save();
	void SaveAs();
	int	 DoReloadFile(bool bShowValidErr);

protected:
	/** From Qt */
	void closeEvent(QCloseEvent* e);

private slots:
	void RefreshPopup(QListViewItem*);
	void OnRightClick(QListViewItem*, const QPoint&);
	void OnPopupMenuSelected(int);
	void OnItemRenamed(QListViewItem*);
	void OnMenuActivated(int id);
	void OnSendTo();

private:
	struct RFI{
		char zone;
		char name[9];
		unsigned short size;
	};	
	struct TFILE{
		RFI rfi;
		void* data;
	};
	typedef QMap<QListViewItem*, TFILE> FileMap;

private:
	QListViewItem* AddFileItem(const RFI& rfi, void* data);
	void           UpdateItem(FileMap::iterator& it);
	unsigned int   GetMaxSize(const RFI& rfi);
	void           ChangeData(FileMap::iterator& it, void* pData, unsigned int nSize);
	
private:
	QListView*  m_pList;
	QMenuBar*   m_pMenuBar;
	QPopupMenu* m_pFileMenu;
	QPopupMenu  m_qMenu;
	FileMap     m_mFile;
	bool        m_bFileModified;
};

#endif
