// CasioVideoMaker.cpp: implementation of the CasioVideoMaker class.
//
//////////////////////////////////////////////////////////////////////

#include "CasioVideoMaker.h"

#include <qmenubar.h>
#include <qheader.h>
#include <qlayout.h>
#include <qtextstream.h>
#include <qvgroupbox.h>
#include <qcombobox.h>
#include <qfile.h>

#include <APIFunction.h>
#include <MessageBox.h>
#include <language.h>
#include <FileList.h>
#include <Label.h>
#include <image_defs.h>
#include <SendToMenu.h>

#define FILTER (CasioVideoMaker::tr("Projets (*.cvpj)")+ '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER))
#define EXT    "cvpj"

#define PROJECT_HEADER "[CVMAKERPRJ]"
#define PROJECT_END    "[END]"

//-----------------------------------------------------------------------------------
CasioVideoMaker::CasioVideoMaker(CONSTRUCTOR_PARAM)
:CToolWindow(CALL_PARAM),
 m_bProjectModified(FALSE)
{
	QWidget* pCentralWidget = new QWidget(0);
	//Dfini la fentre d'o seront insrs les objets graphiques
	SetCentralWidget(pCentralWidget);

	m_pMenuBar    = new QMenuBar(pCentralWidget);
	m_pFileMenu   = new QPopupMenu(pCentralWidget);
	m_pActionMenu = new QPopupMenu(pCentralWidget);
	m_pHelpMenu   = new QPopupMenu(pCentralWidget);
	m_pListMenu   = new QPopupMenu(pCentralWidget);

	m_pMenuBar->insertItem(QString::null, m_pFileMenu,   0);
	m_pMenuBar->insertItem(QString::null, m_pActionMenu, 1);
	m_pMenuBar->insertItem("&?",          m_pHelpMenu,   2);

	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_NEW),  QString::null, this, SLOT(OnNewProject()),  CTRL + Key_N, 0);
	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_OPEN), QString::null, this, SLOT(OnLoadProject()), CTRL + Key_O, 1);
	m_pFileMenu->insertItem(QPixmap::fromMimeSource(IMG_SAVE), QString::null, this, SLOT(OnSaveProject()), CTRL + Key_S, 2);
	m_pFileMenu->insertItem(QString::null, this, SLOT(close()),              ALT + Key_F4, 3);
	m_pFileMenu->insertSeparator(2);
	m_pFileMenu->insertSeparator(4);

	m_pActionMenu->insertItem(QString::null, 0);
	m_pActionMenu->insertItem(QString::null, 1);
	m_pActionMenu->insertSeparator();
	m_pActionMenu->insertItem(QString::null, 2);
	m_pActionMenu->insertSeparator();
	m_pActionMenu->insertItem(QPixmap::fromMimeSource(IMG_SENDTO), QString::null, GetMainWindow()->GetSendToMenu(), 3);

	m_pActionMenu->setItemEnabled(1, FALSE);
	
	m_pHelpMenu->insertItem(QString::null, this, SLOT(OnLinkClicked()), 0, 0);
	m_pHelpMenu->insertItem(QPixmap::fromMimeSource(IMG_HELP), QString::null, this, SLOT(OnShowAbout()),   0, 1);
	m_pHelpMenu->insertSeparator(1);

	m_pListMenu->insertItem(QString::null, 0);
	m_pListMenu->insertItem(QString::null, 1);
	m_pListMenu->insertItem(QString::null, 2);
	m_pListMenu->insertItem(QString::null, 3);
	m_pListMenu->insertSeparator(2);

	m_pListMenu->setAccel(CTRL + Key_A, 0);
	m_pListMenu->setAccel(Key_Delete,   1);
	m_pListMenu->setAccel(CTRL + Key_U, 2);
	m_pListMenu->setAccel(CTRL + Key_D, 3);

	QGridLayout* pLayout = new QGridLayout(pCentralWidget, 3, 2, 5);
	m_pViewsGroup  = new QVGroupBox(pCentralWidget);
	m_pViewerGroup = new QVGroupBox(pCentralWidget);
	m_pSpeedGroup  = new QVGroupBox(pCentralWidget);
	m_pList        = new CFileList(m_pViewsGroup, TRUE);
	m_pViewer      = new QWidget(m_pViewerGroup);
	m_pSizeLbl     = new CLabel(m_pViewsGroup);
	m_pSpeed       = new QComboBox(FALSE, m_pSpeedGroup);

	pLayout->setMenuBar(m_pMenuBar);
	pLayout->addMultiCellWidget(m_pViewsGroup, 0, 2, 0, 0);
	pLayout->addWidget(m_pViewerGroup,  0, 1, AlignVCenter);
	pLayout->addWidget(m_pSpeedGroup,   1, 1, AlignVCenter);

	m_pList->addColumn(QString::null);
	m_pList->header()->hide();
	m_pList->setSorting(-1);

	m_pViewer->setFixedSize(128, 64);

	m_pSizeLbl->SetDefaultStyle();

	for(int i = 1; i <= 30; i++) {
		m_pSpeed->insertItem(QString::number(i));
	}
	
	//Connexions
	connect(m_pList, SIGNAL(Dropped(const QStringList&, QListViewItem*)), SLOT(OnFilesDropped(const QStringList&, QListViewItem*)));
	connect(m_pList, SIGNAL(selectionChanged()), SLOT(OnRefreshViewer()));
	connect(m_pList, SIGNAL(selectionChanged()), SLOT(OnRefreshListMenu()));
	connect(m_pList, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), SLOT(OnMenuRequested(QListViewItem*, const QPoint&)));
	connect(m_pListMenu, SIGNAL(activated(int)), SLOT(OnListMenuActivated(int)));
	connect(m_pActionMenu, SIGNAL(activated(int)), SLOT(OnAction(int)));
	connect(m_pActionMenu, SIGNAL(aboutToShow()), SLOT(OnSendTo()));
	connect(&m_qTimer, SIGNAL(timeout()), SLOT(OnTimerDone()));
	connect(m_pSpeed, SIGNAL(activated(int)), SLOT(OnSpeedChanged()));

	//Demande de mettre  jour les mots  traduire
	languageChange();
	RefreshSizeDisplay();
}

//-----------------------------------------------------------------------------------
int CasioVideoMaker::Init(INIT_PARAM)
{
	return LoadProject(sFileName);
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::DoShowCaption()
{
	QString s("Casio Afx2 Video Maker");
	if(m_sProjectName)
	{
		const char* pName;
		GetNameOfFile(m_sProjectName, pName);
		s += QString(" - ") + pName;
		if(m_bProjectModified)
		{
			s += "*";
		}
	}
	setCaption(s);
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::DoInitWords()
{
	m_pMenuBar->changeItem(0, tr("&Fichier"));
	m_pMenuBar->changeItem(1, tr("&Actions"));

	m_pFileMenu->changeItem(0, tr("&Nouveau Projet"));
	m_pFileMenu->changeItem(1, tr("&Ouvrir un Projet"));
	m_pFileMenu->changeItem(2, tr("&Sauver le Projet"));
	m_pFileMenu->changeItem(3, tr("&Quitter"));

	m_pActionMenu->changeItem(0, tr("Voir la vido"));
	m_pActionMenu->changeItem(1, tr("Arrter la vido"));
	m_pActionMenu->changeItem(2, tr("Assembler..."));
	m_pActionMenu->changeItem(3, CLanguage::GetWordTranslation(CLanguage::WO_SENDTO));

	m_pHelpMenu->changeItem(0, tr("Site &Na-Prod"));
	m_pHelpMenu->changeItem(1, tr("&A propos de Casio Afx2 Video Maker"));

	m_pListMenu->changeItem(0, tr("&Ajouter des vues..."));
	m_pListMenu->changeItem(1, tr("&Supprimer"));
	m_pListMenu->changeItem(2, tr("&Monter"));
	m_pListMenu->changeItem(3, tr("&Descendre"));

	m_pViewsGroup->setTitle(tr("Vues"));
	m_pViewerGroup->setTitle(tr("Aperu"));
	m_pSpeedGroup->setTitle(tr("Vitesse"));
	
	RefreshSizeDisplay();
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnLinkClicked()
{
	LaunchNavigator("www.na-prod.fr.st", TRUE);
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnShowAbout()
{
	CMessageBox::MsgInformation(GetMainWidget(), tr("Casio Video Maker pour Graph100(+) par Na-Prod.\nCr par Superna avec l'aide de Whyp et Dada66.\nNa-Prod Software 2004."));
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnNewProject()
{
	if(AskToSaveProject())
	{
		if(!m_sProjectName.isEmpty())
		{
			emit StateChanged();
		}
		m_bProjectModified = FALSE;
		m_sProjectName     = QString::null;
		m_pList->clear();
		OnRefreshViewer();
		DoShowCaption();
		RefreshSizeDisplay();
	}
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnLoadProject()
{
	if(AskToSaveProject())
	{
		QString sProjectName = CMessageBox::ChooseOpenFileName(m_sProjectName, FILTER, GetMainWidget());
		LoadProject(sProjectName);
	}
}

//-----------------------------------------------------------------------------------
int CasioVideoMaker::LoadProject(const QString& sProjectName)
{
	if(!sProjectName.isEmpty())
	{
		QFile file(sProjectName);
		if(file.open(IO_ReadOnly | IO_Translate))
		{
			QTextStream stream(&file);
			QString s;
			s = stream.readLine();
			if(s != PROJECT_HEADER)
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoValidFile(sProjectName));
				return -1;
			}
			m_pList->clear();
			while(!file.atEnd())
			{
				s = stream.readLine();
				if(s != PROJECT_END)
				{
					AddView(s);
				}
				else
				{
					break;
				}
			}
			if(sProjectName != m_sProjectName)
			{
				m_sProjectName = sProjectName;
				emit StateChanged();
			}
			OnRefreshViewer();
			m_bProjectModified = FALSE;
			DoShowCaption();
			RefreshSizeDisplay();
		}
		else
		{
			CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sProjectName));
			return -1;
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnSaveProject()
{
	QString sProjectName = m_sProjectName;
	if(sProjectName.isEmpty())
	{
		QString sName = CMessageBox::ChooseSaveFileName(sProjectName, FILTER, GetMainWidget(), EXT);
		if(sName.isEmpty())	{
			return;
		}
		sProjectName = sName;
	}
	QFile file(sProjectName);
	if(file.open(IO_WriteOnly | IO_Truncate | IO_Translate))
	{
		file.writeBlock(PROJECT_HEADER, sizeof(PROJECT_HEADER)-1);
		file.putch('\n');
		QListViewItem* pItem = m_pList->firstChild();
		while(pItem)
		{
			file.writeBlock(pItem->text(0), pItem->text(0).length());
			file.putch('\n');			
			pItem = pItem->nextSibling();
		}
		file.writeBlock(PROJECT_END, sizeof(PROJECT_END)-1);
		file.putch('\n');
		
		if(m_sProjectName != sProjectName)
		{
			emit StateChanged();
		}
		m_sProjectName     = sProjectName;
		m_bProjectModified = FALSE;
		DoShowCaption();
	}
	else
	{
		CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sProjectName, TRUE));
	}
}

//-----------------------------------------------------------------------------------
bool CasioVideoMaker::AskToSaveProject()
{
	bool bClose = true;
	if(m_bProjectModified)
	{
		switch(CMessageBox::MsgYesNoCancel(this, tr("Voulez-vous sauver le projet avant de le fermer ?")))
		{
		case QMessageBox::Yes:
			//Sauve le projet
			OnSaveProject();
			break;
		case QMessageBox::Cancel:
			bClose = false;
		}
	}
	return bClose;
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::ProjectIsModified()
{
	m_bProjectModified = TRUE;
	DoShowCaption();
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::closeEvent(QCloseEvent* e)
{
	if(AskToSaveProject())
	{
		CToolWindow::closeEvent(e);
	}
	else
	{
		e->ignore();
	}
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnFilesDropped(const QStringList& sFiles, QListViewItem* pDroppedItem)
{
	for(QStringList::const_iterator it = sFiles.begin(); it != sFiles.end() && AddView(*it); ++it);
}

//-----------------------------------------------------------------------------------
bool CasioVideoMaker::AddView(const QString& sFile)
{
	QPixmap pixmap;
	if(!pixmap.load(sFile))
	{
		CMessageBox::MsgCritical(GetMainWidget(), tr("Impossible de charger le fichier '%1'.").arg(sFile));
		return false;
	}
	QListViewItem* pItem = new QListViewItem(m_pList, m_pList->lastItem(), sFile);
	ProjectIsModified();
	RefreshSizeDisplay();
	return true;
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnRefreshViewer()
{
	m_pViewer->setErasePixmap(GetImage(m_pList->selectedItem()));
}

//-----------------------------------------------------------------------------------
QImage CasioVideoMaker::GetImage(QListViewItem* pItem)
{
	QImage image;
	if(pItem)
	{
		image.load(pItem->text(0));
		image = image.scale(128, 64);
		image = image.convertDepth(1);
	}
	return image;
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnMenuRequested(QListViewItem* pItem, const QPoint& pos)
{
	OnRefreshListMenu();
	m_pListMenu->popup(pos);
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnRefreshListMenu()
{
	QListViewItem* pItem = m_pList->selectedItem();
	m_pListMenu->setItemEnabled(1, pItem);
	m_pListMenu->setItemEnabled(2, pItem && pItem->itemAbove());
	m_pListMenu->setItemEnabled(3, pItem && pItem->itemBelow());
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnListMenuActivated(int id)
{
	QListViewItem* pItem = m_pList->selectedItem();
	switch(id)
	{
	case 0:
		{
			QStringList lFiles = CMessageBox::ChooseOpenFileNames(CLanguage::GetWordTranslation(CLanguage::WO_INPUTIMAGEFILTER), QString::null, GetMainWidget());
			OnFilesDropped(lFiles, NULL);
		}
		break;
	case 1:
		if(pItem)
		{
			m_pList->takeItem(pItem);
			delete pItem;
			ProjectIsModified();
			RefreshSizeDisplay();
		}
		break;
	case 2:
		if(pItem && pItem->itemAbove())
		{
			QListViewItem* pAfter = pItem->itemAbove()->itemAbove();
			if(pAfter)
			{
				pItem->moveItem(pAfter);
			}
			else
			{
				m_pList->takeItem(pItem);
				m_pList->insertItem(pItem);
			}
			m_pList->setSelected(pItem, true);
		}			
		break;
	case 3:
		if(pItem && pItem->itemBelow())
		{
			pItem->moveItem(pItem->itemBelow());
		}			
		break;
	}
	OnRefreshListMenu();
}

//-----------------------------------------------------------------------------------
int	CasioVideoMaker::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1) {
		LoadProject(ConvertToFullPath(element.attribute("m_sProjectName")));
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int	CasioVideoMaker::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1)
	{
		element.setAttribute("m_sProjectName", ConvertToRelativePath(m_sProjectName));
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnAction(int id)
{
	switch(id)
	{
	case 0://Voir vido
	case 1://Stopper la vido
		{
			if(m_pList->childCount() == 0)
			{
				return;
			}
			bool bStart = id == 0;
			m_pActionMenu->setItemEnabled(0, !bStart);
			m_pActionMenu->setItemEnabled(1, bStart);
			m_pActionMenu->setItemEnabled(2, !bStart);
			m_pMenuBar->setItemEnabled(0,    !bStart);
			m_pMenuBar->setItemEnabled(2,    !bStart);
			m_pViewsGroup->setEnabled(!bStart);
			if(bStart)
			{
				QListViewItem* pItem = m_pList->selectedItem();
				if(!pItem)
				{
					m_pList->setSelected(m_pList->firstChild(), TRUE);
				}
				m_qTimer.start(GetSpeed());
			}
			else
			{
				m_qTimer.stop();
			}
		}
		break;
	case 2://Crer le CSV
		{		
			QString sName = CMessageBox::ChooseSaveFileName(QString::null, tr("Fichier Vido (*.csv)") + '\n' + CLanguage::GetWordTranslation(CLanguage::WO_ALLFILEFILTER), GetMainWidget(), "csv");
			if(sName.isEmpty()) {
				return;
			}
			QFile file(sName);
			if(file.open(IO_Truncate | IO_WriteOnly))
			{
				CasioVideoHeader header;
				memcpy(header.IDENT, "CSV", 3);
				memset(header.Reserved, 0, sizeof(header.Reserved));
				header.NbViews   = m_pList->childCount();
				header.Version   = 1;
				header.VideoMode = 0xF3;
				header.Speed     = m_pSpeed->currentItem();	
				file.writeBlock((const char*)&header, sizeof(header));
				QListViewItem* pItem = m_pList->firstChild();
				char video[1024];
				while(pItem)
				{
					QImage image = GetImage(pItem).convertBitOrder(QImage::BigEndian);
					if(!image.isNull())
					{
						char* pBits = (char*)image.bits();
						for(int i = 0; i < 1024; i++)
						{
							video[i] = pBits[1023 - i];
						}
					}
					else
					{
						memset(video, 0, sizeof(video));
					}
					file.writeBlock(video, sizeof(video));
					pItem = pItem->nextSibling();
				}
			}
			else
			{
				CMessageBox::MsgCritical(GetMainWidget(), CLanguage::GetMsgNoOpenFile(sName, TRUE));
			}
		}
		break;
	}
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnTimerDone()
{
	QListViewItem* pItem = m_pList->selectedItem()->itemBelow();
	if(pItem == NULL)
	{
		pItem = m_pList->firstChild();
	}
	m_pList->setSelected(pItem, TRUE);
	m_pList->ensureItemVisible(pItem);
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::RefreshSizeDisplay()
{
	m_pSizeLbl->setText(tr("Nombre de vues : %1. Taille : %2 octets")
		.arg(m_pList->childCount())
		.arg(m_pList->childCount() * 1024 + 10));
}

//-----------------------------------------------------------------------------------
int CasioVideoMaker::GetSpeed()
{
	//Bas sur DirXplor (a peu prs :))
	return 1200 - 40*m_pSpeed->currentItem();
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnSpeedChanged()
{
	if(m_qTimer.isActive()) {
		m_qTimer.changeInterval(GetSpeed());
	}
}

//-----------------------------------------------------------------------------------
void CasioVideoMaker::OnSendTo()
{
	GetMainWindow()->GetSendToMenu()->SetFileToSend(m_sProjectName);
}
