// CasioVideoMaker.h: interface for the CasioVideoMaker class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CASIOVIDEOMAKER_H
#define CASIOVIDEOMAKER_H

#include <qtimer.h>
#include <ToolWindow.h>
#include <qimage.h>

class QPopupMenu;
class QMenuBar;
class QListView;
class QListViewItem;
class QGroupBox;
class CLabel;
class QComboBox;

class CasioVideoMaker : public CToolWindow
{
	Q_OBJECT
public:
	CasioVideoMaker(CONSTRUCTOR_PARAMDEF);
	bool   AddView(const QString& sFile);
	int    Init(INIT_PARAMDEF);
	int	   LoadFrom(QDomElement& element);
	int	   SaveTo(QDomElement& element);

protected:
	bool   AskToSaveProject();
	void   ProjectIsModified();
	int    LoadProject(const QString& sProjectName);
    QImage GetImage(QListViewItem* pItem);
	void   RefreshSizeDisplay();
	int    GetSpeed();

	void   DoShowCaption();
	void   DoInitWords();
	/** From Qt */
	void   closeEvent(QCloseEvent* e);

protected:
	struct CasioVideoHeader{
		char IDENT[3];
		unsigned char Version;
		unsigned char VideoMode;
		unsigned char NbViews;
		unsigned char Speed;
		char Reserved[3];
	};

protected slots:
	/** Les Slots permettent de rcuprer un vnement (SIGNAL sous Qt)*/
	void    OnLinkClicked();
	void    OnShowAbout();
	void    OnNewProject();
	void    OnLoadProject();
	void    OnSaveProject();
	void    OnFilesDropped(const QStringList&, QListViewItem*);
	void    OnRefreshViewer();
	void    OnRefreshListMenu();
	void    OnMenuRequested(QListViewItem*, const QPoint&);
	void    OnListMenuActivated(int);
	void    OnAction(int);
	void    OnTimerDone();
	void    OnSpeedChanged();
	void    OnSendTo();

private:
	QMenuBar*   m_pMenuBar;
	QPopupMenu* m_pFileMenu;
	QPopupMenu* m_pActionMenu;
	QPopupMenu* m_pHelpMenu;
	QPopupMenu* m_pListMenu;
	QListView*  m_pList;
	QWidget*    m_pViewer;
	QGroupBox*  m_pViewsGroup;
	QGroupBox*  m_pViewerGroup;
	QGroupBox*  m_pSpeedGroup;
	CLabel*     m_pSizeLbl;
	QComboBox*  m_pSpeed;
	bool        m_bProjectModified;
	QString     m_sProjectName;
	QTimer      m_qTimer;
};

#endif
