#include "rflmainwnd.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <qsplitter.h>
#include <qvbox.h>

#ifndef F100_PLUGIN
#	include <qsettings.h>
#else
#	include <MessageBox.h>
#	include <image_defs.h>
#endif

#include "slottable.h"
#include "contentedit.h"
//#ifdef Q_OS_UNIX
//#include "myRFhdlr.h"
//#else
#include "../mylibs/myRFhdlr.h"
//#endif

#define FILTER  RflMainWnd::tr("RFL projects (*.rfp);;All files (*)")

QRect icoRects[] = {
	QRect(0,0,16,16),
	QRect(20,0,14,14),
	QRect(38,0,16,13),
	QRect(57,0,14,13),
	QRect(74,0,14,13),
	QRect(92,0,14,13),
	QRect(110,0,16,14),
	QRect(130,0,15,14) };
	
QIconSet makeIconSet(int num)
{
#ifdef F100_PLUGIN
	if(num == ICO_NEW) {
		return QPixmap::fromMimeSource(IMG_NEW);
	}
	else if(num == ICO_OPEN) {
		return QPixmap::fromMimeSource(IMG_OPEN);
	}
	else if(num == ICO_SAVE) {
		return QPixmap::fromMimeSource(IMG_SAVE);
	}
#endif
	QRect rect = icoRects[num];
	QPixmap p(rect.size());
	QPixmap image = QPixmap::fromMimeSource(ICONS_FILE);

	copyBlt(&p, 0,0, &image, rect.left(), rect.top(), rect.width(), rect.height());
	return QIconSet(p);
}

///////////////////////////////////////////////////////////////////////////////////////

RflMainWnd::RflMainWnd( QWidget* parent, QString arg )
    : QMainWindow( parent, "RflMainWnd" )
	, m_bHasStatusBar(FALSE)
{
#ifdef F100_PLUGIN
	SetStatusBar(TRUE);
#endif
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );
    QVBoxLayout* RflMainWndLayout = new QVBoxLayout( centralWidget(), 11, 6, "RflMainWndLayout"); 

    // GroupBox Generation
	m_genGrpBox = new QGroupBox( centralWidget(), "m_genGrpBox" );
    m_genGrpBox->setGeometry( QRect( 11, 300, 462, 90 ) );
    m_genGrpBox->setColumnLayout(0, Qt::Vertical );
    m_genGrpBox->layout()->setSpacing( 6 );
    m_genGrpBox->layout()->setMargin( 11 );
    m_genGrpBoxLayout = new QHBoxLayout( m_genGrpBox->layout() );
    m_genGrpBoxLayout->setAlignment( Qt::AlignTop );

    layout11 = new QVBoxLayout( 0, 0, 6, "layout11"); 

    m_targetLbl = new QLabel( m_genGrpBox, "m_targetLbl" );
    layout11->addWidget( m_targetLbl );

    m_tagLbl = new QLabel( m_genGrpBox, "m_tagLbl" );
    layout11->addWidget( m_tagLbl );
    m_genGrpBoxLayout->addLayout( layout11 );

    layout9 = new QVBoxLayout( 0, 0, 6, "layout9"); 

    layout8b = new QHBoxLayout( 0, 0, 6, "layout8b"); 

	m_targetEdit = new QLineEdit( m_genGrpBox, "m_targetEdit" );
    m_targetEdit->setMinimumHeight(23);
    layout8b->addWidget( m_targetEdit );
	m_pathBtn = new QPushButton("...", m_genGrpBox, "m_pathBtn");
    m_pathBtn->setMaximumWidth(30);
	layout8b->addWidget( m_pathBtn );
    layout9->addLayout( layout8b );

    layout8 = new QHBoxLayout( 0, 0, 6, "layout8"); 

    m_tagEdit = new QLineEdit( m_genGrpBox, "m_tagEdit" );
    m_tagEdit->setMinimumHeight(23);
    m_tagEdit->setMaxLength(3);
    layout8->addWidget( m_tagEdit );
    spacer1 = new QSpacerItem( 163, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout8->addItem( spacer1 );

    m_makeBtn = new QPushButton( m_genGrpBox, "m_makeBtn" );
	m_makeBtn->setIconSet( makeIconSet(ICO_MAKE) );
	m_makeBtn->setMinimumHeight( 26 );
    layout8->addWidget( m_makeBtn );
    layout9->addLayout( layout8 );
    m_genGrpBoxLayout->addLayout( layout9 );

    // GroupBox Datas
    dataGrpBox = new QGroupBox( centralWidget(), "dataGrpBox" );
    dataGrpBox->setColumnLayout(0, Qt::Vertical );
    dataGrpBoxLayout = new QVBoxLayout( dataGrpBox->layout() );
    dataGrpBoxLayout->setAlignment( Qt::AlignTop );

	QSplitter* split = new QSplitter(Qt::Vertical, dataGrpBox);
	dataGrpBox->layout()->add(split);
	split->setOpaqueResize(true);

    m_table = new SlotTable( split );

    layout7 = new QVBox(split,"layout7"); 
	layout7->layout()->setSpacing(6);
	layout7->layout()->setMargin(3);

    m_contentLbl = new QLabel( layout7, "m_contentLbl" );

    m_content = new ContentEdit( layout7 );

    RflMainWndLayout->addWidget( dataGrpBox );
    RflMainWndLayout->addWidget( m_genGrpBox );

    // actions
    fileNewAction = new QAction( this, "fileNewAction" );
    fileNewAction->setIconSet( makeIconSet(ICO_NEW) );
    fileOpenAction = new QAction( this, "fileOpenAction" );
    fileOpenAction->setIconSet( makeIconSet(ICO_OPEN) );
    fileSaveAction = new QAction( this, "fileSaveAction" );
    fileSaveAction->setIconSet( makeIconSet(ICO_SAVE) );
    fileSaveAsAction = new QAction( this, "fileSaveAsAction" );
    fileExitAction = new QAction( this, "fileExitAction" );
    helpAboutAction = new QAction( this, "helpAboutAction" );

	makeFileAction = new QAction(this, "makeFileAction");
    makeFileAction->setIconSet( makeIconSet(ICO_MAKE) );

    // toolbars
 //   toolBar = new QToolBar( QString(""), this, DockTop ); 

 //   fileNewAction->addTo( toolBar );
 //   fileOpenAction->addTo( toolBar );
 //   fileSaveAction->addTo( toolBar );
	//toolBar->addSeparator();

	//makeFileAction->addTo( toolBar );
	//toolBar->addSeparator();

    // menubar
    MenuBar = new QMenuBar( this, "MenuBar" );

    fileMenu = new QPopupMenu( this );
    fileNewAction->addTo( fileMenu );
    fileOpenAction->addTo( fileMenu );
    fileSaveAction->addTo( fileMenu );
    fileSaveAsAction->addTo( fileMenu );
			
#ifndef F100_PLUGIN
    fileMenu->insertSeparator();
	m_RFhandler = new MyRecentFilesHdlr(this, SOCIETY_NAME, PROGRAM_NAME);
	fileMenu->insertItem("", m_RFhandler, RECENTFILES_ID); 
#endif

    fileMenu->insertSeparator();
    fileExitAction->addTo( fileMenu );
    MenuBar->insertItem( "", fileMenu, 1 );

    editMenu = new QPopupMenu( this );
	makeFileAction->addTo( editMenu );
    MenuBar->insertItem( "", editMenu, 2 );

    helpMenu = new QPopupMenu( this );
    helpAboutAction->addTo( helpMenu );
    MenuBar->insertItem( "", helpMenu, 3 );


    // signals and slots connections
    connect( fileNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    connect( fileSaveAction, SIGNAL( activated() ), this, SLOT( fileSave() ) );
    connect( fileSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
    connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
    connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
    connect( makeFileAction, SIGNAL( activated() ), this, SLOT( makeFile() ) );
    connect( m_makeBtn, SIGNAL( clicked()   ), this, SLOT( makeFile() ) );
    connect( m_pathBtn, SIGNAL( clicked()   ), this, SLOT( choosePath() ) );
	connect( m_content, SIGNAL( focusLost() ), this, SLOT( checkContent() ) );
	connect( m_table, SIGNAL( editContent(QString, int) ), this, SLOT( setContent(QString, int) ) );
	connect( m_table, SIGNAL( modifMade() ), this, SLOT( notifyModif() ) );
#ifndef F100_PLUGIN
	connect( m_RFhandler, SIGNAL( fileSelected(QString) ), this, SLOT( loadFile(QString) ) );
#endif

    // tab order
    setTabOrder( m_table, m_content );
    setTabOrder( m_content, m_targetEdit );
    setTabOrder( m_targetEdit, m_tagEdit );
    setTabOrder( m_tagEdit, m_makeBtn );

    // buddies
    m_targetLbl->setBuddy( m_targetEdit );
    m_tagLbl->setBuddy( m_tagEdit );

	//initialisations

#ifndef F100_PLUGIN
	QSettings set;
	set.setPath(SOCIETY_NAME, PROGRAM_NAME);

	int w = set.readNumEntry("/Geometry/width",485);
	int h = set.readNumEntry("/Geometry/height",480);
    resize(w, h);
#endif

    clearWState( WState_Polished );
	m_modified = false;
	languageChange();

	if(arg.isEmpty()) fileNew();
	else loadFile(arg);
}

///////////////////////////////////////////////////////////////////////////////////////

RflMainWnd::~RflMainWnd()
{
	m_table->clearSlots();
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::languageChange()
{
    m_genGrpBox->setTitle( tr( "Generation" ) );
    m_targetLbl->setText( tr( "Target File:" ) );
    m_tagLbl->setText( tr( "Tag (3 chars):" ) );
    m_makeBtn->setText( tr( "Generate File" ) );
    dataGrpBox->setTitle( tr( "Datas" ) );
    m_contentLbl->setText( tr( "Content:" ) );

    fileNewAction->setText( tr( "New" ) );
    fileNewAction->setMenuText( tr( "&New" ) );
    fileNewAction->setAccel( tr( "Ctrl+N, F2" ) );
    fileOpenAction->setText( tr( "Open" ) );
    fileOpenAction->setMenuText( tr( "&Open..." ) );
    fileOpenAction->setAccel( tr( "Ctrl+O" ) );
    fileSaveAction->setText( tr( "Save" ) );
    fileSaveAction->setMenuText( tr( "&Save" ) );
    fileSaveAction->setAccel( tr( "Ctrl+S, F3" ) );
    fileSaveAsAction->setText( tr( "Save As" ) );
    fileSaveAsAction->setMenuText( tr( "Save &As..." ) );
    fileSaveAsAction->setAccel( QString::null );
    fileExitAction->setText( tr( "Exit" ) );
    fileExitAction->setMenuText( tr( "E&xit" ) );
    fileExitAction->setAccel( QString::null );
    helpAboutAction->setText( tr( "About RFL Maker..." ) );
    helpAboutAction->setMenuText( tr( "&About..." ) );
    helpAboutAction->setAccel( QString("F1") );
    //toolBar->setLabel( tr( "Tools" ) );
    if (MenuBar->findItem(1))
        MenuBar->findItem(1)->setText( tr( "&File" ) );
    if (MenuBar->findItem(2))
        MenuBar->findItem(2)->setText( tr( "&Edit" ) );
    if (MenuBar->findItem(3))
        MenuBar->findItem(3)->setText( tr( "&Help" ) );

    makeFileAction->setText( tr( "Create a RFL File" ) );
    makeFileAction->setMenuText( tr( "&Generate File" ) );
    makeFileAction->setAccel( tr( "F4" ) );

#ifndef F100_PLUGIN
	fileMenu->changeItem(RECENTFILES_ID, tr("&Recent files"));
#endif

	UpdateCaption();
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::UpdateCaption()
{
	setCaption( "RFL Maker - " + (m_currentFile.isEmpty()? QString("Untitled") :m_currentFile)
		+ (m_modified? "*":"") );
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::fileNew()
{
	bool ok = askForClose();
	if(ok==false) return;

#ifndef F100_PLUGIN
	if(m_currentFile.isEmpty()==false) m_RFhandler->addFile(m_currentFile);
#endif

	m_table->clearSlots();
	m_targetEdit->clear();
	m_content->clear();
	m_tagEdit->clear();
	m_currentRow = 0;
	m_modified = false;
	m_currentFile = QString::null;
	UpdateCaption();
	emit stateChanged();
}

///////////////////////////////////////////////////////////////////////////////////////

bool RflMainWnd::askForClose()
{
	if(m_modified==false) return true;

	QString msg = QString(tr("Save the changes made on file %1?")).
		arg( (m_currentFile.isEmpty()? QString::null : ("\n"+m_currentFile)) );
	
#ifndef F100_PLUGIN
	int x = QMessageBox::information(this, tr("Save data ?"), msg, tr("&Yes"), tr("&No"), tr("&Cancel"),0, 1 );

	if(x==0) fileSave();
	return (x!=2);
#else
	int r = CMessageBox::MsgYesNoCancel(this, tr("Save data ?"));
	if(r == QMessageBox::Yes) fileSave();
	return (r != QMessageBox::Cancel);
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::loadFile(QString fileName)
{
	if(fileName.isEmpty()) {
		return;
	}
	bool ok = askForClose();
	if(ok==false) return;

#ifndef F100_PLUGIN
	if(m_currentFile.isEmpty()==false) m_RFhandler->addFile(m_currentFile);
#endif
	QFile file( fileName );

	bool fichierOuvert = file.open(IO_ReadOnly);
	ok=false;

	if(fichierOuvert)
	{
		char tag[3];
		file.readBlock(tag,3);  // lecture du tag

		QDataStream stream(&file);

		QString s;
		Q_INT32 magic;
		Q_UINT32 n;

		stream >> magic;
		stream >> n;
		ok = (tag[0]=='R' && tag[1]=='F' && tag[2]=='P' && magic==MAGIC_STRING)
			&& (n == SIZE_RESERVE);
		if(ok) {
			char* reserve = new char[SIZE_RESERVE];
			stream.readRawBytes(reserve, SIZE_RESERVE);
			delete[] reserve;
			stream >> s; m_tagEdit->setText(s);
			stream >> n;
			stream >> s; m_targetEdit->setText(s);

			m_table->loadFromFile(stream,n-1);
		}
		else {
#		ifdef F100_PLUGIN
			CMessageBox::MsgExclamation(this, tr("Incorrect file format!"));
#		else
			QMessageBox::warning(this, "Error", tr("Incorrect file format!"));
#		endif
		}
	}

	message( 
		QString( ok ? tr("Data's successfully loaded from %1")
					: tr("Unable to load file %1!")).arg(fileName), 5000 );

	if(ok) 
	{
		m_currentFile = fileName;
		m_modified = false;
		UpdateCaption();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::fileOpen()
{
	QString s = QFileDialog::getOpenFileName(m_currentFile,
		FILTER, this, "file dialog", tr("Choose a project file to open..."));
	if(s.isEmpty()) return;

	loadFile(s);
}

///////////////////////////////////////////////////////////////////////////////////////

/*
Format RFP

- tag RFP (3 octets)
- "magic string"
- reserve (SIZE_RESERVE octets)
- tag perso (string)
- nbre slots, header compris (int)
- target file (string)
- pour chaque slot:
	- type (int)
	- content (string)
	- comments (string)
*/
void RflMainWnd::fileSave()
{
	if(m_currentFile.isEmpty()) { fileSaveAs(); return; }

	QFile file( m_currentFile );

	bool fichierOuvert = file.open(IO_WriteOnly);

	if(fichierOuvert)
	{
		char reserve[SIZE_RESERVE];
		file.writeBlock("RFP",3);  // ecriture du tag

		QDataStream stream(&file);

		stream << MAGIC_STRING;
		stream << SIZE_RESERVE;
		stream.writeRawBytes(reserve, SIZE_RESERVE);
		stream << m_tagEdit->text();
		stream << m_table->nbrSlots()+1;
		stream << m_targetEdit->text();

		m_table->saveIntoFile(stream);
	}

	message( 
		QString( fichierOuvert ? tr("The save was successful.")
					: tr("Unable to write file %1!")).arg(m_currentFile), 5000 );

	if(fichierOuvert)
	{
		m_modified = false;
		UpdateCaption();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::fileSaveAs()
{
#ifndef F100_PLUGIN
	QString filename = QFileDialog::getSaveFileName(
			    m_currentFile, FILTER, this,
			    "save as", tr("Save As..."));
#else
	QString filename = CMessageBox::ChooseSaveFileName(m_currentFile, FILTER, this, "rfp");
#endif

	if ( !filename.isEmpty() ) 
	{
		int answer = 0;
#	ifndef F100_PLUGIN
		//Code fait dans CMessageBox::ChooseSaveFileName()
		if( !filename.lower().endsWith(".rfp") ) filename += ".rfp";
		if ( QFile::exists( filename ) ) {
			answer = QMessageBox::warning(
					this, tr("Already existing file"),
					QString( tr("The file \'%1\' already exists!\nDo you want to replace it?") ).
					arg( filename ),
					tr("&Yes"), tr("&No"), QString::null, 1, 1 );
		}
#	endif
		if(answer == 0)
		{
			m_currentFile = filename;
			emit stateChanged();
			fileSave();
		}
    }
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::closeEvent(QCloseEvent* e)
{ 
	bool ok = askForClose();
	if(ok==false) return;

#ifndef F100_PLUGIN
	if(m_currentFile.isEmpty()==false) m_RFhandler->addFile(m_currentFile);
	QSettings set;
	set.setPath(SOCIETY_NAME, PROGRAM_NAME);

	set.writeEntry("/Geometry/width",width());
	set.writeEntry("/Geometry/height",height());
#endif
	e->accept();
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::helpAbout()
{
     QString text = tr(
			"<center>RFL Maker\n"
			"Version 1.00"
			"<p>The aim of this software is to create data files in a specific format (RFL), which can be directly loaded on a Casio Graph100/AFX calculator with the appropriate libraries."
			"<br>It may never be commercialized in any way.</p>"
			"<p>Feel free to send me any comments, bug reports or suggestions!</p>"
			"<p>Julien Patte, January 2005"
			"<br>orwell01@hotmail.com</p></center>");			;

	QMessageBox box( tr("About RFL Maker"),text,QMessageBox::NoIcon,0,0,0, this );
    box.exec();
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::notifyModif()
{
	if(m_modified) return;
	m_modified = true;
	UpdateCaption();
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::setContent(QString c, int row)
{
	checkContent();
	m_currentRow = row;
	m_content->setText(c);
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::checkContent()
{
	if(m_content->isModified()) 
	{
		m_table->updateContent(m_content->text(),m_currentRow);
		m_content->setModified(false);
		notifyModif();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::choosePath()
{
	QString s = QFileDialog::getSaveFileName(m_targetEdit->text(),
		"RFL files (*.rfl);;All files (*)", this, "file dialog", tr("Choose a file to make..."));
	if(s.isEmpty()) return;

	m_targetEdit->setText(s);
}

///////////////////////////////////////////////////////////////////////////////////////

#define TEST( cond, booleen ) booleen=(booleen&&(cond))

void RflMainWnd::makeFile()
{
	bool ok = m_table->okToMakeFile();

	if(ok == false)
	{
#	ifndef F100_PLUGIN
		QMessageBox::critical(this, tr("Error"),
			tr("Unable to make file since some slots have invalid content!"));
#	else
		CMessageBox::MsgCritical(this, tr("Unable to make file since some slots have invalid content!"));
#	endif
		return ;
	}

	bool fichierOuvert;
	QFile file( m_targetEdit->text() );

	ok = fichierOuvert = file.open(IO_WriteOnly);

	TEST( file.writeBlock("RFL",3)==3 , ok);  // ecriture du tag

	char tag[4] = "\0\0\0";
	int n = m_tagLbl->text().length();
	const char* sTag = m_tagEdit->text().ascii();
	for(int i=0; i<n; i++) tag[i] = sTag[i];

	TEST( file.writeBlock(tag,4)==4 , ok);

	TEST( m_table->makeFile(file), ok);

	if(fichierOuvert) file.close();

	message( 
		QString( ok ? tr("File successfully created at %1.")
					: tr("Unable to create file %1!")).arg(m_targetEdit->text()), 5000 );
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::message(const QString& msg, int ms)
{
	if(m_bHasStatusBar) {
		statusBar()->message(msg, ms);
	}
	else {
		emit Message(msg, ms);
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void RflMainWnd::SetStatusBar(bool bVisible)
{
	if(bVisible) {
		statusBar();
	}
	else {
		delete statusBar();
	}
	m_bHasStatusBar = bVisible;
}
