#include "slot.h"

#include <qregexp.h>

#include <qmessagebox.h>

Slot::Slot()
	: m_valid(true)
	, m_data(NULL)
	, m_size(0)
	, m_type(Char)
{
}

///////////////////////////////////////////////////////////////////////////////////////

Slot::~Slot()
{
	if(m_data) delete[] m_data;
}

///////////////////////////////////////////////////////////////////////////////////////

void Slot::setType(DataType t)
{
	if(m_type==t) return;

	if(t==String)
	{
		m_valid = true;
		m_size = (m_content.isEmpty() ? 0 : m_content.length()+1);
	}
	else if(m_type==String)
	{
		analyzeContent();
	}

	m_type = t;
}

///////////////////////////////////////////////////////////////////////////////////////

void Slot::setContent(QString c, DataType t)
{
	m_content = c;

	if(t==String)
	{
		m_valid = true;
		m_size = c.length()+1;
	}
	else analyzeContent();

	m_type = t;
}

///////////////////////////////////////////////////////////////////////////////////////

void Slot::analyzeContent()
{
	QRegExp rx("^" RX_BLANC "$");

	if(m_content.isEmpty() || rx.search(m_content)>-1)
	{
		m_valid = true;
		m_size = 0;
		if(m_data) delete[] m_data;
		m_data = NULL;
		return;
	}

	rx.setPattern(RX_EXPR);

	if(rx.search(m_content) != -1)
	{
		QRegExp rx_decp( "(" RX_NBR_DECP ")" );
		QRegExp rx_hex( "(" RX_NBR_HEX ")" );
		QValueList<int> data;
		int pos = 0, n;
		rx.setPattern( "(" RX_NOMBRE ")" );

		while ( pos >= 0 ) 
		{
			pos = rx.search( m_content, pos );
			if ( pos > -1 ) 
			{
				QString nbr = rx.cap( 1 );

				if(rx_hex.search(nbr) > -1) n = rx_hex.cap(1).toInt(0,16);
				else
				{
					rx_decp.search(nbr);
					QString s = rx_decp.cap(1);
					n = s.toInt(0,10);
					if(nbr.contains('-')) n = -n;
				}

				data.push_back( n );
				pos += rx.matchedLength();
			}
		}
		
		QValueList<int>::iterator it  = data.begin();
		QValueList<int>::iterator end = data.end();
		m_size = data.size();

		if(m_data) delete[] m_data;
		m_data = new int[m_size*sizeof(int)];

		for(int i=0; it != end; ++it) m_data[i++] = *it;

		m_valid = true;
	}
	else
	{
		m_valid = false;
		if(m_data) delete[] m_data;
		m_data = NULL;
		m_size = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

int convertData(int* data, int size, DataType t, char* &dst)
{
	int tabSize;
	int i;
	switch(t)
	{
	case Char:
		tabSize = size;
		dst = new char[tabSize];
		for(i=0; i<size; i++) dst[i] = (char)data[i];
		break;

	case Short: case Int:
		tabSize = size*2;
		dst = new char[tabSize];
		for(i=0; i<size; i++) *((short*)dst+i) = (short)data[i];
		break;

	case Long:
		tabSize = size*4;
		dst = new char[tabSize];
		for(i=0; i<size; i++) *((int*)dst+i) = data[i];
		break;
	}

	return tabSize;
}

///////////////////////////////////////////////////////////////////////////////////////

int Slot::formattedData(DataType t, char* &dst)
{
	int s;
	if(t!=String) s = convertData(m_data,m_size,t,dst);
	else
	{
		s = m_content.length() + 1;
		const char* tmp = m_content.ascii();
		dst = new char[s];
		for(int i=0; i<s; i++) dst[i] = tmp[i];
	}

	return s;
}

///////////////////////////////////////////////////////////////////////////////////////

