#include "slottable.h"

#include <qaction.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qtable.h>
#include <qpushbutton.h>
#include <qlabel.h>

#ifndef F100_PLUGIN
#	include <qsettings.h>
#else
#	include <MessageBox.h>
#endif

SlotTable::SlotTable(QWidget* parent)
	: QWidget(parent,"slottable")
{
	   // Layouts
	QVBoxLayout* mainLayout = new QVBoxLayout(this, 3, 3); 
    QHBoxLayout* topLayout = new QHBoxLayout(0, 0, 3); 

    // Titre
	m_title = new QLabel("", this);
    topLayout->addWidget( m_title );

	topLayout->addItem( new QSpacerItem( 120, 20, QSizePolicy::Expanding, QSizePolicy::Minimum ) );

	// Boutons
	m_addBtn = new QPushButton( makeIconSet(ICO_ADD), "", this);
	m_addBtn->setMaximumWidth(25);
    topLayout->addWidget( m_addBtn );

    m_deleteBtn = new QPushButton(makeIconSet(ICO_DEL), "", this);
	m_deleteBtn->setMaximumWidth(23);
    topLayout->addWidget( m_deleteBtn );

	m_moveUpBtn = new QPushButton(makeIconSet(ICO_UP), "", this);
	m_moveUpBtn->setMaximumWidth(23);
    topLayout->addWidget( m_moveUpBtn );

	m_moveDownBtn = new QPushButton(makeIconSet(ICO_DOWN), "", this);
	m_moveDownBtn->setMaximumWidth(23);
    topLayout->addWidget( m_moveDownBtn );

    mainLayout->addLayout( topLayout );

    // Table
	m_table = new QTable(1,3,this);
	m_table->verticalHeader()->setLabel( 0, tr( "Header" ) );
	m_table->setShowGrid( false );
    m_table->setSelectionMode( QTable::SingleRow );
    m_table->setFocusStyle( QTable::FollowStyle );
	m_table->setColumnReadOnly(COL_SIZE,true);
    mainLayout->addWidget( m_table );

	// Connexions
	connect(m_addBtn, SIGNAL( clicked() ), SLOT( addLine() ) );
	connect(m_deleteBtn, SIGNAL( clicked() ), SLOT( deleteLine() ) );
	connect(m_moveUpBtn, SIGNAL( clicked() ), SLOT( moveLineUp() ) );
	connect(m_moveDownBtn, SIGNAL( clicked() ), SLOT( moveLineDown() ) );
	connect(m_table, SIGNAL(currentChanged(int,int)), this, SLOT(lineSelected(int,int)));	
	connect(m_table, SIGNAL(valueChanged(int,int)), this, SLOT(saveNewValueAt(int,int)));	

    // Initialisations
	m_typeList << "char" << "short" << "int" << "long" << "string";
	
	setNewLine(0);
	
#ifndef F100_PLUGIN
	QSettings set;
	set.setPath(SOCIETY_NAME, PROGRAM_NAME);

	m_table->setColumnWidth(COL_TYPE, set.readNumEntry("/Geometry/col_type",55) );
	m_table->setColumnWidth(COL_SIZE, set.readNumEntry("/Geometry/col_size",40) );
	m_table->setColumnWidth(COL_COMMENTS, set.readNumEntry("/Geometry/col_comments",250) );
#endif
	m_slotList.append( new Slot() );

	m_slotList.setAutoDelete(true);

	languageChange();
}

///////////////////////////////////////////////////////////////////////////////////////

SlotTable::~SlotTable()
{
#ifndef F100_PLUGIN
	QSettings set;
	set.setPath(SOCIETY_NAME, PROGRAM_NAME);

	set.writeEntry("/Geometry/col_type", m_table->columnWidth(COL_TYPE) );
	set.writeEntry("/Geometry/col_size", m_table->columnWidth(COL_SIZE) );
	set.writeEntry("/Geometry/col_comments", m_table->columnWidth(COL_COMMENTS) );
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::languageChange()
{
	QHeader* h = m_table->horizontalHeader();
    h->setLabel( 0, tr( "Type" ) );
    h->setLabel( 1, tr( "Size" ) );
    h->setLabel( 2, tr( "Comments" ) );

	m_title->setText( tr( "Slots:" ) );
}

///////////////////////////////////////////////////////////////////////////////////////

int SlotTable::nbrSlots()
{
	return m_table->numRows()-1;
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::addLine()
{
	int n = m_table->numRows();
	m_table->setNumRows(n+1);
	setNewLine(n);
	m_slotList.append( new Slot() );
    m_table->verticalHeader()->setLabel( n, tr( QString("Slot %1").arg(n-1) ) );
	m_table->clearSelection();
	m_table->selectCells(n, 0, n, m_table->numCols()-1);
	m_table->setCurrentCell(n, m_table->currentColumn());
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::deleteLine()
{
	int i = m_table->currentRow(); if(i<1) return;

	QString msg = 
		QString(tr("Do you really want to delete this slot?"));
#ifndef F100_PLUGIN
	if(QMessageBox::warning(
				this,
				tr("Delete"),
				msg,
				tr("Yes"),
				tr("No"),
				QString::null,
				1,1) != 0) {
		return;
	}
#else
	if(!CMessageBox::MsgQuestion(this, msg)) {
		return;
	}
#endif

	m_table->removeRow(i);
	m_slotList.remove(i);

	m_table->clearSelection();
	m_table->selectCells(i, 0, i, m_table->numCols()-1);
	m_table->setCurrentCell(i, m_table->currentColumn());
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::moveLineUp()
{
	int i = m_table->currentRow(); if(i<2) return;

	m_table->swapRows(i,i-1);
	m_slotList.insert(i-1, m_slotList.take(i) );

	m_table->clearSelection();
	m_table->selectCells(i-1, 0, i-1, m_table->numCols()-1);
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::moveLineDown()
{
	int i = m_table->currentRow(); 
	if(i<1 || i==m_table->numRows()-1) return;

	m_table->swapRows(i,i+1);
	m_slotList.insert(i+1, m_slotList.take(i) );

	m_table->clearSelection();
	m_table->selectCells(i+1, 0, i+1, m_table->numCols()-1);
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::setNewLine(int num)
{
	QComboTableItem* i = new QComboTableItem(m_table,m_typeList,0);
	m_table->setItem(num, COL_TYPE, i);

	m_table->setText(num, COL_SIZE, "0");
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::updateContent(QString c, int row)
{
	Slot* s = m_slotList.at(row);
	s->setContent(c, (DataType)((QComboTableItem*)(m_table->item(row,COL_TYPE)))->currentItem());
	m_table->setText(row, COL_SIZE, s->isValid() ? QString::number(s->size()) : "???");
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::clearSlots()
{
	m_table->setNumRows(1);
	m_slotList.clear();
	m_slotList.append(new Slot());

	QComboTableItem* i = new QComboTableItem(m_table,m_typeList,0);
	m_table->setItem(0, COL_TYPE, i);
	m_table->setText(0, COL_SIZE, "0");
	m_table->setText(0, COL_COMMENTS, "");
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::lineSelected(int row, int)
{
	emit editContent( m_slotList.at(row)->content(), row );
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::saveNewValueAt(int row, int col)
{
	Slot* s = m_slotList.at(row);
	
	if(col==COL_TYPE) 
	{
		s->setType((DataType)((QComboTableItem*)(m_table->item(row,COL_TYPE)))->currentItem());
		m_table->setText(row, COL_SIZE, s->isValid() ? QString::number(s->size()) : "???");
	}

	emit modifMade();
}

///////////////////////////////////////////////////////////////////////////////////////

bool SlotTable::okToMakeFile()
{
	bool ok=true;

	for(QPtrList<Slot>::Iterator it = m_slotList.begin(); ok && it != m_slotList.end(); it++)
	{
		ok = (*it)->isValid();
	}

	return ok;
}

///////////////////////////////////////////////////////////////////////////////////////

#define TEST( cond, booleen ) booleen=(booleen&&(cond))

	//- tag ( "RFL" + 3 lettres au choix + '\0' )

	//- taille du header (2 octet)  ( = TAILLEP )
	//- header (TAILLEP octets) 

	//- nbre de blocs de donnes (2 octets) ( = NBRE )
	//- taille des donnees (2 octets) ( = TAILLED )
	//- NBRE fois: offsets des blocs sur 2 octets

	//- donnes (TAILLED octets)

bool SlotTable::makeFile(QFile &file)
{
	char* temp;
	bool ok = true;
	QPtrList<Slot>::Iterator it = m_slotList.begin();

	// Header
	DataType t = (DataType)((QComboTableItem*)(m_table->item(0,COL_TYPE)))->currentItem();

	int s = (*it)->formattedData(t, temp);
	TEST( file.writeBlock((char*)&s,2)==2 , ok);  // ecriture de la taille du header
	TEST( file.writeBlock(temp,s)==s , ok);  // ecriture du header
	delete[] temp;

	// Slots	
	int nbre = m_slotList.count()-1;
	SlotData* Slots = new SlotData[nbre];
	int curOffset = 0;
	SlotData d;

	int i=0;
	for(it++; ok && it != m_slotList.end(); it++)
	{
		t = (DataType)((QComboTableItem*)(m_table->item(i+1,COL_TYPE)))->currentItem();
		d.size = (*it)->formattedData(t, d.data);
		d.offset = curOffset;
		curOffset += d.size;
		
		Slots[i++] = d;
	}
	
	TEST( file.writeBlock((char*)&nbre,2)==2 , ok);			// nbre slots
	TEST( file.writeBlock((char*)&curOffset,2)==2 , ok);	// taille des donnees
	for(i=0; ok && i<nbre; i++)								// ecriture des offsets
	{
		TEST( file.writeBlock((char*)&(Slots[i].offset),2)==2 , ok);
	}

	for(i=0; ok && i<nbre; i++)								// ecriture de chaque slot
	{
		int t = Slots[i].size;
		TEST( file.writeBlock((char*)(Slots[i].data),t)==t , ok);
		delete[] Slots[i].data;
	}

	delete[] Slots;

	return ok;
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::saveIntoFile(QDataStream &stream)
{
	int i=0;
	for(QPtrList<Slot>::Iterator it = m_slotList.begin(); it != m_slotList.end(); it++)
	{
		stream << ((QComboTableItem*)(m_table->item(i,COL_TYPE)))->currentItem();
		stream << (*it)->content();
		stream << m_table->text(i,COL_COMMENTS);
		i++;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void SlotTable::loadFromFile(QDataStream &stream, int nbreSlots)
{
	nbreSlots++;
	m_table->setNumRows(nbreSlots);
	m_slotList.clear();

	QHeader*h = m_table->verticalHeader();

	for(int i=0; i<nbreSlots; i++)
	{
		Q_UINT32 tt;
		DataType t;
		stream >> tt; t = (DataType)tt;
		QComboTableItem* item = new QComboTableItem(m_table,m_typeList,0);
		item->setCurrentItem(t);
		m_table->setItem(i, COL_TYPE, item);

		if(i==0) h->setLabel(0,"Header");
		else h->setLabel(i,QString("Slot %1").arg(i-1));

		Slot* slot = new Slot();
		QString s;
		stream >> s;
		slot->setContent(s,t);
		m_slotList.append(slot);

		m_table->setText(i, COL_SIZE, slot->isValid()? QString::number(slot->size()) : "???");

		stream >> s;
		m_table->setText(i, COL_COMMENTS, s);
	}

	emit editContent(m_slotList.at(0)->content(),0);
}

///////////////////////////////////////////////////////////////////////////////////////
