#include <qtextedit.h>
#include <qtextstream.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qptrlist.h>
#include <qpixmap.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qfiledialog.h>
#include <qstringlist.h>
#include <qfileinfo.h>

#include "coder.h"
#include "sprite.h"
#include "config.h"
#include "config_f100.h"

#define WRITE( file, SrcAdd, size, test )	test=(test&&file.writeBlock((const char*)SrcAdd,size)==size);

void Coder::buildForm()
{
    resize( 600, 400 );
	
	QVBoxLayout* vlayout = new QVBoxLayout(this);

		// actions
	m_makeRflAction			= new QAction(this, "makeRflAction" );
	m_saveFileAction		= new QAction(this, "saveFileAction" );
	m_saveClipboardAction	= new QAction(this, "saveClipboardAction" );

	    // menubars
    m_menuBar = new QMenuBar( this, "MenuBar" );

    m_saveMenu = new QPopupMenu( this );
	m_saveFileAction->addTo( m_saveMenu );
    m_makeRflAction->addTo( m_saveMenu );
    m_saveClipboardAction->addTo( m_saveMenu );
    m_menuBar->insertItem( "&Enregistrer", m_saveMenu, 1 );

		// zone de texte
	textedit = new QTextEdit(this,"textedit");
	vlayout->addSpacing(m_menuBar->height()-2);
	vlayout->addWidget(textedit);

		// connexions
	connect( m_makeRflAction, SIGNAL(activated()), SLOT(makeRfl()) );
	connect( m_saveFileAction, SIGNAL(activated()), SLOT(saveFile()) );
	connect( m_saveClipboardAction, SIGNAL(activated()), SLOT(copyText()) );

	m_mode = m_pConfig->mode();
	m_lang = m_pConfig->language();
	m_curOffset = 0;
	m_list.setAutoDelete(true);
	languageChange();
}

///////////////////////////////////////////////////////////////////////////////////////

Coder::Coder(Config* pConfig, Tab &tab, QString name, bool coul5)
    : QDialog( NULL, "coder", false)
	, m_pConfig(pConfig)
{
	QString text;
	QTextOStream stream(&text);

	buildForm();
	codeImage(tab, name, coul5, stream);

	textedit->setText(text);
	textedit->setFont( QFont("Courier",8) );
}

///////////////////////////////////////////////////////////////////////////////////////

//Coder::Coder(QString nom, Tab tab, QWidget* parent )
Coder::Coder(Config* pConfig, Sprite* spr, QWidget* parent )
    : QDialog( parent, "coder", false)
	, m_pConfig(pConfig)
{
	QString text;
	QTextOStream stream(&text);

	buildForm();
	codeSprite(spr, stream);

	textedit->setText(text);
	textedit->setFont( QFont("Courier",8) );
}

///////////////////////////////////////////////////////////////////////////////////////

Coder::Coder(Config* pConfig, QPtrList<Sprite>* ptrlist, QWidget* parent, QString groupName )
    : QDialog( parent, "coder", false)
	, m_pConfig(pConfig)
{
	QString text;
	QTextOStream stream(&text);

	buildForm();

	QStringList nameList;
	int cpt=0;
	for(QPtrList<Sprite>::Iterator it = ptrlist->begin(); it!=ptrlist->end(); it++)
	{
		if( (*it)->isSelected() ) 
		{
			codeSprite(*it,stream);
			stream << "\n\n";
			nameList << (*it)->nom();
			cpt++;
		}
	}

	if(m_lang==1 && m_mode>1)
	{
		groupName = QFileInfo(groupName).baseName().replace(' ','_');
		stream << "unsigned char* " << groupName << "[] = {\n    ";
		for(int i=0; i<cpt-1; i++)	stream << nameList[i] << ",\n    ";
		stream << nameList.last() << "\n};";
	}

	textedit->setText(text);
	textedit->setFont( QFont("Courier",8) );
}

///////////////////////////////////////////////////////////////////////////////////////

Coder::~Coder()
{
	m_list.clear();
	delete m_saveMenu;
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::languageChange()
{
    setCaption( tr("Generated code","Caption of the window") );

	m_saveFileAction->setMenuText(tr("In a text file...","Item of \'Save\' Menu"));
	m_saveFileAction->setText(tr("Save in a text file","Item Description"));

	m_makeRflAction->setMenuText(tr("Make an RFL file","Item of \'Save\' Menu"));
	m_makeRflAction->setText(tr("Make an RFL file to load on AFX","Item Description"));

	m_saveClipboardAction->setMenuText(tr("In the clipboard...","Item of \'Save\' Menu"));
	m_saveClipboardAction->setText(tr("Copy the code in the clipboard","Item Description"));

    m_menuBar->changeItem(1, tr("&Save","Title of the \'Save\' Menu"));
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::codeSprite(Sprite* spr, QTextOStream &stream)
{
	int taille=0;
	QString nom=spr->nom();

	if(m_mode==NORMAL_BB || m_mode==NORMAL_WW)	// mode normal
	{
		uchar* codes[3];
		if(m_mode==0) taille = makeCodeNormalbb(spr->tableau(), codes);
		else taille = makeCodeNormalww(spr->tableau(), codes);

		writeCode(nom+"_m", codes[0], taille, stream);	// masque
		writeCode(nom+"_f", codes[1], taille, stream);	// couche avant
		writeCode(nom+"_b", codes[2], taille, stream);	// couche arrire

		m_list.append( new SprData(taille,codes[0],m_curOffset) );
		m_curOffset+=taille;
		m_list.append( new SprData(taille,codes[1],m_curOffset) );
		m_curOffset+=taille;
		m_list.append( new SprData(taille,codes[2],m_curOffset) );
		m_curOffset+=taille;
	}
	else
	{
		uchar* code;
		switch(m_mode)
		{
		case DRAWLIB_13:	taille = makeCodeDL13(spr->tableau(), code);				break;
		case DRAWLIB_14:	taille = makeCodeDL14(spr->tableau(), spr->origin(), code); break;
		case GXLIB_10:		taille = makeCodeGX10(spr->tableau(), spr->origin(), code); break;
		}
		writeCode(nom, code, taille, stream);

		m_list.append( new SprData(taille,code,m_curOffset) );
		m_curOffset+=taille;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

	// code une image 128*64
void Coder::codeImage(Tab &tab, QString name, bool coul5, QTextOStream &sortie)
{
	if(coul5)
	{
		uchar code[3*1024];
		makeCodeImage5Coul(tab,code);		
		writeCode(name, code, 3*1024, sortie);
	}
	else
	{
		uchar code[2*1024];
		makeCodeImage3Coul(tab,code);		
		writeCode(name+"_1", code, 1024, sortie);
		writeCode(name+"_2", code+1024, 1024, sortie);
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::saveFile()
{	
	int answer=0;
	QString filename = QFileDialog::getSaveFileName(
		QString::null, "Fichier au format texte (*.*)", this,
		QString::null, "Enregistrer le code gnr" );
	if(filename.isEmpty()) return;

	if ( QFile::exists( filename ) )
		answer = QMessageBox::warning(
				this, "Fichier dj existant",
				QString( "Le fichier \'%1\' existe dj! \nVoulez-vous le remplacer?" ).
				arg( filename ),
				"&Oui", "&Non", QString::null, 1, 1 );
	if ( answer == 0 ) 
	{
		QFile file(filename);
		file.open(IO_WriteOnly);
		QTextStream stream( &file );
		stream << textedit->text();
		file.close();
    }
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::copyText()
{ 
	qApp->clipboard()->setText(textedit->text());
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::makeRfl()
{
	int answer=0;
	QString filename = QFileDialog::getSaveFileName(
		QString::null, "Fichiers chargeables en RAM (*.rfl)", this,
		QString::null, "Crer un fichier RFL" );

	if(filename.isEmpty()) return;
	if(!filename.lower().endsWith(".rfl")) filename += ".rfl";

	if ( QFile::exists( filename ) )
		answer = QMessageBox::warning(
				this, "Fichier dj existant",
				QString( "Le fichier \'%1\' existe dj! \nVoulez-vous le remplacer?" ).
				arg( filename ),
				"&Oui", "&Non", QString::null, 1, 1 );
	
	if ( answer != 0 ) return;


	//- tag ( "RFL" + 3 lettres au choix + '\0' )

	//- taille du header (2 octet)  ( = TAILLEP )
	//- header (TAILLEP octets) 

	//- nbre de blocs de donnes (2 octets) ( = NBRE )
	//- taille des donnees (2 octets) ( = TAILLED )
	//- NBRE fois: offsets des blocs sur 2 octets

	//- donnes (TAILLED octets)
	
	short nbre = m_list.count();
	if(m_mode==0) nbre/=3;

	bool ok, fileOpened;
	QFile file(filename);
	ok = fileOpened = file.open(IO_WriteOnly);

	// TAG
	WRITE(file,"RFLspr",7,ok);	// tag

	// HEADER
	uchar header[5] = { m_mode };
	short s =5;
	WRITE(file,&s,2,ok);		// taille du header
	WRITE(file,header,s,ok);	// header

	// OFFSETS
	WRITE(file,&nbre,2,ok);			// nbre de blocs de donnees
	WRITE(file,&m_curOffset,2,ok);	// taille des donnees

	QPtrList<SprData>::iterator it = m_list.begin();
	for(; ok && it!=m_list.end(); it++)
	{
		short offset = (*it)->offset;
		if(m_mode==0)
		{
			it++;
			it++;
		}

		WRITE(file,&offset,2,ok);
	}

	// DATAS
	for(it = m_list.begin(); ok && it!=m_list.end(); it++)
	{
		if(m_mode==0)
		{
			WRITE(file,(*it)->code,(*it)->taille,ok);
			it++;
			WRITE(file,(*it)->code,(*it)->taille,ok);
			it++;
		}
		
		WRITE(file,(*it)->code,(*it)->taille,ok);
	}

	if(fileOpened) file.close();

	if(ok==false)
		QMessageBox::critical(this, "Erreur", 
			QString("L'enregistrement dans %1\na chou!").arg(filename) );

}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::writeCode(QString nom, uchar* tab, int taille, QTextOStream &sortie)
{
	if( m_lang == 1 ) // ecriture du code en mode cpp
	{
		// debut
		sortie << "unsigned char " << nom << "[] = {\n";
		
		//corps
		int compteur=0;
		for( int i=0; i<taille-1; i++ )
		{
			sortie << (int)tab[i] << ',';
			compteur++;
			if(compteur==16) 
			{
				compteur=0;
				sortie << '\n';
			}
		}

		// fin
		sortie << (int)tab[taille-1] << " };\n";
	}
	else	// ecriture en mode asm
	{
		// debut
		sortie << nom << ":\ndb ";
		
		//corps
		int compteur=0;
		for( int i=0; i<taille-1; i++ )
		{
			sortie << (int)tab[i];
			compteur++;
			if(compteur<16)	sortie << ',';
			else
			{
				compteur=0;
				sortie << "\ndb ";
			}
		}

		// fin
		sortie << (int)tab[taille-1] << "\n";
	}
}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode DrawLib 1.3
//int Coder::makeCodeDL13(Tab &tab, uchar* &le_code)
//{
//	short pxldebut=0;
//	uchar couche1, couche2, coul;
//	int i;
//	int largeur = tab.width(), hauteur = tab.height();
//	int l = largeur;
//		
//	if(l%8) l=l-l%8+8;		// si la largeur n'est pas multiple de 8
//
//	int taille = 1										// hauteur du sprite
//			+ (l/16)*(1+4*hauteur)			// blocs de 16 pixels
//			+ ((l%16)==8)*(1+2*hauteur)		// bloc de 8 pixel si necessaire
//			+ 1;			
//
//	le_code = new uchar[taille];
//	l = largeur;
//
//	le_code[0]=(uchar)hauteur;
//	le_code[1]=(uchar)largeur;
//	int offset=2;
//
//	do
//	{
//		if (l>8)	// bloc de 16 pixels a traiter si la largeur restante est >8
//		{
//			if (pxldebut) le_code[offset++]=16;
//			
//			for (int h=0;h<hauteur;h++)
//			{		
//				// 1ere couche
//				
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2;
//					coul = tab[h][pxldebut+i];
//					couche1 += ( coul==NOIR || coul==TRANS );	// noir ou transparent	
//	
//					couche2*=2;
//					coul = ( (l>8+i) ?  tab[h][pxldebut+8+i] : TRANS );
//					couche2 += ( coul==NOIR || coul==TRANS );	// noir ou transparent			
//				}
//			
//				le_code[offset++] = couche1;
//				le_code[offset++] = couche2;
//				
//				// 2eme couche
//				
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2;
//					coul = tab[h][pxldebut+i];
//					couche1 += ( coul==NOIR || coul==GRIS );		// noir ou gris	
//				
//					couche2*=2;
//					coul = ( (l>8+i) ? tab[h][pxldebut+8+i] : TRANS );
//					couche2 += ( coul==NOIR || coul==GRIS );		// noir ou gris	
//				}
//			
//				le_code[offset++] = couche1;
//				le_code[offset++] = couche2;
//			}	
//			
//			pxldebut+=16;
//			l-=16;
//		}
//		else 	// bloc de 8 pixels a traiter si la largeur restante est <=8
//		{
//			if (pxldebut) le_code[offset++] = 8;
//			
//			for (int h=0;h<hauteur;h++)
//			{
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2; couche2*=2;
//					coul = ( i<l ? tab[h][pxldebut+i] : TRANS);
//
//					couche1 += ( coul==NOIR || coul==TRANS ); // noir ou transparent
//					couche2 += ( coul==NOIR || coul==GRIS );		// noir ou gris					
//				}
//			
//				le_code[offset++] = couche1;
//				le_code[offset++] = couche2;
//			}
//
//			pxldebut+=8;
//			l-=8;
//		}
//	
//	} while ( pxldebut < largeur );
//
//	le_code[offset++] = 0;
//	return taille;
//}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode DrawLib 1.4
//int Coder::makeCodeDL14(Tab &tab, QPoint orig, uchar* &le_code)
//{
//	short pxldebut=0;
//	uchar couche1, couche2, coul;
//	int i;
//	int largeur = tab.width(), hauteur = tab.height();
//	int l = largeur;
//		
//	if(l%8) l=l-l%8+8;		// si la largeur n'est pas multiple de 8
//
//	int taille = 2							// origine
//			+ 1								// hauteur du sprite
//			+ (l/16)*(1+4*hauteur)			// blocs de 16 pixels
//			+ ((l%16)==8)*(1+2*hauteur)		// bloc de 8 pixel si necessaire
//			+ 1;			
//
//	le_code = new uchar[taille];
//	l = largeur;
//
//	le_code[0]=(uchar)orig.x();
//	le_code[1]=(uchar)orig.y();
//	le_code[2]=(uchar)hauteur;
//	le_code[3]=(uchar)largeur;
//	int offset=4;
//
//	do
//	{
//		if (l>8)	// bloc de 16 pixels a traiter si la largeur restante est >8
//		{
//			if (pxldebut) le_code[offset++]=16;
//			
//			for (int h=0;h<hauteur;h++)
//			{		
//				// 1ere couche
//				
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2;
//					coul = tab[h][pxldebut+i];
//					couche1 += ( coul==NOIR || coul==BLANC );	// noir ou blanc	
//	
//					couche2*=2;
//					coul = ( (l>8+i) ?  tab[h][pxldebut+8+i] : TRANS );
//					couche2 += ( coul==NOIR || coul==BLANC );	// noir ou blanc			
//				}
//			
//				le_code[offset++] = couche2; // inversion des octets pour viter xchg ah,al
//				le_code[offset++] = couche1;
//				
//				// 2eme couche
//				
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2;
//					coul = tab[h][pxldebut+i];
//					couche1 += ( coul==NOIR || coul==GRIS );		// noir ou gris	
//				
//					couche2*=2;
//					coul = ( (l>8+i) ? tab[h][pxldebut+8+i] : TRANS );
//					couche2 += ( coul==NOIR || coul==GRIS );		// noir ou gris	
//				}
//			
//				le_code[offset++] = couche2; // inversion des octets pour viter xchg ah,al
//				le_code[offset++] = couche1;
//			}	
//			
//			pxldebut+=16;
//			l-=16;
//		}
//		else 	// bloc de 8 pixels a traiter si la largeur restante est <=8
//		{
//			if (pxldebut) le_code[offset++] = 8;
//			
//			for (int h=0;h<hauteur;h++)
//			{
//				couche1=0; couche2=0;
//				for (i=0; i<8;i++)
//				{
//					couche1*=2; couche2*=2;
//					coul = ( i<l ? tab[h][pxldebut+i] : TRANS);
//
//					couche1 += ( coul==NOIR || coul==BLANC ); // noir ou blanc
//					couche2 += ( coul==NOIR || coul==GRIS );		// noir ou gris					
//				}
//			
//				le_code[offset++] = couche2;
//				le_code[offset++] = couche1;
//			}
//
//			pxldebut+=8;
//			l-=8;
//		}
//	
//	} while ( pxldebut < largeur );
//
//	le_code[offset++] = 0;
//	return taille;
//}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode Normal octet/octet
int Coder::makeCodeNormalbb(Tab &tab, uchar** les_codes)
{
	int i, p, offset=0, taille, hauteur=tab.height(), largeur=tab.width();
	uchar mask, front, back;
	taille = hauteur*(largeur/8+((largeur%8)!=0));

	les_codes[0] = new uchar[taille];
	les_codes[1] = new uchar[taille];
	les_codes[2] = new uchar[taille];

	for(i=0; i<hauteur; i++) // pour chaque ligne
	{
		p=0;
		while(p+8<=largeur)
		{
			mask = front = back = 0;
			for(int j=0; j<8; j++)
			{
				uchar coul = tab[i][p+j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}

			les_codes[0][offset] = mask;
			les_codes[1][offset] = front;
			les_codes[2][offset] = back;
			offset++;
			p+=8;
		}
		if(p<largeur)
		{
			mask = front = back = 0;
			for(int j=p; j<largeur; j++)
			{
				uchar coul = tab[i][j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}
				
			uchar sh = largeur-p;
			les_codes[0][offset] = (mask<<sh);
			les_codes[1][offset] = (front<<sh);
			les_codes[2][offset] = (back<<sh);
			offset++;
		}
	}

	return taille;
}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode Normal mot/mot
int Coder::makeCodeNormalww(Tab &tab, uchar** les_codes)
{
	int i, p, offset=0, taille, hauteur=tab.height(), largeur=tab.width();
	uchar mask, front, back;
	taille = hauteur*(largeur/8+((largeur%8)!=0));

	les_codes[0] = new uchar[taille];
	les_codes[1] = new uchar[taille];
	les_codes[2] = new uchar[taille];

	for(i=0; i<hauteur; i++) // pour chaque ligne
	{
		p=0;
		while(p+16<=largeur)
		{
			int j;
			mask = front = back = 0;
			for(j=0; j<8; j++)
			{
				uchar coul = tab[i][p+j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}

			les_codes[0][offset+1] = mask;
			les_codes[1][offset+1] = front;
			les_codes[2][offset+1] = back;

			mask = front = back = 0;
			for(; j<16; j++)
			{
				uchar coul = tab[i][p+j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}

			les_codes[0][offset] = mask;
			les_codes[1][offset] = front;
			les_codes[2][offset] = back;
			offset+=2;
			p+=16;
		}
		if(p+8<=largeur)
		{
			mask = front = back = 0;
			for(int j=0; j<8; j++)
			{
				uchar coul = tab[i][p+j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}

			les_codes[0][offset] = mask;
			les_codes[1][offset] = front;
			les_codes[2][offset] = back;
			offset++;
			p+=8;
		}
		if(p<largeur)
		{
			mask = front = back = 0;
			for(int j=p; j<largeur; j++)
			{
				uchar coul = tab[i][j];
				mask  = (mask<<1)  + (coul!=TRANS);
				front = (front<<1) + (coul==NOIR);
				back  = (back<<1)  + (coul==NOIR || coul==GRIS);
			}
				
			uchar sh = largeur-p;
			les_codes[0][offset] = (mask<<sh);
			les_codes[1][offset] = (front<<sh);
			les_codes[2][offset] = (back<<sh);
			offset++;
		}
	}

	return taille;
}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode DrawLib 1.3
int Coder::makeCodeDL13(Tab &tab, uchar* &le_code)
{
	short pxldebut=0;
	int i;
	int largeur = tab.width(), hauteur = tab.height();
	int l = largeur;
		
	if(l%8) l=l-l%8+8;	// si la largeur n'est pas multiple de 8

	int size = 2							// hauteur+fin du sprite
			+ (l/16)*(1+6*hauteur)			// blocs de 16 pixels
			+ ((l%16)==8)*(1+3*hauteur);	// bloc de 8 pixel si necessaire

	le_code = new uchar[size];

	l=largeur;
	le_code[0]=(uchar)hauteur;
	le_code[1]=(uchar)largeur;
	uchar* Code = le_code+2;

	do
	{
		if (l>8)	// bloc de 16 pixels a traiter si la largeur restante est >8
		{
			if (pxldebut) *(Code++)=16;
			
			for (int h=0;h<hauteur;h++)
			{		
				uchar oa1=0, oa2=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = tab[h][pxldebut+i];
					
					oa1 <<= 1; oa2 <<= 1;

					switch(coul)
					{
					case BLANC:		 								   break;
					case TRANS:		  oa1 += 1;						   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														oa2 += 1;	   break;
					case NOIR:	 	  oa1 += 1;			oa2 += 1;	   break;
					}
				}
			
				uchar ob1=0, ob2=0;

				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+8+i < largeur) ? tab[h][pxldebut+8+i] : TRANS);
					
					ob1 <<= 1; ob2 <<= 1;

					switch(coul)
					{
					case BLANC:		 								   break;
					case TRANS:		  ob1 += 1;						   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														ob2 += 1;	   break;
					case NOIR:	 	  ob1 += 1;			ob2 += 1;	   break;
					}
				}

				*(Code++) = oa1;
				*(Code++) = ob1;
				*(Code++) = oa2;
				*(Code++) = ob2;
			}	
			
			pxldebut+=16;
			l-=16;
		}
		else 	// bloc de 8 pixels a traiter si la largeur restante est <=8
		{
			if (pxldebut) *(Code++) = 8;
			
			for (int h=0;h<hauteur;h++)
			{
				uchar o1=0, o2=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+i < largeur) ? tab[h][pxldebut+i] : TRANS);
					
					o1 <<= 1; o2 <<= 1;

					switch(coul)
					{
					case BLANC:		 								   break;
					case TRANS:		  o1 += 1;						   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														o2 += 1;	   break;
					case NOIR:	 	  o1 += 1;			o2 += 1;	   break;
					}
				}
			
				*(Code++) = o1;
				*(Code++) = o2;
			}

			pxldebut+=8;
			l-=8;
		}
	
	} while ( pxldebut < largeur );

	*(Code++) = 0;
	return size;
}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode DrawLib 1.4
int Coder::makeCodeDL14(Tab &tab, QPoint orig, uchar* &code)
{
	short pxldebut=0;
	int i;
	int largeur = tab.width(), hauteur = tab.height();
	int l = largeur;
		
	if(l%8) l=l-l%8+8;	// si la largeur n'est pas multiple de 8

	int size = 4								// origine+hauteur+fin du sprite
			+ (l/16)*(1+6*hauteur)			// blocs de 16 pixels
			+ ((l%16)==8)*(1+3*hauteur);	// bloc de 8 pixel si necessaire

	code = new uchar[size];

	l=largeur;
	code[0]=(uchar)orig.x();
	code[1]=(uchar)orig.y();
	code[2]=(uchar)hauteur;
	code[3]=(uchar)largeur;
	uchar* Code = code+4;

	do
	{
		if (l>8)	// bloc de 16 pixels a traiter si la largeur restante est >8
		{
			if (pxldebut) *(Code++)=16;
			
			for (int h=0;h<hauteur;h++)
			{		
				uchar oa1=0, oa2=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = tab[h][pxldebut+i];
					
					oa1 <<= 1; oa2 <<= 1;

					switch(coul)
					{
					case BLANC:		  oa1 += 1;						   break;
					case TRANS:		 								   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														oa2 += 1;	   break;
					case NOIR:	 	  oa1 += 1;			oa2 += 1;	   break;
					}
				}
			
				uchar ob1=0, ob2=0;

				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+8+i < largeur) ? tab[h][pxldebut+8+i] : TRANS);
					
					ob1 <<= 1; ob2 <<= 1;

					switch(coul)
					{
					case BLANC:		  ob1 += 1;						   break;
					case TRANS:		 								   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														ob2 += 1;	   break;
					case NOIR:	 	  ob1 += 1;			ob2 += 1;	   break;
					}
				}

				*(Code++) = ob1;
				*(Code++) = oa1;
				*(Code++) = ob2;
				*(Code++) = oa2;
			}	
			
			pxldebut+=16;
			l-=16;
		}
		else 	// bloc de 8 pixels a traiter si la largeur restante est <=8
		{
			if (pxldebut) *(Code++) = 8;
			
			for (int h=0;h<hauteur;h++)
			{
				uchar o1=0, o2=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+i < largeur) ? tab[h][pxldebut+i] : TRANS);
					
					o1 <<= 1; o2 <<= 1;

					switch(coul)
					{
					case BLANC:		  o1 += 1;						   break;
					case TRANS:		 								   break;
					case GRIS_CLAIR: case GRIS:	case GRIS_FONCE:			   
														o2 += 1;	   break;
					case NOIR:	 	  o1 += 1;			o2 += 1;	   break;
					}
				}
			
				*(Code++) = o2;
				*(Code++) = o1;
			}

			pxldebut+=8;
			l-=8;
		}
	
	} while ( pxldebut < largeur );

	*(Code++) = 0;
	return size;
}

///////////////////////////////////////////////////////////////////////////////////////

	// codage du sprite en mode gxLib 1.0
int Coder::makeCodeGX10(Tab &tab, QPoint orig, uchar* &code)
{
	short pxldebut=0;
	int i;
	int largeur = tab.width(), hauteur = tab.height();
	int l = largeur;
		
	if(l%8) l=l-l%8+8;	// si la largeur n'est pas multiple de 8

	int size = 4								// origine+hauteur+fin du sprite
			+ (l/16)*(1+6*hauteur)			// blocs de 16 pixels
			+ ((l%16)==8)*(1+3*hauteur);	// bloc de 8 pixel si necessaire

	code = new uchar[size];

	l=largeur;
	code[0]=(uchar)orig.x();
	code[1]=(uchar)orig.y();
	code[2]=(uchar)hauteur;
	code[3]=(uchar)largeur;
	uchar* Code = code+4;

	do
	{
		if (l>8)	// bloc de 16 pixels a traiter si la largeur restante est >8
		{
			if (pxldebut) *(Code++)=16;
			
			for (int h=0;h<hauteur;h++)
			{		
				uchar oa1=0, oa2=0, oa3=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = tab[h][pxldebut+i];
					uchar trame = !((h+i)&1);
					
					oa1 <<= 1; oa2 <<= 1; oa3 <<= 1; 

					switch(coul)
					{
					case BLANC:		  oa1 += 1;								        break;
					case TRANS:		 										        break;
					case GRIS_CLAIR:  oa1 += 1;					    oa3 += 1;       break;
					case GRIS:		  oa1 += trame;  oa2 += trame;  oa3 += !trame;  break;
					case GRIS_FONCE:  oa1 += trame;  oa2 += 1;      oa3 += trame;   break;
					case NOIR:	 	 	 	         oa2 += 1;      oa3 += 1;	    break;
					}
				}
			
				uchar ob1=0, ob2=0, ob3=0;

				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+8+i < largeur) ? tab[h][pxldebut+8+i] : TRANS);
					uchar trame = !((h+i)&1);
					
					ob1 <<= 1; ob2 <<= 1; ob3 <<= 1; 

					switch(coul)
					{
					case BLANC:		  ob1 += 1;								        break;
					case TRANS:		 										        break;
					case GRIS_CLAIR:  ob1 += 1;					    ob3 += 1;       break;
					case GRIS:		  ob1 += trame;  ob2 += trame;  ob3 += !trame;  break;
					case GRIS_FONCE:  ob1 += trame;  ob2 += 1;      ob3 += trame;   break;
					case NOIR:	 	 	 	         ob2 += 1;      ob3 += 1;	    break;
					}
				}

				*(Code++) = ob1;
				*(Code++) = oa1;
				*(Code++) = ob2;
				*(Code++) = oa2;
				*(Code++) = ob3;
				*(Code++) = oa3;
			}	
			
			pxldebut+=16;
			l-=16;
		}
		else 	// bloc de 8 pixels a traiter si la largeur restante est <=8
		{
			if (pxldebut) *(Code++) = 8;
			
			for (int h=0;h<hauteur;h++)
			{
				uchar o1=0, o2=0, o3=0;
			
				for (i=0; i<8;i++)
				{
					uchar coul = ((pxldebut+i < largeur) ? tab[h][pxldebut+i] : TRANS);
					uchar trame = !((h+i)&1);
					
					o1 <<= 1; o2 <<= 1; o3 <<= 1; 

					switch(coul)
					{
					case BLANC:		  o1 += 1;								     break;
					case TRANS:		 										     break;
					case GRIS_CLAIR:  o1 += 1;					  o3 += 1;       break;
					case GRIS:		  o1 += trame;  o2 += trame;  o3 += !trame;  break;
					case GRIS_FONCE:  o1 += trame;  o2 += 1;      o3 += trame;   break;
					case NOIR:	 	 	 	        o2 += 1;      o3 += 1;	     break;
					}
				}
			
				*(Code++) = o2;
				*(Code++) = o3;
				*(Code++) = o1;
			}

			pxldebut+=8;
			l-=8;
		}
	
	} while ( pxldebut < largeur );

	*(Code++) = 0;
	return size;
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::makeCodeImage3Coul(Tab &tab, uchar* code)
{
	uint offset=0;
	for(char ligne=63; ligne>=0; ligne--)  for(char octet=15; octet>=0; octet--) 
	{
		uchar c1=0, c2=0;
		
		for(uchar pix=0; pix<8; pix++)
		{
			c1 <<= 1; c2 <<= 1;

			switch( tab[ligne][octet*8+pix] )
			{
			case BLANC:						break;
			case GRIS_CLAIR: 
			case GRIS: 
			case GRIS_FONCE:       c2++;	break;
			case NOIR:		 c1++; c2++;	break;
			}
		}

		code[offset] = c1;
		code[offset+1024] = c2;
		offset++;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Coder::makeCodeImage5Coul(Tab &tab, uchar* code)
{
	uint offset=0;
	for(char ligne=63; ligne>=0; ligne--)  for(char octet=15; octet>=0; octet--) 
	{
		uchar c1=0, c2=0, c3=0;
		
		for(uchar pix=0; pix<8; pix++)
		{
			uchar trame = (ligne+pix)&1;
			c1 <<= 1; c2 <<= 1; c3 <<= 1; 

			switch( tab[ligne][octet*8+pix] )
			{
			case BLANC:												  break;
			case GRIS_CLAIR: c1 += trame;				c3 += !trame; break;
			case GRIS:       c1 += trame; c2 += !trame; c3 += trame;  break;
			case GRIS_FONCE: c1 += trame; c2 += 1;      c3 += !trame; break;
			case NOIR:		 c1 += 1;     c2 += 1;      c3 += 1;	  break;
			}
		}

		code[offset] = c1;
		code[offset+1024] = c2;
		code[offset+2048] = c3;
		offset++;
	}
}

///////////////////////////////////////////////////////////////////////////////////////
