#include "config.h"
#include "config_f100.h"

#include <qsettings.h>
#include <qcolordialog.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qdir.h>
#include <qtranslator.h>
#include <qapplication.h>

#ifdef F100_PLUGIN
#	include <IMainWindow.h>
#	include <IOptionWindow.h>
#endif

#ifndef F100_PLUGIN
///////////////////////////////////////////////////////////////////////////////////////

Config::~Config()
{
	DoSave();
	delete m_pSettings;
}
#else
Config::~Config()
{
}
		
#endif

///////////////////////////////////////////////////////////////////////////////////////

void Config::chooseColor()
{
    QColor c = QColorDialog::getColor(m_color,this,"colordialog");
    if( c.isValid() ) m_color = c;
}

///////////////////////////////////////////////////////////////////////////////////////

#ifndef F100_PLUGIN
Config::Config( QWidget* parent )
   : QDialog( parent, "Config", true )
	, m_pTranslator(NULL)
#else
Config::Config(IMainWindow* pMainWindow)
  : CSettingsWidget(pMainWindow, "SpriteMaker")

#endif
{
#ifndef F100_PLUGIN
	m_pSettings = new QSettings();
    m_pSettings->setPath(SOCIETY_NAME, PROGRAM_NAME);
#endif

    QVBoxLayout* ConfigLayout = new QVBoxLayout( this, 11, 6, "ConfigLayout"); 

    appGrpBox = new QGroupBox( this, "appGrpBox" );
    appGrpBox->setColumnLayout(0, Qt::Vertical );
    appGrpBox->layout()->setSpacing( 6 );
    appGrpBox->layout()->setMargin( 11 );
    QVBoxLayout* appGrpBoxLayout = new QVBoxLayout( appGrpBox->layout() );
    appGrpBoxLayout->setAlignment( Qt::AlignTop );

#ifndef F100_PLUGIN
    tradBox = new QComboBox( FALSE, appGrpBox, "tradBox" );
    tradBox->setMinimumSize( QSize( 0, 24 ) );
    appGrpBoxLayout->addWidget( tradBox );
#endif

    colorBtn = new QPushButton( appGrpBox, "colorBtn" );
    colorBtn->setMinimumSize( QSize( 0, 30 ) );
    appGrpBoxLayout->addWidget( colorBtn );
    ConfigLayout->addWidget( appGrpBox );

    codeGrpBox = new QGroupBox( this, "codeGrpBox" );
    codeGrpBox->setColumnLayout(0, Qt::Vertical );
    codeGrpBox->layout()->setSpacing( 6 );
    codeGrpBox->layout()->setMargin( 11 );
    QVBoxLayout* codeGrpBoxLayout = new QVBoxLayout( codeGrpBox->layout() );
    codeGrpBoxLayout->setAlignment( Qt::AlignTop );

    asmBtn = new QRadioButton( codeGrpBox, "asmButton" );
    codeGrpBoxLayout->addWidget( asmBtn );

    cppBtn = new QRadioButton( "C/C++", codeGrpBox, "cppBtn" );
    codeGrpBoxLayout->addWidget( cppBtn );

    modeBox = new QComboBox( FALSE, codeGrpBox, "modeBox" );
    modeBox->setMinimumSize( QSize( 0, 24 ) );
    modeBox->insertItem( "Normal b/b" );
    modeBox->insertItem( "Normal w/w" );
    modeBox->insertItem( "DrawLib 1.3" );
    modeBox->insertItem( "DrawLib 1.4" );
    modeBox->insertItem( "gxLib 1.0" );
    codeGrpBoxLayout->addWidget( modeBox );
    ConfigLayout->addWidget( codeGrpBox );

    QHBoxLayout* layout2 = new QHBoxLayout( 0, 0, 6, "layout2"); 

#ifndef F100_PLUGIN
    okBtn = new QPushButton( this, "okBtn" );
    okBtn->setMinimumSize( QSize( 0, 30 ) );
    layout2->addWidget( okBtn );

    cancelBtn = new QPushButton( this, "cancelBtn" );
    cancelBtn->setMinimumSize( QSize( 0, 30 ) );
    layout2->addWidget( cancelBtn );
#endif

    ConfigLayout->addLayout( layout2 );
    languageChange();
    resize( QSize(219, 266).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

	resize( QSize(239, 240).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

#ifndef F100_PLUGIN
	QDir tradDir(ProgramPath + "Languages");
	m_tradList = tradDir.entryList("*.qm");
	for(QStringList::Iterator i = m_tradList.begin(); i != m_tradList.end(); i++)
	{
		*i = (*i).section('.',0,0);
        tradBox->insertItem( *i );
	}

    connect( tradBox, SIGNAL( activated(int) ), SLOT( setTraduc(int) ) );
#endif
	DoRefresh();
	
	modeBox->setCurrentItem(m_mode);
	asmBtn->setChecked(m_lang==0);
	cppBtn->setChecked(m_lang==1);
#ifndef F100_PLUGIN
	okBtn->setFocus();
    connect( okBtn,     SIGNAL( clicked() ), SLOT( accept() ) );
    connect( cancelBtn, SIGNAL( clicked() ), SLOT( reject() ) );
#endif
		    // signals and slots connections
    connect( modeBox,	   SIGNAL( activated(int) ), SLOT( setCodeMode(int) ) );
    connect( asmBtn,       SIGNAL( clicked() ),    SLOT( setLangAsm() ) );
    connect( cppBtn,       SIGNAL( clicked() ),    SLOT( setLangCpp() ) );
    connect( colorBtn,     SIGNAL( clicked() ),    SLOT( chooseColor() )	);

#ifndef F100_PLUGIN
	updateWords();
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

void Config::languageChange()
{
#ifndef F100_PLUGIN
    setCaption( tr( "Settings", "Caption of the window" ) );
    okBtn->setText( tr( "OK", "Caption of the \'OK\' button" ) );
    cancelBtn->setText( tr( "&Cancel", "Caption of the \'Cancel\' button" ) );
    cancelBtn->setAccel( QKeySequence( tr( "Alt+C", "Accelerator for the \'Cancel\' button" ) ) );
#else
	setCaption("Sprite Maker");
#endif
    appGrpBox->setTitle( tr( "Appearance", "Title of the first box" ) );
    colorBtn->setText( tr( "Mask Color", "Colour of the \'Mask Color\' button" ) );
    codeGrpBox->setTitle( tr( "Code Generation", "Title of the second box" ) );
    asmBtn->setText( tr( "Assembly", "Choice of Assembly language" ) );
}

///////////////////////////////////////////////////////////////////////////////////////

void Config::setLang(int lang)
{
	if(lang == 0) {
		setLangAsm();
	}
	else if(lang == 0) {
		setLangCpp();
	}
}

#ifndef F100_PLUGIN

///////////////////////////////////////////////////////////////////////////////////////

void Config::setTraduc(int num)
{
	m_trFile = m_tradList[num];
	updateWords();
}

///////////////////////////////////////////////////////////////////////////////////////

void Config::updateWords()
{
	if(m_pTranslator != NULL) {
		qApp->removeTranslator( m_pTranslator );
		delete m_pTranslator;
	}
	m_pTranslator = new QTranslator(this);
	m_pTranslator->load( trFile(), ProgramPath + "Languages/" );
	qApp->installTranslator( m_pTranslator );
}

#endif

///////////////////////////////////////////////////////////////////////////////////////

void Config::writeEntry(const QString& sParentKey, const QString& sKey, int nValue)
{
#ifndef F100_PLUGIN
    m_pSettings->writeEntry('/' + sParentKey + '/' + sKey, nValue);
#else
	WriteSettings('/' + sParentKey + '/' + sKey, nValue);
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

void Config::writeEntry(const QString& sParentKey, const QString& sKey, const QString& sValue)
{
#ifndef F100_PLUGIN
    m_pSettings->writeEntry('/' + sParentKey + '/' + sKey, sValue);
#else
	WriteSettings('/' + sParentKey + '/' + sKey, sValue);	
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

int Config::readEntry(const QString& sParentKey, const QString& sKey, int nDefaultValue)
{
#ifndef F100_PLUGIN
	return m_pSettings->readNumEntry('/' + sParentKey + '/' + sKey, nDefaultValue);
#else
	int nNumber;
	ReadSettings('/' + sParentKey + '/' + sKey, nNumber, nDefaultValue);
	return nNumber;
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

QString Config::readEntry(const QString& sParentKey, const QString& sKey, const QString& sDefaultValue)
{
#ifndef F100_PLUGIN
	return m_pSettings->readEntry('/' + sParentKey + '/' + sKey, sDefaultValue);
#else
	QString s;
	ReadSettings('/' + sParentKey + '/' + sKey, s, sDefaultValue);
	return s;
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

int Config::DoSave()
{
    writeEntry("Settings", "mode",  m_mode);
    writeEntry("Settings", "lang",  m_lang);
    writeEntry("Settings", "color", m_color.name());
#ifndef F100_PLUGIN
	writeEntry("Settings", "trFile", m_trFile);
#endif
	Colors[TRANS] = m_color;
	emit modified();
	return 1;
}

///////////////////////////////////////////////////////////////////////////////////////

void Config::DoRefresh()
{
    m_mode = readEntry("Settings", "mode", 1);
    m_lang = readEntry("Settings", "lang", 1);
    m_color.setNamedColor( readEntry("Settings", "color", "#ffff00") );
#ifndef F100_PLUGIN
    m_trFile = readEntry("Settings", "trFile", "English");
	tradBox->setCurrentItem(m_tradList.findIndex(m_trFile));
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

QString Config::GetDescription() const
{
    return QString::null;
}

#ifndef F100_PLUGIN
///////////////////////////////////////////////////////////////////////////////////////

void Config::accept()
{
	QDialog::accept();
	DoSave();
}
#endif

///////////////////////////////////////////////////////////////////////////////////////

void Config::showSettings()
{
#ifndef F100_PLUGIN
	show();
#else
	GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(this);
#endif
}
