#ifndef CONVERTER_H
#define CONVERTER_H

#include "main.h"

#include <qvariant.h>
#include <qdialog.h>
#include <qimage.h>
#ifdef Q_OS_UNIX
#include "tab.h"
#else
#include "../mylibs/matrice.h"
#endif
#include <qlabel.h>
#include <qfileinfo.h>
#include <qlineedit.h>

class QSpinBox;
class QLineEdit;
class QRadioButton;
class QSlider;
class QLabel;
class QGroupBox;
class QPushButton;
class Config;

struct NivCouleurs
{
	NivCouleurs(uchar n=0, uchar dg=0, uchar mg=0, uchar lg=0) 
		: noir(n), grisFonce(dg), grisMoyen(mg), grisClair(lg) {}
	
	uchar noir, grisFonce, grisMoyen, grisClair;
};

///////////////////////////////////////////////////////////////////////////////////////

class ConvImageLbl : public QWidget
{
Q_OBJECT
public:
	ConvImageLbl(QWidget* parent, QString imageFile, NivCouleurs &niv, QRgb masque, int mode);
	~ConvImageLbl();
	void getConvertedTab(Tab &dest);
	void getConvertedImage(QImage &img) { img=m_img; }
	void changeMask(QRgb masque);

public slots:
	void changeNiv(NivCouleurs niv);
	void changeZoom(int zoom);
	void changeMode(int mode);

protected slots:
	void paintEvent(QPaintEvent* e);
	void resizeEvent(QResizeEvent* e);

protected:
	void createFromImage(QImage &img, NivCouleurs &niv, QRgb masque, int mode);
	void createPalette(NivCouleurs &niv, QRgb masque, int mode);
	QString m_imageFile;
	QImage m_img;
	NivCouleurs m_niveaux;
	QRgb m_masque;
	int m_mode;
	QRect m_posImg;
	uchar* m_imgData;
	QRgb* m_imgPal;
	int m_bpl;
	int m_zoom;
};

///////////////////////////////////////////////////////////////////////////////////////

class OrigImageLbl : public QWidget
{
Q_OBJECT
public:
	OrigImageLbl(QWidget* parent, QString imageFile);

public slots:
	void changeZoom(int zoom);

protected slots:
	void paintEvent(QPaintEvent* e);
	void resizeEvent(QResizeEvent* e);

protected:
	QImage m_img;
	QRect m_posImg;
	int m_zoom;
};

///////////////////////////////////////////////////////////////////////////////////////

class Converter : public QDialog
{
Q_OBJECT

public:
    Converter(QWidget* parent, Config* pConfig, QString &imageFile );
    ~Converter();

	void getConvertedTab(Tab &dest) { imageSortie->getConvertedTab(dest); }
	void getConvertedImage(QImage &img) { imageSortie->getConvertedImage(img); }
	QString nomSprite() { return nomSpriteEdit->text(); }
	uchar mode() { return m_mode; }

signals:
	void nivChanged(NivCouleurs);
	void modeChanged(int);
	void zoomChanged(int);
	void masqueChanged(QRgb);

protected slots:
	void setMode(int mode);
	void setZoom(int zoom);
	void chooseMask();
	void setLGNiv(int niv)	{ setNiveau(GRIS_CLAIR,	niv); }
	void setMGNiv(int niv)	{ setNiveau(GRIS,		niv); }
	void setFGNiv(int niv)	{ setNiveau(GRIS_FONCE,	niv); }
	void setBNiv(int niv)	{ setNiveau(NOIR,		niv); }
    void languageChange();

protected:
	void setNiveau(uchar coul, int niv);
 	NivCouleurs m_niveaux;
	int m_zoom;
	QRgb m_masque;
	uchar m_mode;
	QString m_imgFile;

	OrigImageLbl* imageEntree;
	ConvImageLbl* imageSortie;
	QSpinBox* zoomSpin;
    QLineEdit* nomSpriteEdit;
    QRadioButton* coul3Btn;
    QRadioButton* coul5Btn;

    QGroupBox* EntreeGrpBox;
	QGroupBox* SortieGrpBox;
    QGroupBox* ReglagesGrpBox;

	QLabel* zoomLbl;
	QLabel* nomSpriteLbl;
	QLabel* maskLbl;

	QPushButton* annulerBtn;
	QPushButton* maskBtn;

	QLabel* nivNoirLbl;
    QLabel* nivGrFonceLbl;
    QLabel* nivGrMoyLbl;
    QLabel* nivGrClairLbl;
    QSpinBox* nivNoirSpin;
    QSpinBox* nivGrFonceSpin;
    QSpinBox* nivGrMoySpin;
    QSpinBox* nivGrClairSpin;
    QSlider* nivNoirSlider;
    QSlider* nivGrMoySlider;
    QSlider* nivGrClairSlider;
    QSlider* nivGrFonceSlider;
	Config*  m_pConfig;

};

#endif
