#ifndef __DOITEM
#define __DOITEM

#include "main.h"

#include "select.h"

#include "../mylibs/myBackuphdlr.h"

#include <qptrlist.h>

struct Pixel
{
	Pixel(QPoint Pos, int OldCol, int NewCol)
		: oldCol(OldCol), newCol(NewCol), pos(Pos) {}
	int oldCol,newCol;
	QPoint pos;
};

#define TYPE_PIX	  0
#define TYPE_PART_TAB 1
#define TYPE_DIM	  2
#define TYPE_FLIP	  3
#define TYPE_SELECT   4
#define TYPE_SEL_CREATE 0
#define TYPE_SEL_FUSION 1
#define TYPE_SEL_DELETE 2
#define TYPE_SEL_MOVE   3
#define TYPE_SEL_APPEAR 4

class DoItem : public DoItemBase
{
public:
	DoItem()
	{
		m_type = TYPE_PIX;
		for(int i=0; i<5; i++) m_tabs[i]=NULL;
		m_listePxl.setAutoDelete(true);
	}
	QPtrList<Pixel> & listePxl() { return m_listePxl; }

	DoItem(QPoint PosPart, Tab &OldPart, Tab &NewPart)
	{
		m_type = TYPE_PART_TAB;
		m_tabs[0] = new Tab(OldPart);
		m_tabs[1] = new Tab(NewPart);
		for(int i=2; i<5; i++) m_tabs[i]=NULL;
		m_posPart = PosPart;
		m_listePxl.setAutoDelete(true);
	}
	QPoint & posPart() { return m_posPart; }
	Tab & oldPart() { return *m_tabs[0]; }
	Tab & newPart() { return *m_tabs[1]; }

	DoItem(QSize &OldSize, QSize &NewSize, Tab* PartBottom=NULL, Tab* PartRight=NULL)
	{
		m_oldSize = OldSize;
		m_newSize = NewSize;
		m_type = TYPE_DIM;
		m_tabs[0] = PartBottom;
		m_tabs[1] = PartRight;
		for(int i=2; i<5; i++) m_tabs[i]=NULL;
	}
	QSize oldSize() { return m_oldSize; }
	QSize newSize() { return m_newSize; }
	Tab* partBottom() { return m_tabs[0]; }
	Tab* partRight() { return m_tabs[1]; }

	DoItem(bool horiz)
	{
		m_type = TYPE_FLIP;
		for(int i=0; i<5; i++) m_tabs[i]=NULL;
		m_flipHoriz = horiz;
	}
	bool flipHoriz() { return m_flipHoriz; }

	DoItem(int TypeSel)
	{
		m_type = TYPE_SELECT;
		m_typeSel = TypeSel;
		m_oldSel = NULL;
		for(int i=0; i<5; i++) m_tabs[i]=NULL;
	}
	int typeSel() { return m_typeSel; }
	QRect & oldSelPos() { return m_oldSelPos; }
	QRect & newSelPos() { return m_newSelPos; }
	Tab* & underSel() { return m_tabs[0]; }
	Selection* & oldSel() { return m_oldSel; }


	~DoItem() 
	{
		for(int i=0; i<5; i++) if(m_tabs[i]) delete m_tabs[i];
		switch(m_type)
		{
		case TYPE_PIX: 
			m_listePxl.clear(); 
			break;
		case TYPE_SELECT: 
			if(m_oldSel) delete m_oldSel;
			break;
		default: ;
		}
	}

	int type() { return m_type; }


	

protected:
	int m_type, m_typeSel;
	QPtrList<Pixel> m_listePxl;
	Tab* m_tabs[5];
	QPoint m_posPart;
	bool m_flipHoriz;
	QSize m_oldSize, m_newSize;
	Selection *m_oldSel;
	QRect m_oldSelPos, m_newSelPos;
};

#endif
