#include <qlayout.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qstatusbar.h>
#include <qfiledialog.h>
#include <qimage.h>
#include <qcolordialog.h> 
#include <qtextstream.h> 
#include <qaccel.h>
#include <malloc.h>

#include "handler.h"
#include "config.h"
#include "config_f100.h"
#include "icones.h"
#include "coder.h"
#include "converter.h"
#include "sprite.h"

#include "../mylibs/myRFhdlr.h"

#define READ( file, DestAdd, size, test)	test=(test&&file.readBlock((char*)DestAdd,size)==size);
#define WRITE( file, SrcAdd, size, test )	test=(test&&file.writeBlock((const char*)SrcAdd,size)==size);
		
void DrawItem::paint(QPainter * p, const QColorGroup & cg, const QRect & cr, bool selected )
{
	QPixmap buff(cr.size());
	buff.fill( selected? cg.highlight() : cg.base() );
	QPoint plot;
	plot.setX( (cr.width()-m_pix.width())/2 );
	plot.setY( (cr.height()-m_pix.height())/2 );
	bitBlt(&buff,plot,&m_pix);
	p->drawPixmap(0,0,buff);
}

///////////////////////////////////////////////////////////////////////////////////////

QSize DrawItem::sizeHint() const 
{ 
	return QSize(m_pix.width(),m_pix.height()+2);
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void Handler::updateRow(uint i)
{
	Sprite* spr = m_liste.at(i);
	
	QCheckTableItem *item = new QCheckTableItem( spritesTable, "" );
	item->setChecked( spr->isSelected() );
	spritesTable->setItem(i, COL_CODE, item); // efface l'ancien si deja existant

	QTableItem* tItem = new QTableItem(spritesTable,QTableItem::WhenCurrent,spr->nom());
	spritesTable->setItem( i, COL_NAME, tItem);
	spritesTable->adjustColumn( COL_NAME );

	DrawItem* dItem = new DrawItem(spritesTable);
	dItem->setPixmap(spr->image());
	spritesTable->setItem(i, COL_APER, dItem);

	spritesTable->setText(i, COL_LARG, QString::number(spr->largeur()));
	spritesTable->setText(i, COL_HAUT, QString::number(spr->hauteur()));

	spritesTable->adjustRow(i);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::addSprite(Sprite* spr, bool andEdit)
{
	if(spr->isCorrect()==false) { delete spr; return; }
	
	m_liste.append( spr );
	connect( spr, SIGNAL(dataChanged(Sprite*)), this, SLOT(updateRow(Sprite*)) );
	connect( spr, SIGNAL(editionEnded(Sprite*)), this, SLOT(activeRow(Sprite*)) );
	connect( this, SIGNAL(modeChanged(int)), spr, SIGNAL(modeChanged(int)) ); // relais
	spritesTable->setNumRows(spritesTable->numRows()+1);
	updateRow(spritesTable->numRows()-1);
	spritesTable->adjustColumn(COL_APER);
	enableActions( true );

	int n = m_liste.count();
	statNbreSprites->setText( QString("%1 sprite%2").arg(n).arg(n!=1?"s":"") );

	spritesTable->verticalHeader()->setLabel(n-1,QString::number(n-1));

	if(andEdit)
	{
		spr->edit();
		spritesTable->setRowReadOnly(spritesTable->numRows()-1,true);
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::NotifyModif()
{
	if(m_modified==false)
	{
		m_modified = true;
		if(!m_cspName.isEmpty()) setCaption( caption()+"*");
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileNew() 
{ 
	addSprite( new Sprite(this, m_pConfig), true ); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileOpen() 
{ 
	QString filename = QFileDialog::getOpenFileName(QString::null,"Casio Sprites (*.csp)",
			this,"opendialog", tr("Open a File", "caption of the \'Open File\' dialog"));
	
	openCSPfile(filename); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::openCSPfile(QString filename)
{
	if( filename.isEmpty() ) {
		return;
	}
	int ans = 0;
	if(m_CSPloaded || m_liste.count()!=0) // si des donnes sont deja charges
	{
		QString text;
		if(m_CSPloaded) text = tr("A CSP file is already opened!\n"
									"Do you want to close the actual file\n"
									"or import the contents of the selected file?", "Warning");	
		else text = tr("Some sprites have already been loaded!\n"
						"Do you want to save and close the actual data\'s\n"
						"or import the contents of the selected file?", "Warning");

		ans = QMessageBox::question(this,
			tr("Choose an option:","Caption of the Warning box"),
			text,
			tr("&Close","Warning option: Close the CSP file"),
			tr("&Import","Warning option: Import the contents of the CSP file"),
			tr("&Cancel","Warning option: Cancel the action"),
			0,2);
	}

	if(ans==2) return;

	if(ans==0) fileClose();

	/* on charge un fichier .CSP */
	/* format CSP:
		- tag "CSP"
		- nombre de sprites (n)
		- zone de reserve de 20 octets
		- n fois:
			- nom du sprite sur (20+1) octets
			- largeur
			- hauteur
			- tableau du sprite
	*/

	char tag[20], nom[21];
	uchar* buff;
	Tab tab;
	int nombre, size=0, haut, larg, xOrig=0, yOrig=0, sizeOfInt=sizeof(int);
	QFile file(filename);
	bool ok = file.open(IO_ReadOnly);
	bool origOK=false;

	READ(file,tag,3,ok); // lecture du tag
	ok = (ok && tag[0]=='C' && tag[1]=='S' && tag[2]=='P' ); // test du tag
	READ(file,&nombre,sizeOfInt,ok); // nbre de sprites
	READ(file,tag,1,ok); // reconnaissance des origines de sprites?
	if(tag[0]=='N') origOK = true;
	READ(file,tag,19,ok); // zone de rserve
	
	for(int cpt=0; ok && cpt<nombre; cpt++) // lecture de chaque sprite
	{
		READ(file,nom,21,ok);
		READ(file,&larg,sizeOfInt,ok);
		READ(file,&haut,sizeOfInt,ok);
		if(origOK)
		{
			READ(file,&xOrig,sizeOfInt,ok);
			READ(file,&yOrig,sizeOfInt,ok);
		}

		if(ok) size = larg*haut;
		buff = new uchar[size];

		READ(file,buff,size,ok);

		if(ok) 
		{
			tab.adaptFromPtr(larg,haut,buff);
			addSprite( new Sprite(this, m_pConfig,tab,nom,QPoint(xOrig,yOrig)), false );
		}
		delete[] buff;
	}
	file.close();
	
	if(ok==false) // si une erreur a t rencontre
	{
		QString erreur = tr("The file %1\nhas not been correctly loaded!", "Error message");
		QMessageBox::critical(this,tr("Error","Caption of the Error message"),erreur.arg(filename),"OK");
		return;
	}

	if(ans==0) 
	{
		setCaption( QString("Sprite Maker - %1").arg(filename) );
		m_cspName = filename;
		m_CSPloaded = true;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

bool Handler::fileClose()
{
	if(!okToClose()) return false;
	
	int nbr = m_liste.count();
	//for(int i=0; i<nbr; i++) spritesTable->removeRow(0);
	m_liste.clear();
	m_modified = false;

	while(spritesTable->numRows() > 0) {
		spritesTable->removeRow(0);
	};

#ifndef F100_PLUGIN
	m_RFHandler->addFile(m_cspName);
#endif
	m_cspName = QString::null;
	m_CSPloaded = false;
	statNbreSprites->clear();
	setCaption(tr("Sprite Maker - Untitled", "Default caption of the window"));
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileSave()
{
	if(m_liste.isEmpty()) return;

	if(m_cspName.isEmpty()) { fileSaveAs(); return; }

	char buff[20];
	int nombre = m_liste.count(), size, sizeOfInt=sizeof(int);
	QFile file(m_cspName);
	bool ok = file.open(IO_WriteOnly);

	WRITE(file,"CSP",3,ok); // tag
	WRITE(file,&nombre,sizeOfInt,ok); // nbre de sprites
	WRITE(file,"N",1,ok); // indique que ce fichier reconnait les origines de sprites
	WRITE(file,buff,19,ok); // zone de rserve
	
	for(int cpt=0; ok && cpt<nombre; cpt++) // enregistrement de chaque sprite
	{
		Sprite* spr = m_liste.at(cpt);
		int larg = spr->largeur(), haut = spr->hauteur();
		int x = spr->origin().x(), y = spr->origin().y();

		char nom[21];
		int l=spr->nom().length();
		const char* ptr = spr->nom().ascii();
		for(int t=0; t<l; t++) nom[t]=ptr[t];
		nom[l]='\0';
		WRITE(file,nom,21,ok);
		WRITE(file,&larg,sizeOfInt,ok);
		WRITE(file,&haut,sizeOfInt,ok);
		WRITE(file,&x,sizeOfInt,ok);
		WRITE(file,&y,sizeOfInt,ok);
		
		size = larg*haut;
		uchar* tab = spr->tableau().toNewPtr();

		WRITE(file,tab,size,ok);

		delete[] tab;
	}
	file.close();

	if(ok) 
	{
		setCaption(QString("Sprite Maker - %1").arg(m_cspName));
		m_modified = false;
		m_CSPloaded = true;
	}
	QString msg = (ok ? tr("The save was successful.","Status bar message") :
						tr("The save has failed!!","Status bar message"));
	statusBar()->message(msg);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileSaveAs()
{
	if(m_liste.isEmpty()) return;
    
	QString filename = QFileDialog::getSaveFileName(
			    m_cspName, "Casio Sprites (*.csp)", this,
			    "save as", tr("Save As...","Caption of the \'Save As\' dialog") );

	if ( !filename.isEmpty() ) 
	{
		int answer = 0;
		if( !filename.lower().endsWith(".csp") ) filename += ".csp";

		if ( QFile::exists( filename ) )
			answer = QMessageBox::warning(
					this, tr("Already existing file","Caption of the Warning message"),
					QString( tr("The file \'%1\' already exists!\nDo you want to replace it?","Warning") ).
					arg( filename ),
					tr("&Yes","Warning option"), tr("&No","Warning option"), QString::null, 1, 1 );
		if ( answer == 0 ) 
		{
			m_cspName = filename;
			fileSave();
		}
    }
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::closeEvent(QCloseEvent* e)
{ 
	if(fileClose())	{
		e->accept();
		emit closed();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

bool Handler::okToClose()
{
	if( m_modified || ((!m_CSPloaded) && (!m_liste.isEmpty())) ) return askForSave();
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////

bool Handler::askForSave()
{
	QString msg = QString(tr("Save the changes made on file %1?","Save confirmation text")).
		arg( (m_cspName.isEmpty()? QString::null : ("\n"+m_cspName)) );
	
	int x = QMessageBox::information( this, 
		tr("Save","Caption of the confirmation box"),
		msg, 
		tr("&Yes","Confirmation option"), 
		tr("&No","Confirmation option"), 
		tr("&Cancel","Confirmation option"),
		0, 1 );

	if(x==0) fileSave();
	return (x!=2);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::helpAbout()
{
     QString text = tr(
			"<center>Sprite Maker\n"
			"Version 2.31"
			"<p>This software allows you to create and code sprites (3/5 colors + mask) to be easily drawn on the screen of a Casio Graph100/AFX calculator with some specific routines like those of the DrawLib library."
			"<br>It may never be commercialized in any way.</p>"
			"<p>Don't hesitate to send me comments, bug reports or suggestions!</p>"
			"<p>Julien Patte, January 2005"
			"<br>orwell01@hotmail.com</p>"
			"<p>English translation by: Default</p></center>",
			"\'About...\' text");			;

	QMessageBox box( tr("About Sprite Maker","\'About...\' caption"),text,QMessageBox::NoIcon,0,0,0, this );
    box.exec();
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileExport()
{
	if(m_liste.isEmpty()) return;
	
	QString filename = QFileDialog::getSaveFileName(QString::null,"Bitmap (*.bmp)",
				this,"exportdialog", tr("Save a bitmap", "Title of the \'save bitmap\' dialog"));
	
	if( !filename.isEmpty() ) 
	{
		if(filename.lower().endsWith(".bmp")==false) filename+=".bmp";
		int i=spritesTable->currentRow();
		if(i==-1) i=0;
		if( m_liste.at(i)->image().convertToImage().save(filename,"BMP") )
 			statusBar()->message( QString(tr("Picture saved in \'%1\'","Status bar message")).arg(filename) );
		else statusBar()->message( tr("The save has failed!","Status bar message") );
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileImport()
{
	QStringList list = QFileDialog::getOpenFileNames(tr("Pictures (*.bmp *.jpg *.gif *.png)", "Filter for picture files"),QString::null,
				this,"importdialog", tr("Import pictures", "Title of the \'import picture\' dialog"));
	
	uchar save = ColorMode = (m_pConfig->mode()==GXLIB_10 ? MODE_5 : MODE_3);
	for(QStringList::Iterator it = list.begin(); it!=list.end(); it++)
		addSprite( new Sprite(this, m_pConfig,*it),list.count()==1 ); 

	if(ColorMode!=save) 
		setMode((ColorMode==MODE_5 ? GXLIB_10 : DRAWLIB_14), m_pConfig->language());
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::fileImportBMP()
{
	QString imgFile = QFileDialog::getOpenFileName(QString::null,tr("Pictures (*.bmp *.jpg *.gif *.png)", "Filter for picture files"),
				this,"importdialog", tr("Open a 128*64 picture","Title for the \'import 128*64\' dialog"));

	if(imgFile.isEmpty()) return;

	QImage img;
	if(img.load(imgFile)==false || img.width()!=128 || img.height()!=64)
	{
		QMessageBox::warning(this,tr("Error","Warning title"), tr("File type unrecognized!","Error: bad imported file or picture"),"OK");
		return;
	}

	Tab tab;
	Converter conv(this, m_pConfig, imgFile);
	if(conv.exec()==QDialog::Rejected) return;
	conv.getConvertedTab(tab);

	Coder cod(m_pConfig, tab, conv.nomSprite(), conv.mode());
	cod.exec();
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::dragEnterEvent(QDragEnterEvent* event)
{ 
	event->accept( QTextDrag::canDecode(event) ); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::dropEvent(QDropEvent* event)
{
	QString dropped;
	if( ! QTextDrag::decode(event,dropped) ) return;
	addDroppedStuff(dropped);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::setValue(int row, int col)
{
	Sprite* spr = m_liste.at(row);
	if( col == COL_CODE ) spr->setSelected( ! spr->isSelected() );
	else spr->setNom( spritesTable->text(row, COL_NAME) );
}

///////////////////////////////////////////////////////////////////////////////////////
	
void Handler::editCurrentSprite()
{
	if(!m_liste.isEmpty())
	{
		int i = spritesTable->currentRow(); if(i==-1) i=0;
		m_liste.at(i)->edit();
		spritesTable->setRowReadOnly(i,true);
	}
	else addSprite( new Sprite(this, m_pConfig) );
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::moveCurrentSpriteUp()
{
	int i = spritesTable->currentRow(); if(i<1) return;

	spritesTable->swapRows(i,i-1,true);
	Sprite* spr = m_liste.take(i);
	m_liste.insert(i-1,spr);
	updateRow(i-1); spritesTable->verticalHeader()->setLabel(i-1,QString::number(i-1));
	updateRow(i); spritesTable->verticalHeader()->setLabel(i,QString::number(i));
	spritesTable->clearSelection();
	spritesTable->selectCells(i-1, 0, i-1, spritesTable->numCols()-1);
	m_modified = true;
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::moveCurrentSpriteDown()
{
	int i = spritesTable->currentRow(); if(i==-1 ||	i==(int)(m_liste.count())-1) return;

	spritesTable->swapRows(i,i+1,true);
	Sprite* spr = m_liste.take(i);
	m_liste.insert(i+1,spr);
	updateRow(i+1); spritesTable->verticalHeader()->setLabel(i+1,QString::number(i+1));
	updateRow(i); spritesTable->verticalHeader()->setLabel(i,QString::number(i));
	spritesTable->clearSelection();
	spritesTable->selectCells(i+1, 0, i+1, spritesTable->numCols()-1);
	m_modified = true;
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::deleteCurrentSprite()
{
	int i = spritesTable->currentRow(); if(i==-1) return;

	QString msg = 
		QString(tr("Do you really want to delete\nthe sprite \'%1\'?","Confirmation to suppress a sprite")).arg(m_liste.at(i)->nom());
	int ans = QMessageBox::warning(this,tr("Delete","Title of the \'supress sprite\' confirmation"),msg,tr("Yes","Confirmation option"),tr("No","Confirmation option"),QString::null,1,1);
	if(ans!=0) return;

	m_liste.remove(i);
	spritesTable->removeRow(i);
	m_modified = true;

	int n = m_liste.count();
	QString s;
	if(n==0) 
	{
		s = QString::null;
		enableActions( false );
	}
	else 
	{ 
		s = QString("%1 sprite%2").arg(n).arg(n!=1?"s":"");
	}
	statNbreSprites->setText( s );

	for(int j=i; j<n; j++) spritesTable->verticalHeader()->setLabel(j,QString::number(j));
	spritesTable->selectCells(i, 0, i, spritesTable->numCols()-1);
	spritesTable->setCurrentCell(i,spritesTable->currentColumn());
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::editClickedSprite(int row,int,int,const QPoint&) 
{ 
	if(row<0) return;
	
	m_liste.at(row)->edit(); 
	spritesTable->setRowReadOnly(row,true);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::enableActions(bool oui)
{
	fileCloseAction->setEnabled(oui);
	fileSaveAction->setEnabled(oui);
	fileSaveAsAction->setEnabled(oui);
	fileExportAction->setEnabled(oui);
	codeAction->setEnabled(oui);
	moveCurUpAction->setEnabled(oui);
	moveCurDownAction->setEnabled(oui);
	deleteCurAction->setEnabled(oui);
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::onSettingsChanged()
{
	for(uint i=0; i<m_liste.count(); i++) {
		m_liste.at(i)->changeTransColor();
		updateRow(i);
	}	
	setMode(m_pConfig->mode(), m_pConfig->language());
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::setMode(int mode, int lang)
{
	emit modeChanged(mode);

	int nbre = m_liste.count();
	for(int i=0; i<nbre; i++) updateRow(i);

	char* modesNames[] = { "Normal b/b","Normal w/w","DrawLib 1.3","DrawLib 1.4","gxLib 1.0" };
	char* langNames[] = { "ASM","C/C++" };
	statModeLang->setText(tr("Mode %1 (%2)","Code generation mode and language used").arg(modesNames[mode]).arg(langNames[lang]));
	m_pConfig->setCodeMode(mode);
	m_pConfig->setLang(lang);
}

///////////////////////////////////////////////////////////////////////////////////////

QString formate (QString str)
{
	const char* text = str.latin1();
	char text2[100];
	int index1=0,index2=0;
	
#ifdef Q_OS_UNIX
	if(str.startsWith("file://")) text+=7;
#endif

	for(char c = text[0]; c!='\0'; c = text[++index1])
	{
		if(c!='%') text2[index2++] = c;
		else
		{
			char c1 = text[index1+1];
			char c2 = text[index1+2];
			if(  ( ('0'<=c1 && c1<='9') || ('a'<=c1 && c1<='f') )   &&
				 ( ('0'<=c2 && c2<='9') || ('a'<=c2 && c2<='f') )
				 )
			{
				if(c1=='c' && c2=='3' && text[index1+3]=='%')
				{
					char c3 = text[index1+4];
					char c4 = text[index1+5];
					c3 -= ((c3>'9')? 'a'-10: '0'); 
					c4 -= ((c4>'9')? 'a'-10: '0'); 
					text2[index2++] = c3*16+c4+64;
					index1+=5;
				}
				else if(c1=='c' && c2=='2' && text[index1+3]=='%') index1+=2; // ignor
				else
				{
					c1 -= ((c1>'9')? 'a'-10: '0'); 
					c2 -= ((c2>'9')? 'a'-10: '0'); 
					text2[index2++] = c1*16+c2;
					index1+=2;
				}
			}
			else text2[index2++]='%';
		}
	}
	text2[index2]='\0';

	return QString(text2); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::addDroppedStuff(QString dropped)
{
	uchar save = ColorMode = ( m_pConfig->mode()==GXLIB_10 ? MODE_5 : MODE_3);
	QTextIStream s(&dropped);
	while(!s.atEnd()) 
	{
		QString file = formate(s.readLine());
		if(file.lower().endsWith(".csp")) openCSPfile(file);
		else addSprite( new Sprite(this, m_pConfig,file) ,false);
	}

	if(ColorMode!=save) 
		setMode((ColorMode==MODE_5 ? GXLIB_10 : DRAWLIB_14), m_pConfig->language());
}

///////////////////////////////////////////////////////////////////////////////////////

Handler::~Handler()
{
#ifndef F100_PLUGIN
	m_pConfig->writeEntry("Geometry", "hand_width", width());
	m_pConfig->writeEntry("Geometry", "hand_height", height());
	m_pConfig->writeEntry("Geometry", "hand_x", x());
	m_pConfig->writeEntry("Geometry", "hand_y", y());
	m_RFHandler->save();
#endif
}

///////////////////////////////////////////////////////////////////////////////////////

Handler::Handler( int argc, char ** argv, Config* pConfig) 
	: QMainWindow( 0, "Handler", WType_TopLevel )
{
	m_pConfig = pConfig;
#ifndef F100_PLUGIN
	if(pConfig == NULL) {
		m_pConfig = new Config(this);
	}
#endif

    setCentralWidget( new QWidget( this, "qt_central_widget" ) );
	QVBoxLayout* HandlerLayout = new QVBoxLayout( centralWidget(), 11, 6, "HandlerLayout"); 

    // table
	spritesTable = new DropTable( centralWidget(), "spritesTable" );
    spritesTable->setNumRows( 0 );
    spritesTable->setNumCols( 5 );
	spritesTable->setColumnReadOnly(COL_APER,true);
	spritesTable->setColumnReadOnly(COL_LARG,true);
	spritesTable->setColumnReadOnly(COL_HAUT,true);	
	spritesTable->setSelectionMode(QTable::SingleRow);
	spritesTable->setFocusStyle(QTable::FollowStyle);
	spritesTable->setShowGrid(false);
    spritesTable->setAcceptDrops(true);

	HandlerLayout->addWidget( spritesTable );

	// actions
    fileOpenAction = new QAction(this, "fileOpenAction" );
	fileOpenAction->setIconSet(ICO_OPEN);

    fileCloseAction = new QAction( this, "fileCloseAction" );

	fileSaveAction = new QAction( this, "fileSaveAction" );
	fileSaveAction->setIconSet(ICO_SAVE);

	fileSaveAsAction = new QAction( this, "fileSaveAsAction" );

    fileExitAction = new QAction( this, "fileExitAction" );

    helpAboutAction = new QAction( this, "helpAboutAction" );

    fileNewAction = new QAction( this, "fileNewAction" );
	fileNewAction->setIconSet(ICO_NEW);

	fileImportAction = new QAction( this, "fileImportAction" );

	fileImportBMPAction = new QAction( this, "fileImportAction" );

    fileExportAction = new QAction( this, "fileExportAction" );

    codeAction = new QAction( this, "codeAction" );
	codeAction->setIconSet(QPixmap(xpm_conv));

    editCurAction = new QAction( this, "editCurAction" );
	editCurAction->setIconSet(QPixmap(xpm_edition));

    moveCurUpAction = new QAction( this, "moveCurUpAction" );
	moveCurUpAction->setIconSet(QPixmap(xpm_monter));

    moveCurDownAction = new QAction( this, "moveCurDownAction" );
	moveCurDownAction->setIconSet(QPixmap(xpm_descendre));

    deleteCurAction = new QAction( this, "deleteCurAction" );
	deleteCurAction->setIconSet(QPixmap(xpm_retrait));

	editPrefAction = new QAction( this, "codeAsmAction" );
	
    // toolbars
    toolBar = new QToolBar("",this, DockTop ); 

    fileOpenAction->addTo( toolBar );
    fileSaveAction->addTo( toolBar );
    toolBar->addSeparator();
    fileNewAction->addTo( toolBar );
    editCurAction->addTo( toolBar );
    moveCurUpAction->addTo( toolBar );
    moveCurDownAction->addTo( toolBar );
    deleteCurAction->addTo( toolBar );
    toolBar->addSeparator();
    codeAction->addTo( toolBar );

    // menubars
    MenuBar = new QMenuBar( this, "MenuBar" );

#ifndef F100_PLUGIN
	m_RFHandler = new MyRecentFilesHdlr(this,SOCIETY_NAME,PROGRAM_NAME,5);
#endif
	fileMenu = new QPopupMenu( this );
    fileOpenAction->addTo( fileMenu );
    fileSaveAction->addTo( fileMenu );
    fileSaveAsAction->addTo( fileMenu );
	fileCloseAction->addTo( fileMenu );
	fileMenu->insertSeparator(); 
#ifndef F100_PLUGIN
	fileMenu->insertItem("",m_RFHandler,RECENTFILES_ID);
    fileMenu->insertSeparator();
#endif
    fileExitAction->addTo( fileMenu );
    MenuBar->insertItem( "",fileMenu, 1 );

    QPopupMenu * spriteMenu = new QPopupMenu( this );
    fileNewAction->addTo( spriteMenu );
	fileImportAction->addTo( spriteMenu );
	fileImportBMPAction->addTo( spriteMenu );
	fileExportAction->addTo( spriteMenu );
    spriteMenu->insertSeparator();
    editCurAction->addTo( spriteMenu );
    codeAction->addTo( spriteMenu );
    spriteMenu->insertSeparator();
	editPrefAction->addTo( spriteMenu );
    MenuBar->insertItem( "",spriteMenu, 2 );

    QPopupMenu * helpMenu = new QPopupMenu( this );
    helpAboutAction->addTo( helpMenu );
    MenuBar->insertItem( "&?", helpMenu, 3 );

	// statusbar
	statNbreSprites = new QLabel(statusBar(),"status bar - nbre sprites");
	statModeLang = new QLabel(statusBar(),"status bar - mode, lang");
	//statNbreSprites->setFrameStyle(QFrame::Box|QFrame::Sunken);
	//statModeLang->setFrameStyle(QFrame::Box|QFrame::Sunken);
	statModeLang->setAlignment(Qt::AlignRight);
	statModeLang->setMinimumWidth(135);
	statusBar()->addWidget(statNbreSprites, 0, true);
	statusBar()->addWidget(statModeLang, 0, true);

    // accels
	QAccel *accel = new QAccel( this );
    accel->connectItem( accel->insertItem( Key_Delete ),
                        this, SLOT(deleteCurrentSprite()) );

    // signals and slots connections
    connect( fileNewAction, SIGNAL( activated() ), SLOT( fileNew() ) );
    connect( fileOpenAction, SIGNAL( activated() ), SLOT( fileOpen() ) );
    connect( fileCloseAction, SIGNAL( activated() ), SLOT( close() ) );
    connect( fileSaveAction, SIGNAL( activated() ), SLOT( fileSave() ) );
    connect( fileSaveAsAction, SIGNAL( activated() ), SLOT( fileSaveAs() ) );
    connect( fileExitAction, SIGNAL( activated() ), SLOT( fileExit() ) );
    connect( helpAboutAction, SIGNAL( activated() ), SLOT( helpAbout() ) );
    connect( codeAction, SIGNAL( activated() ), SLOT( codeAll() ) );
    connect( editCurAction, SIGNAL( activated() ), SLOT( editCurrentSprite() ) );
    connect( moveCurUpAction, SIGNAL( activated() ), SLOT( moveCurrentSpriteUp() ) );
    connect( moveCurDownAction, SIGNAL( activated() ), SLOT( moveCurrentSpriteDown() ) );
    connect( deleteCurAction, SIGNAL( activated() ), SLOT( deleteCurrentSprite() ) );
    connect( fileExportAction, SIGNAL( activated() ), SLOT( fileExport() ) );
    connect( fileImportAction, SIGNAL( activated() ), SLOT( fileImport() ) );
    connect( fileImportBMPAction, SIGNAL( activated() ), SLOT( fileImportBMP() ) );
    connect( spritesTable, SIGNAL( valueChanged(int,int) ), SLOT( setValue(int,int) ) );
	connect( spritesTable, SIGNAL( doubleClicked(int,int,int,const QPoint&) ),
				SLOT( editClickedSprite(int,int,int,const QPoint&) ) );
	connect( spritesTable, SIGNAL(newDrop(QString)), SLOT(addDroppedStuff(QString)));
	connect( editPrefAction, SIGNAL(activated()), m_pConfig, SLOT(showSettings()) );
	connect(m_pConfig, SIGNAL(modified()), SLOT(onSettingsChanged()));

#ifndef F100_PLUGIN
	connect( m_RFHandler, SIGNAL(fileSelected(QString)), SLOT(openCSPfile(QString)) );
	resize( m_pConfig->readEntry("Geometry", "hand_width", 400),
	m_pConfig->readEntry("Geometry", "hand_height", 250) );
	move( m_pConfig->readEntry("Geometry", "hand_x", 140),
	m_pConfig->readEntry("Geometry", "hand_y", 120) );
#endif

    clearWState( WState_Polished );
	
	Colors[TRANS] = m_pConfig->color();

	// creation de la liste
	m_liste.setAutoDelete( true );

	setAcceptDrops(true);

	m_cspName = QString::null;
	m_CSPloaded = false;
	m_modified = false;
	languageChange();
	setIcon( QPixmap::fromMimeSource(ICO) );

	ColorMode = NO_MODE;
	for(int i=1; i<argc; i++) 
	{
		QString file(argv[i]);
		if(file.lower().endsWith(".csp")) openCSPfile(file);
		else addSprite(new Sprite(this, m_pConfig,file),(argc<4));
	}

	if(m_liste.count()==0) enableActions( false );

	int M;
	if(ColorMode==NO_MODE) M = m_pConfig->mode();
	else M = (ColorMode==MODE_5 ? GXLIB_10 : DRAWLIB_14);
	setMode(M, m_pConfig->language());

	spritesTable->adjustColumn( COL_CODE );
	spritesTable->adjustColumn( COL_NAME );
	spritesTable->adjustColumn( COL_APER );
	spritesTable->adjustColumn( COL_LARG );
	spritesTable->adjustColumn( COL_HAUT );
}

///////////////////////////////////////////////////////////////////////////////////////

void Handler::languageChange()
{
    QHeader *th = spritesTable->horizontalHeader();
	th->setLabel( COL_CODE, tr("Code","Title of the 1st column") );
    th->setLabel( COL_NAME, tr("Name","Title of the 2nd column") );
    th->setLabel( COL_APER, tr("Preview","Title of the 3rd column") );
    th->setLabel( COL_LARG, tr("Width","Title of the 4th column") );
    th->setLabel( COL_HAUT, tr("Height","Title of the 5th column") );

	// actions
    MenuBar->changeItem(1, tr("&File","Title of the first menu"));

    fileOpenAction->setMenuText(tr("&Open...","Item of \'File' Menu"));
    fileOpenAction->setText(tr("Open an existing CSP file","Item Description"));
    fileOpenAction->setAccel(tr("Ctrl+O","Accelerator for the \'Open\' action"));
 
    fileCloseAction->setMenuText(tr("&Close the file","Item of \'File' Menu"));
    fileCloseAction->setText(tr("Close the current CSP file","Item Description"));
 
	fileSaveAction->setMenuText(tr("&Save","Item of \'File' Menu"));
	fileSaveAction->setText(tr("Save the current CSP file","Item Description"));
	fileSaveAction->setAccel(tr("Ctrl+S","Accelerator for the \'Save\' action"));

	fileSaveAsAction->setMenuText(tr("Save &As...","Item of \'File' Menu"));
	fileSaveAsAction->setText(tr("Choose another file name to save the current data's","Item Description"));

	fileMenu->changeItem(RECENTFILES_ID,tr("&Recent files","Item of the \'File\' Menu"));

	fileExitAction->setMenuText(tr("&Quit","Item of \'File' Menu"));
    fileExitAction->setText(tr("Quit Sprite Maker","Item Description"));
	fileExitAction->setAccel(tr("Ctrl+Q","Accelerator for the \'Quit\' action"));

    helpAboutAction->setMenuText(tr("A&bout...","Item of \'File' Menu"));
	helpAboutAction->setText(tr("A short description of this software","Item Description"));
	helpAboutAction->setAccel(QString("F1"));

    MenuBar->changeItem(2, tr("&Sprites","Title of the second menu"));

	fileNewAction->setMenuText(tr("&New","Item of \'Sprites' Menu"));
	fileNewAction->setText(tr("New Sprite","Item Description"));
	fileNewAction->setAccel(QString("F2"));

	fileImportAction->setMenuText(tr("&Import pictures","Item of \'Sprites' Menu"));
	fileImportAction->setText(tr("Import pictures","Item Description"));
	fileImportAction->setAccel(QString("F3"));

	fileImportBMPAction->setMenuText(tr("&Convert and code a 128*64 picture","Item of \'Sprites' Menu"));
	fileImportBMPAction->setText(tr("Convert and code a 128*64 picture","Item Description"));
	fileImportBMPAction->setAccel(QString("F4"));

 	fileExportAction->setMenuText(tr("Export a &bitmap","Item of \'Sprites' Menu"));
    fileExportAction->setText(tr("Export a bitmap","Item Description"));

	codeAction->setMenuText(tr("&Make the code for all sprites","Item of \'Sprites' Menu"));
    codeAction->setText(tr("Make the code for all sprites","Item Description"));

	editCurAction->setMenuText(tr("Edit the &sprite","Item of \'Sprites' Menu"));
    editCurAction->setText(tr("Edit the sprite","Item Description"));
	editCurAction->setAccel(QString("F5"));

	moveCurUpAction->setMenuText(tr("Move the sprite &up","Item of \'Sprites' Menu"));
    moveCurUpAction->setText(tr("Move the sprite up","Item Description"));

	moveCurDownAction->setMenuText(tr("Move the sprite &down","Item of \'Sprites' Menu"));
    moveCurDownAction->setText(tr("Move the sprite down","Item Description"));

	deleteCurAction->setMenuText(tr("Dele&te the sprite","Item of \'Sprites' Menu"));
    deleteCurAction->setText(tr("Delete the sprite","Item Description"));

	editPrefAction->setMenuText(tr("Settin&gs","Item of \'Sprites' Menu"));
	editPrefAction->setText(tr("Change the settings","Item Description"));

    toolBar->setLabel(tr("Tools","Toolbar name")); 

    QString caption = "Sprite Maker - ";
	caption += ( m_cspName.isEmpty() ? tr("Untitled","Default caption") : m_cspName );
	if(m_modified==true) caption+="*";
	setCaption(caption);
}

///////////////////////////////////////////////////////////////////////////////////////

