#include <qapplication.h>
#include <qstring.h>
#include <qfileinfo.h>
#include <math.h>

#ifdef F100_PLUGIN
#	include <F100_Op.h>
#	include <ToolWindow.h>
#	include <IMainWindow.h>
#	include <IOptionWindow.h>
#	include "config.h"
#	include "config_f100.h"
#endif

#include "handler.h"

#define NOIR		 0
#define BLANC		 1
#define GRIS		 2
#define TRANS		 3
#define GRIS_CLAIR   4
#define GRIS_FONCE   5

QColor Colors[6] = 
	{ QColor(0,0,0), QColor(255,255,255), QColor(128,128,128), 
		QColor(255,255,0), QColor(192,192,192),	QColor(64,64,64)  };
QString ProgramPath;

uchar ColorMode;

void swap(int &a, int &b) { int save=a; a=b; b=save; }

int Round(double a)
{
	int f = (int)floor(a);
	return f + (a-f >= 0.5 );
}

QRect makeRect(QPoint p1, QPoint p2)
{
	int x = ( p1.x()<p2.x() ? p1.x() : p2.x() );	// MIN(x)
	int y = ( p1.y()<p2.y() ? p1.y() : p2.y() );	// MIN(y)
	int w = p1.x()-p2.x(); if(w<0) w=-w; w++;		// largeur
	int h = p1.y()-p2.y(); if(h<0) h=-h; h++;		// hauteur

	return QRect(x,y,w,h);
}

#ifdef F100_PLUGIN
class SpriteMaker : public CToolWindow
{
public:
	SpriteMaker(CONSTRUCTOR_PARAMDEF)
		: CToolWindow(CALL_PARAM) {
		Config* pConfig = (Config*)GetMainWindow()->GetOptionWindow()->GetSettingsWidget("SpriteMaker");
		m_pCentralWidget = new Handler(0, NULL,	pConfig);
		SetCentralWidget(m_pCentralWidget);
		m_pCentralWidget->installEventFilter(this);
		connect(m_pCentralWidget, SIGNAL(closed()),       this, SLOT(close()));
		connect(m_pCentralWidget, SIGNAL(stateChanged()), this, SIGNAL(StateChanged()));
		DoShowCaption();
		setIcon(QPixmap::fromMimeSource(ICO));
	}
	int Init(INIT_PARAMDEF) {
		m_pCentralWidget->openCSPfile(sFileName);
		return 1;
	}
	bool eventFilter(QObject* watched, QEvent* e ) {
		if(watched == m_pCentralWidget) {
			if(e->type() == QEvent::CaptionChange) {
				DoShowCaption();
			}
		}
		return CToolWindow::eventFilter(watched, e);
	}
	void DoShowCaption() {
		setCaption(m_pCentralWidget->caption());
	}
	void DoInitWords() {};
	int	LoadFrom(QDomElement& element) {
		if(CToolWindow::LoadFrom(element) == 1) {
			QString s = ConvertToFullPath(element.attribute("m_cspName"));
			m_pCentralWidget->openCSPfile(s);
			return 1;
		}
		return -1;
	}
	int	SaveTo(QDomElement& element) {
		if(CToolWindow::SaveTo(element) == 1) {
			element.setAttribute("m_cspName", ConvertToRelativePath(m_pCentralWidget->m_cspName));
			return 1;
		}
		return -1;
	}
private:
	Handler* m_pCentralWidget;
};

EXPORT_SYMB void Tool_Request(F100_Op op, void** ret, IMainWindow* ptr1, Qt::WFlags ptr2, QTabWidget* ptr3, QWidget* parent)
{
	switch(op)
	{
	case OP_TOOLNAME:
		*((QString*)ret) = "Sprite Maker";
		break;
	case OP_BUILDDATE:
		*ret = (void*)__DATE__;
		break;
	case OP_VERSION:
#ifndef _DEBUG
		*ret = (void*)"1.00";
#else
		*ret = (void*)"1.00 (Debug)";
#endif
		break;
	case OP_AUTHOR:
		*ret = (void*)"Patte Julien";
		break;
	case OP_CREATEINSTANCE:
		//Fentre non 'Dockable' et non possibilit de transparence
		*ret = new SpriteMaker(parent, ptr1, ptr2, NULL/*ptr3*/, FALSE);
		break;
	case OP_WINDOWTYPESTR:
		*ret = (void*)"SpriteMaker";
		break;
	case OP_FILEFILTER:
		*((QString*)ret) = QString::null;
		break;
	case OP_DESCRIPTION:
		*((QString*)ret) = Handler::tr("Allows you to create sprite code");
		break;
	case OP_SUPPORTEDFILE:
		*ret = (void*)"csp";
		break;
	case OP_SETTINGSWIDGET:
		*ret = new Config(ptr1);
		break;
	case OP_TOOLBARICON:
		*((QPixmap*)ret) = QPixmap::fromMimeSource(ICO);
		break;
	}
}
#else
int main( int argc, char ** argv )
{
	QApplication a( argc, argv );

	// repertoire du programme
	ProgramPath = QFileInfo(argv[0]).dirPath(true) + "/";
	QMimeSourceFactory::defaultFactory()->addFilePath(ProgramPath + "images");

	// fentre principale
	Handler hand( argc, argv );
	a.setMainWidget(&hand);
	hand.show();

	return a.exec();
}

#endif
