
#include "paintzone.h"

#include <qpainter.h>
#include <qrect.h>
#include <qevent.h>
#include <qimage.h>
#include "select.h"

PaintZone::PaintZone(QWidget* parent)
	: QScrollView(parent,"paintzone", WNoAutoErase|WStaticContents)
{
	viewport()->setBackgroundColor(QColor(210,210,210)); 
	viewport()->setMouseTracking(true);

	m_label = new PaintLabel(viewport());
	addChild(m_label);
	m_label->setMouseTracking(true);
}

///////////////////////////////////////////////////////////////////////////////////////

PaintZone::~PaintZone()
{
}

///////////////////////////////////////////////////////////////////////////////////////

QPoint PaintZone::PlotToPixel(int x, int y) { return QPoint( x/(m_zoom+1), y/(m_zoom+1) ); }
QPoint PaintZone::PlotToPixel(QPoint plot) { return PlotToPixel(plot.x(),plot.y()); }
QPoint PaintZone::PixelToPlot(int x, int y) { return QPoint( x*(m_zoom+1), y*(m_zoom+1) ); }
QPoint PaintZone::PixelToPlot(QPoint pxl) { return PixelToPlot(pxl.x(),pxl.y()); }

///////////////////////////////////////////////////////////////////////////////////////

void PaintZone::init()
{
	m_data_saved.resize(m_largeur,m_hauteur);
	m_data_saved.fill(255); 

	QPixmap image( (1+m_zoom)*m_largeur+1, (1+m_zoom)*m_hauteur+1 );
	m_label->setPixmap(image);
	refresh( QRect(0,0,m_largeur,m_hauteur) );
}

///////////////////////////////////////////////////////////////////////////////////////

void PaintZone::setConfig(Tab* data, int zoom)
{
	m_data = data;
	m_largeur = data->width();
	m_hauteur = data->height();
	m_zoom = zoom;
	m_globalzone = QRect(0,0,m_largeur,m_hauteur);
	init();
}

///////////////////////////////////////////////////////////////////////////////////////

	// rafraichit les pixels du modele dans le rectangle indique par rx, ry, rw et rh
void PaintZone::refresh( QRect zone )
{
	zone = zone & m_globalzone;
	
	int data;
	QPoint plot;
	QPixmap pixmap(m_zoom+2, m_zoom+2);
	QPainter p(m_label->pixmap());
	QRect selRect = m_select->position();

			p.setPen( QColor(210,210,210 ) );

	// pixels (grossis)
	for( int i=zone.left(); i<=zone.right(); i++ ) 
		for( int j=zone.top(); j<=zone.bottom(); j++ )
	{
		if( selRect.contains(i,j) && (m_select->status()==Ready || m_select->status()==Moving) )
		{
			data = m_select->m_data[j-selRect.top()][i-selRect.left()];
		}
		else data = m_data->element(j,i); // on va lire dans le tab du viewer

		if( m_select->status() != Hidden || data != m_data_saved[j][i] )
		{
			p.setBrush( Colors[data] );
			plot = PixelToPlot(i,j);
			p.drawRect(plot.x(), plot.y(), m_zoom+2, m_zoom+2 );

			//plot=PixelToPlot(i,j);
			//bitBlt(m_label->pixmap(), plot, &pixmap );
			//m_label->update( plot.x(), plot.y(), plot.x()+m_zoom+2, plot.y()+m_zoom+2 );

			m_data_saved[j][i] = data;
		}
	}
			
	p.end();
	
	plot = PixelToPlot(zone.left(),zone.top());
	m_label->update( plot.x(), plot.y(),		// update global
						zone.width()*(m_zoom+1)+1, zone.height()*(m_zoom+1)+1  );


	if( m_select->status() == Cleared ) m_select->setStatus(Hidden); // le cadre a t effac

	if( m_select->status() != Hidden )	// cadre de la selection
	{
		QPoint topleft = PixelToPlot(selRect.topLeft());
		QPoint bottomright = PixelToPlot(selRect.bottomRight());
		bottomright = QPoint( bottomright.x()+m_zoom+1, bottomright.y()+m_zoom+1 );
		p.begin(m_label->pixmap());
		p.setPen( QPen(red,0,DashLine) );
		p.setBrush( NoBrush );
		p.drawRect( QRect(topleft,bottomright) );
		p.end();
		m_label->update( QRect(topleft,bottomright) );
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void PaintZone::contentsMousePressEvent(QMouseEvent* e)
{ 
	ButtonState btn = e->button();
	if(btn!=LeftButton && btn!=RightButton) return;
	QPoint pxl = PlotToPixel(e->pos());
	if( !QRect(0,0,m_largeur,m_hauteur).contains(pxl) ) return;

	emit mousePressed( pxl, btn ); 
}

///////////////////////////////////////////////////////////////////////////////////////

void PaintZone::contentsMouseReleaseEvent(QMouseEvent* e)
{ 
	ButtonState btn = e->button();
	if(btn!=LeftButton && btn!=RightButton) return;
	emit mouseReleased( btn ); 
}

///////////////////////////////////////////////////////////////////////////////////////

void PaintZone::contentsMouseMoveEvent(QMouseEvent* e)
{ 
	QPoint pxl = PlotToPixel(e->pos());
	if( !QRect(0,0,m_largeur,m_hauteur).contains(pxl) ) return;

	emit mouseMoved( pxl ); 
}
