#include "select.h"


Selection::Selection(Selection &sel)
	: QObject()
{
	m_largeur = sel.m_largeur;
	m_hauteur = sel.m_hauteur;

	m_position = sel.position();
	m_status = sel.status();

	m_data.copy(sel.m_data);
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::clear()
{
	m_data.clear();
	m_largeur = m_hauteur = 0;
	m_status = Hidden;
	emit cleared();
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::wannaSet(QPoint initPoint)
{
	clear();
	m_basePointForSet = initPoint;
	m_position = makeRect(m_basePointForSet, m_basePointForSet);
	m_status = Setting;
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::set(QPoint newPos)
{
	m_position = makeRect(m_basePointForSet, newPos);
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::stopSetting(Tab &dataTab, int coulfond)
{
	m_largeur = m_position.width();
	m_hauteur = m_position.height();
	m_data.copy(dataTab,m_position);
	
	dataTab.fill(m_position,coulfond);

	m_status = Ready;
	emit ready();
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::wannaMove(QPoint initPoint)
{
	m_basePointForMove = initPoint;
	m_status = Moving;
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::move(QPoint newPos)
{
	QRect oldRect = m_position;
	QPoint pos = m_position.topLeft();
	int depX = newPos.x() - m_basePointForMove.x();
	int depY = newPos.y() - m_basePointForMove.y();

	pos = QPoint( pos.x()+depX, pos.y()+depY );
	m_position.moveTopLeft( pos );
	m_basePointForMove = newPos;
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::stopMoving()
{
	m_status = Ready;
	emit ready();
}

///////////////////////////////////////////////////////////////////////////////////////

void Selection::fusion(Tab &dataTab)
{
	// fusion des donnees de la selection dans l'image

	QRect pos = m_position & QRect(0,0,dataTab.width(),dataTab.height());

	int c = m_position.left();
	int l = m_position.top();
	for(int i=pos.left(); i<=pos.right(); i++) for( int j=pos.top(); j<=pos.bottom(); j++)
		dataTab[j][i] = m_data[j-l][i-c];

	clear();
	m_status = Cleared;
	m_position = pos;
}
