#include <qfile.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qfileinfo.h>
#include <qfiledialog.h>

#include "sprite.h"
#include "viewer.h"
#include "converter.h"
#include "config.h"
#include "config_f100.h"

Sprite::Sprite(QWidget* parent, Config* pConfig, QString imagefile)
	: QWidget(parent,"sprite") 
	, m_editor(NULL)
	, m_orig(0,0)
	, m_selected(true)
	, m_pConfig(pConfig)
{
	hide();
	loadFile(imagefile);
}

///////////////////////////////////////////////////////////////////////////////////////

Sprite::Sprite(QWidget* parent, Config* pConfig, Tab &tab, QString nom, QPoint orig)
	: QWidget(parent,"sprite") 
	, m_editor(NULL)
	, m_nom(nom)
	, m_tableau(tab)
	, m_orig(orig)
	, m_selected(true)
	, m_changed(false)
	, m_correct(true)
	, m_pConfig(pConfig)
{
	hide();
	m_largeur = m_tableau.width();
	m_hauteur = m_tableau.height();
	drawImage();
}

///////////////////////////////////////////////////////////////////////////////////////

Sprite::Sprite(QWidget* parent, Config* pConfig)
	: QWidget(parent,"sprite") 
	, m_editor(NULL)
	, m_nom(tr("newSprite","Default sprite name"))
	, m_orig(0,0)
	, m_selected(true)	
	, m_changed(false)
	, m_correct(true)
	, m_pConfig(pConfig)
{
	hide();

	m_largeur = m_pConfig->readEntry("Pref", "last_spr_l", 16);
	m_hauteur = m_pConfig->readEntry("Pref", "last_spr_h", 16);

	m_tableau.resize(m_largeur,m_hauteur);
	m_tableau.fill(TRANS);
	m_image.resize(m_largeur,m_hauteur);
	m_image.fill( Colors[TRANS] );
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::loadFile(QString &imagefile)
{
	m_correct=false;
	
	Converter conv(this, m_pConfig,imagefile);
	if(conv.exec()==QDialog::Rejected) return;

	conv.getConvertedTab(m_tableau);

	m_hauteur = m_tableau.height();
	m_largeur = m_tableau.width();
	m_nom = conv.nomSprite();
	m_changed=true;
	m_correct=true;

	//if( m_correct==false )
	//{
	//	QMessageBox::warning(this,"Erreur",
	//		QString("Fichier invalide ou de type inconnu:\n%1").arg(imagefile),"Ok");
	//}
	/*else*/ drawImage();
}

///////////////////////////////////////////////////////////////////////////////////////

bool Sprite::saveTo(QFile*)
{
	bool ok = true;


	return ok;
}

///////////////////////////////////////////////////////////////////////////////////////

bool Sprite::loadFrom(QFile*)
{
	bool ok = true;


	return ok;
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::drawImage()
{
	QPixmap buffer(m_largeur, m_hauteur);
	QPainter p;
	for( int i=0; i<m_hauteur; i++ )  for( int j=0; j<m_largeur; j++ )
	{
		p.begin(&buffer);
		p.setPen( Colors[ m_tableau[i][j] ] );
		p.drawPoint(j,i);
		p.end();
	}
	m_image=buffer;
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::edit()
{
	if( m_editor==0 ) 
	{
		m_editor = new Viewer(this, m_pConfig);
		connect( m_editor, SIGNAL(dataChanged()), SLOT(updateData()) );
		connect( m_editor, SIGNAL(closingEditor()), SLOT(endEdit()) );
		connect( this, SIGNAL(modeChanged(int)), m_editor, SLOT(changeMode(int)) );
		m_editor->show();
	}
	else m_editor->setActiveWindow();
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::endEdit() 
{ 
	delete m_editor; 
	m_editor=NULL; 
	emit editionEnded(this); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::updateData()
{
	m_nom = m_editor->nom();
	m_largeur = m_editor->largeur();
	m_hauteur = m_editor->hauteur();

	m_tableau = m_editor->tableau();
	m_orig = m_editor->origin();
	drawImage();
	emit dataChanged(this);
}

///////////////////////////////////////////////////////////////////////////////////////

void Sprite::changeTransColor()
{
	drawImage();
	if(m_editor!=NULL) m_editor->changeTransColor();
}

