#ifndef __STACK
#define __STACK

template <class Type>
struct StackNode
{
	StackNode(Type* v=NULL, StackNode* p=NULL, StackNode* n=NULL) 
		: value(v), prev(p), next(n) {}
	~StackNode() { }
	Type* value;
	StackNode* prev;
	StackNode* next;

};

template <class Type>
class Stack
{
public:
	Stack(bool canDelete=false, int tailleMax=-1) 
		: m_size(0), m_head(NULL), m_last(NULL), m_max(tailleMax), m_canDel(canDelete) {}
	~Stack() { clear(); }

	void push(Type &item);
	Type & pop();
	Type & top() { return (m_head ? *(m_head->value) : Type()); } 
	int size() { return m_size; }
	void clear();
	bool isEmpty() { return m_head==NULL; }
	
protected:
	typedef StackNode<Type> Node;
	int m_size;
	Node* m_head;
	Node* m_last;
	int m_max;
	bool m_canDel;
};

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Stack<Type>::push(Type &item) 
{ 
	Node* oldHead = m_head;
	m_head = new Node(&item,NULL,oldHead); 
	if(oldHead) oldHead->prev=m_head;
	m_size++; 
	if(m_max!=-1 && m_size>m_max)
	{
		Node* vict = m_last;
		m_last = m_last->prev;
		if(m_canDel) delete vict->value;
		delete vict;
		m_last->next=NULL;
		m_size--;
	}
	if(m_last==NULL) m_last = m_head;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Type & Stack<Type>::pop() 
{ 
	if(m_head==NULL) return *(new Type());
	
	Type* ptr = m_head->value;
	Node* vict = m_head;
	m_head = m_head->next;
	if(m_last==vict) m_last = NULL;
	delete vict;
	m_size--;
	return *ptr;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Stack<Type>::clear()
{
	for(Node* ptr = m_head; ptr!=NULL;)
	{
		Node* vict = ptr;
		ptr = ptr->next;
		delete vict;
	}

	m_head = m_last = NULL;
	m_size = 0;
}

#endif
