#include "tab.h"
Tab::Tab(int larg, int haut)
{
	m_tab = new uchar* [haut];
	for(int i=0; i<haut; i++) m_tab[i] = new uchar[larg];
	m_larg = larg;
	m_haut = haut;
}

///////////////////////////////////////////////////////////////////////////////////////

Tab & Tab::copy(Tab &modele)
{
	clear();
	
	resize(modele.size());
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) 
		m_tab[i][j] = modele.m_tab[i][j];

	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

Tab & Tab::copy(Tab &modele, QRect rect)
{
	rect = rect & QRect(0,0,modele.m_larg,modele.m_haut);

	resize(rect.size());

	for(int i= 0; i<m_haut; i++)
		for(int j=0; j<m_larg; j++)
			m_tab[i][j] = modele.m_tab[ i + rect.top() ][ j + rect.left() ];
	
	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

Tab & Tab::part(QRect r)
{
	Tab m;
	m.copy(*this,r);
	return m;
}

///////////////////////////////////////////////////////////////////////////////////////

void Tab::resize(int larg, int haut)
{
	if(m_larg == larg && m_haut == haut) return;
	if(larg <= 0 || haut <= 0)  { clear(); return; }
	
	int largmin = (larg<m_larg? larg : m_larg);
	int hautmin = (haut<m_haut? haut : m_haut);

	uchar** newtab = new uchar* [haut];
	for(int r=0; r<haut; r++) newtab[r] = new uchar[larg];

	for(int i=0; i<hautmin; i++) for(int j=0; j<largmin; j++) newtab[i][j] = m_tab[i][j];

	clear();
	m_tab = newtab;
	m_haut = haut;
	m_larg = larg;
}

///////////////////////////////////////////////////////////////////////////////////////

Tab & Tab::adaptFromPtr(int larg, int haut, uchar* data)
{
	resize(larg,haut);
	int offset = 0;
	for(int i=0; i<haut; i++) for(int j=0; j<larg; j++) m_tab[i][j] = data[offset++];

	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

uchar* Tab::toNewPtr()
{
	uchar* newptr = new uchar[m_larg*m_haut];
	int offset=0;
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) newptr[offset++] = m_tab[i][j];
	return newptr;
}

///////////////////////////////////////////////////////////////////////////////////////

void Tab::clear()
{
	if(isNull()) return;
	
	for(int i=0; i<m_haut; i++) delete[] m_tab[i];
	delete[] m_tab;

	m_tab = NULL;
	m_larg = 0;
	m_haut = 0;
}

///////////////////////////////////////////////////////////////////////////////////////

void Tab::fill(uchar value)
{
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) m_tab[i][j] = value;
}

///////////////////////////////////////////////////////////////////////////////////////

void Tab::fill(QRect rect, uchar value)
{
	rect = rect & QRect(0,0,m_larg,m_haut);
	for(int i=rect.top(); i<=rect.bottom(); i++) 
		for(int j=rect.left(); j<=rect.right(); j++) 
			m_tab[i][j] = value;
}

///////////////////////////////////////////////////////////////////////////////////////

void Tab::flip(bool horiz)
{
	if(horiz==false) // retournement vertical
	{
		for(int i=0; i<m_haut/2; i++)
		{
			uchar* row = m_tab[i];
			m_tab[i] = m_tab[m_haut-i-1];
			m_tab[m_haut-i-1] = row;
		}
	}
	else // retournement horizontal
	{
		for(int i=0; i<m_haut; i++)
		{
			uchar* row = m_tab[i];
			for(int j=0; j<m_larg/2; j++)
			{
				uchar save = row[j];
				row[j] = row[m_larg-j-1];
				row[m_larg-j-1] = save;
			}
		}	
	}
}
