#include <qevent.h>
#include <qmessagebox.h>
#include <qbuttongroup.h> 
#include <qradiobutton.h>
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qaction.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qaccel.h>
#include <qcursor.h>

#include "viewer.h"
#include "paintzone.h"
#include "colorbutton.h"
#include "sprite.h"
#include "coder.h"
#include "select.h"
#include "doitem.h"
#include "icones.h"
#include "config.h"
#include "config_f100.h"

#include "../mylibs/myBackuphdlr.h"

#define ABS(a) (a>=0?a:-a)


Prevue::Prevue(QWidget* parent) : QWidget(parent,"prevue",WNoAutoErase)
{
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}

///////////////////////////////////////////////////////////////////////////////////////

void Prevue::setPixmap(QPixmap &pix)
{
	m_pixmap.resize(pix.size());
	bitBlt(&m_pixmap,0,0,&pix);
	m_posPix = m_pixmap.rect();
	m_posPix.moveCenter(QPoint(width()/2,height()/2));
	repaint(false);
}

///////////////////////////////////////////////////////////////////////////////////////

void Prevue::redrawPix(QRect r) 
{ 
	r.moveBy(m_posPix.left(),m_posPix.top());
	repaint(r,false);
}

///////////////////////////////////////////////////////////////////////////////////////

void Prevue::redrawPix() 
{ 
	repaint(m_posPix,false);
}

///////////////////////////////////////////////////////////////////////////////////////

void Prevue::paintEvent(QPaintEvent* e)
{
	QRect r = e->rect();
	QPixmap pix(r.size());
	pix.fill(colorGroup().background());
	if(r.intersects(m_posPix))
	{
		QRect inter = m_posPix & r; // intersection de la zone a repeindre et du pixmap
		QPoint pt = inter.topLeft();
		inter.moveBy(-m_posPix.left(),-m_posPix.top());
		pt-=r.topLeft();
		bitBlt(&pix,pt,&m_pixmap,inter);
	}

	bitBlt(this,r.topLeft(),&pix);
}	

///////////////////////////////////////////////////////////////////////////////////////

void Prevue::resizeEvent(QResizeEvent* e)
{
	m_posPix.moveCenter(QPoint(e->size().width()/2,e->size().height()/2));
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void Viewer::mousePressed(QPoint pxl, ButtonState btn)
{ 
	if(m_souris_gauche_presse) mouseReleased(LeftButton);
	if(m_souris_droite_presse) mouseReleased(RightButton);

	if(m_shift_enfonce) // pipette
	{
		setCouleurBouton(btn,m_tableau[pxl]); 
		return;
	}

	int bouton;
	if(btn == LeftButton) {	m_souris_gauche_presse=true; bouton=0; }
	else { m_souris_droite_presse=true; bouton=1; }

	switch(m_tool)
	{
	case PEN: 

		if(m_curDoItem) delete m_curDoItem;
		m_curDoItem = new DoItem();
		setPixel(pxl, m_couleursChoisies[bouton]);
		break;

	case FILL: 

		m_tab_saved = m_tableau; 
		fillColor(pxl.x(), pxl.y(), m_tableau[pxl], m_couleursChoisies[bouton] );
		emit imageChanged(QRect(0,0,m_largeur,m_hauteur));
		break;

	case LINE: case RECT: 

		m_tab_saved = m_tableau; 
		m_basePoint = pxl; 
		break;

	case SELECT: 

		if(m_curDoItem) delete m_curDoItem;

		if(m_selection.status()==Ready)	// si une selection est deja dessinee sur l'image
		{
			if(m_selection.position().contains(pxl))
			{
				// l'utilisateur a appuy sur la selection pour la deplacer
				m_curDoItem = new DoItem(TYPE_SEL_MOVE);
				m_curDoItem->oldSelPos() = m_selection.position();
				m_selection.wannaMove(pxl);
			}
			else	// sinon on initialise une nouvelle selection
			{
				fusionSel();
				m_curDoItem = new DoItem(TYPE_SEL_CREATE);
				m_selection.wannaSet(pxl);
			}			
		}
		else 
		{
			m_curDoItem = new DoItem(TYPE_SEL_CREATE);
			m_selection.wannaSet(pxl);
		}

		break;
	}
	paintzone->setFocus();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::mouseReleased(ButtonState btn)
{ 
	if(btn == LeftButton)
	{
		if(!m_souris_gauche_presse) return;
		else m_souris_gauche_presse=false;
	}
	else 
	{
		if(!m_souris_droite_presse) return;
		else m_souris_droite_presse=false;
	}

	switch(m_tool)
	{
	case PEN:

		NotifyModif();
		break;

	case FILL:

		if(! m_tab_saved.isNull()) 
		{
			if(m_curDoItem) delete m_curDoItem;
			m_curDoItem = new DoItem(QPoint(0,0), m_tab_saved, m_tableau );

			m_tab_saved.clear();
			redrawPrevue();

			NotifyModif();
		}
		break;

	case LINE: case RECT:

		if(! m_tab_saved.isNull()) 
		{ 
			QRect rect = makeRect(m_basePoint, m_secondPoint);
			if(m_curDoItem) delete m_curDoItem;
			m_curDoItem = new DoItem(rect.topLeft(), m_tab_saved.part(rect), m_tableau.part(rect) );
			m_tab_saved.clear();
			redrawPrevue();

			NotifyModif(); // s'occupe de l'envoi de m_curDoItem
		}
		break;

	case SELECT:

		if(m_selection.status() == Moving) 
		{
			m_selection.stopMoving();
			m_curDoItem->newSelPos() = m_selection.position();
		}
		else if(m_selection.status() == Setting) 
		{
			m_selection.stopSetting(m_tableau, m_couleursChoisies[1]);
			m_curDoItem->oldSelPos() = m_selection.position();
			emit imageChanged(m_selection.position());
		}
		
		NotifyModif();

		break;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::mouseMoved(QPoint position)
{ 
	if( position == m_oldPlaceCursor ) return;

	statusBar()->message( QString("(%1,%2)").arg(position.x()).arg(position.y()) );
	paintzone->setFocus();

	m_oldPlaceCursor = position;

	if( !m_souris_gauche_presse && !m_souris_droite_presse) return;

	switch(m_tool)
	{
	case PEN:

		if( m_souris_gauche_presse ) setPixel( position, m_couleursChoisies[0] );
		else if( m_souris_droite_presse ) setPixel( position, m_couleursChoisies[1] );
		break;

	case FILL:

		break;

	case LINE: case RECT:
	{
		m_tableau = m_tab_saved; 

		m_secondPoint = position;
		if(m_tool == LINE) drawline(m_basePoint, m_secondPoint);
		else drawrect(m_basePoint, m_secondPoint);

		QRect rect = makeRect(m_basePoint, m_secondPoint);
		emit imageChanged( rect|m_oldRect );
		m_oldRect=rect;
		
		break;
	}
	case SELECT:

		if(m_selection.status() == Moving)
		{
			QRect oldPos = m_selection.position();
			m_selection.move(position);
			emit imageChanged(m_selection.position()|oldPos);
		}
		else if(m_selection.status() == Setting) 
		{
			QRect oldPos = m_selection.position();
			m_selection.set(position);
			emit imageChanged(m_selection.position()|oldPos);
		}

		break;

	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::keyPressEvent(QKeyEvent* e)
{
	if(e->key()!=Key_Shift) e->ignore();
	else
	{
		m_shift_enfonce = true;
		paintzone->setCurs( *m_curs_pip );
		e->accept();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::keyReleaseEvent(QKeyEvent* e)
{
	if(e->key()!=Key_Shift) e->ignore();
	else
	{
		m_shift_enfonce = false;
		paintzone->setCurs( m_tool==PEN ? *m_curs_pen : CrossCursor );
		e->accept();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

	// affiche un pixel sur l'image
void Viewer::setPixel( QPoint pxl, int couleur)
{ 
	m_curDoItem->listePxl().append( new Pixel(pxl,m_tableau[pxl],couleur) );

	m_tableau[pxl] = couleur;
	
	 //previsualisation
	QPainter p(m_prevue->pixmap());
	p.setPen(Colors[couleur]);
	p.drawPoint(pxl);
	p.end();
	m_prevue->redrawPix(QRect(pxl,pxl));
	emit imageChanged(QRect(pxl,pxl));
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::fillColor(int x, int y, uchar oldColor, uchar newColor)
{
	if(x>=0 && x<m_largeur && y>=0 && y<m_hauteur && m_tableau[y][x] == oldColor)
	{
		m_tableau[y][x] = newColor;
		fillColor(x + 1, y, oldColor, newColor);
		fillColor(x, y + 1, oldColor, newColor);
		fillColor(x - 1, y, oldColor, newColor);
		fillColor(x, y - 1, oldColor, newColor);
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::drawline(QPoint p1, QPoint p2)
{	
	if(p1.x()>p2.x()) { QPoint p = p1; p1 = p2;	p2 = p;	}
	
	int i,j;
	int A = p1.x();
	int B = p1.y();
	int C = p2.x();
	int D = p2.y(); 
	int coul;
	if(m_souris_gauche_presse) coul = m_couleursChoisies[0];
	else coul = m_couleursChoisies[1];

	statusBar()->message( QString(tr("Line from (%1,%2) to (%3,%4)  [%5,%6]","Status bar message")).
							arg(A).arg(B).arg(C).arg(D).arg(C-A+1).arg(ABS(D-B)+1)	);

	if(A==C)	// ligne verticale
	{
		if(B>D) swap(B,D);
		for(j=B; j<=D; j++) m_tableau[j][A] = coul;
	}
	else
	{
		double pente = ((double)(D-B))/(C-A);

		if( -1 <= pente && pente <= 1 )		// si |pente| <= 1
		{
			for(i=A; i<=C; i++)		// on balaie les x
			{
				j = Round( (i-A)*pente + B );	// on trouve y en fct de x
				m_tableau[j][i] = coul;
			}
		}
		else
		{
			if(pente<0) { swap(A,C); swap(B,D); }
			for(j=B; j<=D; j++)		// on balaie les y
			{
				i = Round( (j-B)/pente + A );	// on trouve x en fct de y
				m_tableau[j][i] = coul;
			}
		}
	}

}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::drawrect(QPoint p1, QPoint p2)
{
	QRect rect = makeRect(p1,p2);

	statusBar()->message( QString(tr("Rectangle from (%1,%2) to (%3,%4)  [%5,%6]","Status bar message")).
		arg(p1.x()).arg(p1.y()).arg(p2.x()).arg(p2.y()).arg(rect.width()).arg(rect.height())	);

	for(int i=rect.left(); i<=rect.right(); i++)
		for(int j=rect.top(); j<=rect.bottom(); j++)
		{
			if( i==rect.left() || i==rect.right() || j==rect.top() || j==rect.bottom() )
				m_tableau[j][i]=m_couleursChoisies[m_souris_droite_presse];	// bord du rectangle
			else
			{
				if(m_couleursChoisies[m_souris_gauche_presse]!=TRANS
						|| m_couleursChoisies[m_souris_droite_presse]==TRANS)
					m_tableau[j][i]=m_couleursChoisies[m_souris_gauche_presse];
			}
		}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::NewSprite(int larg, int haut)
{
	m_tableau.resize(larg,haut);
	m_tableau.fill(TRANS);
	m_largeur=larg;
	m_hauteur=haut;
	m_nom = tr("newSprite","Default sprite name");
	nomLineEdit->setText(m_nom);
	setCaption( tr( "Editor", "Caption" ) + " - " + m_nom );
	m_changed=true;
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::redrawPrevue()		// creation de l'image miniature
{
	QPixmap buffer(m_largeur, m_hauteur);
	QPainter p(&buffer);
	
	for( int i=0; i<m_hauteur; i++ )  for( int j=0; j<m_largeur; j++ )
	{
		p.setPen( Colors[m_tableau[i][j]] );
		p.drawPoint(j,i);
	}
	
	p.end();
	m_prevue->setPixmap(buffer);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::CreeImage()
{
	emit newImage(&m_tableau,m_zoom);
	redrawPrevue();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::setNewDim(int larg, int haut)
{
	if( larg==m_largeur && haut==m_hauteur ) return;
	
	if(m_selection.status()==Ready) fusionSel();
	
	QSize oldSize(m_largeur,m_hauteur), newSize(larg,haut);
	Tab* partBottom=NULL;
	Tab* partRight=NULL;
	if(haut<m_hauteur) partBottom = new Tab(m_tableau,QRect(0,haut,m_largeur,m_hauteur-haut));
	if(larg<m_largeur) partRight = new Tab(m_tableau,QRect(larg,0,m_largeur-larg,m_hauteur));
	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(oldSize,newSize,partBottom,partRight);

	m_tableau.resize(larg,haut);

	for(int i=0; i<haut; i++) for(int j=0; j<larg; j++) 
		if(i>=m_hauteur || j>=m_largeur) m_tableau[i][j] = TRANS;
		
	m_largeur=larg;
	m_hauteur=haut;

	largSpinBox->setValue(larg);
	hautSpinBox->setValue(haut);

	CreeImage();

	NotifyModif();
}

///////////////////////////////////////////////////////////////////////////////////////

bool Viewer::okToClear()
{
    int x=1;

	if ( m_changed ) 
	{
		QString msg = tr("Save the changes made on the sprite?","\'Ask for Save\' warning");
		x = QMessageBox::information( this, 
				tr("Save","Warning title"),
				msg, 
				tr("&Yes","Warning option"), 
				tr("&No","Warning option"), 
				tr("&Cancel","Warning option"),
				0, 1 );

		if( x!=2 && m_selection.status() == Ready ) fusionSel();

		if(x==0) emit dataChanged();
    }

    return (x!=2);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::fileNew()
{
//	if( okToClear() )
//	{
		NewSprite(m_largeur,m_hauteur);
		CreeImage();
//	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::fileSave()
{
	if( m_selection.status() == Ready ) fusionSel();
	emit dataChanged();
	m_changed = false;
	setCaption( tr("Editor","Caption of the window") + " - " + m_nom );
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::setNom(const QString &nom)
{
	m_nom = nom;
	if(nomLineEdit->text()!=nom) nomLineEdit->setText(nom);
	setCaption( tr("Editor","Caption of the window") + " - " + m_nom + ( m_changed? "*":"") );
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::closeEvent( QCloseEvent * e)
{
    if ( okToClear() ) 
	{
		emit closingEditor();
		e->accept();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::NotifyModif()
{
	if(m_curDoItem==NULL) 
	{
		//QMessageBox::warning(this,"Oups","Do Item nul!");
		return;
	}
	
	if(m_noDoItemAccepted) { if(m_curDoItem) delete m_curDoItem; }
	else emit newDoItem( m_curDoItem ); 
	m_curDoItem = NULL;

	if(m_changed==false)
	{
		setCaption( caption()+"*");
		m_changed=true;
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::undo(DoItemBase* itembase)
{
	DoItem* item;
	try { item = dynamic_cast<DoItem*>(itembase); }  catch(...) { return; }
	
	m_noDoItemAccepted = true;

	switch(item->type())
	{
	case TYPE_PIX: // liste de pixels
	{
		QPainter p(m_prevue->pixmap());
		QPtrList<Pixel>* list = & item->listePxl();
		for(int i=list->count()-1; i>=0; i--)
		{
			Pixel* pix = list->at(i);
			m_tableau[ pix->pos ] = pix->oldCol;
			p.setPen( Colors[ pix->oldCol ] );
			p.drawPoint( pix->pos );
		}

		p.end();
		m_prevue->redrawPix();
		emit imageChanged(QRect(0,0,m_largeur, m_hauteur));
		break;
	}
	case TYPE_PART_TAB: // morceau de l'image
	{
		QPainter p(m_prevue->pixmap());
		Tab &t = item->oldPart();
		int l = t.width(), h = t.height();
		int x = item->posPart().x(), y = item->posPart().y();
		for(int i=0; i<l; i++) for(int j=0; j<h; j++) 
		{
			m_tableau[j+y][i+x] = t[j][i];
			p.setPen( Colors[ t[j][i] ] );
			p.drawPoint(i+x,j+y);
		}

		p.end();
		m_prevue->redrawPix(QRect(x,y,l,h));
		emit imageChanged(QRect(x,y,l,h));
		break;
	}
	case TYPE_DIM: // dimensions
	{
		int larg = item->oldSize().width(), haut = item->oldSize().height();
		m_tableau.resize(larg,haut);

		if(haut>m_hauteur)
		{
			Tab &bottom = *(item->partBottom());
			for(int i=m_hauteur; i<haut; i++) for(int j=0; j<larg; j++) 
				m_tableau[i][j] = bottom[i-m_hauteur][j];
		}
		if(larg>m_largeur)	// si la largeur avait ete rduite
		{
			Tab &right = *(item->partRight());
			for(int i=0; i<haut; i++) for(int j=m_largeur; j<larg; j++) 
				m_tableau[i][j] = right[i][j-m_largeur];
		}
			
		m_largeur=larg;
		m_hauteur=haut;

		largSpinBox->setValue(larg);
		hautSpinBox->setValue(haut);

		CreeImage();
		break;
	}
	case TYPE_FLIP: // retournements
	{
		editFlip(item->flipHoriz());
		break;
	}
	case TYPE_SELECT: // selections
	{
		switch(item->typeSel())
		{
		case TYPE_SEL_CREATE: // on recolle la selection sur le tableau
			fusionSel(false);
			break;

		case TYPE_SEL_FUSION: // on dcolle la selection et on restaure ce qui tait en dessous
			{ 
			m_selection = *(item->oldSel());
			QRect pos = m_selection.position() & QRect(0,0,m_largeur,m_hauteur);
			Tab* tab = item->underSel();
			for(int i=0; i<tab->height(); i++) for(int j=0; j<tab->width(); j++) 
				m_tableau[i+pos.top()][j+pos.left()] = (*tab)[i][j];
			emit imageChanged(pos);
			
			break;
			}

		case TYPE_SEL_DELETE: // on fait rapparaitre la slection
			m_selection = *(item->oldSel());
			selectionReady();
			emit imageChanged(m_selection.position());
			break;
		case TYPE_SEL_MOVE:
			{
			QRect oldRect = m_selection.position();
			m_selection.wannaMove(oldRect.topLeft());
			m_selection.move(item->oldSelPos().topLeft());
			m_selection.stopMoving();
			emit imageChanged(oldRect|item->oldSelPos());
			break;
			}

		case TYPE_SEL_APPEAR:
			deleteSelection();
			break;
		}

		break;
	}
	}

	if(m_selection.status()==Ready)
	{
		selectionReady();
		toolSelectionAction->setOn(true);
	}
	
	m_noDoItemAccepted = false;
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::redo(DoItemBase* itembase)
{
	DoItem* item;
	try { item = dynamic_cast<DoItem*>(itembase); }  catch(...) { return; }
	
	m_noDoItemAccepted = true;

	switch(item->type())
	{
	case TYPE_PIX: // liste de pixels
	{
		QPainter p(m_prevue->pixmap());
		QPtrList<Pixel>* list = & item->listePxl();
		for(QPtrList<Pixel>::Iterator it=list->begin(); it!=list->end(); it++)
		{
			m_tableau[ (*it)->pos ] = (*it)->newCol;
			p.setPen( Colors[ (*it)->newCol ] );
			p.drawPoint( (*it)->pos );
		}

		p.end();
		m_prevue->redrawPix();
		emit imageChanged(QRect(0,0,m_largeur, m_hauteur));
		break;
	}
	case TYPE_PART_TAB: // morceau de l'image
	{
		QPainter p(m_prevue->pixmap());
		Tab &t = item->newPart();
		int l = t.width(), h = t.height();
		int x = item->posPart().x(), y = item->posPart().y();
		for(int i=0; i<l; i++) for(int j=0; j<h; j++) 
		{
			m_tableau[j+y][i+x] = t[j][i];
			p.setPen( Colors[ t[j][i] ] );
			p.drawPoint(i+x,j+y);
		}

		p.end();
		m_prevue->redrawPix(QRect(x,y,l,h));
		emit imageChanged(QRect(x,y,l,h));
		break;
	}
	case TYPE_DIM: // dimensions
	{
		int larg = item->newSize().width(), haut = item->newSize().height();
		m_tableau.resize(larg,haut);

		if(haut>m_hauteur)
		{
			for(int i=m_hauteur; i<haut; i++) for(int j=0; j<larg; j++) 
				m_tableau[i][j] = TRANS;
		}
		if(larg>m_largeur)
		{
			for(int i=0; i<haut; i++) for(int j=m_largeur; j<larg; j++) 
				m_tableau[i][j] = TRANS;
		}
			
		m_largeur=larg;
		m_hauteur=haut;

		largSpinBox->setValue(larg);
		hautSpinBox->setValue(haut);

		CreeImage();
		break;
	}
	case TYPE_FLIP: // retournements
	{
		editFlip(item->flipHoriz());
		break;
	}
	case TYPE_SELECT: // selections
	{
		switch(item->typeSel())
		{
		case TYPE_SEL_CREATE: // on recree la selection qu'on vient de fusionner
			{
			QRect pos = item->oldSelPos();
			m_selection.wannaSet(pos.topLeft());
			m_selection.set(pos.bottomRight());
			m_selection.stopSetting(m_tableau,m_couleursChoisies[1]);
			emit imageChanged(pos);
			break;
			}

		case TYPE_SEL_FUSION: // on recolle la selection
			fusionSel(false);
			break;

		case TYPE_SEL_DELETE:
			deleteSelection();
			break;

		case TYPE_SEL_MOVE:
			{
			QRect oldRect = m_selection.position();
			m_selection.wannaMove(oldRect.topLeft());
			m_selection.move(item->newSelPos().topLeft());
			m_selection.stopMoving();
			emit imageChanged(oldRect|item->newSelPos());
			break;
			}

		case TYPE_SEL_APPEAR:
			m_selection = *(item->oldSel());
			selectionReady();
			emit imageChanged(m_selection.position());
			break;
		}	
	
		break;
	}
	}

	if(m_selection.status()==Ready)
	{
		selectionReady();
		toolSelectionAction->setOn(true);
	}

	m_noDoItemAccepted = false;

}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::editCut()
{
	editCopy();
	deleteSelection();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::editCopy()
{	// enregistrement de la portion d'image dans le clipboard
	QRect rect = m_selection.position();
	int largeur = rect.width(); 
	int hauteur = rect.height();
	QPixmap pixmap(largeur, hauteur);
	QPainter p;
	
	for(int i=0; i<largeur; i++) for(int j=0; j<hauteur; j++)
	{
		p.begin(&pixmap);
		p.setPen( Colors[ m_selection.m_data[j][i] ] );
		p.setBrush( NoBrush );
		p.drawPoint(i,j);
		p.end();
	}

	qApp->clipboard()->setPixmap( pixmap, QClipboard::Clipboard );
}

///////////////////////////////////////////////////////////////////////////////////////

	// extraction de l'image dans le clipboard et creation d'une selection
void Viewer::editPaste()
{
	if( m_selection.status() == Ready ) fusionSel();

	QImage image = qApp->clipboard()->image( QClipboard::Clipboard );
	if( image.isNull() ) 
	{
		QMessageBox::warning(this, tr("Error","Title of Error window"), tr("The clipboard is empty!","Error message"), "OK");
		editPasteAction->setEnabled(false);
		return;
	}

	int larg = image.width();
	int haut = image.height();
	if(larg>m_largeur || haut>m_hauteur)
	{
		if(larg>128 || haut>100) 
		{
			QMessageBox::warning(this, tr("Error","Title of Error window"), 
				tr("The picture in the clipboard is too big!","Error message"), "OK");
			return;
		}
		else setNewDim(larg,haut);
	}

	m_selection.setPosition( QRect(0,0,larg,haut) );
	m_selection.m_data.resize(larg,haut);

	uchar nivNoir, nivGrisFonce, nivGrisMoyen, nivGrisClair;

	nivNoir		 = m_pConfig->readEntry("Prefs", "last_niv_b",   51);
	nivGrisFonce = m_pConfig->readEntry("Prefs", "last_niv_dg", 102);
	nivGrisMoyen = m_pConfig->readEntry("Prefs", "last_niv_mg", 153);
	nivGrisClair = m_pConfig->readEntry("Prefs", "last_niv_lg", 204);

	QRgb coul;
	QRgb mask = m_pConfig->readEntry("Prefs", "conv_mask", qRgb(255,255,0));

	for(int i=0; i<haut; i++)  for(int j=0; j<larg; j++) 
	{
		uchar c;
		coul = image.pixel(j,i);		// recupere les composantes d'un pixel
		
		if(coul==mask) c=TRANS;
		else
		{
			uchar moy = qGray(coul);

			if(moy<nivNoir) c=NOIR;
			else if(moy<nivGrisFonce) c=GRIS_FONCE;
			else if(moy<nivGrisMoyen) c=GRIS;
			else if(moy<nivGrisClair) c=GRIS_CLAIR;
			else c=BLANC;
		}

		m_selection.m_data[i][j] = c;
	}
		
	m_selection.setStatus(Ready);
	selectionReady();
	toolSelectionAction->setOn(true);
	emit imageChanged( m_selection.position() );
	
	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(TYPE_SEL_APPEAR);
	m_curDoItem->oldSel() = new Selection(m_selection);
	NotifyModif();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::editFlip(bool horiz)
{
	if( m_selection.status() == Ready ) 
	{
		m_selection.m_data.flip(horiz);
		emit imageChanged(m_selection.position());
	}
	else 
	{
		m_tableau.flip(horiz);
		emit imageChanged(QRect(0,0,m_largeur,m_hauteur));
		redrawPrevue();
	}
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::editFlipHoriz()
{ 
	editFlip(true); 
	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(true); 
	NotifyModif(); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::editFlipVertic() 
{ 
	editFlip(false); 
	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(false); 
	NotifyModif(); 
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::fusionSel(bool canUndo)
{
	QRect posFus = m_selection.position() & QRect(0,0,m_largeur,m_hauteur);

	if(canUndo)
	{
		if(m_curDoItem) delete m_curDoItem;
		m_curDoItem = new DoItem(TYPE_SEL_FUSION);
		m_curDoItem->oldSel() = new Selection(m_selection);
		m_curDoItem->underSel() = new Tab(m_tableau,posFus);
	}

	m_selection.fusion(m_tableau);
	emit imageChanged(posFus);
	redrawPrevue();
	if(canUndo) NotifyModif();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::selectionReady()
{
	editCopyAction->setEnabled(true);
	editCutAction->setEnabled(true);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::selectionCleared()
{
	editCopyAction->setEnabled(false);
	editCutAction->setEnabled(false);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::deleteSelection()
{
	if(m_selection.status()!=Ready) return;

	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(TYPE_SEL_DELETE);
	m_curDoItem->oldSel() = new Selection(m_selection);

	QRect rect = m_selection.position();
	m_selection.clear();
	m_selection.setStatus(Cleared);
	emit imageChanged(rect);
	redrawPrevue();

	NotifyModif();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::selectAll()
{
	if( m_selection.status() == Ready ) fusionSel();

	if(m_curDoItem) delete m_curDoItem;
	m_curDoItem = new DoItem(TYPE_SEL_CREATE);

	m_selection.wannaSet(QPoint(0,0));
	m_selection.set(QPoint(m_largeur-1,m_hauteur-1));
	m_selection.stopSetting(m_tableau, m_couleursChoisies[1]);
	m_curDoItem->oldSelPos() = m_selection.position();
	toolSelectionAction->setOn(true);
	emit imageChanged( m_selection.position() );

	NotifyModif();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::zoomIn()
{
	if(m_zoom==16) return;
	else m_zoom+=2;
	CreeImage();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::zoomOut()
{
	if(m_zoom==2) return;
	else m_zoom-=2;
	CreeImage();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::setCouleurBouton(Qt::ButtonState btn, int coul)
{
	int num = (btn==LeftButton)? 0 : 1;

	QToolButton* bouton = (btn==LeftButton) ? IndicChoixCoulGauche : IndicChoixCoulDroite;

	m_couleursChoisies[num] = coul;

	QPixmap buffer(15,15);
	QPainter p(&buffer);
	p.setPen(red);
	p.setBrush( Colors[coul] );
	p.drawRect(0,0,15,15);
	p.end();
	bouton->setIconSet(buffer);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::setTool( QAction * action )
{
	if(action != toolSelectionAction && m_selection.status()==Ready) fusionSel();

	if(action == toolCrayonAction) m_tool = PEN;
	else if(action == toolLigneAction) m_tool = LINE;
	else if(action == toolRectAction) m_tool = RECT;
	else if(action == toolSelectionAction) m_tool = SELECT;
	else if(action == toolFillAction) m_tool = FILL;

	if(m_tool==PEN) paintzone->setCurs(*m_curs_pen);
	else if(m_tool==FILL) paintzone->setCurs(*m_curs_fill);
	else paintzone->setCurs(QCursor(CrossCursor));	
	
	paintzone->setFocus();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::changeMode(int mode)
{

	// origine
	if(mode==DRAWLIB_14 || mode==GXLIB_10)
	{
        xOrigSpinBox->setEnabled(true);
		xOrigSpinBox->setValue(m_sprite->origin().x());
		yOrigSpinBox->setEnabled(true);
		yOrigSpinBox->setValue(m_sprite->origin().y());
	}
	else
	{
        xOrigSpinBox->setEnabled(false);
		yOrigSpinBox->setEnabled(false);
	}

	// couleurs
	if(mode!=GXLIB_10)
	{
		CoulGrisClairButton->hide();
		CoulGrisFonceButton->hide();
		Colors[GRIS_CLAIR] = Colors[GRIS];
		Colors[GRIS_FONCE] = Colors[GRIS];
		if(m_couleursChoisies[0]==GRIS_CLAIR || m_couleursChoisies[0]==GRIS_FONCE)
			setCouleurBouton(LeftButton,GRIS);
		if(m_couleursChoisies[1]==GRIS_CLAIR || m_couleursChoisies[1]==GRIS_FONCE)
			setCouleurBouton(RightButton,GRIS);
	}
	else
	{
		CoulGrisClairButton->show();
		CoulGrisFonceButton->show();
		Colors[GRIS_CLAIR] = QColor(192,192,192);
		Colors[GRIS_FONCE] = QColor(64,64,64);
	}

	CreeImage();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::genereCode()
{
	Coder coder(m_pConfig, m_sprite,this);
	coder.exec();
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::changeTransColor()
{
	CreeImage();

	QPixmap pix(15,15);
	QPainter p(&pix);
	p.setBrush( Colors[TRANS] );
	p.drawRect(0,0,15,15);
	p.end();
	CoulTransButton->setIconSet(pix);

	setCouleurBouton(LeftButton, m_couleursChoisies[0]);
	setCouleurBouton(RightButton, m_couleursChoisies[1]);
}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::clipboardChanged()
{
	 editPasteAction->setEnabled( ! qApp->clipboard()->image(QClipboard::Clipboard).isNull() );
}

///////////////////////////////////////////////////////////////////////////////////////

QPoint Viewer::origin()
{
	return QPoint(xOrigSpinBox->value(),yOrigSpinBox->value());
}

///////////////////////////////////////////////////////////////////////////////////////

Viewer::~Viewer() 
{ 
	m_pConfig->writeEntry("Geometry", "view_width", width() );
	m_pConfig->writeEntry("Geometry", "view_height", height() );
	m_pConfig->writeEntry("Prefs", "last_coulg",m_couleursChoisies[0]);
	m_pConfig->writeEntry("Prefs", "last_could",m_couleursChoisies[1]);
	m_pConfig->writeEntry("Prefs", "last_spr_l",m_largeur);
	m_pConfig->writeEntry("Prefs", "last_spr_h",m_hauteur);
}

///////////////////////////////////////////////////////////////////////////////////////

Viewer::Viewer( Sprite* spr, Config* pConfig ) 
	: QMainWindow(spr,"viewer")
	, m_pConfig(pConfig)
{
	/* BARRE D'ETAT */
	statusBar();

    /* FENETRE PRINCIPALE */
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );
    QHBoxLayout* ViewerLayout = new QHBoxLayout( centralWidget(), 11, 6, "ViewerLayout"); 

    /* PAINTZONE */
	paintzone = new PaintZone(centralWidget());
	paintzone->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	paintzone->setSelection( &m_selection );
    ViewerLayout->addWidget( paintzone );

    /* GROUPBOX SPRITE */
	QGroupBox* spriteGroupBox = new QGroupBox( "Sprite", centralWidget(), "spriteGroupBox" );
	//spriteGroupBox->setMaximumWidth(120);
	//spriteGroupBox->setMinimumWidth(120);
    spriteGroupBox->setColumnLayout(0, Qt::Vertical );
    spriteGroupBox->layout()->setSpacing( 6 );
    spriteGroupBox->layout()->setMargin( 11 );
	QVBoxLayout* spriteGroupBoxLayout = new QVBoxLayout( spriteGroupBox->layout() );
    spriteGroupBoxLayout->setAlignment( Qt::AlignTop );

    nomTextLabel = new QLabel( spriteGroupBox, "nomTextLabel" );
	spriteGroupBoxLayout->addWidget( nomTextLabel );

    nomLineEdit = new QLineEdit( spriteGroupBox, "nomLineEdit" );
    spriteGroupBoxLayout->addWidget( nomLineEdit );
	nomTextLabel->setBuddy(nomLineEdit);

    dimTextLabel = new QLabel( spriteGroupBox, "dimTextLabel" );
    dimTextLabel->setAlignment( int( QLabel::AlignBottom ) );
    spriteGroupBoxLayout->addWidget( dimTextLabel );

    QHBoxLayout* dimLayout = new QHBoxLayout( 0, 0, 6, "dimLayout"); 

    largTextLabel = new QLabel(spriteGroupBox, "largTextLabel" );
    largTextLabel->setAlignment( int( QLabel::AlignCenter ) );
    dimLayout->addWidget( largTextLabel );

	largSpinBox = new QSpinBox( spriteGroupBox, "largSpinBox" );
    largSpinBox->setMaximumWidth(30);
    largSpinBox->setMaxValue( 128 ); largSpinBox->setMinValue( 1 );
 	largTextLabel->setBuddy(largSpinBox);
    dimLayout->addWidget( largSpinBox );

	hautTextLabel = new QLabel( spriteGroupBox, "hautTextLabel" );
    hautTextLabel->setAlignment( int( QLabel::AlignCenter ) );
    dimLayout->addWidget( hautTextLabel );

    hautSpinBox = new QSpinBox( spriteGroupBox, "hautSpinBox" );
    hautSpinBox->setMaximumWidth(30);
    hautSpinBox->setMaxValue( 100 );  hautSpinBox->setMinValue( 1 );
	hautTextLabel->setBuddy(hautSpinBox);
    dimLayout->addWidget( hautSpinBox );

    spriteGroupBoxLayout->addLayout( dimLayout );

    origTextLabel = new QLabel( spriteGroupBox, "origTextLabel" );
    origTextLabel->setAlignment( AlignBottom );
    spriteGroupBoxLayout->addWidget( origTextLabel );

    QHBoxLayout* origLayout = new QHBoxLayout( 0, 0, 6, "origLayout"); 

    xOrigTextLabel = new QLabel("x:", spriteGroupBox, "xOrigTextLabel" );
    xOrigTextLabel->setAlignment( QLabel::AlignCenter );
    origLayout->addWidget( xOrigTextLabel );

	xOrigSpinBox = new QSpinBox( spriteGroupBox, "xOrigSpinBox" );
    xOrigSpinBox->setMaximumWidth(30);
    xOrigSpinBox->setRange(-100,100);
    origLayout->addWidget( xOrigSpinBox );

	QLabel* yOrigTextLabel = new QLabel( "y:", spriteGroupBox, "yOrigTextLabel" );
    yOrigTextLabel->setAlignment( QLabel::AlignCenter );
    origLayout->addWidget( yOrigTextLabel );

    yOrigSpinBox = new QSpinBox( spriteGroupBox, "yOrigSpinBox" );
    yOrigSpinBox->setMaximumWidth(30);
    yOrigSpinBox->setRange(-100,100);
    origLayout->addWidget( yOrigSpinBox );

    spriteGroupBoxLayout->addLayout( origLayout );

    m_prevue = new Prevue( spriteGroupBox );
	spriteGroupBoxLayout->addWidget( m_prevue );

	genererPushButton = new QPushButton( "&Gnrer!", spriteGroupBox, "genererPushButton" );
    genererPushButton->setAccel( QKeySequence( "Alt+G" ) );
	genererPushButton->setMinimumHeight( 30 );
    spriteGroupBoxLayout->addWidget( genererPushButton );
	
	spriteGroupBox->setMaximumWidth(spriteGroupBox->minimumSizeHint().width());
	
	ViewerLayout->addWidget( spriteGroupBox );
	
	
    /* ACTIONS */
	fileNewAction = new QAction( this, "fileNewAction" );
	fileNewAction->setIconSet(ICO_NEW);

	fileSaveAction = new QAction( this, "fileSaveAction" );
	fileSaveAction->setIconSet(ICO_SAVE);

	fileExitAction = new QAction( this, "fileExitAction" );

	editCutAction = new QAction( this, "editCutAction" );
	editCutAction->setIconSet(QPixmap(xpm_cut));

	editCopyAction = new QAction( this, "editCopyAction" );
	editCopyAction->setIconSet(QPixmap(xpm_copy));

	editPasteAction = new QAction( this, "editPasteAction" );
	editPasteAction->setIconSet(QPixmap(xpm_paste));

    toolsActionGroup = new QActionGroup( this, "toolsActionGroup", true );
	toolsActionGroup->setExclusive(true);
		
	toolCrayonAction = new QAction( toolsActionGroup, "toolCrayonAction" );
	toolCrayonAction->setIconSet(QPixmap(xpm_pen));
	toolCrayonAction->setToggleAction(true);

	toolLigneAction = new QAction(toolsActionGroup, "toolLigneAction" );
	toolLigneAction->setIconSet(QPixmap(xpm_ligne));
	toolLigneAction->setToggleAction(true);

	toolRectAction = new QAction( toolsActionGroup, "toolRectAction" );
	toolRectAction->setIconSet(QPixmap(xpm_rect));
	toolRectAction->setToggleAction(true);

	toolSelectionAction = new QAction( toolsActionGroup, "toolSelectionAction" );
	toolSelectionAction->setIconSet(QPixmap(xpm_select));
	toolSelectionAction->setToggleAction(true);

	toolFillAction = new QAction( toolsActionGroup, "toolFillAction" );
	toolFillAction->setIconSet(QPixmap(xpm_fill));
	toolFillAction->setToggleAction(true);

	zoomInAction = new QAction( this, "zoomInAction" );
	zoomInAction->setIconSet(QPixmap(xpm_zoomin));
	zoomInAction->setAccel(Key_Plus);

	zoomOutAction = new QAction( this, "zoomOutAction" );
	zoomOutAction->setIconSet(QPixmap(xpm_zoomout));
	zoomInAction->setAccel(Key_Minus);



	MyBackupHdlr* backupHandler = new MyBackupHdlr(this,20);

    /* BARRE D'OUTILS */
    editToolBar = new QToolBar( "", this, DockTop ); 

	fileNewAction->addTo( editToolBar );
    fileSaveAction->addTo( editToolBar );

    editToolBar->addSeparator();

    backupHandler->addTo( editToolBar );
    editCutAction->addTo( editToolBar );
    editCopyAction->addTo( editToolBar );
    editPasteAction->addTo( editToolBar );

    editToolBar->addSeparator();

    zoomInAction->addTo( editToolBar );
    zoomOutAction->addTo( editToolBar );

    toolsToolBar = new QToolBar( "", this, DockTop ); 

    toolCrayonAction->addTo( toolsToolBar );
	toolFillAction->addTo( toolsToolBar );
    toolLigneAction->addTo( toolsToolBar );
    toolRectAction->addTo( toolsToolBar );
    toolSelectionAction->addTo( toolsToolBar );

    toolsToolBar->addSeparator();

	QPixmap icone_couleur(15,15);
	
	CoulTransButton = new ColorButton(icone_couleur, toolsToolBar,"CoulTranspButton");

	CoulBlancButton = new ColorButton(icone_couleur, toolsToolBar,"CoulBlancButton");

	CoulGrisClairButton = new ColorButton(icone_couleur, toolsToolBar,"CoulGrisClairButton");

	CoulGrisButton = new ColorButton(icone_couleur, toolsToolBar,"CoulGrisButton");

	CoulGrisFonceButton = new ColorButton(icone_couleur, toolsToolBar,"CoulGrisFonceButton");

	CoulNoirButton = new ColorButton(icone_couleur, toolsToolBar,"CoulNoirButton");

    toolsToolBar->addSeparator();

	IndicChoixCoulGauche = 
		new QToolButton(icone_couleur,"", "", 
		0, QString::null, toolsToolBar,"IndicChoixCoulGauche");

	IndicChoixCoulDroite = 
		new QToolButton(icone_couleur,"", "", 
		0, QString::null, toolsToolBar,"IndicChoixCoulDroite");


    /* MENUS */
    MenuBar = new QMenuBar( this, "MenuBar" );

    QPopupMenu* fileMenu = new QPopupMenu( this );
    fileNewAction->addTo( fileMenu );
    fileSaveAction->addTo( fileMenu );
    fileMenu->insertSeparator();
    fileExitAction->addTo( fileMenu );
    MenuBar->insertItem( "", fileMenu, 1 );

    editMenu = new QPopupMenu( this );
    backupHandler->addTo( editMenu );
    editMenu->insertSeparator();
    editCutAction->addTo( editMenu );
    editCopyAction->addTo( editMenu );
    editPasteAction->addTo( editMenu );
    editMenu->insertSeparator();
    zoomInAction->addTo( editMenu );
    zoomOutAction->addTo( editMenu );
    editMenu->insertSeparator();
	editMenu->insertItem("",this,SLOT(editFlipHoriz()),QString::null,FLIP_H_ID);
	editMenu->insertItem("",this,SLOT(editFlipVertic()),QString::null,FLIP_V_ID);
    MenuBar->insertItem( "", editMenu, 2 );

    QPopupMenu* OutilsMenu = new QPopupMenu( this );
    toolCrayonAction->addTo( OutilsMenu );
	toolFillAction->addTo( OutilsMenu );
    toolLigneAction->addTo( OutilsMenu );
    toolRectAction->addTo( OutilsMenu );
    toolSelectionAction->addTo( OutilsMenu );
    MenuBar->insertItem( "", OutilsMenu, 3 );


	/* TABULATIONS */
    setTabOrder( paintzone,    nomLineEdit );
    setTabOrder( nomLineEdit,  largSpinBox );
    setTabOrder( largSpinBox,  hautSpinBox );
    setTabOrder( hautSpinBox,  xOrigSpinBox );
    setTabOrder( xOrigSpinBox, yOrigSpinBox );
    setTabOrder( yOrigSpinBox, paintzone );

    /* ACCELS */
	QAccel *accel = new QAccel( this );
    accel->connectItem( accel->insertItem( Key_Delete ),
                        this, SLOT(deleteSelection()) );
    accel->connectItem( accel->insertItem( CTRL+Key_A ),
                        this, SLOT(selectAll()) );

	/* CONNEXIONS */
    connect( fileNewAction, SIGNAL( activated() ), SLOT( fileNew() ) );
    connect( fileSaveAction, SIGNAL( activated() ), SLOT( fileSave() ) );
    connect( fileExitAction, SIGNAL( activated() ), SLOT( fileExit() ) );
    connect( backupHandler, SIGNAL( undoAsked(DoItemBase*) ), SLOT( undo(DoItemBase*) ) );
    connect( backupHandler, SIGNAL( redoAsked(DoItemBase*) ), SLOT( redo(DoItemBase*) ) );
    connect( this, SIGNAL( newDoItem(DoItemBase*) ), backupHandler, SLOT( addDoItem(DoItemBase*) ) );
    connect( editCutAction, SIGNAL( activated() ), SLOT( editCut() ) );
    connect( editCopyAction, SIGNAL( activated() ), SLOT( editCopy() ) );
    connect( editPasteAction, SIGNAL( activated() ), SLOT( editPaste() ) );

	connect( CoulTransButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorTrans(Qt::ButtonState) ) );
	connect( CoulBlancButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorBlanc(Qt::ButtonState) ) );
	connect( CoulNoirButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorNoir(Qt::ButtonState) ) );
	connect( CoulGrisButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorGris(Qt::ButtonState) ) );
	connect( CoulGrisFonceButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorGrisFonce(Qt::ButtonState) ) );
	connect( CoulGrisClairButton, SIGNAL( selected(Qt::ButtonState) ),
			 SLOT( setColorGrisClair(Qt::ButtonState) ) );

    connect( toolsActionGroup, SIGNAL( selected(QAction*) ), SLOT( setTool(QAction*) ) );
	connect( zoomInAction, SIGNAL( activated() ), SLOT( zoomIn() ) );
	connect( zoomOutAction, SIGNAL( activated() ), SLOT( zoomOut() ) );

	connect( this,		SIGNAL(newImage(Tab*,int)), 
			 paintzone, SLOT(setConfig(Tab*,int))			);
	connect( this,		SIGNAL(imageChanged(QRect)), 
			 paintzone, SLOT(refresh(QRect))							);
		
	connect( paintzone,	SIGNAL(mousePressed(QPoint, Qt::ButtonState)),
			 this,		SLOT(mousePressed(QPoint, Qt::ButtonState))	);
	connect( paintzone,	SIGNAL(mouseReleased(Qt::ButtonState)),
			 this,		SLOT(mouseReleased(Qt::ButtonState))			);
	connect( paintzone,	SIGNAL(mouseMoved(QPoint)),
			 this,		SLOT(mouseMoved(QPoint))						);

	connect( genererPushButton, SIGNAL(clicked()), SLOT( genereCode()) );
	connect( largSpinBox, SIGNAL(valueChanged(int)), SLOT(setLargeur(int)) );
	connect( hautSpinBox, SIGNAL(valueChanged(int)), SLOT(setHauteur(int)) );

	connect( nomLineEdit, SIGNAL(textChanged(const QString&)), SLOT(setNom(const QString&)) );

	connect( qApp->clipboard(), SIGNAL(dataChanged()), SLOT(clipboardChanged()) );
	
	connect( &m_selection, SIGNAL(ready()), SLOT(selectionReady()) );
	connect( &m_selection, SIGNAL(cleared()), SLOT(selectionCleared()) );

	uchar curs_pen_bit[] = 
	{ 0x00,0x00,0x03,0x80,0x04,0x40,0x04,0x40,0x0C,0x80,0x0B,0x80,0x11,0x00,
	0x13,0x00,0x22,0x00,0x26,0x00,0x44,0x00,0x6C,0x00,0x78,0x00,0x70,0x00,
	0x60,0x00,0x40,0x00,0x00,0x00 };
	uchar curs_pen_mask[] = 
	{ 0x03,0x80,0x07,0xC0,0x0F,0xE0,0x0F,0xE0,0x1F,0xC0,0x1F,0xC0,0x3F,0x80,
	0x3F,0x80,0x7F,0x00,0x7F,0x00,0xFE,0x00,0xFE,0x00,0xFC,0x00,0xF8,0x00,
	0xF0,0x00,0xE0,0x00,0x40,0x00 };	
	uchar curs_pip_bit[] = 
	{ 0x00,0x00,0x00,0x1C,0x00,0x2E,0x00,0x7E,0x01,0xFE,0x00,0xFC,0x01,0x78,
	0x02,0x30,0x04,0x50,0x08,0x80,0x11,0x00,0x22,0x00,0x44,0x00,0x48,0x00,
	0x70,0x00,0x00,0x00 };
	uchar curs_pip_mask[] = 
	{ 0x00,0x1C,0x00,0x3E,0x00,0x7F,0x03,0xFF,0x03,0xFF,0x01,0xFE,0x03,0xFC,
	0x07,0xF8,0x0F,0xF8,0x1F,0xD0,0x3F,0x80,0x7F,0x00,0xFE,0x00,0xFC,0x00,
	0xF8,0x00,0xF0,0x00 };
	uchar curs_fill_bit[] = 
	{ 0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0xA0,0x00,0x00,0x1D,0x18,
	0x00,0x00,0x3A,0x24,0x00,0x00,0x74,0x0E,0x00,0x00,0x68,0x1F,0x00,0x00,
	0x6D,0x3E,0x80,0x00,0x72,0x7C,0x80,0x00,0x72,0xF9,0x00,0x00,0x71,0xF2,
	0x00,0x00,0x70,0xE4,0x00,0x00,0x30,0x48,0x00,0x00,0x30,0x30,0x00,0x00,
	0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	};
	uchar curs_fill_mask[] = 
	{ 0x00,0x60,0x00,0x00,0x00,0xF0,0x00,0x00,0x1F,0xF8,0x00,0x00,0x3F,0xFC,
	0x00,0x00,0x7F,0xFE,0x00,0x00,0xFF,0xFF,0x00,0x00,0xFF,0xFF,0x80,0x00,
	0xFF,0xFF,0xC0,0x00,0xFF,0xFF,0xC0,0x00,0xFF,0xFF,0x80,0x00,0xFB,0xFF,
	0x00,0x00,0xF9,0xFE,0x00,0x00,0x78,0xFC,0x00,0x00,0x78,0x78,0x00,0x00,
	0x38,0x30,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00	};

	m_curs_pen = 
		new QCursor( QBitmap(16,17,curs_pen_bit), QBitmap(16,17,curs_pen_mask), 1, 15);
	m_curs_pip = 
		new QCursor( QBitmap(16,16,curs_pip_bit), QBitmap(16,16,curs_pip_mask), 1, 14);
	m_curs_fill = 
		new QCursor( QBitmap(32,17,curs_fill_bit), QBitmap(32,17,curs_fill_mask), 3, 14);
	
	/* INITIALISATIONS */

	resize( m_pConfig->readEntry("Geometry", "view_width", 480),
			m_pConfig->readEntry("Geometry", "view_height",415) );
	clearWState( WState_Polished );

	m_sprite=spr;


	// dessin des icones de couleurs
	ColorButton* btns[] = 
		{ CoulNoirButton, CoulBlancButton, CoulGrisButton,
			CoulTransButton, CoulGrisClairButton, CoulGrisFonceButton };

	Colors[GRIS_CLAIR] = QColor(192,192,192);
	Colors[GRIS_FONCE] = QColor(64,64,64);
			
	for(int c=0; c<6; c++)
	{
		QPainter p(&icone_couleur);
		p.setBrush( Colors[c] );
		p.drawRect(0,0,15,15);
		p.end();
		btns[c]->setIconSet(icone_couleur);
	}

	m_souris_gauche_presse=false;
	m_souris_droite_presse=false;


	m_zoom=8;
	toolCrayonAction->setOn(true);

	setCouleurBouton(LeftButton, m_pConfig->readEntry("Prefs",  "last_coulg", NOIR) );
	setCouleurBouton(RightButton, m_pConfig->readEntry("Prefs", "last_could", BLANC) );

	editCopyAction->setEnabled(false);
	editCutAction->setEnabled(false);
	clipboardChanged();	// regarde si le clipboard contient une image

	m_largeur = spr->largeur();
	m_hauteur = spr->hauteur();
	m_tableau = spr->tableau();
	m_changed = false; //spr->isModified();
	m_nom = spr->nom();
	nomLineEdit->setText( m_nom );
	
	languageChange();

	m_oldPlaceCursor = QPoint(m_largeur+1, m_hauteur+1);
	m_oldRect = QRect(0,0,0,0);

	m_shift_enfonce = false;

	m_curDoItem = NULL;
	m_noDoItemAccepted = false;

	changeMode( m_pConfig->mode() );
	//CreeImage();

	largSpinBox->setValue( m_largeur );
    hautSpinBox->setValue( m_hauteur );   
    

}

///////////////////////////////////////////////////////////////////////////////////////

void Viewer::languageChange()
{
	setCaption( tr("Editor","Caption of the window") + " - " + m_nom );

    nomTextLabel->setText(tr("&Name:"));
    dimTextLabel->setText(tr("Size:"));
    largTextLabel->setText(tr("&W:","first letter of \'Width\'"));
	hautTextLabel->setText(tr("&H:","first letter of \'Height\'"));
    origTextLabel->setText(tr("Origin:"));
	genererPushButton->setText(tr("&Code it!","Text of the \'Code\' button"));
    genererPushButton->setAccel( tr("Alt+C","Accelerator of the \'Code\' button") );

    MenuBar->changeItem(tr("&Sprite","Title of the first menu"), 1 );

	fileNewAction->setMenuText(tr("&Clear","Item of the \'Sprite\' menu"));
	fileNewAction->setText(tr("Clear the sprite","Item description"));
	fileNewAction->setAccel(tr("Ctrl+C","Accelerator for the \'Clear\' action"));

	fileSaveAction->setMenuText(tr("&Save","Item of the \'Sprite\' menu"));
	fileSaveAction->setText(tr("Save the sprite","Item description"));
	fileSaveAction->setAccel(tr("Ctrl+S","Accelerator for the \'Save\' action"));

	fileExitAction->setMenuText(tr("Clos&e","Item of the \'Sprite\' menu"));
	fileExitAction->setText(tr("Close the editor","Item description"));

    MenuBar->changeItem(tr("&Edit","Title of the second menu"), 2 );

	editCutAction->setMenuText(tr("Cut","Item of the \'Edit\' menu"));
	editCutAction->setText(tr("Cut","Item description"));
	editCutAction->setAccel(tr("Ctrl+X","Accelerator for the \'Cut\' action"));

	editCopyAction->setMenuText(tr("Copy","Item of the \'Edit\' menu"));
	editCopyAction->setText(tr("Copy","Item description"));
	editCopyAction->setAccel(tr("Ctrl+C","Accelerator for the \'Copy\' action"));

	editPasteAction->setText(tr("Paste","Item of the \'Edit\' menu"));
	editPasteAction->setMenuText(tr("Paste","Item description"));
	editPasteAction->setAccel(tr("Ctrl+V","Accelerator for the \'Paste\' action"));

 	editMenu->changeItem(tr("Flip &horizontaly","Item of the \'Edit\' menu"), FLIP_H_ID);
	editMenu->changeItem(tr("Flip &verticaly","Item of the \'Edit\' menu"), FLIP_V_ID);

	MenuBar->changeItem(tr("&Tools","Title of the third menu"), 3 );

	toolsActionGroup->setText( tr("Tools", "Name of the \'Tools\' group") );

	toolCrayonAction->setMenuText(tr("Pen"));
	toolCrayonAction->setText(tr("Pen"));

	toolLigneAction->setMenuText(tr("Line"));
	toolLigneAction->setText(tr("Line"));

	toolRectAction->setMenuText(tr("Rectangle"));
	toolRectAction->setText(tr("Rectangle"));

	toolSelectionAction->setMenuText(tr("Selection"));
	toolSelectionAction->setText(tr("Selection"));
	
	toolFillAction->setMenuText(tr("Fill"));
	toolFillAction->setText(tr("Fill"));

	zoomInAction->setMenuText(tr("Zoom In"));
	zoomInAction->setText(tr("Zoom In"));

	zoomOutAction->setMenuText(tr("Zoom Out"));
	zoomOutAction->setText(tr("Zoom Out"));

	toolsToolBar->setLabel(tr("Tools","Name of the \'Tools\' toolbar")); 
	editToolBar->setLabel(tr("Edit","Name of the \'Edit\' toolbar")); 

	CoulTransButton->setTextLabel(tr("Transparent","Mask color"));
	CoulBlancButton->setTextLabel(tr("White","White color"));
	CoulGrisClairButton->setTextLabel(tr("Light Gray","Light Gray color"));
	CoulGrisButton->setTextLabel(tr("Normal Gray","Normal Gray color"));
	CoulGrisFonceButton->setTextLabel(tr("Dark Gray","Dark Gray color"));
	CoulNoirButton->setTextLabel(tr("Black","Black color"));

	QPixmap icone_couleur(15,15);
	delete IndicChoixCoulGauche;
	IndicChoixCoulGauche = 
		new QToolButton(icone_couleur,tr("Left Button","Descripton of the Left Button color"), tr("Selected colors"), 
		0, QString::null, toolsToolBar,"IndicChoixCoulGauche");

	delete IndicChoixCoulDroite;
	IndicChoixCoulDroite = 
		new QToolButton(icone_couleur,tr("Right Button","Descripton of the Right Button color"), tr("Selected colors"), 
		0, QString::null, toolsToolBar,"IndicChoixCoulDroite");

	setCouleurBouton(LeftButton, m_couleursChoisies[0] );
	setCouleurBouton(RightButton, m_couleursChoisies[1] );
}
