#ifndef __VIEWER
#define __VIEWER

#include "main.h"

#include <qvariant.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <qmainwindow.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qrect.h>

class QAction;
class QToolButton;
class QSpinBox;
class QPushButton;
class QActionGroup;

class DoItemBase;
class DoItem;

class PaintZone;
class ColorButton;
class Sprite;
class DoItem;
class Config;

#include "select.h"
#ifdef Q_OS_UNIX
#include "tab.h"
#else
#include "../mylibs/matrice.h"
#endif

void swap(int &a, int &b);
int Round(double a);
QRect makeRect(QPoint p1, QPoint p2);


class Prevue : public QWidget
{
Q_OBJECT
public:
	Prevue(QWidget* parent);
	QPixmap* pixmap() { return &m_pixmap; }
	QRect posPix() { return m_posPix; }
	void setPixmap(QPixmap &pix);
	void redrawPix(QRect r);
	void redrawPix();
	QSize minimumSizeHint() { return m_pixmap.size(); }

protected slots:
	void paintEvent(QPaintEvent* e);
	void resizeEvent(QResizeEvent* e);

protected:
	QPixmap m_pixmap;
	QRect m_posPix;
};


class Viewer : public QMainWindow
{
Q_OBJECT
public:
	Viewer( Sprite* spr, Config* pConfig );
	~Viewer();

	QString nom() { return m_nom; }
	int largeur() { return m_largeur; }
	int hauteur() { return m_hauteur; }
	Tab &tableau() { return m_tableau; }
	QPixmap prevue() { return *(m_prevue->pixmap()); }
	QPoint origin();
	bool isModified() { return m_changed; }
	void changeTransColor();

public slots:
	void setNom(const QString &nom);


protected slots:
	void mousePressed(QPoint, Qt::ButtonState);
	void mouseReleased(Qt::ButtonState);
	void mouseMoved(QPoint);
	void languageChange();

	void fileNew();
	void fileSave();
	void fileExit() { close(); }
	void closeEvent( QCloseEvent* );

	void keyPressEvent(QKeyEvent* e);
	void keyReleaseEvent(QKeyEvent* e);

	void undo(DoItemBase*);
	void redo(DoItemBase*);

	void editPaste();
	void editCopy();
	void editCut();

	void editFlip(bool horiz);
	void editFlipHoriz();
	void editFlipVertic();

	void zoomIn();
	void zoomOut();

	void setTool( QAction * );

	void setColorBlanc(Qt::ButtonState btn)		 { setCouleurBouton(btn,BLANC); }
	void setColorGrisClair(Qt::ButtonState btn)  { setCouleurBouton(btn,GRIS_CLAIR); }
	void setColorGris(Qt::ButtonState btn)		 { setCouleurBouton(btn,GRIS); }
	void setColorGrisFonce(Qt::ButtonState btn)  { setCouleurBouton(btn,GRIS_FONCE); }
	void setColorNoir(Qt::ButtonState btn)		 { setCouleurBouton(btn,NOIR); }
	void setColorTrans(Qt::ButtonState btn)		 { setCouleurBouton(btn,TRANS); }

	void setLargeur(int val) { setNewDim(val,m_hauteur); }
	void setHauteur(int val) { setNewDim(m_largeur,val); }
	
	void genereCode();

	void clipboardChanged();
	void changeMode(int mode);
	void selectionReady();
	void selectionCleared();
	void deleteSelection();
	void selectAll();

signals:
	void dataChanged();	// pour le sprite parent
	void closingEditor(); // idem
	void imageChanged(QRect);	// pour le paintzone enfant
	void newImage(Tab*,int);// tableau et zoom
	void newDoItem(DoItemBase*);

protected:
	void NewSprite(int larg, int haut);
	void CreeImage();
	void redrawPrevue();
	void setPixel(QPoint pxl, int couleur);
	void fillColor(int x, int y, uchar oldColor, uchar newColor);
	void drawline(QPoint p1, QPoint p2);
	void drawrect(QPoint p1, QPoint p2);
	void fusionSel(bool canUndo=true);
	void setCouleurBouton(Qt::ButtonState btn, int);

	void setNewDim(int larg, int haut);

	void NotifyModif();

	bool okToClear();
	bool m_changed;

	QString m_nom;	// nom du tab a creer

	PaintZone* paintzone;
 	Sprite* m_sprite;	// sprite parent

	QCursor* m_curs_pen;
	QCursor* m_curs_pip;
	QCursor* m_curs_fill;

	bool m_souris_gauche_presse, m_souris_droite_presse;
	QPoint m_oldPlaceCursor;
	QPoint m_basePoint, m_secondPoint;

	Tab m_tableau;
	Prevue* m_prevue;	// image de prvisualisation
	int m_largeur, m_hauteur;

	bool m_shift_enfonce;

	uchar m_couleursChoisies[2];
	int m_zoom;
	int m_tool;
	Tab m_tab_saved;	// utilis pour garder la vraie image pendant que m_tableau
						// affiche les selections, rectangles et lignes provisoires
	QRect m_oldRect;

	Selection m_selection;
	Selection m_oldSel;
	Tab m_underOldSel;

	DoItem* m_curDoItem;
	bool m_noDoItemAccepted;


	QLabel* nomTextLabel;
	QLabel* dimTextLabel;
	QLabel* largTextLabel;
	QLabel* hautTextLabel;
	QLabel* origTextLabel;
	QLabel* xOrigTextLabel;
	QPushButton* genererPushButton;
	QActionGroup* toolsActionGroup;
	QAction* fileNewAction;
	QAction* fileSaveAction;
	QAction* fileExitAction;
	QAction* zoomInAction;
	QAction* zoomOutAction;

	QToolBar* editToolBar;
	QToolBar* toolsToolBar;
	QSpinBox* largSpinBox;
	QSpinBox* hautSpinBox;
	QSpinBox* xOrigSpinBox;
	QSpinBox* yOrigSpinBox;

	QAction* editCutAction;
	QAction* editCopyAction;
	QAction* editPasteAction;

	QAction* toolCrayonAction;
    QAction* toolLigneAction;
    QAction* toolRectAction;
    QAction* toolSelectionAction;
    QAction* toolFillAction;

	QMenuBar* MenuBar;
	QPopupMenu* editMenu;
	ColorButton* CoulTransButton;
	ColorButton* CoulBlancButton;
	ColorButton* CoulGrisClairButton;
	ColorButton* CoulGrisButton;
	ColorButton* CoulGrisFonceButton;
	ColorButton* CoulNoirButton;

	QToolButton* IndicChoixCoulGauche;
	QToolButton* IndicChoixCoulDroite;

    QLineEdit* nomLineEdit;

	Config* m_pConfig;
};

#endif
