#ifndef __MATRICE
#define __MATRICE

#include <qsize.h>
#include <qrect.h>
#include <qpoint.h>

template <typename Type>
class Matrice
{
public:
	typedef Matrice<Type> Mat;
	
	Matrice() : m_larg(0), m_haut(0), m_tab(NULL) {}
	Matrice(Mat &modele)  : m_larg(0), m_haut(0), m_tab(NULL) { copy(modele); }
	Matrice(Mat &modele, QRect r)  : m_larg(0), m_haut(0), m_tab(NULL) { copy(modele,r); }
	Matrice(int larg, int haut);
	~Matrice() { clear(); }

	Mat & adaptFromPtr(int larg, int haut, Type* data);
	Type* toNewPtr();

	Type* row(int num) { Q_ASSERT(num<m_haut && num>=0); return m_tab[num]; }
	Type* operator [] (int num) { return row(num); }

	Type & operator [] (QPoint &elem) { return m_tab[elem.y()][elem.x()]; }
	Type & element(int i, int j) { return m_tab[i][j]; }
	Type & element(QPoint &elem) { return m_tab[elem.y()][elem.x()]; }
	const Type operator [] (QPoint &elem) const { return element(elem); }
	const Type element(int i, int j) const { return m_tab[i][j]; }
	const Type element(QPoint &elem) const { return element(elem); }
	Mat part(QRect r);
	

	void resize(int larg, int haut);
	void resize(QSize &size) { resize(size.width(),size.height()); }
	QSize size() { return QSize(m_larg, m_haut); }
	int height() { return m_haut; }
	int width() { return m_larg; }

	void flip(bool horiz); // true: horiz; false: vertic

	void clear();
	void fill(Type value);
	void fill(QRect rect, Type value);
	bool isNull() { return m_larg==0 || m_haut==0; }

	Mat & copy(Mat &modele);
	Mat & copy(Mat &modele, QRect rect);
	Mat & operator = (Mat &modele) { copy(modele); return *this; }

protected:
	Type** m_tab;
	int m_larg, m_haut;
};

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Matrice<Type>::Matrice(int larg, int haut)
{
	m_tab = new Type* [haut];
	for(int i=0; i<haut; i++) m_tab[i] = new Type[larg];
	m_larg = larg;
	m_haut = haut;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Matrice<Type> & Matrice<Type>::copy(Mat &modele)
{
	clear();
	
	resize(modele.size());
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) 
		m_tab[i][j] = modele.m_tab[i][j];

	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Matrice<Type> & Matrice<Type>::copy(Mat &modele, QRect rect)
{
	rect = rect & QRect(0,0,modele.m_larg,modele.m_haut);

	resize(rect.size());

	for(int i= 0; i<m_haut; i++)
		for(int j=0; j<m_larg; j++)
			m_tab[i][j] = modele.m_tab[ i + rect.top() ][ j + rect.left() ];
	
	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Matrice<Type> Matrice<Type>::part(QRect r)
{
	Mat m;
	m.copy(*this,r);
	return m;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Matrice<Type>::resize(int larg, int haut)
{
	if(m_larg == larg && m_haut == haut) return;
	if(larg <= 0 || haut <= 0)  { clear(); return; }
	
	int largmin = (larg<m_larg? larg : m_larg);
	int hautmin = (haut<m_haut? haut : m_haut);

	Type** newtab = new Type* [haut];
	for(int r=0; r<haut; r++) newtab[r] = new Type[larg];

	for(int i=0; i<hautmin; i++) for(int j=0; j<largmin; j++) newtab[i][j] = m_tab[i][j];

	clear();
	m_tab = newtab;
	m_haut = haut;
	m_larg = larg;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Matrice<Type> & Matrice<Type>::adaptFromPtr(int larg, int haut, Type* data)
{
	resize(larg,haut);
	int offset = 0;
	for(int i=0; i<haut; i++) for(int j=0; j<larg; j++) m_tab[i][j] = data[offset++];

	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Type* Matrice<Type>::toNewPtr()
{
	Type* newptr = new Type[m_larg*m_haut];
	int offset=0;
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) newptr[offset++] = m_tab[i][j];
	return newptr;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Matrice<Type>::clear()
{
	if(isNull()) return;
	
	for(int i=0; i<m_haut; i++) delete[] m_tab[i];
	delete[] m_tab;

	m_tab = NULL;
	m_larg = 0;
	m_haut = 0;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Matrice<Type>::fill(Type value)
{
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) m_tab[i][j] = value;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Matrice<Type>::fill(QRect rect, Type value)
{
	rect = rect & QRect(0,0,m_larg,m_haut);
	for(int i=rect.top(); i<=rect.bottom(); i++) 
		for(int j=rect.left(); j<=rect.right(); j++) 
			m_tab[i][j] = value;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void Matrice<Type>::flip(bool horiz)
{
	if(horiz==false) // retournement vertical
	{
		for(int i=0; i<m_haut/2; i++)
		{
			Type* row = m_tab[i];
			m_tab[i] = m_tab[m_haut-i-1];
			m_tab[m_haut-i-1] = row;
		}
	}
	else // retournement horizontal
	{
		for(int i=0; i<m_haut; i++)
		{
			Type* row = m_tab[i];
			for(int j=0; j<m_larg/2; j++)
			{
				Type save = row[j];
				row[j] = row[m_larg-j-1];
				row[m_larg-j-1] = save;
			}
		}	
	}
}


typedef Matrice<uchar> Tab;

#endif
