#include "../mylibs/myBackuphdlr.h"

static const char* xpm_undo[]={
"12 7 3 1",
". c None",
"# c #4d4da6",
"a c #a6a6a6",
"......####a.",
"#...##....#a",
"##.#.......#",
"###........#",
"####.......#",
"#####.....#a",
".........a#."};

static const char* xpm_redo[]={
"12 7 3 1",
". c None",
"a c #4d4da6",
"# c #a6a6a6",
".#aaaa......",
"#a....aa...a",
"a.......a.aa",
"a........aaa",
"a.......aaaa",
"#a.....aaaaa",
".a#........."};

MyBackupHdlr::MyBackupHdlr(QObject* parent, uint nbreBack)
	: QObject(parent,"MyBackupHdlr")
	, m_undoStack(true,nbreBack)
	, m_redoStack(true,-1)
{
	m_undoAct = new QAction( this, "undo action");
	m_undoAct->setIconSet(QPixmap(xpm_undo));
	m_undoAct->setAccel(CTRL+Key_Z);
	m_undoAct->setEnabled(false);

	m_redoAct = new QAction( this, "redo action");
	m_redoAct->setIconSet(QPixmap(xpm_redo));
	m_redoAct->setAccel(CTRL+Key_Y);
	m_redoAct->setEnabled(false);

	connect(m_undoAct, SIGNAL(activated()), this, SLOT(undoUsed()));
	connect(m_redoAct, SIGNAL(activated()), this, SLOT(redoUsed()));
	languageChange();
}

///////////////////////////////////////////////////////////////////////////////////////

void MyBackupHdlr::addDoItem(DoPtr item)
{
	m_undoStack.push(*item);
	m_undoAct->setEnabled(true);

	m_redoStack.clear();
	m_redoAct->setEnabled(false);
}

///////////////////////////////////////////////////////////////////////////////////////

DoItemBase* MyBackupHdlr::getUndo()
{
	DoItemBase &item = m_undoStack.pop();
	m_undoAct->setEnabled(canUndo());

	m_redoStack.push(item);
	m_redoAct->setEnabled(true);
	return &item;
}

///////////////////////////////////////////////////////////////////////////////////////

DoItemBase* MyBackupHdlr::getRedo()
{
	DoItemBase &item = m_redoStack.pop();
	m_redoAct->setEnabled(canRedo());

	m_undoStack.push(item);
	m_undoAct->setEnabled(true);
	return &item;
}

///////////////////////////////////////////////////////////////////////////////////////

void MyBackupHdlr::languageChange()
{
	m_undoAct->setMenuText(tr("&Undo"));
	m_undoAct->setText(tr("Undo action"));
	m_redoAct->setMenuText(tr("&Redo"));
	m_redoAct->setText(tr("Redo action"));
}

///////////////////////////////////////////////////////////////////////////////////////



