#ifndef __MYBACKUPHDLR
#define __MYBACKUPHDLR

#include <qobject.h>
#include <qaction.h>
class QWidget;

#include "../mylibs/stack.h"

/*
	*** class MyBackupHdlr ***
	
	Gestionnaire des actions Undo et Redo

	Cet outil s'occupe de la gestion de "donnes de sauvegarde" pour les actions effectues.
	Il fournit les 2 actions Undo et Redo qui peuvent tre insres dans les menus ou dans
	les barres d'outils.
	
	Il manipule des objets de Backup que l'utilisateur doit crer " son got" avant de lui
	transmettre l'adresse (avec la fonction/le slot addDoItem()):
	Ces objets doivent driver de la classe DoItemBase, et sont destins  contenir des
	donnes suffisantes pour caractriser l'action effectue et la situation initiale.
	
	Lorsqu'on souhaite annuler une action (Undo), le gestionnaire emet un signal undoAsked()
	avec l'adresse de l'objet qui lui a t donn lorsque l'action avait t effectue
	auparavant. Cet objet doit donc contenir assez d'infos pour pouvoir annuler l'action et
	rtablir la situation initiale.

	La situation est identique pour le Redo, le signal redoAsked() contient l'adresse de
	l'objet correspondant  l'action: Il faut donc que ce mme objet contienne assez d'infos
	pour pouvoir rpter l'action.

	La structure des objets de Backup est donc totalement libre, du moment qu'ils drivent 
	de la classe DoItemBase et qu'ils caractrisent suffisamment l'action effectue pour
	pouvoir la rpter, ainsi que la situation initiale pour pouvoir annuler l'action.

	Note: - Les objets drivs de DoItemBase ne peuvent perdre leur information qu'au
		moment de leur destruction (donc dans leur destructeur)!
		  - Les signaux undoAsked et redoAsked renvoient des pointeurs DoItemBase* .
		Il faut donc effectuer un dynamic_cast pour les convertir en des pointeurs vers
		les objets de Backup voulu. (Les templates ne peuvent utiliser les signals and slots)
		  - Le dynamic_cast requiert l'activation du RTTI... 
				(rajouter /GR dans la ligne de commande globale)
		  - Le gestionnaire s'integre dans les menus et les barres d'outils avec addTo()
		  - Ne pas oublier de "MOC-er" le fichier myBackuphdlr.h avant de compiler...
*/

class DoItemBase
{
public:
	DoItemBase() : m_prev(NULL), m_next(NULL) {}
	virtual ~DoItemBase() { }

private:
	friend class MyBackupHdlr;
	DoItemBase* m_prev;
	DoItemBase* m_next;
};


class MyBackupHdlr : public QObject
{
Q_OBJECT
public:
	MyBackupHdlr(QObject* parent, uint nbreBack=10);
	~MyBackupHdlr() { }

	void addTo(QWidget* w) { m_undoAct->addTo(w);  m_redoAct->addTo(w); }

signals:
	void undoAsked(DoItemBase*);
	void redoAsked(DoItemBase*);

public slots:
	void addDoItem(DoItemBase* item);

protected slots:
	void undoUsed() { if(canUndo()) emit undoAsked(getUndo()); }
	void redoUsed() { if(canRedo()) emit redoAsked(getRedo()); }
	void languageChange();

protected:
	typedef DoItemBase* DoPtr;
	bool canUndo() { return (!m_undoStack.isEmpty()); }
	bool canRedo() { return (!m_redoStack.isEmpty()); }
	DoPtr getUndo();
	DoPtr getRedo();

	QAction* m_undoAct;
	QAction* m_redoAct;
	Stack<DoItemBase> m_undoStack;
	Stack<DoItemBase> m_redoStack;

};

#endif

