
#ifndef F100_PLUGIN

#ifndef __MYRFHDLR
#define __MYRFHDLR

#include <qpopupmenu.h>
#include <qstring.h>
#include <qstringlist.h>

/*
	*** class MyRecentFilesHdlr ***
	
	Gestionnaire d'une liste des derniers fichiers utiliss par un programme.

	Cette liste est enregistre dans le registre de Windows ou dans un fichier externe
	si le programme est compil pour Linux.
	Les arguments 'domain' et 'progName'  passer au constructeur serviront  spcifier
	le chemin du dossier  utiliser dans le registre (voir QSettings).

	Un objet MyRecentFilesHdlr est un menu indpendant, il peut tre plac dans une barre
	d'outils ou plus gnralement comme sous-menu:
	
		QMenuBar* bar = new QMenuBar(this);		// barre de menus
		QPopupMenu* FileMenu = new QPopupMenu(this);	// menu "Fichiers"
		MyRecentFilesHdlr* hdlr = new MyRecentFilesHdlr(this,"/MyCompany","/MyApp");
		//...
				// insertion du gestionnaire dans le menu Fichiers
		FileMenu->insertItem("&Derniers fichiers utiliss", hdlr); 

	Quand l'utilisateur choisit un fichier dans la liste, le gestionnaire emet un signal 
	fileSelected(QString) avec le nom et l'adresse du fichier correspondant.

	Les slots save() et reload() lisent ou crivent le contenu du registre. Ceci est fait
	automatiquement a la cration et a la destruction du gestionnaire.

	Les slots addFile() et removeFile() agissent sur le contenu de la liste. Le nombre
	de fichiers dans la liste est limit; un nouveau fichier est toujours plac en tte
	de liste. Si un fichier est dj prsent il sera ramen au dbut. Un nom de fichier
	vide est ignor.

	Note: Ne pas oublier de "MOC-er" le fichier myRFhdlr.h avant de compiler...
*/

class MyRecentFilesHdlr : public QPopupMenu
{
Q_OBJECT
public:
	MyRecentFilesHdlr(QWidget* parent, QString domain, QString progName, uint maxsize=5);
	~MyRecentFilesHdlr() { save(); }

signals:
	void fileSelected(QString);

public slots:
	void save();
	void reload();
	void setMaxSize(int s) { m_max = s; }
	void addFile(QString f);
	void removeFile(QString f) { m_list.remove(f); createMenu(); }
	void setPath(QString domain, QString progName) { m_domain = domain; m_prog = progName; }

protected slots:
	void itemSelected(int num) { emit fileSelected( m_list[num] ); }

protected:
	void createMenu();
	QStringList m_list;
	uint m_max;
	QString m_domain, m_prog;
};

#endif
#endif
