#pragma once

#include <qsize.h>
#include <qrect.h>
#include <qpoint.h>

template <typename Type>
class PtrMatrice
{
public:
	typedef PtrMatrice<Type> Mat;
	typedef Type* Ptr;
	
	PtrMatrice(bool canDelete=true) : m_canDel(canDelete), m_larg(0), m_haut(0), m_tab(NULL) {}
	PtrMatrice(int larg, int haut, bool canDelete=true);
	~PtrMatrice() { clear(); }

	Mat & adaptFromPtr(int larg, int haut, Ptr* data);
	Ptr* toNewPtr();

	Ptr* row(int num) { Q_ASSERT(num<m_haut && num>=0); return m_tab[num]; }
	Ptr* operator [] (int num) { return row(num); }

	Ptr & operator [] (QPoint &elem) { return m_tab[elem.y()][elem.x()]; }
	Ptr & element(int i, int j) { return m_tab[i][j]; }
	Ptr & element(QPoint &elem) { return m_tab[elem.y()][elem.x()]; }
	const Ptr operator [] (QPoint &elem) const { return element(elem); }
	const Ptr element(int i, int j) const { return m_tab[i][j]; }
	const Ptr element(QPoint &elem) const { return element(elem); }

	void resize(int larg, int haut);
	void resize(QSize &size) { resize(size.width(),size.height()); }
	QSize size() { return QSize(m_larg, m_haut); }
	int height() { return m_haut; }
	int width() { return m_larg; }

	void clear();
	bool isNull() { return m_larg==0 || m_haut==0; }

	//Mat & copy(Mat &modele);
	//Mat & copy(Mat &modele, QRect rect);
	//Mat & operator = (Mat &modele) { copy(modele); return *this; }
	
protected:
	Ptr** m_tab;
	int m_larg, m_haut;
	bool m_canDel;
};

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
PtrMatrice<Type>::PtrMatrice(int larg, int haut, bool canDelete)
{
	m_tab = new Ptr* [haut];
	for(int i=0; i<haut; i++) 
	{
		Ptr* row = new Ptr[larg];
		for(int j=0; j<larg; j++) row[j] = new Type;
		m_tab[i] = row;
	}

	m_larg = larg;
	m_haut = haut;
	m_canDel = canDelete;
}

///////////////////////////////////////////////////////////////////////////////////////

//template <typename Type>
//PtrMatrice<Type> & PtrMatrice<Type>::copy(Mat &modele)
//{
//}

///////////////////////////////////////////////////////////////////////////////////////

//template <typename Type>
//PtrMatrice<Type> & PtrMatrice<Type>::copy(Mat &modele, QRect rect)
//{
//}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void PtrMatrice<Type>::resize(int larg, int haut)
{
	if(m_larg == larg && m_haut == haut) return;
	if(larg <= 0 || haut <= 0)  { clear(); return; }
	
	int largmin = (larg<m_larg? larg : m_larg);
	int hautmin = (haut<m_haut? haut : m_haut);
	int largmax = (larg>m_larg? larg : m_larg);
	int hautmax = (haut>m_haut? haut : m_haut);

	Ptr** newtab = new Ptr* [haut];
	for(int r=0; r<haut; r++) newtab[r] = new Ptr[larg];

	int i,j;
	for(i=0; i<hautmin; i++)
	{
		Ptr* row = m_tab[i];
		Ptr* newrow = newtab[i];
		for(j=0; j<largmin; j++) newrow[j] = row[j]; // copie tous les lments en commun
		for(; j<largmax; j++)
		{
			if(j>=larg) 
			{
				 // on a dpass le nouveau tab, on efface l'ancien item
				if(m_canDel) delete row[j];
			}
			else newrow[j] = new Type; // on a dpass l'ancien tab, on cre un nouvel item
		}
	}
	for(; i<hautmax; i++)
	{
		if(i>=haut) // on a dpass le nouveau tableau
		{
			if(m_canDel) 
			{
				Ptr* row = m_tab[i];
				for(j=0; j<m_larg; j++) delete row[j]; //supprime l'ancienne ligne
			}
		}
		else	// on a dpass l'ancien
		{
			Ptr* newrow = newtab[i];
			for(j=0; j<larg; j++) newrow[j] = new Type; // complete le nouveau avec des new items
		}
	}

	for(i=0; i<m_haut; i++) delete[] m_tab[i];
	delete[] m_tab;

	m_tab = newtab;
	m_haut = haut;
	m_larg = larg;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
PtrMatrice<Type> & PtrMatrice<Type>::adaptFromPtr(int larg, int haut, Type** data)
{
	resize(larg,haut);
	int offset = 0;
	for(int i=0; i<haut; i++) for(int j=0; j<larg; j++) m_tab[i][j] = data[offset++];

	return *this;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
Type** PtrMatrice<Type>::toNewPtr()
{
	Type* newptr = new Type[m_larg*m_haut];
	int offset=0;
	for(int i=0; i<m_haut; i++) for(int j=0; j<m_larg; j++) newptr[offset++] = m_tab[i][j];
	return newptr;
}

///////////////////////////////////////////////////////////////////////////////////////

template <typename Type>
void PtrMatrice<Type>::clear()
{
	if(isNull()) return;
	
	for(int i=0; i<m_haut; i++)
	{
		Ptr* row = m_tab[i];
		for(int j=0; j<m_larg; j++) delete row[j]; // supprime tous les lments points
		delete[] row;
	}
	delete[] m_tab;

	m_tab = NULL;
	m_larg = 0;
	m_haut = 0;
}


