/****************************************************************************
** $Id: qt/qtetrix.cpp   3.2.1   edited May 13 09:08 $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "qtetrix.h"
#include <qapplication.h>
#include <qdatetime.h>
#include <qlayout.h>

#include <MessageBox.h>


void drawTetrixButton( QPainter *p, int x, int y, int w, int h,
		       const QColor *color, QWidget *widg)
{
    if ( color ) {
        QPointArray a;
	a.setPoints( 3,  x,y+h-1, x,y, x+w-1,y );
	p->setPen( color->light() );
	p->drawPolyline( a );
	a.setPoints( 3, x+1,y+h-1, x+w-1,y+h-1, x+w-1,y+1 );
	p->setPen( color->dark() );
	p->drawPolyline( a );
	x++;
	y++;
	w -= 2;
	h -= 2;
	p->fillRect( x, y, w, h, *color );
    }
    else if(widg) {
	widg->erase(x, y, w, h);
    } else {
	p->fillRect(x, y, w, h, p->backgroundColor());
    }
}


ShowNextPiece::ShowNextPiece( QWidget *parent, const char *name )
    : QFrame( parent, name )
{
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
    xOffset = -1;     // -1 until first resizeEvent.
}

void ShowNextPiece::resizeEvent( QResizeEvent *e )
{
    QSize sz = e->size();
    blockWidth  = (sz.width()  - 3)/5;
    blockHeight = (sz.height() - 3)/6;
    xOffset     = (sz.width()  - 3)/5;
    yOffset     = (sz.height() - 3)/6;
}


void ShowNextPiece::paintEvent( QPaintEvent * )
{
    QPainter p( this );
    drawFrame( &p );
    p.end();			// explicit end() so any slots can paint too
    emit update();
}


void ShowNextPiece::drawNextSquare(int x, int y,QColor *color)
{
    if (xOffset == -1)		// Before first resizeEvent?
        return;

    QPainter paint;
    paint.begin(this);
    drawTetrixButton( &paint, xOffset+x*blockWidth, yOffset+y*blockHeight,
		      blockWidth, blockHeight, color, this );
    paint.end();
}

QTetrix::QTetrix(CONSTRUCTOR_PARAM)
    : CToolWindow(CALL_PARAM)
{
    QTime t = QTime::currentTime();
    TetrixPiece::setRandomSeed( (((double)t.hour())+t.minute()+t.second())/
                                 (24+60+60) );

	QWidget* pCentralWidget = new QWidget(0);

	SetCentralWidget(pCentralWidget);

	createLabel(m_pNextLabel, pCentralWidget);
	createLabel(m_pScoreLabel, pCentralWidget);
	createLabel(m_pLevelLabel, pCentralWidget);
	createLabel(m_pLinesLabel, pCentralWidget);

    board       = new QTetrixBoard(pCentralWidget);
    showNext    = new ShowNextPiece(pCentralWidget);

	board->setFocusPolicy(QWidget::StrongFocus);

    showScore   = new QLCDNumber(5, pCentralWidget);
    showLevel   = new QLCDNumber(2, pCentralWidget);
    showLines   = new QLCDNumber(5, pCentralWidget);

	showScore->setSegmentStyle(QLCDNumber::Filled);
	showLevel->setSegmentStyle(QLCDNumber::Filled);
	showLines->setSegmentStyle(QLCDNumber::Filled);

    m_pStartButton = new QPushButton(pCentralWidget);
    m_pPauseButton = new QPushButton(pCentralWidget);

	QGridLayout* pLayout = new QGridLayout(pCentralWidget, 6, 3, 10);

	pLayout->addWidget(m_pNextLabel,  0, 0);
	pLayout->addWidget(m_pScoreLabel, 0, 2);
	pLayout->addWidget(m_pLevelLabel, 2, 0);
	pLayout->addWidget(m_pLinesLabel, 2, 2);

	pLayout->addWidget(showNext,      1, 0);
	pLayout->addWidget(showScore,     1, 2);
	pLayout->addWidget(showLevel,     3, 0);
	pLayout->addWidget(showLines,     3, 2);

	pLayout->addWidget(m_pStartButton, 4, 0);
	pLayout->addWidget(m_pPauseButton, 5, 0);

	pLayout->setColSpacing(2, 170);
	pLayout->setRowSpacing(3, 80);

	pLayout->setRowStretch(1, 1);
	pLayout->setRowStretch(3, 1);

	pLayout->setColStretch(0, 1);
	pLayout->setColStretch(1, 2);
	pLayout->setColStretch(2, 2);

	pLayout->addMultiCellWidget(board, 0, 5, 1, 1);

    // Don't let the buttons get keyboard focus
    m_pStartButton->setFocusPolicy( QWidget::NoFocus );
    m_pPauseButton->setFocusPolicy( QWidget::NoFocus );

    connect( board, SIGNAL(gameOverSignal()), SLOT(gameOver()) );
    connect( board, SIGNAL(drawNextSquareSignal(int,int,QColor*)), showNext,
	     SLOT(drawNextSquare(int,int,QColor*)) );
    connect( showNext, SIGNAL(update()), board, SLOT(updateNext()) );
    connect( board, SIGNAL(updateScoreSignal(int)), showScore,
	     SLOT(display(int)) );
    connect( board, SIGNAL(updateLevelSignal(int)), showLevel,
	     SLOT(display(int)));
    connect( board, SIGNAL(updateRemovedSignal(int)), showLines,
	     SLOT(display(int)));

    connect( m_pStartButton, SIGNAL(clicked()), board, SLOT(start()) );
    connect( m_pPauseButton, SIGNAL(clicked()), board, SLOT(pause()) );

	board->setMinimumSize(153, 333);
	showNext->setMinimumSize(78, 94);

    showScore->display( 0 );
    showLevel->display( 0 );
    showLines->display( 0 );
   
    board->revealNextPiece(TRUE);

	languageChange();
}

void QTetrix::gameOver()
{
	CMessageBox::MsgInformation(GetMainWidget(), tr("Game Over!"));
}

void QTetrix::createLabel(QLabel*& pLabel, QWidget* parent)
{
	pLabel = new QLabel(parent);
	pLabel->setAlignment(AlignCenter);
}

//-----------------------------------------------------------------------------------
int QTetrix::Init(INIT_PARAM)
{
	return 1;
}

//-----------------------------------------------------------------------------------
void QTetrix::DoShowCaption()
{
	setCaption("Tetrix");
}

//-----------------------------------------------------------------------------------
void QTetrix::DoInitWords()
{
	m_pNextLabel->setText(tr("Next"));
	m_pScoreLabel->setText(tr("Score"));
	m_pLevelLabel->setText(tr("Level"));
	m_pLinesLabel->setText(tr("Lines removed"));

	m_pStartButton->setText(tr("&New"));
	m_pPauseButton->setText(tr("&Pause"));
}
