
#ifndef F100KERNEL_H
#define F100KERNEL_H

/** \file F100Kernel.h
 * \brief Regroupe des macros bas niveau
 * \ingroup include
 */

/** Used by CPUCOST_INIT and CPUCOST_CALC.
 * @see CPUCOST_INIT, CPUCOST_CALC
 */
#define cpuid __asm __emit 0fh __asm __emit 0a2h

/** Used by CPUCOST_INIT and CPUCOST_CALC.
 * @see CPUCOST_INIT, CPUCOST_CALC
 */
#define rdtsc __asm __emit 0fh __asm __emit 031h

#ifdef _CYRIX_PROCESSOR_
#	define CPUCOST_INIT(cost)	cost = 0;
#	define CPUCOST_CALC(cost)	cost = 0;
#else

/** CPU usage macro.
 * Use this macro together with CPUCOST_CALC to get a delta num of cpu cycles.
 * @param cost is an unsigned int
 * @note This macro is ineffective if compiled with _CYRIX_PROCESSOR_ define.
 *
 * @code
 *      unsigned int nCost;
 *      CPUCOST_INIT(nCost);
 *      // ... code to test
 *      CPUCOST_CALC(nCost);
 *      printf("This code costs %lu CPU cycles!", nCost);
 * @endcode
 */
#	define CPUCOST_INIT(cost) \
		__asm pushad          \
		__asm cpuid           \
		__asm rdtsc           \
		__asm mov cost, eax   \
		__asm popad

/** CPU usage macro.
 * Use this macro together with CPUCOST_INIT to get a delta num of cpu cycles.
 * @param cost is an unsigned int
 * @note This macro is ineffective if compiled with _CYRIX_PROCESSOR_ define.
 * @see CPUCOST_INIT
 */
#	define CPUCOST_CALC(cost) \
		__asm pushad          \
		__asm cpuid           \
		__asm rdtsc           \
		__asm sub eax, cost   \
		__asm mov cost, eax   \
		__asm popad

#endif

#endif //F100KERNEL_H
