
#ifndef F100_DEBUG_H
#define F100_DEBUG_H

/** \file F100_Debug.h
 * \brief Contient des macros pour la gestion des messages de dboggage
 * \ingroup include
 */

#include <assert.h>

#ifdef WIN32
#	include <crtdbg.h>
#	define _WARNING0(text)         _RPT0(_CRT_WARN, text)
#	define _WARNING1(text, p1)     _RPT1(_CRT_WARN, text, p1)
#	define _WARNING2(text, p1, p2) _RPT2(_CRT_WARN, text, p1, p2) 
#	define _ERROR0(text)           _RPT0(_CRT_ERROR, text)
#	define _ERROR1(text, p1)       _RPT1(_CRT_ERROR, text, p1)
#	define _ERROR2(text, p1, p2)   _RPT2(_CRT_ERROR, text, p1, p2) 
#else
#	ifdef _DEBUG
#		include <stdio.h>
#		define _ASSERTE assert
#		define _WARNING0(text)         printf("Warning : %s", text)
#		define _WARNING1(text, p1)     printf("Warning : %s", text, p1)
#		define _WARNING2(text, p1, p2) printf("Warning : %s", text, p1, p2)
#		define _ERROR0(text)           printf("Error : %s", text)
#		define _ERROR1(text, p1)       printf("Error : %s", text, p1)
#		define _ERROR2(text, p1, p2)   printf("Error : %s", text, p1, p2)
#	else
/** Assertion utilisant les rglages spcifiques pour Visual C++.\n
 * Peut tre affiche dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _ASSERTE

/** Affiche un \a Warning en utilisant les rglages spcifiques pour Visual C++.\n
 * Pas de paramtre supplmentaire, affiche juste le texte.\n
 * Peut tre affich dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _WARNING0(text)

/** Affiche un \a Warning en utilisant les rglages spcifiques pour Visual C++.\n
 * Affiche le texte + un paramtre.\n
 * Peut tre affich dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _WARNING1(text, p1)

/** Affiche un \a Warning en utilisant les rglages spcifiques pour Visual C++.\n
 * Affiche le texte + deux paramtres.\n
 * Peut tre affich dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _WARNING2(text, p1, p2)

/** Affiche une \a Erreur en utilisant les rglages spcifiques pour Visual C++.\n
 * Pas de paramtre supplmentaire, affiche juste le texte.\n
 * Peut tre affiche dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _ERROR0(text)

/** Affiche une \a Erreur en utilisant les rglages spcifiques pour Visual C++.\n
 * Affiche le texte + un paramtre.\n
 * Peut tre affiche dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _ERROR1(text, p1)

/** Affiche une \a Erreur en utilisant les rglages spcifiques pour Visual C++.\n
 * Affiche le texte + deux paramtres.\n
 * Peut tre affiche dans :\n
 * - Une boite de dialogue\n
 * - Un fichier\n
 * - Dans la fentre de Debug
 */
#		define _ERROR2(text, p1, p2)
#	endif
#endif

#endif
