
#ifndef F100_OP_H
#define F100_OP_H

class IProtocol;

/** \file F100_Op.h
 * \brief Contient des macros et numrations pour la gestion des protocoles et outils
 * \ingroup include
 */

#ifdef WIN32
#	ifndef STATIC_EXPORT
#		ifdef MAKE_DLL
#			define DLLEXPORT __declspec(dllexport)
#		else
#			define DLLEXPORT __declspec(dllimport)
#		endif
#	else
#		define DLLEXPORT
#	endif
#	define EXPORT_SYMB extern "C" __declspec(dllexport)
#else
#	define DLLEXPORT
#	define EXPORT_SYMB extern "C"
#endif

/** Enumration pour la gestion des ordres dans les points d'entre d'un protocole ou d'un outil
 * @see @ref page1
 */
enum F100_Op
{
	OP_TOOLNAME,		/*!< Nom de l'outil */
	OP_BUILDDATE,		/*!< Date de cration */
	OP_VERSION,			/*!< Version */
	OP_AUTHOR,			/*!< Nom de l'auteur */
	OP_CREATEINSTANCE,	/*!< Cre une instance */
	OP_DELETEINSTANCE,  /*!< Dtruit une instance */
	OP_WINDOWTYPESTR,	/*!< Type de l'outil (Nom sans espace, sans accent) */
	OP_TOOLBARICON,		/*!< Icone de la barre d'outils */
	OP_FILEFILTER,		/*!< Filtre pour la boite de dialogue */
	OP_DESCRIPTION,		/*!< Description de la DLL */
	OP_SUPPORTEDFILE,	/*!< Extension des fichiers supports */
	OP_SETTINGSWIDGET,	/*!< Fentre de setting pour la fenetre de l'outil */
	OP_ADDTOTOOLBAR,	/*!< A vrai si on veut que par dfaut l'application ajoute l'icone de \a OP_TOOLBARICON. Cette oprande est ignore si \a OP_TOOLBARICON retourne une image nulle. */
};

/*! \page page1 Utilisation de l'numration F100_Op
 * \section sec1 Pour les protocoles
 * \subsection subsec1 Valeurs utiles
 * OP_TOOLNAME       : Prfrable \n
 * OP_BUILDDATE      : Prfrable \n
 * OP_VERSION        : Prfrable \n
 * OP_AUTHOR	     : Prfrable \n
 * OP_CREATEINSTANCE : Ncessaire. Permet de crer un protocole. \n
 * OP_DELETEINSTANCE : Ncessaire. Permet de dtruire un protocole.
 * \subsection subsec2 Retours associs
 * OP_TOOLNAME       : ret = const char* \n
 * OP_BUILDDATE      : ret = const char* \n
 * OP_VERSION        : ret = const char* \n
 * OP_AUTHOR         : ret = const char* \n
 * OP_CREATEINSTANCE : ret = IProtocol* \n
 * OP_DELETEINSTANCE : ret = NULL
 * \section sec2 Pour les outils
 * \subsection subsec3 Valeurs utiles
 * OP_TOOLNAME       : Prfrable \n
 * OP_BUILDDATE      : Prfrable \n
 * OP_VERSION        : Prfrable \n
 * OP_AUTHOR         : Prfrable \n
 * OP_CREATEINSTANCE : Ncessaire \n
 * OP_WINDOWTYPESTR  : Ncessaire \n
 * OP_TOOLBARICON    : Optionnel \n
 * OP_FILEFILTER     : Optionnel \n
 * OP_DESCRIPTION    : Prfrable \n
 * OP_SUPPORTEDFILE  : Optionnel \n
 * OP_SETTINGSWIDGET : Optionnel \n
 * OP_ADDTOTOOLBAR   : Optionnel (Par dfaut FALSE) \n
 * \subsection subsec4 Retours associs
 * OP_TOOLNAME       : ret = QString* \n
 * OP_BUILDDATE      : ret = char* \n
 * OP_VERSION        : ret = char* \n
 * OP_AUTHOR         : ret = char* \n
 * OP_CREATEINSTANCE : ret = CToolWindow* \n
 * OP_WINDOWTYPESTR	 : ret = char* \n
 * OP_TOOLBARICON    : ret = QPixmap* \n
 * OP_FILEFILTER     : ret = QString* \n
 * OP_DESCRIPTION    : ret = QString* \n
 * OP_SUPPORTEDFILE  : ret = char* \n
 * OP_SETTINGSWIDGET : ret = CSettingsWidget* \n
 * OP_ADDTOTOOLBAR   : ret = bool* \n
 */

/** Dfinition du point d'entre des protocoles */
#define EP_PROTOCOL_REQUEST "Protocol_Request"

/** Point d'entre pour les protocoles */
typedef const char* (*Protocol_Op)(F100_Op op, IProtocol*);

#endif
