
#ifndef IOPTIONWINDOW_H
#define IOPTIONWINDOW_H

#include <qstring.h>

class CSettingsWidget;
class CCalcInfo;
class QWidget;
class CTransfertWindow;

/*!
	\class IOptionWindow IOptionWindow.h
	\brief Permet de recuprer des prfrences communes et d'afficher une page d'option
	\author Quaranta David
	\version 1.00
	
	\ingroup include
*/
class IOptionWindow
{
public:
	/** Liste des prfrences disponibles */
	enum Preference{
		PREF_AUTOLAUNCH = 0,      /*!< Au dmarrage de l'application le dernier projet est ouvert. */
		PREF_DRIVE_VARIABLE_SIZE, /*!< Si la taille des lecteurs est variable (ie. <= 128Ko). */
		PREF_MULTI_INSTANCE,      /*!< Si plusieurs instances de Flash100 sont permises. */
		PREF_AUTO_SAVE_PROJECT,   /*!< Si la sauvegarde automatique des projets est active */
	};

public:
	/** Retourne la valeur de la prfrence
	 * @param[in] nPref Prfrence dont on veut connaitre l'tat
	 * @see Preference
	 */
	virtual int               GetPreference(Preference nPref) const = 0;
	
	/** Affiche la fentre des options sur la page de prfrence dsire
	 * @param[in] pWidget Page  afficher, ou NULL pour afficher la page des options
	 */
	virtual void              ShowSettingsWidget(CSettingsWidget* pWidget) = 0;

	/** Permet de rcuprer la fentre d'option d'un outil
	 * @param[in] sToolName Nom de l'outil
	 * @return Retourne la fentre d'option associ  un nom d'outil, ou NULL si non trouve
	 */
	virtual CSettingsWidget*  GetSettingsWidget(const QString& sToolName) const = 0;

	/** Cre une fentre de transfert
	 * @param[in] pParent Fentre parente
	 * @param[in] nTotalSize Taille totale en octets  envoyer ou recevoir. Une valeur infrieur  0 implique
	 * que cette valeur est inconnue
	 * @return Retourne un pointeur sur une nouvelle instance d'une fentre de transfert. La fentre de transfert cre
	 * utilise le protocole choisi dans les options.
	 */
	virtual CTransfertWindow* CreateTransfertWindow(QWidget* pParent, int nTotalSize = -1) = 0;

	/** @return Retourne les informations connues sur la calculatrice en cours */
	virtual CCalcInfo*        GetCalcInfo() = 0;
};

#endif
